###############################################################################
# File: 		test_addslash.tcl
# Purpose:		Test file for [addslash]
# Author: 		M Shaw
# Language:		Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Aug/2019 First version
# 1.01 21/Nov/2022 Updated for -unix and -separator args
###############################################################################
# Notes:
###############################################################################

option expression off

set s "c:"
echo [addslash $s]
assert [addslash $s] {eq $_ {c:\\}} -v -line #__LINE__		# FAIL
assert [addslash $s] {eq $_ "c:\\"} -v -line #__LINE__		# FAIL

set s "c:\\"
assert [addslash $s] {eq $_ "c:\\"} -v -line #__LINE__

set s "c:\\"
assert [addslash $s] {eq $_ "c:\\"} -v -line #__LINE__

set s "."
assert [addslash $s] {eq $_ ".\\"} -v -line #__LINE__

set s ".."
assert [addslash $s] {eq $_ "..\\"} -v -line #__LINE__

set s "\\path"
assert [addslash $s] {eq $_ "\\path\\"} -v -line #__LINE__

set s "\\path\\to"
assert [addslash $s] {eq $_ "\\path\\to\\"} -v -line #__LINE__

set s "\\path\\to\\"
assert [addslash $s] {eq $_ "\\path\\to\\"} -v -line #__LINE__

set s "C:\\path"
assert [addslash $s] {eq $_ "C:\\path\\"} -v -line #__LINE__
newline

# Unix and Web URL
textcolor yellow
puts "Testing -unix options"
textcolor

set s "c:"
assert [addslash $s -unix] {eq $_ "c:/"} -v -line #__LINE__

set s "c:/"
assert [addslash $s -unix] {eq $_ "c:/"} -v -line #__LINE__

set s "."
assert [addslash $s -unix] {eq $_ "./"} -v -line #__LINE__

set s ".."
assert [addslash $s -unix] {eq $_ "../"} -v -line #__LINE__

set s "/path"
assert [addslash $s -unix] {eq $_ "/path/"} -v -line #__LINE__

set s "/path/to"
assert [addslash $s -unix] {eq $_ "/path/to/"} -v -line #__LINE__


set s http://foo.com/bar
assert [addslash $s -unix] {eq $_ "http://foo.com/bar/"} -v -line #__LINE__

set s http://foo.com/bar/
assert [addslash $s -unix] {eq $_ "http://foo.com/bar/"} -v -line #__LINE__

set s http://foo.com/bar
assert [addslash $s -separator /] {eq $_ "http://foo.com/bar/"} -v -line #__LINE__

set s http://foo.com/bar/
assert [addslash $s -separator /] {eq $_ "http://foo.com/bar/"} -v -line #__LINE__


unset s -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #