###############################################################################################
# File:				test_all_lib.tcl
# Author:			M Shaw
# Purpose:			Test file for all ticol_*.dll libs
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 04/Dec/2019	First version
###############################################################################################
# Notes:
###############################################################################################

option echo push on			# Only a single push level is supported ATM so any local use will break this

# Ensure we can found our own components...
set working_path [addslash [file dirname [array item argv 0 -nocomplain]]]
#set working_path [file dirname [info script]]		# Alternative method
if {ne $working_path ""} {
	chdir $working_path
}
unset working_path -nocomplain

textcolor darkyellow
#set root_scriptname [file name [info script]]
puts "*** Start Tests: [info script] *** Line:"#__LINE__
newline
textcolor

#clear						# Ensure no vars if run from CLI (can't do for loop testing)
option expression on
set _start_time [timer]

set main::test_count 0
# puts "Debug: _scriptname'$_scriptname' info script '[info script]' ext -> '[file extension [info script]]'"
set main::filetype [file extension [info script]]
assert $main::filetype {ne $_ ""} -v -line #__LINE__				# Won't work in the DLL
#puts "filetype is $filetype"
set main::logfile "tests.log"
unlink $main::logfile

writefile $main::logfile "Start Test Run [date]@[time]\r\n"			# Log the date / time of run

if {! [defined run_test] } {
	vars -level 0
	#####################################################
	# This proc cannot depend on persistent global vars 
	# as these may be cleared by called scripts
	#####################################################
	proc run_test {filename line} {
		option expression off
		set name $filename
		
		if {[eq [file extension $filename] {}]} {
			if {eq $::main::filetype {}} {							# May be embedded DLL script
				append filename ".tcl"
			} else {
				append filename $::main::filetype					# ".tcl" ".tcx"
			}
		}

		textcolor yellow
		puts "\nTesting external file: $filename (Loop: [item ::loop::i 0])"
		textcolor
		newline

		set ::main::tested([++ ::main::test_count]) $filename
		console set_title "Processing (Test: $::main::test_count, Loop: [item ::loop::i 0]): '$filename'"
#array walk ::main::tested
		textcolor darkyellow
		puts "tests$::main::filetype -> run_test@Line: $line -> source $filename\n"
		textcolor
		#walk -level 0
		#vars -level 0
		if {[file exists $filename]} {
			writefile $::main::logfile "[time] start: $name\r\n" -append	# Filename minus ".tc?"

			try {										# Not using -quiet argument (error messages will be shown)
				uplevel @0 "source $filename"			# Call external file at level 0
				halt									# Stop the debuggger if running
			} catch {
				puts "\ntests.tcl -> run_test: catch failure in $filename"
				pause
				die
			}
			writefile $::main::logfile "[time] end:   $name\r\n" -append	# Filename minus ".tc?"
		} else {
			die "Can't find $filename in [getcwd]"
		}
		newline
# Ad-hoc var trace and halt
watch trace 106 -var -halt -line #__LINE__
/*
		writefile $::main::logfile "[time] end:   [info exists "1"]\r\n" -append
		foreach x [vars -list -level 0] {
			if {is_numeric $x} {
			# if {is_numeric [lindex [vars -list] 0]} {
			# if {[vars 5*]} {
				die "TRACE VAR FOUND"
			} 
		}# */
		writefile $::main::logfile "[time] end:   '[info vars ?????????? -level 0]'\r\n" -append
	}
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

#####################################################################################################
# Check prerequisites
textcolor yellow
puts "Checking external dependencies ..."
textcolor
newline
# ADD '//Lenovo/C$/VC5/MyProjects/ticol/release/internal.tcl' - test_forward_slash.tcl
if {! [file exists array_data.tcl]} {
	die "Test for test_include_array.tcl requires array_data.tcl"
}
if {! [file exists lorem.txt]} {
	die "Tests test_lib_bmp, test_lib_line, test_fileio require lorem.txt"
}
if {! [file exists empty.tcl]} {
	die "Test test_is_empty.tcl requires empty.tcl (a completely empty file)"
}
if {! [file exists defs.tcl]} {
	die "Tests test_data_ranges, test_loop_max, test_macro_literals require defs.tcl (defined constants)"
}
if {! [file exists md5.exe]} {
	die "Test test_md5 requires md5.exe (author's own MD5 generator)"
}
if {! [file exists vbs.exe]} {
	die "Tests test_spawn_vbs requires vbs.exe (author's own VBScript shell)"
}
if {! [file exists mslib145.dll]} {
	die "Test test_lib_calldll requires mslib145.dll (author's own VBToolbox DLL)"
}
if {! [file exists cdecl.dll]} {
	die "Test test_lib_calldll requires cdecl.dll (A simple __Cdecl DLL)"
}
if {! [file exists albums.db]} {
	die "Test test_lib_sqlite requires albums.db (A simple SQLite DB)"
}
if {! [file exists ticol_stub.exe]} {
	die "Test test_spawn_stub requires ticol_stub.exe (Example C++ DLL Link EXE)"
}
############################################################################################




###########################################################################################
#/*
#run_test test_lib_clash.tcl							# Not yet run
#run_test test_lib_load_unload.tcl						# NYR
#
run_test test_lib_arp #__LINE__							# Plugin - ticol_arp.dll (ARP table handling)
run_test test_lib_asm #__LINE__							# Plugin - ticol_asm.dll (Pseudo-Assembler Interpreter)
run_test test_lib_asm_file #__LINE__					# Plugin - ticol_asm.dll (Pseudo ASM File handling)
#/*	# Why was this commented out ?
run_test test_lib_bf #__LINE__							# Plugin - ticol_bf.dll	(poss slight leak by here)
run_test test_lib_big #__LINE__							# Plugin - ticol_bignum.dll (big numbers) (leaks or uses a LOT of memory)
run_test test_lib_bigbitshift #__LINE__					# Plugin - [big] bitshifting
run_test test_lib_bigdiv_simple #__LINE__				# Plugin - [big] division
# */
run_test test_lib_bigfib #__LINE__						# Plugin - [big] fib
run_test test_lib_bignum #__LINE__						# Plugin - Big numbers
run_test test_lib_bmp #__LINE__							# Plugin - ticol_bmp.dll
run_test test_lib_calldll #__LINE__						# Plugin - ticol_calldll.dll  (Poss very slight leak up to this line somewhere)
run_test test_lib_carray #__LINE__						# Plugin - ticol_carray.dll
run_test test_lib_cpu #__LINE__							# Plugin - ticol_cpu.dll
run_test test_lib_crc #__LINE__							# Plugin - ticol_crc.dll
run_test test_lib_crt #__LINE__							# Test calls to MSVC CRT via ticol_calldll plugin
run_test test_lib_cscript #__LINE__						# Plugin - ticol_cscript.dll (VBScript)	(Not a full test yet)
run_test test_lib_db #__LINE__							# Plugin - ticol_db.dll (Ticol specific random-access DB files)
run_test test_lib_dns #__LINE__							# Plugin - ticol_dns.dll
#run_test test_lib_event #__LINE__						# Plugin - ticol_event.dll Event log handling (must be elevated)
run_test test_lib_highlight #__LINE__					# Plugin - ticol_highlight.dll (HTML syntax highlighting)
run_test test_lib_html #__LINE__						# Plugin - ticol_html.dll [html]
run_test test_lib_html_chomp #__LINE__					# Plugin - ticol_html.dll [html chomp] (Big leak FIXED)
run_test test_lib_ini #__LINE__							# Plugin - ticol_ini.dll
run_test test_lib_json #__LINE__						# Plugin - ticol_json.dll
run_test test_lib_json_to_list #__LINE__				# Plugin - ticol_json.dll
run_test test_lib_line #__LINE__						# Plugin - ticol_line.dll (Text processing)
run_test test_lib_mapdrive #__LINE__					# Plugin - ticol_mapdrive.dll
run_test test_lib_misc #__LINE__						# Plugin - ticol_misc.dll
run_test test_lib_numword #__LINE__						# Plugin - ticol_numword.dll (new, 2019)
run_test test_lib_ping #__LINE__						# Plugin - ticol_ping.dll
run_test test_lib_reg #__LINE__							# Plugin - ticol_reg.dll
run_test test_lib_rlm  #__LINE__						# Plugin - ticol_rlm.dll (not for final release)
#run_test test_lib_rtf #__LINE__						# Plugin - ticol_rtf.dll (no tests yet)
run_test test_lib_sci #__LINE__							# Plugin - ticol_sci.dll
#run_test test_lib_service #__LINE__					# Plugin - Requires elevation
run_test test_lib_sqlite #__LINE__						# Plugin - ticol_sqlite.dll
run_test test_lib_sockping #__LINE__					# Problems 22/Aug/2016. Causes instability (moved sock init code to main)
run_test test_lib_tcx #__LINE__							# Plugin - ticol_tcx.dll (Code obfuscation)
run_test test_lib_ticol_vb #__LINE__					# Plugin - ticol_vb.dll (VB-like commands)
run_test test_lib_variant #__LINE__						# Plugin - ticol_variant.dll (Rudimentary VARIANT handling)
run_test test_lib_wordcount #__LINE__					# Non-standard command - [wordcount]
run_test test_lib_xbase #__LINE__						# Plugin - ticol_xbase.dll (DBase file handling)
run_test test_lib_zip #__LINE__							# Plugin - ticol_zip.dll (ZIP file handling)
# */
###########################################################################################

beep; beep; beep

newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #