######################################################################################
# File: 			test_anonymous.tcl
# Purpose:			Test use of an anonymised command
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 07/Dec/2017	First version
######################################################################################
# Notes:
# Re: http://jim.tcl.tk/fossil/doc/trunk/Tcl_shipped.html#_curry
# Using an anonymous command via a var and {*}
######################################################################################
# Similar to alias except it creates an anonymous procedure (lambda) instead of a named procedure.
# the following creates a local, unnamed alias for the command info exists.
# set e [local curry info exists]
# if {[$e var]} {
#   ...
# }
# See also: use of [lmap]
# Lambda like call with argument passed via [lmap]
#	set square {lmap {x {expr $x * $x}}}
# {*} will break string into separate arguments
#	puts [{*}$square 42]
######################################################################################

option expression off

unset a -nocomplain
set e {info exists}
puts [{*}$e a]
assert [{*}$e a] {== $_ 0} -v -line #__LINE__
set a Hello
puts [{*}$e a]
assert [{*}$e a] {== $_ 1} -v -line #__LINE__

set e {chomp}
assert [{*}$e a] {eq $_ Hello} -v -line #__LINE__

set e {string repeat}
puts [{*}$e q 10]
assert [{*}$e q 10] {eq $_ qqqqqqqqqq} -v -line #__LINE__

set e {string repeat x}
puts [{*}$e 10]
assert [{*}$e 10] {eq $_ xxxxxxxxxx} -v -line #__LINE__

set e {string repeat y 10}
puts [{*}$e]
assert [{*}$e] {eq $_ yyyyyyyyyy} -v -line #__LINE__

unset a e -nocomplain

vars

#####################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #