######################################################################################
# File: 			test_any_all.tcl
# Purpose:			Test for [any] and [all]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 04/Nov/2019	New commands added
######################################################################################
# Notes: 
# [any] and [all] provide compact and efficient ways to test multiple boolean 
# clause results without complex and unreadable nesting of commands or the use of 
# [expr]
# Neither command performs any evaluation. The results supplied must be a boolean 
# or pseudo-boolean numeric result of some other command
# Should need no further testing as the inputs should be purely bool/pseudo-bool
######################################################################################

option expression off

unset a b i -nocomplain

######################################################################################
# Check insufficient arg count
set i 0
try {
	any
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

set i 0
try {
	all
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

######################################################################################
# ANY
newline
textcolor yellow
puts "Testing \[any\] ..."
textcolor
newline

assert [any 1] {== $_ $true} -v -line #__LINE__
assert [any 1 1] {== $_ $true} -v -line #__LINE__
assert [any 1 1 1] {== $_ $true} -v -line #__LINE__

assert [any 0] {== $_ $false} -v -line #__LINE__
assert [any 0 0] {== $_ $false} -v -line #__LINE__
assert [any 0 0 0] {== $_ $false} -v -line #__LINE__

assert [any 1 0 0] {== $_ $true} -v -line #__LINE__
assert [any 0 1 0] {== $_ $true} -v -line #__LINE__
assert [any 0 0 1] {== $_ $true} -v -line #__LINE__

assert [any 6 0 0] {== $_ $true} -v -line #__LINE__
assert [any 0 6 0] {== $_ $true} -v -line #__LINE__
assert [any 0 0 6] {== $_ $true} -v -line #__LINE__

assert [any 0 0 -6] {== $_ $true} -v -line #__LINE__
assert [any 0 -6 0] {== $_ $true} -v -line #__LINE__
assert [any -6 0 0] {== $_ $true} -v -line #__LINE__

assert [any 0 0 {1}] {== $_ $true} -v -line #__LINE__
assert [any 0 {1} 0] {== $_ $true} -v -line #__LINE__
assert [any {1} 0 0] {== $_ $true} -v -line #__LINE__

set a 1
set b 0
assert [any $b $b $a] {== $_ $true} -v -line #__LINE__
assert [any $b $a $b] {== $_ $true} -v -line #__LINE__
assert [any $a $b $b] {== $_ $true} -v -line #__LINE__

assert [any $a $b $b $a $b $a $b] {== $_ $true} -v -line #__LINE__
assert [all $a $b $b $a $b $a $b] {== $_ $false} -v -line #__LINE__
assert [all $a $a $a $a $a $a $a] {== $_ $true} -v -line #__LINE__

unset a b -nocomplain

######################################################################################
# ALL
newline
textcolor yellow
puts "Testing \[all\] ..."
textcolor
newline

assert [all 1] {== $_ $true} -v -line #__LINE__
assert [all 1 1] {== $_ $true} -v -line #__LINE__
assert [all 1 1 1] {== $_ $true} -v -line #__LINE__

assert [all 0] {== $_ $false} -v -line #__LINE__
assert [all 0 0] {== $_ $false} -v -line #__LINE__
assert [all 0 0 0] {== $_ $false} -v -line #__LINE__

assert [all 1 0 0] {== $_ $false} -v -line #__LINE__
assert [all 0 1 0] {== $_ $false} -v -line #__LINE__
assert [all 0 0 1] {== $_ $false} -v -line #__LINE__

assert [all 6 0 0] {== $_ $false} -v -line #__LINE__
assert [all 0 6 0] {== $_ $false} -v -line #__LINE__
assert [all 0 0 6] {== $_ $false} -v -line #__LINE__

assert [all 0 0 -6] {== $_ $false} -v -line #__LINE__
assert [all 0 -6 0] {== $_ $false} -v -line #__LINE__
assert [all -6 0 0] {== $_ $false} -v -line #__LINE__

assert [all 0 0 {1}] {== $_ $false} -v -line #__LINE__
assert [all 0 {1} 0] {== $_ $false} -v -line #__LINE__
assert [all {1} 0 0] {== $_ $false} -v -line #__LINE__

set a 1
set b 0
assert [all $b $b $a] {== $_ $false} -v -line #__LINE__
assert [all $b $a $b] {== $_ $false} -v -line #__LINE__
assert [all $a $b $b] {== $_ $false} -v -line #__LINE__

assert [all 6 6 6] {== $_ $true} -v -line #__LINE__
assert [all 6 6 6] {== $_ $true} -v -line #__LINE__
assert [all 6 6 6] {== $_ $true} -v -line #__LINE__

assert [all -6 -6 -6] {== $_ $true} -v -line #__LINE__
assert [all -6 -6 -6] {== $_ $true} -v -line #__LINE__
assert [all -6 -6 -6] {== $_ $true} -v -line #__LINE__

assert [all {1} {1} {1}] {== $_ $true} -v -line #__LINE__
assert [all {1} {1} {1}] {== $_ $true} -v -line #__LINE__
assert [all {1} {1} {1}] {== $_ $true} -v -line #__LINE__

assert [all $a $a $a] {== $_ $true} -v -line #__LINE__
assert [all $a $a $a] {== $_ $true} -v -line #__LINE__
assert [all $a $a $a] {== $_ $true} -v -line #__LINE__


# man e.g.
# if { || $a [|| $b [|| $c [|| $d $e]]] } {puts "Test ok"}

unset a b i -nocomplain

######################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #