#################################################################
# File:				test_append.tcl
# Purpose:			Testing [append] and trial use of xmalloc
# Author:			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 24/Nov/2017	First version
#################################################################
# Notes:
# Tests [append] v's other methods
# Possibly unstable
#################################################################
/*
no xmalloc in append
Testing string append using [set] concatenation [randomstr 100]
1041 microsecond(s) per iteration
Testing string append using [set] concatenation [randomstr 1000]
5605 microsecond(s) per iteration
Testing string append using [append] [randomstr 100]
76 microsecond(s) per iteration
Testing string append using [append] [randomstr 1000]
761 microsecond(s) per iteration
Testing string append using [store] [randomstr 100]
8 microsecond(s) per iteration
Testing string append using [store] [randomstr 1000]
38 microsecond(s) per iteration
*/
#################################################################

option expression off
option echo on

textcolor yellow
puts "Testing string \[append\]"
textcolor
newline

puts "Line: "#__LINE__
set s {}
append s "Hello "
append s "world"
puts "Line: "#__LINE__
assert $s {eq $_ "Hello world"} -v -line #__LINE__

puts "Line: "#__LINE__
set s {}
foreach x [split $qbf] {
	#puts "Appending '$x'"
	append s $x
	store add $x
}
puts "split x is '$s'"
assert $s {eq $_ Thequickbrownfoxjumpsoverthelazydog} -v -line #__LINE__
assert $s {eq $_ [store get]} -v -line #__LINE__

# TIMINGS # # # # # # # # # # # # # # # # # # # # # # # 
/*
# Operate simple [set] concatenation and [append] in parallel
# We should get the same result
puts "Line: "#__LINE__
textcolor yellow
puts "Testing string \[append\] in parallel with concatenation \[randomstr 100\]"
textcolor
set s {}
set t {}
time {
	set r [randomstr 100]
	append s $r
	set t "$t$r"
} 1000
assert [eq $s $t] {== $_ 1} -v -line #__LINE__


# TIME Operate simple [set] concatenation and [append] separately
puts "Line: "#__LINE__
textcolor yellow
puts "Testing string append using \[set\] concatenation \[randomstr 100\]"
textcolor
set s {}
time {
	set r [randomstr 100]
	set t $t$r					# 1031-1057 microsecond(s) per iteration
} 1000

puts "Line: "#__LINE__
textcolor yellow
puts "Testing string append using \[set\] concatenation \[randomstr 1000\]"
textcolor
set s {}
time {
	set r [randomstr 1000]
	set t $t$r					# 1031-1057 microsecond(s) per iteration
} 1000
*/

puts "Line: "#__LINE__
textcolor yellow
puts "Testing string append using \[append\] \[randomstr 100\]"
textcolor
set s {}
time {
	set r [randomstr 100]
	append s $r					# 77 microsecond(s) per iteration with xmalloc, 86 without
} 1000

puts "Line: "#__LINE__
textcolor yellow
puts "Testing string append using \[append\] \[randomstr 1000\]"
textcolor
set s {}
time {
	set r [randomstr 1000]
	append s $r					# 77 microsecond(s) per iteration with xmalloc, 86 without
} 1000

puts "Line: "#__LINE__
textcolor yellow
puts "Testing string append using \[store\] \[randomstr 100\]"
textcolor
time {
	set r [randomstr 100]
	store add $r				# 7 microsecond(s) per iteration
} 1000
set s [store get]

puts "Line: "#__LINE__
textcolor yellow
puts "Testing string append using \[store\] \[randomstr 1000\]"
textcolor
time {
	set r [randomstr 1000]
	store add $r				# 7 microsecond(s) per iteration
} 1000
set s [store get]
puts "Line: "#__LINE__
assert $s {!= $_ ""} -v -line #__LINE__

unset r s t x -nocomplain

vars

########################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #