#################################################################################
# File:				test_apply.tcl
# Author:			M Shaw
# Purpose:			Test [stack]
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 09/Jan/2016	First version (implemented)
#################################################################################
# Notes:
# [lmap] is implemented in Tcl 8.6 https://wiki.tcl-lang.org/13920
# No memory leaks
#################################################################################

#undef map							# [map] is defined internally now
/*
undef lmap -nocomplain
proc lmap {lambda list} {
	# puts "lmap: $lambda, $list"
	set result {}
	foreach item $list {
		# puts "CALLING map: apply '$item' '$list' as\n$lambda $item"
		lappend result [apply $lambda $item]
	}
	return $result
}
*/
#################################################################################

puts "Line: "#__LINE__
# [lmap expression list]
puts [lmap {x {return [string length $x]:$x}} {a bb ccc dddd}] 
set q [lmap {x {return [string length $x]:$x}} {a bb ccc dddd}] 
#? 1:a 2:bb 3:ccc 4:dddd
puts "Result is $q"
assert $q {eq $_ "1:a 2:bb 3:ccc 4:dddd"} -v -line #__LINE__

puts "Line: "#__LINE__
set x ""
#puts [lmap {x {expr "pow($x,2) + (3*$x) - 2"}} "-4 -3 -2 -1 0 1 2 3 4"] 
set q [lmap {x {expr {($x**2) + (3*$x) - 2}}} {-4 -3 -2 -1 0 1 2 3 4}]
puts "-4 -> [expr (-4**2) + (3*-4) - 2]"
puts "-4 -> [expr (-4*-4) + (3*-4) - 2]"
puts " 2 -> [expr (2**2) + (3*2) - 2]"
#? 2 -2 -4 -4 -2 2 8 16 26
assert $q {eq $_ "2 -2 -4 -4 -2 2 8 16 26"} -v -line #__LINE__

puts "Line: "#__LINE__
set x ""
# [- [+ [** $x 2] [* 3 $x]] 2]
set q [lmap {x {[- [+ [** $x 2] [* 3 $x]] 2]}} {-4 -3 -2 -1 0 1 2 3 4}]	# Note that we have no AOPM for ** yet
assert $q {eq $_ "2 -2 -4 -4 -2 2 8 16 26"} -v -line #__LINE__
# Should be the same as the bracketed calculation and Tcl native above
set q [lmap {x {expr {$x**2 + 3*$x - 2}}} {-4 -3 -2 -1 0 1 2 3 4}]	# Note that we have no AOPM for ** yet

assert $q {eq $_ "2 -2 -4 -4 -2 2 8 16 26"} -v -line #__LINE__

# -4 -3 -2 -1 0 1 2 3 4
#assert $q {eq $_ "-78 -38 -16 -6 -2 2 12 34 74"} -v -line 	#__LINE__
#assert $q {eq $_ "-4 -3 -2 -1 0 1 2 3 4"} -v -line	#__LINE__			 # [map]


#################################################################################
puts "Line: "#__LINE__
set q [apply {x {expr $x*$x}} 5]
assert $q {eq $_ 25} -v -line #__LINE__

puts "Line: "#__LINE__
set q [apply {{x y} {expr hypot($x,$y)}} 3 4]				# PASS
puts "$q (hypot(3,4) should be 5.0)"
assert $q {== $_ 5.0} -v -line #__LINE__

puts "Line: "#__LINE__
set q [apply {{x y} {expr sqrt(sin($x))*log($y)}} 3 4]		# PASS  (VB 0.520774550824151)
puts "$q (expr sqrt(sin(3))*log(4)} should be 0.520774550824151)"
assert $q {== $_ 0.520774550824151} -v -line #__LINE__

puts "Line: "#__LINE__
set q [apply {{x y z} {expr sqrt(sin($x))*log($y)*$z}} 3 4 1.234]		# PASS  (VB 0.642635795717002)
puts "$q (expr sqrt(sin(3))*log(4)} should be 0.642635795717002)"
assert $q {== $_ 0.642635795717002} -v -line #__LINE__

/*	
textcolor yellow
puts "Checking for memory leaks ..."			# No leaks
textcolor
time {
	apply {{x y} {expr hypot($x,$y)}} 3 4
} 1000000
*/

puts "Line: "#__LINE__
# lmap {1 2 3 4 5} {expr {$0 * $0}} 		# https://wiki.tcl-lang.org/13920
set q [lmap {0 {expr {$0 * $0}}} {1 2 3 4 5}]
assert $q {eq $_ "1 4 9 16 25"} -v -line #__LINE__


puts "Line: "#__LINE__
# Lambda functions using Tcl
# https://stackoverflow.com/questions/16493986/tcl-write-lambda-and-map-using-tcl-8-6
set square {lmap {x {expr $x * $x}}}
set cube {lmap {x {expr $x * $x * $x}}}
puts [{*}$square 42]
assert [{*}$square 42] {== $_ 1764} -v -line #__LINE__
puts [{*}$cube 42]
assert [{*}$cube 42] {== $_ 74088} -v -line #__LINE__

puts "Line: "#__LINE__
set q [apply {{} {return [info level]}} {}]
puts "apply level is $q"
assert $q {== $_ 1} -v -line #__LINE__


#################################################################################
unset q square cube -nocomplain

puts "Line: "#__LINE__
undef foo -nocomplain
proc foo {} {
	set q [apply {{} {return [info level]}} {}]
	puts "apply level in \[foo\] is $q"
	assert $q {== $_ 2} -v -line #__LINE__
}
foo

undef foo -nocomplain
unset x q -nocomplain

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #