############################################################################################
# File:				test_gets.tcl
# Author:			M Shaw
# Purpose:			Test file for [proc] args option
#
# Versions:
# 1.00 04/Jul/2017 First version
# 1.01 09/Sep/2018 Added more tests after bugfix in callproc
###########################################################################################
# Notes:
###########################################################################################

undef foo -nocomplain
proc foo {args} {
	puts "args='$args'"
}
foo "The quick brown fox jumps over the lazy dog" Hello

undef foo -nocomplain
/* Should be (ActiveState)
(System32) 50 % foo "hi there" hello
args='{hi there} hello'
*/
##########################################
proc foo {a args} {
	puts "foo a is '$a'"
	puts "foo args is '$args'"
}

proc bar {a} {
	puts "bar a is '$a'"
}

foo Hello
foo Hello World
foo Hello World How {Are You} Today
newline

bar Hello
bar Hello World
bar Hello World How {Are You} Today
newline

undef foo -nocomplain

proc foo {args} {
	puts "args='$args'"
	assert $args {eq $_ $::qbf} -v -line #__LINE__
}

foo $qbf
foo "The quick brown fox jumps over the lazy dog"

undef foo -nocomplain
proc foo {args} {
	puts "args='$args'"
	assert [lindex $args 0] {eq $_ $::qbf} -v -line #__LINE__
	assert [lindex $args 1] {eq $_ hello} -v -line #__LINE__
}
foo $qbf hello

undef foo -nocomplain
proc foo {args} {
	puts "args='$args'"
	assert [lindex $args 1] {eq $_ $::qbf} -v -line #__LINE__
	assert [lindex $args 0] {eq $_ hello} -v -line #__LINE__
}

foo hello $qbf 

# FIX this one
undef foo -nocomplain
proc foo {x args} {
	puts "x='$x' args='$args'"
	assert $args {eq $_ $::qbf} -v -line #__LINE__
	assert $x {eq $_ hello} -v -line #__LINE__
}
foo hello $qbf 

undef foo -nocomplain
proc foo {args x} {
	puts "x='$x' args='$args'"
	assert $args {eq $_ $::qbf} -v -line #__LINE__
	assert $x {eq $_ hello} -v -line #__LINE__
}
foo $qbf hello 


undef foo -nocomplain
proc foo {args x} {
	puts "x='$x' args='$args'"
	assert $x {eq $_ $::qbf} -v -line #__LINE__
	assert $args {eq $_ hello} -v -line #__LINE__
}
foo hello $qbf


# args takes only one arg if in first position
# if not in the last position any remaining args are ignored
undef foo -nocomplain
proc foo {args x} {
	puts "x='$x' args='$args'"
	assert $x {eq $_ two} -v -line #__LINE__
	assert $args {eq $_ one} -v -line #__LINE__
}
foo one two three

# args in the last position takes all remaining args
undef foo -nocomplain
proc foo {x args} {
	puts "x='$x' args='$args'"
	assert $x {eq $_ one} -v -line #__LINE__
	assert $args {eq $_ "two three"} -v -line #__LINE__
}
foo one two three

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

#undef +
# Backtick command
proc ` {args} {
	#puts "x is $x at line: "#__LINE__
	set x 0
    foreach e $args {
        set x [expr $x+$e]
    }
	puts "Returning $x at line: "#__LINE__
    return $x
}

assert [` 1 2 3 4] {== $_ 10} -v -line #__LINE__

undef ` -nocomplain

undef bar foo -nocomplain
vars

puts [procs]

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #