##############################################################################
# File:				test_array_append.tcl
# Author:			M Shaw
# Purpose:			Test appending one array into another
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 02/Nov/2017	First version
##############################################################################
# Notes:
# DONE Multi-column script needs a mod to:
# A) Return an array from a proc
#	Use complex dereference ${${x}(subscript)}
# B) Append that return into an earlier array
#	Works just fine
##############################################################################
# upvar: 'a' already exists at level 1 (check proc args)
# If arg is same as the resolved arg then conflict at that level
# This might be probematic as we can't always know what arg name will be passed
# to some generic proc
# Workaround is to mangle the name and explicitly name the local var
# If no fix then DOC
# FIX upvar $_a "a" -> local var with no name
##############################################################################
option expression off
option echo on

unset b -nocomplain
array unset a -nocomplain
array unset q -nocomplain
array unset r -nocomplain

textcolor yellow
puts "Testing array append (proc) ..."
textcolor
newline

undef array_append -nocomplain

proc array_append {_a _q} {
	##################################################
	# Append one integer-indexed array onto another
	##################################################
	puts "DEBUG: Enter proc \[array_append($_a $_q)\]"
					# Testing a potential problem here...
	upvar $_a a		# $_a resolves to 'a' but will create local as $_a
	upvar $_q b
	walk 10
	vars
	#puts "DEBUG: type a: [type $a]"
	set q [array size a]
	set r [array size b]
	puts "Script DEBUG: array size a '$q'"
	puts "Script DEBUG: array size b '$r'"
	for {set i 0} {< $i $r} {++ i} {
		puts "set a([+ $q $i])  $b($i)"		# Does not update parent
		set a([+ $q $i]) $b($i)
	}
	return $_a
}

split $qbf " " -array a
assert [array size a] {== $_ 9} -v -line #__LINE__

split "Now is the time for all good men to come to the party" " " -array q
puts "array size a '[array size q]' @"#__LINE__
assert [array size q] {== $_ 13} -v -line #__LINE__
newline

set r [array_append a q]
puts "array size a '[array size a]' @"#__LINE__
assert [array size a] {== $_ 22} -v -line #__LINE__
newline

puts "type r: '[type $r]'"
textcolor cyan
for {set i 0} {< $i [array size a]} {++ i} {
	#puts "a $i: '$a($i)'"
	puts "\$a($i)=='[array item a $i <unset>]'"
}

textcolor
newline
puts "Line: "#__LINE__
assert [array size a] {== $_ 22} -v -line #__LINE__

textcolor magenta
for {set i 0} {< $i [array size $r]} {++ i} {
	# puts "r is $r"	# $r resolves to 'a' - we need to do a double resolve
	#set q $r
	#puts "r $i: '${${q}($i)}'"	# Correct syntax (variable indirection not required)
	puts "\$r($i)=='${${r}($i)}'"	# Correct syntax
	#puts "r $i: '[array item $r $i <not-set>]'"
}

textcolor
newline
unset b i -nocomplain
array unset a -nocomplain
array unset q -nocomplain
array unset r -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Built-in [array append]
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

split $qbf " " -array a
puts "Start size of a is [array size a]"
assert [array size a] {== $_ 9} -v -line #__LINE__
split "Now is the time for all good men to come to the party" " " -array b
assert [array size b] {== $_ 13} -v -line #__LINE__
set r [array append a b]
puts "Appended size of a is [array size a]"
puts "Appended size of b is [array size b]"
assert [array size a] {== $_ 22} -v -line #__LINE__
puts "New size is $r"


array unset a -nocomplain
split $qbf " " -array a
assert [array size a] {== $_ 9} -v -line #__LINE__
set r [array append a b 3]
assert [array size a] {== $_ 12} -v -line #__LINE__
puts "New size is $r"

array unset a b -nocomplain
unset r -nocomplain
undef array_append -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor green
newline
puts "* Done *"
textcolor

# EOF #