##############################################################################################
# File: 		test_array_double_deref.tcl
# Purpose:		Test cases for multi-level array dereferencing
# Author: 		M Shaw
# Language:		Ticol Tcl v1.26+
#
# Versions:
# 1.00 30/Apr/2018 First version
# 1.01 24/Oct/2019 Updated tests
##############################################################################################
# Notes: 
# Add more tests
##############################################################################################

textcolor yellow
puts "Testing array multi-level dereference ..."
textcolor
newline

##############################################################################################
# Double dereference array to standard var
set a(1) b
set b Hello

puts $$a(1)
assert $$a(1) {eq $_ Hello} -v -line #__LINE__

##############################################################################################
# Double dereference array to array using a complex dereference
set a(1) b
unset b -nocomplain
set b(1) Hello

puts ${$a(1)(1)}			# $a(1) -> "b" -> $b(1) -> "Hello"
assert ${$a(1)(1)} {eq $_ Hello} -v -line #__LINE__

##############################################################################################
# Double dereference array to array using a simple dereference
set c(0) b(2)
set b(2) a(1)
set a(1) Hello

puts $$$c(0)
assert $$$c(0) {eq $_ Hello} -v -line #__LINE__

##############################################################################################
# Double dereference array to array using a simple dereference with intermediat array
array unset a b c -nocomplain
set c b(2)
set b(2) a
set a Hello

puts $$$c
assert $$$c {eq $_ Hello} -v -line #__LINE__

##############################################################################################
# Raise error accessing invalid level of indirection
set i 0
try {
	puts $$$$c(0)
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

unset a c i -nocomplain
array unset a b c -nocomplain

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #