############################################## #############################################
# File:				test_array_of_struct.tcl
# Author:			M Shaw
# Purpose:			Test file for arrays of struct
#
# Versions:
# 1.00 02/Jul/2017 First version
###########################################################################################
# Notes:
# One element
# This is dated as of v1.27 / 2022
###########################################################################################

option echo on
option expression on

unset foo -nocomplain -const

if {is_set foo} {
	die "trapped: foo exists"
}

struct foo {
	a 20
	b 40
	c 60
}

# Setup #
puts "Line "#__LINE__
set q foo_1												# Set string to the struct instance name (we reuse later)
new foo $q												# Create a new struct instance of struct foo as foo_1
puts "\tSetting: ${q}.a as Hello_a_1 @"#__LINE__		# Display action
struct set ${q}.a "Hello_a_1"							# Set the struct field. [struct set] is required to set here
set a(1) $q												# Assign this struct to an array element
set q $a(1)												# Pull our array elment back into a referencing variable

# Tests #
puts "Line "#__LINE__
set p ${q}.a											# Get the name
puts "A\tresult: '$p' -> '[struct item $p]"				# Dereference the name. [struct item] is used here
assert [struct item $p] {eq $_ Hello_a_1} -v -line 		#__LINE_)

puts "Line "#__LINE__
puts "B\tresult: '$p' -> '[struct set $p]"				# Dereference the name. [struct set] is used here
assert [struct set $p] {eq $_ Hello_a_1} -v -line 		#__LINE_)

puts "Line "#__LINE__
puts "C\tresult: '$p' -> '[struct item ${q}.a]"			# Dereference the name. [struct item] is used here
assert [struct item ${q}.a] {eq $_ Hello_a_1} -v -line	#__LINE_)

puts "D\tresult: '$p' -> '[struct set ${q}.a]"			# Dereference the name. [struct set] is used here
assert [struct set ${q}.a] {eq $_ Hello_a_1} -v -line 	#__LINE_)

puts "E'\t\[set ${q}.a\]=\t[set ${q}.a]'"				# Returns ''
assert [set ${q}.a] {eq $_ Hello_a_1} -v -line 			#__LINE_)

puts "F\t'${q}.a'"										# Returns 'foo_N.a'
assert ${q}.a {eq $_ foo_1.a} -v -line #__LINE_)

# ??
#puts "\tG\t'$${q}.a'"									# Returns 'foo_N.a'. Should either err at struct or resolve struct
#assert $${q}.a {eq $_ Hello_a_1} -v -line #__LINE_)

puts "H\t\[struct set ${q}.a\]==\t'[struct set ${q}.a]'"	# Dereference the name. [struct set] is used here
assert [struct set ${q}.a] {eq $_ Hello_a_1} -v -line 	#__LINE_)

puts "I\t\[ofaddressb ${q}.a\]==\t'[ofaddressb ${q}.a]'"		# [ofaddressb $foo_4.a]					# Returns 'foo_N.a'
assert [struct set ${q}.a] {eq $_ Hello_a_1} -v -line	#__LINE_)

puts "J\t'[set ${q}.a]'"								# Won't work
assert [set ${q}.a] {eq $_ Hello_a_1} -v -line #__LINE_)

puts "K\t'$${q}.a'"										# Won't work. Does not additionally dereference
assert $${q}.a {> $_ 0} -v -line #__LINE_)

printf "L\t'${${q}.a}' ... "
printf "*'${q}.a' ->"	# ...
printf "'[struct item ${q}.a]'\r\n"
assert [struct item ${q}.a] {eq $_ Hello_a_1} -v -line 	#__LINE_)
	
puts "M\t\[struct set \${q}.a\] == '[struct set ${q}.a]'"					# Returns 'foo_N.a'
assert [struct set ${q}.a] {eq $_ Hello_a_1} -v -line 	#__LINE_)

# Clean up #
struct unset foo				# Our original master struct
struct unset foo_1				# Our instance
unset p q -nocomplain			# Simple vars
array unset a					# The test array

vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #