############################################################################################
# File: 			test_array_setint.tcl
# Purpose:			Test [array setint]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Oct/2019	First version
# 1.03 01/Jun/2022	Revised for changes to [loop] bounds
############################################################################################
# Notes:
# [array setint] is non-standard
# It allows an array to be set by one or more Tcl lists of singular, unmatched items
# as an integer-indexed array. The default array base is 0, change with '-base N'
############################################################################################

textcolor yellow
puts "Test \[array setint\] ..."
newline

############################################################################################
# Check bad sytax
set i 0
try {
	array setint a {}
} catch {
	set i 1
}
assert $i {eq $_ 1} -v -line #__LINE__

############################################################################################
# Add singular list
puts "Line: "#__LINE__
array setint a {Hi there how are you today?}
assert [array exists a] {eq $_ 1} -v -line #__LINE__
assert [array size a] {eq $_ 6} -v -line #__LINE__

loop i 0 [array size a] {
	puts "array a: $i -> '$a($i)'"
}

assert $a(0) {eq $_ Hi}
assert $a(1) {eq $_ there}
assert $a(2) {eq $_ how}
assert $a(3) {eq $_ are}
assert $a(4) {eq $_ you}
assert $a(5) {eq $_ today?}

puts "Line: "#__LINE__
#array walk a
array unset a
assert [array exists a] {eq $_ 0} -v -line #__LINE__

############################################################################################
# 2 lists 
array setint a {Hi there how are you today?} {quite well thank you very much}
assert [array exists a] {eq $_ 1} -v -line #__LINE__
assert [array size a] {eq $_ 12} -v -line #__LINE__

loop i 0 [array size a] {
	puts "array a: $i -> '$a($i)'"
}
puts "Line: "#__LINE__
#array walk a
assert [array exists a] {eq $_ 1} -v -line #__LINE__
assert $a(0) {eq $_ Hi}
assert $a(1) {eq $_ there}
assert $a(2) {eq $_ how}
assert $a(3) {eq $_ are}
assert $a(4) {eq $_ you}
assert $a(5) {eq $_ today?}
assert $a(6) {eq $_ quite}
assert $a(7) {eq $_ well}
assert $a(8) {eq $_ thank}
assert $a(9) {eq $_ you}
assert $a(10) {eq $_ very}
assert $a(11) {eq $_ much}
array unset a
assert [array exists a] {eq $_ 0} -v -line #__LINE__

############################################################################################
# MAN file example
array setint a {one two three} {four five six}
loop i 0 [array size a] {
	puts "array a: $i -> '$a($i)'"
}
newline

array setint a {THREE FOUR} -base 2
loop i 0 [array size a] {
	puts "array a: $i -> '$a($i)'"
}

puts "Line: "#__LINE__
array setint a {NINE TEN} -base 8
puts "\[array size a\]=[array size a]"
loop i 8 10 {		# 8..9 (1)
	puts "array a: $i -> '[array item a $i]'"
}
array unset a
assert [array exists a] {eq $_ 0} -v -line #__LINE__

############################################################################################
# Check for conflict with -args
puts "Line: "#__LINE__
array setint a {-THREE -FOUR} -const -nocase -base 2
assert [array exists a] {eq $_ 1} -v -line #__LINE__
assert [array size a] {eq $_ 2} -v -line #__LINE__
loop i 2 4 {		# 2..3
	puts "array a: $i -> '$a($i)'"
}
puts "Line: "#__LINE__
puts $a(2)
puts "Line: "#__LINE__
assert $a(2) {eq $_ "-THREE"} -v -line #__LINE__
assert $a(3) {eq $_ "-FOUR"} -v -line #__LINE__
assert [is_const a] {eq $_ 1} -v -line #__LINE__ 
set i 0
try {
	array unset a
} catch {
	set i 1
}
assert $i {eq $_ 1} -v -line #__LINE__

array unset a -const
unset i

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #