#########################################################################
# Name:				array_sort.tcl
# Purpose:			Testing [array sort -array x]
# Author:			M Shaw
# Requires:			Ticol Tcl 1.22+
#
# Versions:
# 1.00 22/Dec/2016	First version
# 1.01 05/Jan/2017	Added array sort -name and relevant tests
# 1.02 22/Jun/2017	Replace [checkex] with [assert]
#########################################################################
# Notes:
#########################################################################
option echo on
option expression off

# For re-entrancy loops
array unset sorted -nocomplain
array unset unsorted -nocomplain

textcolor yellow
puts "Testing \[array sort\] ..."
textcolor

/*
array sort env -array out
set s [array size out]
for {set i 0} {[< $i $s]} {incr i} {
	printf "%4i:\t%-40.40s\r\n" $i $out($i)
} 
*/

array set unsorted {
	The 	orange
	quick	yellow
	brown	blue
	fox 	black
	jumps	magenta
	over 	white
	the 	cyan
	lazy	green
	dog		red	
}

/*
set s [array size unsorted]
for {set i 0} {[< $i $s]} {incr i} {
	printf "%4i:\tvalue: %-40.40s\r\n" $i $unsorted($i)
} //*/

textcolor yellow
puts "array foreach unsorted val ss"
textcolor
set i 0
array foreach unsorted val ss {
	printf "%4i:\tvalue: \$unsorted(%s) = %-40.40s\r\n" $i $ss $val
	++ i
}


/* Output:
   0:   value: $unsorted(the) = cyan
   1:   value: $unsorted(brown) = blue
   2:   value: $unsorted(jumps) = magenta
   3:   value: $unsorted(lazy) = green
   4:   value: $unsorted(fox) = black
   5:   value: $unsorted(quick) = yellow
   6:   value: $unsorted(dog) = red
   7:   value: $unsorted(over) = white
*/

textcolor yellow
puts "array sort unsorted -array sorted"
textcolor
array sort unsorted -array sorted
set s [array size sorted]
for {set i 0} {[< $i $s]} {incr i} {
	printf "%4i:\tvalue: %-40.40s\r\n" $i $sorted($i)
}


textcolor yellow
puts "array sort unsorted -array sorted -names"
textcolor
array sort unsorted -array sorted -names
set s [array size sorted]
for {set i 0} {[< $i $s]} {incr i} {
	printf "%4i:\tname: %-40.40s\r\n" $i $sorted($i)
}

textcolor yellow
puts "for {set i 0} {[< $i $s]} {incr i} {"
textcolor
for {set i 0} {[< $i $s]} {incr i} {
	printf "%4i:\tref: %s =>\t%-40.40s\r\n" $i $sorted($i) $unsorted($sorted($i))
}
/* output
   0:   ref: brown =>   blue
   1:   ref: dog =>     red
   2:   ref: fox =>     black
   3:   ref: jumps =>   magenta
   4:   ref: lazy =>    green
   5:   ref: over =>    white
   6:   ref: quick =>   yellow
   7:   ref: The =>     orange
   8:   ref: the =>     cyan
*/   

newline
textcolor yellow
puts "Tests..."
textcolor
assert [&& [eq $sorted(0) brown] [eq $unsorted($sorted(0)) blue]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(1) dog] [eq $unsorted($sorted(1)) red]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(2) fox] [eq $unsorted($sorted(2)) black]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(3) jumps] [eq $unsorted($sorted(3)) magenta]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(4) lazy] [eq $unsorted($sorted(4)) green]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(5) over] [eq $unsorted($sorted(5)) white]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(6) quick] [eq $unsorted($sorted(6)) yellow]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(7) The] [eq $unsorted($sorted(7)) orange]] {== $_ 1} -v -line #__LINE__
assert [&& [eq $sorted(8) the] [eq $unsorted($sorted(8)) cyan]] {== $_ 1} -v -line #__LINE__

array unset sorted -nocomplain
array unset unsorted -nocomplain
unset i s -nocomplain

vars

###############################################################
newline
textcolor green
puts "* Done *"
textcolor


# EOF #