######################################################################################
# File: 			array_to_string.tcl
# Purpose:			Test for [array_to_string]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 26/Sep/2018	First version
# 1.01 01/Nov/2019	Updated
######################################################################################
# Notes:
# ADD Test for full range of arguments
#	string [array_to_string arrayname ?-join string? 
#		?-start element? ?-stop element? ?-stopstr string? ?-nocase?]
#
#	integer [string_to_array string arrayname ?-start element? 
#		?-chars|-asc? ?-append? ?-stopchar char?]
######################################################################################

option expression off
unset a -nocomplain

## array_to_string #############################################################
textcolor yellow
puts "array_to_string"
textcolor

array set a {
	0 H
	1 e
	2 l
	3 l
	4 o
}
puts [array_to_string a]
assert [array_to_string a] {eq $_ Hello} -v -line #__LINE__

puts "'[array_to_string a -stopstr l]'"
assert [array_to_string a -stopstr l] {eq $_ He} -v -line #__LINE__
assert [array_to_string a -stopstr o] {eq $_ Hell} -v -line #__LINE__
assert [array_to_string a -stopstr x] {eq $_ Hello} -v -line #__LINE__

array unset a -nocomplain

#split <string> mask ?-array var?

split $qbf " " -array a
array walk a
newline

puts [array size a]
assert [array size a] {== $_ 9} -v -line #__LINE__

set s [array_to_string a -join " "]
puts "s is '$s'"
assert $s {eq $_ $::qbf} -v -line #__LINE__
unset s -nocomplain 

set s [array_to_string a -join " " -stopstr The]
puts "s is '$s'"
assert $s {eq $_ ""} -v -line #__LINE__
unset s -nocomplain

set s [array_to_string a -join " " -stopstr the -nocase]
puts "s is '$s'"
assert $s {eq $_ ""} -v -line #__LINE__
unset s -nocomplain

set s [array_to_string a -join " " -stopstr quick]
puts "s is '$s'"
assert $s {eq $_ "The"} -v -line #__LINE__
unset s -nocomplain

set s [array_to_string a -join " " -stopstr jumps]
puts "s is '$s'"
assert $s {eq $_ "The quick brown fox"} -v -line #__LINE__
unset s  -nocomplain
newline

## string_to_array ############################################################
#	integer [string_to_array string arrayname ?-start element? 
#		?-chars? ?-asc? ?-append? ?-stopchar char?]
###############################################################################
textcolor yellow
puts "string_to_array"
textcolor
array unset a -nocomplain
set count [string_to_array $qbf a -chars]			# Split by characers
puts "Count is $count"
puts "a(42)-> '$a(42)"
assert $count {eq $_ 43} -v -line #__LINE__
assert $a(39) {eq $_ " "} -v -line #__LINE__
assert $a(40) {eq $_ "d"} -v -line #__LINE__
assert $a(41) {eq $_ "o"} -v -line #__LINE__
assert $a(42) {eq $_ "g"} -v -line #__LINE__
array unset a -nocomplain
#array walk a

set count [string_to_array $qbf a -chars -asc]		# Split by characers
assert $count {eq $_ 43} -v -line #__LINE__
assert $a(42) {eq $_ 103} -v -line #__LINE__


# DONE #
array unset a -nocomplain
unset count s -nocomplain

vars

#############################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #