####################################################################################
# File: 			test_arrays.tcl
# Purpose:			Test for arrays
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 12/Dec/2016	First version
# 1.01 09/Jul/2017	Implemented [assert], fixed some tests
# 1.01 25/Nov/2019	Added some [array item] tests
####################################################################################
# Notes: 
####################################################################################

option echo on
option expression off

################################################################################
# Arrays
################################################################################
textcolor yellow
puts "Testing arrays ..."
textcolor
newline

array unset a -nocomplain
unset s str -nocomplain

set s Hello
set str There
set a(1) Hello
set a(two) There
set a("with spaces") Fox
set q a("with spaces")
#puts "Eval abc\${a(1)}def"
#puts abc${a(1)}def
#puts abc${$a(1)}def

# Brace Delimiters
textcolor yellow
puts "Testing brace delimiters in simple var interpolation ..."
textcolor

assert abc${s}def {eq $_ abcHellodef} -v -line #__LINE__
assert abc${str}def {eq $_ abcTheredef} -v -line #__LINE__
assert abc${s}.def {eq $_ abcHello.def} -v -line #__LINE__
assert abc${str}.def {eq $_ abcThere.def} -v -line #__LINE__

################################################################################
# Simple arrays
################################################################################
textcolor yellow
puts "Testing simple array use ..."
textcolor

assert $a(1) {eq $_ Hello} -v -line #__LINE__
assert $a(two) {eq $_ There} -v -line #__LINE__
assert $a("with spaces") {eq $_ Fox} -v -line #__LINE__

# Double-dereference
# $a(two) -> $There -> "def"
set There def
assert "abc${$a(two)}ghi" {eq $_ abcdefghi} -v -line #__LINE__	# 

# Double-dereference
set Hello def
# $a(1) -> $hello -> "def"
assert "abc${$a(1)}ghi" {eq $_ abcdefghi} -v -line #__LINE__	# 

# Redef array element
set $a("with spaces") def  # sets var fox to "def"
# Thus we dereference $a("with spaces") -> $fox -> "def"
assert "abc${$a("with spaces")}ghi" {eq $_ abcdefghi} -v -line #__LINE__	# Double-dereference

################################################################################
# Brace delimiters
################################################################################
textcolor yellow
puts "Testing brace delimiters in array interpolation ..."
textcolor

# Braced Arrays
# 2017/May/07 - Some of these tests are incorrect...
/*
set a(1) Hello
set a(two) There
set a("with spaces") Fox
puts "a(1) is $a(1)"
*/

assert abc$a(1)def {eq $_ abcHellodef} -v -line #__LINE__				# Correct
assert abc${a(1)}def {eq $_ abcHellodef} -v -line #__LINE__				# Correct - fails on new, nested
assert abc${a(two)}def {eq $_ abcTheredef} -v -line #__LINE__			# Correct
assert abc${a("with spaces")}def {eq $_ abcFoxdef} -v -line #__LINE__	# Correct

set i 1
assert abc$a($i)def {eq $_ abcHellodef} -v -line #__LINE__

puts "Line: "#__LINE__
set i t
#             $a(two)
assert abc$a(${i}wo)def {eq $_ abcTheredef} -v -line #__LINE__

puts "Line: "#__LINE__
set i t
# Dubious syntax: do we want to cover ${array(ss)} ?
# Or ${array}(ss)
#             $a(two)
#assert abc${a(${i}wo)}def {eq $_ abcTheredef} -v -line #__LINE__			# Astoundingly, this passes! under non-recursive

# SHOULD FAIL
puts "Line: "#__LINE__
textcolor darkyellow
puts "Catch fail case"
textcolor
set i t
catch {
#             $a(two)
	assert abc$a{(${i}wo)}def {eq $_ abcTheredef} -v -line #__LINE__			# Astoundingly, this passes!
}

# SHOULD FAIL
puts "Line: "#__LINE__
textcolor darkyellow
puts "Catch fail case"
textcolor
set i t
catch {
	assert abc${a}(${i}wo)def {eq $_ abcTheredef} -v -line #__LINE__			# Should trap erroneous array name dereference
}
assert abc${a}(${i}wo)def {eq $_ abc<array>(two)def} -v -line #__LINE__			# Bad syntax should give this

set a("a b") hello
assert $a(a b) {eq $_ hello} -v -line #__LINE__
assert $a("a b") {eq $_ hello} -v -line #__LINE__

# Problem cases ################################################################
textcolor yellow
puts "\nProblem cases"
textcolor
puts "Line: "#__LINE__
puts "<none>"
################################################################################
# Test erroneous duplicate set
textcolor yellow
puts "\nDuplicate set"
textcolor
array unset a -nocomplain
array set a {
	1 One
	1 Two
	1 Three
}
array walk a
assert $a(1) {eq $_ Three} -v -line #__LINE__
assert [array size a] {== $_ 1} -v -line #__LINE__

array unset a -nocomplain
array set a {
	One One
	One Two
	One Three
}
array walk a
assert $a(One) {eq $_ Three} -v -line #__LINE__
assert [array size a] {== $_ 1} -v -line #__LINE__


################################################################################
# Check [array item] defaults where array does not exist
assert [array item foobar 0 -nocomplain] {eq $_ {}} -v -line #__LINE__
assert [array item foobar 0 "notfound" -nocomplain] {eq $_ notfound} -v -line #__LINE__

array unset a -nocomplain
array unset q -nocomplain
unset i s Fox hello Hello str There  -nocomplain

################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #