#################################################################################################
# File:				test_assert.tcl
# Author:			M Shaw
# Purpose:			Test [assert]
#
# Versions:
# 1.00 16/Jul/2018 First version
#################################################################################################
# Notes:
# A few sanity checks on [assert]
#################################################################################################
option expression off

textcolor yellow
puts "Line: "#__LINE__
textcolor
newline

set line #__LINE__
set r 1
catch {
	puts "Checking \[assert\] at line "#__LINE__
	assert		# Should raise exception (insufficient args)
	set r 0
}
if {!= $r 1} {
	die "Test failed at line $line"
} else {
	textcolor green
	puts "Line $line PASS"
	textcolor
}
newline

/*
# Syntax amended 05/Nov/2019
set line #__LINE__
set r 1
catch {
	assert 1		# Should raise exception (insufficient args)
	set r 0
}
if {!= $r 1} {
	die "Test failed at line $line"
} else {
	textcolor green
	puts "Line $line PASS"
	textcolor
}
newline # */

set line #__LINE__
set r 1
catch {
	assert 1 0		# Test should fail and trip [catch]
	set r 0
}
if {!= $r 1} {
	die "Test failed at line $line"
} else {
	textcolor green
	puts "Line $line PASS"
	textcolor
}
newline

#################################################################################################
# Mixed arg count
puts "Line: "#__LINE__

set i 0
try {
	assert 1 -line #__LINE__ 
} catch {
	set i 1
}
assert $i {== $_ 0} -v -line #__LINE__

assert 1 {== $_ 1} -line #__LINE__ 

set i 0
try {
	assert 0 -line #__LINE__ 
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

assert 0 {== $_ 0} -line #__LINE__ 

# Verbose
set i 0
try {
	assert 1 -v -line #__LINE__ 
} catch {
	set i 1
}
assert $i {== $_ 0} -v -line #__LINE__

assert 1 {== $_ 1} -v -line #__LINE__ 

set i 0
try {
	assert 0 -v -line #__LINE__ 
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

assert 0 {== $_ 0} -v -line #__LINE__ 

#################################################################################################

puts "Line: "#__LINE__
assert - 1 -v -line #__LINE__				# Valid if 2nd line is true
assert "" 1 -v -line #__LINE__				# Valid if 2nd line is true
assert {} 1 -v -line #__LINE__				# Valid if 2nd line is true
assert 0 1 -v -line #__LINE__				# ?
assert [! 0] 1 -v -line #__LINE__			# ?
assert 1 1 -v -line #__LINE__				#

puts "Line: "#__LINE__
assert 0 {== $_ 0} -v -line #__LINE__		#
assert 1 {== $_ 1} -v -line #__LINE__		#
assert [! 1] 1 -v -line #__LINE__			#
#assert 0 0 -v -line #__LINE__				# Can only be tested via [catch]
assert [! 1] {== $_ 0} -v -line #__LINE__	#

unset line r -nocomplain
vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #