#################################################################
# File:				test_base64.tcl
# Purpose:			Test base 64 encoding/decoding
# Author:			M Shaw
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 07/Jan/2016	First version
# 1.01 28/Dec/2016	Revised with bugfix for single char -encode
# 1.02 22/Jun/2017	Replace [checkex] with [assert]
#################################################################
# Notes:
# Reference site: https://www.base64encode.org
#################################################################
option echo on
option expression off

textcolor yellow
puts "Testing \[base64\] ..."
textcolor
newline

unset a -nocomplain
set a [base64 "Hello world!" -encode]
set a [base64 $a -decode]
assert $a {eq $_ "Hello world!"} -v -line #__LINE__

unset s -nocomplain
# s (known) is "cw=="
set a [base64 "s" -encode]		# Might have to change this ambiguity
assert $a {eq $_ "cw=="} -v -line #__LINE__		# BROKEN is: "cwDycw=="
set a [base64 $a -decode]
assert $a {eq $_ "s"} -v -line #__LINE__


# Check safe with x0 "" call
set a [base64 "" -encode]
puts "a is '$a'"
assert $a {eq $_ ""} -v -line #__LINE__
set a [base64 $a -decode]
assert $a {eq $_ ""} -v -line #__LINE__

# Check safe with x1 char call
unset p -nocomplain

set a [base64 p -encode]
assert $a {eq $_ cA==} -v -line #__LINE__			# External validation
set a [base64 $a -decode]
assert $a {eq $_ p} -v -line #__LINE__

# Check safe with x2 char call
set a [base64 pq -encode]
assert $a {eq $_ cHE=} -v -line #__LINE__			# External validation
set a [base64 $a -decode]
assert $a {eq $_ pq} -v -line #__LINE__

# Check safe with x3 char call
set a [base64 pqr -encode]
assert $a {eq $_ cHFy} -v -line #__LINE__			# External validation
set a [base64 $a -decode]
assert $a {eq $_ pqr} -v -line #__LINE__

# Check safe with x4 char call
set a [base64 pqrs -encode]
assert $a {eq $_ cHFycw==} -v -line #__LINE__			# External validation
set a [base64 $a -decode]
assert $a {eq $_ pqrs} -v -line #__LINE__

# Check safe with x5 char call
set a [base64 pqrst -encode]
assert $a {eq $_ cHFyc3Q=} -v -line #__LINE__			# External validation
set a [base64 $a -decode]
assert $a {eq $_ pqrst} -v -line #__LINE__

## WRAP ###############################################################
# The quick brown fox jumps over the lazy dog
# External source https://www.base64encode.org
# VGhlIHF1aWNrIGJyb3duIGZveCBqdW1wcyBvdmVyIHRoZSBsYXp5IGRvZw==
# Ticol
# VGhlIHF1aWNrIGJyb3duIGZveCBqdW1wcyBvdmVyIHRoZSBsYXp5IGRvZw==
# base64 string|varName -encode|-decode ?-format width?
#######################################################################
set line #__LINE__
assert [base64 $qbf -encode -format 10] {eq $_ "VGhlIHF1aWNr
IGJyb3duIGZv
eCBqdW1wcyBv
dmVyIHRoZSBs
YXp5IGRvZw=="
} -v -line #__LINE__

set line #__LINE__
assert [base64 $qbf -encode -format 4] {eq $_ "VGhl
IHF1
aWNr
IGJy
b3du
IGZv
eCBq
dW1w
cyBv
dmVy
IHRo
ZSBs
YXp5
IGRv
Zw=="
} -v -line #__LINE__

unset a line -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #