############################################################################################
# File: 			test_baseconv.tcl
# Purpose:			Test [baseconv]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 15/Oct/2019	First version
############################################################################################
# Notes:
# integer [convbase number base]
# integer [baseconv number base]
############################################################################################

textcolor yellow
puts "Testing \[baseconv\] ... @"#__LINE__
textcolor
newline

# [baseconv] returns raw results without any type prefix such as "0x" or "0b"
assert [baseconv 10 2] {== $_ 1010} -v -line #__LINE__
assert [baseconv 2423423 2] {== $_ 1001001111101001111111} -v -line #__LINE__
assert [baseconv 2423423 8] {== $_ 11175177} -v -line #__LINE__
assert [baseconv 10 16] {eq $_ a} -v -line #__LINE__
assert [baseconv 10 16 -ucase] {eq $_ A} -v -line #__LINE__
newline

textcolor yellow
puts "Testing \[convbase\] ... @"#__LINE__
textcolor
newline

# [convbase] returns a decimal value
assert [convbase 499602D2 16] {== $_ 1234567890} -v -line #__LINE__  # (Base 10)
assert [convbase 0 2] {== $_ 0} -v -line #__LINE__  # (Base 10)
assert [convbase 1 2] {== $_ 1} -v -line #__LINE__  # (Base 10)
assert [convbase -1 2] {== $_ 1} -v -line #__LINE__  # (Base 10) (signs are discarded)
assert [convbase 1010 2] {== $_ 10} -v -line #__LINE__  # (Base 10)
assert [convbase 1001001111101001111111 2] {== $_ 2423423} -v -line #__LINE__
assert [convbase 11175177 8] {== $_ 2423423} -v -line #__LINE__
assert [convbase A 16] {== $_ 10} -v -line #__LINE__  # (Base 10)
assert [convbase a 16] {== $_ 10} -v -line #__LINE__  # (Base 10)
assert [convbase f00 16] {== $_ 3840} -v -line #__LINE__  # (Base 10)
newline

textcolor yellow
puts "Testing \[baseconv\] -> \[convbase\] round trips ... @"#__LINE__
textcolor
newline

# Round trip tests baseconv->convbase
assert [convbase [baseconv 0 2] 2] {== $_ 0} -v -line #__LINE__  #
assert [convbase [baseconv 10 2] 2] {== $_ 10} -v -line #__LINE__  #

assert [convbase [baseconv 0 2] 2] {== $_ 0} -v -line #__LINE__  #
assert [convbase [baseconv 1 2] 2] {== $_ 1} -v -line #__LINE__  #
puts [baseconv 82884990982 2] 
assert [baseconv 82884990982 2] {eq $_ 1001101001100010101001001100000000110} -v -line #__LINE__  # 
puts [convbase 1001101001100010101001001100000000110 2]	# FAIL
assert [convbase 1001101001100010101001001100000000110 2] {eq $_ 82884990982} -v -line #__LINE__  #
assert [convbase [baseconv 82884990982 2] 2] {== $_ 82884990982} -v -line #__LINE__ # 1280612358
assert [convbase [baseconv 980345018739710377 2] 2] {== $_ 980345018739710377} -v -line #__LINE__ 
assert [convbase [baseconv 9223372036854775807 2] 2] {== $_ 9223372036854775807} -v -line #__LINE__  # INT_MAX
newline

textcolor yellow
puts "Testing \[convbase\] -> \[baseconv\] round trips ... @"#__LINE__
textcolor
newline

# Round trip tests convbase->baseconv
assert [baseconv [convbase 499602D2 16] 16] {eq $_ 499602d2} -v -line #__LINE__  # (Base 10)
assert [baseconv [convbase 499602D2 16] 16 -ucase] {eq $_ 499602D2} -v -line #__LINE__  # (Base 10)
assert [baseconv [convbase 777777777777777777777 8] 8] {eq $_ 777777777777777777777} -v -line #__LINE__  # INT_MAX

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #



