###########################################################################
# Name:				test_binary.tcl
# Purpose:			Example for [binary]
# Author:       	M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Dec/2016	First version
# 1.01 22/Jul/2017	Replaced [checkex] with [assert]
# 1.02 18/Jul/2018	Fix for changes in [binary] re: '-var' arg
###########################################################################
# Notes:
# TODO: Add more tests
###########################################################################
option expression off
option echo on

textcolor yellow
puts "Testing \[binary\] ..."
textcolor
newline

# memcpy will copy to a standard Tcl var and will assign memory + new var if required

set a 23
binary memcpy b [addressofb a] 2		# Copy standard var data-block by address to var b
puts "\$b=='$b'"
puts "binary length=[binary length b]"
puts "string length=[string length b]"
set line #__LINE__
assert $b {eq $_ 23} -v -line #__LINE__

binary set a \xfe\xffhello\x21\x00\x22world\x02\xff\xfeI-am-invisible -length 18
#puts "'$a'"
#puts "binary  length=[binary length a]"
#puts "tohex:  '[binary tohex a]'"
#puts "base64: '[binary base64 a]'"

assert [binary length a] {== $_ 18} -v -line #__LINE__
# FIX returns hex 'a'
puts "'[binary tohex a -var]' a='$a'"
assert [binary tohex a -var] {eq $_ feff68656c6c6f210022776f726c6402fffe} -v -line #__LINE__
assert [binary base64 a] {eq $_ "YQ=="} -v -line #__LINE__
assert [binary base64 a -var] {eq $_ "/v9oZWxsbyEAIndvcmxkAv/+"} -v -line #__LINE__

# NULL cases 
assert [binary length ""] {== $_ 0} -v -line #__LINE__		# empty
assert [binary length "Hello"] {== $_ 5} -v -line #__LINE__	# literal
assert [binary tohex ""] {eq $_ ""} -v -line #__LINE__
assert [binary tohex "Hello"] {eq $_ "48656c6c6f"} -v -line #__LINE__
assert [binary base64 "Hello"] {eq $_ "SGVsbG8="} -v -line #__LINE__
set s Hello
assert [binary base64 $s] {eq $_ "SGVsbG8="} -v -line #__LINE__
assert [binary base64 s] {eq $_ "cw=="} -v -line #__LINE__

unset a b line s -nocomplain
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #