######################################################################################
# File:			test_binary_encode.tcl
# Author:		M Shaw
# Purpose:		Test various [binary] coding ops
# Language:		Ticol Tcl
#
# Versions:
# 1.00 01/Mar/2017	First version (test results)
# 1.01 22/Jul/2017	Replaced [checkex] with [assert]
# 1.02 18/Jul/2018	Fix for changes in [binary] re: '-var' arg
######################################################################################
# Notes:
# FIX [option escape] status may be important here
# We cannot 'chain' commands which return binary data as Ticol strings are not normally
# 'counted' strings. [binary] support is limited
######################################################################################
option echo on
option expression off
option escape on		# Force ON in case launched via [source] from the CLI
option

textcolor yellow
puts "Test \[binary\] encoding operations...\n"
textcolor

set s 618221d18003637476a106800458b5186ca209800100810100820100a31580010081100000000000000000000000000000000011

binary fromhex $s q
#puts "[string length $s] [binary length q] bool=[binary base64 -encode q]"

puts "[string length $s] bool=[binary fromhex $s q] [binary length q]"

#puts $q

#104 52 LWVuY29kZQ==

set r [binary tohex q -var]
assert [eq $s $r] {eq $_ 1} -v -line #__LINE__

# Test null sequences

# Fault here \x0d
echo "Setting s to: '"abc\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x00def"'"
binary set s "abc\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x00def"	# MPP will normalise ALL \x0 to \x00
# binary set s "abc\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00def"
puts "Dump of NULL escaped string s"
dumpmem [addressofb s] 32				# Dump binary contents
#stop
puts "Length=[binary length s] (should be 17)"
newline

puts "binary tohex s='[binary tohex s -var]' (will truncate at first NULL)"
puts "from hex s='[fromhex [binary tohex s]]'"
puts "Line "#__LINE__
puts "Should be:      6162630000000000000000000000646566"
puts "Line "#__LINE__
puts "\[binary base64 s\]='[binary base64 s]'"		# YWJjAAAAAAAAAAAAAABkZWY=
puts "Line "#__LINE__
puts "Match?: [eq [binary base64 s] "YWJjAAAAAAAAAAAAAABkZWY="]"
puts "Line "#__LINE__
puts "\[binary base64 s -var\]->'[binary base64 s -var]'"
#                                  YWJjAHgwAHgwAHgwAHgwAHgwAGRlZg==
assert [eq [binary base64 s -var] "YWJjAAAAAAAAAAAAAABkZWY="] {== $_ 1} -v -line #__LINE__
newline

# Round trip encode/decode # # # # # # # # # # #
puts "Line "#__LINE__
set s [encrypt $qbf password]	# To base64
puts "Line "#__LINE__
puts "s1==$s @ line "#__LINE__
binary base64 s -var -decode q		# To binary
set s [binary tohex q -var]			# Bin to hex
puts "s2==$s @ line "#__LINE__
binary fromhex $s q				# Hex to bin
set s ""						# Clear to ensure we're not misled on failure
binary base64 q -var -encode s		# Back to base64
puts "Line "#__LINE__
puts "s3==$s"
set q [decrypt $s password]		# Base64 to ASCII
puts "Line "#__LINE__
puts "Password=$q"
assert [eq $q $qbf] {== $_ 1} -v -line #__LINE__

unset q r s -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #