#####################################################################
# File:				test_bincvt.tcl
# Author:			M Shaw
# Purpose:			Binary conversion procs to test split and join
# Versions:
# 1.00 18/Apr/2015 First version
# 1.01 19/Nov/2016 Bugfixes
#####################################################################
# Notes:
# Already implemented as inttobin and bintoint but useful for testing
#####################################################################
option expression off
option echo on

if {! [defined fromBinary] } {
	proc fromBinary { digitString } {
		set r 0
		foreach d [split $digitString {}] {
			incr r $r
			incr r $d
		}
			return $r
		}
}

# Requires binary scan and scan
if {! [defined bits2int] } {
	proc bits2int {bits} {
	   #returns integer equivalent of a bitlist
	   set bits [format %032s [join $bits {}]]
#	puts "Bits is: $bits"   
	   binary scan [binary format B* $bits] I1 x
	   set x
	}
}

puts "Running tests ..."
puts "Line: "#__LINE__
puts "fromBinary 21162: [fromBinary 0101001010101010]"
assert [fromBinary 0101001010101010] {== $_ 21162} -v -line #__LINE__

puts "Line: "#__LINE__
puts "fromBinary 21162 0x52aa: 0x[format %x [fromBinary 0101001010101010]]"
assert [fromBinary 0101001010101010] {== $_ "0x52aa"} -v -line #__LINE__

puts "Line: "#__LINE__
puts "Frombinary: [fromBinary [inttobin 999]]"
assert [fromBinary [inttobin 999]] {== $_ 999} -v -line #__LINE__

puts "Line: "#__LINE__
puts "Literal 0xffee+1 \[inttohex \[expr 0xffee+1\]\]->[inttohex [expr 0xffee+1]]"
assert [inttohex [expr 0xffee+1]] {eq $_ "0xffef"} -v -line #__LINE__

puts "Line: "#__LINE__
puts "Literal 0b1010101+1: [inttobin [expr 0b1010101+1]]"
assert [inttobin [expr 0b1010101+1]] {== $_ 0b1010110} -v -line #__LINE__

undef bits2int -nocomplain
undef fromBinary -nocomplain

 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #