#################################################################################
# File:				test_brace.tcl
# Author:			M Shaw
# Purpose:			Test cases braces
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 30/Dec/2017	First version
# 1.01 04/Jan/2023	Check malformed var with scope symbol :
#################################################################################
# Notes:
#################################################################################
/*
puts "Line: "#__LINE__
puts $b					# (var) a (correct)
puts "Line: "#__LINE__
puts $$b				# Hello (correct)
puts "Line: "#__LINE__
puts $${b}				# a FAIL  (dereferences b)
puts "Line: "#__LINE__
puts ${b}
puts "Line: "#__LINE__
puts ${${b}}			# {a} FAIL (dereferences b)
*/

option expression off
option echo on

textcolor yellow
puts "Testing braces ..."
textcolor
newline

set a 1
set b 2
set c 3
set d a


assert $a {eq $_ 1} -v -line #__LINE__					# -> a
assert ${a} {eq $_ 1} -v -line #__LINE__					# -> a

assert $$d {eq $_ 1} -v -line #__LINE__					# -> a
assert ${$d} {eq $_ 1} -v -line #__LINE__					# -> a

assert ${b}${a} {eq $_ 21} -v -line #__LINE__				# -> a,b
assert ${c}${b}${a} {eq $_ 321} -v -line #__LINE__				# -> a,b

set " " Space!
assert ${ } {eq $_ Space!} -v -line #__LINE__				# -> a,b

set b1 1
set c1 Resolved 


assert ${c{${b${a}}}} {eq $_ Resolved} -v -line #__LINE__			# -> a,b,c

set b1 2
set c2 3
#${a}                     -> a
assert ${b{$a}} {eq $_ 2} -v -line #__LINE__					# -> a -> b1 -> 2
assert ${c${b${a}}} {eq $_ 3} -v -line #__LINE__					# -> a -> b1 -> c2 -> 3

unset a b c -nocomplain


# [unwrap]
set s "{Hello}"
assert [unwrap $s] {eq $_ Hello} -v -line #__LINE__
set s " \t {Hello} \t \t"
assert [unwrap $s] {eq $_ Hello} -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# 01/Jan/2018 - Var resolve inconsistencies
# Does not print error in CGI with option echo on
# (11997) eval: No variable '{::_EXENAME}' at level 0
#const _EXENAME "TestProg"
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# We use braces with $ and variables as a special case to resolve
# So ${varname} is valid
# And {$varname} also is valid for $
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

set _EXENAME "TestProg"
puts "Hello $::_EXENAME world"				# Works
assert "Hello $::_EXENAME world" {eq $_ "Hello TestProg world"} -v -line #__LINE__
puts "Hello ${::_EXENAME}world"			# FIX (12014) eval: No variable '{::_EXENAME}' at level 0
assert "Hello ${::_EXENAME}world" {eq $_ "Hello TestProgworld"} -v -line #__LINE__
puts "Hello {$::_EXENAME}world"				# PASS Resolves but leaves braces intact

# FIX? (ActiveState will attempt to resolve same e.g. braced var)
#assert "Hello {$::_EXENAME}world" {eq $_ "Hello TestProgworld"} -v -line #__LINE__
#(System32) 76 % puts "Hello {$::_EXENAME}world"
#can't read "::_EXENAME": no such variable
# ActiveState also leaves braces intact
#(System32) 78 % puts "Hello {$::_EXENAME}world"
#Hello {foo}world

set r 0
try {
	puts "Hello ${::_EXENAME} world"		# Does not work prior to 16/Jul/2018
} catch {
	set r 1									# Set to 1 on error
}
assert $r {== $_ 0} -v -line #__LINE__

#assert {$::_EXENAME} {eq $_ $::_EXENAME} -v -line #__LINE__		# assert bug ?
# Eval braced var...
# DONE FIX FAILS - must be quoted to keep literal braces
# \{ \} does not work
assert "{$::_EXENAME}" {eq $_ "{TestProg}"} -v -line #__LINE__
# Double-bracing also works OK
assert "{$::_EXENAME}" {eq $_ {{TestProg}}} -v -line #__LINE__

#################################################################################
# Bogus/malformed scope separator 
#################################################################################
puts "Line "#__LINE__
set foo:x 1
assert ${foo:x} {== $_ 1} -v -line #__LINE__
# Check we can unset this ok...
unset foo:x
assert [is_set foo:x] {== $_ $::false} -v -line #__LINE__

unset { } b1 c1 c2 d r s -nocomplain
unset _EXENAME -nocomplain

vars

#################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #