##############################################=#############################################
# File:				test_break.tcl
# Author:			M Shaw
# Purpose:			Tests for nested [break] statements
#
# Versions:
# 1.00 27/Jun/2016	First version
# 1.01 27/Jul/2019	Fixed leaks
# 1.02 17/Oct/2021	Fix for tightened [lsearch] arg syntax
# 1.03 21/Jan/2023	Regression fixes
###########################################################################################
# Notes:
# FIX nested [break] exits script
#		if { for { if { break
# DONE FIX leak in foreach-break
# DONE FIX leak in array_to_list
###########################################################################################
option expression off

textcolor yellow
puts "Testing nested \[break\] ..."
textcolor
newline

puts "Test at line "#__LINE__
for {set i 0} {< $i 100} {++ i} {
	if {> $i 9} {
		break
	}
}
puts "I am still here i==$i"
assert $i {== $_ 10} -v -line #__LINE__
newline


puts "Test at line "#__LINE__
if {$true} {
	for {set i 0} {< $i 100} {++ i} {
		if {> $i 9} {
			break
		}
	}
}
puts "I am still here i==$i"
assert $i {== $_ 10} -v -line #__LINE__
newline


puts "Test at line "#__LINE__
if {$true} {
	for {set i 0} {< $i 100} {++ i} {
		if {$true} {
			if {> $i 9} {
				break
			}
		}
	}
}
puts "I am still here i==$i"
assert $i {== $_ 10} -v -line #__LINE__
newline


set i 0
set j 0
puts "Test at line "#__LINE__
# i should iter all the way to 100
if {$true} {
	for {set i 0} {< $i 50} {++ i} {
		if {$true} {
			for {set j 0} {< $j 100} {++ j} {
				# if {is_mod $i 10} {puts "i is $i"}
				# puts "j=$j" # OK
				if {> $j 9} {
					puts "Inner level for break in j at i $i, j $j line:"#__LINE__		# Should allow for i to continue and not cascade down to exit
					break
				}
			}
		}
	}
}
puts "I am still here i==$i @line "#__LINE__
assert $i {== $_ 50} -v -line #__LINE__
newline

unset i j -nocomplain

# # # # 
# FIXED - LEAK!!!!!!!!!!!
# Possibly in array_to_list
#/ *
# DONE FIX [break] will exit the script
option echo on
# foreach+break
puts "Test at line "#__LINE__
set l [array_to_list env]		# Leak fixed

textcolor white
# puts '$l'
textcolor
set i 0
foreach x $l {								# Iter X over list L
	# echo [string left $x 60]
	if {eqn $x "{T" 2 -nocase} {
		++ i
		textcolor yellow
		#echo "$i * $x *"
		puts "$i * [escape $x] *"
		textcolor
	}
}
textcolor

puts "Test at line "#__LINE__
puts "\[lsearch -nocase l \"env\"\]=" -nonewline 
puts "[lsearch -nocase $l env]"

puts "Test at line "#__LINE__
set y {}
foreach x $l {								# Iter X over list L
	textcolor magenta
	puts "x is-> '$x' @"#__LINE__
	textcolor yellow
	puts "'$x' [lsearch -nocase $x PATH]' @"#__LINE__
	textcolor
	set r [lsearch -nocase $x path ]		# No leak
	if {>= $r 0} {
		puts "found path at $r"
		set y [lindex $x 0]
		break		# FIXED Leaks (triggers a leak)
	}
}
puts "y is '$y'"
assert [eq $y path -nocase] {== $_ 1} -v -line #__LINE__

# /*
#
# Test [foreach] break does not exit script at outer [if]
puts "Test at line "#__LINE__
set y {}
if {$true} {
	foreach x $l {
		# puts "'$x' [lsearch -nocase $x PATH]"
		set r [lsearch -nocase $x path ]
		if {>= $r 0} {
			#puts "found path at $r"
			set y [lindex $x 0]
			break
		}
	}
}
assert [eq $y path -nocase] {== $_ 1} -v -line #__LINE__


# Test [foreach] break does not exit outer [for]
puts "Test at line "#__LINE__
for {set i 0} {< $i 100} {++ i} {
	set y {}
	foreach x $l {
		# puts "'$x' [lsearch -nocase $x PATH]"
		set r [lsearch -nocase $x path ]
		if {>= $r 0} {
			#puts "found path at $r"
			set y [lindex $x 0]
			break
		}
	}
}
#puts "I am still here i==$i"
assert $i {== $_ 100} -v -line #__LINE__


puts "Line: "#__LINE__
unset i l r x y -nocomplain
# * /
##############################################=#############################################

#break		# Will exit the script at v1.23
#puts "I am still here i==$i"

###########################################################################################
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #