############################################## #############################################
# File:				test_calc_aop.tcl
# Author:			M Shaw
# Purpose:			Test file for [elevate] command
#
# Versions:
# 1.00 30/Jun/2016 First version
###########################################################################################
# Notes:
# Test operator precedence in the AOP (apply_operator_precedence)
#
# Failed tests:
# Check calc for both
# assert [expr +1--2--3--4--5] {== $_ 15}			# expr FAIL (11) Same as 1+2+3+4+5 (ActiveState == 15)
# assert [expr --1--2--3--4--5] {== $_ 15}			# Should simplify to +1--2--3--4--5
###########################################################################################
# Operators handled. + - * / ** ^ (power)
# FIX AOP needs improvment in handling mixed/multiple leading signs
###########################################################################################

# Simple test
assert [expr 1+2+3+4+5] {== $_ 15} -v -line #__LINE__
assert [expr 1-2-3-4-5] {== $_ -13} -v -line #__LINE__
# Calc
assert [calc 1+2+3+4+5] {== $_ 15} -v -line #__LINE__
assert [calc 1-2-3-4-5] {== $_ -13} -v -line #__LINE__

# Sign prefix
assert [expr 1--2--3--4--5] {== $_ 15} -v -line #__LINE__			# Same as 1+2+3+4+5 (ActiveState PASS)
assert [expr 1+-2+-3+-4+-5] {== $_ -13} -v -line #__LINE__		# Same as 1-2-3-4-5 (ActiveState PASS)
# Calc
assert [calc 1--2--3--4--5] {== $_ 15} -v -line #__LINE__			# Same as 1+2+3+4+5 (ActiveState PASS)
assert [calc 1+-2+-3+-4+-5] {== $_ -13} -v -line #__LINE__		# Same as 1-2-3-4-5 (ActiveState PASS)

# Test leading -ve
assert [expr -1+2+3+4+5] {== $_ 13} -v -line #__LINE__			# ActiveState == 13 
assert [expr -1-2-3-4-5] {== $_ -15} -v -line #__LINE__			# ActiveState == -15
assert [expr -1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13)
try {
	assert [expr -+1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13)
} catch {
	textcolor red
	puts "Leading sign fail at line "#__LINE__
}
# FAILS as we only handle - prefix
try {
	assert [expr +-1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13)
} catch {
	textcolor red
	puts "Leading sign fail at line "#__LINE__
}
try {
	assert [expr --1--2--3--4--5] {== $_ 15} -v -line #__LINE__		# FAIL Same as 1+2+3+4+5 (ActiveState PASS == 15)
} catch {
	textcolor red
	puts "Leading sign fail at line "#__LINE__
}	

assert [expr -1+-2+-3+-4+-5] {== $_ -15} -v -line #__LINE__		# Same as 1-2-3-4-5 (ActiveState PASS)
# Calc
assert [calc -1+2+3+4+5] {== $_ 13} -v -line #__LINE__			# ActiveState == 13 
assert [calc -1-2-3-4-5] {== $_ -15} -v -line #__LINE__			# ActiveState == -15
assert [calc -1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13)
# 05/2018 Disabled this test as the syntax is iffy
#assert [calc --1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13) (surely == +1 ?)
try {
	assert [calc -+1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13)  (+- == -)
} catch {
	textcolor red
	puts "Leading sign fail at line "#__LINE__
}
assert [calc +-1--2--3--4--5] {== $_ 13} -v -line #__LINE__		# Same as 1+2+3+4+5 (ActiveState PASS == 13)  (+- == -)
assert [calc -1+-2+-3+-4+-5] {== $_ -15} -v -line #__LINE__		# Same as 1-2-3-4-5 (ActiveState PASS)

# Test leading +ve
assert [calc +1+2+3+4+5] {== $_ 15} -v -line #__LINE__				# 15
assert [expr +1+2+3+4+5] {== $_ 15} -v -line #__LINE__				# 15
assert [expr +1-2-3-4-5] {== $_ -13} -v -line #__LINE__			# ActiveState == -15
try {
	assert [expr +1--2--3--4--5] {== $_ 15} -v -line #__LINE__			# expr FAIL (11) Same as 1+2+3+4+5 (ActiveState == 15)
} catch {
	textcolor red
	puts "Test failed at line "#__LINE__
}
assert [expr +1+-2+-3+-4+-5] {== $_ -13} -v -line #__LINE__		# Same as 1-2-3-4-5 (ActiveState PASS)
# Calc
assert [calc +1+2+3+4+5] {== $_ 15} -v -line #__LINE__				# 15
assert [calc +1-2-3-4-5] {== $_ -13} -v -line #__LINE__			# ActiveState == -15
assert [calc +1--2--3--4--5] {== $_ 15} -v -line #__LINE__		# calc PASS (15) Same as 1+2+3+4+5 (ActiveState == 15)
assert [calc +1+-2+-3+-4+-5] {== $_ -13} -v -line #__LINE__		# Same as 1-2-3-4-5 (ActiveState PASS)

# Complex expressions
assert [expr 1+2*3-4/5**6+7] {== $_ 14} -v -line #__LINE__		# ActiveState == 14
assert [expr 3+3*3-3+3] {== $_ 12} -v -line #__LINE__
# Calc
assert [calc 1+2*3-4/5**6+7] {== $_ 14} -v -line #__LINE__		# ActiveState == 14
assert [calc 3+3*3-3+3] {== $_ 12} -v -line #__LINE__

# Recent stuff
assert [calc 3*-2**4+3] {== $_ 1299} -v -line #__LINE__



###########################################################################################
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #