#################################################################################
# File:				test_call_cdecl.tcl
# Author:			M Shaw
# Purpose:			Test cases for special __cdecll test DLL
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 20/Nov/2019	First version
#################################################################################
# Notes:
# Not part of main tests
# ADD asserts
#################################################################################

textcolor yellow
puts "Testing special DLL - cdecl.dll ..."
textcolor

textcolor cyan
lib ticol_calldll -verbose
textcolor
newline

set h1 [dll_open cdecl]
set h2 [dll_open msvcrt40.dll]
# puts "cdecl.dll handle is $h1"

assert [calldll_cdecl $h1 test_3_long_cdecl 3 4 5] {== $_ 17} -v -line #__LINE__		# a*b+c
assert [calldll_cdecl $h1 test_1_string_byval_cdecl "Hello"] {eq $_ 5} -v -line #__LINE__		# a*b+c
# FIX passes junk
#assert [calldll_cdecl $h1 test_1_string_cdecl qbf] {eq $_ 5} -v -line #__LINE__		# a*b+c
assert [calldll_cdecl $h1 test_1_string_byval_cdecl $qbf] {eq $_ 43} -v -line #__LINE__		# a*b+c

# FIX returning handle 0
set ts_ptr [calldll_cdecl $h1 test_tm_ptr_cdecl 33]		# Remember that the lib will have unloaded
puts "test_tm_ptr_cdecl static struct address was '$ts_ptr'"

# struct tm *localtime( const time_t *timer );
# set ts_ptr [calldll_cdecl $h2 localtime t]		# Return a struct tm* to converted time

puts "Line: "#__LINE__
textcolor yellow
dumpmem $ts_ptr 32
textcolor

puts "Line: "#__LINE__
struct unset tm -nocomplain
struct tm {		# Define a struct. We don't need any instances of it other than this
	#			 C++	Field order must be maintained
	tm_sec	4	#int	Seconds after the minute   0-61
	tm_min	4	#int	Minutes after the hour     0-59
	tm_hour	4	#int	Hours since midnight       0-23
	tm_mday	4	#int	Day of the month           1-31
	tm_mon	4	#int	Months since January       0-11
	tm_year	4	#int	Years since                1900	
	tm_wday	4	#int	Days since Sunday          0-6
	tm_yday	4	#int	Days since January 1       0-365
	tm_isdst 4 	#int	Daylight Saving Time flag
}


# There is no actual need to pass back into a Tcl struct other than to demo that we can do this and pass in/out
# We can call asctime directly with ts_ptr unless we want to inspect the data
struct copy tm $ts_ptr									# Copy the returned binary struct to our var
dump tm

# Syntax: time_t asctime(struct tm* t)
set ret [calldllstr_cdecl $h2 asctime tm]					# Passing directly via ptr ts_ptr

# Wed Nov 20 01:10:12 2019
# 0    1   2     3    4
textcolor darkyellow
puts "asctime: -[string trim $ret [chr 10]]-"						# Why did I quote it like this?
textcolor
newline


########################################################################################
# Pass a Tcl string byref and change it as char*
textcolor yellow
puts "Testing calldll_cdecl passing a Tcl string by reference (dangerous!)"
textcolor
set s [makestr 100]			# Ensure we have a large enough buffer
setl s $qbf
textcolor yellow darkcyan
puts "s is '$s'"
textcolor
assert [calldll_cdecl cdecl.dll test_1_string_byval_cdecl s] {eq $_ 100} -v -line #__LINE__		# a*b+c
dump s
puts "s is now '$s'"
assert $s {eq $_ "Changed string"} -v -line #__LINE__

set s [makestr 100]			# Ensure we have a large enough buffer
setl s "Hello cdecl DLL"
assert [calldll_cdecl cdecl.dll test_1_string_byval_cdecl [strptr s]] {eq $_ 100} -v -line #__LINE__		# a*b+c

/*
########################################################################################
# Pass a Tcl string byref and change it as char**
textcolor yellow
puts "Testing calldll_cdecl passing a Tcl string by reference (dangerous!)"
textcolor
set s [makestr 100]			# Ensure we have a large enough buffer
setl s $qbf
# We can't cast to char**
#assert [calldll_cdecl cdecl.dll test_1_string_byref_cdecl [strptr s]] {eq $_ 100} -v -line #__LINE__		# a*b+c
# */

dll_close $h1
dll_close $h2

lib ticol_calldll -unload

struct unset tm -nocomplain
unset h1 h2 ret s ts_ptr 

########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #