##############################################################################
# File: 			test_cast.tcl
# Purpose:			Test file for casting commands
# Author: 			M Shaw
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 03/Jan/2017	First version
# 1.01 22/Jul/2017	Replaced [checkex] with [assert]
# 1.02 06/Aug/2018	Added more tests
##############################################################################
# Notes: 
##############################################################################

option echo on
option expression off
/* //////////////////////////////////////////////////////////////////////
# double
# float
# int
# integer
# integral
# wide
################################
[char value]
[short value]
[long value]
[int value]
[double value]
[signed value]
[unsigned value]
[short 0x123ffff]             -1
[int 0x123ffff]               19136511
[double 0x123ffff]            19136511.000000000000
-1                            -1
[unsigned -1]                 18446744073709551615
[signed 18446744073709551615] -1
[signed 0xFFFFFFFFFFFFFFFF]   -1
////////////////////////////////////////////////////////////////////// */

textcolor yellow
puts "Testing casting commands ..."
textcolor
newline

assert [int $pi] {== $_ 3} -v -line #__LINE__
assert [fpart $pi] {== $_ 1415926536} -v -line #__LINE__
assert [integral $pi] {== $_ 3} -v -line #__LINE__
assert [/ 22 7] {== $_ 3} -v -line #__LINE__
assert [/ [double 22] [double 7]] {== $_ 3.142857142857143} -v -line #__LINE__
assert [char 12345] {== $_ 57} -v -line #__LINE__
assert [double 0b111100011011] {== $_ 3867.000000000000000} -v -line #__LINE__
assert [short 0o123] {== $_ 83} -v -line #__LINE__
assert [int 0x123ffff] {== $_ 19136511} -v -line #__LINE__
assert [unsigned -123456789] {== $_ 18446744073586094827} -v -line #__LINE__		# Check design
assert [signed 18446744073586094827] {== $_ -123456789} -v -line #__LINE__		# Check design
assert [signed 18446744073709551615] {== $_ -1} -v -line #__LINE__

# [chr 12345]                   Overflow

assert [float 0] {== $_ 0.0}  -v -line #__LINE__
assert [float 0.0] {== $_ 0.0}  -v -line #__LINE__
assert [float 0.1] {== $_ 0.1}  -v -line #__LINE__
assert [float .1] {== $_ 0.1}  -v -line #__LINE__

assert [double 0] {== $_ 0.0}  -v -line #__LINE__
assert [double 0.0] {== $_ 0.0}  -v -line #__LINE__
assert [double 0.1] {== $_ 0.1}  -v -line #__LINE__
assert [double .1] {== $_ 0.1}  -v -line #__LINE__
newline

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #