######################################################################################
# File: 			test_catch.tcl
# Purpose:			Test [catch]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 23/Jul/2017	First version
# 1.01 08/Jun/2018	Improved tests and updated [catch]
######################################################################################
# Notes:
######################################################################################
option expression off
option echo on

textcolor yellow
puts "Testing \[catch\] ..."
textcolor
newline

puts "Test catch div OK"
catch {/ 23 1}		# Should not exit
assert [catch {/ 23 1}] {== $_ 0} -v -line #__LINE__

puts "Test catch div OK"
assert [catch {/ 23 2.0} result] {== $_ 0}
puts "Result is '$result'"
assert $result {== $_ 11.5} -v -line #__LINE__

# Test wrapping [catch]
if {! [catch {/ 23 2.0} result]} {
	assert $result {== $_ 11.5} -v -line #__LINE__
}

puts "Test catch div 0 FAIL"
catch {/ 23 0}		# Should not exit
assert [catch {/ 23 0}] {== $_ 1} -v -line #__LINE__

# FIX bad error message for DB0
puts "Test catch div 0 FAIL with var"
catch {/ 23 0}		# Should not exit
assert [catch {/ 23 0} err] {== $_ 1} -v -line #__LINE__
puts "Err was '$err' - errorMsg was '$errorMsg'"

# Test string ops # # # # # # 

set s "hello"
catch {string left $nosuchvar 1} s
assert $s {ne $_ hello} -v -line #__LINE__

set s ""
catch {string mids $qbf 5 5} s		# "The quick ..."
assert $s {eq $_ quick} -v -line #__LINE__

# Test commands # # # # # # #
textcolor yellow
puts "Testing flow control commands ..."
textcolor

puts "Test catch return"
catch {return}		# Should not exit
assert [catch {return}] {== $_ 2} -v -line #__LINE__

puts "Test catch break"
catch {break}		# Should not exit
assert [catch {break}] {== $_ 3} -v -line #__LINE__

puts "Test catch continue"
catch {continue}	# Should not exit
assert [catch {continue}] {== $_ 4} -v -line #__LINE__

puts "Test catch exit"
catch {exit}		# Should not exit
assert [catch {exit}] {== $_ 5} -v -line #__LINE__

# Returns 5 in TCX file
# _script_was_encrypted
puts "Test catch stop"
catch {stop}		# Should not stop if TCX
if {eq [file extension [info script]] ".tcx" -nocase} {
	assert [catch {stop}] {== $_ 5} -v -line #__LINE__
} else {
	assert [catch {stop}] {== $_ 6} -v -line #__LINE__
}

puts "Test catch goto heck (ignore errors)"
catch {goto heck}		# Should not exit
assert [catch {goto heck}] {== $_ 7} -v -line #__LINE__

unset err s this result -nocomplain

vars

puts [procs]

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Done *"
textcolor

# EOF #