######################################################################################
# File: 			test_chain.tcl
# Purpose:			Test cases for [chain]
# Language:			Ticol Tcl v1.26
#
# Versions:
# 1.00 11/Dec/2019	First implemented
# 1.01 04/Mar/2021	Updated for Win environment
######################################################################################
# Notes:
# Chain can resolve any $ resolvable variable via any level of indirection
######################################################################################
option expression off

# Attempt cleanup if re-running in Windows IDE
unset a b c d e -nocomplain
struct unset st -nocomplain
#stack unset st -nocomplain	
#unset st -nocomplain


set a Hello		# Terminal node
set b a 
set c b
set d c
set e d

puts [chain e]
assert [chain e] {eq $_ Hello} -v -line #__LINE__

puts [chain d]
assert [chain d] {eq $_ Hello} -v -line #__LINE__

puts [chain c]
assert [chain c] {eq $_ Hello} -v -line #__LINE__

puts [chain b]
assert [chain b] {eq $_ Hello} -v -line #__LINE__

puts [chain a]
assert [chain a] {eq $_ Hello} -v -line #__LINE__

# Default value ""
puts [chain foo ""]
assert [chain foo ""] {eq $_ {}} -v -line #__LINE__

# Default value ""
puts [chain foo bar]
assert [chain foo bar] {eq $_ bar} -v -line #__LINE__

# Break the chain
# [chain] won't know if a terminating case string is a reference
# to a varname or a string literal
unset c
puts [chain e]
assert [chain e] {eq $_ c} -v -line #__LINE__

set i 0
try {
	chain foo
} catch {
	set i 1
}
assert $i -v -line #__LINE__

#######################
# MAN FILE Examples ...
# PASS
#/*
	set a Hello                       # Terminal node
	set b a                           # b points to a
	set c b                           # c points to b
	puts [chain c]
	unset a                           # Break the reference chain
 	puts [chain c]
	puts [chain foobar {Not found}]   # No such reference
	catch {
	puts [chain foobar]               # Will raise a catchable error
	}
# */

set a Hello                       # Terminal node
set b(23) a                       # b(23) points to a
set c b(23)                           # c points to b
puts [chain c]
assert [chain c] {eq $_ Hello} -v -line #__LINE__

set a Hello
struct st {b 10}
struct set st.b a
set c st.b

puts [chain c]
assert [chain c] {eq $_ Hello} -v -line #__LINE__


set c 1                # Create a subscript variable
set b($c) Hello           # Create an array variable  b(1)
#set a($b($c)) Hello    # Set a nested associative array
set a($b($c)) Hello
set d a($b($c))
set e d
set f e
puts "[chain d] @"#__LINE__
assert [chain d]  {eq $_ Hello} -v -line #__LINE__

assert [chain f -depth 0] {eq $_ Hello} -v -line #__LINE__
assert [chain f -depth 1] {eq $_ e} -v -line #__LINE__		# f->e == 1
assert [chain f -depth 2] {eq $_ d} -v -line #__LINE__		# f->e->d == 2
assert [chain f -depth 3] {eq $_ a(Hello)} -v -line #__LINE__		# f->e->d->a(Hello) == 3
assert [chain f -depth 4] {eq $_ Hello} -v -line #__LINE__		# f->e->d->a(Hello)-> == 4

array unset b -nocomplain
struct unset st -nocomplain
unset a b c d e f i -nocomplain
######################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #