###################################################################################
# File:				test_class_getset.tcl
# Purpose:			Test emulated classses
# Author:			M Shaw
# Language:			Ticol Tcl v1.21c+..v1.26+
#
# Versions:
# 1.00 25/Dec/2016	First version
# 1.01 04/Jan/2017	Fix re: proc args
# 1.02 22/Jul/2017	Replaced [checkex] with [assert]
# 1.03 09/Oct/2018	Fix regression in striptor (added -remove)
###################################################################################
# Notes:
# 11/Nov/2017 Broken due to regressions in struct
# =>> ('dereference then eval' is borked)
#	Dereferences to final string then tries to evaluate it
# For entertainment purposes only!
###################################################################################
option expression off

textcolor yellow
puts "Testing emulated classes"
textcolor
newline

unset c -nocomplain
unset d -nocomplain

#define NUM_MAX 255
#deflist													# Just for debugging...
undef class -nocomplain										# Undef for re-running in CLI

alias class struct											# Required at v1.21+

class test {												# Could create an alias to struct called [class]
	this NUM_MAX											# Our object instance (not used yet)
	class_name NUM_MAX										# Source class name (not used yet)
	get NUM_MAX
	set NUM_MAX
	data NUM_MAX 											# ADD arg -private ?
}

undef test::constructor -nocomplain

proc test::constructor {x} {								# A pseudo constructor for the 'class' (Emulates C++ class::class())
	upvar $x
	set class_name [info procname]							# Not strictly necesscary but an example of extracting the class name from the procname
	puts "SCRIPT DEBUG: striptor debug 1/2: class_name:'$class_name'"
	set class_name [striptor :: class_name -remove -var]	# stripto -> [striptor] strip R/H to "::" and return "class::" unless -remove is used
	puts "SCRIPT DEBUG: striptor debug 2/2: class_name:'$class_name'"
	assert [string left $class_name 2] {ne $_ "::"} -v -line #__LINE__
	struct set ${x}.this $x									# We record our own object instance name. e.g. 'c'
	struct set ${x}.class_name ${class_name}
	struct set ${x}.get "${class_name}::get ::${x}.data"	# {x} transforms "32220128.data -> c.data" (add scope)
	struct set ${x}.set "${class_name}::set ::${x}.data"	# We concatenate the proc name with our object reference ::{c}.data (scoping is fixed at global)
}

undef test::get -nocomplain									# Only required if we rerun the script from the CLI

proc test::get {x} {										# 'class::' prefix is dummy name-mangling
	textcolor magenta
	puts "Enter proc test::get"
	textcolor
	set line #__LINE__
	puts "Exit proc test::get $x"	
	return [-> $x]											# PASS (shorthand) dereference $x
}

undef test::set -nocomplain									# Only required if we rerun the script from the CLI

proc test::set {args} {										# PASS x2 args (must use special arg 'args')
	set line #__LINE__
	#upvar $x
	eval "=> $args"											# PASS  (shorthand put/=>) expand
}

#############################################################
puts "Line: "#__LINE__
new test c													# Create an object instance

struct set c.data Hello
#checkex [=>> c.get]  {eq $x "Hello"}

puts "Line: "#__LINE__
new test d													# Create an instance

set line #__LINE__
test::constructor c											# Call the 'constructor' for this object
test::constructor d

puts "Line: "#__LINE__
#struct set c.data "Default data"							# Set to some default string so we can test if the data was (re)set
=>> c.set "Default data"

puts "Line: "#__LINE__
textcolor yellow
puts "Initial test data in: \$c.data is: '[-> c.data]'"		# Check and test using standard access method
newline
textcolor

puts "Line: "#__LINE__
assert [=>> c.get]  {eq $_ "Default data"}					# picolCommandStructDereference -> picolCommandStructItem

#eval "[-> ::c.set] {$qbf}"									# Alternate syntax
#eval "[-> ::d.set] {Hello from struct D}"					# Alternate syntax

puts "Line: "#__LINE__
=>> c.set $qbf												# Set c to qbf
assert [=>> c.get]  {eq $_ $::qbf} -v -line #__LINE__

puts "Line: "#__LINE__
=>> d.set "Hello from struct D"								# Set d to some arbitrary string
assert [=>> d.get]  {eq $_ "Hello from struct D"} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor darkyellow
puts "\[=>> c.get\]=='[=>> c.get]'"
textcolor
# BORKED HERE c.get is ok and =>> d.set is ok separately but they won't chain
=>> d.set [=>> c.get]											# Copy from c to d
#=>> d.set $qbf

puts "Line: "#__LINE__
puts "\[=>> d.get\]=[=>> d.get]"
puts "Line: "#__LINE__
assert [=>> d.get]  {eq $_ $::qbf} -v -line #__LINE__		# d now contains c's data

puts "Line: "#__LINE__
class unset test
class unset c
class unset d


undef test::constructor test::get  test::set -nocomplain
undef class -nocomplain
unset line -nocomplain

###################################################################################
vars
puts [procs]
newline

###################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #