##################################################################
# File: 			test_clock_scan.tcl
# Purpose:			Test file for [clock scan]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.20+
#
# Versions:
# 1.00 13/Nov/2016	First version	
# 1.01 15/Nov/2016	Updated
# 1.02 22/Jul/2017	Replaced [checkex] with [assert]
# 1.03 05/Jan/2018	Fixed clock scan "nov 14 00:39:23" @66
##################################################################
# Notes: 
##################################################################
option echo on

set i 0
set year [date -year]

##
set b [clock scan "Jan 12 97 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "Jan 12 1997 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "January 12 1997 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "January 12 97 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

# DONE FIX FAIL
set b [clock scan "Jan  12  1997  23:59:12"]				# Multiple-SPACE separators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "Jan	12	1997 23:59:12"]					# One TAB separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "Jan		12		1997 	23:59:12"]		# Multiple TAB separators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "Jan	 12	 1997	 23:59:12"]				# Multiple TAB and SPACE separators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "Jan. 12.,,,, 1997 23:59:12"]				# Superfluous punctuators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "Jan. 12.,,,, 1997"]
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 00:00:00"} -v -line #__LINE__

set b [clock scan "nov 14"]
puts "\t$b [date [ctime_to_double $b]]"	#2036/02/05 06:28:17	
assert [date [ctime_to_double $b]] {eq $_ "$::year/11/14 00:00:00"} -v -line #__LINE__

# DONE FIX FAIL (use current year as [date -year])
set b [clock scan "nov 14 00:39:23"]
puts "\t$b [date [ctime_to_double $b]]"	#2036/02/05 06:28:17	
assert [date [ctime_to_double $b]] {eq $_ "[date -year]/11/14 00:39:23"} -v -line #__LINE__
###########################################

set b [clock scan "12 Jan 97 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12 Jan 1997 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12 January 1997 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12 January 97 23:59:12"]					# One SPACE separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

# FAIL
set b [clock scan "12  Jan  1997  23:59:12"]				# Multiple-SPACE separators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12	Jan	1997 23:59:12"]					# One TAB separator
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12		Jan		1997 	23:59:12"]		# Multiple TAB separators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12	 Jan	 1997	 23:59:12"]				# Multiple TAB and SPACE separators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12. Jan.,,,, 1997 23:59:12"]				# Superfluous punctuators
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 23:59:12"} -v -line #__LINE__

set b [clock scan "12. Jan.,,,, 1997"]
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "1997/01/12 00:00:00"} -v -line #__LINE__


###########################################
# Style 2 #
set b [clock scan "11/13/2016 23:59:12"]
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "2016/11/13 23:59:12"} -v -line #__LINE__

set b [clock scan "11/13/16 23:59:12"]
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "2016/11/13 23:59:12"} -v -line #__LINE__

set b [clock scan "11/13 23:59:12"]
puts "\t$b [date [ctime_to_double $b]]"
assert [date [ctime_to_double $b]] {eq $_ "$::year/11/13 23:59:12"} -v -line #__LINE__

newline
textcolor red
puts "Catch ..."
textcolor
catch {
	set b [clock scan "11/13/ 23:59:12"]
	puts "\t$b [date [ctime_to_double $b]]"
	assert [date [ctime_to_double $b]] {eq $_ "$::year/11/13 23:59:12"} -v -line #__LINE__
}

unset b i year -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor
 
 # EOF #