################################################################################
# File:			test_clock_split.tcl
# Author:		M Shaw
# Purpose:		Tests of [clock split]
# Language:		Ticol Tcl 1.25+
#
# Versions:
# 22/Dec/2017	First version
################################################################################
# Notes:
# [clock split seconds d h m s]
# [clock split seconds - - - s]
# [clock split seconds d]
################################################################################

option expression off

puts "Line: "#__LINE__
unset d h m s -nocomplain
set seconds 0
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 0} -v -line #__LINE__
assert $h {== $_ 0} -v -line #__LINE__
assert $m {== $_ 0} -v -line #__LINE__
assert $s {== $_ 0} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds 23
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 0} -v -line #__LINE__
assert $h {== $_ 0} -v -line #__LINE__
assert $m {== $_ 0} -v -line #__LINE__
assert $s {== $_ 23} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds 60
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 0} -v -line #__LINE__
assert $h {== $_ 0} -v -line #__LINE__
assert $m {== $_ 1} -v -line #__LINE__
assert $s {== $_ 0} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds 83
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 0} -v -line #__LINE__
assert $h {== $_ 0} -v -line #__LINE__
assert $m {== $_ 1} -v -line #__LINE__
assert $s {== $_ 23} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds 3600
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 0} -v -line #__LINE__
assert $h {== $_ 1} -v -line #__LINE__
assert $m {== $_ 0} -v -line #__LINE__
assert $s {== $_ 0} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds 86400
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 1} -v -line #__LINE__
assert $h {== $_ 0} -v -line #__LINE__
assert $m {== $_ 0} -v -line #__LINE__
assert $s {== $_ 0} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds 864000
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 10} -v -line #__LINE__
assert $h {== $_ 0} -v -line #__LINE__
assert $m {== $_ 0} -v -line #__LINE__
assert $s {== $_ 0} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [now -ctime]
puts "$seconds [/ $seconds 86400] days"
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
puts "\[now -ctime\] == $d day(s) $h hour(s) $m minutes $s second(s)"
assert $d {> $_ 17521} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds d h m s] {== $_ 4} -v -line #__LINE__
assert $d {== $_ 1} -v -line #__LINE__
assert $h {== $_ 2} -v -line #__LINE__
assert $m {== $_ 3} -v -line #__LINE__
assert $s {== $_ 4} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds - h m s] {== $_ 3} -v -line #__LINE__
assert $h {== $_ 2} -v -line #__LINE__
assert $m {== $_ 3} -v -line #__LINE__
assert $s {== $_ 4} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds - - m s] {== $_ 2} -v -line #__LINE__
assert $m {== $_ 3} -v -line #__LINE__
assert $s {== $_ 4} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds - - - s] {== $_ 1} -v -line #__LINE__
assert $s {== $_ 4} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds d h m] {== $_ 3} -v -line #__LINE__
assert $d {== $_ 1} -v -line #__LINE__
assert $h {== $_ 2} -v -line #__LINE__
assert $m {== $_ 3} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds d h] {== $_ 2} -v -line #__LINE__
assert $d {== $_ 1} -v -line #__LINE__
assert $h {== $_ 2} -v -line #__LINE__
newline

unset d h m s -nocomplain
set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
assert [clock split $seconds d] {== $_ 1} -v -line #__LINE__
assert $d {== $_ 1} -v -line #__LINE__
newline

catch {
	unset d h m s -nocomplain
	set seconds [expr 86400+(3600*2)+(60*3)+(1*4)]
	assert [clock split $seconds -] {== $_ 0} -v -line #__LINE__
	newline
}

unset d h m s seconds -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #