###############################################################################################
# File:				test_cmp.tcl
# Author:			M Shaw
# Purpose:			Test file for compare commands
# Language:			Ticol Tcl v1.21c+
#
# Versions:
# 1.00 23/Dec/2016	Migrated from tests.tcl
# 1.01 22/Jul/2017	Replaced [checkex] with [assert]
###############################################################################################
# Notes
# Math: > < >= <= == !=
# String: eq ge gt le lt ne
# ADD test escaped string cmps
###############################################################################################
/*
eq
eqi
gt
gti
ge
gei
lt
lti
le
lei
ne
nei
*/

option expression off

textcolor yellow
puts "Testing string comparison commands ...."
textcolor
newline

# Relational
assert [expr {4*2 < 7}] {== $_ 0} -v -line #__LINE__		# Should be false/0  (8 < 7 )
assert [expr {4*2 > 7}] {== $_ 1} -v -line #__LINE__ 		# Should be true/0  (8 < 7 )
assert [expr {0 > 7}] {== $_ 0} -v -line #__LINE__ 			# Should be true/0  (8 < 7 )

# misc relational #
# FIX ME 
# To speed up math, no text detection is done
# option expression on;puts [expr "b > a"]
assert [expr {"0x03" > "2"}] {== $_ 1} -v -line #__LINE__	# https://www.tcl.tk/man/tcl8.5/TclCmd/expr.htm#M9
# both return 1. The first comparison is done using integer comparison, and the second is done using string comparison.
# Ticol usually requires string operators
#assert [expr {"0y" > "0x12"}] {== $_ 1} -v -line #__LINE__	# https://www.tcl.tk/man/tcl8.5/TclCmd/expr.htm#M9
assert [expr {"0y" gt "0x12"}] {== $_ 1} -v -line #__LINE__	# https://www.tcl.tk/man/tcl8.5/TclCmd/expr.htm#M9
assert [expr {"0x03" lt "2"}] {== $_ 1} -v -line #__LINE__
assert [expr {"0x03" le "2"}] {== $_ 1} -v -line #__LINE__
assert [expr {"0x03" gt "2"}] {== $_ 0} -v -line #__LINE__
assert [expr {"0x03" ge "2"}] {== $_ 0} -v -line #__LINE__	# https://www.tcl.tk/man/tcl8.5/TclCmd/expr.htm#M9
assert [expr {"0y" gt "0x12"}] {== $_ 1} -v -line #__LINE__	# https://www.tcl.tk/man/tcl8.5/TclCmd/expr.htm#M9


# cmp
# cmp var|string1 string2 start-pos ?len? ?-ignorecase?
newline
textcolor yellow
puts "Testing \[cmp\]"
textcolor
assert [cmp $qbf "brown" 11 5 ] {== $_ 1} -v -line #__LINE__
#assert [cmp $qbf "BROWN" 11 5 ] {== $_ 0} -v -line #__LINE__	# FAILS
assert [cmp $qbf "brown" 11 5 -ignorecase] {== $_ 1} -v -line #__LINE__

# eq
newline
textcolor yellow
puts "Testing \[eq\]"
textcolor
assert [eq 1 1] {== $_ 1} -v -line #__LINE__
assert [eq 0 0] {== $_ 1} -v -line #__LINE__
assert [eq 0 1] {== $_ 0} -v -line #__LINE__
assert [eq 1 0] {== $_ 0} -v -line #__LINE__
set h hello
assert [eq $h hello] {== $_ 1} -v -line #__LINE__
assert [eq "The quick brown fox jumps over the lazy dog" $qbf] {== $_ 1}
set s "\[dummy\] hello\[proc\] {\[ignored\]} world"
assert [eq "\[dummy\] hello\[proc\] {\[ignored\]} world" $s] {== $_ 1} -v -line #__LINE__
assert [eq AARDVARK zoo] {== $_ 0} -v -line #__LINE__
assert [eq ZOO aardvark] {== $_ 0} -v -line #__LINE__
assert [eq This That] {== $_ 0} -v -line #__LINE__
assert [eq This that] {== $_ 0} -v -line #__LINE__		# True (this > that)
assert [eq that This] {== $_ 0} -v -line #__LINE__		# True (this > that)
assert [eq This this] {== $_ 0} -v -line #__LINE__		# True (both are equal)
assert [eq this This] {== $_ 0} -v -line #__LINE__		# True (both are equal)
assert [eq THIS this] {== $_ 0} -v -line #__LINE__		# True (both are equal)
assert [eq this this] {== $_ 1} -v -line #__LINE__		# True (both are equal)
assert [eq thiS thiS] {== $_ 1} -v -line #__LINE__		# True (both are equal)

# eqi
newline
textcolor yellow
puts "Testing \[eqi\]"
textcolor
set h hello
assert [eq $h hello] {== $_ 1} -v -line #__LINE__
assert [eqi "THE QUICK brown FOX Jumps oVEr the LAZY doG" $qbf] {== $_ 1} -v -line #__LINE__
set s "\[dummy\] hello\[proc\] {\[ignored\]} world"
assert [eqi "\[duMMy\] HELLO\[proC\] {\[ignoreD\]} WOrld" $s] {== $_ 1} -v -line #__LINE__
assert [eqi AARDVARK zoo] {== $_ 0} -v -line #__LINE__
assert [eqi ZOO aardvark] {== $_ 0} -v -line #__LINE__
assert [eqi This That] {== $_ 0} -v -line #__LINE__
assert [eqi This that] {== $_ 0} -v -line #__LINE__		# True (this > that)
assert [eqi that This] {== $_ 0} -v -line #__LINE__		# True (this > that)
assert [eqi This this] {== $_ 1} -v -line #__LINE__		# True (both are equal)
assert [eqi this This] {== $_ 1} -v -line #__LINE__		# True (both are equal)
assert [eqi THIS this] {== $_ 1} -v -line #__LINE__		# True (both are equal)
assert [eqi this this] {== $_ 1} -v -line #__LINE__		# True (both are equal)
assert [eqi thiS thiS] {== $_ 1} -v -line #__LINE__		# True (both are equal)

# ge
newline
textcolor yellow
puts "Checking \[ge\]"
textcolor
assert [ge 10 1] {== $_ 1} -v -line #__LINE__
assert [ge 1 1] {== $_ 1} -v -line #__LINE__
assert [ge z a] {== $_ 1} -v -line #__LINE__
assert [ge a z] {== $_ 0} -v -line #__LINE__
assert [ge zoo aardvark] {== $_ 1} -v -line #__LINE__
assert [ge aardvark zoo] {== $_ 0} -v -line #__LINE__
assert [ge This That] {== $_ 1} -v -line #__LINE__
assert [ge This that] {== $_ 0} -v -line #__LINE__		# False (T < t)
assert [ge that This] {== $_ 1} -v -line #__LINE__		# True (this > that)
assert [ge This this] {== $_ 0} -v -line #__LINE__		# False (T < t)
assert [ge this This] {== $_ 1} -v -line #__LINE__		# True (both are equal)

# gei
newline
textcolor yellow
puts "Checking \[gei\]"
textcolor
assert [gei 10 1] {== $_ 1} -v -line #__LINE__
assert [gei 1 1] {== $_ 1} -v -line #__LINE__
assert [gei z a] {== $_ 1} -v -line #__LINE__
assert [gei a z] {== $_ 0} -v -line #__LINE__
assert [gei zoo aardvark] {== $_ 1} -v -line #__LINE__
assert [gei aardvark zoo] {== $_ 0} -v -line #__LINE__
assert [gei ZOO aardvark] {== $_ 1} -v -line #__LINE__
assert [gei AARDVARK zoo] {== $_ 0} -v -line #__LINE__
assert [gei ZOO aardvark] {== $_ 1} -v -line #__LINE__
assert [gei This That] {== $_ 1} -v -line #__LINE__
assert [gei This that] {== $_ 1} -v -line #__LINE__		# True (this > that)
assert [gei that This] {== $_ 0} -v -line #__LINE__		# True (this > that)
assert [gei This this] {== $_ 1} -v -line #__LINE__		# True (both are equal)
assert [gei this This] {== $_ 1} -v -line #__LINE__		# True (both are equal)


# gt
newline
textcolor yellow
puts "Testing \[gt\]"
textcolor
assert [gt 10 1]  {== $_ 1} -v -line #__LINE__
assert [gt Z A]  {== $_ 1} -v -line #__LINE__
assert [gt A Z]  {== $_ 0} -v -line #__LINE__
assert [gt 1 10]  {== $_ 0} -v -line #__LINE__
assert [gt 0 0]  {== $_ 0} -v -line #__LINE__
assert [gt A A]  {== $_ 0} -v -line #__LINE__
assert [gt 1 1]  {== $_ 0} -v -line #__LINE__
assert [gt -1 1]  {== $_ 0} -v -line #__LINE__
assert [gt 1 -1]  {== $_ 1} -v -line #__LINE__
assert [gt zoo aardvark] {== $_ 1} -v -line #__LINE__
assert [gt aardvark zoo] {== $_ 0} -v -line #__LINE__
assert [gt ZOO aardvark] {== $_ 0} -v -line #__LINE__		# Z < a
assert [gt AARDVARK zoo] {== $_ 0} -v -line #__LINE__
assert [gt ZOO aardvark] {== $_ 0} -v -line #__LINE__		# Z < a
assert [gt This That] {== $_ 1} -v -line #__LINE__
assert [gt This that] {== $_ 0} -v -line #__LINE__		# True (this > that)
assert [gt that This] {== $_ 1} -v -line #__LINE__		# True (t > T)
assert [gt This this] {== $_ 0} -v -line #__LINE__		#
assert [gt this This] {== $_ 1} -v -line #__LINE__		#

# gti
newline
textcolor yellow
puts "Testing \[gti\]"
textcolor
assert [gti 10 1]  {== $_ 1} -v -line #__LINE__
assert [gti Z A]  {== $_ 1} -v -line #__LINE__
assert [gti A Z]  {== $_ 0} -v -line #__LINE__
assert [gti 1 10]  {== $_ 0} -v -line #__LINE__
assert [gti 0 0]  {== $_ 0} -v -line #__LINE__
assert [gti A A]  {== $_ 0} -v -line #__LINE__
assert [gti 1 1]  {== $_ 0} -v -line #__LINE__
assert [gti -1 1]  {== $_ 0} -v -line #__LINE__
assert [gti 1 -1]  {== $_ 1} -v -line #__LINE__
assert [gti zoo aardvark] {== $_ 1} -v -line #__LINE__
assert [gti aardvark zoo] {== $_ 0} -v -line #__LINE__
assert [gti ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [gti AARDVARK zoo] {== $_ 0} -v -line #__LINE__
assert [gti ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [gti This That] {== $_ 1} -v -line #__LINE__
assert [gti This that] {== $_ 1} -v -line #__LINE__		# True (this > that)
assert [gti that This] {== $_ 0} -v -line #__LINE__		# True (t > T)
assert [gti This this] {== $_ 0} -v -line #__LINE__		#
assert [gti this This] {== $_ 0} -v -line #__LINE__		#


# le
newline
textcolor yellow
puts "Testing \[le\]"
textcolor
assert [le 1 10] {== $_ 1} -v -line #__LINE__
assert [le 10 1] {== $_ 0} -v -line #__LINE__
assert [le 1 1] {== $_ 1} -v -line #__LINE__
assert [le zoo aardvark] {== $_ 0} -v -line #__LINE__
assert [le aardvark zoo] {== $_ 1} -v -line #__LINE__
assert [le ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [le AARDVARK zoo] {== $_ 1} -v -line #__LINE__
assert [le ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [le This That] {== $_ 0} -v -line #__LINE__
assert [le This that] {== $_ 1} -v -line #__LINE__		# True (this > that)
assert [le that This] {== $_ 0} -v -line #__LINE__		# True (t > T)
assert [le This this] {== $_ 1} -v -line #__LINE__		#
assert [le this This] {== $_ 0} -v -line #__LINE__		#

# lt
newline
textcolor yellow
puts "Testing \[lt\]"
textcolor
assert [lt 10 1]  {== $_ 0} -v -line #__LINE__	# False
assert [lt Z A]  {== $_ 0} -v -line #__LINE__		# False
assert [lt A Z]  {== $_ 1} -v -line #__LINE__		# True
assert [lt 1 10]  {== $_ 1} -v -line #__LINE__	# True
assert [lt 0 0]  {== $_ 0} -v -line #__LINE__		# Same
assert [lt A A]  {== $_ 0} -v -line #__LINE__		# Same
assert [lt 1 1]  {== $_ 0} -v -line #__LINE__		# False
assert [lt 1 -1]  {== $_ 0} -v -line #__LINE__	# False
assert [lt -1 1]  {== $_ 1} -v -line #__LINE__	# True
assert [lt zoo aardvark] {== $_ 0} -v -line #__LINE__
assert [lt aardvark zoo] {== $_ 1} -v -line #__LINE__
assert [lt ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [lt AARDVARK zoo] {== $_ 1} -v -line #__LINE__
assert [lt ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [lt This That] {== $_ 0} -v -line #__LINE__
assert [lt This that] {== $_ 1} -v -line #__LINE__		# True (this > that)
assert [lt that This] {== $_ 0} -v -line #__LINE__		# True (t > T)
assert [lt This this] {== $_ 1} -v -line #__LINE__		#
assert [lt this This] {== $_ 0} -v -line #__LINE__		#

# ne
newline
textcolor yellow
puts "Testing \[ne\]"
textcolor
assert [ne 1 10] {== $_ 1} -v -line #__LINE__
assert [ne 1 1] {== $_ 0} -v -line #__LINE__
assert [ne zoo aardvark] {== $_ 1} -v -line #__LINE__
assert [ne aardvark zoo] {== $_ 1} -v -line #__LINE__
assert [ne ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [ne AARDVARK zoo] {== $_ 1} -v -line #__LINE__
assert [ne ZOO aardvark] {== $_ 1} -v -line #__LINE__		# Z < a
assert [ne This That] {== $_ 1} -v -line #__LINE__
assert [ne This that] {== $_ 1} -v -line #__LINE__		# True (this > that)
assert [ne that This] {== $_ 1} -v -line #__LINE__		# True (t > T)
assert [ne This this] {== $_ 1} -v -line #__LINE__		#
assert [ne this This] {== $_ 1} -v -line #__LINE__		#


#debug on
if {catch {
	# Excaped #
	# FIX All cases fail **********************************
	newline
	textcolor yellow
	puts "Testing escaped strings"
	textcolor
	set s "puts \[funct foo 23\]"
	puts $s
	option echo on
	option expression off
	set s
	assert [eq $s "puts \[funct foo 23\]"] {== $_ 1} -v -line #__LINE__							# PASS
	#assert [eq $s [unescape "puts \[funct foo 23\]"]] {== $_ 1} -v -line #__LINE__				# Will FAIL
	assert [eq [unescape $s] [unescape "puts \[funct foo 23\]"]] {== $_ 1} -v -line #__LINE__		# PASS
	} } {
		textcolor red
		puts "Failed at line $line"
		textcolor
}

unset h s -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #