##ifdef comment
###########################################################################################
# File: 			test_comment.tcl
# Purpose:			Test file for comments
# Author: 			M Shaw
# Language:			Ticol Tcl v1.21+
#
# Versions:
# 1.00 17/Aug/2016	First version
# 1.01 16/Dec/2016	Fix re: altered long comments
# 1.02 23/Oct/2019	Add test for string-embedded long comment terminating tag
# 1.03 07/Nov/2019	Reverted ignore close tags in quoted strings in long comments
# 1.04 02/Dec/2019	Reverted ability to embed long comment /* */ tags within strings
###########################################################################################
# Notes: 
# FIX not working
# Included in the test framework via tests.tcl
# FIX Currently long comments have priority over macro ifdef blocks
# Ignoring * / in quoted strings in long comments worked well except 
# that a single dquote in a long comment could induce a fatal error
###########################################################################################

# FIX ifdef blocks should precede comments
#ifdef comment
# Dev test block for erroneous termination of embedded long comment tag within a string
puts "Line: "#__LINE__
set i 0
/* 		# Should act as a comment and thus i should NOT be set to 1
set i 1
set q " * / "		# TEMP DISABLED WITH SPACES				# DIS CLOSING TAG 02/DEC/2019
# */
set i 1
#exit
#endif
###########################################################################################


option expression off
puts "File: (macro) "#__FILE__
#set filename #__FILE__
set filename [info script]

newline
textcolor yellow
puts "Testing comments in $filename"
textcolor

# Test C++ comments
puts "line: "#__LINE__
set line #__LINE__
/*
die "$filename failed at line $line"		# die Failed
*/

# die "$filename failed at line $line"

###########################################################################################

# Feature disabled 02/Dec/2019
puts "line: "#__LINE__
set s "Hel / * .. * / lo"				# TEMP DISABLED WITH SPACES  # DIS TAGS 02/DEC/2019
if {ne $s "Hel / * .. * / lo"} {		# TEMP DISABLED WITH SPACES	 # DIS TAGS 02/DEC/2019
	die "Failed at line "#__LINE__
}

###########################################################################################
puts "line: "#__LINE__
set line #__LINE__
# Feature disabled 02/Dec/2019
# 2016/Dec/16 - Altered this behaviour to allow comment ignore only in quoted strings now (preproc_stream)
set s [list Hel/* .. */lo]				# FIX loses x1 trailing space
# if {ne $s "Hel /* .. */ lo"} {		# TEMP DISABLED WITH SPACES
if {ne $s "Hello"} {
	puts "s is '$s'"
	die "Failed at line "#__LINE__
}

###########################################################################################
puts "line: "#__LINE__
set line #__LINE__
set s [list Hel /* .. */lo]
# if {ne $s "Hel /* .. */ lo"} {	# TEMP DISABLED WITH SPACES
if {ne $s "Hel lo"} {	
	puts "s is '$s'"
	die "Failed at line "#__LINE__
}

###########################################################################################
puts "line: "#__LINE__
set line #__LINE__
set s [list Hel/* .. */ lo]
# if {ne $s "Hel /* .. */ lo"} {		# TEMP DISABLED WITH SPACES
if {ne $s "Hel lo"} {
	puts "s is '$s'"
	die "Failed at line "#__LINE__
}

###########################################################################################
puts "line: "#__LINE__
set line #__LINE__
set s [list Hel /* .. */ lo]			# FIX loses x1 trailing space
# if {ne $s "Hel /* .. */ lo"} {		# TEMP DISABLED WITH SPACES
if {ne $s "Hel lo"} {					# Should be "Hel  lo"
	puts "s is '$s'"
	die "Failed at line "#__LINE__
}
unset s

###########################################################################################
# Test long comment is effective
puts "line: "#__LINE__
set line #__LINE__
set s 0
/*
	set s 1
*/	
if {[!= $s 0]} {
	die "Test failed at line $line"
}

###########################################################################################
# Test commenting out a long comment is effective
puts "line: "#__LINE__
set line #__LINE__
set s 0
#/*
	set s 1
#*/	
if {[!= $s 1]} {
	die "Test failed at line $line"
}

###########################################################################################
# Test commenting out long comment with comment-embedded tail is effective
puts "line: "#__LINE__
set line #__LINE__
set s 0
/*
	set s 1
#*/	
if {[!= $s 0]} {
	die "Test failed at line $line"
}

##ifdef test_embedded_tags	# WONT WORK

#############################################################################
# Test string-embedded long comment
set q " /* foo */ "		# TEMP DISABLED WITH SPACES
assert $q {eq $_ " /* foo */ "} -v -line #__LINE__ 		# TEMP DISABLED WITH SPACES

puts "Line: "#__LINE__
set i 0
set q "  /* "		# Should not act as a comment and thus i should be set to 1 OK
set i 1
# */
puts "Line: "#__LINE__
assert $i {== $_ 1} -v -line #__LINE__

##############################################################################
# Test that a terminating tag embedded in a string does not terminate a comment
puts "Line: "#__LINE__
set i 0
/* 					# Should act as a comment start and thus i should NOT be set to 1
set i 1				# Ineffective
set q " * / "		# Comment should not be terminated by this embedded tag		# Feature disabled 02/Dec/2019
set i 2				# Comment should NOT end so i should NOT be set to 2
# */				# Comment should end HERE
puts "Line: "#__LINE__
assert $i {== $_ 0} -v -line #__LINE__

puts "Line: "#__LINE__
set i 0
/* 					# Should act as a comment start and thus i should NOT be set to 1
set i 1				# Ineffective
set q " Foobar * /"		# Comment should not be terminated by this embedded tag	# Feature disabled 02/Dec/2019
set i 2				# Comment should NOT end so i should NOT be set to 2
# */				# Comment should end HERE
puts "Line: "#__LINE__
assert $i {== $_ 0} -v -line #__LINE__

unset i q -nocomplain

#############################################################################
# Test comment within an array subscript def
# Should be fine as long comments are one of the earliest structures
# parsed by the MPP

# Leading comment
set a(sub\ script/* hi */) hi
assert $a(sub\ script) {eq $_ hi} -v -line #__LINE__
array unset a -nocomplain

# Middle comment
set a(sub\ /* hi */script) hi
assert $a(sub\ script) {eq $_ hi} -v -line #__LINE__
array unset a -nocomplain

# Trailing comment
set a(/* hi */sub\ script) hi
assert $a(sub\ script) {eq $_ hi} -v -line #__LINE__
array unset a -nocomplain

# Comment breaking an escape sequence
set a(sub\/* hi */ script) hi
assert $a(sub\ script) {eq $_ hi} -v -line #__LINE__
array unset a -nocomplain

# Check that string termination within long comment does not mask the trailing comment tag
set i 0
/*
set i "\
2
"  */
set i 1
set s " /* */ "		# Stop block (Should hopefully terminate the string parse if it overruns)
set s " /* */ "		# Stop block (Should hopefully terminate the string parse if it overruns)

assert $i {== $_ 1} -v -line #__LINE__
#############################################################################


##endif
unset filename i line s -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #
# Test for overrun hang at EOF... (remove/add lines here as nesc as 1, 2 or 3 lines)
# TEMP DISABLED WITH SPACES
/*
" * / "		# Feature disabled 02/Dec/2019
*/
puts "* Test overhang at EOF Done *"