#################################################################################
# File: 			test_compares.tcl
# Purpose:			Tests for various comparisons
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
# See also:			test_cmp.tcl (merge later)
#
# Versions:
# 1.00 30/Mar/2018	First version
#################################################################################
# Notes:
# Todo: Include weird comparisons often the subject of humour in say Javascript
#################################################################################
option expression off

assert [== {} {}] {eq $_ 1} -v -line #__LINE__
assert [== "" {}] {eq $_ 1} -v -line #__LINE__
assert [== {} ""] {eq $_ 1} -v -line #__LINE__
assert [== "" ""] {eq $_ 1} -v -line #__LINE__

# \0 not yet supported
#assert [== "" "\0"] {eq $_ 1} -v -line #__LINE__
#assert [== "" "\x00"] {eq $_ 1} -v -line #__LINE__

assert [== 0 0] {eq $_ 1} -v -line #__LINE__

assert [== "0" "0"] {eq $_ 1} -v -line #__LINE__
assert [== 0 "0"] {eq $_ 1} -v -line #__LINE__
assert [== "0" 0] {eq $_ 1} -v -line #__LINE__

assert [== {0} {0}] {eq $_ 1} -v -line #__LINE__
assert [== 0 {0}] {eq $_ 1} -v -line #__LINE__
assert [== {0} 0] {eq $_ 1} -v -line #__LINE__

assert [== "0" {0}] {eq $_ 1} -v -line #__LINE__
assert [== {0} "0"] {eq $_ 1} -v -line #__LINE__

#################################################################################
# EQ
#################################################################################
assert [eq {} {}] {eq $_ 1} -v -line #__LINE__
assert [eq "" {}] {eq $_ 1} -v -line #__LINE__
assert [eq {} ""] {eq $_ 1} -v -line #__LINE__
assert [eq "" ""] {eq $_ 1} -v -line #__LINE__

assert [eq 0 0] {eq $_ 1} -v -line #__LINE__

assert [eq "0" "0"] {eq $_ 1} -v -line #__LINE__
assert [eq 0 "0"] {eq $_ 1} -v -line #__LINE__
assert [eq "0" 0] {eq $_ 1} -v -line #__LINE__

assert [eq {0} {0}] {eq $_ 1} -v -line #__LINE__
assert [eq 0 {0}] {eq $_ 1} -v -line #__LINE__
assert [eq {0} 0] {eq $_ 1} -v -line #__LINE__

assert [eq "0" {0}] {eq $_ 1} -v -line #__LINE__
assert [eq {0} "0"] {eq $_ 1} -v -line #__LINE__


#################################################################################
# SC 
# Compare two strings and return base 1 index-offset at which there is
# a mismatch or return 0 if the strings are identical
#################################################################################
assert [sc foo bar] {== $_ 1} -v -line #__LINE__
assert [sc $qbf $qbf] {== $_ 0} -v -line #__LINE__
puts "$qbf [string toupper $qbf]"
assert [sc $qbf [string toupper $qbf] -nocase] {== $_ 0} -v -line #__LINE__
assert [sc [string tolower $qbf] $qbf -nocase] {== $_ 0} -v -line #__LINE__

#################################################################################
# CMP
# Returns a boolean TRUE if match found
#################################################################################
set q $qbf
assert [cmp foo bar 1] {== $_ $false} -v -line #__LINE__
assert [cmp foo foo 1] {== $_ $true} -v -line #__LINE__
assert [cmp $qbf $qbf 1] {== $_ $true} -v -line #__LINE__
# FIX false +ve match with differing case
#puts [cmp $qbf [string tolower $qbf]]
assert [cmp $qbf [string tolower $qbf]] {== $_ $false} -v -line #__LINE__
assert [cmp $qbf [string tolower $qbf] -nocase] {== $_ $true} -v -line #__LINE__

assert [cmp $qbf [string tolower $qbf] 1] {== $_ $false} -v -line #__LINE__
assert [cmp $qbf [string tolower $qbf] 1 -nocase] {== $_ $true} -v -line #__LINE__

assert [cmp $qbf [string tolower $qbf] 10] {== $_ $false} -v -line #__LINE__
assert [cmp $qbf [string tolower $qbf] 10 10 -nocase] {== $_ $false} -v -line #__LINE__

assert [cmp $qbf [string tolower $qbf] 42] {== $_ $false} -v -line #__LINE__
assert [cmp $qbf [string tolower $qbf] 42 -nocase] {== $_ $false} -v -line #__LINE__

assert [cmp $qbf [string tolower $qbf] 1 10] {== $_ $false} -v -line #__LINE__
assert [cmp $qbf [string tolower $qbf] 1 10 -nocase] {== $_ $true} -v -line #__LINE__


# The quick brown fox jumps over the lazy dog
# 1234567890123456789012345678901234567890
#         10        20        30 
assert [cmp $qbf fox 17 3] {== $_ $true} -v -line #__LINE__
assert [cmp $qbf FOX 17 3 -nocase] {== $_ $true} -v -line #__LINE__
assert [cmp $qbf "lazy dog" 36 8] {== $_ $true} -v -line #__LINE__


# Traps
set i 0
try {
	cmp $qbf [string tolower $qbf] -3
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

set i 0
try {
	cmp $qbf [string tolower $qbf] 1 -3
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__


option expression off
set in_comment $false
set data "abcd efgh /* ijkl */ mnop"	# Comment starts at 11
set count [string length $data]
for {set i 1} {< $i $count} {incr i} {
   if { cmp data "/*" $i 2 } {
	  set in_comment $true
	  set comment_offset $i
	  break
   }
}
puts "in comment? $in_comment comment offset? $comment_offset"
assert $in_comment {== $_ $true} -v -line #__LINE__
assert $comment_offset {== $_ 11} -v -line #__LINE__

#################################################################################

unset comment_offset count data i in_comment q -nocomplain

#################################################################################
vars
newline
textcolor green
puts "* Pass *"
textcolor

# EOF #