######################################################################################
# File:			test_convert.tcl
# Author:		M Shaw
# Purpose:		Test [mkn] [cvn]
# Language:		Ticol Tcl
#
# Versions:
# 1.00 24/Feb/2017	First version (test results)
######################################################################################
# Notes:
# Tests classic BASIC - like Mki(), Cvi() commands
# Renamed as [mkn/cvn] since [mki]==[mkf]
# We use BitPack lib functions to return a string which packs into a 4-bit charset
# This charset is guaranteed not to contain embedded NULL chars
# ------------------------------------------------------------------
# ASCII Value Input
# 			Encoding Symbol
# 						Comment
# ------------------------------------------------------------------
# 48..57		0x1..0xa	'0..9' Decimal digit
# 43			0xb 11		'+'    Positive sign
# 44			0xc 12		','    Comma (non-UK thousands separator)
# 45			0xd 13		'-'    Negative sign 
# 46			0xe 14		'.'	   Decimal point
# 32			0xf 15		' '    Space
#  0			0x0			'\0'   Reserved for C/C++ NULL terminator
# ------------------------------------------------------------------
######################################################################################

/*
Bitpack HEX[4E2526A376469A8A434957375449438A6139952A827A4AA486216931A85A556A3418927517397319AA9739145936453228178A11]
BitUnPack= [3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067900]
*/
option echo on
option expression off

textcolor yellow
puts "Test Conversion Commands \[mkn\] \[cvn\] ...\n"
textcolor

set bigpie 3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067900
set s [mkn $bigpie]
puts "s='$s' len=[string length $bigpie]->[string length $s]"
assert [cvn $s] {== $_ 3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067900} -v -line #__LINE__
assert [mkn 0] {eq $_ [chr 16]} -v -line #__LINE__

assert [mkn 123.456] {eq $_ "#NVp"} -v -line #__LINE__

for {set i 0} {< $i 10} {++ i} {
	#puts "$i [mkn $i] [inttohex [asc [mkn $i]]] asc:[asc [mkn $i]] shift:[<<  [+ 1 $i] 4] chr:'[chr [<<  $i 4]]'"
	puts "\[eq \[mkn $i\] \[chr \[<< \[+ 1 $i\] 4\]\]\]"
#	puts "\[eq [mkn $i] [chr [<< [+ 1 $i] 4]]\]"
	assert [eq [mkn $i] [chr [<< [+ 1 $i] 4]]]  {== $_ 1} -v -line #__LINE__
}

unset bigpie i s -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #