##########################################################################
# File: 			test_copymove.tcl
# Purpose:			Testing [copyfile] and [movefile]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 07/Jul/2017	First version
##########################################################################
# Notes:
# TODO add copyfile test
# FIX bug in parser not unescaping \$, move processing from commands?
##########################################################################

option expression off
set testfile "test.\$\$\$"

# set testfile "test.tst"
# puts "escaped dollars \$\$\$"
# stop
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

puts "Line "#__LINE__
md "test"

# [movefile]
puts "Line "#__LINE__
textcolor yellow
puts "Test \[movefile\] ..."
textcolor

set line #__LINE__
puts "Line: $line"
# @ dir test* /ad

if {file exists "test/"} {
	file delete "test/$testfile"		# Ensure no lingering file in the target
	file delete $testfile -verbose				# We delete before starting to
	puts "Exists? [file exists $testfile] (should be false) line: "#__LINE__
	assert [file exists $testfile] {== $_ 0} -v -line #__LINE__
	if {file exists $testfile} {		# Ensure we don't have $testfile as R/O or a directory etc.
		die "Can't remove $testfile before starting test"
	}
	## Create a file ########
	set fp [open $testfile w]
	puts $fp "Hello"
	close $fp
	#########################
	set line #__LINE__
	puts "Line: $line"
	puts [file exists test]
	assert [file exists "test"] {== $_ 1} -v -line #__LINE__			# If target dir exists
	assert [file exists $testfile] {== $_ 1} -v -line #__LINE__			# If source file exists
	assert [movefile $testfile "test/"] {== $_ 1} -v -line #__LINE__	# Move to ./test
	assert [file exists "test/$testfile"] {== $_ 1} -v -line #__LINE__
	assert [movefile test/$testfile .] {== $_ 1} -v -line #__LINE__		# Move back
	assert [file exists $testfile] {== $_ 1} -v -line #__LINE__
	file delete $testfile 												# Clean up after
} else {
	die "Failed to create ./test subfolder at $line"
}
newline

# [copyfile]
puts "Line "#__LINE__
textcolor yellow
puts "Test \[copyfile\] ..."
textcolor

if {file exists "test/"} {
	file delete "test/$testfile"				# Ensure no lingering file in the target
	file delete $testfile -verbose				# We delete before starting to
	puts "Exists? [file exists $testfile]"
	if {file exists $testfile} {				# Ensure we don't have $testfile as R/O or a directory etc.
		die "Can't remove $testfile before starting test"
	}
	## Create a file ########
	set fp [open $testfile w]
	puts $fp "Hello"
	close $fp
	#########################
	puts "Line "#__LINE__
	puts [file exists test]
	assert [file exists "test"] {== $_ 1} -v -line #__LINE__			# If target dir exists

	assert [copyfile $testfile "test/"] {== $_ 1} -v -line #__LINE__	# Copy to ./test

	puts "Line "#__LINE__
	assert [file exists test/$testfile] {== $_ 1} -v -line #__LINE__

	puts "Line "#__LINE__
	file delete test/$testfile											# Clean up after
	assert [! [file exists test/$testfile]] {== $_ 1} -v -line #__LINE__

	puts "Line "#__LINE__
	file delete $testfile 												# Clean up after
	assert [! [file exists $testfile]] {== $_ 1} -v -line #__LINE__
	
} else {
	die "Failed to create ./test subfolder"
}

puts "Line "#__LINE__
unset fp line testfile -nocomplain


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #