######################################################################################
# File:				test_csv.tcl
# Author:			M Shaw
# Purpose:			Test CSV input
# Language:			Ticol Tcl
#
# Versions:
# 1.00 04/Aug/2019	First version
# 1.01 15/Jan/2022	Added file open retries and amended syntax
######################################################################################
# Notes:
#   input-csv filename ?varname | -? ?-max items-max? ?-crlf? ?-array name?\
#	 ?-r? ?-retries retry-count?
# DEPRECATED input-csv filename ?varname | -? ?items-max? ?-crlf? ?-array name? ?-r?
# DONE FIX Multi-spawn tests require [is_locked r|w] to control (add open retries)
######################################################################################
option expression off
option echo push on

set q [chr 34]
set filename test.csv

/*
# We now unlink in the root file, tests.tcl so we can run multiple simultaneous tests
unlink $filename -nocomplain
if {file exists $filename} {
	die "$filename exists. Ensure it is safe to remove"
} # */

puts "Line: "#__LINE__
writefile $filename "${q}Hello world${q},1234,${q}ABC${q},5.678"
writefile $filename "${q}The quick brown fox${q},0123,${q}CDE${q},23.23"
set s [input_csv $filename - -max 4 -retries 200]
  
assert $s {eq $_ "{The quick brown fox} 0123 CDE 23.23"} -v -line #__LINE__

puts "Line: "#__LINE__
unset s -nocomplain

puts "Line: "#__LINE__
# FIX improve and add a retry loop
try {
	input_csv $filename - -max 4 -array a -retries 200
} catch {
	try {
		sleep 1000
		input_csv $filename - -max 4 -array a -retries 200
	} catch {
		puts "Was unable to open input_csv after a 1 second retry"
		unlink $filename -nocomplain
		array unset a -nocomplain
		unset s -nocomplain
		option echo pop
		return
	}
}
vars a*
assert [array exists a] {== $_ 1} -v -line #__LINE__
array unset a -nocomplain

# Loop test for stability
# Input/set, assert and unset loop
# On multiple, simultaneous tests it may fail due to file locking
# For heavy soak-testing we need to increase lock retries in this loop to max of 200
loop i 0 10 {		# 0..9 inclusive (10)
	input_csv $filename - -max 4 -array a -retries 200
	assert [array exists a] {== $_ 1} -v -line #__LINE__
	array unset a -nocomplain
}

puts "Line: "#__LINE__
unlink $filename -nocomplain

array unset a -nocomplain
unset s -nocomplain

option echo pop

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
vars
newline
textcolor green
puts "* PASS *"
textcolor

# EOF #