#################################################################################
# File: 			test_ctype.tcl
# Purpose:			Tests for emulation of "C" / C++ macros
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 30/Sep/2018	First version
# 1.01 27/Jul/2019	Added ctype like comparison command tests (is_*)
#################################################################################
# Notes:
# FIX beeps
# Some tests using C++ macro emulation
# Not strictly necessary but adds more stress points to the test range
#################################################################################
option expression off
option echo on

textcolor yellow
puts "C++ macro emulation"
textcolor
newline

undef toupper -nocomplain

##define _toupper(_c)	( (_c)-'a'+'A' )
# "C" - toupper  puts [chr [& [asc a] [~ 32]]]

proc toupper c {
	if {&& [>= $c a] [<= $c z]} {
		return [chr [& [asc $c] [~ 32]]]
	} else {
		return $c
	}
}
undef tolower -nocomplain

# Works only for strict a-z range
# "C" - tolower puts [chr [| [asc B] 32]]
#define _tolower(_c)	( (_c)-'A'+'a' )
proc tolower c {
	if {&& [>= $c A] [<= $c Z]} {
		return [chr [| [asc $c] 32]]
	}
	return $c
}

for {set i 1} {< $i 255} {++ i} {
	puts $i
	# if {[>= $i 65]} {puts "$i >= 65"}
	# if {[<= $i 90]} {puts "$i >= 90"}
	if {[&& [>= $i 65] [<= $i 90]]} {					# A..Z
		#puts "A tolower($i) [chr $i]->[tolower [chr $i]]"
		assert [tolower [chr $i]] {eq $_ [string tolower [chr $i]]} -v -line #__LINE__
	} elseif {&& [>= $i 97] [<= $i 122]} {			# a..z 97..122
		#puts "B toupper($i) [chr $i]]->[toupper [chr $i]]"
		assert [toupper [chr $i]] {eq $_ [string toupper [chr $i]]} -v -line #__LINE__
	} else {
		#puts "C toupper char $i -> [toupper [chr $i]]"
		assert [toupper [chr $i]] {eq $_ [chr $i]} -v -line #__LINE__
		#puts "C tolower char $i -> [tolower [chr $i]]"
		assert [tolower [chr $i]] {eq $_ [chr $i]} -v -line #__LINE__
	}
}
newline

assert [toupper .] {eq $_ .} -v -line #__LINE__
assert [toupper @] {eq $_ @} -v -line #__LINE__
assert [toupper a] {eq $_ A} -v -line #__LINE__
assert [toupper j] {eq $_ J} -v -line #__LINE__
assert [toupper z] {eq $_ Z} -v -line #__LINE__

assert [tolower @] {eq $_ @} -v -line #__LINE__
assert [tolower A] {eq $_ a} -v -line #__LINE__
assert [tolower J] {eq $_ j} -v -line #__LINE__
assert [tolower Z] {eq $_ z} -v -line #__LINE__
newline

undef tolower toupper -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Test C++ ctype emulation..."
textcolor

# is_* #
# is_alpha
# is_alnum
# is_digit
# is_lower
# is_punct
# is_space
# is_upper
# is_xdigit
set s "abc"
assert [is_alpha s -var] {eq $_ 1} -v -line #__LINE__
assert [is_alpha $s] {eq $_ 1} -v -line #__LINE__
assert [is_alnum s -var] {eq $_ 1} -v -line #__LINE__
assert [is_alnum $s] {eq $_ 1} -v -line #__LINE__

set s "."
assert [is_alpha s -var] {eq $_ 0} -v -line #__LINE__
assert [is_alpha $s] {eq $_ 0} -v -line #__LINE__
assert [is_alnum s -var] {eq $_ 0} -v -line #__LINE__
assert [is_alnum $s] {eq $_ 0} -v -line #__LINE__

set s "012"
assert [is_alnum s -var] {eq $_ 1} -v -line #__LINE__
assert [is_alnum $s] {eq $_ 1} -v -line #__LINE__
assert [is_digit s -var] {eq $_ 1} -v -line #__LINE__
assert [is_digit $s] {eq $_ 1} -v -line #__LINE__
assert [is_xdigit s -var] {eq $_ 1} -v -line #__LINE__
assert [is_xdigit $s] {eq $_ 1} -v -line #__LINE__

set s "."
assert [is_alnum s -var] {eq $_ 0} -v -line #__LINE__
assert [is_alnum $s] {eq $_ 0} -v -line #__LINE__
assert [is_digit s -var] {eq $_ 0} -v -line #__LINE__
assert [is_digit $s] {eq $_ 0} -v -line #__LINE__
assert [is_xdigit s -var] {eq $_ 0} -v -line #__LINE__
assert [is_xdigit $s] {eq $_ 0} -v -line #__LINE__

set s "abcdef0123456789"
assert [is_xdigit s -var] {eq $_ 1} -v -line #__LINE__
assert [is_xdigit $s] {eq $_ 1} -v -line #__LINE__

set s "ABCDEF0123456789"
assert [is_xdigit s -var] {eq $_ 1} -v -line #__LINE__
assert [is_xdigit $s] {eq $_ 1} -v -line #__LINE__

set s "abc"
assert [is_lower s -var] {eq $_ 1} -v -line #__LINE__
assert [is_upper s -var] {eq $_ 0} -v -line #__LINE__

set s "abc."
assert [is_lower s -var] {eq $_ 0} -v -line #__LINE__
assert [is_upper s -var] {eq $_ 0} -v -line #__LINE__

set s "ABC"
assert [is_lower s -var] {eq $_ 0} -v -line #__LINE__
assert [is_upper s -var] {eq $_ 1} -v -line #__LINE__

set s "ABC."
assert [is_lower s -var] {eq $_ 0} -v -line #__LINE__
assert [is_upper s -var] {eq $_ 0} -v -line #__LINE__

set s "."
assert [is_lower s -var] {eq $_ 0} -v -line #__LINE__
assert [is_upper s -var] {eq $_ 0} -v -line #__LINE__

set s ".,;(){}"	# PUNCTUATOR
assert [is_punct s -var] {eq $_ 1} -v -line #__LINE__
assert [is_space s -var] {eq $_ 0} -v -line #__LINE__

set s " "		# SPACE
assert [is_punct s -var] {eq $_ 0} -v -line #__LINE__
assert [is_space s -var] {eq $_ 1} -v -line #__LINE__

set s "\t"		# TAB
assert [is_punct s -var] {eq $_ 0} -v -line #__LINE__
assert [is_space s -var] {eq $_ 1} -v -line #__LINE__


###########################################################################################
unset i s -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #