#################################################################################
# File: 			test_data_ranges.tcl
# Purpose:			Tests for data type ranges, round-trips etc.
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 16/Nov/2017	First version
# 1.01 01/Aug/2018	Check for presence of defs.tcl and report
# 1.02 27/Jul/2019	Undefed print_binary before def to enable loop testing
# 1.03 31/Jul/2019	Bugfix for misalignment on re-inclusion
# 1.04 31/Aug/2019	Updates to signed/unsigned arithmetic
#################################################################################
# Notes: 
# FIX misalignment in replacement on 2nd loop call of 0xFFFFFFFFFFFFFFFF
#################################################################################
# TODO 
# 0xFFFFFFFFFFFFFFFF
# 0xFFFFFFFFFFFFFFFF
# Admin@Lenovo 1068> puts [unsigned 0xFFFFFFFFFFFFFFFF]
# 18446744073709551615
# 18446744073709551615
# Add min max range tests
# 9223372036854775807
# 8446744073709551615 WRONG
# 20000000000000000000
# char               -128 to 127
# short              -32768 to 32767
# long               -2147483648 to 2147483647
# int                -9223372036854775808 to 9223372036854775807
# int unsigned        0 to 18446744073709551615
# 8446744073709551615
# 9223372036854775807
#################################################################################

option echo on
option expression off
option 
newline

textcolor yellow
puts "Testing data ranges and casting in test_data_ranges.tcl ..."
textcolor
newline

if {! [file exists defs.tcl]} {
	die "File [info script] requires defs.tcl"
}

source defs.tcl		# Include useful const definitions for minima/maxima

#/*
undef print_binary -nocomplain
puts "Line: "#__LINE__

proc print_binary {x} {
	# Prints formatted binary
	puts "Enter proc print_binary {$x}"
	set s [makestr 66]
	set b [inttobin $x -noprefix]
	setr s $b
	textcolor darkyellow
	puts "$s ([string length $b])"
	textcolor
}

puts "Line: "#__LINE__

assert [hextoint [inttohex 9223372036854775807]] {== $_ 9223372036854775807} -v -line #__LINE__

textcolor yellow
puts "Testing signed 8-bit ..."
textcolor
# Signed 8 bit # # # # # # # # # # # # # # # # # #
puts '[char 2]'
assert [char 2] {== $_ 2} -v -line #__LINE__
puts '[char 127]'
assert [char 127] {== $_ 127} -v -line #__LINE__
puts '[char 128]'
puts '[char 129]'
puts '[char 250]'
assert [char 128] {== $_ -128} -v -line #__LINE__
puts '[char 255]'
assert [char 255] {== $_ -1} -v -line #__LINE__
puts '[char 256]'
assert [char 256] {== $_ 0} -v -line #__LINE__
puts '[char 257]'
assert [char 257] {== $_ 1} -v -line #__LINE__
newline 

textcolor yellow
puts "Testing unsigned 8-bit ..."
textcolor
# Unsigned 8 bit # # # # # # # # # # # # # # # # # #
puts '[uchar 2]'
assert [uchar 2] {== $_ 2} -v -line #__LINE__
puts '[uchar 255]'
assert [uchar 255] {== $_ 255} -v -line #__LINE__
puts '[uchar 256]'
assert [uchar 256] {== $_ 0} -v -line #__LINE__
puts '[uchar 257]'
assert [uchar 257] {== $_ 1} -v -line #__LINE__
newline

textcolor yellow
puts "Testing signed 16-bit ..."
textcolor
# [short] 16 bit # # # # # # # # # # # # # # # # # #
puts '[short 255]'
assert [short 255] {== $_ 255} -v -line #__LINE__
puts '[short 32767]'
assert [short 32767] {== $_ 32767} -v -line #__LINE__
puts '[short 32768]'
print_binary 32768
print_binary [short 32768]
assert [short 32768] {== $_ -32768} -v -line #__LINE__		# 2's compliment+1
puts '[short 32769]'
assert [short 32769] {== $_ -32767} -v -line #__LINE__		# Do.
puts '[short 32770]'
assert [short 32770] {== $_ -32766} -v -line #__LINE__		# Do.
puts '[short 65535]'
assert [short 65535] {== $_ -1} -v -line #__LINE__
puts '[short 65536]'
assert [short 65536] {== $_ 0} -v -line #__LINE__
puts '[short 65537]'
assert [short 65537] {== $_ 1} -v -line #__LINE__
newline

# [ushort] 16 bit # # # # # # # # # # # # # # # # # #
textcolor yellow
puts "Testing unsigned 16-bit ..."
textcolor
puts '[ushort 255]'
assert [ushort 255] {== $_ 255} -v -line #__LINE__
puts '[ushort 32767]'
assert [ushort 32767] {== $_ 32767} -v -line #__LINE__
puts '[ushort 32768]'
print_binary 32768
print_binary [ushort 32768]
assert [ushort 32768] {== $_ 32768} -v -line #__LINE__		# 2's compliment+1
puts '[ushort 32769]'
assert [ushort 32769] {== $_ 32769} -v -line #__LINE__		# Do.
puts '[ushort 32770]'
assert [ushort 32770] {== $_ 32770} -v -line #__LINE__		# Do.
puts '[ushort 65535]'
assert [ushort 65535] {== $_ 65535} -v -line #__LINE__
puts '[ushort 65536]'
assert [ushort 65536] {== $_ 0} -v -line #__LINE__
puts '[ushort 65537]'
assert [ushort 65537] {== $_ 1} -v -line #__LINE__
newline

# [long] 32 bit # # # # # # # # # # # # # # # # # #
textcolor yellow
puts "Testing signed 32-bit ..."
textcolor
puts '[long 65537]'
assert [long 65537] {== $_ 65537} -v -line #__LINE__
# CHECK
puts "Line: "#__LINE__
print_binary 0xFFFFFFFFFFFFFFFF
puts "Line: "#__LINE__
print_binary 0xFFFFFFFF
puts "Line: "#__LINE__
if {! $tcl_threaded} {
	print_binary [~ 0xFFFFFFFF]
}
puts "Line: "#__LINE__
print_binary [long 655376553765537]
puts "Line: "#__LINE__
print_binary 655376553765537
puts "Line: "#__LINE__
print_binary [long 655376553765537]
puts "Line: "#__LINE__
assert [long 655376553765537] {== $_ -1095865695} -v -line #__LINE__
newline

puts "Line: "#__LINE__
puts 9223372036854775807
puts "Line: "#__LINE__
print_binary 9223372036854775807
puts "Line: "#__LINE__
puts [inttobin 9223372036854775807]
puts "Line: "#__LINE__
puts [bintoint 0b111111111111111111111111111111111111111111111111111111111111111]
puts "Line: "#__LINE__
puts "9223372036854775807 round trip: [bintoint [inttobin 9223372036854775807]]"
puts "Line: "#__LINE__
puts "long..."
#print_binary [long 9223372036854775807]		# wrong!!!!!!! (bin==64 bits)
puts "RT cast: [bintoint [inttobin [long 9223372036854775807]]]"
puts "Line: "#__LINE__
puts "int..."
puts "Line: "#__LINE__
print_binary $UINT_MAX							# 65 bits
puts "Line: "#__LINE__
puts "UINT_MAX: [bintoint [inttobin $UINT_MAX]]"
puts "Line: "#__LINE__
print_binary [int 9223372036854775807]		# 64 bits
puts "Line: "#__LINE__
assert [long 9223372036854775807] {== $_ -1} -v -line #__LINE__
assert [int 9223372036854775807] {== $_ 9223372036854775807} -v -line #__LINE__
newline

textcolor yellow
puts "Testing unsigned 32-bit ..."
textcolor
# [ulong] 32 bit # # # # # # # # # # # # # # # # # #
# const ULONG_MIN 0
# const ULONG_MAX 4294967295
assert $ULONG_MAX  {== $_ $::ULONG_MAX} -v -line #__LINE__
assert $ULONG_MIN  {== $_ $::ULONG_MIN} -v -line #__LINE__
assert [ulong $ULONG_MAX]  {== $_ $::ULONG_MAX} -v -line #__LINE__
assert [ulong $ULONG_MIN]  {== $_ $::ULONG_MIN} -v -line #__LINE__
assert [ulong 0]  {== $_ 0} -v -line #__LINE__
assert [ulong 4294967295]  {== $_ 4294967295} -v -line #__LINE__
puts [ulong -1]
assert [ulong -1]  {== $_ 4294967295} -v -line #__LINE__				# Will truncate
puts [ulong 4294967296]
assert [ulong 4294967296] {== $_ 0} -v -line #__LINE__				# Will truncate as 0xFFFF:FFFF
puts [ulong 4294967297]
assert [ulong 4294967297] {== $_ 1} -v -line #__LINE__				# Will truncate as 0xFFFF:FFFF
puts [ulong 4294967298]
assert [ulong 4294967298] {== $_ 2} -v -line #__LINE__				# Will truncate as 0xFFFF:FFFF

puts [long 4294967296]
assert [long 4294967296] {== $_ 0} -v -line #__LINE__				# Will truncate as 0xFFFF:FFFF
puts [long 4294967297]
assert [long 4294967297] {== $_ 1} -v -line #__LINE__				# Will truncate as 0xFFFF:FFFF
puts [long 4294967298]
assert [long 4294967298] {== $_ 2} -v -line #__LINE__				# Will truncate as 0xFFFF:FFFF


puts "Line: "#__LINE__
# [unsigned]
print_binary [unsigned 255]
assert [unsigned 255] {== $_ 255} -v -line #__LINE__
print_binary [unsigned 65535]
assert [unsigned 65535] {== $_ 65535} -v -line #__LINE__
# QUERY...
print_binary -65535
assert [int -65535] {== $_ -65535} -v -line #__LINE__		# Wrong!
print_binary [unsigned -65535]
#const UINT_MAX 18446744073709551615-65535+1
puts "[- 18446744073709551615 65535]"
assert [unsigned -65535] {== $_ 18446744073709486081} -v -line #__LINE__		# Wrong!

textcolor yellow
puts "Testing some binary conversion ..."
textcolor
# Some binary stuff
#  8446744073709551615 Actual range
#  9223372036854775807 INT_MAX
# 18446744073709551615 UINT_MAX
print_binary 8446744073709551615
puts "\[bintoint \[inttobin 8446744073709551615\]\]=[bintoint [inttobin 8446744073709551615]]"
assert [bintoint [inttobin 8446744073709551615]] {== $_ 8446744073709551615} -v -line #__LINE__
puts "Line: "#__LINE__
# Mis parse in tcl_threaded version
# Possible string overrun in MPP
puts "\[bintoint \[inttobin 0xFFFFFFFFFFFFFFFF\]\] (2+16)=[bintoint [inttobin 0xFFFFFFFFFFFFFFFF]]"
puts "Line: "#__LINE__
puts [uint 0xFFFFFFFFFFFFFFFF]" UINT_MAX"		# FIX parse error or use [printf]
puts [uint 0x7FFFFFFFFFFFFFFF]"  INT_MAX"
printf "%s %s\n" [uint 0xFFFFFFFFFFFFFFFF] " UINT_MAX"		# FIX parse error
printf "%s %s\n" [uint 0x7FFFFFFFFFFFFFFF] "  INT_MAX"
assert [bintoint [inttobin [uint 0xFFFFFFFFFFFFFFFF]]] {== $_ 18446744073709551615} -v -line #__LINE__	# UNSIGNED INT MAX
print_binary [inttobin 0xFFFFFFFFFFFFFFFF]
puts [bintoint [inttobin 0xFFFFFFFFFFFFFFFF]]	#  Binary conversion casts to unsigned
puts [signed [bintoint [inttobin 0xFFFFFFFFFFFFFFFF]]]
assert [bintoint [inttobin 0xFFFFFFFFFFFFFFFF]] {== [signed $_] -1} -v -line #__LINE__	# SIGNED INT MAX
puts "\[bintoint \[inttobin 0x7FFFFFFFFFFFFFFF\]\]=[bintoint [inttobin 0x7FFFFFFFFFFFFFFF]]"
assert [bintoint [inttobin 0x7FFFFFFFFFFFFFFF]] {== $_ 9223372036854775807} -v -line #__LINE__	# SIGNED INT_MAX
puts "\[bintoint \[inttobin \"0xFFFFFFFFFFFFFFFF\"\]\]=[bintoint [inttobin "0xFFFFFFFFFFFFFFFF"]]"
puts "\[bintoint \[inttobin \"0x7FFFFFFFFFFFFFFF\"\]\]=[bintoint [inttobin "0x7FFFFFFFFFFFFFFF"]]"
set s [string repeat 1 64]
puts $s
puts [bintoint $s]
newline

textcolor yellow
puts "Testing misc casting ..."
textcolor
# bintoint 0xFFFFFFFF:FFFFFFFF -> 18446744073709551615 (uint64_max)		# 8446744073709551615
# char
puts "assert CHAR_MAX : $CHAR_MAX cycle @"#__LINE__
assert [char $CHAR_MAX] {== $_ $::CHAR_MAX} -v -line #__LINE__		#
puts "assert CHAR_MIN : '$CHAR_MIN' cycle, '$::CHAR_MIN' \[char\]-> [char $CHAR_MIN] @"#__LINE__
# FIX [char] removes sign bit
assert [char $CHAR_MIN] {== $_ $::CHAR_MIN} -v -line #__LINE__		#
puts [char 0]
assert [char 0] {== $_ 0} -v -line #__LINE__			#
puts [char "0xFF"]
assert [char "0xFF"] {== $_ -1} -v -line #__LINE__		#
# short
assert [short $SHORT_MAX] {== $_ $::SHORT_MAX} -v -line #__LINE__		#
assert [short $SHORT_MIN] {== $_ $::SHORT_MIN} -v -line #__LINE__		#

assert [short "0x7FFF"] {== $_ $::SHORT_MAX} -v -line #__LINE__		#
assert [short "0xFFFF"] {== $_ -1} -v -line #__LINE__		#
puts [inttohex -32768]
#assert [inttohex -32768] {eq $_ "0x8000"} -v -line #__LINE__		#
puts "0x8000 binary is:"
print_binary 0x8000
assert [short "0x8000"] {== $_ -32768} -v -line #__LINE__		#
# long
assert [long $LONG_MAX] {== $_ $::LONG_MAX} -v -line #__LINE__		#
assert [long $LONG_MIN] {== $_ $::LONG_MIN} -v -line #__LINE__		#
assert [int $LONG_MAX] {== $_ $::LONG_MAX} -v -line #__LINE__		#
assert [int $LONG_MIN] {== $_ $::LONG_MIN} -v -line #__LINE__		#

# int
puts "Line: "#__LINE__
# const UINT_MAX 18446744073709551615			# Macro const
# */

# vars -const
dummy 0xFFFFFFFFFFFFFFFF
puts "Line: "#__LINE__
echo "ef ef.. ->0xFFFFFFFFFFFFFFFF<-"
printf "assert %I64u == 18446744073709551615\n" 0xFFFFFFFFFFFFFFFF
assert 0xFFFFFFFFFFFFFFFF {== $_ 18446744073709551615} -v -line #__LINE__				# WRONG (why?)
printf "assert %I64u == ::UINT_MAX ($::UINT_MAX)\n" 0xFFFFFFFFFFFFFFFF
assert 0xFFFFFFFFFFFFFFFF {== $_ $::UINT_MAX} -v -line #__LINE__				# WRONG (why?)
puts "Line: "#__LINE__
assert [int "0xFFFFFFFFFFFFFFFF"] {== $_ $::UINT_MAX} -v -line #__LINE__		# WRONG
puts "Line: "#__LINE__
assert [unsigned 0xFFFFFFFFFFFFFFFF] {== $_ $::UINT_MAX} -v -line #__LINE__		# WRONG

puts "Line: "#__LINE__
unset s -nocomplain
undef print_binary -nocomplain

#################################################################################
newline
vars
newline
textcolor green
puts "* test_data_ranges.tcl Done *"
textcolor

# EOF #