##############################################################################
# File:				testdate.tcl
# Author:			M Shaw
# Purpose:			Test VB/.Net style dates (double)
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 18/Oct/2016	First version
# 1.01 27/Oct/2016	Added test cases
# 1.02 15/Jan/2017	Migrated in misc date/time from tests.tcl
# 1.03 25/Jan/2017	Added [date -separator X] arg
# 1.04 06/Apr/2017	Added addition/subtraction offset cases
# 1.05 22/Jul/2017	Replaced [checkex] with [assert]
# 1.06 04/Dec/2017	Fix broken last test
# 1.07 02/Jan/2020	Bugfix
##############################################################################
# Notes:
# Move time-related commands to test_time ?
##############################################################################

option expression off

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# vbnow
textcolor yellow
puts "Testing \[vbnow\]"
textcolor
assert [now] {> $_ 42121} -v -line #__LINE__
newline

# ticks
textcolor yellow
puts "Testing \[ticks\]"
textcolor
assert [ticks] {> $_ 0} -v -line #__LINE__

# time
textcolor yellow
puts "Testing \[time\]"
textcolor
assert [left [time] 2] {>= $_ 0} -v -line #__LINE__

# clock #
# clock clicks|seconds|format
# clock format seconds|now ?-format %Y-%m-%dT%H:%M:%S?
textcolor yellow
puts "Testing \[clock\]"
textcolor

assert [clock clicks] {!= $_ 0} -v -line #__LINE__
assert [clock seconds] {!= $_ 0} -v -line #__LINE__
assert [clock format now -format "%Y %m %d %H:%M:%S"] {== [lcount $_] 4} -v -line #__LINE__
assert [clock format now -format "%Y %m %d %H:%M:%S"] {>= [left $_ 4] 2015} -v -line #__LINE__
# Broke clock 9/Apr/2016
assert [clock format 9999 -format "%Y %m %d %H:%M:%S"] {eq $_ "1970 01 01 02:46:39"} -v -line #__LINE__

	
# ctime_to_double
textcolor yellow
puts "Testing \[ctime_to_double\]"
textcolor
assert [ctime_to_double 1] {== $_ 25569.000011574073} -v -line #__LINE__
assert [ctime_to_double 10000] {== $_ 25569.115740740741} -v -line #__LINE__

# date
textcolor yellow
puts "Testing \[date\]"
textcolor
debug off
# works puts [eq [left 2015-04-01 4] 2015]
assert [date] {== [left $_ 3] 202} -v -line #__LINE__		# Valid 2020 to 2029

# double_to_ctime
# Round-trip test
textcolor yellow
puts "Testing \[double_to_ctime \[ctime_to_double x\]\]"
textcolor
assert [double_to_ctime [ctime_to_double 1]] {== $_ 1} -v -line #__LINE__
assert [double_to_ctime [ctime_to_double 768027063]] {== $_ 768027063} -v -line #__LINE__

#ifdef comment ////////////////////////////////////////////////////////////////////
struct tm *today, *gmt, xmas = { 0, 0, 12, 25, 11, 93 };

OS time:                                21:51:03
OS date:                                05/03/94
Time in seconds since UTC 1/1/70:      768027063
UNIX time and date:                     Tue May 03 21:51:03 1994
Coordinated universal time:             Wed May 04 04:51:03 1994
12-hour time:                           09:51:03 PM
Plus milliseconds:                      279
Zone difference in seconds from UTC:    480
Time zone name:                         {}
Daylight savings:                       YES
Christmas                               Sat Dec 25 12:00:00 1993
Today is 								Tuesday, day 03 of May in the year 1994.

#endif ////////////////////////////////////////////////////////////////////////////


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

textcolor yellow
puts "Testing \[date -year|-month|-day\]"
textcolor

set year [date -year]
set month [date -month]
set day [date -day]
set yday [date -yday]


assert [>= $yday $day] {== $_ 1} -v -line #__LINE__
# Checking: 2017-2-3 42769.0677893519 9999999
# Line 154: '$x'=='0' [expr == $x 1]:
puts "Checking: $year-$month-$day [date now] [date $year-$month-$day]"
#assert [eq [int [date now]] [date [format %4.4i-%2.2i-%2.2i $year-$month-$day]]] {== $_ 1}
assert [eq [int [date now]] [date $year-$month-$day]] {== $_ 1} -v -line #__LINE__
assert [int [date $year/$month/$day]] {== $_ [int [now]]} -v -line #__LINE__

puts "date: Testing VB style date conversion..."
newline

puts "Checking eq [date -separator @] [format %4i@%2.2i@%2.2i $year $month $day]"
assert [eq [date -separator @] [format "%4i@%2.2i@%2.2i" $year $month $day]] {== $_ 1} -v -line #__LINE__		# 2017@01@25
assert [eq [date -separator /] [format "%4i/%2.2i/%2.2i" $year $month $day]] {== $_ 1} -v -line #__LINE__		# 2017@01@25
assert [eq [date -separator " "] [format "%4i %2.2i %2.2i" $year $month $day]] {== $_ 1} -v -line #__LINE__		# 2017@01@25

newline

textcolor yellow
puts "Testing format YYYY/DD/DD"
textcolor
puts "Should be:-14964 -> [date 1859/01/10]"
assert [date 1859/01/10] {== $_ -14964} -v -line #__LINE__
puts "Should be: 21560 -> [date 1959/01/10]"
assert [date 1959/01/10] {== $_ 21560} -v -line #__LINE__
puts "Should be: 25569 -> [date 1970/01/01]"
assert [date 1970/01/01] {== $_ 25569} -v -line #__LINE__
puts "Should be: 36526 -> [date 2000/01/01]"
assert [date 2000/01/01] {== $_ 36526} -v -line #__LINE__
puts "Should be: 36892 -> [date 2001/01/01]"
assert [date 2001/01/01] {== $_ 36892} -v -line #__LINE__
puts "Should be: 42370 -> [date 2016/01/01]"
assert [date 2016/01/01] {== $_ 42370} -v -line #__LINE__
puts "Should be: 42430 -> [date 2016/03/01]"
assert [date 2016/03/01] {== $_ 42430} -v -line #__LINE__
puts "Should be: 42661 -> [date 2016/10/18]"
assert [date 2016/10/18] {== $_ 42661} -v -line #__LINE__
puts "Should be: 42735 -> [date 2016/12/31]"
assert [date 2016/12/31] {== $_ 42735} -v -line #__LINE__

newline
textcolor yellow
puts "date: Testing format YYYY/DDD/DD"
textcolor
puts "Should be:-14964 -> [date 1859/jan/10]"
assert [date 1859/jan/10] {== $_ -14964} -v -line #__LINE__
puts "Should be: 21560 -> [date 1959/jan/10]"
assert [date 1959/jan/10] {== $_ 21560} -v -line #__LINE__
puts "Should be: 25569 -> [date 1970/jan/01]"
assert [date 1970/jan/01] {== $_ 25569} -v -line #__LINE__
puts "Should be: 36526 -> [date 2000/jan/01]"
assert [date 2000/jan/01] {== $_ 36526} -v -line #__LINE__
puts "Should be: 36892 -> [date 2001/jan/01]"
assert [date 2001/jan/01] {== $_ 36892} -v -line #__LINE__
puts "Should be: 42370 -> [date 2016/jan/01]"		# lower case
assert [date 2016/jan/01] {== $_ 42370} -v -line #__LINE__
puts "Should be: 42430 -> [date 2016/MAR/01]"		# CAPS case
assert [date 2016/MAR/01] {== $_ 42430} -v -line #__LINE__
puts "Should be: 42661 -> [date 2016/oCT/18]"		# mIxEd cAsE
assert [date 2016/oCT/18] {== $_ 42661} -v -line #__LINE__
puts "Should be: 42735 -> [date 2016/Dec/31]"		# Proper Case
assert [date 2016/Dec/31] {== $_ 42735} -v -line #__LINE__

# Round-trip
newline
textcolor yellow
puts "date: Testing roundtrip ..."
textcolor
puts "Should be:1859/jan/10 -> [date [date 1859/jan/10]]"
assert [date [date 1859/jan/10] -notime] {eq $_ 1859/01/10} -v -line #__LINE__
puts "Should be:1959/jan/10 -> [date [date 1959/jan/10]]"
assert [date [date 1959/jan/10] -notime] {eq $_ 1959/01/10} -v -line #__LINE__
puts "Should be:1970/jan/01 -> [date [date 1970/jan/01]]"
assert [date [date 1970/jan/01] -notime] {eq $_ 1970/01/01} -v -line #__LINE__
puts "Should be:2000/jan/01 -> [date [date 2000/jan/01]]"
assert [date [date 2000/jan/01] -notime] {eq $_ 2000/01/01} -v -line #__LINE__


# Addition. Verified using VB5
newline
textcolor yellow
puts "date: Testing addition"
textcolor

# 0001-01-01 - 0028-05-19 10000 days
assert [date [+ 10000 [date 0001/jan/1]] -notime] {== $_ 0028/05/19} -v -line #__LINE__		# We get /20


# 0001-01-01  - 0027-05-20 9635 days
assert [date [+ 9635 [date 0001/jan/1]] -notime] {== $_ 0027/05/20} -v -line #__LINE__		# We get /21

puts "Should be:0027/may/20 -> [date [+ 10000 [date 0001/jan/1]]]"
# ? cdate(10000.0) -> 18/05/1927 
assert [date [+ 10000 [date 0001/jan/1]] -notime] {== $_ 0028/05/19} -v -line #__LINE__

assert [now] {> $_ 42121} -v -line #__LINE__
newline

puts "Should be:0127/may/20 -> [date [+ 10000 [date 0100/jan/1]]]"
assert [date [+ 10000 [date 0100/jan/1]] -notime] {== $_ 0127/05/20} -v -line #__LINE__
puts "Should be:1803/may/20 -> [date [+ 10000 [date 1776/jan/1]]]"
assert [date [+ 10000 [date 1776/jan/1]] -notime] {== $_ 1803/05/20} -v -line #__LINE__
puts "Should be:1859/feb/09 -> [date [+ 30 [date 1859/jan/10]]]"
assert [date [+ 1000000 [date 1776/jan/1]] -notime] {== $_ 4513/11/28} -v -line #__LINE__
puts "Should be:4513/nov/28 -> [date [+ 1000000 [date 1776/jan/1]]]"

assert [date -45288.0]  {== $_ "1776/01/01 00:00:00"}		# Validated in VB -45288=01/01/1776 
assert [date [date 1776/jan/1]] {== $_ "1776/01/01 00:00:00"} -v -line #__LINE__

# FIX 
assert [date [+ 30 [date 1859/jan/10]] -notime] {eq $_ 1859/02/09} -v -line #__LINE__
puts "Should be:1959/mar/11 -> [date [+ 60 [date 1959/jan/10]]]"
assert [date [+ 60 [date 1959/jan/10]] -notime] {eq $_ 1959/03/11} -v -line #__LINE__
puts "Should be:1970/apr/01 -> [date [+ 90 [date 1970/jan/01]]]"
assert [date [+ 90 [date 1970/jan/01]] -notime] {eq $_ 1970/04/01} -v -line #__LINE__
puts "Should be:2000/jun/29 -> [date [+ 180 [date 2000/jan/01]]]"
assert [date [+ 180 [date 2000/jan/01]] -notime] {eq $_ 2000/06/29} -v -line #__LINE__
puts "Should be:2000/dec/31 -> [date [+ [date 2000/jan/01] 365]]"
assert [date [+ [date 2000/jan/01] 365] -notime] {eq $_ 2000/12/31} -v -line #__LINE__

# Subtraction. Verified using VB5
newline
textcolor yellow
puts "date: Testing subtraction"
textcolor
puts "Should be:1858/Dec/11 -> [date [- [date 1859/jan/10] 30]]"
assert [date [- [date 1859/jan/10] 30] -notime] {eq $_ 1858/12/11} -v -line #__LINE__
puts "Should be:1958/Nov/11 -> [date [- [date 1959/jan/10] 60]]"
assert [date [- [date 1959/jan/10] 60] -notime] {eq $_ 1958/11/11} -v -line #__LINE__
puts "Should be:1969/Oct/03 -> [date [- [date 1970/jan/01] 90]]"
assert [date [- [date 1970/jan/01] 90] -notime] {eq $_ 1969/10/03} -v -line #__LINE__
puts "Should be:1999/Jul/05 -> [date [- [date 2000/jan/01] 180]]"
assert [date [- [date 2000/jan/01] 180] -notime] {eq $_ 1999/07/05} -v -line #__LINE__
puts "Should be:1999/jan/01 -> [date [- [date 2000/jan/01] 365]]"
assert [date [- [date 2000/jan/01] 365] -notime] {eq $_ 1999/01/01} -v -line #__LINE__

# Division. Verified using VB5
newline
textcolor yellow
puts "date: Testing subtraction"
textcolor
puts "Should be:1933/may/01	 -> [date [/ [date 2000/jan/01] 3.0] -longmonth -list]"
assert [date [/ [date 2000/jan/01] 3.0] -longmonth -list] {eq $_ "1933 May 01 08 00 00"} -v -line #__LINE__

# Multiplication. Verified using VB5
newline
textcolor yellow
puts "date: Testing subtraction"
textcolor
puts "Should be:2200/jan/04 -> [date [* [date 2000/jan/01] 3.0] -longmonth -list]"
assert [date [* [date 2000/jan/01] 3.0] -longmonth -list] {eq $_ "2200 Jan 04 00 00 00"} -v -line #__LINE__

newline
textcolor yellow
puts "date: Testing extreme ranges"
textcolor
puts "Should be: 0 -> [date 0]" 								# FAIL should be 00:00:00 (all 0.x should be time)
assert [date 0 -notime] {eq $_ 1899/12/30} -v -line #__LINE__
puts "Should be: 1 -> 31-12-1899 -> [date 1]"					# Correct per VB5 31/12/1899
assert [date 1 -notime -separator -] {eq $_ 1899-12-31} -v -line #__LINE__
puts "Should be: 2 -> 31-12-1899 -> [date 2]"					# Correct per VB5 01/01/1900
assert [date 2 -notime -separator -] {eq $_ 1900-01-01} -v -line #__LINE__

# Pending...
# FIX dates < 1900
set d [+ -1000000 693959]
puts "Should be:$d -> [date $d]"								# -306041 -> 1062/01/31
set d -693593
puts "Should be:$d -> [date $d]"								# 0 AD PASS (1/01/01)
unset d -nocomplain

puts "Should be:-1000 -> [date -1000]"
assert [date -1000] {eq $_ "1897/04/04 00:00:00"} -v -line #__LINE__
puts "Should be:-10000 -> [date -10000]"
# FIX leap year
assert [date -10000] {eq $_ "1872/08/13 00:00:00"} -v -line #__LINE__
puts "Should be:-100000 -> [date -100000]"						# 1626
assert [date -100000] {eq $_ "1626/03/16 00:00:00"} -v -line #__LINE__
#puts "Should be:-100000 -> [date -1000000]"					# 1626
newline

# Addition																						# Verified in
assert [date [+ 10000 [date 0001/jan/1]]] {eq $_ "0028/05/19 00:00:00"} -v -line #__LINE__		# Ticol
assert [date [+ 10000 [date 0100/jan/1]]] {eq $_ "0127/05/20 00:00:00"} -v -line #__LINE__		# VB
assert [date [+ 30000 [date 0100/jan/1]]] {eq $_ "0182/02/20 00:00:00"} -v -line #__LINE__		# VB
assert [date [+ 36500 [date 0100/jan/1]]] {eq $_ "0199/12/08 00:00:00"} -v -line #__LINE__		# VB
assert [date [+ 136500 [date 0100/jan/1]]] {eq $_ "0473/09/22 00:00:00"} -v -line #__LINE__	# VB
assert [date [+ 265000 [date 0100/jan/1]]] {eq $_ "0825/07/19 00:00:00"} -v -line #__LINE__	# VB
assert [date [+ 365000 [date 0100/jan/1]]] {eq $_ "1099/05/04 00:00:00"} -v -line #__LINE__	# VB
assert [date [+ 999999 [date 0100/jan/1]]] {eq $_ "2837/11/27 00:00:00"} -v -line #__LINE__	# VB
assert [date [+ 3000000 [date 0100/jan/1]]] {eq $_ "8313/09/22 00:00:00"} -v -line #__LINE__	# VB
newline

# Subtraction (inverse)																			# Verified in
assert [date [- [date 0028/05/19] 10000]] {eq $_ "0001/01/01 00:00:00"} -v -line #__LINE__		# Ticol
assert [date [- [date 0127/05/20] 10000]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__		# VB
assert [date [- [date 0182/02/20] 30000]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__		# VB
assert [date [- [date 0199/12/08] 36500]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__		# VB
assert [date [- [date 0473/09/22] 136500]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__	# VB
assert [date [- [date 0825/07/19] 265000]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__	# VB
assert [date [- [date 1099/05/04] 365000]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__	# VB
assert [date [- [date 2837/11/27] 999999]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__	# VB
assert [date [- [date 8313/Sep/22] 3000000]] {eq $_ "0100/01/01 00:00:00"} -v -line #__LINE__	# VB
newline


set a [date now]	# ?
unset a 

newline
textcolor yellow
puts "date: Testing round-trip"
textcolor
puts [date [date [date [date now]]]]

# Errors ################################################################################
newline
puts "date: Testing Errors"
# DONE FIX Time error
# VB    21560.0034722222 
# Ticol 21560.0034722222
puts  [date 1959/01/10 00:05:00]
puts "should be: 1959/01/10 00:05:00 -> [date 21560.0034722222]"
# Fault is in converting back from double
puts " Round-trip 1959/01/10 00:05:00 + 1 year: [date [+ [date 1959/01/10 00:05:00] 365]]"
# Negative time bugs (required abs()
puts [date [date "1823/mar/27 00:01:01"]]
  
# Increment year from 325 AD to 1776 AD, keeping the time
set d [date 0325/jan/01 09:30:00]						# VB -575255.395833333
set t [fpart $d]										# Store time
set i [integral $d]										# Store date
puts "Stored double is: $d"
puts "date==[date $d] integral as $i and time as $t"
set r [+ [* 1452 365] $i]								# 1452 x 365 + 325 == 1777 (we are dealing with -ve years)
textcolor yellow
puts "Result is: '${r}.$t' -> [date ${r}.$t]"
textcolor
assert [date ${r}.$t] {eq $_ "1776/01/14 09:30:00"} -v -line #__LINE__	# TODO assert math here for 15th
unset d i r t -nocomplain

# Test [time] round trip resolution (should be to 1s)
# => 25569.0000115741 => 1
# i is upvar'ed in checkex
assert [time [date 1970/01/01 00:00:01] -ctime] {eq $_ 1} -v -line #__LINE__
assert [date 1970/01/01 00:00:01] {eq $_ 25569.0000115741} -v -line #__LINE__


textcolor yellow
puts "date: Testing round-trip seconds"
textcolor
for {set i 0} {< $i 60} {++ i} {
	puts [date 1970/01/01 00:00:$i]
	puts [time [date 1970/01/01 00:00:$i]]
	assert [time [date 1970/01/01 00:00:$i] -ctime] {eq $_ $i} -v -line #__LINE__			# assert seconds
}

textcolor yellow
puts "date: Testing round-trip minutes"
textcolor
for {set i 0} {< $i 60} {++ i} {
	#puts [time [date 1970/01/01 00:$i:00] -ctime]
	assert [time [date 1970/01/01 00:$i:00] -ctime] {eq $_ [* $i 60]} -v -line #__LINE__	# assert mins as seconds
}

textcolor yellow
puts "date: Testing round-trip hours"
textcolor
for {set i 0} {< $i 24} {++ i} {
	#puts [time [date 1970/01/01 00:$i:00] -ctime]
	assert [time [date 1970/01/01 $i:00:00] -ctime] {eq $_ [* $i 3600]} -v -line #__LINE__	# assert mins as seconds
}
newline
unset yday day month year -nocomplain

# Test [is_dst]
textcolor yellow
puts "date: Testing \[is_dst\]"
textcolor

assert [is_dst -daylightname] {ne $_ ""} -v -line #__LINE__
assert [is_dst -standardname] {ne $_ ""} -v -line #__LINE__
puts "Line: "#__LINE__
if {[is_dst]} {
	assert [is_dst -offset] {> $_ 0} -v -line #__LINE__		# If in DST then should be > 0
} else {
	puts "We are not in DST"
}
puts "Line: "#__LINE__

# invalid conversion
set r 0
try {
	assert [date_to_ctime 1960 04 08] {ne $_ ""} -v -line #__LINE__
} catch {
	puts "Invalid date"
	set r 1
}
assert $r {== $_ 1} -v -line #__LINE__

unset i r -nocomplain
#################################################
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #