##############################################################################################
# File: 			test_dates.tcl
# Purpose:			Test cases fixed/known/checked dates against date routines
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 02/May/2018 First version
# 1.01 07/May/2018 Revised calculation method from
#					https://blog.artofmemory.com/how-to-calculate-the-day-of-the-week-4203.html
##############################################################################################
# Notes:
# Range 1000 AD to 4099 AD
# These tests found serious bugs in calendar date routine as follows
# int day_of_week(int day, int month, int year) in datew.h
# Rewritten using the simple heuristic method
##############################################################################################
/*
FIX Calendar Sept 12 1697 was a thurs (cal says Wed)
	http://5ko.free.fr/en/year.php?y=1697
	FIX http://mentalfloss.com/article/51370/why-our-calendars-skipped-11-days-1752
	ADD to static tests
	PASS 12/Sep/1897
	PASS 12/Sep/1797
	PASS 12/Sep/1753 wed
	# Changeover Sep/1752
	PASS 12/Sep/1750
	FAIL 12/Dec/1700 (thu, was sun)
	PASS 12/Sep/1700
	
FIX TRY https://blog.artofmemory.com/how-to-calculate-the-day-of-the-week-4203.html
FIX Calendar routine only needs to find the DOW for 1st of month then increment rot mod 7
FIX For dates during lost 11 days of Gregorian calendar, return next day
															3 days lost before 3rd Sep 1700

*/
##############################################################################################

option echo on

# Catch high limit

try {
	assert [day_of_week 01 01 4100] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
} catch {
	textcolor red
	puts "Successfully trapped date > 4000"
	textcolor
}

assert [day_of_week 01 01 4000] {== $_ 6} -v -line #__LINE__		# Sat (6)

assert [day_of_week 01 02 3456] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 01 01 3456] {== $_ 2} -v -line #__LINE__		# Tue (2)

assert [day_of_week 12 09 2212] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 12 09 2112] {== $_ 1} -v -line #__LINE__		# Mon (1)

assert [day_of_week 12 09 2020] {== $_ 6} -v -line #__LINE__		# Sat (6)

assert [day_of_week 06 05 2018] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 04 05 2018] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 14 04 2018] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 29 04 2018] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 22 04 2018] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 21 04 2018] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 28 04 2018] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 23 12 2017] {== $_ 6} -v -line #__LINE__		# Sat (6)	# Windows checked

assert [day_of_week 27 02 2016] {== $_ 6} -v -line #__LINE__		# Sat (6)	# Windows checked
assert [day_of_week 01 01 2000] {== $_ 6} -v -line #__LINE__		# Sat (6) 1 January 2000

# 20th C

assert [day_of_week 04 04 1972] {== $_ 2} -v -line #__LINE__		# Tue (2)	# (Flying Scotsman Day) verified Windows
assert [day_of_week 20 07 1969] {== $_ 0} -v -line #__LINE__		# 20 July 1969 moon landing (?)
assert [day_of_week 08 04 1968] {== $_ 1} -v -line #__LINE__		# Mon (1)	# (Reference Date)
assert [day_of_week 10 01 1959] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 13 01 1950] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 01 1939] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 12 1935] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 01 1933] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 12 1929] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 01 1928] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 01 1922] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 12 1918] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 12 1912] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 01 1911] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 12 1907] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 01 1905] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 13 12 1901] {== $_ 5} -v -line #__LINE__		# Fri (5)

assert [day_of_week 31 12 1900] {== $_ 1} -v -line #__LINE__		# Mon (1)
assert [day_of_week 12 09 1900] {== $_ 3} -v -line #__LINE__		# Wed (3)
assert [day_of_week 28 02 1900] {== $_ 3} -v -line #__LINE__		# Wed (3)
assert [day_of_week 01 01 1900] {== $_ 1} -v -line #__LINE__		# Mon (1)

# 19th C

assert [day_of_week 31 12 1897] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 12 09 1897] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 28 02 1897] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 01 01 1897] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 14 03 1897] {== $_ 0} -v -line #__LINE__		# Sun (0) 14 March 1897 Einstein's Birthday
assert [day_of_week 21 02 1804] {== $_ 2} -v -line #__LINE__		# Tue (2)	# First steam locomotive ran

assert [day_of_week 31 12 1800] {== $_ 3} -v -line #__LINE__		# Wed (3)
assert [day_of_week 12 09 1800] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 28 02 1800] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 01 01 1800] {== $_ 3} -v -line #__LINE__		# Wed (3)

# 18th C

assert [day_of_week 31 12 1797] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 12 09 1797] {== $_ 2} -v -line #__LINE__		# Tue (2)
assert [day_of_week 28 02 1797] {== $_ 2} -v -line #__LINE__		# Tue (2)
assert [day_of_week 01 01 1797] {== $_ 0} -v -line #__LINE__		# Sun (0)

assert [day_of_week 12 09 1753] {== $_ 3} -v -line #__LINE__		# Wed (3)

assert [day_of_week 31 12 1752] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 12 12 1752] {== $_ 2} -v -line #__LINE__		# Tue (2)
assert [day_of_week 30 09 1752] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 11 09 1752] {== $_ 1} -v -line #__LINE__		# Mon (1)
assert [day_of_week 10 09 1752] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 03 09 1752] {== $_ 0} -v -line #__LINE__		# Sun (0)

# FAIL
assert [day_of_week 31 01 1752] {== $_ 1} -v -line #__LINE__		# Mon (1)
assert [day_of_week 01 01 1752] {== $_ 6} -v -line #__LINE__		# Sat (6)

textcolor yellow
puts "Dates prior to date change of September 1752"
textcolor

#assert [day_of_week 02 09 1752] {== $_ 6} -v -line #__LINE__		# Sat (6)

assert [day_of_week 12 12 1750] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 12 09 1750] {== $_ 6} -v -line #__LINE__		# Sat (6)
assert [day_of_week 01 02 1700] {== $_ 1} -v -line #__LINE__		# Mon (1)	Checked
assert [day_of_week 31 01 1700] {== $_ 0} -v -line #__LINE__		# Sun (0)	Checked
assert [day_of_week 01 01 1700] {== $_ 5} -v -line #__LINE__		# Fri (5)
assert [day_of_week 12 09 1700] {== $_ 0} -v -line #__LINE__		# Sun (0)
assert [day_of_week 12 08 1700] {== $_ 4} -v -line #__LINE__		# Thu (4)
assert [day_of_week 12 07 1700] {== $_ 1} -v -line #__LINE__		# Mon (1)	PASS
assert [day_of_week 12 05 1700] {== $_ 3} -v -line #__LINE__		# Wed (3)	Online
assert [day_of_week 01 03 1700] {== $_ 1} -v -line #__LINE__		# Mon (1)	PASS
assert [day_of_week 28 02 1700] {== $_ 0} -v -line #__LINE__		# Sun (0)	PASS
assert [day_of_week 12 01 1700] {== $_ 2} -v -line #__LINE__		# Tue (2)	PASS
assert [day_of_week 30 01 1700] {== $_ 6} -v -line #__LINE__		# Sat (6)	PASS
assert [day_of_week 03 01 1700] {== $_ 0} -v -line #__LINE__		# Sun (0)	PASS
assert [day_of_week 02 01 1700] {== $_ 6} -v -line #__LINE__		# Sat (6)	PASS
assert [day_of_week 01 01 1700] {== $_ 5} -v -line #__LINE__		# Fri (5)	PASS

#######################################################################################################
# Adjustment is in September 1752
# Dates after this should be okay
# Before this may fail
# GAP
# Six and a half million Britons went to bed on September 2, 1752, and woke up on September 14
# The reason? The Calendar (New Style) Act of 1750, of course.
# Should we reject Sep 3 to Sep 13 1752
# Failure cases ...
#######################################################################################################

# 17th C #
textcolor magenta
puts "* Cases prior to 1700 AD *"
textcolor

assert [day_of_week 31 12 1699] {== $_ 4} -v -line #__LINE__		# Thu (4)	FAIL (Wed:3) http://www.webconversiononline.com/day-of-week.aspx?day=31&month=December&year=1699
assert [day_of_week 12 12 1699] {== $_ 6} -v -line #__LINE__		# Sat (6)	FAIL
assert [day_of_week 12 09 1699] {== $_ 6} -v -line #__LINE__		# Sat (6)	FAIL
assert [day_of_week 12 09 1698] {== $_ 5} -v -line #__LINE__		# Fri (5)	FAIL
assert [day_of_week 12 09 1697] {== $_ 4} -v -line #__LINE__		# Thu (4)	FAIL

# Leap
assert [day_of_week 31 12 1620] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked
assert [day_of_week 12 09 1620] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 28 02 1620] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked
assert [day_of_week 29 02 1620] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 01 01 1620] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked

# Not a leap year ?
assert [day_of_week 31 12 1600] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 12 09 1600] {== $_ 2} -v -line #__LINE__		# Tue (2) Checked
assert [day_of_week 28 02 1600] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 01 01 1600] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked

assert [day_of_week 12 09 1582] {== $_ 0} -v -line #__LINE__		# Sun (0)

# Leap
assert [day_of_week 31 12 1520] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked
assert [day_of_week 12 09 1520] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 28 02 1520] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 29 02 1520] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 01 01 1520] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked

assert [day_of_week 31 12 1501] {== $_ 2} -v -line #__LINE__		# Tue (2) Checked
assert [day_of_week 12 12 1501] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked
assert [day_of_week 12 09 1501] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked
assert [day_of_week 28 02 1501] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked
assert [day_of_week 01 01 1501] {== $_ 2} -v -line #__LINE__		# Tue (2) Checked

assert [day_of_week 12 09 1501] {== $_ 4} -v -line #__LINE__		# Thu (4)

# Leap
assert [day_of_week 31 12 1420] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 12 09 1420] {== $_ 2} -v -line #__LINE__		# Tue (2) Checked
assert [day_of_week 28 02 1420] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 29 02 1420] {== $_ 2} -v -line #__LINE__		# Tue (2) Checked
assert [day_of_week 01 01 1420] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked

assert [day_of_week 31 12 1403] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 12 09 1403] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 28 02 1403] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 01 01 1403] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked

assert [day_of_week 12 09 1401] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
# Friday, 13 October 1307, the date Philip IV of France arrested hundreds of the Knights Templar
# Fri (5) (Julian date != Gregorian)
assert [day_of_week 13 10 1307] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked http://www.webconversiononline.com/day-of-week.aspx?day=13&month=October&year=1307
assert [day_of_week 12 09 1301] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked

# Leap
assert [day_of_week 31 12 1220] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked
assert [day_of_week 12 09 1220] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 28 02 1220] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked
assert [day_of_week 29 02 1220] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 01 01 1220] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked

assert [day_of_week 12 09 1201] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked

assert [day_of_week 31 12 1199] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked
assert [day_of_week 12 09 1199] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 01 01 1199] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked

assert [day_of_week 01 01 1172] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked	# Ftumch

# Leap
assert [day_of_week 31 12 1120] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked
assert [day_of_week 12 09 1120] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 28 02 1120] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked
assert [day_of_week 29 02 1120] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 01 01 1120] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked

assert [day_of_week 12 09 1101] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked

assert [day_of_week 31 12 1100] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 12 09 1100] {== $_ 3} -v -line #__LINE__		# Thu (3) Checked
assert [day_of_week 01 01 1100] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked

assert [day_of_week 31 12 1066] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
# Friday 13th of October 1066
# Timeline Saturday 14th of October 1066 (Julian date)
#assert [day_of_week 14 10 1066] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked	# Battle of Hastings (Gregorian)
assert [day_of_week 30 09 1066] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 20 06 1066] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked
assert [day_of_week 01 03 1066] {== $_ 4} -v -line #__LINE__		# Thu (4) Checked
assert [day_of_week 28 02 1066] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked
assert [day_of_week 31 01 1066] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked
assert [day_of_week 01 01 1066] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 12 09 1001] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked

# Leap
assert [day_of_week 31 12 1020] {== $_ 0} -v -line #__LINE__		# Sun (0) Checked
assert [day_of_week 28 02 1020] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
assert [day_of_week 29 02 1020] {== $_ 2} -v -line #__LINE__		# Tue (2) Checked
assert [day_of_week 01 01 1020] {== $_ 6} -v -line #__LINE__		# Sat (6) Checked

assert [day_of_week 31 12 1000] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked
assert [day_of_week 28 02 1000] {== $_ 5} -v -line #__LINE__		# Fri (5) Checked
assert [day_of_week 01 01 1000] {== $_ 3} -v -line #__LINE__		# Wed (3) Checked

# 2 digit year calc method breaks down for dates < 1000 AD
try {
	assert [day_of_week 12 09 0901] {== $_ 1} -v -line #__LINE__		# Mon (1) Checked
} catch {
	textcolor red
	puts "Successfully trapped date < 1000"
	textcolor
}

##############################################################################################
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #