#############################################################################
# File:				test_deep_lindex.tcl
# Purpose:			Testing [lindex] with nested lists
# Author:			M Shaw
# Language:			Ticol Tcl v1.27+
# Versions:
# 1.00 18/Jan/2023	First version
#############################################################################
# Notes:
# The output of [lindex] on nested lists looks a bit counter-intuitive
# string [lindex list index ?index|end-N?... -blanks]
#
# There are only 2 items in this list; 0 and {1...}
# To index deeper you have to add more arguments
#############################################################################

set s {0 {1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}}}
puts [lindex $s]			# Activestate eq "{0 {1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}}}"
assert [lindex $s] {eq $_ "0 {1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}}"} -v -line #__LINE__

puts [lindex $s 1]			# Activestate eq "1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}"
assert [lindex $s 1] {eq $_ "1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}"} -v -line #__LINE__

puts [lindex $s 2 0]		# ActiveState eq ""
assert [lindex $s 2] {eq $_ ""} -v -line #__LINE__

puts [lindex $s 3 0]		# ActiveState eq ""
assert [lindex $s 3 0] {eq $_ ""} -v -line #__LINE__

puts [lindex $s 0 0]		# Activestate eq 0
assert [lindex $s 0 0] {eq $_ 0} -v -line #__LINE__

puts [lindex $s 1 0]		# Activestate eq 1
assert [lindex $s 1 0] {eq $_ 1} -v -line #__LINE__

puts [lindex $s 1 1 0]		# Activestate eq 2
assert [lindex $s 1 1 0] {eq $_ 2} -v -line #__LINE__

puts [lindex $s 1 1 1]		# Activestate eq "3 {4 {5 {6 {7 {8 {9 0}}}}}}"
assert [lindex $s 1 1 1] {eq $_ "3 {4 {5 {6 {7 {8 {9 0}}}}}}"} -v -line #__LINE__

puts [lindex $s 1 1 1 0]	# Activestate eq "3"
assert [lindex $s 1 1 1 0] {eq $_ 3} -v -line #__LINE__

puts [lindex $s 1 1 1 1]	# Activestate eq "4 {5 {6 {7 {8 {9 0}}}}}"
assert [lindex $s 1 1 1 1] {eq $_ "4 {5 {6 {7 {8 {9 0}}}}}"} -v -line #__LINE__

puts [lindex $s 1 1 1 1 0]	# Activestate eq 4
assert [lindex $s 1 1 1 1 0] {eq $_ 4} -v -line #__LINE__

puts [lindex $s 1 1 1 1 1]	# Activestate eq "5 {6 {7 {8 {9 0}}}}"
assert [lindex $s 1 1 1 1 1] {eq $_ "5 {6 {7 {8 {9 0}}}}"} -v -line #__LINE__


puts [lindex $s 1 1 1 1 1 1 1 1 0]	# Activestate eq 8
assert [lindex $s 1 1 1 1 1 1 1 1 0] {eq $_ 8} -v -line #__LINE__

puts [lindex $s 1 1 1 1 1 1 1 1 1]	# Activestate eq {9 0}
assert [lindex $s 1 1 1 1 1 1 1 1 1] {eq $_ "9 0"} -v -line #__LINE__


puts [lindex $s 1 1 1 1 1 1 1 1 1 0]	# Activestate eq 9
assert [lindex $s 1 1 1 1 1 1 1 1 1 0] {eq $_ 9} -v -line #__LINE__

puts [lindex $s 1 1 1 1 1 1 1 1 1 1]	# Activestate eq 0
assert [lindex $s 1 1 1 1 1 1 1 1 1 1] {eq $_ 0} -v -line #__LINE__

#############################################################################
unset s
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #