############################################################################################
# File: 			test_all_dict.tcl
# Purpose:			Test all current [dict] unit tests
# Language:			Ticol Tcl v1.26+
# See also:			dict_eg_employee.tcl
#
# Versions:
# 1.00 08/Oct/2019	First version
############################################################################################
# Notes:
############################################################################################

option expression on
set _start_time [timer]

set main::test_count 0
# puts "Debug: _scriptname'$_scriptname' info script '[info script]' ext -> '[file extension [info script]]'"
set main::filetype [file extension [info script]]
assert $main::filetype {ne $_ ""} -v -line #__LINE__	# Won't work in the DLL
#puts "filetype is $filetype"
set main::logfile "tests.log"
unlink $main::logfile

writefile $main::logfile "Start Test Run [date]@[time]\r\n"				# Log the date / time of run

if {! [defined run_test] } {
	vars -level 0
	#####################################################
	# This proc cannot depend on persistent global vars 
	# as these may be cleared by called scripts
	#####################################################
	proc run_test {filename line} {
		option expression off
		set name $filename
		
		if {[eq [file extension $filename] {}]} {
			if {eq $::main::filetype {}} {							# May be embedded DLL script
				append filename ".tcl"
			} else {
				append filename $::main::filetype					# ".tcl" ".tcx"
			}
		}

		textcolor yellow
		puts "\nTesting external file: $filename (Loop: [item ::loop::i "0"])"
		textcolor
		newline

		set ::tested([++ ::main::test_count]) $filename
		console set_title "Processing (Test: $::main::test_count, Loop: [item ::loop::i "0"]): '$filename'"
#array walk ::tested
		textcolor darkyellow
		puts "tests$::main::filetype -> run_test@Line: $line -> source $filename\n"
		textcolor
		#walk -level 0
		#vars -level 0
		if {[file exists $filename]} {
			writefile $::main::logfile "[time] start: $name\r\n" -append	# Filename minus ".tc?"

			try {										# Not using -quiet argument (error messages will be shown)
				uplevel @0 "source $filename"			# Call external file at level 0
				halt									# Stop the debuggger if running
			} catch {
				puts "\ntests.tcl -> run_test: catch failure in $filename"
				pause
				die
			}
			writefile $::main::logfile "[time] end:   $name\r\n" -append	# Filename minus ".tc?"
		} else {
			die "Can't find $filename in [getcwd]"
		}
		newline
# Ad-hoc var trace and halt
watch trace 106 -var -halt -line #__LINE__
/*
		writefile $::main::logfile "[time] end:   [info exists "1"]\r\n" -append
		foreach x [vars -list -level 0] {
		if {is_numeric $x} {
		# if {is_numeric [lindex [vars -list] 0]} {
		# if {[vars 5*]} {
			die "TRACE VAR FOUND"
		} 
		}# */
		writefile $::main::logfile "[time] end:   '[info vars ?????????? -level 0]'\r\n" -append
	}
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


run_test test_dict #__LINE__							# [dict]
run_test test_dict_append #__LINE__						# [dict append]
run_test test_dict_clear #__LINE__						# [dict clear]
run_test test_dict_create #__LINE__						# [dict create]
#test_dict_emptying.tcl #__LINE__						# NYI
run_test test_dict_exists #__LINE__						# [dict exists]
#test_dict_filter.tcl #__LINE__							#  NYI
run_test test_dict_for #__LINE__						# [dict for]
run_test test_dict_get_set_nested #__LINE__				# [dict set] and [dict get] with nested dictionaries
run_test test_dict_item #__LINE__						# [dict item]
run_test test_dict_keys #__LINE__						# [dict keys]
run_test test_dict_lappend #__LINE__					# [dict lappend]
run_test test_dict_map #__LINE__						# [dict map]
run_test test_dict_merge #__LINE__						# [dict merge]
run_test test_dict_proc #__LINE__						# [dict] with [proc]
run_test test_dict_remove #__LINE__						# [dict remove]
run_test test_dict_set #__LINE__						# [dict set]
run_test test_dict_size #__LINE__						# [dict size]
run_test test_dict_unset #__LINE__						# [dict unset]
run_test test_dict_upvar #__LINE__						# [dict] with [upvar] in [proc]
run_test test_dict_with #__LINE__						# [dict with]



# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
set line #__LINE__
puts "Line "#__LINE__
textcolor darkyellow
puts "[string repeat = 75]"
vars

if {! [array exists tested]} {
	die "No files were tested"
}

textcolor green
puts "Tests of $main::test_count files completed successfully\n"
puts "Error code [item errorCode {errorCode not found!}] was recorded"
textcolor
newline

puts "Pre-cleanup vars are..."
vars
newline

textcolor darkyellow
/*
array foreach tested val ss {
	puts "$ss:\t$val"
}
*/

option expression off
#puts "\$tested size==[array size tested]"
array sort tested -array sorted		# Sort to integer indexed array...

set s [array size sorted]
#puts "Sorted array \$size is: $s"
#array walk sorted
for {set i 0} {[< $i $s]} {++ i} {
	puts "[+ $i 1]:\t$sorted($i)"
	#puts $i
}
unset i s -nocomplain
newline
puts "Tested [array size tested] external file(s)"
textcolor
newline

writefile $main::logfile "Tested [array size tested] external file(s)\r\n" -append	# Write log summary

set line #__LINE__
option echo on
puts "All tests took: " -nonewline
#printf "DEBUG %.3f \[calc \"(\[timer\]-$start_time)/1000.0\"]\r\n"
printf "%.3f" [calc "([timer]-$_start_time)/1000.0"]
#puts [format "%.4f" [calc "round( (([timer]-$start_time)/1000.0) ,4)" ] ] -nonewline
puts " sec\n"
halt		# Single-step the debugger

# KEEP tcl_cgi autoexec_ran
array unset sorted -nocomplain
array unset tested -nocomplain
unset main::test_count -nocomplain
unset line _start_time start_time echostate err expression_state main::filetype -nocomplain
unset {*}[split [range a-z]] flag handle num ptr -nocomplain
unset test* -glob -const -nocomplain
unset *_MIN *_MAX *_BITS -glob -const -nocomplain	# Unset is case-sensitive
unset main::logfile _start_time -nocomplain

textcolor magenta
puts "Storage status '[store list]' @"#__LINE__
textcolor
newline

################################################################
puts "varlist->'[vars -auto -list]'"
vars
vars -const

set line #__LINE__
if {== [item loop::i 0] 0} {
	beep;beep
}
option echo off
newline
textcolor green
puts "* ALL TESTS PASS *"
#puts "(test_sockping is disabled)"
textcolor
newline
#pause

#return foo -code ok
#return -code ok

option echo on

# EOF #
