#################################################################################
# File: 			test_dict_for.tcl
# Purpose:			Tests for [dict for]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 07/Oct/2019	First version
#################################################################################
# Notes:
# [dict for] returns a NULL string
#################################################################################

option expression off

#################################################################################
# DICT FOR - ITER NO CHANGE (return {})
puts "Line: "#__LINE__
set newdict [dict for {k v} {a b c d} {}]
puts "newdict is: '$newdict' @"#__LINE__		# A/S returns ""
assert $newdict {eq $_ {}} -v -line #__LINE__

#################################################################################
# DICT FOR WITH ASSERT
puts "Line: "#__LINE__
set i 0
dict for {k v} {a b c d} {
	puts "$k $v"
	if {== $i 0} {
		assert "$k $v" {eq $_ {a b}} -v -line #__LINE__
	} elseif {== $i 1} {
		assert "$k $v" {eq $_ {c d}} -v -line #__LINE__
	}
	++ i
}
# a b
# c d

unset k v i
newline

#################################################################################
# DICT FOR WITH ASSIGN KEY
# Check reassign key var
puts "Line: "#__LINE__
set mydict [dict create a b c d]
set i 1
set newdict [dict for {k v} $mydict {
	set k $i
	puts "k=$k, v=$v"
	assert $k {eq $_ $i} -v -line #__LINE__
	puts "\$newdict=$newdict"
	incr i
}]

puts "\$newdict=$newdict @"#__LINE__
assert $newdict {eq $_ {}} -v -line #__LINE__


/* A/S 8.6
k=1, v=b
k=2, v=d
% puts $mydict
a b c d
*/
newline
puts "mydict is: '$mydict' @"#__LINE__
puts "newdict is: '$newdict' @"#__LINE__
unset newdict
unset mydict


#################################################################################
# DICT FOR WITH ASSIGN VAL
#
set mydict [dict create a b c d]
set i 1
set newdict [dict for {k v} $mydict {
	set v "val$i"
	puts "k=$k, v=$v"
	incr i
}]

assert $newdict {eq $_ {}} -v -line #__LINE__
puts "mydict is: '$mydict' @"#__LINE__
puts "newdict is: '$newdict' @"#__LINE__		# A/S returns ""
dict walk mydict

unset i k v 
unset newdict
unset mydict

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #