############################################################################################
# File: 			test_dict_get_set_nested.tcl
# Purpose:			Test [dict set] and [dict get] speficially with nested dictionaries
# Language:			Ticol Tcl v1.26+
# See also:			test_dict_map.tcl
#
# Versions:
# 1.00 07/Oct/2019	First version
# 1.01 28/Oct/2019	Changes after MPP \ line continuation (removed space)
############################################################################################
# Notes:
# A/S standard Tcl does not search 'deep' into any dictionary for a key, i.e. it does not
# work like a 'find' function
# Nested lookups must be performed using explicit keys at each level when using [dict get]
############################################################################################
/* A/S example - will not search deep for keys using dict get
% dict set a4 60028 4487 Sea Eagle
60014 {2509 {Silver Link}} 60015 {2510 Quicksilver} 60016 {2511 {Silver King}} 6
0017 {2512 {Silver Fox}} 60023 {4482 {Golden Eagle}} 60024 {4483 Kingfisher} 600
25 {4484 Falcon} 60026 {4485 Kestrel} 60027 {4486 Merlin} 60028 {4487 {Sea Eagle
}}
%
% puts $a4
60014 {2509 {Silver Link}} 60015 {2510 Quicksilver} 60016 {2511 {Silver King}} 6
0017 {2512 {Silver Fox}} 60023 {4482 {Golden Eagle}} 60024 {4483 Kingfisher} 600
25 {4484 Falcon} 60026 {4485 Kestrel} 60027 {4486 Merlin} 60028 {4487 {Sea Eagle
}}
% puts [dict get 4486]
missing value to go with key
% puts [dict get $mydict 4486]
key "4486" not known in dictionary
% puts [dict get $mydict 60027]
key "60027" not known in dictionary

% puts "60023 => '[dict get $a4 60023]'"
60023 => '4482 {Golden Eagle}'
% puts "60023 => '[dict get $a4 60023 4482]'"
60023 => 'Golden Eagle'

############################################################################################ */
option expression off

textcolor yellow
puts "Testing \[dict\] with nested dictionaries ... @"#__LINE__
textcolor
newline

# LNER A4 Locomotives
# BR number, LNER number, Name (x3 fields)
dict set a4 60014 2509 Silver Link
dict set a4 60015 2510 Quicksilver
dict set a4 60016 2511 Silver King
dict set a4 60017 2512 Silver Fox
dict set a4 60023 4482 Golden Eagle
dict set a4 60024 4483 Kingfisher
dict set a4 60025 4484 Falcon
dict set a4 60026 4485 Kestrel
dict set a4 60027 4486 Merlin
dict set a4 60028 4487 Sea Eagle
dict walk a4 -bare

############################################################################################
textcolor yellow
puts "Line: "#__LINE__
puts "Testing \[dict\] with nested \[dict\] commands ... @"#__LINE__
textcolor
newline

# Note that dicts are NOT ordred. This test may fail
assert [dict get [dict get $a4]] {eq $_ {60014 {2509 {Silver Link}} 60028 {4487 {Sea Eagle}} 60015\
 {2510 Quicksilver} 60026 {4485 Kestrel} 60016 {2511 {Silver King}}\
 60024 {4483 Kingfisher} 60027 {4486 Merlin} 60023 {4482 {Golden Eagle}}\
 60017 {2512 {Silver Fox}} 60025 {4484 Falcon}}} -v -line #__LINE__						# A/S 8.6

# FIX case returning whole dictionary
puts "Line: "#__LINE__
# FIX flat case
puts "60023 => '[dict get $a4 60023]' @"#__LINE__
puts "Line: "#__LINE__
puts "60023 => '[dict get [dict get $a4 60023]]' @"#__LINE__
puts "Line: "#__LINE__
assert [dict get [dict get $a4 60023]] {eq $_ {4482 {Golden Eagle}}}  -v -line #__LINE__	# A/S 8.6

############################################################################################
# Indirect nested search (multiple [dict] commands)
puts "60023 => '[dict get [dict get $a4 60023] 4482]' @"#__LINE__
assert [dict get [dict get $a4 60023] 4482] {eq $_ {Golden Eagle}} -v -line #__LINE__		# A/S 8.6
# */

newline
textcolor yellow
puts "Line: "#__LINE__
puts "Testing \[dict\] with multiple keys searching nested dictionary ... @"#__LINE__
textcolor

############################################################################################
# Direct multiple key search
puts "Line: "#__LINE__
puts "60023 => '[dict get $a4 60023 4482]'"
assert [dict get $a4 60023 4482] {eq $_ {Golden Eagle}} -v -line #__LINE__		# A/S 8.6

############################################################################################
# Test invalids
newline
textcolor yellow
puts "Line: "#__LINE__
puts "Testing \[dict\] with multiple keys searching nested dictionary ... @"#__LINE__
textcolor

puts "Line: "#__LINE__
set i 0
try {
	puts "Line: "#__LINE__
	assert [dict get $a4 60023 0001] {eq $_ {Golden Eagle}} -v -line #__LINE__		# A/S 8.6
} catch {
	set i 1
}
assert $i {== $_ 1}  -v -line #__LINE__
catch {
assert [dict get $a4 60023 0] {eq $_ {Golden Eagle}} -v -line #__LINE__		# A/S 8.6
}
catch {
assert [dict get $a4 70000 4482] {eq $_ {Golden Eagle}} -v -line #__LINE__		# A/S 8.6
}

unset a4 i -nocomplain

############################################################################################
# Literal
assert [dict get {a {b c}} a] {eq $_ {b c}} -v -line #__LINE__			# A/S 8.6 return 'b c'
assert [dict get {a b c d} a] {eq $_ b} -v -line #__LINE__		# A/S 8.6 return 'a b c d'
set i 0
try {
	# Invalid as nested dict is not present so (nested) 2nd key is not valid
	assert [dict get {a b c d} a c] {eq $_ {b d}} -v -line #__LINE__		# A/S 8.6 return 'a b c d'
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
unset i

# FIX fail
puts [dict get {a {b {c d}}} a b c]
assert [dict get {a {b {c d}}} a b c] {eq $_ d} -v -line #__LINE__		# A/S 8.6 return 'd' successfully

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #