#################################################################################
# File: 			test_dict_item.tcl
# Purpose:			Tests for [dict item]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 07/Oct/2019	First version
#################################################################################
# Notes:
# [dict item] is non-standard but matches [item] either standalone or across a
# range of other data types and operations in Ticol Tcl
#################################################################################

option expression off
option echo on

unset mydict -nocomplain
assert [is_set mydict] {== $_ $false} -v -line #__LINE__

set mydict [dict create 4470 {Flying Fox} 4472 {Flying Scotsman} 2553 Manna 2555 Centenary 60100 Spearmint 2550 {Blink Bonny}]
dict walk mydict
assert [dict size mydict] {== $_ 6} -v -line #__LINE__
assert [dict item $mydict 2550] {eq $_ {Blink Bonny}} -v -line #__LINE__
assert [dict item $mydict 2553] {eq $_ {Manna}} -v -line #__LINE__
assert [dict item $mydict 2555] {eq $_ {Centenary}} -v -line #__LINE__
assert [dict item $mydict 4470] {eq $_ {Flying Fox}} -v -line #__LINE__
assert [dict item $mydict 4472] {eq $_ {Flying Scotsman}} -v -line #__LINE__
assert [dict item $mydict 60100] {eq $_ {Spearmint}} -v -line #__LINE__
assert [dict item $mydict 6?1?? -glob] {eq $_ {Spearmint}} -v -line #__LINE__
assert [dict item $mydict 700?? -glob] {eq $_ {}} -v -line #__LINE__
assert [dict item $mydict 4471] {eq $_ {}} -v -line #__LINE__
assert [dict item $mydict 4473 {Not Found}] {eq $_ {Not Found}} -v -line #__LINE__
unset mydict
assert [is_set mydict] {== $_ $false} -v -line #__LINE__

# Create a new dictionary
set mydict [dict create The quick brown fox]
assert [dict size mydict] {== $_ 2} -v -line #__LINE__
# Test case without glob
assert [dict item $mydict the] {eq $_ {}} -v -line #__LINE__
assert [dict item $mydict the -nocase] {eq $_ {quick}} -v -line #__LINE__
assert [dict item $mydict BROWN] {eq $_ {}} -v -line #__LINE__
assert [dict item $mydict BROWN -nocase] {eq $_ {fox}} -v -line #__LINE__

# Test case with glob
assert [dict item $mydict t?e -glob] {eq $_ {}} -v -line #__LINE__
assert [dict item $mydict t?e -nocase -glob] {eq $_ {quick}} -v -line #__LINE__
assert [dict item $mydict BR*N] {eq $_ {}} -v -line #__LINE__
assert [dict item $mydict BR*N -nocase -glob] {eq $_ {fox}} -v -line #__LINE__

# Man example data...
puts "'[dict item $mydict the]'"
puts "'[dict item $mydict The]'"
puts "'[dict item $mydict the -nocase]'"
puts "'[dict item $mydict t?e]'"
puts "'[dict item $mydict T?e -glob]'"
puts "'[dict item $mydict t?e -glob -nocase]'"

unset mydict

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #