#################################################################################
# File: 			test_dict_lappend.tcl
# Purpose:			Tests for [dict lappend]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 05/Oct/2019	First version
#################################################################################
# Notes:
# DESIGN DEF [dict] will store rvalues unbraced
# Reason: the purpose of dict is to rapidly reference rvalues and this is the
# most frequent use. Return of the whole or part dictionaries is less frequent
# if rvalues are braced then removing the braces will be less efficient
# Thus, anything which returns the whole dict must brace the dict rvalues
#################################################################################

textcolor darkyellow
puts "Testing \[dict lappend\] ..."
textcolor
newline

unset mydict -nocomplain

puts [dict lappend mydict bar moo par]			# Create a dictionary var
# 1: 0x206ecd0 key:'bar' => 'moo par'
# A/S % puts $mydict
# bar {moo par}
puts "type [type mydict] contents: '$mydict' @"#__LINE__
assert $mydict {eq $_ {bar {moo par}}} -v -line #__LINE__		# A/S 8.6 'bar {moo par}'
dump mydict
dict walk mydict
unset mydict

############################################################################################
# [set <dictvar] etc.
# CAUTION. [dict lappend rvalue with spaces should NOT be braced or double-bracing will occur
# All values after arg 4 will be concatenated to use as the Rvalue and THEN braced
# Thus: 
# dict lappend mydict 4472 "Flying Scotsman"
# is wrong, and so is...
# dict lappend mydict 4472 {Flying Scotsman}
# whereas...
# dict lappend mydict 4472 Flying Scotsman
# is correct

/* ActiveState
	% unset mydict
	% set mydict [dict create 4470 {Great Northern} 4471 "Sir Frederic Banbury"]
	4470 {Great Northern} 4471 {Sir Frederic Banbury}
	% dict lappend mydict 4472 Flying Scotsman
	4470 {Great Northern} 4471 {Sir Frederic Banbury} 4472 {{Flying Scotsman}}
																WHY??
*/
puts "Line: "#__LINE__
unset mydict -nocomplain
set mydict [dict create 4470 {Great Northern} 4471 "Sir Frederic Banbury"]
puts "mydict -> '$mydict' @"#__LINE__
assert $mydict {eq $_ "4470 {Great Northern} 4471 {Sir Frederic Banbury}"} -v -line #__LINE__

puts "Line: "#__LINE__
dict lappend mydict 4472 "Flying Scotsman"
# NOTE: Double-bracing here is CORRECT!!
assert $mydict {eq $_ {4470 {Great Northern} 4472 {{Flying Scotsman}} 4471 {Sir Frederic Banbury}}} -v -line #__LINE__
dict walk mydict

# FIX Requires ordred <dict>
puts "Line: "#__LINE__
#assert $mydict {eq $_ "4470 {Great Northern} 4471 {Sir Frederic Banbury}"} -v -line #__LINE__
dict lappend mydict 4473 "Solario"
dict lappend mydict 4474 Victor Wild
dict lappend mydict 4475 Flying Fox
dict lappend mydict 4476 Royal Lancer
textcolor darkyellow
puts "mydict -> '$mydict' @"#__LINE__
textcolor
# FIX Order may change as we are unordered ATM
assert $mydict {eq $_ {4475 {Flying Fox} 4476 {Royal Lancer} 4470 {Great Northern} 4472 {{Flying Scotsman}} 4474 {Victor Wild} 4473 Solario 4471 {Sir Frederic Banbury}}} -v -line #__LINE__
dict walk mydict
dict info mydict
puts "Line: "#__LINE__
assert [dict size $mydict] {== $_ 7} -v -line #__LINE__

puts "Line: "#__LINE__
textcolor darkyellow
puts "mydict -> '$mydict'"
textcolor
newline

unset mydict -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #