#################################################################################
# File: 			test_dict_merge.tcl
# Purpose:			Tests for [dict merge]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 05/Oct/2019	First version
#################################################################################
# Notes:
# DESIGN DEF [dict] will store rvalues unbraced
# Reason: the purpose of dict is to rapidly reference rvalues and this is the
# most frequent use. Return of the whole or part dictionaries is less frequent
# if rvalues are braced then removing the braces will be less efficient
# Thus, anything which returns the whole dict must brace the dict rvalues
#################################################################################

textcolor darkyellow
puts "Testing \[dict merge\] ... @"#__LINE__
textcolor
newline

#################################################################################
# Merge L/R empty case. 
# Handles case where an empty dict is returned from a command
# We create an empty dictionary with no key/value - just {}
puts "Line: "#__LINE__
dict set dict1 1 {}
dict set dict2 2 {}
puts "merge='[dict merge $dict1 $dict2]' @"#__LINE__

puts "Line: "#__LINE__
set dict3 [dict merge $dict1 $dict2]
puts "\$dict3 is '$dict3' @"#__LINE__
assert [type dict3] {eq $_ dictionary} -v -line #__LINE__
assert $dict3 {eq $_ {}} -v -line #__LINE__
dict walk dict3
puts "Line: "#__LINE__
unset dict1 dict2 dict3
vars


/*
# dict create ?key value ...? 
# dict set dictionaryVariable key ?key ...? value 
dict set dict1 1 {foo bar}
dict set dict1 2 bar
dict set dict1 3 moo		# This key+val should disappear

dict set dict2 3 {Hi there}
dict set dict2 4 how
dict set dict2 5 {are you?}

set newdic [dict merge $dict1 $dict2]

newline
textcolor darkyellow
puts "\$newdic is '$newdic' @"#__LINE__
textcolor

# 3 Hi 5 man 1 {foo bar} 2 bar 4 there
assert [dict size newdic] {== $_ 5} -v -line #__LINE__		# A/S 8.6
assert [dict get $newdic 1] {eq $_ {foo bar}} -v -line #__LINE__
assert [dict get $newdic 2] {eq $_ bar} -v -line #__LINE__
assert [dict get $newdic 3] {eq $_ {Hi there}} -v -line #__LINE__
assert [dict get $newdic 4] {eq $_ how} -v -line #__LINE__
assert [dict get $newdic 5] {eq $_ {are you?}} -v -line #__LINE__
# */

unset dict1 dict2 newdic -nocomplain

#################################################################################
puts "Line: "#__LINE__

set mydata [dict create "Cardiff Central" 06:28 Newport 06:42 "Severn Tunnel Junction" 06:53 Pilning 07:03 Patchway 07:06]
dict walk mydata
textcolor green
puts "mydata -> '$mydata' @"#__LINE__
textcolor

set moredata [dict create "Bristol Temple Meads" dunno Bath\ Spa 07:35 Bradford-on-Avon 07:47 Trowbridge 07:53 Westbury 08:00]
textcolor green
puts "moredata='$moredata'"
textcolor
puts "type='[type moredata]' @"#__LINE__

textcolor red
puts "\[dict merge\] ... @"#__LINE__
textcolor

# Merge two dictionaries
textcolor magenta
puts "mydata '$mydata'"
textcolor blue
puts "moredata '$moredata'"
textcolor
set all [dict merge $mydata $moredata]
textcolor darkyellow
puts "\$all is '$all' @"#__LINE__
textcolor
unset all moredata mydata -nocomplain
newline

#################################################################################
puts "Line: "#__LINE__
unset dict1 dict2 dict3 -nocomplain
newline
puts "Merging three dictionaries ..."
set dict1 [dict create 2550 *Unknown* 4498 {Sir Nigel Gresley}]
set dict2 [dict create 4470 {Flying Fox} 4472 {Flying Scotsman}]
set dict3 [dict create 2553 Manna 2555 Centenary 2550 {Blink Bonny}]
set dict [dict merge $dict1 $dict2 $dict3]
textcolor white
dict for {key val} $dict {
	puts "$key  $val"
}
textcolor
unset dict1 dict2 dict3 dict key val

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #