#################################################################################
# File: 			test_dict_proc.tcl
# Purpose:			Tests [dict] with [proc]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
# See also:			test_dict_upvar.tcl, leak_dict.tcl
#
# Versions:
# 1.00 06/Oct/2019	First version
#################################################################################
# Notes:
# Tests which focus on proc return and proc memory allocation go in here
# Tests passing a dictionary to a proc by name (by reference), setting an entry
# locally and then successfully passing it back
#################################################################################

option expression off
option echo on

textcolor darkyellow
puts "Testing \[dict\] with \[proc\] ... @"#__LINE__
textcolor
newline

###########################################################################################
# DICT with PROC
puts "Line: "#__LINE__
proc foo {_d} {
	##############################
	# [in]	A dictionary by name
	##############################
	puts "Enter foo with _d == '$_d'"
	#assert [type $_d] {eq $_ dictionary} -v -line #__LINE__		# This won't work because the arg has not yet been upvar'ed
	upvar $_d d						# PASS
	assert [type d] {eq $_ dictionary} -v -line #__LINE__
	assert [dict get $d 4472] {eq $_ {Flying Scotsman}} -v -line #__LINE__
	assert [dict size d] {== $_ 2} -v -line #__LINE__
	dict lappend d 4475 Flying Fox
}

set mydict [dict create 4472 {Flying Scotsman} 4770 {Great Northern}]
puts "Line: "#__LINE__
assert [dict size mydict] {== $_ 2} -v -line #__LINE__
newline

puts "Calling proc foo to set dict mydict ..."
foo mydict
assert [dict size mydict] {== $_ 3} -v -line #__LINE__
puts "Line: "#__LINE__
assert [dict get $mydict 4475] {eq $_ {Flying Fox}} -v -line #__LINE__

puts "mydict='$mydict'"
unset mydict
undef foo

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #