#################################################################################
# File: 			test_dict_upvar.tcl
# Purpose:			Tests for [upvar] with dictionaries
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
# See also:			test_dict_proc.tcl, leak_dict.tcl
#
# Versions:
# 1.00 07/Oct/2019	First version
#################################################################################
# Notes:
# Tests which focus on [upvar] go in here
# PASS (no leaks)
#################################################################################

option expression off
option echo on

textcolor yellow
puts "Testing \[dict\] with \[upvar\] in \[proc\]... @"#__LINE__
textcolor
newline

#################################################################################
#
puts "Line: "#__LINE__
proc foo {d} {
	#assert [type $_d] {eq $_ dictionary} -v -line #__LINE__		# This won't work because the arg has not yet been upvar'ed
	upvar $d						# PASS
	vars
	puts "New upvar is called '$d'"
	assert [type $d] {eq $_ dictionary} -v -line #__LINE__
	dump $d							# Show that it is local
	puts "type $d='[type $d]'"		# PASS
	dict lappend $d new data		# PASS (not sure how as there's no code implemented yet)
	vars
}

puts "Line: "#__LINE__
dict set mydict foo bar
assert [dict size mydict] {== $_ 1} -v -line #__LINE__
newline

puts "Calling proc foo..."
foo mydict
dict walk mydict
assert [dict size mydict] {== $_ 2} -v -line #__LINE__
assert [dict item $mydict foo ""] {== $_ bar} -v -line #__LINE__
assert [dict item $mydict new ""] {== $_ data} -v -line #__LINE__

undef foo
unset mydict

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #