###########################################################################
# File:			double_deref_adt.tcl
# Purpose:		Tests for $$ dereference on other ADTs
# Author: 		M Shaw
# Language:		Ticol Tcl v1.26+
#
# Versions:
# 1.00 07/Aug/2019 First version
###########################################################################
# Notes:
# 4 cases:
# DONE carray (Plugin)
# DONE struct
# DONE stack
# DONE linkedlist (Plugin)
# DONE variant (Plugin)
# FIX Rework syntax on [carray] to include indexing
# FIXED * POSSIBLE LEAK *
###########################################################################

/*
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# CARRAY (PASS)
# No leaks detected
# carrays are addressed by handle. The handle var is a standard Tcl var
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__


lib ticol_carray -unload -verbose -nocomplain
lib ticol_carray -verbose
set h [carray create]
carray add $h Hi 1
carray add $h There 2
set q h
set r q
puts [carray item $$q 1]
assert [carray item $$q 1] {eq $_ "There"} -v -line #__LINE__
assert [carray item $$$r 1] {eq $_ "There"} -v -line #__LINE__
carray unset $h
lib ticol_carray -unload
unset h q r s -nocomplain



# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# STRUCT (PASS)
# No leaks detected
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
struct create s {a 10 b 20}
struct set s.a Hi
struct set s.b There
puts "s.a '$s.a'"
puts "s.b '$s.b'"
set q s
set r q
puts "Line: "#__LINE__
puts "s.a '${${$r}.a}' @"#__LINE__
puts "s.a '${${$r}.b}' @"#__LINE__
puts "Line: "#__LINE__
assert ${${$r}.a} {eq $_ "Hi"} -v -line #__LINE__
assert ${${$r}.b} {eq $_ "There"} -v -line #__LINE__
puts "Line: "#__LINE__
puts ${s.a}
struct unset s -nocomplain
unset q r s -nocomplain



# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# STACK (PASS)
# 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
stack create st 100
stack push st Hello
stack push st world
set q st
set r q
puts "Line: "#__LINE__
puts [stack item $$r 0]
assert [stack item $$r 0] {eq $_ "Hello"} -v -line #__LINE__
puts "Line: "#__LINE__
stack unset st -nocomplain
unset q r s -nocomplain
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# LINKED LIST (PASS)
# No leaks detected
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
lib ticol_list
set h [linkedlist create]
linkedlist add $h Hello
linkedlist add $h world
set q h
set r q
puts [linkedlist item $$$r 0]
assert [linkedlist item $$$r 0] {eq $_ "Hello"} -v -line #__LINE__
lib ticol_list -unload
unset h q r s -nocomplain
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# VARRAY (Variant Array) (PASS)
# LEAKING MEMORY 2404-> @ 1000
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
lib ticol_variant
set h [varray create 100]
varray set $h 0 Hello
varray set $h 1 world
puts [varray list $h]
set q h
set r q
puts [varray get $$r 0]
puts [varray get $$r 1]
assert [varray get $$r 0] {eq $_ "Hello"} -v -line #__LINE__
# FIX check that unload will destroy any variants
varray unset $h
lib ticol_variant -unload

# */

unset ? -glob -nocomplain

newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# END #