############################################################################
# File:				test_elseif.tcl
# Author:			M Shaw
# Purpose:			Test file for the if/else/elseif
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 06/Jan/2017	First version
# 1.01 21/Jul/2017	Replace [checkex] with [assert]
############################################################################
# Notes:
############################################################################
option echo on
option expression off

textcolor yellow
puts "Testing \[test_elseif\] ..."
textcolor

/* if(argc < 3 || argc == 4 || argc == 7)
	// 1, 3, 5 bad
	// if										// 1 BAD
	// if {x}									// 2 BAD
	// if {x} {code}							// 3
	// if {x} else								// 3 BAD
	// if {x} {code} else						// 4 BAD
	// if {x} {code} elseif						// 4 BAD
	// if {x} {code} {code}						// 4 OK but potentially slow and adds complexity to code
	// if {x} {code} else {code}				// 5
	// if {x} {code} else {code} else			// 6 BAD
	// if {x} {code} elseif {x} {code}			// 6
	// if {x} {code} elseif {x} {code}			// 6
	// if {x} {code} else {x} elseif			// 6 BAD
	
	// if {x} {code} else {x} else				// 6 BAD
	// if {x} {code} elseif {x} elseif			// 6 BAD
	// if {x} {code} elseif {x} else			// 6 BAD
	// if {x} {code} elseif {x} {code}			// 6
	// if {x} {code} elseif {x} else {code}		// 7 BAD
*/

set flag $false
if {[catch if]} {set flag $true}			# if
assert $flag {== $_ 1} -v -line #__LINE_

set flag $false
textcolor red
puts "Ignore errors ..."
textcolor

if {[catch {if {1}}]} {set flag $true}		# if {x}
assert $flag {== $_ 1} -v -line #__LINE_

set flag $false
if {1} {set flag $true}						# if {x} {code}
assert $flag {== $_ 1} -v -line #__LINE_

/*	// FIX will spawn else
set flag $false
if {[catch {if {1} else}]} {set flag $true}	# if {x} else
assert $flag {== $_ 1} -v -line #__LINE_
*/

/* # FIX
// Inverted brackets
set flag $false
if {[catch {if {1} else]}} {set flag $true}	# if {x} elseif
#                      ^^
assert $flag {== $_ 1} -v -line #__LINE_
*/

/*	# FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX # FIX 
set flag $false
if {1} {set flag $true} {set flag $false}						# if {x} {code} else {code}
assert $flag {== $_ 1} -v -line #__LINE_
*/

set flag $false
if {1} {set flag $true} else {set flag $false}					# if {x} {code} else {code}
assert $flag {== $_ 1} -v -line #__LINE_

set flag $false
if {0} {set flag $false} else {set flag $true}					# if {x} {code} else {code} INVERTED
assert $flag {== $_ 1} -v -line #__LINE_

set flag $false
if {[catch {
	if {0} {1} else {1} else
}]
} {set flag $true} 			# if {x} {code} else {code} INVERTED
assert $flag {== $_ 1} -v -line #__LINE_

# if {x} {code} elseif {x} {code}			// 6
set flag $false
if {1} {set flag $true} elseif {1} {set flag $true}				# if {x} {code} elseif {x} {code}
assert $flag {== $_ 1} -v -line #__LINE_

# INVERT if {x} {code} elseif {x} {code}			// 6
set flag $false
if {0} {set flag $true} elseif {1} {set flag $true}				# INVERT if {x} {code} elseif {x} {code}
assert $flag {== $_ 1} -v -line #__LINE_

# // if {x} {code} else {x} elseif			// 6 BAD
set flag $false
textcolor red
puts "Ignore errors ..."
textcolor
if {[catch {
	if {0} {1} else {1} elseif
} ] 
}     {set flag $true} 			# if {x} {code} else {code} INVERTED
assert $flag {== $_ 1} -v -line #__LINE_

# // if {x} {code} else {x} elseif			// 6 BAD
set flag $false
if {[catch {
	if {0} {1} else {1} else
}  ]}     {
	set flag $true
} 			# if {x} {code} else {code} INVERTED
assert $flag {== $_ 1} -v -line #__LINE_

unset flag -nocomplain
unset i -nocomplain

###############################################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #