###################################################################################
# File: 			test_enum.tcl
# Purpose:			Test file for [enum]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23...v1.26+
#
# Versions:
# 1.00 16/Aug/2016	First version
# 1.01 21/Jul/2017	Replace [checkex] with [assert]
# 1.03 06/Sep/2019	Add unset of test*
###################################################################################
# Notes: 
###################################################################################
# ADD single line simple and complex
###################################################################################

option expression off
option echo on			# We want to see inline error messages

textcolor yellow
puts "Testing \[enum\] ..."
textcolor
newline

enum {
	DRIVE_UNKNOWN
	DRIVE_NO_ROOT_DIR
	DRIVE_REMOVABLE
	DRIVE_FIXED      
	DRIVE_REMOTE
	DRIVE_CDROM
	DRIVE_RAMDISK
} -nocomplain

halt
enum {
	MB_OK
	MB_OKCANCEL
	MB_ABORTRETRYIGNORE
	MB_YESNOCANCEL
	MB_YESNO
	MB_RETRYCANCEL
	MB_CANCELTRYCONTINUE
	MB_ICONSTOP = 0x10
	MB_ICONERROR = 0x10
	MB_ICONHAND = 0x10
	MB_ICONQUESTION = 0x20
	MB_ICONWARNING = 0x30
	MB_ICONINFORMATION = 0x40
	MB_ICONASTERISK = 0x40
	MB_DEFBUTTON1 = 0
	MB_DEFBUTTON2 = 0x100
	MB_DEFBUTTON3 = 0x200
	MB_DEFBUTTON4 = 0x300
	MB_APPLMODAL = 0
	MB_SYSTEMMODAL = 0x1000
	MB_TASKMODAL = 0x2000
	MB_HELP = 0x4000
} -nocomplain

#set DRIVE_NO_ROOT_DIR 1

# Test const attribute. Should not assign
option expression off
puts "Ignore error raised attempting to set const value..."
set line #__LINE__
if {catch {set DRIVE_NO_ROOT_DIR 1}} {
	textcolor green
	puts "TEST catch \[set\] const PASS"		# We trapped the error
	textcolor
} else {
	textcolor red
	puts "TEST const set FAIL line $line"		# No error occurred
	textcolor
	die
}

assert $DRIVE_UNKNOWN {== $_ 0} -v -line #__LINE__
assert $DRIVE_NO_ROOT_DIR {== $_ 1} -v -line #__LINE__
assert $DRIVE_REMOVABLE {== $_ 2} -v -line #__LINE__
assert $DRIVE_FIXED {== $_ 3} -v -line #__LINE__
assert $DRIVE_REMOTE {== $_ 4} -v -line #__LINE__
assert $DRIVE_CDROM {== $_ 5} -v -line #__LINE__
assert $DRIVE_RAMDISK {== $_ 6} -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Test assigned complex enum 
option expression on
option echo on
halt
set line #__LINE__
assert $MB_OK {== $_ 0} -v -line #__LINE__
assert $MB_OKCANCEL {== $_ 1} -v -line #__LINE__
assert $MB_ABORTRETRYIGNORE {== $_ 2} -v -line #__LINE__
assert $MB_YESNOCANCEL {== $_ 3} -v -line #__LINE__
assert $MB_YESNO {== $_ 4} -v -line #__LINE__
assert $MB_RETRYCANCEL {== $_ 5} -v -line #__LINE__
assert $MB_CANCELTRYCONTINUE {== $_ 6} -v -line #__LINE__
assert $MB_DEFBUTTON1 {== $_ 0} -v -line #__LINE__
assert $MB_ICONSTOP {== $_ 0x10} -v -line #__LINE__
assert $MB_ICONERROR {== $_ 0x10} -v -line #__LINE__
assert $MB_ICONHAND {== $_ 0x10} -v -line #__LINE__
assert $MB_ICONQUESTION {== $_ 0x20} -v -line #__LINE__
assert $MB_ICONWARNING {== $_ 0x30} -v -line #__LINE__
assert $MB_ICONINFORMATION {== $_ 0x40} -v -line #__LINE__
assert $MB_ICONASTERISK {== $_ 0x40} -v -line #__LINE__
assert $MB_DEFBUTTON1 {== $_ 0} -v -line #__LINE__
assert $MB_DEFBUTTON2 {== $_ 0x100} -v -line #__LINE__
assert $MB_DEFBUTTON3 {== $_ 0x200} -v -line #__LINE__
assert $MB_DEFBUTTON4 {== $_ 0x300} -v -line #__LINE__
assert $MB_APPLMODAL {== $_ 0} -v -line #__LINE__
assert $MB_SYSTEMMODAL {== $_ 0x1000} -v -line #__LINE__
assert $MB_TASKMODAL {== $_ 0x2000} -v -line #__LINE__
assert $MB_HELP {== $_ 0x4000} -v -line #__LINE__
newline

puts "Line: "#__LINE__
option expression off
option echo on

set line #__LINE__
puts "$line: Clearing vars individually ..."
unset MB_OK -const
unset MB_OKCANCEL -const
unset MB_ABORTRETRYIGNORE -const
unset MB_YESNOCANCEL -const
unset MB_YESNO -const
unset MB_RETRYCANCEL -const
unset MB_CANCELTRYCONTINUE -const
unset MB_ICONSTOP -const
unset MB_ICONERROR -const
unset MB_ICONHAND -const
unset MB_ICONQUESTION -const
unset MB_ICONWARNING -const
unset MB_ICONINFORMATION -const
unset MB_ICONASTERISK -const
unset MB_DEFBUTTON1 -const
unset MB_DEFBUTTON2 -const
unset MB_DEFBUTTON3 -const
unset MB_DEFBUTTON4 -const
unset MB_APPLMODAL -const
unset MB_SYSTEMMODAL -const
unset MB_TASKMODAL -const
unset MB_HELP -const

unset DRIVE_RAMDISK -const
unset DRIVE_CDROM -const
unset DRIVE_REMOTE -const
unset DRIVE_FIXED -const
unset DRIVE_REMOVABLE -const
unset DRIVE_NO_ROOT_DIR -const
unset DRIVE_UNKNOWN -const

puts "Line: "#__LINE__

unset test? -glob -nocomplain -const
enum {
	test1
	test2 = 10
	test3
	test4 = 15
	test5 = 16
}

assert $test1 {== $_ 0} -v -line #__LINE__
assert $test2 {== $_ 10} -v -line #__LINE__
assert $test3 {== $_ 11} -v -line #__LINE__
assert $test4 {== $_ 15} -v -line #__LINE__
assert $test5 {== $_ 16} -v -line #__LINE__

option echo on
unset line -nocomplain
unset test? -glob -nocomplain -const
walk 10

newline
puts "Finished clearing vars individually ..."
vars
vars * -const

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
option echo off

newline
textcolor green
puts "* \[enum\] Done *"
textcolor

# EOF #