#############################################################################################
# File:				test_equal_assign.tcl
# Purpose:			Test the [=] and [unknown] commands
# Author:			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 16/Jan/2017	Ported from tests.tcl
# 1.01 21/Jul/2017	Replace [checkex] with [assert]
#############################################################################################
# Notes:
# The drawback with proc unknown is that its var scope is > root which requires upvar
# The workaround is to use [uplevel] to call [expr]. However the args must be quoted, and
# not braced otherwise var $args is masked
#############################################################################################

undef unknown -nocomplain	# Required if repeatedly calling in a loop

textcolor yellow
puts "Testing \[=\] ..."
textcolor
newline

= x 23
assert $x {== $x 23} -v -line #__LINE__

expr x = 46
assert $x {== $x 46} -v -line #__LINE__

#/* ///////////////////////////////////////////////////////////
# DONE FIX unknown not working for assign

option autoexec off
option echo on

/*
set x 0
proc foo {x} {
	puts "Enter foo($x)"
	return [uplevel 0 [expr $::true]]
}
foo ::a
puts "called foo()"
puts "direct expr call: '[expr x = (22 / 7.0)]' x=$x"
*/

puts "Defining proc unknown"
# proc unknown {args} {expr $args}
option autoexec off
option echo on
proc unknown {args} {
	newline
	textcolor darkyellow
	puts ">> ENTER \[unknown\] in [info script] args: '$args'"
	set r [uplevel @0 "expr $args"	]
	puts "<< EXIT proc unknown ::x=$::x r=$r"
	textcolor
	newline
	return $r
}

set x 0						# Prevents illusory results

# DONE FIX "::x" is passed via args as "x" (picolEval@9248)
# DONE FIX is called twice
#::x = ::y
option autoexec off
option echo on
x = 3.14					# Call proc unknown
textcolor green
puts "* proc unknown call done *"
textcolor
newline

assert $x {== $x 3.14} -v -line #__LINE__
option autoexec on

#/////////////////////////////////////////////////////////// */

# Test with expr off (command mode)
option expression off
set j 0
for {= i 0} {< $i 10} {++ i} {
	++ j
}
assert $j {== $_ 10} -v -line #__LINE__

# Test with expr on (expr mode detecting [=])
option expression on
option autoexec push off
set j 0
for {i = 0} {$i < 10} {++ i} {
	++ j
}
assert $j {== $_ 10} -v -line #__LINE__
option autoexec pop

undef unknown -nocomplain
unset i j x -nocomplain
#############################################################################################
vars
puts [procs]

newline
textcolor green
puts "* Done *"
textcolor

# EOF #