#################################################################################
# File: 			test_escspace.tcl
# Purpose:			Tests for escaped spaces within arrays
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 21/Feb/2021	First version
#################################################################################
# Notes:
# Various combinations of tests addressing array elements with whitespace
# and delimiters
# The a({sub script}) case needs to be looked at
# MPP should perhaps replace \t only within dquotes?
#################################################################################

option expression off
option echo on

set a(foo) bar			# Ensure we have an array with at least one element

# Slash create, noslash delete
puts "Line: "#__LINE__
set a(hello\ world) 23		# FIXED [set] should remove backslash(es)
assert $a("hello world") {== $_ 23} -v -line #__LINE__
array unset a("hello world");		# FIXED

puts "Line: "#__LINE__
set a("hello world") 24
#set a(hello\tworld) 25
assert $a("hello world") {== $_ 24} -v -line #__LINE__
array unset a("hello world");		# FIXED


puts "Line: "#__LINE__
array walk a

# Nolash create, slash delete
puts "Line: "#__LINE__
set a("hello world") 25		# FIXED [set] should remove backslash(es)
assert $a("hello world") {== $_ 25} -v -line #__LINE__
array unset a("hello world");		# FIXED

puts "Line: "#__LINE__
set a(hello\ world) 26
puts "Line: "#__LINE__
assert $a("hello world") {== $_ 26} -v -line #__LINE__
array unset a("hello world");		# FIXED
array walk a

# Nolash create, slash delete
puts "Line: "#__LINE__
set a("hello world") 27		# FIXED [set] should remove backslash(es)
assert $a("hello world") {== $_ 27} -v -line #__LINE__
array unset a("hello world");		# FIXED

puts "Line: "#__LINE__
set a("hello\ world") 28
puts "Line: "#__LINE__
assert $a("hello world") {== $_ 28} -v -line #__LINE__
array unset a("hello world");		# FIXED

puts "Line: "#__LINE__
set a(hello\ world) 29
puts "Line: "#__LINE__
assert $a(hello\ world) {== $_ 29} -v -line #__LINE__
array unset a(hello\ world);		# FIXED

array walk a
puts "Line: "#__LINE__
set a("hello world") 30
assert $a("hello world") {== $_ 30} -v -line #__LINE__
array unset a("hello world");		# FIXED

puts "Line: "#__LINE__
set a(hello\ world) 31
array walk a
assert $a("hello\ world") {== $_ 31} -v -line #__LINE__
array unset a("hello\ world")		# FIXED


## FIX in picolEval
puts "Line: "#__LINE__
set a(hello\ world) 32
puts "Line: "#__LINE__
puts $a(hello\ world)
assert $a("hello\ world") {== $_ 32} -v -line #__LINE__
puts "Line: "#__LINE__
array unset a("hello\ world")		# FIXED

puts "Line: "#__LINE__
# Due to the MPP the literal escape '\t' is never passed into the interpreted source 
# This is by design. The only whitespace escape passed in is '\ '
#set a(hello\tworld) 33				# FIX
set a("hello\tworld") 34			# PASS
array walk a
#puts $a(hello\tworld)				# FIX (not fixable due to MPP?)
puts $a("hello\tworld")
assert $a("hello\tworld") {== $_ 34} -v -line #__LINE__

puts "Line: "#__LINE__
set p 35
set q("hello world") p

puts "\$\$q(hello world) is '$$q(hello world)'"
assert $$q(hello world) {== $_ 35} -v -line #__LINE__		# Now works with no dquotes or escapes (for arrays)
assert $$q(hello\ world) {== $_ 35} -v -line #__LINE__
assert $$q("hello world") {== $_ 35} -v -line #__LINE__
array unset q -nocomplain

#######################################################################

# FIX
puts "Line: "#__LINE__
set p(hello) 23
set q p(hello)
puts "q is '$q'"
puts "\$\$q is '$$q'"
assert ${$q} {== $_ 23} -v -line #__LINE__

puts "Line: "#__LINE__
set p("hello world") 24
set q p("hello world")
puts "q is '$q'"
puts "\$\$q is '$$q'"
assert $$q {== $_ 24} -v -line #__LINE__		# Lazy
assert ${$q} {== $_ 24} -v -line #__LINE__		# Formal


# FIX March 2021 escaped space is not resolved for some reason. 
# May be internal to nested deref call
# Should be stored in escaped form
# First resolve returns escaped array reference
# Second resolve should call and translate same as 
puts "Line: "#__LINE__
set p("hello world") 25
set q p("hello\ world")
array walk p
echo "q is '$q'"		# Is stored escaped
puts "q is '$q'"		# Is stored escaped
#puts "\$\$q is '$$q'"
assert $$q {== $_ 25} -v -line #__LINE__
assert ${$q} {== $_ 25} -v -line #__LINE__

# Cancel
# LPSTR TCALL picolGetVarExNF(struct picolInterp* i, LPSTR name)
# int TCALL picolSetVarEx(struct picolInterp* i, LPCSTR varname_in, LPCSTR value_in, const short verbose_in)
# DEBUG_DD in picolEval
# L 

array unset a p -nocomplain
unset q -nocomplain
#######################################################################
vars
newline
textcolor green
puts "* Pass *"
textcolor

# EOF #