###########################################################################
# Name:				test_eval_multiline.tcl
# Purpose:			Test file for multiline expressions etc.
# Author:           M Shaw
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 07/Jan/2017	First version
###########################################################################
# Notes:
# Syntax {command} is invalid in Tcl
###########################################################################
option echo on
option expression off
option
/*
autoexec      off
echo           on
escape         on
expression    off
preprocessor   on
stacktrace    off
breakpoint    off 	*/

textcolor yellow
puts "Testing multiline expressions ...\n"
textcolor

###########################################################################
# Should be valid...
option expression off
set r ""
puts [
        set r [
			eval "/ 22.0 7.0"
				]
]

option expression off
assert $r {== $_ 3.142857142857143} -v -line #__LINE__

###########################################################################
# Should be valid...
set r ""
call [
	call [
        set r [
			eval "/ 22.0 7.0"
				]
	]
]
assert $r {== $_ 3.142857142857143} -v -line #__LINE__

/* Template
if {[catch {
	*
	]}
} {set r 1}
*/

###########################################################################
textcolor yellow
puts "Check catch syntax anomaly ...\n"
textcolor

option expression off
option autoexec off					# Required here
puts "Line: "#__LINE__
textcolor magenta

puts "Catch invalid syntax?: [catch {
 {puts hello}			# Invalid in ActiveState Tcl 8.6 (in Ticol it will autoexec)
	}]"					# <-- Syntax anomaly (added ])
textcolor


###########################################################################
option expression off
option autoexec off					# Required here
set r 0
textcolor magenta
if {[catch {
 {puts hello}			# Invalid in ActiveState Tcl 8.6 (in Ticol it will autoexec)
	]}					# <-- Syntax anomaly (added ])
} {set r 1}
textcolor
assert $r {== $_ 1} -v -line #__LINE__

###########################################################################
option expression off
#Unwound
puts "Line: "#__LINE__
textcolor yellow
puts "Check catch transposed brackets ...\n"
textcolor

if {     [catch {  {puts hello} ]}   } {set r 1}		# <-- transposed brackets
newline

###########################################################################
option expression off
puts "Line: "#__LINE__
textcolor yellow
puts "Check catch invalid autoexec ...\n"
textcolor

if {[catch {
 {puts hello}			# Invalid in ActiveState Tcl 8.6 (in Ticol it will autoexec)
	}]} {set r 1}
set line #__LINE__
assert $r {== $_ 1} -v -line #__LINE__

unset line r -nocomplain

###########################################################################

vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #