#################################################################################################
# File:				test_explain.tcl
# Author:			M Shaw
# Purpose:			Test file for [explain] command (see [calc -explain])
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 05/Aug/2018	First version
#################################################################################################
# Notes
# More tests to be added
#################################################################################################

option expression off

textcolor yellow
puts "Testing \[explain\] ..."
textcolor
newline

# SIMPLE
assert [explain 1+2*3] {eq $_ {[+ 1 [* 2 3]]}} -v -line #__LINE__
assert [explain 2*3+1] {eq $_ {[+ [* 2 3] 1]}} -v -line #__LINE__
assert [explain 2*3+1-2] {eq $_ {[- [+ [* 2 3] 1] 2]}} -v -line #__LINE__
assert [explain 1-2+2*3] {eq $_ {[+ [- 1 2] [* 2 3]]}} -v -line #__LINE__
assert [explain 1*2+3/4-5] {eq $_ {[- [+ [* 1 2] [/ 3 4]] 5]}} -v -line #__LINE__

assert [explain 1+2/3.0*4-5] {eq $_ {[- [+ 1 [* [/ 2 3.0] 4]] 5]}} -v -line #__LINE__
assert [round [eval [explain 1+2/3.0*4-5]] 4] {== $_ -1.3333 } -v -line #__LINE__

# MORE COMPLEX
puts [explain 1+2*3/4.0-5+6*7/8.0-9+0*1/2.0*3+4]
assert [explain 1+2*3/4.0-5+6*7/8.0-9+0*1/2.0*3+4] {eq $_ {[+ [+ [- [+ [- [+ 1 [/ [* 2 3] 4.0]] 5] [/ [* 6 7] 8.0]] 9] [* [/ [* 0 1] 2.0] 3]] 4]}} -v -line #__LINE__
assert [eval [explain 1+2*3/4.0-5+6*7/8.0-9+0*1/2.0*3+4]] {== $_ -2.25} -v -line #__LINE__

# AND / OR
assert [explain 2+3 > 4 || 12] {eq $_ {[|| [> [+ 2 3] 4] 12]}} -v -line #__LINE__
assert [explain {$a < 1 || $a >2}] {eq $_ {[|| [< $a 1] [> $a 2]]}} -v -line #__LINE__
assert [explain "2 < 4+3 || 2 > 5"] {eq $_ {[|| [+ [< 2 4] 3] [> 2 5]]}} -v
assert [explain 1+2*3 || 4*5 || 6+7] {eq $_ {[|| [|| [+ 1 [* 2 3]] [* 4 5]] [+ 6 7]]}} -v -line #__LINE__
assert [explain (1+2*3) || (4*5) || (6+7)] {eq $_ {[|| [|| [+ 1 [* 2 3]] [* 4 5]] [+ 6 7]]}} -v -line #__LINE__
assert [explain 1+2*3 || 4*5] {eq $_ {[|| [+ 1 [* 2 3]] [* 4 5]]}} -v -line #__LINE__

# CMP
assert [explain 1 ne 2 < 3] {eq $_ {[< [ne 1 2] 3]}} -v -line #__LINE__
assert [explain !1 > 2 ne 3] {eq $_ {[ne [> [! 1]  2] 3]}} -v -line #__LINE__

# MATH FUNCTS
assert [explain 3*abs(23)-12] {eq $_ {[- [* 3 [funct abs 23]] 12]}} -v -line #__LINE__
assert [explain tan(4*3+1/2)*3||4*5-6 ] {eq $_ {[|| [* [funct tan [+ [* 4 3] [/ 1 2]]] 3] [- [* 4 5] 6]]}} -v -line #__LINE__
#              tan(12)*3
#              -0.6358599286615808*3 || 14 -> 1

# [|| [* [* [* 3.14159265358979323 [funct tan [+ [* 4 3] [/ 1 2]]]] 3] [funct atan [/ 12 1.4]]] [- [* 4 5] 6]]
assert [explain $pi*tan(4*3+1/2)*3*atan(12/1.4) || 4*5-6 ] {eq $_ {[|| [* [* [* 3.14159265358979323 [funct tan [+ [* 4 3] [/ 1 2]]]] 3] [funct atan [/ 12 1.4]]] [- [* 4 5] 6]]}} -v -line #__LINE__
# Ticol == 2.0, - activestate == 1
puts [eval [explain $pi*tan(4*3+1/2)*3*atan(12/1.4) || 4*5-6 ]]
#              tan(12)*3   * atan(8.571428571428571)  || 4*5-6
#              -0.6358599286615808*3 * 1.4546546999149064   || 14
#				-2.774869901145377 || 14
# ActiveState == 1  (rounds up -2.774869901145377 to 3 and uses abs)
# Ticol -2.00  (truncates -2.774869901145377 to -2)
# VB5 == -1, -2 or 14 == -2, -3 or 14 == -1 (Thus, is rounding up during conversion)
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #