############################################################################################
# Name:				test_expr_ops.tcl (Unit test)
# Purpose:			Test valid operators in [expr]
# Author:       	M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 03/Feb/2022	Updated
############################################################################################
# Notes:
# Some rudimentary tests on suppored [expr] operators
# & | && || ** == != > < >= <= &= |= ^= += -= *= /=
# FIX (not working, << >> are returning 0 with raw variable names)
# FIX Some commands handle raw variable names, others don't
# ADD %= operator
############################################################################################

option expression off

############################################################################################
# Bitwise
############################################################################################
# &
puts "Line: "#__LINE__
assert [expr 0 & 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 & 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 & 0] {== $_ 0} -v -line #__LINE__
assert [expr 0 & 0] {== $_ 0} -v -line #__LINE__

assert [expr 23 & 1] {== $_ 1} -v -line #__LINE__
assert [expr 23 & 2] {== $_ 2} -v -line #__LINE__
assert [expr 3 & 2] {== $_ 2} -v -line #__LINE__
assert [expr 23 & 23] {== $_ 23} -v -line #__LINE__
assert [expr 23 & 0] {== $_ 0} -v -line #__LINE__

# |
assert [expr 0 | 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 | 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 | 0] {== $_ 1} -v -line #__LINE__
assert [expr 1 | 1] {== $_ 1} -v -line #__LINE__
assert [expr 23 | 23] {== $_ 23} -v -line #__LINE__
assert [expr 23 | 0] {== $_ 23} -v -line #__LINE__

############################################################################################
# Logical 
############################################################################################

puts "Line: "#__LINE__
assert [expr 0 && 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 && 0] {== $_ 0} -v -line #__LINE__
assert [expr 0 && 1] {== $_ 0} -v -line #__LINE__
assert [expr 1 && 1] {== $_ 1} -v -line #__LINE__
assert [expr 23 && 23] {== $_ 1} -v -line #__LINE__


assert [expr 0 || 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 || 0] {== $_ 1} -v -line #__LINE__
assert [expr 0 || 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 || 1] {== $_ 1} -v -line #__LINE__
assert [expr 23 || 23] {== $_ 1} -v -line #__LINE__
assert [expr 23 || 23] {== $_ 1} -v -line #__LINE__


############################################################################################
puts "Line: "#__LINE__
# ** Power
# FIX should be 2?
# Anything to the power of 0 is 1
# https://www.cuemath.com/questions/how-to-express-2-to-the-power-of-0/
assert [expr 2 ** 0] {== $_ 1} -v -line #__LINE__		# VB is 2 ^ 5
assert [expr 2 ** 1] {== $_ 2} -v -line #__LINE__		# VB is 2 ^ 5
assert [expr 2 ** 2] {== $_ 4} -v -line #__LINE__		# VB is 2 ^ 5
assert [expr 2 ** 3] {== $_ 8} -v -line #__LINE__		# VB is 2 ^ 5
assert [expr 2 ** 4] {== $_ 16} -v -line #__LINE__		# VB is 2 ^ 5
assert [expr 2 ** 5] {== $_ 32} -v -line #__LINE__		# VB is 2 ^ 5
newline


############################################################################################
# Relational: ==   !=   >   <   >=   <=
############################################################################################
puts "Line: "#__LINE__
# ==
textcolor yellow
puts "Operator =="
textcolor
assert [expr 0 == 0] {== $_ 1} -v -line #__LINE__
assert [expr 0 == 1] {== $_ 0} -v -line #__LINE__
assert [expr 1 == 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 == 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 == 23] {== $_ 0} -v -line #__LINE__
assert [expr 1 == {}] {== $_ 0} -v -line #__LINE__
assert [expr {} == {}] {== $_ 1} -v -line #__LINE__
assert [expr {} == 1] {== $_ 0} -v -line #__LINE__
newline

# !=
textcolor yellow
puts "Operator !="
textcolor
assert [expr 0 != 0] {== $_ 0} -v -line #__LINE__
assert [expr 0 != 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 != 0] {== $_ 1} -v -line #__LINE__
assert [expr 1 != 1] {== $_ 0} -v -line #__LINE__
assert [expr 1 != 23] {== $_ 1} -v -line #__LINE__
assert [expr 1 != {}] {== $_ 1} -v -line #__LINE__
assert [expr {} != {}] {== $_ 0} -v -line #__LINE__
assert [expr {} != 1] {== $_ 1} -v -line #__LINE__
newline

# >
textcolor yellow
puts "Operator >"
textcolor
assert [expr 0 > 0] {== $_ 0} -v -line #__LINE__
assert [expr 0 > 1] {== $_ 0} -v -line #__LINE__
assert [expr 1 > 0] {== $_ 1} -v -line #__LINE__
assert [expr 1 > 1] {== $_ 0} -v -line #__LINE__
assert [expr 22 > 0] {== $_ 1} -v -line #__LINE__
#assert [expr 0 > {}] {== $_ 0} -v -line #__LINE__		# Should this be true?
assert [expr {} > 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 > {}] {== $_ 1} -v -line #__LINE__
assert [expr {} > 1] {== $_ 0} -v -line #__LINE__
assert [expr 22 > 0] {== $_ 1} -v -line #__LINE__
assert [expr 1.23 > 1.23] {== $_ 0} -v -line #__LINE__
assert [expr 1.23 > 4.56] {== $_ 0} -v -line #__LINE__
assert [expr 4.56 > 1.23] {== $_ 1} -v -line #__LINE__
assert [expr 1 > 4.56] {== $_ 0} -v -line #__LINE__
assert [expr 4.56 > 1] {== $_ 1} -v -line #__LINE__
newline

# <
textcolor yellow
puts "Operator <"
textcolor
assert [expr 0 < 0] {== $_ 0} -v -line #__LINE__
assert [expr 0 < 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 < 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 < 1] {== $_ 0} -v -line #__LINE__
assert [expr 22 < 0] {== $_ 0} -v -line #__LINE__
assert [expr 0 < {}] {== $_ 0} -v -line #__LINE__		# Should this be true?
#assert [expr {} < 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 < {}] {== $_ 0} -v -line #__LINE__
assert [expr {} < 1] {== $_ 1} -v -line #__LINE__
assert [expr 22 < 0] {== $_ 0} -v -line #__LINE__
assert [expr 1.23 < 1.23] {== $_ 0} -v -line #__LINE__
assert [expr 1.23 < 4.56] {== $_ 1} -v -line #__LINE__
assert [expr 4.56 < 1.23] {== $_ 0} -v -line #__LINE__
assert [expr 1 < 4.56] {== $_ 1} -v -line #__LINE__
assert [expr 4.56 < 1] {== $_ 0} -v -line #__LINE__
newline

# >=
textcolor yellow
puts "Operator >="
textcolor
assert [expr 0 >= 0] {== $_ 1} -v -line #__LINE__
assert [expr 0 >= 1] {== $_ 0} -v -line #__LINE__
assert [expr 1 >= 0] {== $_ 1} -v -line #__LINE__
assert [expr 1 >= 1] {== $_ 1} -v -line #__LINE__
assert [expr 22 >= 0] {== $_ 1} -v -line #__LINE__
#assert [expr 0 >= {}] {== $_ 0} -v -line #__LINE__		# Should this be true?
#assert [expr {} >= 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 >= {}] {== $_ 1} -v -line #__LINE__
assert [expr {} >= 1] {== $_ 0} -v -line #__LINE__
assert [expr 22 >= 0] {== $_ 1} -v -line #__LINE__
assert [expr 1.23 >= 1.23] {== $_ 1} -v -line #__LINE__
assert [expr 1.23 >= 4.56] {== $_ 0} -v -line #__LINE__
assert [expr 4.56 >= 1.23] {== $_ 1} -v -line #__LINE__
assert [expr 1 >= 4.56] {== $_ 0} -v -line #__LINE__
assert [expr 4.56 >= 1] {== $_ 1} -v -line #__LINE__
newline

# <=
textcolor yellow
puts "Operator <="
textcolor
assert [expr 0 <= 0] {== $_ 1} -v -line #__LINE__
assert [expr 0 <= 1] {== $_ 1} -v -line #__LINE__
assert [expr 1 <= 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 <= 1] {== $_ 1} -v -line #__LINE__
assert [expr 22 <= 0] {== $_ 0} -v -line #__LINE__
#assert [expr 0 <= {}] {== $_ 0} -v -line #__LINE__		# Should this be true?
#assert [expr {} <= 0] {== $_ 0} -v -line #__LINE__
assert [expr 1 <= {}] {== $_ 0} -v -line #__LINE__
assert [expr {} <= 1] {== $_ 1} -v -line #__LINE__
assert [expr 22 <= 0] {== $_ 0} -v -line #__LINE__
assert [expr 1.23 <= 1.23] {== $_ 1} -v -line #__LINE__
assert [expr 1.23 <= 4.56] {== $_ 1} -v -line #__LINE__
assert [expr 4.56 <= 1.23] {== $_ 0} -v -line #__LINE__
assert [expr 1 <= 4.56] {== $_ 1} -v -line #__LINE__
assert [expr 4.56 <= 1] {== $_ 0} -v -line #__LINE__
newline

############################################################################################
# Combined math operators: &=   |=   ^=   +=   -=   *=   /=   (%=)
# Supports naked variables
############################################################################################
# &=
textcolor yellow
puts "Operator &="
textcolor
set a 1
assert [expr a &= 0] {== $_ 0} -v -line #__LINE__
set a 1
assert [expr a &= 1] {== $_ 1} -v -line #__LINE__
set a 1
assert [expr a &= {}] {== $_ 0} -v -line #__LINE__
set a 1
set i 0
try {
	assert [expr a &= 1.23] {== $_ 1} -v -line #__LINE__		# Floats are invalid for bitops
} catch {
	puts "Safely caught error at "#__LINE__
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
newline


# |=
textcolor yellow
puts "Operator |="
textcolor
set a 0
assert [expr a |= 1] {== $_ 1} -v -line #__LINE__
set a 0
assert [expr a |= 2] {== $_ 2} -v -line #__LINE__
set a 0
assert [expr a |= {}] {== $_ 0} -v -line #__LINE__
set a 0
set i 0
try {
	assert [expr a |= 1.23] {== $_ 1} -v -line #__LINE__		# Floats are invalid for bitops
} catch {
	puts "Safely caught error at "#__LINE__
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__


# ^=
textcolor yellow
puts "Operator ^="
textcolor
set a 0
assert [expr a ^= 1] {== $_ 1} -v -line #__LINE__
set a 0
assert [expr a ^= 0] {== $_ 0} -v -line #__LINE__
set a 1
assert [expr a ^= 1] {== $_ 0} -v -line #__LINE__
set a 1
assert [expr a ^= 0] {== $_ 1} -v -line #__LINE__
set a 0
assert [expr a ^= 2] {== $_ 2} -v -line #__LINE__
set a 2
assert [expr a ^= 3] {== $_ 1} -v -line #__LINE__
set a 3
assert [expr a ^= 7] {== $_ 4} -v -line #__LINE__

set a 0
assert [expr a ^= {}] {== $_ 0} -v -line #__LINE__
set a 0
set i 0
try {
	assert [expr a ^= 1.23] {== $_ 1} -v -line #__LINE__		# Floats are invalid for bitops
} catch {
	puts "Safely caught error at "#__LINE__
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__


# +=
textcolor yellow
puts "Operator +="
textcolor
set a 0
assert [expr a += 0] {== $_ 0} -v -line #__LINE__
set a 0
assert [expr a += 1] {== $_ 1} -v -line #__LINE__
set a 1
assert [expr a += 2] {== $_ 3} -v -line #__LINE__
set a 2
assert [expr a += 2] {== $_ 4} -v -line #__LINE__
set a 1
assert [expr a += {}] {== $_ 1} -v -line #__LINE__
set a 1
assert [expr a += 1.23] {== $_ 2.23} -v -line #__LINE__		# Floats are invalid for bitops
newline

# -=
textcolor yellow
puts "Operator -="
textcolor
set a 1
assert [expr a -= 0] {== $_ 1} -v -line #__LINE__
set a 1
assert [expr a -= 1] {== $_ 0} -v -line #__LINE__
set a 2
assert [expr a -= 1] {== $_ 1} -v -line #__LINE__
set a 1
assert [expr a -= 2] {== $_ -1} -v -line #__LINE__
set a 2
assert [expr a -= 2] {== $_ 0} -v -line #__LINE__
set a 1
assert [expr a -= {}] {== $_ 1} -v -line #__LINE__
set a 1
assert [expr a -= 1.23] {== $_ -0.23} -v -line #__LINE__		# Floats are invalid for bitops
newline

# *=
textcolor yellow
puts "Operator *="
textcolor
set a 2
assert [expr a *= 0] {== $_ 0} -v -line #__LINE__
set a 2
assert [expr a *= 1] {== $_ 2} -v -line #__LINE__
set a 2
assert [expr a *= 2] {== $_ 4} -v -line #__LINE__
set a 2
assert [expr a *= -2] {== $_ -4} -v -line #__LINE__
set a 1
assert [expr a *= $pi] {== $_ $pi} -v -line #__LINE__
set a 2
assert [expr a *= $pi] {== $_ [* 2 $pi]} -v -line #__LINE__
set a 1
assert [expr a *= {}] {== $_ 0} -v -line #__LINE__
set a 1
assert [expr a *= 1.23] {== $_ 1.23} -v -line #__LINE__		# Floats are invalid for bitops
set a -1
assert [expr a *= 1.23] {== $_ -1.23} -v -line #__LINE__		# Floats are invalid for bitops
newline

# /=
textcolor yellow
puts "Operator /="
textcolor
set a 2
set i 0
try {
	assert [expr a /= 0] {== $_ 0} -v -line #__LINE__
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
set a 2
assert [expr a /= 1] {== $_ 2} -v -line #__LINE__
set a 2
assert [expr a /= 2] {== $_ 1} -v -line #__LINE__
set a 2
assert [expr a /= -2] {== $_ -1} -v -line #__LINE__
set a $pi
assert [expr a /= 1] {== $_ $pi} -v -line #__LINE__
set a $pi
assert [expr a /= 2] {== $_ [/ $pi 2]} -v -line #__LINE__
set a 2
set i 0
try {
	assert [expr a /= {}] {== $_ 0} -v -line #__LINE__
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
set a 1
puts [trunc [expr a /= 1.23] 3]
set a 1
assert [trunc [expr a /= 1.23] 3] {== $_ 0.813} -v -line #__LINE__		# Floats are invalid for bitops
set a 1
assert [trunc [expr a /= -1.23] 3] {== $_ -0.813} -v -line #__LINE__		# Floats are invalid for bitops
newline

/* 
# %= NYI
textcolor yellow
puts "Operator %="
textcolor
set a 4
assert [expr a %= 2] {== $_ 1} -v -line #__LINE__
# */

############################################################################################
# Bit-shifting operators:  <<   >>                       
############################################################################################
# FIX (not working, returning 0)
# <<
set a 2
assert [expr $a << 0] {== $_ 2} -v -line #__LINE__		# 0?
assert [expr $a << 1] {== $_ 4} -v -line #__LINE__
assert [expr $a << 2] {== $_ 8} -v -line #__LINE__
assert [expr $a << 3] {== $_ 16} -v -line #__LINE__
/*
#assert [expr a << 0] {== $_ 2} -v -line #__LINE__		# 0?
#assert [expr a << 1] {== $_ 4} -v -line #__LINE__
#assert [expr a << 2] {== $_ 8} -v -line #__LINE__
#assert [expr a << 3] {== $_ 16} -v -line #__LINE__
# */

# FIX (not working, returning 0)
# >>
set a 16
assert [expr $a >> 1] {== $_ 8} -v -line #__LINE__
assert [expr $a >> 2] {== $_ 4} -v -line #__LINE__
assert [expr $a >> 3] {== $_ 2} -v -line #__LINE__
assert [expr $a >> 4] {== $_ 1} -v -line #__LINE__
assert [expr $a >> 5] {== $_ 0} -v -line #__LINE__

/*
#assert [expr a >> 0] {== $_ 16} -v -line #__LINE__		# 0?
assert [expr a >> 1] {== $_ 8} -v -line #__LINE__
assert [expr a >> 2] {== $_ 4} -v -line #__LINE__
assert [expr a >> 3] {== $_ 2} -v -line #__LINE__
assert [expr a >> 4] {== $_ 1} -v -line #__LINE__
assert [expr a >> 5] {== $_ 0} -v -line #__LINE__
# */

unset a i -nocomplain
############################################################################################
newline
vars
newline

textcolor green
puts "* Done *"
textcolor
# EOF #