##################################################################################
# File:				test_file_io.tcl
# Purpose:			Testing [file] I/O commands
# Author:			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 17/Jan/2022	First version
# 1.01 07/Jun/2022	Revised for changes to [loop] bounds
# 1.02 28/Dec/2022	Revised again for reversion to [loop] bounds
##################################################################################
# Notes:
# FIX puts $fp writes only 0x0a and omits 0x0d from CRLFs in wb mode
##################################################################################

option expression off
option echo on

textcolor yellow
puts "File: \[read\] in [info script]"
textcolor
newline

set filename "test-file-io.txt"

puts "Line: "#__LINE__
textcolor yellow
puts "Writing file..."
textcolor

# Write the test file...
set fp [open $filename wb]
#set fp [open $filename wt]
if {$fp} {							# File is open
	puts "Opened file ok"
	loop i 0 5 {			# 0..4 inclusive (5)
		#puts $fp "Line $i"						# Use for "wt" (text) mode
		puts $fp "Line $i\r\n" -nonewline		# Use for "wb" (binary) mode 
	}
	close $fp
} else {
	puts "Failed to open $filename for writing in [info script] at line "#__LINE__
	die
}
newline
puts "Written file size is [file size $filename]"

############################################################################
# @ type $filename

puts "Line: "#__LINE__
textcolor yellow
puts "Reading whole file..."
textcolor
newline

 set fp [open $filename rb]
 if {$fp} {							# File is open
	textcolor magenta
	set s [read $fp]
	puts $s
	textcolor
 	close $fp
} else {
	puts "Failed to open $filename for reading in [info script] at line "#__LINE__
}

assert [string length $s] {== $_ 40} -v -line #__LINE__		# 40 bytes size

textcolor yellow
puts "Reading file in line input mode..."
textcolor

set fp [open $filename rb]			# Ensure we open binary so we can correctly calculate the \r\n line ends
if {$fp} {							# File is open
	puts "File is open"
	set i 0
	do {
		set s [read $fp -line -tab]		# Line input and convert tab to space
		if {! [file eof $fp] }	{
			#set s [read $fp -bytes 30 -line -tab]			# Read 8 bytes (6 bytes text + CRLF)
			puts "$i: '$s'"	# Will also read in the CRLFs
			assert $s {eq $_ "Line $i"} -v -line #__LINE__
			++ i
		}
	} while {! [file eof $fp]} 
	close $fp
} else {
	puts "Failed to open $filename for reading in [info script] at line "#__LINE__
}

assert $s {eq $_ ""} -v -line #__LINE__			# Final read will be NULL

newline

textcolor yellow
puts "Reading file in byte-read mode..."
textcolor

set fp [open $filename rb]			# Ensure we open binary so we can correctly calculate the \r\n line ends
if {$fp} {							# File is open
	puts "File is open"
	set i 0
	do {
		set s [read $fp -bytes 8]	# Read 8 bytes at a time (will include CRLFs)
		# Final read will reach EOF and return "" with code SUCCESS
		if {! [file eof $fp]} {
			#puts "$i $s" -nonewline	
			assert $s {eq $_ "Line $i\r\n"} -v -line #__LINE__
			#puts "DEBUG EOF? [file eof $fp]"
		}
		++ i
	} while {! [file eof $fp]} 
	close $fp
} else {
	puts "Failed to open $filename for reading in [info script] at line "#__LINE__
}

unset filename fp i s -nocomplain

vars

########################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #