####################################################################################################
# File: 			test_file_stat.tcl
# Purpose:			Test for [file stat]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 16/Oct/2017	First version
####################################################################################################
# Notes: 
# As per man file example for .stat
# [stat] isn't much use but is included for the sake of completeness as C++ has _stat() we can use
####################################################################################################
/*

  $a(mode)        33279
  $a(ino)         0
  $a(dev)         2
  $a(atime)       1481501018
  $a(mtime)       1481501108
  $a(gid)         0
  $a(size)        190976
  $a(nlink)       1
  $a(rdev)        2l
  $a(ctime)       1481501018
  $a(drive)       C
  $a(uid)         0
*/  
####################################################################################################

option expression off
option echo on

set filename [info executablename] 		# ticol.exe (or embedded.exe etc.)

textcolor darkyellow
puts "Testing \[file stat $filename a\] ..."
textcolor
newline

if {! [file exists $filename]} {		# Can't think how this would happen, but...
	die "I can't find [info executablename]"
}

puts [file stat $filename a]

unset ss -nocomplain

textcolor magenta
array foreach a val ss {
	puts "\$a($ss) \t$val"
}
textcolor
newline

puts "\$a(mode)=$a(mode)"
assert $a(mode) {|| [== $_ 33206] [== $_ 33279]} -v -line #__LINE__
puts "\$a(ino)=$a(ino)"
assert $a(ino) {== $_ 0} -v -line #__LINE__
# can be 2, 18 etc.
puts "\$a(dev)=$a(dev)"
assert $a(dev) {>= $_ 0} -v -line #__LINE__
puts "\$a(atime)=$a(atime)"
assert $a(atime) {>= $_ 1400000000} -v -line #__LINE__	# Tue May 13 17:53:20 2014
puts "\$a(mtime)=$a(mtime)"
assert $a(mtime) {>= $_ 1400000000} -v -line #__LINE__	# Tue May 13 17:53:20 2014
puts "\$a(gid)=$a(gid)"
assert $a(gid) {== $_ 0} -v -line #__LINE__
puts "\$a(size)=$a(size)"
assert $a(size) {> $_ 0} -v -line #__LINE__
puts "\$a(nlink)=$a(nlink)"
assert $a(nlink) {== $_ 1} -v -line #__LINE__
# can be 2 or 18 etc.
puts "\$a(rdev)=$a(rdev)"
assert $a(rdev) {>= $_ 0} -v -line #__LINE__
puts "\$a(ctime)=$a(ctime)"
assert $a(ctime) {>= $_ 1400000000} -v -line #__LINE__	# Tue May 13 17:53:20 2014
puts "\$a(drive)=$a(drive)"
assert $a(drive) {ge $_ C} -v -line #__LINE__
puts "\$a(uid)=$a(uid)"
assert $a(uid) {== $_ 0} -v -line #__LINE__

array unset a -nocomplain
unset filename -nocomplain

################################################################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #