###############################################################################
# File:				test_for.tcl
# Author:			M Shaw
# Purpose:			Test file for [for]
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 22/Mar/2016	First version
# 1.01 16/Aug/2016	First proper test case
# 1.02 21/Jul/2017	Replaced [checkex] with [assert]
###############################################################################
# Notes:
# Created to do optimisation timings
# ADD continue. Note it does not call the for incr
###############################################################################
/*
Command          us/Call    Calls
set                    0   100003
incr                   0   100000
for                   29   100000
<                      0   100000
39 microsecond(s) per iteration
# */
###############################################################################

option echo on
option expression off

set i 0

newline
textcolor yellow
puts "Testing \[for\]\n"
textcolor

option expression on
for {[set i 0]} {$i < 10} {incr i} {
}
assert $i {== $_ 10} -v -line #__LINE__


set line #__LINE__
option expression off
set i 0
for {set i 0} {< $i 10} {incr i} {
}
assert $i {== $_ 10} -v -line #__LINE__


textcolor yellow
puts "Testing \[for\]->\[continue\] ..."
textcolor
option expression off
set line #__LINE__
set sum 0
for {set x 0} {< $x 10} {incr x} {
    if {== $x 5} {
        continue
    }
	+= sum $x
    puts "x is $x"
}
assert $sum {== $_ 40} -v -line #__LINE__ 


# Full args #
for {set i 0} {< $i 10} {incr i} {
	if {== $i 5} {break}
}
assert $i {== $_ 5} -v -line #__LINE__

# Empty initiator #
set i 0
for {} {< $i 10} {incr i} {
	if {== $i 5} {break}
}
assert $i {== $_ 5} -v -line #__LINE__

/* # FIX {} should return OK and continue
# Empty arg #
for {set i 0} {} {incr i} {
	if {== $i 5} {break}
	if {< $i 10} {break}
}
assert $i {== $_ 5} -v -line #__LINE__
*/

# Empty increment #
set line #__LINE__
for {set i 0} {< $i 10} {} {
	if {== $i 5} {break}
	incr i
}
assert $i {== $_ 5} -v -line #__LINE__

# Multiple initiators
set line #__LINE__
for {set i 0; set j 5} {< $i 10} {incr i} {
	if {== $i 5} {break}
}
assert $j {== $_ 5} -v -line #__LINE__
assert $i {== $_ 5} -v -line #__LINE__

# Multiple args
for {set i 0; set j 0} {&& [< $i 10] [< $j 20]} {incr i} {
	if {== $i 5} {break}
	incr j
}
assert $j {== $_ 5} -v -line #__LINE__
assert $i {== $_ 5} -v -line #__LINE__

# Multiple incrementors
set line #__LINE__
for {set i 0; set j 0} {&& [< $i 10] [< $j 20]} {incr i; incr j} {
	puts "i:$i j:$j"
	if {== $i 5} {break}
}
assert $j {== $_ 5} -v -line #__LINE__
assert $i {== $_ 5} -v -line #__LINE__

newline
textcolor yellow
puts "\[for\] Timer tests (option expression off)"
textcolor
set limit 1
time {for {set i 0} {< $i $limit} {incr i} {} } 1000 -info
puts "Timer Test Done"

textcolor yellow
puts "\[for\] Timer tests (option expression off)"
textcolor
option expression on
set limit 1
time {for {set i 0} {$i < $limit} {incr i} {} } 1000 -info
puts "Timer Test Done"
option expression off


##############################################
# For with empty args
set i 0
for {} {} {} {
	++ i
	if {== $i 10} {break}
}
assert $i {== $_ 10} -v -line #__LINE__

unset i j l limit line sum x -nocomplain
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "\[for\] Tests Done"
textcolor

# EOF #