#################################################################
# File:				test_format.tcl
# Purpose:			Test the [format] command
# Author:			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 15/Jan/2017	Ported from tests.tcl
# 1.01 21/Jul/2017	Replaced [checkex] and [check] with [assert]
# 1.02 26/Oct/2018	Test cases added
#################################################################
# FIX memory leaks somewhere. Looped from loop_test.tcl -> tests.tcl -> this
# Loops		Start		End			Lost		Comment
# 10000		1884		4172
################################################################################
# 
# Notes:
# [format] (printf now returns count)
# [printf] calls [format]
#################################################################

textcolor yellow
puts "Testing \[format\] ..."
textcolor
newline

# Hex #
assert [format "0x%x" 123456] {eq $_ "0x1e240"} -v -line #__LINE__
assert [format "0x%X" 123456] {eq $_ "0x1E240"} -v -line #__LINE__
assert [format "0x%4.4x" 123456] {eq $_ "0x1e240"} -v -line #__LINE__
assert [format "0x%8.8x" 123456] {eq $_ "0x0001e240"} -v -line #__LINE__
assert [format "0x%16.16x" 123456] {eq $_ "0x000000000001e240"} -v -line #__LINE__

# Octal #
assert [format "0o%o" 123456] {eq $_ "0o361100"} -v -line #__LINE__

# Decimal #
assert [format "%d\n" 123456] {eq $_ "123456\n"} -v -line #__LINE__

# General #
assert [format "%g" 123456] {eq $_ "123456"} -v -line #__LINE__

# Integer #
assert [format "%i" "123456"] {eq $_ "123456"} -v -line #__LINE__
assert [format "%1.1i" 123456] {eq $_ "123456"} -v -line #__LINE__
assert [format "%2.2i" 123456] {eq $_ "123456"} -v -line #__LINE__
assert [format "%3.3i" 123456] {eq $_ "123456"} -v -line #__LINE__
assert [format "%4.4i" 123456] {eq $_ "123456"} -v -line #__LINE__
assert [format "%5.5i" 123456] {eq $_ "123456"} -v -line #__LINE__
assert [format "%6.6i" 123456] {eq $_ "123456"} -v -line #__LINE__
assert [format "%7.7i" 123456] {eq $_ "0123456"} -v -line #__LINE__
assert [format "%8i" 123456] {eq $_ "  123456"} -v -line #__LINE__
assert [format "%7.7i" 123456] {eq $_ "0123456"} -v -line #__LINE__
assert [format "%20i" 123456] {eq $_ "              123456"} -v -line #__LINE__
assert [format "%-8i" 123456] {eq $_ "123456  "} -v -line #__LINE__
assert [format "%-9i" 123456] {eq $_ "123456   "} -v -line #__LINE__
assert [format "%-20i" 123456] {eq $_ "123456              "} -v -line #__LINE__
assert [format "%020i" 123456] {eq $_ "00000000000000123456"} -v -line #__LINE__

assert [format "%.5i" 123]	{eq $_ 00123}  -v -line #__LINE__	# -> "00123"
assert [format "%5.5i" 123]	{eq $_ 00123}  -v -line #__LINE__	# -> "00123"
assert [format "%5i" 123]	{eq $_ "  123"}  -v -line #__LINE__	# -> "  123"
assert [format "%-5i" 123]	{eq $_ "123  "}  -v -line #__LINE__	# -> "123  "

# Float #
assert [format "%.2f %%" $pi] {eq $_ "3.14 %"} -v -line #__LINE__
assert [format "%.5f" 1.23]	{eq $_ 1.23000}  -v -line #__LINE__	# -> "00123"
assert [format "%5.5f" 1.23]	{eq $_ 1.23000}  -v -line #__LINE__	# -> "00123"
assert [format "%5f" 1.23]	{eq $_ "1.230000"}  -v -line #__LINE__	# -> "  123"
assert [format "%-5f" 1.23]	{eq $_ "1.230000"}  -v -line #__LINE__	# -> "123  "

# Char #
assert [format "%c" 98] {eq $_ b} -v -line #__LINE__
assert [format "%c" [asc Hello]] {eq $_ H} -v -line #__LINE__

# String #
assert [format "%s" Hello] {eq $_ "Hello"} -v -line #__LINE__
assert [format "%20.20s" Hello] {eq $_ "               Hello"} -v -line #__LINE__
assert [format "%-20.20s" Hello] {eq $_ "Hello               "} -v -line #__LINE__
assert [format "%-20s" Hello] {eq $_ "Hello               "} -v -line #__LINE__

assert [format "%-*s" 20 Hello]      {[eq $_ "Hello               "]} -v -line #__LINE__ # "Hello               "
assert [format "%-*.*s" 20 20 Hello] {[eq $_ "Hello               "]} -v -line #__LINE__ # "Hello               "
assert [format "%-*.*s" 20 2 Hello]  {[eq $_ "He                  "]} -v -line #__LINE__ # "He                  "

assert [format "%3s" $qbf]  {[eq $_ $::qbf]} -v -line #__LINE__ # Will not crop! $qbf
assert [format "%.3s" $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"
assert [format "%3.3s" $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"
assert [format "%-3s" $qbf]  {[eq $_ $::qbf]} -v -line #__LINE__ # Will not crop! $qbf
assert [format "%-.3s" $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"
assert [format "%-3.3s" $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"

assert [format "%*s" 3 $qbf]  {[eq $_ $::qbf]} -v -line #__LINE__ # Will not crop! $qbf
assert [format "%.*s" 3 $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"
assert [format "%*.*s" 3 3 $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"
assert [format "%-*s" 3 $qbf]  {[eq $_ $::qbf]} -v -line #__LINE__ # Will not crop! $qbf
assert [format "%-.*s" 3 $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"
assert [format "%-*.*s" 3 3 $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to 3 "The"

assert [format "%*s" 3 $qbf]  {[eq $_ $::qbf]} -v -line #__LINE__ # Will not crop! $qbf
assert [format "%.*s" 3 $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to "The"
assert [format "%*.*s" 3 1 $qbf]  {[eq $_ "  T"]} -v -line #__LINE__ # Will crop to "   T"
assert [format "%-*s" 3 $qbf]  {[eq $_ $::qbf]} -v -line #__LINE__ # Will not crop! $qbf
assert [format "%-.*s" 3 $qbf]  {[eq $_ "The"]} -v -line #__LINE__ # Will crop to "The"
assert [format "%-*.*s" 3 1 $qbf]  {[eq $_ "T  "]} -v -line #__LINE__ # Will crop to "T  "

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #