################################################################################
# File:			test_forward_slash.tcl
# Author:		M Shaw
# Purpose:		Tests for use of forward slash / with file routines
# Language:		Ticol Tcl
#
# Versions:
# 11/Oct/2017	First version
# 19/Jul/2018	Add [string match ".tc?"]
################################################################################
# Notes:
# Build up a UNC path of 'self' and translate to Unix format
# $script_filename is not valid in the VB IDE at present
################################################################################
# FIX memory leaks somewhere
# Loops		Start		End			Lost		Comment
# 10000		1884
################################################################################
# FIX [file attributes]  escape? Wrap flags in braces for lindex?
#	-archive 1 -hidden 0 -longname xwreg.dll -readonly 0 -shortname xwreg.dll -system 0
# FIX [file gets filename] hangs (infinite read if interpreted as console handle)
# FIX Look at how we need to use [escape] and [unescape]
# FIX readfile should allow -array varname for syntax standardisation
# FIX should reject or accept [file normalise] with an s
# FIX undebug commandset()
################################################################################
# Tests which take a filename
#
# Command				Escapes 	Unix	Win		Comments
# [readfile]			Y			PASS
# [file open]			Y			PASS
# [file exists]			Y			PASS
# [file readable]		Y			PASS	
# [file attributes]		Y			PASS
# [file md5]			Y			PASS			MD5 OK
# [file normal_path]	-			PASS
# [file executable]		Y			PASS
# [file extension]		N			PASS
# [file isdirectory]	Y			PASS			False result
# [file isfile]			Y			PASS			
# [file normalize]		Y
# [file rename]
# [file runtype]
# [file type]
# [file unlink]
# [file writeable]
# [file dirname]		?							Truncates path
################################################################################
option echo on
option expression off

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Various tests #
/*
set test1 "\\\\server\\share\\q\$\\path"
set test2 [unescape "\\\\server\\share\\q\$\\path"]
vars test?
set q [unescape "\\"]							# FIX set will append literal \\
vars q
*/

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Testing escapes using UNC paths ..."
textcolor
newline

puts "Line: "#__LINE__
lib ticol_svc -verbose -nocomplain
newline
/*
puts "\[service workstation\]=[service workstation]"
if {> [service workstation] 2} {
	# 7    OpenSCManager failed     Could not open the SCM. Check access rights
	puts "Line: "#__LINE__
	textcolor red
	puts "Microsoft workstation service is not running. Not tested @line:"#__LINE__
	textcolor
	return
}

 # */

puts "Line: "#__LINE__
puts "\[item script_filename\]='[item script_filename]' @"#__LINE__
if {any [eqi [item script_filename] ""] [eqi [item script_filename] "internal.tcl"]} {		# Can't run from VB5 interface at present
	puts "Running in the IDE. No valid script filename"
	return
}

puts "Line: "#__LINE__
set self_path [file normalize .]					# Stores unescaped
append self_path [unescape "\\"]					# Fix appends literal \\

#puts "Escaped path is: '[escape $path]'"

puts "Line: "#__LINE__
set unc_path ""
set split_path [escape [file split $self_path]]		# Split into drive, path, filename
#puts "Debug: File split is: '$split_path'"

puts "Line: "#__LINE__
if {> [lcount split_path] 0} {
	set drive [left [lindex $split_path 0] 1]
	set path_only [string replace [lindex $split_path 1] "\\" "/" ]
	set hostname [hostname]
	set script_filename [file name [info script]]
}

puts "Line: "#__LINE__
if {eq $drive C -nocase} {
	set unc_path "//$hostname/$drive\$$path_only$script_filename"
	# //server/c$/path
	#vars unc*
	newline
	#puts "Debug: unc_path: $unc_path"

	puts "Line: "#__LINE__
	textcolor yellow
	puts "Reading self via UNC share as\r\n$unc_path ..."
	textcolor
	newline
	
	set r [readfile [unescape $unc_path] a -array]
	puts "Read $r line(s)"
	newline
	assert $r {> $_ 0} -v -line #__LINE__
	newline
	
	if {ne $unc_path ""} {
	
		textcolor yellow
		puts "Testing general commands using\r\n$unc_path ..."
		textcolor
		newline
		
		#vars 
		#puts "MD5?\t\t[file md5 [unescape $unc_path]]"
		puts "MD5?\t\t[file md5 $unc_path]"
		puts "isdirectory?\t[bool [file isdirectory $unc_path]]"
		assert [file isdirectory $unc_path] {== $_ 1} -v -line #__LINE__
		puts "isfile?\t\t[bool [file isfile $unc_path]]"
		assert [file isfile $unc_path] {== $_ 1} -v -line #__LINE__
		puts "Executable?\t[bool [file executable $unc_path]]"
		assert [file executable $unc_path] {== $_ 0} -v -line #__LINE__
		puts "Extension\t[file extension $unc_path]"
		# FIX could be tcl or tcx
		# string match
		assert [file extension $unc_path] {string match ".tc?" $_ -nocase} -v -line #__LINE__
		puts "Normalpath\t[escape [file normal_path $unc_path]]"
		puts "Exists?\t\t[bool [file exists $unc_path]]"
		assert [file exists $unc_path] {== $_ 1} -v -line #__LINE__
		puts "Readable?\t[bool [file readable $unc_path]]"
		assert [file readable $unc_path] {== $_ 1} -v -line #__LINE__
		puts "Attributes:\t'[file attributes $unc_path]'"		# Should probably escape the filename
		set q [file attributes $unc_path]
		vars q
		
		puts "Dirname: '[file dirname [file name $unc_path]]'"
		set fp [file open $unc_path]
		puts "Handle \$fp is '$fp'"
		set r [file gets $fp s 100]			# Read a line
		puts "\[file gets\] return r is '$r'"
		puts "binary data s is '$s"
		assert $s {ne $_ ""} -v -line #__LINE__
		file close $fp
	}
} else {
	puts "We are not hosted on drive C: - not testing"
}

puts "Unsetting @"#__LINE__
unset a fp q r s drive hostname normal_path path_only script_filename self_path split_path unc_path -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor
# EOF #