#######################################################################################
# File: 			test_ftoa.tcl
# Purpose:			Unit test for [ftoa] [atof]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 20/Mar/2022	First version
#######################################################################################
# Notes:
# [ftoa] requires normalisation. Remove trailing zeroes, remove superfulous exp zero prefix, set case of E 
# [ftoa] and [atof] export the "C" atof() function
# [ftoa] and [atof] are built-in. Use the ticol_sci.dll plugin as an alternate
# Derived from tests in test_lib_sci.tcl
#######################################################################################

option expression off

assert [atof 2e4] {== $_ 20000} -v -line #__LINE__
assert [atof 2E4] {== $_ 20000} -v -line #__LINE__
#check 2e+4
assert [atof 2e+4] {== $_ 20000} -v -line #__LINE__
assert [atof 2E+4] {== $_ 20000} -v -line #__LINE__
#check 2e-4
assert [atof 2e-4] {== $_ .0002} -v -line #__LINE__
assert [atof 2E-4] {== $_ .0002} -v -line #__LINE__
#check -2e4
assert [atof -2e4] {== $_ -20000} -v -line #__LINE__
assert [atof -2E4] {== $_ -20000} -v -line #__LINE__
#check -2e-4
assert [atof -2e-4] {== $_ -0.0002} -v -line #__LINE__
assert [atof -2E-4] {== $_ -0.0002} -v -line #__LINE__

assert [atof -2e-4] {== $_ -.0002} -v -line #__LINE__ # FAIL .0002

######################################################################################

# Will be in the format '2.000000e+000' NOT "2.0E+0"
assert [ftoa 2] {== $_ "2.000000000000000e+000"} -v -line #__LINE__
assert [atof [ftoa 2]] {== $_ 2} -v -line #__LINE__

# Will be in the format '2.000000e-001' not 2.0E-1
assert [ftoa 0.2] {== $_ "2.000000000000000e-001"} -v -line #__LINE__
assert [atof [ftoa 0.2]] {== $_ 0.2} -v -line #__LINE__

# Will be in the fornmat '-2.000000e-001' not -2.0E-1
assert [ftoa -0.2] {== $_ "-2.000000000000000e-001"} -v -line #__LINE__
assert [atof [ftoa -0.2]] {== $_ -0.2} -v -line #__LINE__

# Will be in the format '-2.000000e-003' not -2.0E-3
assert [ftoa -.002] {eq $_ "-2.000000000000000e-003"} -v -line #__LINE__
assert [atof [ftoa -.002]] {== $_ -0.002} -v -line #__LINE__

# End

assert [atof -2.000000e-03] {== $_ -.002} -v -line #__LINE__
assert [atof 2E24] {== $_ 2000000000000000000000000.000000} -v -line #__LINE__

# Will be in the format '-2.000000e-003' not -2.0E-3
assert [ftoa -.002] {eq $_ "-2.000000000000000e-003"} -v -line #__LINE__


# Checking a 'real world' value...
# Will be in the format '5.051090e+012' not "5.05109e+12"'
assert [ftoa 5051090000000] {eqi $_ "5.051090000000000e+012"} -v -line #__LINE__
assert [atof 5.05109e+12] {!= $_ 5051090000001} -v -line #__LINE__
assert [atof 5.05109e+12] {!= $_ 5051100000000} -v -line #__LINE__
assert [atof 5.05109e+12] {== $_ 5051090000000} -v -line #__LINE__
# Will be in the form '5.051090e+012' not 5.05109e+12'
assert [ftoa [atof 5.05109e+12]] {eq $_ 5.051090000000000e+012} -v -line #__LINE__
# Will be in the form '5.051090e+012' not 5.05109e+12'
assert [ftoa [atof 5.05109e+12]] {== $_ 5.051090000000000e+012} -v -line #__LINE__
#assert 5.05109E+12 {== $_ 5051090000000} -v -line #__LINE__
# Will be in the form '5.05109E+12' not 5.05109e+12'
assert "5.05109e+012" {== $_ 5.05109e+012} -v -line #__LINE__
assert "5.05109e+12" {!= $_ 5.05109} -v -line #__LINE__
assert [expr [atof 2e7]*3] {== $_ 60000000} -v -line #__LINE__

######################################################################################
/*
# FIX Formatting differences
# FIX 1,7 fails. At sero the sign shifts from - to + 0.000000e+000
#
# TEST A: 1,1: dec:'0.100000' scidec:'1.000000e-001' string:'1.0e-1'
# Loop test -ve exponent signs
# zero base gives +ve sign so can't test 0 for -ve
# i = 0.0E+0
# Note that loop i for base cannot exceed 9 since we always represent as X.Y<sign>ENN
TEST A: 1,1: atof:'0.100000' ftoaatof:'1.000000e-001' test:'1.000000e-001'
TEST A: 1,2: atof:'0.010000' ftoaatof:'1.000000e-002' test:'1.000000e-002'
TEST A: 1,3: atof:'0.001000' ftoaatof:'1.000000e-003' test:'1.000000e-003'
TEST A: 1,4: atof:'0.000100' ftoaatof:'1.000000e-004' test:'1.000000e-004'
TEST A: 1,5: atof:'0.000010' ftoaatof:'1.000000e-005' test:'1.000000e-005'
TEST A: 1,6: atof:'0.000001' ftoaatof:'1.000000e-006' test:'1.000000e-006'
TEST A: 1,7: atof:'0.000000' ftoaatof:'0.000000e+000' test:'1.000000e-007'  FAILS (precision overflow)
*/

/*
# We can run into rounding comparison issues here
# Assertion failed: '9.999999999999999e-012' != 'eq $_ "${i}.000000000000000e-[format %03i $::j]"'
loop i 1 10 2 {		## Loop 1 to 9
	loop j 1 16 {		# Bit shift Iters to hi-1
		# puts "TEST A: $i,$j: atof:'[atof ${i}e-$::j]' trip:'[ftoa [atof ${i}e-[format %03i $::j]]]' test:'${i}.000000e-[format %03i $::j]'"
		puts "Test i: $i,j: $j"
		assert [ftoa [atof ${i}e-$::j]] {eq $_ "${i}.000000000000000e-[format %03i $::j]"} -v -line #__LINE__
	}
}
# */

# Will be in the format 0.000000e+000' not 0.0e+0
assert [ftoa [atof 0.000000e0]] {eqi $_ 0.000000000000000e+000} -v -line #__LINE__
# Will be in the format '1.000000e+000' not 1.0e+0'
assert [ftoa [atof 1.0e0]] {eqi $_ 1.000000000000000e+000} -v -line #__LINE__
# Will be in the format '1.000000e+000' not 1.0e+0'
assert [ftoa [atof 1.0e-0]] {eqi $_ 1.000000000000000e+000} -v -line #__LINE__		# Sign remains +ve
# Will be in the format '1.000000e-001' not 1.0e-1'
assert [ftoa [atof 1.0e-1]] {eqi $_ 1.000000000000000e-001} -v -line #__LINE__
# Will be in the format '1.000000e+001' != 'eqi $_ 1.0e+1'
assert [ftoa [atof 1.0e+1]] {eqi $_ 1.000000000000000e+001} -v -line #__LINE__
# Will be in the format '1.100000e+001' != 'eqi $_ 1.1e+1'
assert [ftoa [atof 1.1e+1]] {eqi $_ 1.100000000000000e+001} -v -line #__LINE__
# Will be in the format '1.100000e-001' != 'eqi $_ 1.1e-1'
assert [ftoa [atof 1.1e-1]] {eqi $_ 1.100000000000000e-001} -v -line #__LINE__


# Input values will be shifted and normalised to single-digit values ...
# Will be in MSVC format 1.210000e+002' not 1.21e+2'
puts "[ftoa 12.1e+1] [atof [ftoa 12.100000e+001]]"
assert [ftoa [atof 12.1e+1]] {eqi $_ 1.210000000000000e+002} -v -line #__LINE__
puts "[ftoa 12.1e-1] [atof [ftoa 12.1e-1]]"
assert [ftoa [atof 12.1e-1]] {eqi $_ 1.210000000000000e+000} -v -line #__LINE__	
puts "[ftoa 3887.1e+1] [atof [ftoa 3887.1e+1]]"	# 38871
assert [ftoa [atof 3887.1e+1]] {eqi $_ 3.887100000000000e+004} -v -line #__LINE__
puts "[ftoa 3887.1e-1] [atof [ftoa 3887.1e-1]]"	# 38871
assert [ftoa [atof 3887.1e-1]] {eqi $_ 3.887100000000000e+002} -v -line #__LINE__
puts "[ftoa 8283887.1e+5] [atof [ftoa 8283887.1e-5]]"	# 38871
assert [ftoa [atof 8283887.1e+5]] {eqi $_ 8.283887100000000e+011} -v -line #__LINE__


unset i j -nocomplain

#######################################################################################

vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #