######################################################################################
# File:			test_glob.tcl
# Author:		M Shaw
# Purpose:		Test [glob] file search
# Language:		Ticol Tcl
#
# Versions:
# 1.00 17/Jul/2018	First version
# 1.01 01/Aug/2018	Check for presence of ticol.exe 
######################################################################################
# Notes:
# Some very basic tests on [glob]
# We can't know much about the structure of the current dir during tests other than
# that ticol.exe should be present (if not renamed)
# We could use $exe_name
######################################################################################
option echo on
option expression off

newline
textcolor yellow
puts "Test \[glob\] file search...\n"
textcolor

set exename [info executablename]
if {! [file exists $exename]} {		# Can't think how this would happen, but...
	die "I can't find ticol.exe"
}

set path [file dirname $exename]
set exename [file name $exename]
addslash path -var

puts "exename:'$exename'"
puts "path:   '$path'"

if {eq $::exename ticol.exe} {
	puts "\[glob -type f ${path}ticol.exe\]='[glob -type f ${path}ticol.exe]'"
	assert [string right [glob -type f ${path}ticol.exe] 9] {eq $_ $::exename} -v -line #__LINE__
	puts "\[glob -type f ${path}ti?ol.exe\]='[glob -type f ${path}ticol.exe]'"
	assert [string right [glob -type f ${path}ti?ol.exe] 9] {eq $_ $::exename} -v -line #__LINE__
	puts "\[glob -type f ${path}ti*l.exe\]='[glob -type f ${path}ticol.exe]'"
	assert [string right [glob -type f ${path}ti*ol.exe] 9] {eq $_ $::exename} -v -line #__LINE__
}
set path [file dirname [info executablename]]
#addslash path -var
#set path [escape $path]
# FIX glob path not working, searching current dir
puts "\[glob -type f -directory $path t*.exe\]='[glob -type f -directory $path t*.exe]'"
# FIX won't work for embedded Ticol
# assert [glob -type f -directory $path ticol.exe] {eq $_ "ticol.exe"} -v -line #__LINE__

unset exename path -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #