############################################################################################
# File: 			test_global.tcl
# Purpose:			Test [global]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 24/Jul/2019	First version
############################################################################################
# Notes: 
# The behaviour of [global] may not precisely match that of ActiveState Tcl, 
# particularly as we don't have real alternate namespaces. We just emulate them
# https://www.tcl.tk/man/tcl8.7/TclCmd/global.htm
############################################################################################
# Todo:
# Expand tests a little bit more
# DONE FIX If varname contains namespace qualifiers, the local variable's name is the 
# unqualified name of the global variable, as determined by the namespace tail command.
############################################################################################

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Multi args (no -value)
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Testing \[global\] ..."
newline
textcolor

undef foo -nocomplain
proc foo {} {
	global a b c 1 2 3			# Will create new vars ::a::x and x etc.
	vars
}
foo

# exit


puts "Line: "#__LINE__
unset a b c -nocomplain
undef foo -nocomplain
proc foo {} {
	global a b c			# Will create new vars ::a::x and x etc.
	vars -level 0
	set a 123
	set b 456
	set c 789
	assert $a  {== $_ 123} -v -line #__LINE__
	assert $::a  {== $_ 123} -v -line #__LINE__
	assert $b  {== $_ 456} -v -line #__LINE__
	assert $::b  {== $_ 456} -v -line #__LINE__
	assert $c  {== $_ 789} -v -line #__LINE__
	assert $::c  {== $_ 789} -v -line #__LINE__
	vars
}

foo
puts "Line: "#__LINE__
assert $a  {== $_ 123} -v -line #__LINE__
assert $b  {== $_ 456} -v -line #__LINE__
assert $c  {== $_ 789} -v -line #__LINE__


puts "Line: "#__LINE__
undef foo -nocomplain
vars
unset a b c 1 2 3 -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Multi args (with -value)
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Line: "#__LINE__
newline
textcolor

unset a b c -nocomplain
proc foo {} {
	global a b c -value 4567			# Will create new vars ::a::x and x etc. and set all to 4567
	vars -level 0
	assert $a  {== $_ 4567} -v -line #__LINE__
	assert $::a  {== $_ 4567} -v -line #__LINE__
	assert $b  {== $_ 4567} -v -line #__LINE__
	assert $::b  {== $_ 4567} -v -line #__LINE__
	assert $c  {== $_ 4567} -v -line #__LINE__
	assert $::c  {== $_ 4567} -v -line #__LINE__
	vars
}

foo
undef foo -nocomplain
assert $a  {== $_ 4567} -v -line #__LINE__
assert $b  {== $_ 4567} -v -line #__LINE__
assert $c  {== $_ 4567} -v -line #__LINE__
vars
unset a b c -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Line: "#__LINE__
newline
textcolor

undef foo -nocomplain
proc foo {} {
	global bar		# Create global bar
	set bar 12345
}

foo
undef foo -nocomplain
assert $bar {== $_ 12345} -v -line #__LINE__
unset bar


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Map to pre-existing global without destroying it
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

textcolor yellow
puts "Line: "#__LINE__
newline
textcolor

set bar "exists"			# Pre-create
undef foo -nocomplain
proc foo {} {
	global bar		# Create global bar
	assert $bar {eq $_ exists} -v -line #__LINE__
	assert $::bar {eq $_ exists} -v -line #__LINE__
	set bar 12345
}

foo
undef foo -nocomplain
assert $bar {== $_ 12345} -v -line #__LINE__
unset bar

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

textcolor yellow
puts "Line: "#__LINE__
newline
textcolor

set bar 45678
proc foo {} {
	global bar
	set bar 67891
	assert $bar {== $_ 67891} -v -line #__LINE__
	assert $::bar {== $_ 67891} -v -line #__LINE__
}

# Check pre-call
assert $bar {== $_ 45678} -v -line #__LINE__
foo
undef foo -nocomplain
assert $bar {== $_ 67891} -v -line #__LINE__
unset bar

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Line: "#__LINE__
newline
textcolor

unset a::x -nocomplain

# This procedure sets the *emulated* namespace variable ::a::x
proc foo {} {
	global a::x			# Will create ::a::x and x
	set x 98765			# ::a::x will be set to x on proc exit as the proc unwinds
	assert $::a::x  {== $_ 98765} -v -line #__LINE__
	assert $x  {== $_ 98765} -v -line #__LINE__
	vars
}

foo
undef foo -nocomplain
vars
assert $a::x {== $_ 98765} -v -line #__LINE__
unset a::x



# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Line: "#__LINE__
newline
textcolor

newline
textcolor red
puts "Compare with \[upvar\] ..."
textcolor

proc foo {} {
	upvar a::x x				# global should work like this
	set x 476287
	assert $x  {== $_ 476287} -v -line #__LINE__
	assert $::a::x  {== $_ 476287} -v -line #__LINE__
	vars 
}
foo
undef foo -nocomplain
vars
assert $a::x {== $_ 476287} -v -line #__LINE__
unset a::x

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Cleanup
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor yellow
puts "Cleanup ... @"#__LINE__
textcolor

textcolor yellow
puts "Known procs: '[procs]' @"#__LINE__
textcolor

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #