######################################################################################
# File: 			test_gosub.tcl
# Purpose:			Test [gosub_block] and [gosub] (and amusement)
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 12/Dec/2017	First version
# 1.01 06/Sep/2018	Proc cleanup
######################################################################################
# Notes: 
# Syntax:  [gosub_block ?label? {code}]
# where {code is}
# ?gosub label? ...
# label {code} ...
# [static] is supported within {code}
# gosub within {code} requires a defined scope
######################################################################################
# goto/gosub tests - full list:
# test_gosub.tcl
# test_goto.tcl
# test_goto_block.tcl
# test_goto_gosub.tcl
# test_goto_return.tcl
######################################################################################
# One or more subroutines may be defined only in a [gosub_block]
# A sub (a gosub routine) is a procedure which has no arguments and returns no value
# A sub has global scope and global lifespan
# Any [return] command within a sub is discarded
# Labels are not globally unique across [gosub_block]s at present
# A subroutine can be called only by using [gosub]
# [gosub] may be used internally within a [gosub_block] or globally in code
# Calling a sub recursively will result in a fatal stack-overflow
# The only valid commands in a [gosub_block] are [goto label] and a sub definition
# A [gosub_block] may have an optional scope prefix name. If this is defined then
# gosub can call local labels from outside the [gosub_block], otherwise only inline
# gosub commands within the block can call labels
#
# These 2 commands are completely useless and have been implemented for amusement only
#
# Gosub routines as temp registered procs
# Need to be made anonymous and no scope clash as will be in same interpreter scope
######################################################################################
# FIX
# Make the scope name mandatory?
######################################################################################

option echo on
option expression off

textcolor yellow
puts "Testing \[gosub_block\] and \[gosub\] ..."
textcolor
newline

#/*  # MAN EG *************************************************************
	sub label1 {
		static counter
		++ counter
		puts "I am label 1. Counter is $counter"
		return					# return is optional and values discarded
	}

	sub label2 {
		static counter
		++ counter
		puts "I am label 2. Counter is $counter"
		gosub label1			# Call another sub. Scope prefix required
		return					# return is optional and values discarded
	}

	gosub label1
	gosub label2
	gosub label1
	vars
# ************************************************************************/

# MARK 1
# gosub_block not implemented

/*
gosub_block foo {
	gosub {
		label1
	}
	gosub label2
	
	label1 {
		textcolor magenta
		puts 1:Hello
		#textcolor cyan
		#puts "procs->[procs]"
		#textcolor
		gosub foo::label3
		return 23				# return values are ignored
	}

	label2 {
		textcolor yellow
		puts 2:world
		textcolor
		puts "2:Exit label2"
		return					# Returns are ignored and do not exit a [gosub_block]
	}

	label3 {
		static counter
		++ counter
		textcolor green
		puts "3:I am label3 and static counter is $counter"
		textcolor
		gosub foo::label4
		puts "3:I am still in label3 but about to exit"
		newline
		return
	}

	label4 {
		textcolor cyan
		puts "4:I am label4 (no return)"
		textcolor
	}

	stack_overflow {			# Will stack-overflow!
		puts "stack_overflow: looping..."
		gosub loop
	}
}

gosub_block bar {
	label1 {
		puts "I am label1 in in block bar. I will now call foo::label2"
		gosub foo::label2
		return
	}
	
	empty {
	}
} # ####################################################### */




sub foo::label1 {
	textcolor magenta
	puts 1:Hello
	#textcolor cyan
	#puts "procs->[procs]"
	#textcolor
	gosub foo::label3
	return 23					# return values are ignored
}

sub foo::label2 {
	textcolor yellow
	puts 2:world
	textcolor
	puts "2:Exit label2"
	return						# Returns are ignored and do not exit a [gosub_block]
}

sub foo::label3 {
	static counter
	++ counter
	textcolor green
	puts "3:I am label3 and static counter is $counter"
	textcolor
	gosub foo::label4
	puts "3:I am still in label3 but about to exit"
	newline
	return
}

sub foo::label4 {
	textcolor cyan
	puts "4:I am label4 (no return)"
	textcolor
}

sub stack_overflow {			# Will stack-overflow!
	puts "stack_overflow: looping..."
	gosub loop
}

sub bar::empty {
}

newline
textcolor yellow
puts "Testing \[sub\]..."
textcolor

newline
textcolor yellow
puts "Calling label3 externally ..."
textcolor
newline

# FIX visibility
gosub foo::label1
gosub foo::label3 Hello World	# Any args will be ignored

/**************************************************************************
# gosub_block not implemented
gosub_block moo {
	gosub label1				# Simple call of label1 in local scope
								# Scope name is not required here
	gosub label2				# Call label2 in local scope
								# Scope name is not required here
	label1 {
		static counter
		++ counter
		puts "I am label 1. Counter is $counter"
		return					# return is optional and values discarded
	}
	label2 {
		static counter
		++ counter
		puts "I am label 2. Counter is $counter"
		gosub moo::label1		# Call another sub. Scope prefix required
		return					# return is optional and values discarded
	}
} *************************************************************************/

sub moo::label1 {
	static counter
	++ counter
	puts "I am label 1. Counter is $counter"
	return               		# return is optional and values discarded
}

sub moo::label2 {
	static counter
	++ counter
	puts "I am label 2. Counter is $counter"
	gosub moo::label1			# Call another sub. Scope prefix required
	return						# return is optional and values discarded
}


gosub moo::label1				# Call from within global scope


newline
textcolor yellow
puts "Calling label directly"
textcolor
moo::label1
#stop

# 2017/Dec/13 - No leaks
/****************************************
set start_time [timer]
time {
	gosub bar::empty			# 2 microsecond(s) per iteration
} 10000000
puts "Took [- [timer] $start_time]"	# **/

#gosub stack_overflow
puts "Line: "#__LINE__
unset this -nocomplain
static foo::label3::counter -unset 
static moo::label1::counter -unset 
static label1::counter -unset
static label2::counter -unset

undef bar::empty foo::label1 foo::label2 foo::label3 foo::label4 label1 label2 moo::label1 moo::label2 stack_overflow -nocomplain

newline
vars
puts [procs]

######################################################################################

newline
textcolor green
puts "* Done *"
textcolor

# EOF #