########################################################
# File:				goto_block.tcl
# Author:			M Shaw
# Purpose:			Test file for the goto_block
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 19/Dec/2016	First version 
# 1.01 22/Jun/2017	Replace [checkex] with [assert]
# 1.02 06/Sep/2018	Add tests for $this
########################################################
# Notes:
# goto/gosub tests - full list:
# test_gosub.tcl
# test_goto.tcl
# test_goto_block.tcl
# test_goto_gosub.tcl
# test_goto_return.tcl
########################################################
option echo on
option expression off

# Check catch-handler...
catch {stop}		# Should not exit
catch {exit}		# Should not exit
catch {break}		# Should not exit
catch {continue}	# Should not exit
catch {return}		# Should not exit

textcolor yellow
puts "Testing goto_block/goto exit cases ..."
textcolor
newline

# Test BREAK exit
textcolor yellow
puts "Test goto_block + break"
textcolor
set i 0
goto_block {
	0 { 
		set i 1
		goto 10
	}
	
	10 {
		set i 2
		puts "At label 10 (break)"
		break	# PASS
		#stop	# Fails
		#return	# PASS
		# exit	# FAIL
	}

	end {
		set i 5
		puts "At label END (break)"
	}
}
assert $i {== $_ 2} -v -line #__LINE__
newline


# Test RETURN exit
puts "Line: "#__LINE__
textcolor yellow
puts "Test goto_block + return"
textcolor
set i 0
goto_block {
	0 { 
		set i 1
		goto 10
	}
	
	10 {
		set i 2
		puts "At label 10 (return)"
		return -code break	# PASS	10/Jan/2017 no longer works in goto unles using -code break
	}

	end {
		set i 3
		puts "At label END (return)"
	}
}

assert $i {== $_ 2} -v -line #__LINE__
newline

# Test CONTINUE exit
puts "Line: "#__LINE__
textcolor yellow
puts "Test goto_block + continue"
textcolor
set i 0
goto_block {
	0 { 
		set i 1
		goto 10
	}
	
	10 {
		set i 2
		puts "At label 10 (continue)"
		continue	# Should have no effect
	}

	end {
		set i 3
		puts "At label END (continue)"
	}
}

assert $i {== $_ 3} -v -line #__LINE__
newline

puts "Line: "#__LINE__
catch {
	# Test STOP exit (catch is required)
	textcolor yellow
	puts "Test goto_block + stop"
	textcolor
	set i 0
	goto_block {
		0 { 
			set i 1
			goto 10
		}
		
		10 {
			set i 2
			puts "At label 10 (stop)"
			#break	# PASS
			stop	# Fails ?
			#return	# PASS
			# exit	# FAIL
		}

		end {
			set i 3
			puts "At label END (stop)"
		}
	}
	assert $i {== $_ 2} -v -line #__LINE__
}
newline

puts "Line: "#__LINE__
catch {
	# Test EXIT using [exit] a catch is required
	textcolor yellow
	puts "Test goto_block + exit"
	textcolor
	set i 0
	goto_block {
		0 { 
			set i 1
			goto 10
		}
		
		10 {
			set i 2
			puts "At label 10 (exit)"
			exit	# Should exit script
		}

		end {
			set i 3
			puts "At label END (exit)"
		}
	}
	assert $i {== $_ 3} -v -line #__LINE__
}
newline

# Test child/grandchild goto_block #

puts "Line: "#__LINE__
textcolor yellow
puts "Test nested goto_blocks"
textcolor
newline

set i 0
goto_block {
	label1 {
		+= i 1
		puts "In parent goto_block label1"
		goto_block {
			label1 {
				+= i 2
				puts "In child goto_block label1"
				goto_block {
					goto label2
					label1 {
						+= i 4
						puts "In grandchild goto_block label1"
					}
					label break
					label3 {
						+= i 32
					}
					label2 {
						+= i 8
						puts "In grandchild goto_block label2"
						goto label1
					}
				}
			}
			label2 {
				+= i 16
				puts "In child goto_block label2"
			}
		}
	}
}

assert $i {== $_ 31} -v -line #__LINE__

newline
puts "Line :"#__LINE__
goto_block {
	goto skip
	one {
		textcolor darkyellow
		puts "* In $this"
		textcolor
		assert $this {eq $_ one} -v -line #__LINE__
		return -code ok
	}
	two {
		textcolor magenta
		puts "* In $this"
		textcolor
		assert $this {eq $_ two} -v -line #__LINE__
		return -code ok
	}
	skip {}			# Empty label
	gosub two
	gosub one
}

puts "Line :"#__LINE__

unset i this -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #