######################################################################################
# File: 			test_goto_gosub.tcl
# Purpose:			Test modifications to [goto_block] to add [gosub]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 16/Dec/2017	First version
# 1.01 06/Sep/2018	Cleanup procs, added $this
######################################################################################
# Notes:
# Syntactic sugar (nom nom nom)
# Simple commands end up complex!
# Needs more tests
# [gosub] behaviour differs in root to that in script argument
# In root it will [gosub label], in script it will [gosub sub]
######################################################################################
# goto/gosub tests - full list:
# test_gosub.tcl
# test_goto.tcl
# test_goto_block.tcl
# test_goto_gosub.tcl
# test_goto_return.tcl
######################################################################################

option expression off
option echo on
option stacktrace push on

#define test1
#define test2
#define test3
#define test4
#define test5
#define test6

# man file example #
#ifdef test1
set i 0
puts "Test 1 -----------------------------------------"
undef external -nocomplain
sub external {
	upvar i
	++ i 2
	puts "In external sub"
}

goto_block {
	gosub label_two
	gosub label_two
	gosub label_one
	gosub label_two
	# We should probably allow this...
	# break
	label_zero {break}
	label_one {
		++ i 1
		puts "In yr label_one - calling external"
		gosub external
	}
	label_two {
		++ i 3
		puts "In yr label2"
	}
}

puts "Exited test 1"
assert $i {== $_ 12} -v -line #__LINE__
newline
undef external -nocomplain
newline
#endif


#ifdef test2
puts "Test 2 -----------------------------------------"
set i 0
goto_block {
	label1 { 
		++ i 2
		puts Hello
	}
	#label2 { break }         # Will cause the goto_block to exit
	label2 {}
	label3 {
		++ i 3
		puts world
	}
}
puts "Exited test 2"
assert $i {== $_ 5} -v -line #__LINE__
newline
# undef external -nocomplain
#endif

#/*
# Call			Stack
# -----------------------
# init
# one			one
#	two
#	three		two, one
#	end
# two -> three, end
# one
# -----------------------

#ifdef test3
puts "Test 3 -----------------------------------------"
set i 0
goto_block {
	init {
		++ i 1
		textcolor cyan
		puts "initing"
		textcolor
	}
	#goto one
	gosub three
	quit { 
		puts "* Breaking *"
		break
	}
	one {
		++ i 2
		textcolor magenta
		puts "In label one:"
		textcolor
		goto two
	}
	two {
		++ i 3
		textcolor yellow
		puts "In label two:"
		textcolor
		goto three
	}
	three {
		++ i 5
		textcolor green
		puts "In label three:"
		textcolor
		goto end
	}
	end {
		++ i 7
		textcolor red
		puts "label end:"
		textcolor
	}
} # */
puts "Exited test 3"
newline
assert $i {== $_ 13} -v -line #__LINE__
undef external -nocomplain
#endif

#ifdef test4
puts "Test 4 -----------------------------------------"
set i 0
goto_block {
	# Executes: two, one, then falls through to one, two == 6
	gosub two
	gosub one
	one {
		++ i 1
		textcolor yellow
		puts "In label one:"
		textcolor
	}
	two {
		++ i 2
		textcolor yellow
		puts "In label two:"
		textcolor
	}
}
assert $i {== $_ 6} -v -line #__LINE__
newline
undef external -nocomplain
puts "Exited test 4"
#endif



#ifdef test5
puts "Test 5 -----------------------------------------"
set i 0
goto_block {
	init {
		sub external {
			upvar i
			++ i 3
			textcolor green
			puts "In external sub, defined locally (3)"
			textcolor
		}
		textcolor cyan
		puts "initing"
		textcolor
	}
	gosub two
	gosub one
	#zero break			# Pass - Will exit ok
	#zero return		# Pass - Will exit ok
	one {
		++ i 1
		textcolor magenta
		puts "In label one - I will exit if not called from gosub"
		gosub external
		textcolor
		return			# Will exit if not called from gosub
	}
	two {
		++ i 2
		textcolor magenta
		puts "In label two - I will exit if not called from gosub"
		textcolor
		return			# Will return if called from gosub but not mandatory
	}
}
puts "Exited test 5"
newline
assert $i {== $_ 10} -v -line #__LINE__
undef external -nocomplain
#endif


unset i this -nocomplain

option stacktrace pop

vars

puts "Procedures [procs]"

# # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Done *"
textcolor

# EOF #