###############################################################
# File: 			goto_return.tcl
# Purpose:			Test file return calls in goto_block
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 09/Jan/2017	First version
###############################################################
# Notes:
# goto/gosub tests - full list:
# test_gosub.tcl
# test_goto.tcl
# test_goto_block.tcl
# test_goto_gosub.tcl
# test_goto_return.tcl
###############################################################

textcolor yellow
puts "Testing \[goto_block\] \[return\] ..."
textcolor
newline

# Validate [return] exits [goto_block]
puts "Line: "#__LINE__
set i 0
goto_block {
	0 {
		set i 1
		goto start
	}							# Not strictly necessary
	
	start {
		textcolor red
		puts "In block START (You should not see \"Exiting START\")"
		textcolor
		set i 2
		return 10				# Fails to exit goto_block but will exit the current label
		#break					# Will exit OK
		#return 10 -code break	# Will exit OK
		puts "Exiting START (I should not be visible)"
		set i 3
	}

	end {
		puts "In block END"
		set i 4
	}
}
newline
puts "Now after goto_block at line "#__LINE__
assert $i {== $_ 2} -v -line #__LINE__			# Validate that [return] exits [goto_block] completely

# Man example
# Check that [return] exits goto_block from [while] loop, but not the script
option expression off
puts "Line: "#__LINE__
set i 0
goto_block {
	start {
		while {< $i 10} {
			if {== $i 5} {return $i}
			incr i
		}
	}
}
puts "i is $i"
assert $i {== $_ 5} -v -line #__LINE__			# 


puts "Line: "#__LINE__
unset i this -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor green
newline
puts "* \[goto_block\] \[return\] Done *"
textcolor

# EOF #