###############################################################################################
# File:				test_http_html.tcl
# Author:			M Shaw
# Purpose:			Test file for HTTP and HTML commands (not a unit test)
# Language:			Ticol Tcl v1.22+, v1.26+
#
# Versions:
# 1.00 23/Mar/2017	Migrated from tests.tcl
# 1.01 31/Jan/2022	Updated re: changes in syntax etc.
###############################################################################################
# Notes:
# Cannot be part of the standard tests
###############################################################################################

option expression off

###############################################################################################
/* # Deprecated: use [assert]
if {! [defined checkex] } {
	proc checkex {x expr} {
		#puts "DEBUG: checkex \$x=$x \$expr=$expr"
		upvar i					# Pass-through reference integer 
		upvar line
		#	puts "Trace $line"
		#	upvar NAN NAN		# Global const is N/A (FIX)
		#set line 0
		set NAN $::NAN 			# "-1.[chr 35]IND00000000000"
		# fix puts "\$x==$x: $expr"
		#set q {$x}
		set q \$x
		puts "Line: $line " -nonewline
		puts "'$q'=='$x': expr '$expr'"	# fails here at EOF - poss (due to bug in callproc()?) - July 2016
		#puts "eval expr: [eval $expr]"
		if { [eval $expr] } {
			#puts "Expr true"
		} else {
			textcolor red
			puts "checkex assertion failed: $q -> $x after line $line"
			textcolor
			stop
		}
	}
}
###############################################################################################

if {! [defined check] } {
	proc check {x y} {
		upvar line
		if {!= $x $y} {
			puts "Error in tests at after line $line"
			puts "'$x' != '$y'"
			stop
		}
	}
}
# */
###############################################################################################
set website http://www.google.com
set website http://localhost.com:8800

textcolor yellow
puts "Testing HTTP and HTML commands ...."
textcolor
newline

textcolor cyan
lib ticol_html -verbose
textcolor

set line #__LINE__
puts "Line: $line"

# commented out #
#define comment
#ifdef comment ////////////////////////////////////////////////////////////////

#lib ticol_http

# http_get
set line #__LINE__
textcolor yellow
puts "Checking \[http\]"
textcolor

set q {}
puts "Connecting to $website ..."
# Cannot use q inside checkex call as it will be in proc scope
# Instead set and chain the return
assert [set q [http $website -]] {ne $_ ""} -v -line #__LINE__
newline

puts "Successfully read website..."
newline

# html_header_field
set line #__LINE__
textcolor yellow
puts "Checking \[html header_field\]"
textcolor
puts "Header; date: '[html header_field $q "Date:"]'"
assert [html header_field $q "Date:"] {ne $_ ""} -v -line #__LINE__
newline

# html header
set line #__LINE__
textcolor yellow
puts "Checking \[html header\]"
textcolor magenta
puts "Header: [html header $q]"
textcolor
assert [html header $q] {ne $_ ""} -v -line #__LINE__
newline

# html remove_header
set line #__LINE__
textcolor yellow
puts "Checking \[html remove_header\]"
textcolor
set pb [html remove_header $q] 
assert $pb {ne $_ ""} -v -line #__LINE__
assert [string left $pb 6] {eqi $_ "<html>"} -v -line #__LINE__
puts "[string left $pb 40]..."
newline

lib ticol_html -unload

#endif ///////////////////////////////////////////////////////////////////////


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* PASS *"
textcolor

# EOF #