######################################################################################
# File: 			test_if.tcl
# Purpose:			Test file for [if]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.19+
#
# Versions:
# 1.00 16/Aug/2016	test_if
# 1.01 20/Jul/2017	[checkex] replaced with [assert]
######################################################################################
# Notes: 
# Note that as with C/C++ a return from a root level if ladder will exit the program
# Thus, to test, we have to use [return -code ok]
# ADD test bad syntax
######################################################################################
set i 0

option echo on
option expression off

textcolor yellow
puts "Test cases for \[if\] ...\n"
textcolor

textcolor yellow
puts "Testing \[if/elseif/else\] ..."
textcolor
option expression off
set sum 0
for {set i 0} {< $i 6} {incr i} {
	if {== $i 0} { 
		puts "if $i"
		set sum [+ $sum [+ $i 1]]	# 1
	} elseif {== $i 1} { 
		puts "elseif $i"
		set sum [+ $sum [+ $i 1]]	# 2, 3
	} elseif {== $i 2} { 
		puts "elseif $i"
		set sum [+ $sum [+ $i 1]]	# 3, 6
	} elseif {== $i 3} { 
		puts "elseif $i"
		set sum [+ $sum [+ $i 1]]	# 4, 10
	} elseif {== $i 4} {
		puts "elseif $i"
		set sum [+ $sum [+ $i 1]]	# 5, 15
	} else {
		puts "else $i"
		set sum [+ $sum [+ $i 1]]	# 6, 21
	}	
}
#puts "Sum is $sum"	# 21
assert $sum {== $_ 21}


# FIX look again at return code translation from if
# Hard-coded if ladders #
assert [if {true}  {return 23 -code ok} else {return 1 -code ok}] {== $_ 23} -v -line #__LINE_
assert [if {false}  {return 23 -code ok} else {return 1 -code ok}] {== $_ 1} -v -line #__LINE_
assert [if {false}  {return 23 -code ok} elseif {true} {return 1 -code ok} else {return 2 -code ok}] {== $_ 1} -v -line #__LINE_
assert [if {false}  {return 23 -code ok} elseif {false} {return 1 -code ok} else {return 2 -code ok}] {== $_ 2} -v -line #__LINE_

# Use 1 / 0 instead of true/false #
assert [if {1}  {return 23 -code ok} else {return 1 -code ok}] {== $_ 23} -v -line #__LINE_
assert [if {0}  {return 23 -code ok} else {return 1 -code ok}] {== $_ 1} -v -line #__LINE_
assert [if {0}  {return 23 -code ok} elseif {true} {return 1 -code ok} else {return 2 -code ok}] {== $_ 1} -v -line #__LINE_
assert [if {0}  {return 23 -code ok} elseif {0} {return 1 -code ok} else {return 2 -code ok}] {== $_ 2} -v -line #__LINE_

# Multiline #
assert [if {false}  {
	return 23 -code ok
} else {
	return 1 -code ok
}] {== $_ 1} -v -line #__LINE_

/*
# Hard-coded if ladders - no return code #
# We must use -code ok
# IF
assert [if {true}  {return 23} else {return 1}] {== $_ 23} -v -line #__LINE_
# ELSE
assert [if {false}  {return 23} else {return 1}] {== $_ 1} -v -line #__LINE_
# ELSEIF
assert [if {false}  {return 23} elseif {true} {return 1} else {return 2}] {== $_ 1} -v -line #__LINE_
# DEFAULT ELSE
assert [if {false}  {return 23} elseif {false} {return 1} else {return 2}] {== $_ 2} -v -line #__LINE_
*/

## BAD SYNTAX ##
# subst fred for else
if {! [catch {if {0} {puts "Should not appear"} fred {puts "This should appear at line $line"}}] } {
	die "Test failed at line $line"
}

# subst fred for else
if {! [catch {if {0} {puts "Should not appear"} elseif {0} {puts "Should not appear either"} fred {puts "This should appear at line $line"}}] } {
	die "Test failed at line $line"
}

# Reversed else/elseif
if {! [catch {if {0} {puts "Should not appear"} else {0} {puts "Should not appear either"} elseif {1} {puts "This should appear at line $line"}}] } {
	die "Test failed at line $line"
}

# test catch wrong arg count (omiting [else])
textcolor red
puts "Ignore any errors..."
textcolor
if {! [catch {if {0} {puts "Should not appear"} {puts "Should not appear either"}}] } {
	die "Test failed at line $line"
}
newline

textcolor red
puts "Ignore any errors..."
textcolor
# catch wrong arg count (args)
if {! [catch {if {0} }] } {
	die "Test failed at line $line"
}
newline

# Some code discussed at: http://wiki.tcl.tk/1042
option expression push on
set foo 1

# if $foo==1			{puts PASS} ;#(1)
# if "$foo == 1"		{puts PASS} ;#(2)
# if {$foo == 1}		{puts PASS} ;#(3)
# if {"" == ""}		{puts PASS} ;#(4)
# if {""==""}		{puts PASS} ;# Not valid, spaces reqired
# if {"" eq ""}		{puts PASS} ;#(5)

assert [if {"$foo == 1"} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_
assert [if {$foo == 1} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_
assert [if {"" == ""} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_

assert [if {"" eq ""} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_
assert [if {"x" eq ""} {return 1} else {return 0}] {== $_ 0} -v -line #__LINE_
assert [if {"" eq "x"} {return 1} else {return 0}] {== $_ 0} -v -line #__LINE_
assert [if {"x" eq "x"} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_

option expression pop

option expression push off
if {== $foo 1}		{puts PASS} ;#(1)
if {== "" ""}		{puts PASS} ;#(2)
if {eq "" ""}		{puts PASS} ;#(3)

assert [if {== $foo 1} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_
assert [if {== 1 $foo} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_
assert [if {== "" ""} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_
assert [if {== $foo ""} {return 1} else {return 0}] {== $_ 0} -v -line #__LINE_
assert [if {== "" $foo} {return 1} else {return 0}] {== $_ 0} -v -line #__LINE_
assert [if {== $foo $foo} {return 1} else {return 0}] {== $_ 1} -v -line #__LINE_


option expression pop

unset foo i sum -nocomplain

vars

##########################################
newline
textcolor green
puts "* \[if\] Done *"
textcolor

# EOF #