######################################################################################
# File: 			test_in_ni.tcl
# Purpose:			Tests for [in] and [ni]
# Language:			Ticol Tcl v1.26+
# Author:			M Shaw
#
# Versions:
# 1.00 06/Nov/2017	First version
# 1.01 18/Nov/2017	Rewrote [in], [ni] reversed arguments to [in|ni list value]
# 1.02 25/Jun/2018	Bugfix re "" list fixed, updated tests to add -nocase, -glob
######################################################################################
# Notes: 
# 	bool [in list value ?-glob|-nocase?]
#	bool [ni list value ?-glob|-nocase?]
######################################################################################

set line #__LINE__

option echo on
option expression off

textcolor yellow
puts "Testing \[in\] ..."
textcolor
newline

set l "draw_logo level_p level_q level_r scroll_titles"

assert [in $l level_p] {== $_ 1} -v -line #__LINE__
assert [in $l LEVEL_P -nocase] {== $_ 1} -v -line #__LINE__
assert [in $l l*_p -glob] {== $_ 1} -v -line #__LINE__
assert [in $l l????_p -glob] {== $_ 1} -v -line #__LINE__
assert [in $l level_q] {== $_ 1} -v -line #__LINE__
assert [in $l level_Q -nocase] {== $_ 1} -v -line #__LINE__
assert [in $l level_r] {== $_ 1} -v -line #__LINE__
assert [in $l Level_r -nocase] {== $_ 1} -v -line #__LINE__

assert [in $l foo] {== $_ 0} -v -line #__LINE__
assert [in $l foo -nocase] {== $_ 0} -v -line #__LINE__
assert [in $l f* -glob] {== $_ 0} -v -line #__LINE__

assert [lsearch $l ""] {== $_ -1} -v -line #__LINE__		# Should this return true ?
assert [in $l ""] {== $_ 0} -v -line #__LINE__				# Should this return true ?
assert [in $l "" -nocase] {== $_ 0} -v -line #__LINE__				# Should this return true ?
assert [in $l "" -glob] {== $_ 0} -v -line #__LINE__				# Should this return true ?

assert [lsearch $l {}] {== $_ -1} -v -line #__LINE__		# Should this return true ?
assert [in $l {}] {== $_ 0} -v -line #__LINE__		# Should this return true ?
assert [in $l {} -nocase] {== $_ 0} -v -line #__LINE__		# Should this return true ?
assert [in $l {} -glob] {== $_ 0} -v -line #__LINE__		# Should this return true ?

assert [lsearch "" foo] {== $_ -1} -v -line #__LINE__		# Should not GPF
assert [lsearch -nocase "" foo] {== $_ -1} -v -line #__LINE__		# Should not GPF, FAILS
assert [lsearch -glob "" foo] {== $_ -1} -v -line #__LINE__		# Should not GPF, FAILS

assert [in "" foo] {== $_ 0} -v -line #__LINE__		# Should not GPF
assert [in "" foo -nocase] {== $_ 0} -v -line #__LINE__		# Should not GPF, FAILS
assert [in "" foo -glob] {== $_ 0} -v -line #__LINE__		# Should not GPF, FAILS

assert [lsearch {} foo] {== $_ -1} -v -line #__LINE__		# Should not GPF
assert [lsearch -nocase {} foo] {== $_ -1} -v -line #__LINE__		# Should not GPF
assert [lsearch -glob {} foo] {== $_ -1} -v -line #__LINE__		# Should not GPF

assert [in {} foo] {== $_ 0} -v -line #__LINE__		# Should not GPF
assert [in {} foo -nocase] {== $_ 0} -v -line #__LINE__		# Should not GPF
assert [in {} foo -glob] {== $_ 0} -v -line #__LINE__		# Should not GPF

newline
textcolor yellow
puts "Testing \[ni\] ..."
textcolor
newline

assert [ni $l level_p] {== $_ 0} -v -line #__LINE__
assert [ni $l level_P -nocase] {== $_ 0} -v -line #__LINE__
assert [ni $l foo] {== $_ 1} -v -line #__LINE__
assert [ni $l FOO -nocase] {== $_ 1} -v -line #__LINE__
assert [ni $l ""] {== $_ 1} -v -line #__LINE__		# Should this return true ?
assert [ni $l "" -nocase] {== $_ 1} -v -line #__LINE__		# Should this return true ?
assert [ni $l {}] {== $_ 1} -v -line #__LINE__		# Should this return true ?
assert [ni $l {} -nocase] {== $_ 1} -v -line #__LINE__		# Should this return true ?
assert [ni "" foo] {== $_ 1} -v -line #__LINE__		# Should not GPF
assert [ni "" foo -nocase] {== $_ 1} -v -line #__LINE__		# Should not GPF
assert [ni {} foo] {== $_ 1} -v -line #__LINE__		# Should not GPF
assert [ni {} foo -nocase] {== $_ 1} -v -line #__LINE__		# Should not GPF


unset l line -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #