#############################################################################
# File: 			test_info.tcl
# Purpose:			Test [info]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.19+ ... v1.26+
#
# Versions:
# 1.00 19/Aug/2016	First version
# 1.01 30/Nov/2016	Added more tests
# 1.02 23/Jul/2017	[checkex] replaced with [assert]
# 1.03 06/Sep/2018	Fix exit cleanup
#############################################################################
# Notes: Various tests
#############################################################################
option echo on

set expression_state [option expression]
option expression off
set echostate [option echo]

# Check if this is still correct?
textcolor darkyellow
puts "* WARNING: This test must be compiled with /C and run associated with ticol.exe via the shell to populate\n"
textcolor

if {eq [info script] ""} {
	puts "\[info script\] is empty. Poss running embedded. Skipping tests"
	return 
}

#C:VC5MyProjectspicolReleasetest_info.tcx	 without [escape]
#C:\VC5\MyProjects\ticol\Release\test_info.tcl	with [ecsape]
puts "script_name '[escape [info script]]'"
set script_name [escape [info script]]		# FIX getfullpath
puts "\$script_name '$script_name'"
newline

option echo on
#set $script_name			# Fixed missing err message
puts "$script_name"
assert [escape [info script]] {ne $_ ""} -v -line #__LINE__

unset script_name -nocomplain

#############################################################################
undef foo -nocomplain
proc foo {} {}
assert [info body foo] {eq $_ ""} -trace -v -line #__LINE__
assert [info args foo] {eq $_ ""} -v -line #__LINE__


undef foo -nocomplain
proc foo {x expr} {puts "Foo $x $expr"}		# Dummy proc
assert [info args foo] {eq $_ "x expr"} -v -line #__LINE__
assert [info body foo] {ne $_ ""} -v -line #__LINE__
undef foo -nocomplain

undef foo -nocomplain
proc foo {x} {return}
assert [info body foo] {eq $_ "return"} -trace -v -line #__LINE__
assert [info args foo] {eq $_ "x"} -v -line #__LINE__

undef foo -nocomplain
proc foo {\nx\n} {\nreturn}
assert [info body foo] {eq $_ "\nreturn"} -trace -v -line #__LINE__
assert [info args foo] {eq $_ "x"} -v -line #__LINE__

undef foo -nocomplain
proc foo {\nx\n} {\nreturn\n}
assert [info args foo] {eq $_ "x"} -v -line #__LINE__
assert [info body foo] {eq $_ "\nreturn\n"} -trace -v -line #__LINE__

undef foo -nocomplain
proc foo {
	x
} {
return
}
assert [info args foo] {eq $_ "x"} -trace -v -line #__LINE__
# Note: we must use full binary CRLF not LF
assert [info body foo] {eq $_ "\r\nreturn\r\n"} -trace -v -line #__LINE__

#############################################################################

assert [info cmdcount] {> $_ 0} -v -line #__LINE__
assert [info guid] {ne $_ ""} -v -line #__LINE__
assert [info lib] {ne $_ ""} -v -line #__LINE__

set v Hello
assert [info exists v] {== $_ 1} -v -line #__LINE__
unset v -nocomplain

assert [info level] {>= $_ 0} -v -line #__LINE__
assert [info memavail] {> $_ 0} -v -line #__LINE__
assert [info memtotal] {> $_ 0} -v -line #__LINE__
if {gt [info winver -vernum] 5.0} {
	assert [info monitorcount] {> $_ 0} -v -line #__LINE__		# Not NT4
}
assert [info nameofexecutable] {ne $_ ""} -v -line #__LINE__
#assert [info procs] {ne $_ ""} -v -line #__LINE__
assert [info tclversion] {> $_ 1.20} -v -line #__LINE__
assert [info winver] {ne $_ ""} -v -line #__LINE__

option echo $echostate
option expression $expression_state

unset echostate expression_state -nocomplain


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* \[info\] tests done *"
textcolor

# EOF #