###########################################################################################
# File:				test_info_level.tcl
# Author:			M Shaw
# Purpose:			Tests for [info level]
#
# Versions:
# 1.00 23/Jun/2016	First version
# 1.01 15/Sep/2017	Updated for bugfix multi-level tests
###########################################################################################
# Notes:
# http://wiki.tcl.tk/1720
#
#	If number is specified, then the result is a list consisting of the name and arguments for the procedure 
#	call at level number on the stack. 
#	If number is positive then it selects a particular stack level 
#	(1 refers to the top-most active procedure, 2 to the procedure it called, and so on); otherwise it gives 
#	a level relative to the current level (0 refers to the current procedure, -1 to its caller, and so on). 
#	So:
#	< 1 is relative
#	 0 == current level
#	-1 == is parent
#	-2 == grandparent
#	>= 1 is absolute
#    1 == root
#    2 == first proc call from root
###########################################################################################
/*
# Correct behaviour
foo abc def		# example1
foo abc			# example2
prints:
info level 0 = [foo abc def]
info level 0 = [foo abc]
*/
###########################################################################################

option echo on

undef foo -nocomplain								# Recommended when chaining many other files

proc foo {bar {baz NULL}} {
    #puts "info level 0 = \[[info level 0]\]"
	#puts "info level 1 = \[[info level 1]\]"
	#newline
	return [info level 0]
}

/*
foo
foo abc
foo abc def
*/

assert [foo] {eq $_ foo} -v -line #__LINE__
assert [foo abc] {eq $_ "foo abc"} -v -line #__LINE__
assert [foo abc def] {eq $_ "foo abc def"} -v -line #__LINE__

undef foo -nocomplain
undef bar -nocomplain
undef moo -nocomplain
undef mar -nocomplain
undef par -nocomplain

# Relative level tests # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Absolute info level is always problematic as later call changes may change absolute levels
newline
textcolor yellow
puts "\[info level N\] tests"
textcolor

# Multi-level test 
proc bar {} {
	puts "\nIn bar at level [info level]"						# Nivelo 2
	assert [info level -1] {eq $_ foo} -v -line #__LINE__	# Parent
	assert [info level 0] {eq $_ bar} -v -line #__LINE__	# Current
	assert [info level 1] {eq $_ foo} -v -line #__LINE__	# Absolute
}

proc foo {} {
	puts "\nIn foo at level [info level]"						# Nivelo 1
	assert [info level 0] {eq $_ foo} -v -line #__LINE__
	bar
}

foo

proc mar {} {
	assert [info level -1] {eq $_ par} -v -line #__LINE__
	assert [info level 0] {eq $_ mar} -v -line #__LINE__
	assert [info level 1] {eq $_ moo} -v -line #__LINE__	# Absolute
}

proc par {} {
	assert [info level -1] {eq $_ moo} -v -line #__LINE__
	assert [info level 0] {eq $_ par} -v -line #__LINE__
	assert [info level 1] {eq $_ moo} -v -line #__LINE__
	mar
}

proc moo {} {
	par
}

moo

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #