###############################################################################
# File: 		test_input_csv.tcl
# Purpose:		Test file for [input_csv]
# Author: 		M Shaw
# Language:		Ticol Tcl v1.20c+ ... v1.26+
#
# Versions:
# 1.00 28/Nov/2016 First version
# 1.01 11/Jul/2017	[checkex] replaced with [assert]
# 1.02 01/Jan/2020	Updated
###############################################################################
# Notes:
# Requires write access to the file system
###############################################################################
option expression off

textcolor yellow
puts "Testing \[input_csv\] ..."
textcolor
newline

unset s -nocomplain
unset filename -nocomplain

# We must create a test file
set filename test.csv
# Some text junk from an old spam filter log ...
set s "\"2008/09/28\",\"01:44:27\",\"Log cleared/started on 28 September 2008 at 01:44:27\"\r\n\
\"2008/09/28\",\"01:44:27\",\"127.0.0.1\",\"127.0.0.1\",\"Localhost\",\"*** spam ***jewelry from ornella muti\"\r\n\
\"2008/09/28\",\"01:44:27\",\"90.189.226.85\",\"90.189.226.85\",\"IANA\",\"*** spam ***jewelry from ornella muti\"\r\n\
\"2008/09/28\",\"01:44:28\",\"10.232.28.37\",\"10.232.28.37\",\"IANA\",\"*** spam ***acheter du viaa en ligne?\"\r\n\
\"2008/09/28\",\"01:44:28\",\"127.0.0.1\",\"127.0.0.1\",\"Localhost\",\"*** spam ***acheter du viaa en ligne?\"\r\n\
\"2008/09/28\",\"01:44:29\",\"127.0.0.1\",\"127.0.0.1\",\"Localhost\",\"*** spam ***This is a spam message\""

#puts $s
#puts --------------------------------------------------------------------- 
#@ type $filename
#newline

if {! [writefile $filename [unescape $s]]} {
	textcolor red
	puts "Unable to test. Cannot write to $filename"
	textcolor
	return			# Exit this script
}

if {! [file exists $filename]} {
	die "File $filename not found. Failed to create $filename"
}

input_csv $filename -array b				# Normal index order name(integer)=value
#array walk b
#puts "b3='$b(3)'"
#puts 'b3 lindex 3=[lindex $b(3) 3]'

assert [array size b] {== $_ 6} -v -line #__LINE__

assert [lindex $b(0) 1] {eq $_ 01:44:27} -v -line #__LINE__
assert [lindex $b(1) 2] {eq $_ 127.0.0.1} -v -line #__LINE__
assert [lindex $b(2) 0] {eq $_ 2008/09/28} -v -line #__LINE__
assert [lindex $b(3) 4] {eq $_ IANA} -v -line #__LINE__
assert [lindex $b(4) 1] {eq $_ 01:44:28} -v -line #__LINE__
assert [lindex $b(5) 4] {eq $_ Localhost} -v -line #__LINE__

array unset b -nocomplain
unset filename 
unset s 

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #