###############################################################################################
# File:				test_interp.tcl
# Author:			M Shaw
# Purpose:			Test file for experimental [interp] command
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 29/Nov/2019	First version
###############################################################################################
# Notes:
# #1 support fault will be forgetting to add [exit] at the end of an [interp] script
# For it to be worth compiling as threaded with the overheads [interp] has to offer something
# Our [interp] doesn't need to be as complex as ActiveState
# A goal is to have one or more child threads running via [interp]
# 	Would need tracking and management where > 1 thread offered
# Arg to undef a list of commands?
# Arg to pass variables with their contents?
# FIX Passing vars is a problem
# Add argv[1] keywords?
# interp create ?-safe? ?--? ?path? 
# set i [interp create -safe]
# interp hide $i lappend
# interp alias $i lappend {} loggedLappend $i
# proc loggedLappend {i args} {
#     puts "logged invocation of lappend $args"
#     interp invokehidden $i lappend {*}$args
# }
# interp eval $i $someUntrustedScript
# DONE (kludge) Passing vars ? ...
# 	interp {script} -var a b c
###############################################################################################
textcolor yellow
puts "Testing \[interp]\ (basic implementation) ..."
textcolor
newline

puts "Line: "#__LINE__
#interp [interp [interp [interp [interp exit] exit] exit] exit] exit 
puts "Line: "#__LINE__
puts "Test var upchaining @"#__LINE__
set q "* Back at root *" 
interp {
	set level 0
	puts [interp {
		puts $level
		++ level
		textcolor magenta
		puts "At level $level"
		textcolor
		assert $level {== $_ 1} -v -line #__LINE__
		interp {
			++ level
			textcolor yellow
			puts "At level $level"
			textcolor
			assert $level {== $_ 2} -v -line #__LINE__
			interp {
				++ level
				textcolor cyan
				puts "At level $level"
				textcolor
				assert $level {== $_ 3} -v -line #__LINE__
				newline
				textcolor green
				puts "Returning..."
				textcolor
				exit
			} -var level
			exit
		} -var level
		exit
	} -var level]
	exit
}
textcolor white
puts "->$q"
# exit
# */

# test_interp.tcl
set a 23
assert $a {== $_ 23} -v -line #__LINE__
interp {
	puts "Enter child interpreter @"#__LINE__
	set a 24
	assert $a {== $_ 24} -v -line #__LINE__
	exit
}
assert $a {== $_ 23} -v -line #__LINE__

interp {
	puts "Enter child interpreter @"#__LINE__
	set a 25
	assert $a {== $_ 25} -v -line #__LINE__
	textcolor darkyellow
	puts "Wait for manual \[exit\] command to return to child"
	textcolor
	# BLOCKING... exit
}
# Wait for manual exit
assert $a {== $_ 23} -v -line #__LINE__
newline

textcolor yellow
puts "Launching nested \[interp\] ..."
textcolor

# FIX set a global at each invocation and increment level ID ?

interp {
#	++ level
	textcolor darkyellow
	puts "Enter child interpreter @"#__LINE__
	textcolor
	puts "Level 2"
	interp {
		textcolor magenta
		puts "Enter child interpreter @"#__LINE__
		textcolor
		puts "Level 3"
		interp {
			textcolor cyan
			puts "Enter child interpreter @"#__LINE__
			textcolor
			puts "Level 4"
			exit
		}
		exit
	}
	exit
}
puts "Back at root level"
newline

set r [interp {
	return [* 4 [funct atan 1]]
}]
puts "interp result was $r"
assert $r {== $_ 3.141592653589792} -v -line #__LINE__

textcolor green
puts "* Done *"
textcolor