######################################################################################
# File: 			test_is.tcl
# Purpose:			Test various [is_*] native commands
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 09/Jul/2017	First version using [assert]
######################################################################################
# Notes: 
######################################################################################
/*
is_active
is_admin
is_alnum
is_alpha
is_array
is_command
is_const
is_date
is_digit
is_dir
is_dst
is_elevated
is_empty
is_file
is_leapyear
is_list
is_lower
is_mod
is_null
is_numeric
is_pointer
is_proc
is_punct
is_set
is_space
is_upper
is_xdigit

array exists
array is_set
*/

######################################################################################
# is_active
textcolor yellow
puts "\r\n\[is_active\]"
textcolor
undef bar foo -nocomplain
proc foo {} {
	assert [is_active foo] {== $_ 1} -v -line #__LINE__
	assert [is_active {}] {== $_ 0} -v -line #__LINE__
	assert [is_active moo] {== $_ 0} -v -line #__LINE__

	proc bar {} {
		assert [is_active foo] {== $_ 2} -v -line #__LINE__
		assert [is_active bar] {== $_ 1} -v -line #__LINE__
		assert [is_active {}] {== $_ 0} -v -line #__LINE__
		assert [is_active moo] {== $_ 0} -v -line #__LINE__
	}
	bar
}
foo

######################################################################################
# is_admin

######################################################################################
# is_alnum
textcolor yellow
puts "\r\n\[is_alnum\]"
textcolor
assert [is_alnum 2m55] {== $_ 1} -v -line #__LINE__ 
assert [is_alnum "\t"] {== $_ 0} -v -line #__LINE__ 
assert [is_alnum ""] {== $_ 0} -v -line #__LINE__ 

######################################################################################
# is_alpha
textcolor yellow
puts "\r\n\[is_alpha\]"
textcolor
assert [is_alpha abcd] {== $_ 1} -v -line #__LINE__ 
assert [is_alpha "451"] {== $_ 0} -v -line #__LINE__ 
assert [is_alpha ""] {== $_ 0} -v -line #__LINE__ 

######################################################################################
# is_array
textcolor yellow
puts "\r\n\[is_array\]"
textcolor
set a(0) hello
assert [is_array a] {== $_ 1} -v -line #__LINE__ 
array unset a
assert [is_array a] {== $_ 0} -v -line #__LINE__ 
assert [is_array ""] {== $_ 0} -v -line #__LINE__ 

######################################################################################
# is_command
textcolor yellow
puts "\r\n\[is_command\]"
textcolor
undef foo -nocomplain
assert [is_command foo] {== $_ 0} -v -line #__LINE__
assert [is_command is_command] {== $_ 1} -v -line #__LINE__
assert [is_command ls] {== $_ 1} -v -line #__LINE__
assert [is_command fred] {== $_ 0} -v -line #__LINE__
assert [is_command ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_const
# NB Dereferenced literals have to be distinguished from var const
textcolor yellow
puts "\r\n\[is_const\]"
textcolor
assert [is_const pi] {== $_ 1} -v -line #__LINE__
assert [is_const env] {== $_ 1} -v -line #__LINE__
assert [is_const $pi] {== $_ 0} -v -line #__LINE__
assert [is_const 123] {== $_ 0} -v -line #__LINE__
assert [is_const ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_date
textcolor yellow
puts "\r\n\[is_date\]"
textcolor
assert [is_date [now]] {== $_ 1} -v -line #__LINE__
assert [is_date 2017/07/09] {== $_ 1} -v -line #__LINE__
assert [is_date 2017/07/09 23:08] {== $_ 1} -v -line #__LINE__
assert [is_date 2017/07/09 23:08:23] {== $_ 1} -v -line #__LINE__
assert [is_date foo] {== $_ 0} -v -line #__LINE__
assert [is_date 0] {== $_ 1} -v -line #__LINE__		# 0 is a valid date
assert [is_date {}] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_digit
textcolor yellow
puts "\r\n\[is_digit\]"
textcolor
assert [is_digit 451] {== $_ 1} -v -line #__LINE__ 
assert [is_digit abc] {== $_ 0} -v -line #__LINE__ 
assert [is_digit ""] {== $_ 0} -v -line #__LINE__ 

######################################################################################
# is_dir
textcolor yellow
puts "\r\n\[is_dir\]"
textcolor
assert [is_dir .] {== $_ 1} -v -line #__LINE__
assert [is_dir \$foo\$] {== $_ 0} -v -line #__LINE__
assert [is_dir ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_dst
# Not tested

######################################################################################
# is_elevated
# Not tested

######################################################################################
# is_empty
textcolor yellow
puts "\r\n\[is_empty\]"
textcolor
unset s -nocomplain
assert [is_empty s] {== $_ 1} -v -line #__LINE__
set s hi
assert [is_empty s] {== $_ 0} -v -line #__LINE__
unset s
assert [is_empty ""] {== $_ 1} -v -line #__LINE__

######################################################################################
# is_file
if {ne [info script] ""} {						# Could be running embedded
	if {ne [info script] "internal.tcl"} {		# Not for embedded Tcl (still have to determine default filename)
		textcolor yellow
		puts "\r\n\[is_file\]"
		textcolor
		puts "\[info script\]==[escape [info script]]"
		assert [is_file [escape [info script]]] {== $_ 1} -v -line #__LINE__
	}
}

######################################################################################
# is_leapyear
# See also: test_is_leapyear.tcl
textcolor yellow
puts "\r\n\[is_leapyear\]"
textcolor
unset s -nocomplain
assert [is_leapyear 2000] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2001] {== $_ 0} -v -line #__LINE__
assert [is_leapyear ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_list
textcolor yellow
puts "\r\n\[is_list\]"
textcolor
assert [is_list {a b c}] {== $_ 1} -v -line #__LINE__
assert [is_list {}] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_lower
textcolor yellow
puts "\r\n\[is_lower\]"
textcolor
assert [is_lower abc] {== $_ 1} -v -line #__LINE__
assert [is_lower ABC] {== $_ 0} -v -line #__LINE__
assert [is_lower ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_mod
textcolor yellow
puts "\r\n\[is_mod\]"
textcolor
assert [is_mod 24 2] {== $_ 1} -v -line #__LINE__
assert [is_mod 23 2] {== $_ 0} -v -line #__LINE__
assert [is_mod 1 0] {== $_ 0} -v -line #__LINE__
assert [is_mod 0 1] {== $_ 1} -v -line #__LINE__

######################################################################################
# is_null
textcolor yellow
puts "\r\n\[is_null\]"
textcolor
assert [is_null ""] {== $_ 1} -v -line #__LINE__
assert [is_null {}] {== $_ 1} -v -line #__LINE__
assert [is_null Hello] {== $_ 0} -v -line #__LINE__
assert [is_null $qbf] {== $_ 0} -v -line #__LINE__
set nul {}
assert [is_null $nul] {== $_ 1} -v -line #__LINE__
set nul ""
assert [is_null $nul] {== $_ 1} -v -line #__LINE__
unset nul -nocomplain

######################################################################################
# is_numeric
textcolor yellow
puts "\r\n\[is_numeric\]"
textcolor
assert [is_numeric ""] {== $_ 0} -v -line #__LINE__
assert [is_numeric {}] {== $_ 0} -v -line #__LINE__

assert [is_numeric abc] {== $_ 0} -v -line #__LINE__
assert [is_numeric 0] {== $_ 1} -v -line #__LINE__
assert [is_numeric 1] {== $_ 1} -v -line #__LINE__
assert [is_numeric -1] {== $_ 1} -v -line #__LINE__
assert [is_numeric 012345] {== $_ 1} -v -line #__LINE__
assert [is_numeric +012345] {== $_ 1} -v -line #__LINE__
assert [is_numeric -012345] {== $_ 1} -v -line #__LINE__
assert [is_numeric 1.2345] {== $_ 1} -v -line #__LINE__
assert [is_numeric +1.2345] {== $_ 1} -v -line #__LINE__
assert [is_numeric -1.2345] {== $_ 1} -v -line #__LINE__

######################################################################################
# is_pointer (struct field member)
textcolor yellow
puts "\r\n\[is_pointer\]"
textcolor
struct unset st -nocomplain
struct create st {a 10}
assert [is_pointer st.a] {== $_ 1} -v -line #__LINE__
struct unset st
assert [is_pointer env] {== $_ 0} -v -line #__LINE__
assert [is_pointer ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_proc
textcolor yellow
puts "\r\n\[is_proc\]"
textcolor
undef foo -nocomplain
proc foo x {} {}
assert [is_proc foo] {== $_ 1} -v -line #__LINE__
undef foo
assert [is_proc foo] {== $_ 0} -v -line #__LINE__
assert [is_proc {}] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_punct
textcolor yellow
puts "\r\n\[is_punct\]"
textcolor
assert [is_punct "."] {== $_ 1} -v -line #__LINE__
assert [is_punct ","] {== $_ 1} -v -line #__LINE__
assert [is_punct "{"] {== $_ 1} -v -line #__LINE__
assert [is_punct ""] {== $_ 0} -v -line #__LINE__
assert [is_punct {}] {== $_ 0} -v -line #__LINE__
assert [is_punct abc] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_set
textcolor yellow
puts "\r\n\[is_set\]"
textcolor
unset foo -nocomplain
assert [is_set ""] {== $_ 0} -v -line #__LINE__
assert [is_set {}] {== $_ 0} -v -line #__LINE__

assert [is_set foo] {== $_ 0} -v -line #__LINE__
assert [is_set env] {== $_ 1} -v -line #__LINE__
assert [is_set env(computername)] {== $_ 1} -v -line #__LINE__
set i 1
assert [is_set i] {== $_ 1} -v -line #__LINE__
unset i -nocomplain

set x 1
assert [is_set x] {== $_ 1} -v -line #__LINE__
unset x
assert [is_set x] {== $_ 0} -v -line #__LINE__

unset x -nocomplain

array unset a -nocomplain
newline

######################################################################################
# is_space
textcolor yellow
puts "\r\n\[is_space\]"
textcolor
assert [is_space " "] {== $_ 1} -v -line #__LINE__
assert [is_space { }] {== $_ 1} -v -line #__LINE__
assert [is_space [chr 32]] {== $_ 1} -v -line #__LINE__
assert [is_space [chr 0x20]] {== $_ 1} -v -line #__LINE__

assert [is_space foo] {== $_ 0} -v -line #__LINE__
assert [is_space ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_upper
textcolor yellow
puts "\r\n\[is_upper\]"
textcolor
assert [is_upper A] {== $_ 1} -v -line #__LINE__
assert [is_upper "ABC"] {== $_ 1} -v -line #__LINE__
assert [is_upper abc] {== $_ 0} -v -line #__LINE__
assert [is_upper " "] {== $_ 0} -v -line #__LINE__
assert [is_upper ""] {== $_ 0} -v -line #__LINE__

######################################################################################
# is_xdigit
textcolor yellow
puts "\r\n\[is_xdigit\]"
textcolor
loop i [asc A] [asc G] {		# A..F inclusive
	assert [is_xdigit [chr $i]] {== $_ 1} -v -line #__LINE__
}
loop i [asc a] [asc g] {		# A..F inclusive
	assert [is_xdigit [chr $i]] {== $_ 1} -v -line #__LINE__
}
loop i [asc 0] [+ [asc 9] 1] {	# 0..9 inclusive
	assert [is_xdigit [chr $i]] {== $_ 1} -v -line #__LINE__
}
unset i
assert [is_xdigit z] {== $_ 0} -v -line #__LINE__
assert [is_xdigit ""] {== $_ 0} -v -line #__LINE__
assert [is_xdigit -] {== $_ 0} -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
option expression off
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #