###################################################################################
# File: 			test_is_empty.tcl
# Purpose:			Test file for [is_empty]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Oct/2017	First version
# 1.01 09/Sep/2018	Updated re: bugfix
# 1.02 07/Sep/2019	Revise re: changes in logic to [is_set]
###################################################################################
# Notes: 
# [is_empty] returns TRUE (1) if a var is both set (exists) and empty, 
# otherwise FALSE (0)
#
# Mooted whether to return TRUE (1) if the var does not exist?
#
# Truth Table
# ----------------
# 			Set
#			Y	N
#   Empty
# 		Y	1	0
# 		N	0	N/A (Cannot be 'empty' if 'not set')
###################################################################################
option expression off

textcolor yellow
puts "Testing \[is_empty\] with standard variable ..."
textcolor
newline

unset s -nocomplain
# s not yet declared
assert [is_empty s] {== $_ $true} -v -line #__LINE__		# FALSE because s is not set

# s is set but not empty
set s Hello
assert [is_empty s] {== $_ $false} -v -line #__LINE__		# FALSE because s is set and not empty

# s is set and empty
set s {}
assert [is_empty s] {== $_ $true} -v -line #__LINE__		# TRUE because s is both set and empty
newline

# ARRAY #
textcolor yellow
puts "Testing \[is_empty\] with an array variable ..."
textcolor
newline

unset a(0) -nocomplain
array unset a -nocomplain
# a(0) not yet declared
assert [is_empty a(0)] {== $_ $false} -v -line #__LINE__	# FALSE because a(0) is not set

# a(0) is set but not empty
set a(0) Hello
assert [is_empty a(0)] {== $_ $false} -v -line #__LINE__	# FALSE because a(0) is set and not empty

# a(0) is set and empty
# Should this be true?
set a(0) {}
array walk a
assert [is_empty a(0)] {== $_ $true} -v -line #__LINE__		# TRUE because a(0) is set and empty

# array is merely dimmed
# Should this be true?
array unset a
dim a 10
array statistics a
array walk a
assert [is_empty a] {== $_ $true} -v -line #__LINE__		# TRUE because a is set and empty
assert [is_empty a(0)] {== $_ $false} -v -line #__LINE__	# FALSE because a(0) is not set

array unset a
unset a(0) -nocomplain
unset s -nocomplain
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # #
newline
textcolor green
puts "* Done *"
textcolor

# EOF #