###########################################################################
# File:				test_is_leapyear.tcl
# Author:			M Shaw
# Purpose:			Test file for [is_leapyear]
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 06/Feb/2017	First version (basic tests)
###########################################################################
# Notes:
# Uses Gregorian scheme
# Duplicate test to test_leapyear.tcl
###########################################################################

/*
	/////////////////////////////////////////////////////////////////////////////////////////
	// A leap year is a year in which an extra day is added to the calendar in order to synchronize it with the seasons. Since the tropical year is 365.242190 days long, a leap year must be added roughly once every four years (four times the fractional day gives  ). In a leap year, the extra day (known as a leap day) is added at the end of February, giving it 29 instead of the usual 28 days.
	// In the Gregorian calendar currently in use worldwide (except perhaps the Russian and Iranian calendars), there is a leap year every year divisible by four except for years which are both divisible by 100 and not divisible by 400. Therefore, the year 2000 will be a leap year, but the years 1700, 1800, and 1900 were not. The complete list of leap years in the first half of the 21st century is therefore 2000, 2004, 2008, 2012, 2016, 2020, 2024, 2028, 2032, 2036, 2040, 2044, and 2048.
	// The extra rule involving centuries is an additional correction to make up for the fact that one extra day every four years is slightly too much correction (). This scheme results in the vernal equinox gradually shifting its date between March 19 and 21, being shifted once every leap year, and then being abruptly shifted in non-leap centuries (see figure above).
	// In the Gregorian calendar, 97 years out of every 400 are leap years, giving the total number of days in 400 years as
	// The leap year was introduced in the Julian calendar in 46 BC. 
	// However, around 10 BC, it was found that the priests in charge of computing the calendar had been adding 
	// leap years every three years instead of the four decreed by Caesar (Vardi 1991, p. 239). 
	// As a result of this error, no more leap years were added until 8 AD. Leap years were therefore 45 BC, 42 BC, 39 BC, 36 BC, 33 BC, 30 BC, 27 BC, 24 BC, 21 BC, 18 BC, 15 BC, 12 BC, 9 BC, 8 AD, 12 AD, and every fourth year thereafter (Tøndering), until the Gregorian calendar was introduced (resulting in skipping three out of every four centuries). 
	// The UNIX command cal incorrectly lists 4 AD as a leap year (Vardi 1991).
	// Leap years were therefore 
	// 45 BC, 42 BC, 39 BC, 36 BC, 33 BC, 30 BC, 27 BC, 24 BC, 21 BC, 18 BC, 15 BC, 12 BC, 9 BC, 
	// !!NOT 4AD!!
	// 8 AD, 12 AD, and every fourth year thereafter (Tøndering), 
	/////////////////////////////////////////////////////////////////////////////////////////
	// The Julian Calendar was the system of dating followed from 46BC onwards
	// In 1582 Pope Gregory ruled that this new calendar - thereafter called the Gregorian Calendar - should be brought into use
	// Prior to 16th C there was no additional 100 or 400 year rule
	// This was a straightforward calculation of (year mod 4)
	// Unfortunately for historians, the new calendar was not adopted universally
	// UK adopted the Gregorian calendar in 1752
	/////////////////////////////////////////////////////////////////////////////////////////

	Problem: [% 1700 4]==0 but UK/US did not adopt leap calendar until later
	if (year is not divisible by 4) then (it is a common year)
	else if (year is not divisible by 100) then (it is a leap year)
	else if (year is not divisible by 400) then (it is a common year)
	else (it is a leap year)
	The algorithm is not necessarily valid for years in the Julian calendar, such as years 
	before 1752 in the British Empire. The year 1700 was a leap year in the Julian calendar, 
	but not in the Gregorian calendar.
	/////////////////////////////////////////////////////////////////////////////////////////

	
45 BC, 42 BC, 39 BC, 36 BC, 33 BC, 30 BC, 27 BC, 24 BC, 21 BC, 18 BC, 15 BC, 12 BC, 9 BC, 
!!NOT 4AD!!
8 AD, 12 AD, and every fourth year thereafter (Tøndering), 

1584, 1588, 1592, 1596, *1600*, 1604, 1608, 1612, 1616, 1620, 1624, 1628, 1632, 1636, 1640, 1644, 1648, 1652, 
1656, 1660, 1664, 1668, 1672, 1676, 1680, 1684, 1688, 1692, 1696, <NOT 1700>, 1704, 1708, 1712, 1716, 1720, 1724, 1728, 
1732, 1736, 1740, 1744, 1748, 1752, 1756, 1760, 1764, 1768, 1772, 1776, 1780, 1784, 1788, 1792, 1796, <NOT 1800>, 
1804, 
1808, 1812, 1816, 1820, 1824, 1828, 1832, 1836, 1840, 1844, 1848, 1852, 1856, 1860, 1864, 1868, 1872, 1876,
1880, 1884, 1888, 1892, 1896, <1900 NOT>, 1904, 1908, 1912, 1916, 1920, 1924, 1928, 1932, 1936, 1940, 1944, 1948, 1952, 
1956, 1960, 1964, 1968, 1972, 1976, 1980, 1984, 1988, 1992, 1996, 2000, 2004, 2008, 2012, 2016

Leap Years 1800 - 2400
1804
1808
1812
1816
1820
1824
1828
1832
1836
1840
1844
1848
1852
1856
1860
1864
1868
1872
1876
1880
1884
1888
1892
1896	1904
1908
1912
1916
1920
1924
1928
1932
1936
1940
1944
1948
1952
1956
1960
1964
1968
1972
1976
1980
1984
1988
1992
1996
2000	2004
2008
2012
2016
2020
2024
2028
2032
2036
2040
2044
2048
2052
2056
2060
2064
2068
2072
2076
2080
2084
2088
2092
2096	2104
2108
2112
2116
2120
2124
2128
2132
2136
2140
2144
2148
2152
2156
2160
2164
2168
2172
2176
2180
2184
2188
2192
2196	2204
2208
2212
2216
2220
2224
2228
2232
2236
2240
2244
2248
2252
2256
2260
2264
2268
2272
2276
2280
2284
2288
2292

*/

option echo on
option expression off

textcolor yellow
puts "Test \[is_leapyear\] from -15 AD to 2300+ AD ...\n"
textcolor

assert [is_leapyear -15] {== $_ 1} -v -line #__LINE__
assert [is_leapyear -12] {== $_ 1} -v -line #__LINE__
assert [is_leapyear -9] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 0] {== $_ 0} -v -line #__LINE__		# No such date
assert [is_leapyear 1] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 3] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 4] {== $_ 0} -v -line #__LINE__		# 4AD was not a leap year (resumed at 10AD - also not a LY)
assert [is_leapyear 5] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 6] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 7] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 8] {== $_ 1} -v -line #__LINE__		# 8AD was the first leap year to be resumed
assert [is_leapyear 9] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 10] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 11] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 12] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 99] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 100] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 101] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 120] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 200] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 201] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 220] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 300] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 301] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 320] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 400] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 401] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 420] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 500] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 501] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 520] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 600] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 601] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 620] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 700] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 701] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 620] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 800] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 801] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 820] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 900] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 901] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 920] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 999] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1000] {== $_ 0} -v -line #__LINE__		# Not a Gregorian leap year
assert [is_leapyear 1001] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1066] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1099] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1100] {== $_ 0} -v -line #__LINE__		# Not a Gregorian leap year
assert [is_leapyear 1101] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1120] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 1200] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1201] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1220] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1299] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1300] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1301] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1320] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1399] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1400] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1401] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1403] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1420] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1499] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1500] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1501] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1520] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1584] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1588] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1592] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1596] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1599] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1600] {== $_ 1} -v -line #__LINE__		# Gregorian epoch WAS a leap year
assert [is_leapyear 1602] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1604] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1608] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1612] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1616] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1620] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1624] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1628] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1632] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1636] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1640] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1644] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1648] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1652] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1656] {== $_ 1} -v -line #__LINE__


# Gregorian afer 1752 in UK/USA
assert [is_leapyear 1700] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1701] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1704] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1740] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1744] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1748] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1750] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1751] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1752] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1753] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1756] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1776] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1777] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1796] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1799] {== $_ 0} -v -line #__LINE__

# 1872, 1876,

assert [is_leapyear 1800] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1801] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1804] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1808] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1812] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1816] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1820] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1828] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1832] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1836] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1840] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1844] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1848] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1852] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1856] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1860] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1864] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1868] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1872] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1876] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1896] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1899] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1900] {== $_ 0} -v -line #__LINE__	# Not a leap year
assert [is_leapyear 1901] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1904] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1936] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1940] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1944] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1948] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1950] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1952] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1955] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1956] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1959] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 1960] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1968] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 1969] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1970] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1971] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1972] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1973] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1974] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1975] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1976] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1977] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1978] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1979] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1980] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1981] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1982] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1983] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1984] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1985] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 1988] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1992] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 1996] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 1999] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2000] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2001] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2004] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2008] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2009] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2010] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2011] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2012] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2013] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2014] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2015] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2016] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2017] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2018] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2019] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2021] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2022] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2023] {== $_ 0} -v -line #__LINE__

assert [is_leapyear 2100] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2112] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 2200] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2296] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 2300] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2304] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2308] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2312] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2316] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2320] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2324] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2328] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2332] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2336] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2340] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2340] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2344] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2348] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2352] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2356] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2360] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2364] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2368] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2372] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2376] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2380] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2384] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2388] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2392] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2396] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2400] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 2500] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2520] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 2600] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2620] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 2700] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2720] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2800] {== $_ 1} -v -line #__LINE__		# Is a leap year
assert [is_leapyear 2820] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 2900] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 2920] {== $_ 1} -v -line #__LINE__

assert [is_leapyear 3000] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 3020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 4000] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 4020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 5000] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 5020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 6000] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 6020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 7000] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 7020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 8000] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 8020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 9000] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 9020] {== $_ 1} -v -line #__LINE__
assert [is_leapyear 9999] {== $_ 0} -v -line #__LINE__
assert [is_leapyear 10000] {== $_ 1} -v -line #__LINE__			# ?

vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #