###############################################################
# File:				is_date.tcl
# Author:			M Shaw
# Purpose:			Test test for VB/.Net style dates (double)
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 24/Oct/2016	First version
# 1.01 11/Jul/2017	[checkex] replaced with [assert]
###############################################################
# Notes:
###############################################################

debug off
puts "Testing valid \[is_date\] cases ..."

assert [is_date -1] {== $_ $true} -v -line #__LINE__
assert [is_date 0] {== $_ $true} -v -line #__LINE__
assert [is_date 1] {== $_ $true} -v -line #__LINE__
assert [is_date 2] {== $_ $true} -v -line #__LINE__
assert [is_date 0.1] {== $_ $true} -v -line #__LINE__
assert [is_date 42657.0430902778] {== $_ $true} -v -line #__LINE__
assert [is_date -45287.4999884259] {== $_ $true} -v -line #__LINE__		# 1776/jan/01 12:00:01
assert [is_date -575082.4992824075] {== $_ $true} -v -line #__LINE__ 		# 0325/jun/22 12:01:02
assert [is_date 0325/jun/22 12:01:02] {== $_ $true} -v -line #__LINE__
assert [is_date 1776/jan/01 12:00:01] {== $_ $true} -v -line #__LINE__
assert [is_date 2016/Oct/14] {== $_ $true} -v -line #__LINE__
assert [is_date 2016/10/14] {== $_ $true} -v -line #__LINE__
assert [is_date 00:0] {== $_ $true} -v -line #__LINE__
assert [is_date 0000:00] {== $_ $true} -v -line #__LINE__					# Technically this is valid as 0000 can hold max value of 59 (c/f ? cdate("0000:00:00"))
assert [is_date 00:00] {== $_ $true} -v -line #__LINE__
assert [is_date 23:59] {== $_ $true} -v -line #__LINE__
assert [is_date 00:00:59] {== $_ $true} -v -line #__LINE__
assert [is_date 2016/10/14 01:02:03] {== $_ $true} -v -line #__LINE__
assert [is_date 01:02:03 2016/10/14] {== $_ $true} -v -line #__LINE__
assert [is_date "2016/10/14 01:02:03"] {== $_ $true} -v -line #__LINE__
assert [is_date {2016/10/14 01:02:03}] {== $_ $true} -v -line #__LINE__

# FAIL CASES ########################################################
newline
puts "Testing invalid \[is_date\] cases ..."
assert [is_date 9999999.0430902778] {== $_ $false} -v -line #__LINE__		# Invalid year range as double
assert [is_date Fred]  {== $_ $false} -v -line #__LINE__					# Silly value
assert [is_date 60:00:00]  {== $_ $false} -v -line #__LINE__				# Invalid hh
assert [is_date 00:60:00]  {== $_ $false} -v -line #__LINE__				# Invalid mm
assert [is_date 00:00:60]  {== $_ $false} -v -line #__LINE__				# Invalid ss
assert [is_date 60:60:60]  {== $_ $false} -v -line #__LINE__				# Invalid hh:mm:ss
assert [is_date 0A:00:00]  {== $_ $false} -v -line #__LINE__				# Invalid - H char 
assert [is_date 00:0A:00]  {== $_ $false} -v -line #__LINE__				# Invalid - M
assert [is_date 00:00:0A]  {== $_ $false} -v -line #__LINE__				# Invalid - S
assert [is_date A0:00:00]  {== $_ $false} -v -line #__LINE__				# Invalid - H char 
assert [is_date 00:A0:00]  {== $_ $false} -v -line #__LINE__				# Invalid - M
assert [is_date 00:00:A0]  {== $_ $false} -v -line #__LINE__				# Invalid - S
assert [is_date "-00:00:00"]  {== $_ $false} -v -line #__LINE__				# Invalid - ? VB does not allow (type mismatch)
assert [is_date "00:00:59 2016/oct/39"]  {== $_ $false} -v -line #__LINE__	# Invalid day
assert [is_date "00:00:59 2016/13/39"]  {== $_ $false} -v -line #__LINE__	# Invalid month
assert [is_date "00:00:59 2016/0x0a/39"]  {== $_ $false} -v -line #__LINE__	# Invalid month
assert [is_date 2016/Qwer/14]  {== $_ $false} -v -line #__LINE__			# Invalid month
assert [is_date 10000/01/14]  {== $_ $false} -v -line #__LINE__ 			# Out of range year
assert [is_date 999/01/14]  {== $_ $false} -v -line #__LINE__ 				# Out of range year
assert [is_date "00:00:59 Fred/11/11"]  {== $_ $false} -v -line #__LINE__	# Invalid year
assert [is_date "00:00:59 1999/11/tue"]  {== $_ $false} -v -line #__LINE__	# Invalid year
assert [is_date "jjjj/11/00 00:00:59"]  {== $_ $false} -v -line #__LINE__	# Invalid year
assert [is_date "1999/11/jj 00:00:59"]  {== $_ $false} -v -line #__LINE__	# Invalid day
assert [is_date "1999/jj/00 00:00:59"]  {== $_ $false} -v -line #__LINE__	# Invalid month
assert [is_date "11/00/jjjj 00:00:59"]  {== $_ $false} -v -line #__LINE__	# Invalid year
assert [is_date "11/jj/1999 00:00:59"]  {== $_ $false} -v -line #__LINE__	# Invalid month
assert [is_date "jj/00/1999 00:00:59"]  {== $_ $false} -v -line #__LINE__	# Invalid day

# TO FIX ##################
newline
textcolor darkyellow
textcolor
#assert [is_date "dummy"] $true	# Dummy test trigger

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #