#################################################################################
# File: 			test_item.tcl
# Purpose:			Tests for [item]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 26/Jul/2019	First version
#################################################################################
# Notes:
# [item] is a safe way to reference vars which may not exist
# A default value can be supplied, otherwise "" will be returned if not found
#################################################################################

# Check simple vars # # # # #
textcolor yellow
puts "Test simple vars ...\n"
textcolor

puts "Line: "#__LINE__
array unset a -nocomplain
unset q -nocomplain
assert [item q] {eq $_ ""} -v -line #__LINE__
assert [item q "x"] {eq $_ x} -v -line #__LINE__
set q Hello
puts "q is '$q'"
puts "q is '$q'"
#puts "q is '$q'"
#puts "q is '$q'"
assert [item q "x"] {eq $_ Hello} -v -line #__LINE__

puts "Line: "#__LINE__
assert [item a(1)] {eq $_ ""} -v -line #__LINE__
assert [item a(1) "x"] {eq $_ x} -v -line #__LINE__
set a(1) Hurray
assert $a(1) {eq $_ Hurray} -v -line #__LINE__
assert [item a(1)] {eq $_ Hurray} -v -line #__LINE__
assert [item a(2) "foo"] {eq $_ foo} -v -line #__LINE__
newline

# Check complex dereference values

puts "Line: "#__LINE__

# Check array # # # # # #
textcolor yellow
puts "Test array ...\n"
textcolor

# Complex / indirect array reference # # # # # #
puts "Line: "#__LINE__
set q "a"
puts "Checking complex reference '\${q}(1)' @"#__LINE__
# ${${q}(1)} will dereference a(1) -> "Hurray"
# ${q}(1) will resolve to 'a(1)'
assert [item ${q}(1)] {eq $_ Hurray} -v -line #__LINE__

puts "Checking complex reference '\${q}(\$r)' @"#__LINE__
set r 1
assert [item ${q}($r)] {eq $_ Hurray} -v -line #__LINE__

newline

# Check struct # # # # # #
textcolor yellow
puts "Test structs ...\n"
textcolor
struct st {a 10 b 20}
struct set st.a 12345
puts "st.a=='$st.a'"
assert [item st.a] {eq $_ 12345} -v -line #__LINE__
assert [item st.q] {eq $_ ""} -v -line #__LINE__
assert [item st.q "st.q nosuch"] {eq $_ "st.q nosuch"} -v -line #__LINE__
newline


#################################################################################
# NEW -literal 13/Nov/2019
puts "Line: "#__LINE__
assert [item {}] {eq $_ {}} -v -line #__LINE__
assert [item {} -literal] {eq $_ {}} -v -line #__LINE__
assert [item {} foo -literal] {eq $_ foo} -v -line #__LINE__
assert [item $qbf] {eq $_ {}} -v -line #__LINE__				# Looked for varname 'The quick...' (not found)
assert [item $qbf foo] {eq $_ foo} -v -line #__LINE__			# Looked for varname 'The quick...' (not found)
assert [item $qbf -literal] {eq $_ $qbf} -v -line #__LINE__		# Echo literal value 'The quick...'
set s ""
assert [item $s -literal] {eq $_  {}} -v -line #__LINE__		# 
assert [item $s foo -literal] {eq $_  foo} -v -line #__LINE__	# 
unset s

stack create stk 10
stack push stk Hello
assert [item [stack peek stk] -literal] {eq $_ Hello} -v -line #__LINE__
stack pop stk		# Stack is now empty
assert [item [stack peek stk] -literal] {eq $_ {}} -v -line #__LINE__
assert [item [stack peek stk] foo -literal] {eq $_ foo} -v -line #__LINE__
stack unset stk

/* Currently there are no error cases
set i 0
try {
	item -literal
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__	# */
newline

#################################################################################

array unset a -nocomplain
struct unset st -nocomplain
unset i q r -nocomplain
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Pass *"
textcolor

# EOF #
