############################################################################
# File: 			test_let.tcl
# Purpose:			Test [let]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 19/Dec/2016	First version
# 1.01 09/Jul/2017	Replacd [checkex] with [assert]
############################################################################
# Notes:
# ADD test set array target
############################################################################
/*
DONE let Lvalue  = Rvalue      Evaluate rvalue(s) as an expression then assign
DONE let Lvalue := Rvalue      Assign rvalue(s) without evaluating
DONE let Lvalue ++             Increment lvalue(s))
DONE let Lvalue --             Increment lvalue(s))
DONE let Lvalue += Rvalue      Assign to existing value. Rvalue may be expression
								Will cast to float before addition
DONE let Lvalue -= Rvalue      Assign to existing value. Rvalue may be expression
								Will cast to float before addition

let variable|array  = expression|variable|array|[command]  # Eval with expr
let variable|array := list|variable|array|[command]        # Assignment only
let variable|array  = {variable|array}   # Evaluate array contents with expr
let variable|array := {variable|array}   # Assignment of array contents only

############################################################################# */

option expression off
option echo on
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

textcolor yellow
puts "Testing \[let\] ..."
textcolor
newline

textcolor darkyellow
puts "Testing \[let\ x = y] ..."
textcolor
# let x = <whatever>
assert [let x = 1] {== $_ 1} -v -line #__LINE__
assert [let x = 1234.56789] {== $_ 1234.56789} -v -line #__LINE__
puts [round $pi 14]
let x = [expr 4*atan(1)]
puts [round $x 14]
assert $x {== [round $_ 14] [round $pi 14]} -v -line #__LINE__		# 3.141592653589792
assert [let x = 4*atan(1)] {== [round $_ 14] [round $pi 14]} -v -line #__LINE__		# 3.141592653589792


textcolor darkyellow
puts "Testing \[let\ x := y] ..."
textcolor
# let x := <whatever>
assert [let x := 1] {== $_ 1} -v -line #__LINE__
assert [let x := 1234.56789] {== $_ 1234.56789} -v -line #__LINE__
let x := [expr 4*atan(1)]
puts "x is: '$x'"
assert $x {== [round $_ 14] [round $pi 14]} -v -line #__LINE__		# 3.141592653589792
assert [let x := 4*atan(1)] {eq $_ "4*atan(1)"} -v -line #__LINE__		# 3.141592653589792

textcolor darkyellow
puts "Testing \[let\ x ++] ..."
textcolor
# let x ++
set q 10
assert [let q ++] {== $_ 11} -v -line #__LINE__
assert [let q --] {== $_ 10} -v -line #__LINE__


textcolor darkyellow
puts "Testing \[let\ x +=] ..."
textcolor
set q 10
assert [let q += 13] {== $_ 23} -v -line #__LINE__
set p 1
set q 2
set r 3
set s 4
assert [let p q r s += 13] {== $_ 17}
assert $p {== $_ 14} -v -line #__LINE__
assert $q {== $_ 15} -v -line #__LINE__
assert $r {== $_ 16} -v -line #__LINE__
assert $s {== $_ 17} -v -line #__LINE__

textcolor darkyellow
puts "Testing \[let\ x -=] ..."
textcolor
set q 36
assert [let q -= 13] {== $_ 23} -v -line #__LINE__
set p 33
set q 34
set r 35
set s 36
assert [let p q r s -= 13] {== $_ 23} -v -line #__LINE__
assert $p {== $_ 20} -v -line #__LINE__
assert $q {== $_ 21} -v -line #__LINE__
assert $r {== $_ 22} -v -line #__LINE__

textcolor darkyellow
puts "Testing \[let\]  init multiple vars (simple)..."
textcolor
# let multiple vars (simple)
assert [let p q r s = 23] {== $_ 23}
assert $p {== $_ 23} -v -line #__LINE__
assert $q {== $_ 23} -v -line #__LINE__
assert $r {== $_ 23} -v -line #__LINE__
assert $s {== $_ 23} -v -line #__LINE__

textcolor darkyellow
puts "Testing \[let\]  init multiple vars (= expression)..."
textcolor
# let multiple vars (= expression)
assert [let p q r s = (2*10)+3] {== $_ 23} -v -line #__LINE__
assert $p {== $_ 23} -v -line #__LINE__
assert $q {== $_ 23} -v -line #__LINE__
assert $r {== $_ 23} -v -line #__LINE__
assert $s {== $_ 23} -v -line #__LINE__

textcolor darkyellow
puts "Testing \[let\]  init multiple vars (:= expression)..."
textcolor
# let multiple vars (:= expression)
assert [let p q r s := (2*10)+3] {eq $_ "(2*10)+3"} -v -line #__LINE__
assert $p {eq $_ "(2*10)+3"} -v -line #__LINE__
assert $q {eq $_ "(2*10)+3"} -v -line #__LINE__
assert $r {eq $_ "(2*10)+3"} -v -line #__LINE__
assert $s {eq $_ "(2*10)+3"} -v -line #__LINE__

textcolor darkyellow
puts "Testing \[let\]  init multiple vars (:= expression)..."
textcolor
set p 1
set q 2
set r 3
set s 4
# let multiple vars (:= expression)
assert [let p q r s ++] {eq $_ 5} -v -line #__LINE__
assert $p {== $_ 2} -v -line #__LINE__
assert $q {== $_ 3} -v -line #__LINE__
assert $r {== $_ 4} -v -line #__LINE__
assert $s {== $_ 5} -v -line #__LINE__

textcolor darkyellow
puts "Testing \[let\]  init multiple vars (:= strings)..."
textcolor
# Date #
let x y z := "19/Dec/2016"
assert $x {eq $x "19/Dec/2016"}

assert [let x y z := "Hello world"] {eq $_ "Hello world"} -v -line #__LINE__		# FAILS wraps in braces
let x y z := $qbf
assert [subst [let x y z := $qbf]] {eq $_ "$::qbf"} -v -line #__LINE__		# remove {} using subst to test
# Admin@Lenovo 990> puts [let x y z := $qbf]
#{The quick brown fox jumps over the lazy dog}
## ARRAYS ######################################
textcolor darkyellow
puts "Testing \[let\ x() = y] ..."
textcolor
assert [let a(1) = 1] {== $_ 1} -v -line #__LINE__
assert [let a(1) = 1234.56789] {== $_ 1234.56789} -v -line #__LINE__
assert $a(1) {== $_ 1234.56789} -v -line #__LINE__
assert [let a(1) = [expr 4*atan(1)]] {== [round $_ 14] [round $::pi 14]} -v -line #__LINE__		# 3.141592653589792
assert [let a(1) = 4*atan(1)] {== [round $_ 14] [round $::pi 14]} -v -line #__LINE__		# 3.141592653589792
newline

textcolor darkyellow
puts "Testing \[let\ x() := y] ..."
textcolor
# let x = <whatever>
assert [let a(1) := 1] {== $_ 1} -v -line #__LINE__
assert [let a(1) := 1234.56789] {== $_ 1234.56789} -v -line #__LINE__
assert $a(1) {== $_ 1234.56789} -v -line #__LINE__
assert [let a(1) := [expr 4*atan(1)]] {== [round $_ 14] [round $::pi 14]} -v -line #__LINE__		# 3.141592653589792
assert [let a(1) := 4*atan(1)] {== $_ "4*atan(1)"} -v -line #__LINE__		# 3.141592653589792
newline 

textcolor darkyellow
puts "Testing \[let\ (multiple) x() := y] ..."
textcolor
assert [let a(1) b(2) c(3) = 1] {== $_ 1} -v -line #__LINE__
assert $a(1) {== $_ 1} -v -line #__LINE__
assert $b(2) {== $_ 1} -v -line #__LINE__
assert $c(3) {== $_ 1} -v -line #__LINE__
assert [let a(1) b(2) c(3) = 1234.56789] {== $_ 1234.56789} -v -line #__LINE__
assert $a(1) {== $_ 1234.56789} -v -line #__LINE__
assert $b(2) {== $_ 1234.56789} -v -line #__LINE__
assert $c(3) {== $_ 1234.56789} -v -line #__LINE__

assert $a(1) {== $_ 1234.56789} -v -line #__LINE__
assert [let a(1) b(2) c(3) = [expr 4*atan(1)]] {== [round $_ 14] [round $::pi 14]} -v -line #__LINE__		# 3.141592653589792
assert $a(1) {== $_ 3.141592653589792} -v -line #__LINE__
assert $b(2) {== $_ 3.141592653589792} -v -line #__LINE__
assert $c(3) {== $_ 3.141592653589792} -v -line #__LINE__

assert [let a(1) b(2) c(3) = 4*atan(1)] {== [round $_ 14] [round $::pi 14]}		# 3.141592653589792
assert $a(1) {== $_ 3.141592653589792} -v -line #__LINE__
assert $b(2) {== $_ 3.141592653589792} -v -line #__LINE__
assert $c(3) {== $_ 3.141592653589792} -v -line #__LINE__

assert [let a(1) b(2) c(3) := 4*atan(1)] {== $_ "4*atan(1)"}		# 3.141592653589792
assert $a(1) {eq $_ "4*atan(1)"} -v -line #__LINE__
assert $b(2) {eq $_ "4*atan(1)"} -v -line #__LINE__
assert $c(3) {eq $_ "4*atan(1)"} -v -line #__LINE__

array unset a b c -nocomplain
unset p q r s x y z -nocomplain

################################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #