###############################################################################################
# File:				test_lib_arp.tcl
# Author:			M Shaw
# Purpose:			Test file for [arp] in ticol_arp.dll
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 --/Nov/2019	First version
# 1.01 05/Jan/2022	Updated re: pings
###############################################################################################
# Notes:
# Note: test_lib_arp requires a LIVE internet connection for ping to fill the ARP table
###############################################################################################
/*
   arp ipv4-address ?-list?   # ARP lookup of IPV4 address
   arp table                  # Return the ARP table as a Tcl list
   rarp mac-address ?-list?   # Reverse ARP lookup. Look up in ARP table
*/
###############################################################################################
option expression off
lib ticol_arp -nocomplain

spawn - -echo ping www.google.com -n 1
spawn - -echo ping [info hostname] -n 1		# Ensure the ARP table has an entry
spawn - -echo ping localhost -n 1			# Ensure the ARP table has an entry
spawn - -echo ping [info hostname] -n 1		# Ensure the ARP table has an entry
spawn - -echo ping www.yahoo.com -n 1

sleep 20
set table [arp table]
assert $table {ne $_ {}} -v -line #__LINE__
set i 1
textcolor magenta
foreach x $table {
	puts "$i: $x"
	++ i
}
textcolor
newline
assert [arp 127.0.0.1] {ne $_ {}} -v -line #__LINE__
assert [arp localhost] {ne $_ {}} -v -line #__LINE__

lib ticol_arp -unload -nocomplain

unset i table x -nocomplain

#################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #
