########################################################################
# File: 			test_asm.tcl
# Purpose:			Tests ticol_asm.dll - ASM Pseudo-Interpreter plugin
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 17/Oct/2018	First version
# 1.01 01/Nov/2018	Added MSVCRT40.DLL (C Runtime) test
# 1.02 03/Nov/2018	Added box drawing test
########################################################################
# Notes:
# Test: PARADIGM	Common TicASM paradigms
# Test: BOX			Draw multiple boxes in a loop
# Test: LOOP		Loop test
# Test: MANDELBROT	Fractal	Mandelbrot
# Test: CRT			Call C Runtime MSVCRT40.DLL
#
# TicASM was added as an experiement in creating an optimised bytecode-
# like sub-interpreter
# FIX a static is set in BOX
# FIX NT4 requires a path
########################################################################
option expression off

#set path [addslash [escape [file dirname [file normalize [info script]]]]]
set path [addslash [info scriptpath]]
puts "The script path is '$path'"


textcolor cyan
lib ticol_asm -verbose -nocomplain
puts [lib ticol_asm -info]
textcolor
newline

textcolor yellow
puts "Testing paradigms ..."
textcolor

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# PARADIGM: 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
set s [readfile ${path}paradigm.asm]
if {eq $s ""} {die "Failed to read paradigm.asm"}
set r [asm $s]
puts "asm result in r='$r'"
assert $r {eq $_ "* All tests pass"} -v -line #__LINE__		# Number of iterations in script
unset r
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# BOX: 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#/*
puts "Line: "#__LINE__

set r [asm {
	; Variable declarations
	x:
	y:
	cmd:
	buf:
	buflen:
	cmd_cls:
	msg_bcd_fail:
	ptr:
	loops:
	str_solid:

	; Variable assignments
	db ? ""
	mov buflen, edp
	db ? "256"
	mov cmd, edp
	db ? "box                                                 "
	mov buf, edp
	db 257, ""
	mov cmd_cls, edp
	db ? "cls"
	mov x, edp
	db 21, ""
	mov y, edp
	db 21, ""
	mov msg_bcd_fail, edp
	db ? "BCD Conversion failed\n"
	mov str_solid, edp
	db ? "-solid"

	; Initialisation
	call cls
	mov loops, 500

	; Main loop
	mov bx, 0
loop:

	mov ptr, cmd			; Index ptr to command buffer at each iteration
	mov ax, ptr
	add ax, 4				; Skip over "box " command prefix
	mov ptr, ax				; Write back

	; x
	mov ax, 0x31
	int 0x21
	mul ax, 80
	inc ax
	; BCD conversion
	push buflen				; Push R to L
	push buf				; BCD return buffer
	push ax					; AX is argument for ITOA
	call itoa				; Returns length of converted number in AX
	cmp ax, 0				; Success?
	jz bcd_fail
	mov [ptr], buf
	; add ptr, ax			; increment buffer address (not currently supported)
	mov cx, ptr				; Copy ptr to a register
	add cx, ax				; Increment CX by returned string length
	mov [cx], 32			; Write space at current location
	inc cx					; Increment CX
	mov ptr, cx				; Write-back to ptr

	; y
	mov ax, 0x31
	int 0x21
	mul ax, 24
	inc ax
	; BCD conversion
	push buflen				; Push R to L
	push buf
	push ax
	call itoa				; Returns length of converted number
	cmp ax, 0
	jz bcd_fail
	mov [ptr], buf 
	;add ptr, ax			; increment buffer address
	mov cx, ptr				; Copy ptr to a register
	add cx, ax				; Increment CX by returned string length
	mov [cx], 32			; Write space at current location
	inc cx					; Increment CX
	mov ptr, cx				; Write-back to ptr

	; w
	mov ax, 0x31
	int 0x21
	mul ax, 30
	inc ax
	; BCD conversion
	push buflen				; Push R to L
	push buf
	push ax
	call itoa				; Returns length of converted number
	cmp ax, 0
	jz bcd_fail
	mov [ptr], buf 
	;add ptr, ax			; increment buffer address
	mov cx, ptr				; Copy ptr to a register
	add cx, ax				; Increment CX by returned string length
	mov [cx], 32			; Write space at current location
	inc cx					; Increment CX
	mov ptr, cx				; Write-back to ptr

	; h
	mov ax, 0x31
	int 0x21
	mul ax, 20
	inc ax
	; BCD conversion
	push buflen				; Push R to L
	push buf
	push ax
	call itoa				; Returns length of converted number
	cmp ax, 0
	jz bcd_fail
	mov [ptr], buf 
	; add ptr, ax			; increment buffer address
	mov cx, ptr				; Copy ptr to a register
	add cx, ax				; Increment CX by returned string length
	mov [cx], 32			; Write space at current location
	inc cx					; Increment CX
	mov ptr, cx				; Write-back to ptr
	
	; Random Box Colour
	mov ax, 0x31
	int 0x21
	mul ax, 14
	inc ax
	; BCD conversion
	push buflen				; Push R to L
	push buf
	push ax
	call itoa				; Returns length of converted number
	cmp ax, 0
	jz bcd_fail
							; Save the random integer in buf
	mov [ptr], buf 			; Copy buffer contents to address at ptr
	mov cx, ptr				; Copy ptr to a register
	add cx, ax				; Increment CX by returned string length
	mov [cx], 32			; Write space at current location
	inc cx					; Increment CX
	mov ptr, cx				; Save CX back to ptr address

	push ax
	push dx
	mov ax, 16				; Invert the background colour
	mov dx, [buf]
	sub ax, dx
	mov [buf], ax
	pop dx
	pop ax
	
	mov [ptr], buf 			; Append string at buf into location [ptr]
	push buf
	call strlen
	mov cx, ptr
	add cx, ax
	mov ptr, cx

	mov [cx], 32			; Write space at current location
	inc cx					; Increment CX

	mov ptr, cx				; Index into next location
	mov [ptr], str_solid	; " -solid"  (7)
	mov cx, ptr
	add cx, 7
	mov [cx], 0				; Write terminating NULL char to avoid 'junk' chars from last iteration

	; Call Tcl to draw the box using the command we generated
	; Tcl eval(arg, buffer, buflen) R -> L
	push buflen
	push buf
	push cmd
	call eval

	inc bx
	cmp bx, loops
	jnz loop				; -------------------------------------------------------

	mov [x], 1
	mov [y], 1
	push y
	push x
	call gotoxy

	mov ax, loops			; 'mov [buf], loops' will write contents at address 'loops'
	mov [buf], ax			; 'mov [buf], ax' will write a numeric value
	mov ax, buf
	jmp end 

cls:
	push 0					; We don't want a buffer length
	push 0					; We don't need a buffer return
	push cmd_cls
	call eval
	ret

bcd_fail:
	mov ax, msg_bcd_fail
	call 9

end:
	end
}]
puts "asm result in r='$r'"
assert $r {== $_ 500} -v -line #__LINE__		# Number of iterations in script
unset r
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# END BOX: 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#*/


puts "Line: "#__LINE__

set debug ""
unset s -nocomplain
set s {
	; ------------------------------------------------------------------------
	; Test file must:
	; 	Include comments
	; 	Include random whitespace at LH side of each opcode
	; 	Include random whitespace at RH side of each opcode
	; 	Loop for some value
	; ------------------------------------------------------------------------
;dump
;end
	org 100h			; Optional

	msg_1:
	msg_2:
	msg_start:
	msg_end:
	msg_loop_for:
	msg_items:
	MAX_LOOPS:

	mov msg_1, edp	
	db ? "Oh hai!"
	mov msg_2, edp
	db ? "Hello from DB"
	mov msg_start, edp
	db ? "* Start loops *"
	mov msg_end, edp
	db ? "* End loops *"
	mov msg_loop_for, edp
	db ? "Loop for "
	mov msg_items, edp	
	db ? " items"
	mov MAX_LOOPS, 10 	;1000000

	mov ax 10			; Green	
	call 3				; Set textcolour
	mov ax, msg_1		; Print message
	;dump
	call 9
	mov ax, 10		
	call 2
	mov ax, 7			
	call 3
	; halt
	push ax
	pop bx			; error
	; dump
	inc bx
	call L2
	inc bx
	inc bx
	dec bx
	push bx
	mov ax, msg_loop_for
	call 9
	mov ax, MAX_LOOPS
	call 8
	mov ax, msg_items
	call 9 
	mov ax, 10
	call 2
	mov ax, MAX_LOOPS
L1:
	dec ax
	jnz L1

	mov ax, msg_end
	call 9
	mov ax, 10
	call 2

	pop ax
	; dump
	call 8
	mov ax, 10
	call 2
	mov ax 14
	call 3
	mov ax, msg_1
	call 9
	mov ax, 10
	call 2
	mov ax 7
	call 3
	mov ax, msg_1
	end

L2:
	dump
	ret

	end }
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

puts "Line: "#__LINE__
#set filename "ticol_asm.asm"
#set s [readfile $filename]

set start [timer]
set r [asm $s $debug]
set end [timer]
assert $r {eq $_ "Oh hai!"} -v -line #__LINE__

newline
textcolor cyan
puts " Execution took [- $end $start] ms"
puts " TicAsm result in reg:ax is r:'$r'"
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# MANDELBROT 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

unset s -nocomplain
puts "Line: "#__LINE__
set s { 					; Wrap in braces not dquotes!
	; -- Mandelbrot --
	org 100h				; ORG will accept decimal, octal 0oNN, hex 0xNN, and hex NNNh

	; -- Declarations ----------------------------------------------------------------
	a:
	b:
	i:
	t:
	x:
	y:
	ca:
	cb:
	msg_debug:

	; -- Memory allocation/reservation -----------------------------------------------
	mov msg_debug, edp
	db ? "Debug: "
	mov a, edp
	db ? "                      "
	mov b, edp
	db ? "                      "
	mov i, edp
	db ? "                      "
	mov t, edp
	db ? "                      "
	mov x, edp
	db ? "                      "
	mov y, edp
	db ? "                      "
	mov ca, edp
	db ? "                      "
	mov cb, edp
	db ? "                      "
	mov edp, 0
	; dump

	; -- Run the code ----------------------------------------------------------------
	; 10 FOR Y=-12 TO 12 - Loop for -12 to +12
start:
	add ax, -12					; Y=-12
	mov [y], ax					; Set y = -1
	loop1:

	; 20 FOR X=-39 TO 39
	mov ax, -39					; x=-39 (lower values move to left) (difference between low and high == width)
	mov [x], ax					; Set x = -39

loop2:
	; 30 CA=X*0.0458
	mov ax, [x]
	fmul ax, 0.0458 			; X*0.0458 (check -39 * 0.0458 = -1.7862) PASS
	mov [ca], ax				; CA=X*0.0458
	
	; 40 CB= Y*0.08333
	mov ax, [y]					; Place Y into a free register
	fmul ax, 0.09333			; Y*0.08333 (check -1 * 0.08333 = -0.99996) PASS
	mov [cb], ax				; CB= Y*0.09333 (Should be 0.08333)

	; 50 A=CA
	mov ax, [ca]				; A=CA (can't be done directly) PASS
	mov [a], ax

	; 60 B=CB
	mov ax, [cb]				;B=CB PASS
	mov [b], ax
	
	; 70 FOR I=0 TO 15
	mov ax 0
	mov [i], ax

loop3:							; FOR I /////////////////////////////////////////////////////

	; 80 T=A*A-B*B+CA
	mov ax, [a]
	fmul ax, ax					; A*A (2: a*a:3.190510440000000)				PASS
	mov cx, ax					; Save A*A in CX
	mov bx, [b]
	fmul bx, bx					; B*B (3: b*b:1.254310401600000)				PASS
	mov dx, bx					; Save B*B in DX
	fsub ax, bx					; A*A-B*B -> ax  (a*a - b*b:1.936200038400000)	PASS
	mov bx, [ca]
	fadd ax, bx					; +CA
	mov [t], ax					; (t:0.150000038400000)							PASS
	
	
	; 90 B=2*A*B+CB
	mov ax, 2
	mov bx, [a]
	fmul ax, bx					; 2*A
	mov bx, [b]
	fmul ax, bx					; (2*A)*B
	mov bx, [cb]				; ((2*A)*B) + CB
	fadd ax, bx
	mov [b], ax					; 2.880985104000001 (2*-1.786200000000000*2.880985104000001)+-1.1199600000000 PASS

	; 100 A=T
	mov ax, [t]
	mov [a], ax					; A=T (t:0.150000038400000)						PASS
	
	; 110 IF (A*A+B*B)>4 THEN GOTO 200
;	mov ax, [a]
;	fmul ax, ax					; A*A
	mov ax, cx					; Redb ?e A*A from CX
;	mov bx, [b]					; We could optimise by saving earlier B*B in DX (if it was implemented)
;	fmul bx, bx					; B*B
;	mov bx, dx					; Redb ?e B*B from DX
;	fadd ax, bx					; (A*A+B*B)	(8.322575180989897)					PASS
	fadd ax, dx
	
	sub ax, 4					; >4, >=5
	jg label200					; IF (A*A+B*B)>4 THEN GOTO 200 (line after GOTO 210) - if sign 0

	; 120 NEXT I
	mov ax, [i]					; Retrieve i again
	inc ax						; add ax, 1
	mov [i], ax					; i=i+1
	mov cx, ax
	sub cx, 16					; Test IF > 15 (we must test +1 higher than 15). We loop to include 15

	jnz loop3					; END FOR..NEXT I (FOR I=0 TO 15) Line 120 - (Innermost loop)

	; 130 PRINT " ";
	mov ax, 32					; 32 (space) 42 *
	call 2						; Call BDOS print char

	; 140 GOTO 210
	jmp label210				; Line 140 - GOTO 210

	; 200 IF I>9 THEN I=I+7
label200:
	mov ax, [i]
	sub ax, 9
	jl skip1					; If -ve

	mov ax, [i]
	add ax, 7
	mov [i], ax

skip1:
	; Set text colour - Tcl: textcolor [+ [% $i 7] 1]
	mov ax [i]
	mod ax, 7
	inc ax 						; add ax, 1
	call 3

	; 205 PRINT CHR$(48+I);
	mov ax, [i]					; Retrieve i again as it could be 0
	add ax, 48					; i is in ax
	call 2						; Call BDOS print char 

	; 210 NEXT X (COLS)  FOR X=-39 TO 39
label210:
	mov ax, [x]					; Increment X and save
	inc ax 						; add ax, 1
	mov [x], ax
	sub ax, 39					; Test X (39) (add +1)
	jnz loop2					; NEXT X - (Inner loop)
	
	; END -- NEXT X (Inner loop) Line 210 --------------------------------------------
	; 220 PRINT
	mov ax, 10					; LF
	call 2

	; 230 NEXT Y (12)			; ROWS
	mov ax, [y]
	inc ax 						; add ax, 1
	mov [y], ax
	sub ax, 12					; We test for upper FOR limit +1
	jnz loop1					; NEXT Y - (Outer loop)
	mov dx, y					; Save result for tests.tcl (we must index to memory address)
	
	; Finish loop --------------------------------------------------------------------
	mov edp, 0					; Overwrite current vars
	db ? "* Program Done *"		; db ? a new string
	mov ax, 10
	call 3
	mov ax, 0
	call 9
	mov ax, 7
	call 3
	jmp end

end:
	; xor ax, ax
	 mov ax, dx					; Return loop count for tests
}

puts "Line: "#__LINE__
set start [timer]
set r [asm $s $debug]
set end [timer]
newline
puts "Took [- $end $start] ms\n"

puts "Line: "#__LINE__

assert $r {== $_ 12} -v -line #__LINE__
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# END MANDELBROT 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# CALL CRT:	sin(23.0) = -0.846220
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
textcolor yellow
puts "Testing ASM call to external MSVCRT40.DLL ..."
textcolor
newline

set r [asm {
	tan:						; Declare variables (required)
	cos:
	sin:
	sqrt:
	a:
	b:
	c:
	msvc:
	handle:
	foo:

	; Assign vars
	db ? "0"					; Guard byte
	mov sin, edp
	db ? "sin"					; MSVC runtime function
	mov a, edp
	db 21 ""					; Don't assign a value here or string will fill-repeat
	mov msvc, edp
	db ? "msvcrt40.dll"			; MSVC runtime DLL
	mov foo, edp
	db ? "foo"					; Tcl varname

	mov ax, msvc				; Index string with DLL name
	call dll_open				; Load MSVCRT40.dll
	cmp ax, 0					; Is zero ?
	jz end						; If zo then end (will return 0)
	mov handle, ax				; save the handle

								; sin(23)
	mov [a], 23.0				; MUST be in float format
	push a						; Push index address to number
	mov ax, handle				; Push the DLL handle
	mov bx, sin					; Address of our funct string, "sin"
	mov cx, 1					; x1 arg was pushed
	call dll_run_double			; Call using __stdcall (CALL will clean up)
	fpush ax					; Save result in AX
	call 8						; Call print literal number in AX
	mov ax, 10					; LF
	call 2						; Call print char in AX
	mov ax, handle				; Pass the DLL's handle
	call dll_close				; Close the DLL
	fpop ax						; Read result to ax
	mov [a], ax					; Copy to string var a

	push a						; Push Index into string
	push foo					; Push varname
	call setvar					; Set a Tcl variable
	call 8						; Call print literal number for Tcl result code in AX

	mov ax, a					; Return pointer to var in ax (will be copied)
end:
}]

puts "asm result in r='$r'"
assert $r {== $_ -0.846220} -v -line #__LINE__		# Ticol: -0.846220404175171

vars f*
assert [is_set foo] {== $_ $::true} -v -line #__LINE__	
assert $foo {== $_ -0.846220} -v -line #__LINE__		# Ticol: -0.846220404175171

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# END CRT
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

lib ticol_asm -unload -nocomplain

unset debug end foo path r s start -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #