########################################################################
# File: 			test_asm_file.tcl
# Purpose:			Tests ticol_asm.dll - File handling
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Nov/2018	First version
########################################################################
# Notes:
########################################################################
option expression off

textcolor cyan
lib ticol_asm -verbose
puts [lib ticol_asm -info]
textcolor
newline

puts "Line: "#__LINE__

set r [asm {
	filename:				; Variable declarations
	mode_r:
	mode_w:
	handle:
	buf:
	msg:
	msg_open_fail:
	msg_write_fail:

	db ? ""					; Guard byte
	mov msg, edp
	db ? "Hello from ASM\n"
	mov msg_open_fail, edp
	db ? "Failed to open test file\n"
	mov msg_write_fail, edp
	db ? "Failed to write to file\n"
	mov filename, edp
	db ?, "$testasm.txt"
	mov mode_r, edp
	db ? "r"
	mov mode_w, edp
	db ? "w"
	mov buf, edp
	db 256, ""
	;db 256, 0

	mov ax, 0xf				; Open file function
	mov bx, filename		; BX Filename
	mov cx, mode_w
	int 0x21
	mov handle, ax
	cmp ax, 0
	jz open_fail

	mov ax, 0x15			; Write file function
	mov bx, handle			; Open file handle
	mov cx, msg				; Our message
	mov dx, 0				; Whole string
	int 0x21
	cmp ax, 0
	jz write_fail

	mov ax, 0x10			; Close file function
	mov bx handle
	int 0x21

	mov ax, 0xf				; Open file function
	mov bx, filename		; BX Filename
	mov cx, mode_r
	int 0x21
	mov handle, ax
	call 8					; Print the file handle for Dx
	mov ax, 10
	call 2					; Call print char

	mov ax, 0x14			; Read file
	mov bx, handle			; BX File handle
	mov cx, buf				; CX Buffer
	mov dx, 14				; DX Length
	int 0x21				; Call O/S
							; Null terminate
	mov dx, buf				; Index into buffer address
	add dx, 14				; Add 14
	mov [dx], 0				; Set char at dx to NULL
	mov ax, cx				; Index AX back to buffer address in CX
	call 9					; Print the ASCIIZ char
	mov ax, 10				; Newline
	call 2					; Call print char

	mov ax, 0x10			; Close file
	mov bx handle
	int 0x21

	mov ax, buf				; Return the buffer to Tcl
	jmp end
	
open_fail:
	mov ax, msg_open_fail
	call 9
	jmp end
	
write_fail:
	mov ax, msg_write_fail
	call 9
	jmp end

end:
	end
}]

puts "asm result in r='$r'"
assert $r {== $_ "Hello from ASM"} -v -line #__LINE__		#

lib ticol_asm -unload -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

unset r -nocomplain

vars

textcolor green
puts "* Done *"
textcolor

# EOF #