#################################################################################
# File: 			test_bf.tcl
# Purpose:			Tests for ticol_bf.dll plugin
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Oct/2018	First version
#################################################################################
# Notes:
#################################################################################
option expression off
option echo on

textcolor yellow
puts "Test ticol_bf.dll ..."
textcolor
newline

lib ticol_bf -verbose

/* ////////////////////////////////////////////////////////////////////////
	Possible macro language
	Macro				# Description						# Brainfuck			# Assembler (Pseudo)
	set a 10			# Start var is a					# ++++++++++		# ? INCR ax * 10, ADD AX,10 ?
	while a				# while (current index varptr)		# [					# ? JZ loop
		++ b 7			# SHR var is next in sequence		# > +++++++			# ? ADD bx,7
	wend a				# index into var a using SHL		# <-]				# ? DECR ax
	put b				# index b and output				# >.				# ? printf bx
	
Implementation?:
set:
		set var_counter 0
		set vartable($varcounter) = $varname	# varcounter -> cell #
		vars could span many cells if we also keep count of size
		This would also allow strings
while:
		must move to varname index
		look up varname in $vartable(), get index
			if varname does not exist then create it
		sub from current position index to get # moves to move
		store current position in temp var
++ var:
		increment current cell value by 1 - emit "+"
-- var 		
	decrement current cell value by 1 - emit "-"
puts var:
	lookup var position index
	do math to get # of shifts to move
	move shifts then output	# emit no. of shifts and .

	
//////////////////////////////////////////////////////////////////////// */

set test "++++++++++\[>+++++++<-]>."
puts "[bf $test -color]\r\n"
assert [bf $test -c] {== $_ F} -v -line #__LINE__


# 1234     49 50 51 52 53
set test "		
	++++++++++\[>+++++<-\]>		# 50
	-.							# 1 (49)
	+.							# 2 (50)
	+.							# 3 (51)
	+.							# 4 (52)
	+.							# 5 (53)
"

puts "[bf $test -color]\r\n"
# string [bf script -highlight -nocompress -debug -color|-colour -kc -ks -c]
assert [bf $test -c] {== $_ 12345} -v -line #__LINE__

set test "
++++++++++\[>+++++++<-\]>		# 70 (ADD 10 WHILE SHR ADD 7 SHL ENDL SHR)
++.								# 72 H
--								# 70
<
++++++++++\[>+++<-\]>			# ADD 30
+
.								# 101 e
+++++++..						# ll
+++.							# o
"

puts "[bf $test -color]\r\n"
assert [bf $test -c] {== $_ Hello} -v -line #__LINE__

#ifdef mandelbrot
#puts [bf [readfile coffee.bf]]
puts [bf [readfile mandelbrot.bf] -color]

set s "
+++++ +++++             initialize counter (cell #0) to 10
\[                       use loop to set the next four cells to 70/100/30/10
    > +++++ ++              add  7 to cell #1
    > +++++ +++++           add 10 to cell #2 
    > +++                   add  3 to cell #3
    > +                     add  1 to cell #4
    <<<< -                  decrement counter (cell #0)
\]                   
> ++ .                  print 'H'
> + .                   print 'e'
+++++ ++ .              print 'l'
.                       print 'l'
+++ .                   print 'o'
> ++ .                  print ' '
<< +++++ +++++ +++++ .  print 'W'
> .                     print 'o'
+++ .                   print 'r'
----- - .               print 'l'
----- --- .             print 'd'
> + .                   print ''	(x21)
> .                     print '\n'	(xa)
"
/*

*/
#endif

set test "
	++++++++++\[>+++++++>++++++++++>+++>+<<<<-\]>++.>+.+++++++
	..+++.>++.<<+++++++++++++++.>.+++.------.--------.>+.>."
puts [tohex [bf $test -c]]
bf $test
assert [bf $test -c] {== $_ "Hello World!\n"} -v -line #__LINE__
newline

lib ticol_bf -verbose -unload

unset s1 s2 test -nocomplain

vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #