######################################################################################
# File:			test_bigbitshift.tcl
# Author:		M Shaw
# Purpose:		Test [big <<] [big >>]
# Language:		Ticol Tcl
#
# Versions:
# 1.00 03/Mar/2017	First version (test results)
# 1.01 22/Jun/2017	Replace [checkex] with [assert]
# 1.02 25/Nov/2019	Fixed bugs in 0x8000000000000000 range
######################################################################################
# Notes:
# DONE ADD bitmask limits
# DONE FIX bugs in big << and big >>
######################################################################################
/*
0xffffffffffffffff	18
18446744073709551615 20
*/

/*
puts 0xffffffffff
exit
*/
# FAILS (18446744073709551615)
#puts 0xffffffffffffffff
#puts 0xffff
# exit

/* ***********************************************************************************************
# DONE Improve << and >> handling of differing bit widths
# FIX look at sign-bit backfill handling
# FIX check/handle/trap -ve shifts
# DONE ADD SHR >> tests
# DONE ADD SHL << tests
# DONE ADD bit width argument to << and >>
# DONE FIX handle signed values properly
Bitwise Shift Operators
The shift operators shift their first operand left (<<) or right (>>) by the number of positions the second operand specifies. 

Syntax

shift-expression :

additive-expression
shift-expression << additive-expression
shift-expression >> additive-expression

Both operands must be integral values. 
These operators perform the usual arithmetic conversions; the type of the result is the type of the left operand after conversion. 
For leftward shifts, the vacated right bits are set to 0. 
For rightward shifts, the vacated left bits are filled based on the type of the first operand after conversion. 
If the type is unsigned, they are set to 0. Otherwise, they are filled with copies of the sign bit. 
For left-shift operators without overflow, the statement

expr1 << expr2 
is equivalent to multiplication by 2expr2. For right-shift operators, 

expr1 >> expr2 
is equivalent to division by 2expr2 if expr1 is unsigned or has a nonnegative value.

The result of a shift operation is undefined if the second operand is negative, 
or if the right operand is greater than or equal to the width in bits of the promoted left operand.
Since the conversions performed by the shift operators do not provide for overflow or underflow conditions, 
information may be lost if the result of a shift operation cannot be represented in the type of the first operand after conversion.

unsigned int x, y, z;

x = 0x00AA;
y = 0x5500;

z = ( x << 8 ) + ( y >> 8 );

In this example, x is shifted left eight positions and y is shifted right eight positions. 
The shifted values are added, giving 0xAA55, and assigned to z.
Shifting a negative value to the right yields half the absolute value, rounded down. 
For example, 253 (binary 11111111 00000011) shifted right one bit produces 127 (binary 11111111 10000001). A positive 253 shifts right to produce +126.
Right shifts preserve the sign bit. When a signed integer shifts right, the most-significant bit remains set. 
When an unsigned integer shifts right, the most-significant bit is cleared. 
If 0xF000 is unsigned, the result is 0x7800. 
If 0xF0000000 is signed, a right shift produces 0xF8000000. 
Shifting a positive number right 32 times produces 0xF0000000. Shifting a negative number right 32 times produces 0xFFFFFFFF.

*********************************************************************************************** */

/* Ch C++
DWORD 
0xff << 8  0xff00
0xff << 16 0xff0000
0xff << 24 0xff000000
0xff << 32 0xff
0xff << 40  0xff00
0xff << 48  0xff0000
0xff << 56  0xff000000
0xff << 64  0xff
0xff << 72  0xff00
*/
#/*

option echo on
option expression off

textcolor cyan
lib ticol_bignum -verbose -nocomplain
textcolor
newline

textcolor yellow
puts "Test Big Bitshifting \[big <<\] \[big >>\] ...\n"
textcolor

# WORD (16 bit) - PASS
# Shifts with no wrap
textcolor yellow
puts "Loop Bitshifting \[big <<\] ...\n"
textcolor
textcolor darkyellow
loop i 0 32 4 {
	#printf "\[inttohex \[<< 0xf $i 8\]\]=%s\r\n" [inttohex [<< 0xf $i 8]]
	printf "\[inttohex \[big << 0xf %2i 8\]\]='%20s'\r\n" $i [inttohex [big << 0xf $i 8]]
}
newline

	loop i 0 32 4 {
	printf "\[inttohex \[big << 0xff %2i 8\]\]='%20s'\r\n" $i [inttohex [big << 0xff $i 8]]
}
newline

set line #__LINE__
puts "Line: "#__LINE__
loop i 0 80 8 {
	printf "\[inttohex \[big << 0xff %2i 16\]\]='%20s'\r\n" $i [inttohex [big << 0xff $i 16]]
}
newline

set line #__LINE__
puts "Line: "#__LINE__
loop i 0 80 8 {
	printf "\[inttohex \[big << 0xff %2i 32\]\]='%20s'\r\n" $i [inttohex [big << 0xff $i 32]]
}
newline

# FIX 56 fails
set line #__LINE__
puts "Line: "#__LINE__
loop i 0 80 8 {
	printf "\[inttohex \[big << 0xff %2i 64\]\]='%20s'\r\n" $i [inttohex [big << 0xff $i 64]]
	if {== $i 56} {
		printf "\[inttohex \[big << 0xff %2i 64\]\]='%20s'\r\n" 63 [inttohex [big << 0xff 63 64]]
	}
}




printf "\r\n0xffffffffffffffff is %s\r\n" 0xffffffffffffffff		# FAILS
assert 0xffffffffffffffff {== $_ 18446744073709551615} -v -line #__LINE__			# Check translation in MPP
textcolor

textcolor yellow
set line #__LINE__
puts "\n$line: BYTE (8-bits) \[big <<\]"
textcolor

# BYTE (16-bit) - PASS
puts "Line: "#__LINE__
puts [big & 0xffff 8]
puts "Line: "#__LINE__
puts [big & 0xffff 0xf]		# PASS
puts "Line: "#__LINE__
puts [big & 0xffff 8 8]
puts "Line: "#__LINE__
assert [big << 0xf 0 8] {== $_ 0xf} -v -line #__LINE__
assert [big << 0xff 0 8] {== $_ 0xff} -v -line #__LINE__
assert [big << 0xf 4 8] {== $_ 0xf0} -v -line #__LINE__
assert [big << 0xff 8 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 16 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 24 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 32 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 40 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 48 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 56 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 64 8] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 72 8] {== $_ 0x0} -v -line #__LINE__

textcolor yellow
puts "\n$line: WORD (16-bits) \[big <<\]"
textcolor
# WORD (16-bit) - PASS
assert [big << 0xff 0 16] {== $_ 0xff} -v -line #__LINE__
assert [big << 0xff 8 16] {== $_ 0xff00} -v -line #__LINE__
puts [inttohex [big << 0xff 16 16]]
assert [big << 0xff 16 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 24 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 16 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 40 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 48 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 56 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 64 16] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 72 16] {== $_ 0x0} -v -line #__LINE__

# DWORD (32 bit) - PASS (native)
textcolor yellow
puts "\n$line: DWORD (32-bits) \[big <<\] \[big >>\] ..."
textcolor
assert [big << 0xff 8 32] {== $_ 0xff00} -v -line #__LINE__
assert [big << 0xff 16 32] {== $_ 0xff0000} -v -line #__LINE__
assert [big << 0xff 24 32] {== $_ 0xff000000} -v -line #__LINE__
# Have hard-coded no-wrap limit at 32 bits
# DWORD will otherwise, wrap
/*	// Original CRT results (which are bad)
assert [big << 0xff 32 32] {== $_ 0xff} -v -line #__LINE__						# Full shift range
assert [big << 0xff 40 32] {== $_ 0xff00} -v -line #__LINE__
assert [big << 0xff 48 32] {== $_ 0xff0000} -v -line #__LINE__
assert [big << 0xff 56 32] {== $_ 0xff000000} -v -line #__LINE__
assert [big << 0xff 64 32] {== $_ 0xff} -v -line #__LINE__
assert [big << 0xff 72 32] {== $_ 0xff00} -v -line #__LINE__
*/
assert [big << 0xff 32 32] {== $_ 0x0} -v -line #__LINE__						# Full shift range
assert [big << 0xff 40 32] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 48 32] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 56 32] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 64 32] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 72 32] {== $_ 0x0} -v -line #__LINE__

# QWORD (64 bit) - PASS (default)
textcolor yellow
puts "\n$line: QWORD/__uint64 (64-bits) \[big <<\]"
textcolor
assert [big << 0xff 8 64] {== $_ 0xff00} -v -line #__LINE__
assert [big << 0xff 16 64] {== $_ 0xff0000} -v -line #__LINE__
assert [big << 0xff 24 64] {== $_ 0xff000000} -v -line #__LINE__

#puts "check:"0xff0000000000							# DONE FIX BUG in RNDC
#puts "check:"0xff00000000

assert [big << 0xff 32 64] {== $_ 0xff00000000} -v -line #__LINE__		# FAIL x
assert [big << 0xff 40 64] {== $_ 0xff0000000000} -v -line #__LINE__
assert [big << 0xff 48 64] {== $_ 0xff000000000000} -v -line #__LINE__
assert [big << 0xff 56 64] {== $_ 0xff00000000000000} -v -line #__LINE__
assert [big << 0xff 63 64] {== $_ 0x8000000000000000} -v -line #__LINE__
assert [big << 0xff 64 64] {== $_ 0x0} -v -line #__LINE__					# Full shift range
assert [big << 0xff 72 64] {== $_ 0x0} -v -line #__LINE__
assert [big << 0xff 96 64] {== $_ 0x0} -v -line #__LINE__

# SHR # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

set line #__LINE__
puts "Line: "#__LINE__
# WORD (16 bit) - PASS
# Shifts with no wrap
textcolor yellow
puts "Loop Bitshifting \[big >>\] ...\n"
textcolor

textcolor cyan
set line #__LINE__
puts "Line: "#__LINE__
loop i 0 32 4 {
	#printf "\[inttohex \[>> 0xf $i 8\]\]=%s\r\n" [inttohex [>> 0xf $i 8]]
	printf "\[inttohex \[big >> 0xf000 %2i 8\]\]='%20s'\r\n" $i [inttohex [big >> 0xf $i 8]]
}
newline

set line #__LINE__
puts "Line: "#__LINE__
loop i 0 32 4 {
	printf "\[inttohex \[big >> 0xff00 %2i 8\]\]='%20s'\r\n" $i [inttohex [big >> 0xff $i 8]]
}
newline

set line #__LINE__
puts "Line: "#__LINE__
loop i 0 80 8 {
	printf "\[inttohex \[big >> 0xff %2i 16\]\]='%20s'\r\n" $i [inttohex [big >> 0xff $i 16]]
}
newline

set line #__LINE__
puts "Line: "#__LINE__
loop i 0 80 8 {
	printf "\[inttohex \[big >> 0xff %2i 32\]\]='%20s'\r\n" $i [inttohex [big >> 0xff $i 32]]
}
newline

# FIX 56 fails
set line #__LINE__
puts "Line: "#__LINE__
loop i 0 80 8 {
	printf "\[inttohex \[big >> 0xff %2i 64\]\]='%20s'\r\n" $i [inttohex [big >> 0xff $i 64]]
	if {== $i 56} {
		printf "\[inttohex \[big >> 0xff %2i 64\]\]='%20s'\r\n" 63 [inttohex [big >> 0xff 63 64]]
	}
}


# BYTE (8 bit)
textcolor yellow
set line #__LINE__
puts "\n$line: BYTE (8-bits) \[big >>\]"
textcolor
set line #__LINE__
# BYTE (16-bit) - FAIL
assert [big >> 0xf 0 8] {== $_ 0xf} -v -line #__LINE__
assert [big >> 0xff 0 8] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xf 4 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 4 8] {== $_ 0xf} -v -line #__LINE__
assert [big >> 0xf 8 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 8 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 16 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 24 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 32 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 40 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 48 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 56 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 64 8] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 72 8] {== $_ 0x0} -v -line #__LINE__

textcolor yellow
puts "\n$line: WORD (16-bits) \[big >>\]"
textcolor
# WORD (16-bit) - PASS
assert [big >> 0xff 0 16] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xf 8 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 8 16] {== $_ 0x0} -v -line #__LINE__
#0xff00
assert [big >> 0xff00 0 16] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff00 8 16] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff00 16 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 24 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 32 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 40 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 48 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 56 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 64 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 72 16] {== $_ 0x0} -v -line #__LINE__
# 0xffff
assert [big >> 0xffff 0 16] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffff 8 16] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffff 16 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 24 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 32 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 40 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 48 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 56 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 64 16] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 72 16] {== $_ 0x0} -v -line #__LINE__

# DWORD (32 bit) - PASS (native)
textcolor yellow
set line #__LINE__
puts "\n$line: DWORD (32-bits) \[big >>\] ..."
textcolor
assert [big >> 0xff 0 32] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xf 8 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 8 32] {== $_ 0x0} -v -line #__LINE__
#0xff00
assert [big >> 0xff00 0 32] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff00 8 32] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff00 16 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 24 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 32 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 40 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 48 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 56 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 64 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 72 32] {== $_ 0x0} -v -line #__LINE__
# 0xffff	WORD MAX
assert [big >> 0xffff 0 32] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffff 8 32] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffff 16 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 24 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 32 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 40 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 48 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 56 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 64 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 72 32] {== $_ 0x0} -v -line #__LINE__
# 0xff0000
assert [big >> 0xff0000 0 32] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff0000 8 32] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff0000 16 32] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff0000 24 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 32 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 40 32] {== $_ 0x0} -v -line #__LINE__
# 0xff00000000
assert [big >> 0xff000000 0 32] {== $_ 0xff000000} -v -line #__LINE__
assert [big >> 0xff000000 8 32] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff000000 16 32] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff000000 24 32] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff000000 32 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff000000 40 32] {== $_ 0x0} -v -line #__LINE__
# 0xffffffff	DWORD MAX
assert [big >> 0xffffffff 0 32] {== $_ 0xffffffff} -v -line #__LINE__
assert [big >> 0xffffffff 8 32] {== $_ 0xffffff} -v -line #__LINE__
assert [big >> 0xffffffff 16 32] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffffffff 24 32] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffffffff 32 32] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffff 40 32] {== $_ 0x0} -v -line #__LINE__

# QWORD (64 bit) - PASS (default)
textcolor yellow
puts "\n$line: QWORD/__uint64 (64-bits) \[big >>\]"
textcolor
assert [big >> 0xff 0 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xf 8 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff 8 64] {== $_ 0x0} -v -line #__LINE__
#0xff00
assert [big >> 0xff00 0 64] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff00 8 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff00 16 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 24 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 32 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 40 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xffff	WORD MAX
assert [big >> 0xffff 0 64] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffff 8 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffff 16 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 24 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 32 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 40 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffff 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xff0000
assert [big >> 0xff0000 0 64] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff0000 8 64] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff0000 16 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff0000 24 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 32 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 40 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xff00000000
assert [big >> 0xff000000 0 64] {== $_ 0xff000000} -v -line #__LINE__
assert [big >> 0xff000000 8 64] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff000000 16 64] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff000000 24 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff000000 32 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff000000 40 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff000000 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff000000 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff000000 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff000000 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xffffffff	DWORD MAX
assert [big >> 0xffffffff 0 64] {== $_ 0xffffffff} -v -line #__LINE__
assert [big >> 0xffffffff 8 64] {== $_ 0xffffff} -v -line #__LINE__
assert [big >> 0xffffffff 16 64] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffffffff 24 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffffffff 32 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffff 40 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffff 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffff 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffff 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffff 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xffffffffff	DWORD + BYTE
assert [big >> 0xffffffffff 0 64] {== $_ 0xffffffffff} -v -line #__LINE__
assert [big >> 0xffffffffff 8 64] {== $_ 0xffffffff} -v -line #__LINE__
assert [big >> 0xffffffffff 16 64] {== $_ 0xffffff} -v -line #__LINE__
assert [big >> 0xffffffffff 24 64] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffffffffff 32 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffffffffff 40 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffff 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffff 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffff 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffff 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xffffffffff	DWORD + WORD (8+4)
assert [big >> 0xffffffffffff 0 64] {== $_ 0xffffffffffff} -v -line #__LINE__
assert [big >> 0xffffffffffff 8 64] {== $_ 0xffffffffff} -v -line #__LINE__
assert [big >> 0xffffffffffff 16 64] {== $_ 0xffffffff} -v -line #__LINE__
assert [big >> 0xffffffffffff 24 64] {== $_ 0xffffff} -v -line #__LINE__
assert [big >> 0xffffffffffff 32 64] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffffffffffff 40 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffffffffffff 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffff 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffff 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffff 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xffffffffff	DWORD + WORD (8+4)
assert [big >> 0xff0000000000 0 64] {== $_ 0xff0000000000} -v -line #__LINE__
assert [big >> 0xff0000000000 8 64] {== $_ 0xff00000000} -v -line #__LINE__
assert [big >> 0xff0000000000 16 64] {== $_ 0xff000000} -v -line #__LINE__
assert [big >> 0xff0000000000 24 64] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff0000000000 32 64] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff0000000000 40 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff0000000000 48 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000000000 56 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000000000 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff0000000000 72 64] {== $_ 0x0} -v -line #__LINE__

# 0xffffffffffffffff	QWORD (16)
assert [big >> 0xffffffffffffffff 0 64] {== $_ 0xffffffffffffffff} -v -line #__LINE__
puts [inttohex [big >> 0xffffffffffffffff 8 64]]
puts [inttohex [big >> 0xffffffffffffffff 16 64]]
puts [inttohex [big >> 0xffffffffffffffff 32 64]]
puts [inttohex [big >> 0xffffffffffffffff 48 64]]
puts [inttohex [big >> 0xffffffffffffffff 56 64]]
puts [inttohex [big >> 0xffffffffffffffff 63 64]]
puts [inttohex [big >> 0xffffffffffffffff 64 64]]
newline
puts "Shift 0xffffffffffffffff on base 64"
assert [big >> 0xffffffffffffffff 8 64] {== $_ 0xffffffffffffff} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 16 64] {== $_ 0xffffffffffff} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 24 64] {== $_ 0xffffffffff} -v -line #__LINE__		# FAILS ON LOOP
assert [big >> 0xffffffffffffffff 32 64] {== $_ 0xffffffff} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 40 64] {== $_ 0xffffff} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 48 64] {== $_ 0xffff} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 56 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 72 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 80 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 88 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xffffffffffffffff 96 64] {== $_ 0x0} -v -line #__LINE__

# 0xff00000000000000	QWORD (16)
assert [big >> 0xff00000000000000 0 64] {== $_ 0xff00000000000000} -v -line #__LINE__
puts [inttohex [big >> 0xff00000000000000 8 64]]
puts [inttohex [big >> 0xff00000000000000 16 64]]
puts [inttohex [big >> 0xff00000000000000 32 64]]
puts [inttohex [big >> 0xff00000000000000 48 64]]
puts [inttohex [big >> 0xff00000000000000 56 64]]
puts [inttohex [big >> 0xff00000000000000 63 64]]
puts [inttohex [big >> 0xff00000000000000 64 64]]
assert [big >> 0xff00000000000000 8 64] {== $_ 0xff000000000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 16 64] {== $_ 0xff0000000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 24 64] {== $_ 0xff00000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 32 64] {== $_ 0xff000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 40 64] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff00000000000000 48 64] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff00000000000000 56 64] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff00000000000000 64 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 72 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 80 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 88 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 96 64] {== $_ 0x0} -v -line #__LINE__

puts "Line: "#__LINE__
puts "[big >> 0xff00000000000000 0]\t[inttohex [big >> 0xff00000000000000 0]]"
puts "[big >> 0xff00000000000000 8]\t[inttohex [big >> 0xff00000000000000 8]]"
puts "[big >> 0xff00000000000000 16]\t[inttohex [big >> 0xff00000000000000 16]]"
puts "[big >> 0xff00000000000000 32]\t[inttohex [big >> 0xff00000000000000 32]]"
puts "[big >> 0xff00000000000000 48]\t[inttohex [big >> 0xff00000000000000 48]]"
puts "[big >> 0xff00000000000000 56]\t[inttohex [big >> 0xff00000000000000 56]]"
puts "[big >> 0xff00000000000000 63]\t[inttohex [big >> 0xff00000000000000 63]]"
puts "[big >> 0xff00000000000000 64]\t[inttohex [big >> 0xff00000000000000 64]]"
newline

# 0xff00000000000000	QWORD (16) (With default bitness of 64)
assert [big >> 0xff00000000000000 0] {== $_ 0xff00000000000000} -v -line #__LINE__		# 16 nybbles
assert [big >> 0xff00000000000000 8] {== $_ 0xff000000000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 16] {== $_ 0xff0000000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 24] {== $_ 0xff00000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 32] {== $_ 0xff000000} -v -line #__LINE__
assert [big >> 0xff00000000000000 40] {== $_ 0xff0000} -v -line #__LINE__
assert [big >> 0xff00000000000000 48] {== $_ 0xff00} -v -line #__LINE__
assert [big >> 0xff00000000000000 56] {== $_ 0xff} -v -line #__LINE__
assert [big >> 0xff00000000000000 63] {== $_ 0x1} -v -line #__LINE__
assert [big >> 0xff00000000000000 64] {== $_ 0x0} -v -line #__LINE__
newline 

# 8 byte / 64 bit 0xff00000000000000	QWORD (16) (With default bitness of 64)
assert [big >> 0xff00000000000000 0]  {== $_ 0xff00000000000000} -v -line #__LINE__		# 14 nybbles
assert [big >> 0xff00000000000000 8]  {== $_ 0xff000000000000} -v -line #__LINE__		# 14 nybbles
assert [big >> 0xff00000000000000 16] {== $_ 0xff0000000000} -v -line #__LINE__			# 12 nybbles
assert [big >> 0xff00000000000000 24] {== $_ 0xff00000000} -v -line #__LINE__			# 10 nybbles
assert [big >> 0xff00000000000000 32] {== $_ 0xff000000} -v -line #__LINE__				# 8 nybbles
assert [big >> 0xff00000000000000 40] {== $_ 0xff0000} -v -line #__LINE__				# 6 nybbles
assert [big >> 0xff00000000000000 48] {== $_ 0xff00} -v -line #__LINE__					# 4 nybbles
assert [big >> 0xff00000000000000 56] {== $_ 0xff} -v -line #__LINE__					# 2 nybbles
assert [big >> 0xff00000000000000 64] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 72] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 80] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 88] {== $_ 0x0} -v -line #__LINE__
assert [big >> 0xff00000000000000 96] {== $_ 0x0} -v -line #__LINE__
newline

# MAN example #
loop i 0 40 8 {
	printf "\[inttohex \[big << 0xff %2i 32\]\]='%20s'\r\n" $i\
	 [inttohex [big << 0xff $i 32]]
}

unset i line -nocomplain
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #