###########################################################################
# Name:				test_bignum_DEBUG.tcl
# Purpose:			Test file for bignum integer maths
# Author:       	M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Feb/2016	First version
# 1.01 22/Jul/2017	Replaced [checkex] with [assert]
# 1.01 28/Aug/2018	Debug version
###########################################################################
# Notes:
# Tests:
#	Leading zero combinations
#	Mixed sign combinations
#	Internal digit reversal/re-reversal
# ADD trap error cases (missing args etc.)
###########################################################################
/*
DONE +
DONE -
DONE *
DONE /
%
++
--
==
!=
<
>
<=
>=
&
|
||
^
~
!
+=
-=
*=
/=
&=
|=
^=
DONE fdiv
fmod
div10
**
pow
pow2
mul2
div2
<<
>>
sqrt
abs
fib
fact
is_even
inttobin
bintoint
max
min
rnd
mean
###################################################################################### */
option expression push on

textcolor cyan
lib ticol_bignum.dll -verbose -replace
puts "Library version v[lib ticol_bignum -info]"
textcolor
newline


# fdiv - next FIX 31/Aug/2018
assert [big fdiv 22 7.0] {== $_ 3.142857142857142857} -v -line #__LINE__

textcolor cyan
puts "check=[* 4115226303 3]"
textcolor
assert [big / 12345678910 3]  {== $_ [/ 12345678910 3]}	 -v -line #__LINE__
assert [big / 123098234023489877983876868457546465145140595419696447695 323279997879738777299] {== $_ 380778999105545731892675845428678805}  -v -line #__LINE__ 	# 1.230982340234899e56 ecalc

# Temporary new tests - Aug/2018
assert [big + 1 1 ] {== $_ 2} -v -line #__LINE__			# OK
assert [big + 1 -1 ] {== $_ 0} -v -line #__LINE__			# FIXED GPF
assert [big + -1 1 ] {== $_ 0} -v -line #__LINE__			# OK
assert [big + 1 2 ] {== $_ 3} -v -line #__LINE__			# OK
assert [big - 3 2 ] {== $_ 1} -v -line #__LINE__			# OK
assert [big + 123 456 ] {== $_ 579} -v -line #__LINE__		# OK

/*
lib ticol_sci
puts [sci [big + 12345678901234567890 12345678901234567890]]
puts "dec 2.469136e+019 -> [dec 2.469136e+019]"
assert [sci [big + 12345678901234567890 12345678901234567890]] {== $_ "2.469136e+019"} -v -line #__LINE__			# OK
# */


# DONE FIX corrupts i
set i 12345
big ++ i
assert $i {== $_ 12346} -v -line #__LINE__			# PASS
big -- i
assert $i {== $_ 12345} -v -line #__LINE__			# PASS GPF
#unset i


puts "Line :"#__LINE__
assert [big * 3 2 ] {== $_ 6} -v -line #__LINE__			# FIXED FAIL, DONE GPF
assert [big pow 1 1 ] {== $_ 1} -v -line #__LINE__			# PASS
assert [big pow 1 2 ] {== $_ 1} -v -line #__LINE__			# PASS
assert [big pow 2 2 ] {== $_ 4} -v -line #__LINE__			# PASS
assert [big pow 2 4 ] {== $_ 16} -v -line #__LINE__			# PASS
assert [big fact 10] {== $_ 3628800} -v -line #__LINE__	# PASS
assert [big fib 21] {== $_ [fib 21]} -v -line #__LINE__		# PASS
assert [big * 0 1 ] {== $_ 0} -v -line #__LINE__			# PASS
assert [big * 0 -1 ] {== $_ 0} -v -line #__LINE__			# PASS
assert [big * -0 -1 ] {== $_ 0} -v -line #__LINE__			# PASS
assert [big * -0 1 ] {== $_ 0} -v -line #__LINE__			# PASS
assert [big * 1 1 ] {== $_ 1} -v -line #__LINE__			# PASS
assert [big * 1 -1 ] {== $_ -1} -v -line #__LINE__			# PASS
assert [big * -1 1 ] {== $_ -1} -v -line #__LINE__			# PASS
assert [big * -1 -1 ] {== $_ 1} -v -line #__LINE__			# PASS
assert [big * 1 2 ] {== $_ 2} -v -line #__LINE__			# PASS
assert [big * 2 3 ] {== $_ 6} -v -line #__LINE__			# PASS
assert [big * 4 23 ] {== $_ 92} -v -line #__LINE__			# PASS
assert [big * 12345 67890 ] {== $_ 838102050} -v -line #__LINE__			# PASS

assert [big / 0 1 ] {== $_ 0} -v -line #__LINE__			# PASS
assert [big / 1 1 ] {== $_ 1} -v -line #__LINE__			# PASS
assert [big / 3 1 ] {== $_ 3} -v -line #__LINE__			# PASS
assert [big / -3 1 ] {== $_ -3} -v -line #__LINE__			# PASS
# 6.465599957594775e20
# 323279997879738777299
#                     2
# ---------------------
# 646569995759477554598
assert [big * 323279997879738777299 2] {== $_ 646559995759477554598} -v -line #__LINE__	# manual calc
assert [big * 32327999787973877729900000 2] {== $_ 64655999575947755459800000} -v -line #__LINE__	# manual calc
#  323279997879738777299
# 3232799978797387772990
# 3556079976677126550289
assert [big * 323279997879738777299 11] {== $_ 3556079976677126550289} -v -line #__LINE__	# manual calc
# 
#stop
# FAILS not 123098234023489877983876868457546465499657687777776887686
# is:		123098234023489877983876868457546465145140595419696447695
# Returns correct result (380778999105545731892675845428678805) but GPF

assert [big / 123098234023489877983876868457546465145140595419696447695 323279997879738777299] {== $_ 380778999105545731892675845428678805}	 -v -line #__LINE__ # 1.230982340234899e56 ecalc
assert [big * 323279997879738777299 380778999105545731892675845428678805] {== $_ 123098234023489877983876868457546465145140595419696447695}	 -v -line #__LINE__ # 1.230982340234899e56 ecalc

#stop
# Broken cases...
# *, -
#assert [big / 123098234023489877983876868457546465499657687777776887686 323279997879738777299] {== $_ 380778999105545731892675845428678806} -v -line #__LINE__ # ActiveState
#assert [big % 123098234023489877983876868457546465499657687777776887686 323279997879738777299] {== $_ 31237094478341662692} -v -line #__LINE__ # ActiveState
#puts [big - 123098234023489877983876868457546465499657687777776887686 [big * 323279997879738777299 380778999105545731892675845428678805]]

assert [big mean 1234 5678 9101] {== $_ 5337.666666666666666666} -v -line #__LINE__ # GPF
assert [big mean 123413241324 234234234 234234234 2342379879797987 -2234234234234199987987 234 -2] {== $_ -319175984532919782853.714285714285714284} -v -line #__LINE__ # eCalc -3.191759845329197e20 PASS
assert [big mean 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15] {== $_ 8} -v -line #__LINE__ # eCalc v8
#stop

#assert [big / 2343242346 2 ] {== $_ 1171621173} -v -line #__LINE__		# FAIL 
#assert [big / 3 -1 ] {== $_ -3} -v -line #__LINE__		#  INIFINITE LOOP This was bad! 
#assert [big / -3 -1 ] {== $_ 3} -v -line #__LINE__		#  INIFINITE LOOP
#assert [big + 11 -22] {== $_ -11} -v -line #__LINE__		# FAIL sign not set to -
#stop



newline
textcolor yellow
puts "Checking big sqrt at line "#__LINE__
textcolor

big debug off

assert [big sqrt 256] {== $_ 16} -v -line #__LINE__
assert [big sqrt 256] {== $_ [funct sqrt 256]} -v -line #__LINE__


# Exact sqrt series #
puts "Line: "#__LINE__
option echo on
option expression on

/* ###############################################
option preprocessor off
option
halt
puts "Line: "#__LINE__
for {set i 0} {$i < 100} {++ i} {
	puts [big sqrt $i]
}
puts "Line: "#__LINE__
for {set i 0} {$i < 100} {++ i} {
	puts $i\t -nonewline
}
############################################### */

option expression on
puts "Line: "#__LINE__
for {set i 0} {$i < 100} {++ i} {
/*
	puts "Line: "#__LINE__
	set n [big sqrt $i]
	if {[* $n $n] == $i } {
		puts "$i: exact sqrt($n) == [* $n $n]" 
	} else {
		#puts "$i: exact $n != [* $n $n]" 
	}
*/
}

puts "Line: "#__LINE__
big debug on

/*
set x 96142342342342342342341324535235345		# Some arbitrarily large number
puts "Calculating sqrt of $x"
set n [big sqrt $x]
puts "sqrt $x == $n (reverse assert [big * $n $n])"
assert [big - $x [big * $n $n]] {big == $_ 225074475306267949} -v -line #__LINE__	# x does not divide exactly. We have a remainder here of 225074475306267949
# eCalc assert 3.10068286579493200e17 -> 310068286579493200 - our result is 310068286579492586
*/

newline
textcolor yellow
puts "big + @"#__LINE__
textcolor

# Unsigned
assert [big + 0 0] {== $_ 0} -v -line #__LINE__
assert [big + 0 1] {== $_ 1} -v -line #__LINE__
assert [big + 1 0] {== $_ 1} -v -line #__LINE__
assert [big + 1 1] {== $_ 2} -v -line #__LINE__

# Signed -ve 0
assert [big + -0 0] {== $_ 0} -v -line #__LINE__
assert [big + 0 -0] {== $_ 0} -v -line #__LINE__
assert [big + -0 -0] {== $_ 0} -v -line #__LINE__		# FIX should be 0 not -0

# Signed +ve 0
assert [big + +0 0] {== $_ 0} -v -line #__LINE__
assert [big + 0 +0] {== $_ 0} -v -line #__LINE__
assert [big + +0 +0] {== $_ 0} -v -line #__LINE__

# Signed +ve 1
assert [big + 1 1] {== $_ 2} -v -line #__LINE__
assert [big + +1 1] {== $_ 2} -v -line #__LINE__
assert [big + 1 +1] {== $_ 2} -v -line #__LINE__
assert [big + +1 +1] {== $_ 2} -v -line #__LINE__

# Signed mixed 1
assert [big + 1 -1] {== $_ 0} -v -line #__LINE__
assert [big + -1 1] {== $_ 0} -v -line #__LINE__
assert [big + -1 +1] {== $_ 0} -v -line #__LINE__
assert [big + +1 -1] {== $_ 0} -v -line #__LINE__
assert [big + -1 -1] {== $_ -2} -v -line #__LINE__

# Double digits (leading zero)
assert [big + 00 00] {== $_ 0} -v -line #__LINE__
assert [big + 00 01] {== $_ 1} -v -line #__LINE__	# DONE FAIL
assert [big + 01 00] {== $_ 1} -v -line #__LINE__
assert [big + 01 01] {== $_ 2} -v -line #__LINE__
assert [big + -00 00] {== $_ 0} -v -line #__LINE__
assert [big + 00 -00] {== $_ 0} -v -line #__LINE__
assert [big + -00 -00] {== $_ 0} -v -line #__LINE__		# FIX should be 0 not -0
assert [big + +00 00] {== $_ 0} -v -line #__LINE__
assert [big + 00 +00] {== $_ 0} -v -line #__LINE__
assert [big + +00 +00] {== $_ 0} -v -line #__LINE__
assert [big + 01 01] {== $_ 2} -v -line #__LINE__
assert [big + +01 01] {== $_ 2} -v -line #__LINE__
assert [big + 01 +01] {== $_ 2} -v -line #__LINE__
assert [big + +01 +01] {== $_ 2} -v -line #__LINE__
assert [big + 01 -01] {== $_ 0} -v -line #__LINE__
assert [big + -01 01] {== $_ 0} -v -line #__LINE__
assert [big + -01 +01] {== $_ 0} -v -line #__LINE__
assert [big + +01 -01] {== $_ 0} -v -line #__LINE__
assert [big + -01 -01] {== $_ -2} -v -line #__LINE__

# Double digits (leading value)
assert [big + 10 10] {== $_ 20} -v -line #__LINE__
assert [big + 10 11] {== $_ 21} -v -line #__LINE__
assert [big + 11 11] {== $_ 22} -v -line #__LINE__
assert [big + -10 10] {== $_ 0} -v -line #__LINE__
assert [big + 10 -10] {== $_ 0} -v -line #__LINE__
assert [big + -10 -10] {== $_ -20} -v -line #__LINE__		# FIX unwanted zero sign - should be 0 not -0
assert [big + +10 10] {== $_ 20} -v -line #__LINE__
assert [big + 10 +10] {== $_ 20} -v -line #__LINE__
assert [big + +10 +10] {== $_ 20} -v -line #__LINE__
assert [big + 11 11] {== $_ 22} -v -line #__LINE__
assert [big + +11 11] {== $_ 22} -v -line #__LINE__
assert [big + 11 +11] {== $_ 22} -v -line #__LINE__
assert [big + +11 +11] {== $_ 22} -v -line #__LINE__
assert [big + 11 -11] {== $_ 0} -v -line #__LINE__
assert [big + -11 11] {== $_ 0} -v -line #__LINE__
assert [big + -11 +11] {== $_ 0} -v -line #__LINE__
assert [big + +11 -11] {== $_ 0} -v -line #__LINE__
assert [big + -11 -11] {== $_ -22} -v -line #__LINE__

# Differening values
assert [big + 11 22] {== $_ 33} -v -line #__LINE__
assert [big + +11 22] {== $_ 33} -v -line #__LINE__
assert [big + 11 +22] {== $_ 33} -v -line #__LINE__
assert [big + +11 +22] {== $_ 33} -v -line #__LINE__

# Mixed sign
assert [big + 11 -22] {== $_ -11} -v -line #__LINE__		# FAIL sign not set to -
assert [big + +11 -22] {== $_ -11} -v -line #__LINE__		# FAIl do
assert [big + -11 22] {== $_ 11} -v -line #__LINE__
assert [big + -11 +22] {== $_ 11} -v -line #__LINE__
assert [big + -11 -22] {== $_ -33} -v -line #__LINE__


# Bigger values
assert [big + 1111111111 2222222222] {== $_ 3333333333} -v -line #__LINE__
assert [big + +1111111111 -2222222222] {== $_ -1111111111} -v -line #__LINE__	# FAIL sign not set
assert [big + 1111111111 -2222222222] {== $_ -1111111111} -v -line #__LINE__	# FAIL sign not set
assert [big + -1111111111 2222222222] {== $_ 1111111111} -v -line #__LINE__
assert [big + -1111111111 -2222222222] {== $_ -3333333333} -v -line #__LINE__	# FAIL Sets -ve incorrectly
assert [big + +1111111111 +2222222222] {== $_ 3333333333} -v -line #__LINE__
assert [big + +1111111111 -2222222222] {== $_ -1111111111} -v -line #__LINE__	# FAIL sign not set
assert [big + +1111111111 -2222222222] {== $_ -1111111111} -v -line #__LINE__	# FAIL sign not set
assert [big + -1111111111 +2222222222] {== $_ 1111111111} -v -line #__LINE__

# Generate random adds from here
assert [big + 1234567890 8987987962] {== $_ 10222555852} -v -line #__LINE__		# FAIL 9122555863 != 10222555852

assert [big + 1234567890 9876543210] {== $_ 11111111100} -v -line #__LINE__		# FAIL 1111111111 != 11111111100


# Subtraction ################################################
newline
textcolor yellow
puts "big -"
textcolor

# Unsigned
assert [big - 0 0] {== $_ 0} -v -line #__LINE__
assert [big - 0 1] {== $_ -1} -v -line #__LINE__
assert [big - 1 0] {== $_ 1} -v -line #__LINE__
assert [big - 1 1] {== $_ 0} -v -line #__LINE__

# Signed -ve 0
assert [big - -0 0] {== $_ 0} -v -line #__LINE__
assert [big - 0 -0] {== $_ 0} -v -line #__LINE__
assert [big - -0 -0] {== $_ 0} -v -line #__LINE__		# FIX should be 0 not -0

# Signed +ve 0
assert [big - +0 0] {== $_ 0} -v -line #__LINE__
assert [big - 0 +0] {== $_ 0} -v -line #__LINE__
assert [big - +0 +0] {== $_ 0} -v -line #__LINE__

# Signed +ve 1
assert [big - 1 1] {== $_ 0} -v -line #__LINE__
assert [big - +1 1] {== $_ 0} -v -line #__LINE__
assert [big - 1 +1] {== $_ 0} -v -line #__LINE__
assert [big - +1 +1] {== $_ 0} -v -line #__LINE__


# Signed mixed 1
assert [big - 1 -1] {== $_ 2} -v -line #__LINE__
assert [big - -1 1] {== $_ -2} -v -line #__LINE__
assert [big - -1 +1] {== $_ -2} -v -line #__LINE__
assert [big - +1 -1] {== $_ 2} -v -line #__LINE__
assert [big - -1 -1] {== $_ 0} -v -line #__LINE__

# Double digits (leading zero)
assert [big - 00 00] {== $_ 0} -v -line #__LINE__
assert [big - 00 01] {== $_ -1} -v -line #__LINE__
assert [big - 01 00] {== $_ 1} -v -line #__LINE__
assert [big - 01 01] {== $_ 0} -v -line #__LINE__
assert [big - -00 00] {== $_ 0} -v -line #__LINE__
assert [big - 00 -00] {== $_ 0} -v -line #__LINE__
assert [big - -00 -00] {== $_ 0} -v -line #__LINE__		# DONE FIX should be 0 not -0
assert [big - +00 00] {== $_ 0} -v -line #__LINE__
assert [big - 00 +00] {== $_ 0} -v -line #__LINE__
assert [big - +00 +00] {== $_ 0} -v -line #__LINE__
assert [big - 01 01] {== $_ 0} -v -line #__LINE__
assert [big - +01 01] {== $_ 0} -v -line #__LINE__
assert [big - 01 +01] {== $_ 0} -v -line #__LINE__
assert [big - +01 +01] {== $_ 0} -v -line #__LINE__

# FIX ME
textcolor red
#puts "FIX ME - @$line ..." 
textcolor
newline
assert [big - 01 -01] {== $_ 2} -v -line #__LINE__		# FAIL 0 != 2 (num is not reversed) should call add
assert [big - -01 01] {== $_ -2} -v -line #__LINE__	# FAIL -0 != -2
assert [big - -01 +01] {== $_ -2} -v -line #__LINE__	# FAIL -0 != -2
assert [big - +01 -01] {== $_ 2} -v -line #__LINE__	# FAIL -0 != 2
assert [big - -01 -01] {== $_ 0} -v -line #__LINE__

# Mixed sign
assert [big - 11 -22] {== $_ 33} -v -line #__LINE__
assert [big - +11 -22] {== $_ 33} -v -line #__LINE__
assert [big - -11 22] {== $_ -33} -v -line #__LINE__
assert [big - -11 +22] {== $_ -33} -v -line #__LINE__
assert [big - -11 -22] {== $_ 11} -v -line #__LINE__		# FAIL

# Bigger values
assert [big - 1111111111 2222222222] {== $_ -1111111111} -v -line #__LINE__
assert [big - +1111111111 -2222222222] {== $_ 3333333333} -v -line #__LINE__
assert [big - 1111111111 -2222222222] {== $_ 3333333333} -v -line #__LINE__
assert [big - -1111111111 2222222222] {== $_ -3333333333} -v -line #__LINE__	# OVERFLOW in Ticol native without [unsigned]
assert [big - -1111111111 -2222222222] {== $_ 1111111111} -v -line #__LINE__	# 

# Multiplication
newline
textcolor yellow
puts "big *"
textcolor

assert [big * 0 0] {== $_ 0} -v -line #__LINE__
assert [big * 0 1] {== $_ 0} -v -line #__LINE__
assert [big * 1 0] {== $_ 0} -v -line #__LINE__
assert [big * 1 1] {== $_ 1} -v -line #__LINE__
assert [big * 0 -0] {== $_ -0} -v -line #__LINE__
assert [big * -0 1] {== $_ -0} -v -line #__LINE__
assert [big * 1 -0] {== $_ -0} -v -line #__LINE__
assert [big * 1 1] {== $_ 1} -v -line #__LINE__
assert [big * 1 -1] {== $_ -1} -v -line #__LINE__
assert [big * -1 1] {== $_ -1} -v -line #__LINE__
assert [big * -1 -1] {== $_ 1} -v -line #__LINE__
assert [big * 123 456] {== $_ [* 123 456]} -v -line #__LINE__
assert [big * -123 456] {== $_ [* -123 456]} -v -line #__LINE__
assert [big * 123 -456] {== $_ [* 123 -456]} -v -line #__LINE__
assert [big * -123 -456] {== $_ [* -123 -456]} -v -line #__LINE__

# TODO - float multiplication 
/*
assert [big * 123 456.789] {== $_ [* 123 456.789]} -v -line #__LINE__
assert [big * -123 456.789] {== $_ [* -123 456.789]} -v -line #__LINE__
assert [big * 123 -456.789] {== $_ [* 123 -456.789]} -v -line #__LINE__
assert [big * -123 -456.789] {== $_ [* -123 -456.789]} -v -line #__LINE__
*/

assert [big * 872938479273498237498273498723947293847234928374 3142857131] {== $_ 2743520924509009635037380465652877152935934719272500134994} -v -line #__LINE__
assert [big * 872938479273498237498273498723947293847234928374 -3142857131] {== $_ -2743520924509009635037380465652877152935934719272500134994} -v -line #__LINE__
assert [big * -872938479273498237498273498723947293847234928374 3142857131] {== $_ -2743520924509009635037380465652877152935934719272500134994} -v -line #__LINE__
assert [big * -872938479273498237498273498723947293847234928374 -3142857131] {== $_ 2743520924509009635037380465652877152935934719272500134994} -v -line #__LINE__

# Division
newline
textcolor yellow
puts "big /"
textcolor

assert [big / 0 1 ] {== $_ 0} -v -line #__LINE__
assert [big / 1 1 ] {== $_ 1} -v -line #__LINE__
assert [big / 3 1 ] {== $_ 3} -v -line #__LINE__
assert [big / -3 1 ] {== $_ -3} -v -line #__LINE__
assert [big / 3 -1 ] {== $_ -3} -v -line #__LINE__
assert [big / -3 -1 ] {== $_ 3} -v -line #__LINE__
assert [big / 3223433 23] {== $_ 140149} -v -line #__LINE__
assert [big / 3223433 1] {== $_ 3223433} -v -line #__LINE__
assert [big / 3223433 3223433] {== $_ 1} -v -line #__LINE__
assert [big / 123098234023489877983876868457546465499657687777776887686 323279997879738777299] {== $_ 380778999105545731892675845428678806} -v -line #__LINE__ # 3.8077899910554574e+35
assert [big / 982342938029384 23887733] {== $_ 41123322} -v -line #__LINE__			# 41123322
assert [big / 123098234023489877983 7738777299] {== $_ 15906677407} -v -line #__LINE__	# eCalc: (wrong) 1.59066774074758e41
assert [big / 9999999999999999999999999999999999999999999999999999 6666666] {== $_ 1500000150000015000001500000150000015000001500} -v -line #__LINE__

# fdiv 
newline
textcolor yellow
puts "big fdiv"
textcolor

assert [big fdiv 22 7] {== $_ 3.142857142857142857} -v -line #__LINE__
assert [big fdiv 22 -7] {== $_ -3.142857142857142857} -v -line #__LINE__
assert [big fdiv -22 7] {== $_ -3.142857142857142857} -v -line #__LINE__
assert [big fdiv -22 -7] {== $_ 3.142857142857142857} -v -line #__LINE__



#FIX FROM HERE...
# operator / @3299: No early exit. Checking signs
assert [big fdiv 2342393939 93393999993.0] {== $_ 0.025080775415717983} -v -line #__LINE__
assert [big fdiv 982342938029384 23887733] {== $_ 41123322.08457722} -v -line #__LINE__			# 41123322.084577217938596349 will round OK

# 15906677407.475797939097665717 - ActiveState: 15906677407.475798  1.5906677407475798e10
assert [big fdiv 123098234023489877983 7738777299] {== $_ 15906677407.475798} -v -line #__LINE__	# eCalc: (wrong) 1.59066774074758e41

assert [big fdiv 123098234023489877983876868457546465499657687777776887686 323279997879738777299.0] \
 {== $_ 380778999105545731892675845428678805.1096625509413551668} -v -line #__LINE__ # 3.8077899910554574e+35
# Unproven fractional
assert [big fdiv 9999999999999999999999999999999999999999999999999999 6666666] {== $_ 1500000150000015000001500000150000015000001500.000149850014985001} -v -line #__LINE__

assert [big fdiv 23 33.0] {== $_ 0.696969696969696968} -v -line #__LINE__
# ecalc 2.77753153544e38 PASS  # Unproven fractional

assert [big fdiv 872938479273498237498273498723947293847234928374 3142857131] {== $_ 277753153543999973029086920567356612000.965604304461143512} -v -line #__LINE__

#assert [big fdiv 23423.0 2343.33] 9.995604545667918	# Will fail due to fdiv being integer only

assert [big * [big / 309809809809890809809890808080808098 2] 2] {== $_ 309809809809890809809890808080808098} -v -line #__LINE__

# Requires FPN
puts [big fdiv 309809809809890809809890808080808098 23]
# 309809809809890809809890808080808095
#assert [big * [big fdiv 309809809809890809809890808080808098 23] 23] 309809809809890809809890808080808098

# Big Boolean !
newline
textcolor yellow
puts "big !"
textcolor
assert [big ! 0] {== $_ 1} -v -line #__LINE__
assert [big ! 1] {== $_ 0} -v -line #__LINE__
assert [big ! 23] {== $_ 0} -v -line #__LINE__

# Big bitwise ~
newline
textcolor yellow
puts "big ~"
# Note: We don't do two's compiment yet so can't handle the unsigned inversion
# Instead we do simple bit-inversion
textcolor
assert [big ~ 0]	{== $_ 1} -v -line #__LINE__
assert [big ~ 1]	{== $_ 0} -v -line #__LINE__
# Bitwise -ves not handled yet
#assert [big ~ -1] 0
# big inttobin has no 0b prefix
textcolor red
puts "Catch ..."
textcolor
catch {
	assert [big inttobin [big ~ 23]]	{== $_ 1000} -v -line #__LINE__
} # FAIL 0b10111 -> (unsigned) 0b01000
puts "\[~ 23\]==[~ 23]"

# mod
# FAIL 354517092358080439991 != 31237094478341662692
newline
textcolor yellow
puts "big %"
textcolor
assert [big % 12345 7] {== $_ 4} -v -line #__LINE__
assert [big % 123456789 23] {== $_ [% 123456789 23]} -v -line #__LINE__ # PASS
assert [big % 12309823402348987798387 3232799978] {== $_ 2164422255} -v -line #__LINE__ # ActiveState 2164422255 PASS
# FAIL
# Reverse assert against Ticol result 380778999105545731892675845428678805
# DIFF 354517092358080439991
#      323279997879738777299
puts [big - 123098234023489877983876868457546465499657687777776887686 [big * 323279997879738777299 380778999105545731892675845428678805]]
newline
#assert [big * 323279997879738777299 380778999105545731892675845428678805] 123098234023489877983876868457546465499657687777776887686
assert [big / 123098234023489877983876868457546465499657687777776887686 323279997879738777299] {== $_ 380778999105545731892675845428678806} -v -line #__LINE__ # ActiveState
assert [big % 123098234023489877983876868457546465499657687777776887686 323279997879738777299] {== $_ 31237094478341662692} -v -line #__LINE__ # ActiveState

# mean
newline
textcolor yellow
puts "big mean"
textcolor
assert [big mean 123413241324 234234234 234234234 2342379879797987 -2234234234234199987987 234 -2] {== $_ -319175984532919782853.714285714285714284} -v -line #__LINE__ # eCalc -3.191759845329197e20 PASS
assert [big mean 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15] {== $_ 8} -v -line #__LINE__ # eCalc v8
# (1+2+3+4+5+6+7+8+9+10+11+12+13+14+15)/15

lib ticol_bignum -unload

option expression pop

unset i -nocomplain

vars

####################################################
halt
newline
textcolor green
option echo on
puts "* \[big\] (Big number tests) Done *"
textcolor

# EOF #