#################################################################################
# File: 			test_bmp.tcl
# Purpose:			Tests for lib ticol_bmp
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 12/Jan/2018	First version
# 1.01 21/Jul/2018	Updated re: finished DLL
# 1.01 01/Aug/2018	Change to lorem.txt
# 1.02 10/Jan/2022	Make unlink bmp safe
# 1.03 08/Feb/2022	Updated re changes in lorem.txt
#################################################################################
# Notes:
# Test is unique and atomic to this script. Multiple instances can't be run
# bool [bmp write string|var filename ?-var? ?-fore x? ?-back x?]
#
# Need [binary readfile], [binary writefile]
#################################################################################
# #define ndebug

#ifdef ndebug 
die "ndebug defined in [info script]"
#endif

option expression off
unset s -nocomplain

textcolor cyan
lib ticol_bmp -verbose -nocomplain
textcolor

if {! [file exists lorem.txt]} {
	die "[info script] requires lorem.txt (or any suitable text file)"
}

# Trap bad syntax 														# NO LEAKS 04/Aug/2019
puts "Ignore bad syntax message"
set i 0
try {
	bmp write foo -encrypt			# no filename or var
} catch {
	set i 1
}
assert $i {== $_ 1}  -v -line #__LINE__

# Test file write														# NO LEAKS 04/Aug/2019

textcolor yellow
puts "Test read/write lorem.txt ..."
textcolor

# NO LEAKS 04/Aug/2019
# last line: 'non opus habet, et iure usus seu limita'
puts "Line: "#__LINE__										
set r [readfile lorem.txt s]
puts "r = $r"											# is 4246
assert $r {== $_ 4253} -v -line #__LINE__				# Was length 4246 until colour changed (set as +r)
puts "lorem.txt is [string length $s] byte(s)"
#puts [lastchar $s]
#puts $s
assert $r {== $_ [string length $::s]} -v -line #__LINE__
bmp write "lorem.bmp" s -var
unset s

#ifdef comment
puts "Reading at line: "#__LINE__
bmp read lorem.bmp s
textcolor darkyellow
puts "s is '[string left $s 500]...'"
textcolor
stop
#endif

#ifdef comment
////////////////////////////////////////////////////////
Type:                  Windows or OS/2 bitmap
Disk file length:      51,638 byte(s) (50 kb)
Data block length:     51,509 byte(s) (image size)
Dimensions:            672 x 614 pixels
Total pixels           412,608
Width units of 32px:   21.00 (units of 4 bytes)
Total pixel storage:   51,576 byte(s) (if fully-allocated)
Pixel-byte shortfall:  67 byte(s)
Bits per pixel         1
Colours:               2
Planes:                1
Compression?           False
Data starts at:        62 (0x3e)
BMP header size:       40 (0x28)
////////////////////////////////////////////////////////
#endif

# NO LEAKS 04/Aug/2019
## Test BMP metrics ##
puts "Line: "#__LINE__
set l [bmp info lorem.bmp]
puts "\[bmp info lorem.bmp\] == '$l'"
# filesize-bytes data-block-size width height colours bits-per-plane bytes-per-pixel padding-space
# 4334 4253 192 178 2 1 0 -19
assert $l {eq $_ {4334 4253 192 178 2 1 0 -19}} -v -line #__LINE__

# NO LEAKS 04/Aug/2019
assert [lindex $l 0] {== $_ 4334}  -v -line #__LINE__
assert [lindex $l 1] {== $_ 4253}  -v -line #__LINE__
assert [lindex $l 2] {== $_ 192}  -v -line #__LINE__
assert [lindex $l 3] {== $_ 178}  -v -line #__LINE__
unset l

# NO LEAKS 04/Aug/2019
# FIX Corrupted [vars] display
puts "Line: "#__LINE__

try {
	unlink "randomstr.bmp"							# Delete
} catch {
	puts "randomstr.bmp not found - another process may have deleted it"
}

unset s -nocomplain
# FAILS
# FIX picolSetVar should test v->size and if > 0 then copy that length
# binary set s [randomstr 4096 5 -var s -nocomplain]

randomstr 4096 5 -var s -nocomplain
set start_md5 [md5 s -var]
puts "Initial data MD5: $start_md5"
vars

puts "Line: "#__LINE__

#ifdef comment
Disk file length:      106 byte(s) (0 kb)
Data block length:     43 byte(s) (image size)
Dimensions:            32 x 11 pixels
Total pixels           352
Width units of 32px:   1.00 (units of 4 bytes)
Total pixel storage:   44 byte(s) (if fully-allocated)
Pixel-byte shortfall:  1 byte(s)
Bits per pixel         1
Colours:               2
Planes:                1
Compression?           False
Data starts at:        62 (0x3e)
BMP header size:       40 (0x28)
Pallette starts at:    54 (0x36)
Pallete colours are:
1: 16777215 (#ffffff) R=255 G=255 B=255
2: 00000000 (#000000) R=000 G=000 B=000
#endif

# SMALL LEAK? 04/Aug/2019 1332-1436
# Tests
puts "Line: "#__LINE__
for {set i 0} {< $i 100} {++ i} {				# If leaky/corrupt heap should crash (again)
	bmp write "randomstr.bmp" s -var			# randomstr 4096 5 -var s -nocomplain
}
puts "Line: "#__LINE__
assert [binary length s] {== $_ 4096} -v -line #__LINE__
puts "Line: "#__LINE__
newline
# */

# No leak 04/Aug/2019
textcolor yellow
puts "Writing randomstr.bmp ...\n"
textcolor
puts "Line: "#__LINE__
option expression off


if {[bmp write "randomstr.bmp" s -var]} {
	puts "Line: "#__LINE__
	inspect s
	#inspect foo
	#stop
	textcolor green
	puts "Success"
	textcolor
	# exit
	#autoexec on
	#@start "randomstr.bmp"
	textcolor darkyellow
	puts "ADDITIONAL TESTS @"#__LINE__
	textcolor
	## Metrics ##
	puts "Line: "#__LINE__
	try {
		puts "\[bmp info randomstr.bmp\] -> '[bmp info randomstr.bmp]'"
	} catch {
		puts "Error reading randomstr.bmp in lib ticol_bmp"
	}
	#assert [bmp info randomstr.bmp] {eq $_ {4166 4096 192 171 2 1 0 -8}} -v -line #__LINE__
	# ADD read and check MD5
	# Need binary md5
	puts "Line: "#__LINE__
	inspect s
	unset s -nocomplain
	puts "Line: "#__LINE__
	bmp read randomstr.bmp s		# Corrupts var s
	puts "Line: "#__LINE__
	inspect s
	puts "Line: "#__LINE__
	puts "\[binary length s\] = '[binary length s]'"
	puts "Line: "#__LINE__
	puts "Read-in data MD5: [md5 s -var]"
	puts "Checking for consistent MD5s at line: "#__LINE__
	assert [eq $start_md5 [md5 s -var]] {== $_ $::true} -v -line #__LINE__
	unset start_md5 -nocomplain
} else {
	textcolor red
	puts "FAILED at line "#__LINE__
	textcolor
	stop
}


newline


# No leak 04/Aug/2019
# Create and get info about a bmp file
puts "Line: "#__LINE__
assert [bmp write test.bmp $qbf -fore 0xff0000 -back 0x00] {== $_ $::true} -v -line #__LINE__
puts [bmp info test.bmp]
assert [bmp info test.bmp] {eq $_ "106 43 32 11 2 1 0 -1"} -v -line #__LINE__

unset s 

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Test write qbf ..."
textcolor

# No leak 04/Aug/2019
puts "Line: "#__LINE__
puts "bmp write=[bmp write qbf.bmp $qbf]"

# No leak 04/Aug/2019
puts "Line: "#__LINE__
if {bmp write "qbf.bmp" $qbf} {
	textcolor green
	puts "Success"
	textcolor
} else {
	textcolor red
	puts "FAILED at line "#__LINE__
	textcolor
	stop
}

# No leak 04/Aug/2019
puts "Line: "#__LINE__
if {bmp write "qbf.bmp" qbf -var -fore 0xff0000 -back 0x00} {
	textcolor green
	puts "Success"
	textcolor
} else {
	textcolor red
	puts "FAILED at line "#__LINE__
	textcolor
	stop
}

# No leak 04/Aug/2019 (FIXED)
## BMP Metrics ##
assert [bmp info qbf.bmp] {eq $_ "106 43 32 11 2 1 0 -1"} -v -line #__LINE__

unset i -nocomplain

# No leak 04/Aug/2019 (FIXED)
# Test built-in encryption  LEAKS
bmp write "qbf.bmp" qbf -var
assert [bmp read qbf.bmp v] {== $_ $::true} -v -line #__LINE__
assert $v {eq $_ $::qbf} -v -line #__LINE__

# No leak 04/Aug/2019
# 1332-1408
# Test encrypted write (1)
puts "Line: "#__LINE__
bmp write test.bmp [encrypt $qbf secret] -fore 0xff0000 -back 0x0
bmp read test.bmp s			# Performs a binary read


# No leak 04/Aug/2019
# FIXED LEAKS
puts [decrypt $s secret]
assert [decrypt $s secret] {eq $_ $::qbf} -v -line #__LINE__
unset s -nocomplain

# Test encrypt (2)
puts "Line: "#__LINE__
bmp write test.bmp $qbf -encrypt -fore 0xff0000 -back 0x0
bmp read test.bmp in
puts [bmp info test.bmp]
assert $in {eq $_ $::qbf} -v -line #__LINE__
unset in -nocomplain
bmp graph test.bmp -r -w 1
newline

# Test combined internal and external encrypted write (3)
puts "Line: "#__LINE__
bmp write test.bmp [encrypt $qbf secret] -fore 0xff0000 -back 0x0 -encrypt
bmp read test.bmp s			# Performs a binary read
puts [decrypt $s secret]
assert [decrypt $s secret] {eq $_ $::qbf} -v -line #__LINE__
unset s -nocomplain
bmp graph test.bmp -r -w 1						# Draw the BMP
newline

# Test encrypt (4)
puts "Line: "#__LINE__
set s "The quick brown fox jumps over the"
puts "Payload length [string length $s]"		# 34
bmp write test.bmp $s -encrypt -fore 0xff0000 -back 0x0
bmp read test.bmp in
puts [bmp info test.bmp]
assert $in {eq $_ $::s} -v -line #__LINE__
unset in -nocomplain

# Test encrypt (5)
puts "Line: "#__LINE__
set s "The quick brown fox jumps over"
puts "Payload length [string length $s]"		# 30
bmp write test.bmp $s -encrypt -fore 0xff0000 -back 0x0
bmp read test.bmp in
puts [bmp info test.bmp]
assert $in {eq $_ $::s} -v -line #__LINE__
unset in -nocomplain

# Test encrypt (6)
puts "Line: "#__LINE__
set s "y"
puts "Payload length [string length $s]"		# 30
bmp write test.bmp $s -encrypt -fore 0xff0000 -back 0x0
bmp read test.bmp in
puts [bmp info test.bmp]
assert $in {eq $_ "y"} -v -line #__LINE__
unset in -nocomplain

# Test encrypt (7) - empty var
puts "Line: "#__LINE__
set s ""
puts "Payload length of s=[string length $s]"		# 30
set i 0
try {
	bmp write test.bmp $s -encrypt -fore 0xff0000 -back 0x0		# Should fail
} catch {
	set i 1
}
assert $i {== $_ 1}  -v -line #__LINE__
unset i in s -nocomplain
newline

textcolor yellow
puts "Test \[bmp graph\] @"#__LINE__
textcolor
newline

puts "Line: "#__LINE__
bmp graph test.bmp
newline
newline
bmp graph test.bmp -r -w 1				# Draw
newline

# @ start test.bmp
puts "Line: "#__LINE__
unset s -nocomplain

readfile lorem.txt s
puts "Line: "#__LINE__
bmp write "lorem.bmp" s -var -encrypt		# FIXED GPF
puts "Line: "#__LINE__
unset s -nocomplain
puts "\nlorem.bmp..."
#bmp graph lorem.bmp -r		# Print in software-viewable order
puts "\nLine: "#__LINE__
bmp read "lorem.bmp" s
puts "Line: "#__LINE__
assert [string length $s] {== $_ 4253} -v -line #__LINE__
assert [string left $s 22] {eq $_ "Ordo et continuatio in"} -v -line #__LINE__
unset s -nocomplain

try {
	puts "(Ignore any error if winter.bmp not found)"
	bmp graph winter.bmp -crop -r		# 130 x 80			# Draw
} catch {
	puts "bmp graph winter.bmp -crop -r was not run"
}




lib ticol_bmp -unload -nocomplain

puts "Line: "#__LINE__
unset v -nocomplain
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #