#################################################################################
# File:				test_lib_calldll.tcl
# Author:			M Shaw
# Purpose:			Test cases for ticol_calldll.dll
# Language:			Ticol Tcl v1.22..v1.26+
#
# Versions:
# 1.00 31/May/2017	First version
# 1.01 27/Sep/2019	Updated for [dll_close] but and add [watch trace]
# 1.02 12/Nov/2019	Some tests re-enabled
#################################################################################
# Notes:
# Add more tests for individual call styles and returns
# test_lib_crt.tcl tests calldll_cdecl, calldlldouble_cdecl, calldllstr_cdecl
# calldll<_cdecl>					Integer or void* return
# calldlldbl<_cdecl>				Double return
# calldllstr<_cdecl>				LPSTR return
# calldllvariant<_cdecl>			VARIANT return
#################################################################################
/*
	Context:
	| Context 1                                        | Context 2                                  |
	ticol.exe -> eval -> script -> ticol_calldll.dll -> ticol.dll -> eval -> script -> ticol_calldll
	Original:
*/
#################################################################################
option expression off

textcolor cyan
lib ticol_calldll -verbose -nocomplain
textcolor
newline

textcolor yellow
puts "test_calldll.tcl - Testing ticol_calldll.dll"
textcolor
newline
#################################################################################
puts "Showing \[option\] settings..."
option

if {! [defined print_line] } {
	proc print_line {line} {
		textcolor yellow
		puts "Line: $line"
		textcolor 
	}
}
########################################################################################

set dllpath ""							# Default for non NT4
if {== [info winver -vernum] 4.0} {		# If Windows NT4 then handle path prefix
	puts "NT4.0 detected"
	set dllpath "$env(WINDIR)"
	set dllpath [unescape $dllpath]
	#set dllpath [string replace $dllpath [chr 92][chr 92] "/"]
	set dllpath [string replace $dllpath "\\" "/"]
	set dllpath "$dllpath/system32/"
} # */

/*
# 2018/Aug/24 - This test raises GPF when called from Windows ticol.dll
textcolor cyan
lib ticol_calldll -verbose -nocomplain
#lib ticol_variant -verbose -nocomplain
textcolor
newline

# Early unload debug
lib ticol_calldll -unload -verbose
return
# */
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
print_line #__LINE__
set r [calldll_cdecl cdecl test_3_long_cdecl 3 4 5]
puts "r is $r"
print_line #__LINE__
# Raise an error due to insuffient args
#set r [calldll_cdecl cdecl test_3_long_cdecl 3 4]		# Not trapped!
set r [calldll_cdecl cdecl test_1_string_byval_cdecl $qbf]
print_line #__LINE__
set h [dll_open cdecl]
puts "cdecl.dll handle is $h"

# FIX returning handle 0
set t [calldll_cdecl $h test_tm_ptr_cdecl 33]		# Remember that the lib will have unloaded
puts "test_tm_ptr_cdecl static struct address was '$t'"

struct unset tm -nocomplain
struct tm {		# Define a struct. We don't need any instances of it other than this
	#			 C++	Field order must be maintained
	tm_sec	4	#int	Seconds after the minute   0-61
	tm_min	4	#int	Minutes after the hour     0-59
	tm_hour	4	#int	Hours since midnight       0-23
	tm_mday	4	#int	Day of the month           1-31
	tm_mon	4	#int	Months since January       0-11
	tm_year	4	#int	Years since                1900	
	tm_wday	4	#int	Days since Sunday          0-6
	tm_yday	4	#int	Days since January 1       0-365
	tm_isdst 4 	#int	Daylight Saving Time flag
}
print_line #__LINE__
struct copy tm $t						# New, experimental command
print_line #__LINE__
dump tm
print_line #__LINE__
# Syntax: time_t asctime(struct tm* t)
set ret [calldllstr_cdecl ${dllpath}msvcrt40.dll asctime tm]
puts "ret is '$ret'"
dll_close $h
##exit

#/*
# Allocate a new string
print_line #__LINE__
set s [makestr 20]
# Prefix some fill chars
setl s "**"
mid s "**" 8 2
puts "s: '$s'"
print_line #__LINE__
# Do some pointer arithmetic and copy
puts "Calling calldll_cdecl kernel32 RtlMoveMemory"
calldll_cdecl kernel32 RtlMoveMemory [+ 2 [strptr s]] "Hello" 5
puts "s: '$s'"
assert $s {eq $_ "**Hello**           "} -v -line #__LINE__
# */


########################################################################################
#/*
print_line #__LINE__
# FIX finish cdecl updates
puts "Calling calldlldbl_cdecl->"[calldlldbl_cdecl ${dllpath}msvcrt40.dll sqrt $pi]
set d [double [calldlldbl_cdecl ${dllpath}msvcrt40.dll sqrt $pi]]
debug off
print_line #__LINE__
puts "d is $d"
puts "d*d=>[* $d $d] (pi=$pi)"
assert $d {== $_ 1.772453850905516} -v -line #__LINE__
#stop
# */

########################################################################################
#/*
print_line #__LINE__
puts "Creating structs and calling GetVersionExA in kernel32"

struct OsVersionInfo {
	OSVSize         4
	dwVerMajor      4
	dwVerMinor      4
	dwBuildNumber   4
 	PlatformID      4
	szCSDVersion    128
}
puts "Size="[struct size OsVersionInfo]
#call [setb OSVSize [struct size OsVersionInfo] 4 -value]
#setb OsVersionInfo.OSVSize 148 4 -value
print_line #__LINE__
setb OsVersionInfo.OSVSize [struct size OsVersionInfo] 4 -value
puts "OSVSize:       [ofaddressb OsVersionInfo.OSVSize 4] @"#__LINE__

print_line #__LINE__
puts "Calling GetVersionExA...@"#__LINE__
set r [calldll kernel32 GetVersionExA OsVersionInfo]
#puts "GetVersionEx=[calldll kernel32 GetVersionExA OsVersionInfo]"
puts "r is $r @"#__LINE__
option expression off
textcolor white
print_line #__LINE__
if {!= $r 0}  {			# Returns 0 on failure
	puts "OSVSize:       "[ofaddressb OsVersionInfo.OSVSize 4]
	puts "dwVerMajor:    "[ofaddressb OsVersionInfo.dwVerMajor 4]
	puts "dwVerMinor:    "[ofaddressb OsVersionInfo.dwVerMinor 4]
	puts "dwBuildNumber: "[ofaddressb OsVersionInfo.dwBuildNumber 4]
	puts "PlatformID:    "[ofaddressb OsVersionInfo.PlatformID 4]
	puts "szCSDVersion:  "[ofaddress OsVersionInfo.szCSDVersion]
	#dump OsVersionInfo
} else {
	puts "kernel32 GetVersionExA DLL call FAILED (code $r) @"#__LINE__
}
textcolor

print_line #__LINE__
assert $r {== $_ 1} -v -line #__LINE__
assert [ofaddressb OsVersionInfo.dwVerMajor 4] {>= $_ 4} -v -line #__LINE__		# NT 4 onwards
assert [ofaddressb OsVersionInfo.dwVerMinor 4] {>= $_ 0} -v -line #__LINE__
assert [ofaddressb OsVersionInfo.dwBuildNumber 4] {!= $_ 0} -v -line #__LINE__

struct unset OsVersionInfo -nocomplain
# */

########################################################################################
/*
# Call a DLL by ordinal - Shell32.dll @680 #
# User MUST also be in an elevated session
print_line #__LINE__
puts "\[is_admin\] [is_admin]"
puts "\[is_elevated\] [is_elevated]"
puts "Is user an admin Shell32@680 ?:         "[calldll "Shell32" "@680"]
puts "Is user an admin Shell32@IsUserAnAdmin: "[calldll "Shell32" "IsUserAnAdmin"]
if {any [is_admin] [is_elevated]} {
	puts "* We are elevated *"
	assert [calldll "Shell32" "@680"] {== $_ 1} -v -line #__LINE__
} else {
	puts "* We are NOT elevated *"
	assert [calldll "Shell32" "@680"] {== $_ 0} -v -line #__LINE__
}

print_line #__LINE__
struct unset OsVersionInfo
print_line #__LINE__
# */
########################################################################################

# Test VCRT _snprintf
print_line #__LINE__
const BUFLEN 255 -nocomplain

watch trace handle -line #__LINE__
#set dllname "${dllpath}msvcrt40.dll"
set dllname "msvcrt40.dll"
set handle 0
puts "Opening $dllname"
set handle [dll_open $dllname]
#watch trace handle -line #__LINE__

watch trace 5* -line #__LINE__
watch trace 6* -line #__LINE__
watch trace 7* -line #__LINE__

print_line #__LINE__
if {$handle} {
	puts "Handle to $dllname is: $handle"
	set buf [makestr [+ $BUFLEN 1] " "]
	#puts '$buf'
	# calldll* handle function-name|@ordinal arg arg...
# FIX non char range given on NT4.0	
print_line #__LINE__
	puts "Should be >> 0a 0b 0c << (literal)"
# FIX NT4.0 problem is the MPP translation of 0x0a consts
print_line #__LINE__
	calldll_cdecl "msvcrt40" "printf" "\n>>>>>>>>>>%2.2x%2.2x%2.2x<<<<<<<<<<<<\n" "0x0a" "0x0b" "0x0c"		# PASS
print_line #__LINE__	
	calldll_cdecl "msvcrt40" "printf" "\n>>>>>>>>>>%2.2x%2.2x%2.2x<<<<<<<<<<<<\n" 0x0a 0x0b 0x0c		# PASS
print_line #__LINE__	
	calldll_cdecl $dllname "printf" "\n>>>>>>>>>>%2.2x%2.2x%2.2x<<<<<<<<<<<<\n" 0x0a 0x0b 0x0c		# PASS
print_line #__LINE__	
	calldll_cdecl $handle "printf" "\n>>>>>>>>>>%2.2x%2.2x%2.2x<<<<<<<<<<<<\n" 0x0a 0x0b 0x0c		# PASS
#1a80640 1a80670 1a806a0
/*
	watch trace 5* -value -halt -line #__LINE__
	watch trace 6* -value -halt -line #__LINE__
	watch trace 7* -value -halt -line #__LINE__		# */

	# _CRTIMP int __cdecl _snprintf(char *, size_t, const char *, ...);
	print_line #__LINE__
	calldll_cdecl $handle "_snprintf" buf $BUFLEN "%2.2x%2.2x%2.2x" 0x0a 0x0b 0x0c
	puts "buf return is: '$buf' @"#__LINE__
	assert $buf {eq $_ "0a0b0c"} -v -line #__LINE__

	calldll_cdecl msvcrt40 "_snprintf" buf $BUFLEN "%2.2x%2.2x%2.2x" 0xa0 0xb0 0xc0
	calldll_cdecl $handle "_snprintf" buf $BUFLEN "%2.2x%2.2x%2.2x" 0xa0 0xb0 0xc0

	puts "'$buf'"
	assert $buf {eq $_ "a0b0c0"} -v -line #__LINE__

	dll_close $handle			# FIX creates var with $handle value

}
########################################################################################
# VARIANT
textcolor magenta
puts "Calling VARIANT calldllvariant (__stdcall)"
textcolor

print_line #__LINE__
set ptr [calldllvariant mslib145.dll AToAV $qbf]
#set ptr [calldllvariant mslib145.dll AToBV $qbf]		# GPF
assert $ptr -v -line #__LINE__
puts $ptr
lib ticol_variant
dumpvariant $ptr
lib ticol_variant -unload

########################################################################################
# Leak testing...
set loop_count 1000			# Was 10000 but 1000 should be fine
textcolor darkyellow
puts "Loop leak and stability tests re: stack adjustments ..."
puts "Looping for $loop_count loops of DLL open, call, close"
puts "(please be patient on slow machines) ..."
textcolor

# LEAKING a small amount !!/ Check AToAV
# Removed ROSFIX from calldll
# Fixed free but still leaking
# Empty time loop is stable (no leaks)

print_line #__LINE__

time {		# Solid. No leaks!
	set t 0
	set h [dll_open ${dllpath}msvcrt40.dll]
	calldll_cdecl $h time t
	set ts_ptr [calldll_cdecl $h localtime t]
	assert [!= $ts_ptr -1]		# Silent assert
	set ret [calldllstr_cdecl $h asctime $ts_ptr]	# tm, ROS: calldll: Exception in calldllstr_cdecl, DLL call failed (WinError:5)
	#print_line #__LINE__
	#puts "$ret"									# OK
	#puts [string trim $ret [chr 10]]				# OK
	#puts "[string trim $ret [chr 10]]"				# OK
	#puts "'[string trim $ret [chr 10]]'"			# OK
	# FIX...
	#puts "asctime (now): '[string trim $ret [chr 10]]'"	# Does not like the colon char
	#puts "asctime(NOW): '[string trim $ret [chr 10]]' "
	#puts "asctime NOW: -"[string trim ret [chr 10]]-		# Why did I do it like this?
	dll_close $h
} $loop_count	# */

/*
print_line #__LINE__
time {								# Small leak!
	# DON'T delete a variant object with sysfree
	set ptr [calldllvariant mslib145.dll AToAV $qbf]		# __stdcall
	#sysfree ptr		# NO!
	#puts $ptr
	free $ptr
	#sleep 1
} $loop_count	# */
puts "* Loop done *"
unset loop_count

########################################################################################
# Cleanup
print_line #__LINE__

undef print_line -nocomplain
unset h ret ptr t tm ts_ret ts_ptr -nocomplain 
unset BUFLEN -const -nocomplain
unset buf dllname handle -nocomplain
unset d dllname dllpath handle r s -nocomplain
unset 1* -glob -nocomplain

newline
textcolor cyan
lib ticol_calldll -unload -verbose
textcolor

########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #