##########################################################################
# File: 			test_lib_crt.tcl
# Purpose:			Test file for calldll calls to MSVCRT CRT
#					via ticol_calldll.dll
# Author: 			M Shaw
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 28/May/2017	First version
# 1.01 04/Jun/2017	Added rounding for float cmp in ReactOS
# 1.02 08/Jul/2017	Replaced [checkex] with [assert]
# 1.03 29/Oct/2017	Fixed bug re: checking DST with fixed times
##########################################################################
# Notes: 
# Can't call puts() for printf() from the Windows DLL as it will not output to console
# Instead, WriteLn is required
# If puts is repeatedly called then it will return EOF after 256 calls
##########################################################################
# Set command suffix 
#set callstyle "" 			# __stdcall FAIL
set callstyle _cdecl		# CALL SUFFIX __cdecl PASS

option expression off

###########################################################################################
/*
if {! [defined checkex] } {
	proc checkex {x expr} {
		#puts "DEBUG: checkex \$x='$x' \$expr='$expr'"
		upvar i					# Pass-through reference integer 
		upvar line
		set NAN "-1.[calc [chr 35]]IND00000000000"
		set q {$x}			# This swaps x
		puts "Line $line: " -nonewline
		puts "'$q'=='$x' \[expr $expr\]: "	# fails here at EOF - poss (due to bug in callproc()?) - July 2016
		if { [eval $expr] } {
		} else {
			textcolor red	
			puts "checkex assertion failed: '$q' -> '$x' after line $line"
			textcolor
			stop
		}
	}
} */
###########################################################################################

textcolor cyan
try {
	lib ticol_calldll -verbose -nocomplain
} catch {
	puts "Failed to load ticol_calldll in test_crt"
	die
}
puts [lib ticol_calldll -info]
textcolor
newline

textcolor yellow
puts "Testing CRT time API calls via calldll to msvcrt40.dll"
textcolor
newline

textcolor magenta
puts "Testing CRT file and print"
puts "--------------------------"
textcolor

puts "Line: "#__LINE__
#set dirstr "dir ticol.exe /w"

set in_dll [info is_dll]
if {! $in_dll} {
	# not ticol.dll friendly! (needs control flag $in_dll
	assert [calldll$callstyle msvcrt40.dll puts "Hello from puts"] {== $_ 0} -v -line #__LINE__		# ??
	assert [calldll$callstyle msvcrt40.dll printf "String %s int %i float %.4f\n" "str" 123 [/ 22.0 7]] {== $_ 32} -v -line #__LINE__
} else {
	textcolor red
	puts "ticol.dll detected. Not testing MSVC console print functions"
	textcolor
}

assert [calldll$callstyle msvcrt40.dll strlen $qbf] {== $_ 43} -v -line #__LINE__
assert [calldll$callstyle msvcrt40.dll _getpid] {> $_ 0} -v -line #__LINE__
#assert [calldll$callstyle msvcrt40.dll system $dirstr] {eq $_ "<dirlist>"}
assert [calldllstr$callstyle msvcrt40.dll strstr $qbf "fox"] {eq $_ "fox jumps over the lazy dog"} -v -line #__LINE__
assert [calldll$callstyle msvcrt40.dll isxdigit [asc "a"] ] {== $_ 128} -v -line #__LINE__
newline

##########################################################################
# calldll_cdecl returns a long, void* or any other generic pointer which
# can be recast into something else
##########################################################################
textcolor magenta
puts "Testing CRT time API"
puts "--------------------"
textcolor

unset a t -nocomplain

set a 0
set t 0
#set t ""		# Will GPF
calldll$callstyle msvcrt40.dll time t
puts "Current time as t is $t"
assert $t {> $_ 1495968675} -v -line #__LINE__

set h [dll_open msvcrt40.dll]
assert $h -v -line #__LINE__

set ret [calldllstr$callstyle $h ctime t]		# PASS (Shows current date/time)
puts "ctime(): -"[string trim ret [chr 10]]-
assert $ret {ne $_ ""} -v -line #__LINE__		# Tue May 30 01:03:38 2017
assert [lcount $ret] {== $_ 5} -v -line #__LINE__
assert [lindex $ret 2] {>= $_ 1} -v -line #__LINE__

set t [lindex $ret 3]
replacechar t : " "
assert [lindex $t 0] {[&& [>= $_ 0] [<= $_ 23]]} -v -line #__LINE__
assert [lindex $t 1] {[&& [>= $_ 0] [<= $_ 59]]} -v -line #__LINE__
assert [lindex $t 2] {[&& [>= $_ 0] [<= $_ 59]]} -v -line #__LINE__
unset t

assert [lindex $ret 4] {[&& [>= $_ 2017] [<= $_ 2025]]} -v -line #__LINE__		# Test valid up to 2025

struct unset tm -nocomplain
struct tm {		# Define a struct. We don't need any instances of it other than this
	#			 C++	Field order must be maintained
	tm_sec	4	#int	Seconds after the minute   0-61
	tm_min	4	#int	Minutes after the hour     0-59
	tm_hour	4	#int	Hours since midnight       0-23
	tm_mday	4	#int	Day of the month           1-31
	tm_mon	4	#int	Months since January       0-11
	tm_year	4	#int	Years since                1900	
	tm_wday	4	#int	Days since Sunday          0-6
	tm_yday	4	#int	Days since January 1       0-365
	tm_isdst 4 	#int	Daylight Saving Time flag
}

setb tm.tm_year 89
set t 1470404087						# Fri Aug 05 14:34:47 2016 (this date was in DST)

# Syntax: time_t mktime(struct tm*)

##########################################################################
# C++ struct tm *localtime( const time_t *timer );
# Reactos is possibly corrupting the return via stack arrangement as the first few bytes of ret are corrupted
# localtime returns a * to an internal static
##########################################################################
textcolor magenta
puts "Testing localtime API"
puts "---------------------"
textcolor

set ret [calldll_cdecl $h localtime t]
#set t [calldll$callstyle $h mktime [addressof tm]]

puts "MSVC localtime() return is address $ret [inttohex $ret]"

assert $ret {> $_ 0} -v -line #__LINE__

newline
textcolor yellow
dumpmem $ret 32
textcolor

struct copy tm $ret						# New, experimental command
dump tm

puts "tm_sec -> [ofaddressb tm.tm_sec] [& [ofaddressb tm.tm_sec] 0xffff] (47)"
puts "tm_min -> [ofaddressb tm.tm_min] [& [ofaddressb tm.tm_min] 0xffff] (34)"
puts "tm_hour -> [ofaddressb tm.tm_hour] [& [ofaddressb tm.tm_hour] 0xffff] (14)"
puts "tm_year -> [ofaddressb tm.tm_year] [& [ofaddressb tm.tm_year] 0xffff] (116)"
# ReactOS bug: sets high word with random junk
# 0x54004080 (5,521,536) - struct byte order: 80400054

newline

puts "set tm_isdst: ASCII literal:'[tohex $tm.tm_isdst]', binary:[ofaddressb tm.tm_isdst], 0xff-> [& [ofaddressb tm.tm_isdst] 0xff]"
set in_dst [ofaddressb tm.tm_isdst]
if {$in_dst} {
	puts "MSVC API localtime() returned in DST ($in_dst)"
} else {
	puts "MSVC API localtime() returned NOT in DST ($in_dst)"
}

# Syntax: time_t asctime(struct tm* t)
set ret [calldllstr$callstyle $h asctime tm]
#set ret [calldllstr$callstyle $hasctime [addressof tm]]	# Correct 0 == -Thu Jan 01 00:00:01 1970-
set line #__LINE__
set dt [string trim $ret [chr 10]]
puts "$line: asctime(): '$dt'"
#assert $dt {eq $_ {Fri Aug 05 14:34:47 2016}} -v -line #__LINE__
unset dt

##########################################################################
# Can't make head or tail of this...
# NO, this is not valid for a fixed date/time
# Surely, for a date in an epoch, functs filling struct tm should fill in is_dst depending on whether DST was
# effective at the time ? But, we get the wrong answer for Aug 5th
# Too many unanswered questions about how these routines are alleged to work!
##########################################################################
set line #__LINE__
# if {[is_dst]} {
if {$in_dst} {		# Wrong!
	textcolor darkyellow
	puts "* Date was in daylight savings time *\n"
	textcolor
	assert [string trim $ret [chr 10]] {== $_ "Fri Aug 05 14:34:47 2016"} -v -line #__LINE__
} else {
	textcolor red
	puts "* Date was NOT in daylight savings time *\n"
	textcolor
	assert [string trim $ret [chr 10]] {== $_ "Fri Aug 05 13:34:47 2016"} -v -line #__LINE__
}
#*/
#assert [string trim $ret [chr 10]] {== $_ "Fri Aug 05 14:34:47 2016"}
#assert [string trim $ret [chr 10]] {== $_ "Fri Aug 05 13:34:47 2016"}

assert [lcount $ret] {== $_ 5} -v -line #__LINE__
assert [lindex $ret 2] {>= $_ 1} -v -line #__LINE__

if {[is_dst]} {							# See [is_dst] enhanced arguments
	textcolor cyan
	puts "We are currently in daylight savings time but ctime_to_double uses mktime"
	textcolor
	assert [date [ctime_to_double 1470404087]] {== $_ "2016/08/05 13:34:47"}		# Returns gmtime not local time
}
/* 
else {
	textcolor cyan
	puts "We are currently NOT in daylight savings time"
	textcolor
	assert [date [ctime_to_double 1470404087]] {== $_ "Fri Aug 05 13:34:47 2016"}
} 
*/

set t [lindex $ret 3]
puts "t lindex 3 of ret is t:'$t'"
replacechar t : " "
assert [lindex $t 0] {[&& [>= $_ 1] [<= $_ 31]]} -v -line #__LINE__
assert [lindex $t 1] {[&& [>= $_ 0] [<= $_ 59]]} -v -line #__LINE__
assert [lindex $t 2] {[&& [>= $_ 0] [<= $_ 59]]} -v -line #__LINE__
unset t

puts "ret:'$ret'"
assert [lindex $ret 4] {== $_ 2016} -v -line #__LINE__

set t 0
struct clear tm
calldll_cdecl $h time t
if {== $t -1} {
	puts "CRT mktime API call failed at line "#__LINE__
	exit
}

puts "Line: "#__LINE__
# Syntax: time_t mktime(struct tm*)
# Syntax: struct tm* localtime(time_t*)
set ts_ptr [calldll_cdecl $h localtime t]		# Return a struct tm* to converted time
puts "Line: "#__LINE__
textcolor yellow
dumpmem $ts_ptr 32
textcolor
puts "Line: "#__LINE__

# There is no actual need to pass back to a Tcl struct other than to demo that we can do this and pass in/out
# We can call asctime directly with ts_ptr
struct copy tm $ts_ptr									# Copy the returned binary struct to our var
#dump tm

# Syntax: time_t asctime(struct tm* t)
set ret [calldllstr_cdecl $h asctime $ts_ptr]					# Passing directly via ptr ts_ptr
#set ret [calldllstr_cdecl $h asctime tm]						# Get string equiv: PASS (current time)
#set ret [calldllstr$callstyle $h asctime [addressof tm]]	# Correct 0 == -Thu Jan 01 00:00:01 1970-

# Wed Nov 20 01:10:12 2019
# 0    1   2     3    4
textcolor darkyellow
puts "asctime(NOW): -"[string trim $ret [chr 10]]-						# Why did I quote it like this?
textcolor
assert [string trim ret [chr 10]] {ne $_ ""} -v -line #__LINE__
assert [lcount $ret] {== $_ 5} -v -line #__LINE__
assert [string length [lindex $ret 1]] {== $_ 3} -v -line #__LINE__		# Test month field
assert [lindex $ret 2] {>= $_ 1} -v -line #__LINE__						# Test day field
assert [lindex $ret 4] {>= $_ 2019} -v -line #__LINE__					# Test year field
newline
dll_close $h															# Releae the DLL

########################################################
textcolor magenta
puts "Testing CRT math functs"
puts "-----------------------"
textcolor

set a 23.456
#puts "Line: "#__LINE__	# FIXED Corruption if this line is removed (is due to var a corrupted)
set b 5.67
puts "sqrt(4.0) : Calling [chr 34]\[calldlldbl$callstyle msvcrt40.dll sqrt 4.0\][chr 34]"
puts "sqrt(4.0) : [chr 34][calldlldbl$callstyle msvcrt40.dll sqrt 4.0][chr 34]"
assert [calldlldbl$callstyle msvcrt40.dll sqrt 4.0] {== [round $_] [round 2.0]} -v -line #__LINE__
#puts "pow(2,6)  : [chr 34][calldlldbl$callstyle msvcrt40.dll pow 2.0 6.0][chr 34]"			# 64.000000000000000
assert [calldlldbl$callstyle msvcrt40.dll pow 2.0 6.0] {== $_ 64.000000000000000} -v -line #__LINE__
#puts "pow(x,y)  : [chr 34][calldlldbl$callstyle msvcrt40.dll pow $a $b][chr 34]"			# 58794934.825978108000 (MSVCRT)
puts "Round is [round [calldlldbl$callstyle msvcrt40.dll pow $a $b] 5]"
assert [calldlldbl$callstyle msvcrt40.dll pow $a $b] {== [round $_ 4] [round 58794934.825978108000 4]} -v -line #__LINE__	# 58794934.82597
#puts "exp(n)    : [chr 34][calldlldbl$callstyle msvcrt40.dll exp $a][chr 34]"				# 15374866997.000761000
assert [calldlldbl$callstyle msvcrt40.dll exp $a] {== [round $_ 4] [round 15374866997.000761000 4]} -v -line #__LINE__
puts "log(n)    : [chr 34][calldlldbl$callstyle msvcrt40.dll log "23.456"][chr 34]"				# 3.155126325704241
puts "log(n)    : [chr 34][calldlldbl$callstyle msvcrt40.dll log $a][chr 34]"				# 3.155126325704241
assert [calldlldbl$callstyle msvcrt40.dll log $a] {== $_ 3.155126325704241} -v -line #__LINE__
puts "log10(n)  : [chr 34][calldlldbl$callstyle msvcrt40.dll log10 $a][chr 34]"				# 1.370253952961034
assert [calldlldbl$callstyle msvcrt40.dll log10 $a] {== $_ 1.370253952961034} -v -line #__LINE__
# How to cast to int
puts "abs(n)    : [chr 34][calldll$callstyle msvcrt40.dll abs $a][chr 34]"					# 1786706395
assert [calldll$callstyle msvcrt40.dll abs $a] {== $_ 1786706395} -v -line #__LINE__
puts "acos(n)   : [chr 34][calldlldbl$callstyle msvcrt40.dll acos 0.76543][chr 34]"			# 0.699087060879116
assert [calldlldbl$callstyle msvcrt40.dll acos 0.76543] {== $_ 0.699087060879116} -v -line #__LINE__
puts "asin(n)   : [chr 34][calldlldbl$callstyle msvcrt40.dll asin 0.345][chr 34]"			# 0.352238785097065
assert [calldlldbl$callstyle msvcrt40.dll asin 0.345] {== $_ 0.352238785097065} -v -line #__LINE__
puts "atan(n)   : [chr 34][calldlldbl$callstyle msvcrt40.dll atan $a][chr 34]"				# 1.528189113209080
assert [calldlldbl$callstyle msvcrt40.dll atan $a] {== $_ 1.528189113209080} -v -line #__LINE__
puts "ceil(n)   : [chr 34][calldlldbl$callstyle msvcrt40.dll ceil $a][chr 34]"				# 24.000000000000000
assert [calldlldbl$callstyle msvcrt40.dll ceil $a] {== $_ 24.000000000000000} -v -line #__LINE__
puts "cos(n)    : [chr 34][calldlldbl$callstyle msvcrt40.dll cos $a][chr 34]"				# -0.105746819836453
assert [calldlldbl$callstyle msvcrt40.dll cos $a] {== $_ -0.105746819836453} -v -line #__LINE__
puts "cosh(n)   : [chr 34][calldlldbl$callstyle msvcrt40.dll cosh $a][chr 34]"				# 7687433498.5003805000
assert [calldlldbl$callstyle msvcrt40.dll cosh $a] {== [round $_ 4] [round 7687433498.5003805000 4]} -v -line #__LINE__
#puts "div(x,y)  : [chr 34][calldlldbl$callstyle msvcrt40.dll div [int a][int b]][chr 34]"	# 
puts "fabs(n)   : [chr 34][calldlldbl$callstyle msvcrt40.dll fabs $a][chr 34]"				# 23.456000000000000
assert [calldlldbl$callstyle msvcrt40.dll fabs $a] {== $_ 23.456000000000000} -v -line #__LINE__
puts "floor(n)  : [chr 34][calldlldbl$callstyle msvcrt40.dll floor $a][chr 34]"				# 23.000000000000000
assert [calldlldbl$callstyle msvcrt40.dll floor $a] {== $_ 23.000000000000000} -v -line #__LINE__
puts "srand(x)  : [chr 34][calldll$callstyle msvcrt40.dll srand [int a]][chr 34]"			# e.g. 6555600
assert [calldll$callstyle msvcrt40.dll srand [int a]] {> $_ 0} -v -line #__LINE__
puts "rand()    : [chr 34][calldll$callstyle msvcrt40.dll rand][chr 34]"					# e.g. 113
assert [calldll$callstyle msvcrt40.dll rand] {> $_ 0} -v -line #__LINE__
puts "sin(n)    : [chr 34][calldlldbl$callstyle msvcrt40.dll sin $a][chr 34]"				# -0.994393086306656
assert [calldlldbl$callstyle msvcrt40.dll sin $a] {== $_ -0.994393086306656} -v -line #__LINE__
puts "tan(n)    : [chr 34][calldlldbl$callstyle msvcrt40.dll tan $a][chr 34]"				# 9.403527102229447
assert [calldlldbl$callstyle msvcrt40.dll tan $a] {== $_ 9.403527102229447} -v -line #__LINE__

unset a b in_dst ret ts_ptr -nocomplain
struct unset tm
unset callstyle h in_dll line t -nocomplain

lib ticol_calldll -unload

#####################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #