#################################################################################################
# File: 			test_lib_db.tcl
# Purpose:			Test file for [db] in ticol_db.dll
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Jan/2016	First version
# 1.01 15/Jan/2022	Updated for unlink catch
#################################################################################################
# Notes:
# Enhanced over XBase. Bigger fields
#################################################################################################
# TODO
# File appears to have a very small leak in lib load without unload, but I can't trace it
# Leak is somewhere in the create/close cycle
#################################################################################################
# Look at record deletion and packing
# May need to reserve x1 byte for deleted field as per XBase
# db_delete, db_undelete, db_pack
#################################################################################################

option expression on
option echo on
set handle 0

textcolor cyan
lib ticol_db -verbose -nocomplain
textcolor

puts "Line: "#__LINE__
set filename "test.db"
#set filename "e:\\temp\\test.db"

puts "Line: "#__LINE__
try {
	puts "[unlink $filename -verbose] file(s) deleted"	# FAILS!
} catch {
	puts "Unlink $filename failed (another process may have deleted it)"
}
puts [ls $filename]

puts "Line: "#__LINE__

## Create a DB ##
## Will open the file and apply a lock to it ##
try {
	puts "Line: "#__LINE__
	set handle [db create $filename {{Name 20} {Age 5} {Address 20} {Town 15} {Phone 20}} "This is a comment - $qbf$qbf$qbf$qbf$qbf$qbf" -y]
	puts "Line: "#__LINE__
} catch {
	textcolor red
	puts "Local Script exception in db create: [db error]"
	textcolor
	die 
}
puts "Line: "#__LINE__
textcolor green
puts "* CREATE SUCCESS *"
textcolor
newline

puts "Line: "#__LINE__
#puts "Handle is $handle"
textcolor yellow
puts "Closing... at line "#__LINE__
textcolor
#_pdb[j]->dbp

## Close the DB ##
try {
	db close $handle -nocomplain
} catch {
	textcolor red
	puts "Script: Exception in db close: [db error]"
	textcolor
	die 
}

puts "Line: "#__LINE__
puts "Done. closed Handle is $handle"
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Re-opening ..."
try {
	set handle [db open $filename "rw"]
} catch {
	textcolor red
	puts "Exception in db open: [db error]"
	textcolor
	die 
}
puts "DB time is [db time $handle]"
textcolor green
puts "* SUCCESS *"
textcolor
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# ADD some data 
textcolor yellow
puts "Setting records..."
textcolor
newline

textcolor cyan
set r [db set $handle 0 "Biffa Bacon"]
puts "db set 0='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 
set r [db set $handle 1 "23"]
puts "db set 1='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 
set r [db set $handle 2 "49 Letsby Avenue"]
puts "db set 2='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 
set r [db set $handle 3 "Fulchester"]
puts "db set 2='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 
set r [db set $handle 4 "(0123) 4567890"]
puts "db set 3='$r'"		# 15
assert $r {== $_ 1} -v -line #__LINE__		# 
textcolor

textcolor magent
set r [db put $handle 1]
puts "db put 1='$r'"		# Save record
assert $r {== $_ 1} -v -line #__LINE__		# 

textcolor
newline
# Record 2 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
set r [db set $handle 0 "Fatha Bacon"]
puts "db set 0='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 
set r [db set $handle 1 "45"]
puts "db set 1='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 

set r [db set $handle 2 "22 Violence Street"]
puts "db set 2='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 

set r [db set $handle 3 "Fulchester"]
puts "db set 2='$r'"
assert $r {== $_ 1} -v -line #__LINE__		# 

set r [db set $handle 4 "(0123) 377729"]
puts "db set 3='$r'"		# 15
assert $r {== $_ 1} -v -line #__LINE__		# 

textcolor magenta
set r [db put $handle 2]
puts "db put 2='$r'"		# Save record
assert $r {== $_ 1} -v -line #__LINE__		# 

textcolor
# Record 3 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "db set 0='[db set $handle 0 "Mutha Bacon"]'"
puts "db set 1='[db set $handle 1 "66"]'"
puts "db set 2='[db set $handle 2 "36DD Battery Street"]'"
puts "db set 2='[db set $handle 3 "Newcastle on Tyne"]'"
puts "db set 3='[db set $handle 4 "(0123) 666616"]'"		# 15
textcolor magenta
set r [db put $handle 3]
puts "db put 3='$r'"		# Save record
assert $r {== $_ 1} -v -line #__LINE__		# 
textcolor

# Record 3 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "db set 0='[db set $handle 0 "Mutha Bacon"]'"
puts "db set 1='[db set $handle 1 "66"]'"
puts "db set 2='[db set $handle 2 "36DD Battery Street"]'"
puts "db set 2='[db set $handle 3 "Fulchester"]'"
puts "db set 3='[db set $handle 4 "(0123) 666616"]'"		# 15
textcolor magenta
set r [db put $handle 3]
puts "db put 3='$r'"		# * OVERWRITE RECORD 3 * (test highest_write count)
assert $r {== $_ 1} -v -line #__LINE__		# 
textcolor

# Record 4 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "db set 0='[db set $handle 0 "Tracey Tunstall"]'"
puts "db set 1='[db set $handle 1 "69"]'"
puts "db set 2='[db set $handle 2 "1 Slag Street"]'"
puts "db set 2='[db set $handle 3 "Fulchester"]'"
puts "db set 3='[db set $handle 4 "(0123) 23232323"]'"		# 15
puts "db put 3='[db put $handle 4]'"		# Save record

# Record 5 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "db set 0='[db set $handle 0 "Viz Comics"]'"
puts "db set 1='[db set $handle 1 "32"]'"
puts "db set 2='[db set $handle 2 "1 Humour Road"]'"
puts "db set 2='[db set $handle 3 "Newcastle on Tyne"]'"
puts "db set 3='[db set $handle 4 "(0123) 23232323"]'"		# 15
textcolor magenta
puts "db put 3='[db put $handle 5]'"		# *OVERWRITE* (test highest_write count)
textcolor

puts "Line: "#__LINE__
puts "Handle is $handle"
textcolor yellow
puts "Closing... at line "#__LINE__
textcolor
set r [db close $handle]
assert $r {== $_ 1} -v -line #__LINE__		# 

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# Read some data back ...
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Re-opening ..."
try {
	set handle [db open $filename "rw"]
} catch {
	textcolor red
	puts "Exception in db open: [db error]"
	textcolor
	die 
}
textcolor green
puts "* SUCCESS *"
textcolor
newline

puts "Line: "#__LINE__
puts "Handle is $handle"
textcolor yellow
puts "Closing... at line "#__LINE__
textcolor

set r [db close $handle]
assert $r {== $_ 1} -v -line #__LINE__		# 


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor yellow
puts "Re-opening ..."
textcolor
newline

try {
	set handle [db open $filename "rw"]
} catch {
	textcolor red
	puts "Exception in db open: [db error]"
	textcolor
	die 
}
textcolor white
puts "DB time is [db time $handle]"
textcolor green
puts "* SUCCESS *"
textcolor
newline

if {== [db get $handle 1] $true} {
	textcolor green
	puts "* \[db get 1\] SUCCESS *"
	textcolor magenta
	puts "Field 0: '[db field $handle 0]'"
	puts "Field 1: '[db field $handle 1]'"
	puts "Field 2: '[db field $handle 2]'"
	puts "Field 3: '[db field $handle 3]'"
	textcolor
} else {
	textcolor red
	puts "db get 1 failed"
	textcolor
}
newline

if {== [db get $handle 2] $true} {
	textcolor green
	puts "* \[db $handle get 2\] SUCCESS *"
	textcolor magenta
	puts "Field 0: '[db field $handle 0]'"
	puts "Field 1: '[db field $handle 1]'"
	puts "Field 2: '[db field $handle 2]'"
	puts "Field 3: '[db field $handle 3]'"
	textcolor
} else {
	textcolor red
	puts "db get 2 failed"
	textcolor
}
newline

if {== [db get $handle 3] $true} {
	textcolor green
	puts "* \[db get 3\] SUCCESS *"
	textcolor magenta
	puts "Field 0: '[db field $handle 0]'"
	puts "Field 1: '[db field $handle 1]'"
	puts "Field 2: '[db field $handle 2]'"
	puts "Field 3: '[db field $handle 3]'"
	textcolor
} else {
	textcolor red
	puts "db get 3 failed"
	textcolor
}
newline

textcolor yellow
puts "Database Information "
textcolor

db info $handle
newline

textcolor yellow
puts "Closing... at line "#__LINE__
textcolor
db close $handle
newline

textcolor red
puts "* $filename is left on the disk"
textcolor

unset filename handle r -nocomplain

#lib ticol_db -unload	# NEW
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #