################################################################################
# File:			test_ticol_dhcp.tcl
# Author:		M Shaw
# Purpose:		Tests of lib ticol_dhcp
# Language:		Ticol Tcl 1.24+
#
# Versions:
# 14/Oct/2017	First version
################################################################################
# Notes:
# Some DHCP commands
# //////////////////////////////////////////////////////////////
# // Error codes:
# //////////////////////////////////////////////////////////////
# // 0 No error
# // 1 Failed to load DLL library IPHlpApi.DLL
# // 2 Failed to link to GetInterfaceInfo in IPHlpApi.DLL
# // 3 Failed to link to IpReleaseAddress in IPHlpApi.DLL
# // 4 Can't allocate memory
# // 5 Failed to renew the address
# // 7 Failed to find the requested adapter by index
# // 8 GetInterfaceInfo returned other error
# //////////////////////////////////////////////////////////////
################################################################################

option echo on
option expression off

textcolor cyan
lib ticol_dhcp -verbose -nocomplain
puts v[lib ticol_dhcp -info]
textcolor
newline

# Get list of adapters
puts "Line: "#__LINE__
for {set i 0} {< $i [info adapter_count]} {++ i} {
	if {dhcp enabled $i} {
		puts "ID:[info adapter_id $i]"
		puts "Index: [info adapter_index $i]"
		puts "DHCP Server     =[info dhcp_server $i]"
		puts "DHCP Lease Start=[date [dhcp start $i]]"
		puts "DHCP Lease End  =[date [dhcp end $i]]"
		if {> [dhcp end $i] 25569.0} {
			set index [info adapter_index $i]
			textcolor magenta
			puts "Renewing $i (index $index)"
			puts "Released?\t[bool [dhcp release [info adapter_index $i] -verbose]]"
			puts "Renewed?\t[bool [dhcp renew [info adapter_index $i] -verbose]]"
			puts "DHCP Server     =[info dhcp_server $i]"
			puts "DHCP Lease Start=[date [dhcp start $i]]"
			puts "DHCP Lease End  =[date [dhcp end $i]]"
			textcolor
			newline
			break
		}
		newline
	}
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #