################################################################################
# File:			test_lib_event.tcl
# Author:		M Shaw
# Purpose:		Tests of lib ticol_event
# Language:		Ticol Tcl 1.26+
#
# Versions:
# 15/Oct/2017	First version
# 18/Oct/2017	Die if not elevated
# 30/Nov/2019	Add elevate+exit clause
################################################################################
# Notes:
# DONE Test if elevated
# Create ticol_event.mc message text file
# Compile using mc.exe with existing bat script
# Ensure added the resulting *.rc to project
# Ensure Event Viewer is closed as this will lock the DLL
# Compile the project
# Run this script
# Run eventvwr.exe
################################################################################
option expression off
puts "Line "#__LINE__

textcolor yellow
puts "Testing \[event\] ... (Requires and will prompt for elevation)"
textcolor
newline

if {! [is_elevated]} {
	elevate
	exit
}

puts "Line "#__LINE__
lib ticol_event -verbose -nocomplain

puts "Line "#__LINE__
set category 0
#define EVENTLOG_SUCCESS 0
#define EVENTLOG_ERROR_TYPE 1
#define EVENTLOG_WARNING_TYPE 2
#define EVENTLOG_INFORMATION_TYPE 4
#define EVENTLOG_AUDIT_SUCCESS 8
#define EVENTLOG_AUDIT_FAILURE 10



# AddEventSource("VBToolbox", 0)
if {event add ticol EVENTLOG_SUCCESS} {
	puts "Successfully added an event source as category EVENTLOG_SUCCESS"
} else {
	puts "Failed to add an event source. Ensure script is run elevated"
}

if {event add ticol EVENTLOG_ERROR_TYPE} {
	puts "Successfully added an event source as category EVENTLOG_ERROR_TYPE"
} else {
	puts "Failed to add an event source. Ensure script is run elevated"
}

if {event add ticol EVENTLOG_WARNING_TYPE} {
	puts "Successfully added an event source as category EVENTLOG_WARNING_TYPE"
} else {
	puts "Failed to add an event source. Ensure script is run elevated"
}

if {event add ticol EVENTLOG_INFORMATION_TYPE} {
	puts "Successfully added an event source as category EVENTLOG_INFORMATION_TYPE"
} else {
	puts "Failed to add an event source. Ensure script is run elevated"
}

if {event add ticol 666} {						# Can only add 0..4 due to string table
	puts "Successfully added an event source as category EVENTLOG_INFORMATION_TYPE"
} else {
	puts "Failed to add an event source. Ensure script is run elevated"
}
newline

assert [event is_source ticol] {== $_ 1} -v -line #__LINE__
assert [event is_source foo32429389] {== $_ 0} -v -line #__LINE__
newline

# logevent("VBToolbox", vbCrLf & "Testing SUCCESS message", EVENTLOG_SUCCESS, vbCrLf & "(Error: 0x00)")
if {event log ticol "Some event happned successfully" EVENTLOG_SUCCESS (0x00)} {
	puts "Successfully logged a SUCCESS event"
} else {
	puts "Failed to log a success event at line "#__LINE__
}
assert [event log ticol "Some event happned successfully" EVENTLOG_SUCCESS (0x00)] {== $_ 1} -v -line #__LINE__

if {event log ticol "An error event happened. This is probably bad" EVENTLOG_ERROR_TYPE "(Code 2)"} {
	puts "Successfully logged an error event"
} else {
	puts "Failed to log an event at line "#__LINE__
}
assert [event log ticol "An error event happened. This is probably bad" EVENTLOG_ERROR_TYPE "(Code 2)"] {== $_ 1} -v -line  #__LINE__

if {event log ticol "A warning event happened. You should check it" EVENTLOG_WARNING_TYPE "(Code 3)"} {
	puts "Successfully logged an event"
} else {
	puts "Failed to log an event at line "#__LINE__
}
assert [event log ticol "A warning event happened. You should check it" EVENTLOG_WARNING_TYPE "(Code 3)"] {== $_ 1} -v -line #__LINE__

if {event log ticol "Here is some nice information" EVENTLOG_INFORMATION_TYPE} {
	puts "Successfully logged an info event"
} else {
	puts "Failed to log an event at line "#__LINE__
}
assert [event log ticol "Here is some nice information" EVENTLOG_INFORMATION_TYPE] {== $_ 1} -v -line #__LINE__

if {event log ticol "Here is some nice information" 666	} {			# Should fail
	puts "Successfully logged an info event"
} else {
	puts "Failed to log an event"
}

newline

# If the event source is removed then eventvwr.exe can't interpret our old events
#assert [event remove ticol] {== $_ 1} -v -line #__LINE__

unset category -nocomplain

lib ticol_event -unload

pause

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #