###########################################################################
# File: 		test_hasp.tcl
# Purpose:		Test cases for ticol_hasp.dll HASP query plugin
# Author: 		M Shaw
# Language:		Ticol Tcl v1.26+
#
# Versions:
# 1.00 13/Apr/2018 First version
# 1.01 18/Feb/2021 Updated to reflect inability to test without HASP keys
###########################################################################
# Notes: 
# lib ticol_hasp exposes 1 command [hasp]
# Requires Apache running locally and emulating a HASP dongle
# Requires use of -port argument to change from default of 1947
# http://$check_server:8800/_int_/tab_feat.html
###########################################################################

textcolor red
puts "HASP tests disabled as no HASP keys are available for testing as of 2018"
textcolor
pause
return

if {< $tcl_version 1.26} {
	puts "Error: Requires Tcl v1.26 or higher. Found v$tcl_version"
	die
}

textcolor cyan
lib ticol_hasp -verbose
textcolor
puts "Testing ticol_hasp.dll plugin"
newline

textcolor yellow
puts "HASP keys found on localhost: '[hasp keys localhost -port 8800]'"
textcolor

assert [hasp keys localhost -port 8800] {ne $_ ""} -v -line #__LINE__
assert [hasp data localhost -port 8800] {ne $_ ""} -v -line #__LINE__
newline

#puts "HASP raw data on localhost: '[hasp data localhost]'"
puts "Raw JSON data..."
textcolor darkyellow
foreach {x} [hasp data localhost -port 8800] {
	foreach {y} $x {
		puts "[lindex $y 1] " -nonewline
	}
	newline
	break
}
textcolor
newline

puts "Module FIDs found on key 456055135: '[hasp modules localhost "456055135" -port 8800]'"
puts "Module expiry for 456055135 / 76: '[hasp expiry localhost 456055135 76 -port 8800]'"
puts "Module expiry for 456055135 / 77: '[hasp expiry localhost 456055135 77 -port 8800]'"
puts "Product type for 456055135 / 76: '[hasp product localhost 456055135 76 -port 8800]'"

textcolor red
puts "Dongle type for 456055135: '[hasp type localhost 456055135 -port 8800]'"
textcolor cyan
puts "Dongle type for 25199190: '[hasp type localhost 25199190 -port 8800]'"
textcolor

/*
# Test MAN iter all - PASS
set port 8800
set i 0
foreach {x} [hasp data localhost -port $port] {
	puts "$i: " -nonewline
	foreach {y} $x {
		puts "[lindex $y 0] => [lindex $y 1], " -nonewline
	}
	newline 2
	++ i
}
*/

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Pass *"
textcolor

# EOF #