###############################################################################################
# File:				test_lib_highlight.tcl
# Author:			M Shaw
# Purpose:			Test file for ticol_highlight.dll -> [highlight] command
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Dec/2019	First version
# 1.01 03/Feb/2022	Changed "cyan" to "aqua"
###############################################################################################
# Notes:
# FIX Add more tests including multi-line
# string [highlight string ?-bat? ?-rtf?]
# string [highlight {} -reset]
###############################################################################################

lib ticol_highlight -nocomplain

# TCL Language
set s [highlight {puts "Hello"}]
# <font color="aqua">puts</font> <font color="yellow">"Hello"</font>
assert $s {eq $_ [unescape {<font color=\"aqua\">puts</font> <font color=\"yellow\">\"Hello\"</font>}]} -v -trace -line #__LINE__
# Unescaped works due to 'adjacent strings'!
assert $s {eq $_ {<font color="aqua">puts</font> <font color="yellow">"Hello"</font>}} -v -trace -line #__LINE__

# BAT Language
set s [highlight {ECHO Hello} -f bat]
# Unescaped works due to 'adjacent strings'!
assert $s {eq $_ {<font color="aqua">echo</font> Hello}} -v -trace -line #__LINE__

# HTML Language
# FIX Should encode < and > tags
# If supplied with &lt; &gt; these literals are colourised so replacement must be local
# <font color="magenta"><</font><font color="magenta">font</font> name<font color="magenta">=</font>\<font color="yellow">"blue\">
set s [highlight [unescape {<font name=\"blue\">}] -f html]
puts $s
newline

set q "\
<font color=\"magenta\">&lt;</font>\
<font color=\"aqua\">font</font>\
 name<font color=\"aqua\">=</font>\
<font color=\"yellow\">\"blue\"</font>\
<font color=\"magenta\">&gt;</font>"

textcolor magenta
sc $s [unescape $q] -verbose
textcolor

assert $s {eq $_ [unescape $q]} -v -trace -line #__LINE__

# ASM Language
set s [highlight {jmp Hello} -f asm]
# Unescaped works due to 'adjacent strings'!
assert $s {eq $_ {<font color="aqua">jmp</font> Hello}} -v -trace -line #__LINE__

lib ticol_highlight -unload

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #