######################################################################################
# File: 			test_html.tcl
# Purpose:			Test [html <command>]
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 09/Dec/2017	First version
######################################################################################
# Notes: 
#	html <subcommand>
#
#	html chomp_tag             Parse an HTML file (see separate test file)
#	html header_field          Return an HTTP/HTML header
#	html remove_header         Strip off an HTTP/HTML header
#	html strip                 Remove all HTML tags from a string
#
# See also: test_html_chomp.tcl
######################################################################################
# FIX 
# /C
# Error: Mismatched double quotes (65)
#
######################################################################################
option expression off

puts "Line: "#__LINE__
lib ticol_html -verbose -nocomplain

# Define some constant HTML which includes an HTTP 1.1 response header
set s "HTTP/1.1 200 OK
Date: Fri, 12 Jun 2017 13:26:15 GMT
Server: Apache/2.4.10 (Ubuntu)
X-Powered-By: PHP/5.5.12-2ubuntu4.3
Redirect: http://localhost.com/index.php

<html><head>
<title>Ticol v1.25 Manual</title>
<meta name=\"GENERATOR\" content=\"Ticol Tcl v1.25 Manual\">
</head>
<body bgcolor=\"#000000\" text=\"lightgrey\" link=\"#00FFFF\" vlink=\"#9F9FFF\" style=\"font-size: 14pt; font-family: Consolas, monospace\">
<pre><strong><big>Welcome to Ticol Tcl Interpreter (C) M Shaw 2015-2017</big></strong>
<pre><small>See any associated README.TXT for latest updates and amendments
</body>
</html>"
newline

puts "Check s: '$s'"

newline

# HTML strip #
textcolor yellow
puts "Testing \[html strip\] @"#__LINE__
textcolor
newline

puts "Line: "#__LINE__
set t [html strip $s]
puts "Line: "#__LINE__
puts "\[string length \$t\]=[string length $t]"
puts "Line: "#__LINE__
assert [string length $t] {== $_ 317} -v -line #__LINE__
assert [string left $t 15] {eq $_ "HTTP/1.1 200 OK"} -v -line #__LINE__

# HTML remove_header #
textcolor yellow
puts "Testing \[html remove_header\] @"#__LINE__
textcolor
newline

puts "Line: "#__LINE__
set t [html remove_header $s]
puts [string length $t]
#puts $t
puts "Line: "#__LINE__
assert [string left $t 12] {eq $_ "<html><head>"} -v -line #__LINE__

# HTML header_field #
textcolor yellow
puts "Testing \[html header_field\] @"#__LINE__
textcolor
newline

puts "Line: "#__LINE__
puts "Date:        \[[html header_field $s Date:]\]"
puts "Redirect:    \[[html header_field $s rEdIrEcT:]\]"
puts "Server:      \[[html header_field s server: -var]\]"
puts "Foo:         \[[html header_field $s foo:]\]"
puts "\"\":          \[[html header_field $s \"\"]\]"
puts "\{\}:          \[[html header_field $s {}]\]"
puts "Line: "#__LINE__
puts "Date:        \[[html header_field $s Date:]\]"
puts "Redirect:    \[[html header_field $s rEdIrEcT:]\]"
puts "Server:      \[[html header_field s server: -var]\]"
puts "Foo:         \[[html header_field $s foo:]\]"
puts "\"\":          \[[html header_field $s \"\"]\]"
puts "\{\}:          \[[html header_field $s {}]\]"
newline

puts "Line: "#__LINE__
assert [html header_field $s ""] {eq $_ ""} -v -line #__LINE__
assert [html header_field $s {}] {eq $_ {}} -v -line #__LINE__
assert [html header_field $s "foo:"] {eq $_ ""} -v -line #__LINE__
assert [html header_field $s "Date:"] {eq $_ "Fri, 12 Jun 2017 13:26:15 GMT"} -v -line #__LINE__
assert [html header_field $s "server:"] {eq $_ "Apache/2.4.10 (Ubuntu)"} -v -line #__LINE__
assert [html header_field $s "reDIRECt:"] {eq $_ "http://localhost.com/index.php"} -v -line #__LINE__
assert [html header_field $s "x-POWERED-by:"] {eq $_ "PHP/5.5.12-2ubuntu4.3"} -v -line #__LINE__


puts "Line: "#__LINE__
lib ticol_html -unload -nocomplain


unset s t -nocomplain

vars

#####################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #