############################################## #############################################
# File:				test_json.tcl
# Author:			M Shaw
# Purpose:			Tests for nested [json_to_list] command
#
# Versions:
# 1.00 26/Nov/2017 First version. json_to_list moved to ticol_json.dll
# 1.01 28/Jul/2018	Return array element
###########################################################################################
# Notes:
# Data base been obfuscated
# ADD Test array element return
###########################################################################################
option expression off
option echo on

textcolor yellow
puts "Testing nested \[json_to_list\] ..."
textcolor
newline

textcolor cyan
lib ticol_json -verbose -nocomplain
textcolor

# We will use some existing HASP data pulled from the Safenet HASP server page (details removed)
set data {
/*JSON:features*/ 
{ "ndx":"99", "ven":"99999",		# Vertical layout
"haspname":"123456789", 
"haspid":"123456789", 
"file":"0x0261", 
"ip":"Local", 
"fid":"99", 
"fn":"", 
"loc":"Local", 
"isloc":"1", 
"acc":"Loc Net Disp ", 
"cnt":"Station", 
"logc":"0", 
"logl":"50", 
"det":"0", 
"detc":"0", 
"locked":"1", 
"dis":"0", 
"ex":"0", 
"unusable":"0", 
"lic":"Expiration Date Tue Jun 16, 2015 23:55", 
"sesc":"0", 
"prname":"Some licensed product 2014-07-04", 
"prid":"1356", 
"typ":"HASP HL NetTime 50" }, 
{ "ndx":"42", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0262", "ip":"Local", "fid":"62", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 50" }, 
{ "ndx":"43", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0263", "ip":"Local", "fid":"63", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"44", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0264", "ip":"Local", "fid":"64", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"45", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0265", "ip":"Local", "fid":"68", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"46", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0266", "ip":"Local", "fid":"73", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"47", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0267", "ip":"Local", "fid":"74", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"48", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0268", "ip":"Local", "fid":"76", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Some product bundle 2014-07-04", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"49", "ven":"67001", "haspname":"123456789", "haspid":"123456789", "file":"0x0269", "ip":"Local", "fid":"78", "fn":"", "loc":"Local", "isloc":"1", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"50", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Expiration Date Tue Jun 16, 2015 23:55", "sesc":"0", "prname":"Maintenance", "prid":"1356", "typ":"HASP HL NetTime 10" }, 
{ "ndx":"1", "ven":"Master", "haspname":"65432109", "haspid":"8765432", "file":"0xFFCB", "ip":"192.168.1.12", "fid":"0", "fn":"", "loc":"SNOOPY", "isloc":"0", "acc":"Loc Net Disp ", "cnt":"Station", "logc":"0", "logl":"0", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Perpetual", "sesc":"0", "prname":"", "prid":"0", "typ":"Sentinel HASP Master" }, 
{ "ndx":"2", "ven":"Master", "haspname":"8765432", "haspid":"8765432", "file":"0xFFE0", "ip":"192.168.1.12", "fid":"-1", "fn":"", "loc":"SNOOPY", "isloc":"0", "acc":"Loc Net ", "cnt":"Login", "logc":"0", "logl":"0", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Perpetual", "sesc":"0", "prname":"", "prid":"0", "typ":"Sentinel HASP Master" }, 
{ "ndx":"3", "ven":"Master", "haspname":"8765432", "haspid":"8765432", "file":"0xFFE1", "ip":"192.168.1.12", "fid":"-1", "fn":"", "loc":"SNOOPY", "isloc":"0", "acc":"Loc Net ", "cnt":"Login", "logc":"0", "logl":"0", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Perpetual", "sesc":"0", "prname":"", "prid":"0", "typ":"Sentinel HASP Master" }, 
{ "ndx":"4", "ven":"Master", "haspname":"8765432", "haspid":"8765432", "file":"0xFFE2", "ip":"192.168.1.12", "fid":"-1", "fn":"", "loc":"SNOOPY", "isloc":"0", "acc":"Loc Net ", "cnt":"Login", "logc":"0", "logl":"0", "det":"0", "detc":"0", "locked":"1", "dis":"0", "ex":"0", "unusable":"0", "lic":"Perpetual", "sesc":"0", "prname":"", "prid":"0", "typ":"Sentinel HASP Master" }, 
{ "fhaspid":"0", "ffea":"0", "cnt":"4" } /* 0 SNTL_ADMIN_STATUS_OK */
}

######################################################################
json_to_list $data tcl_list

puts "Count: [lcount $tcl_list] field(s)"
assert [lcount $tcl_list] {== $_ 14} -v -line #__LINE__
#puts $tcl_list
foreach {x} $tcl_list {
	#puts '$x'
	set server [lindex $x 8 1]
	puts "server:$server"
	if {ne $server ""} {
		assert $server {|| [eq $_ Local] [eq $_ SNOOPY]} -v -line #__LINE__
	}
	set ip [lindex $x 5 1]
	puts "ip:$ip"
	if {ne $ip ""} {
		assert $ip {|| [eq $_ "192.168.1.12"] [eq $_ Local]} -v -line #__LINE__
	}
}

# Test -var arg
set tcl_list {
	/*JSON:features*/ 
	"foo":"123", "bar":"111",		# Vertical layout
	"moo":"666", "par":"999"
}

######################################################################
# FIX list equal ?
# ADD need a cmp command which returns 0 if equal or > 0 at index of first differening char
set l [unescape "{\"foo\" \"123\"} {\"bar\" \"111\"} {\"moo\" \"666\"} {\"par\" \"999\"}"]
puts "l           ='$l'"
puts "json_to_list='[json_to_list tcl_list -var]'"
if {! [string equal $l [json_to_list tcl_list -var]]} {
	textcolor magenta
	puts "[tohex $l] ([string length $l]"
	textcolor cyan
	puts "[tohex [json_to_list tcl_list -var]] ([string length [json_to_list tcl_list -var] ]"
	textcolor
}
textcolor red
puts "equal? [bool [string equal $l [json_to_list tcl_list -var]]]"
textcolor
assert [json_to_list tcl_list -var] {eq $_ $::l} -v -line #__LINE__
# ADD test outvar 
# ADD test outvar array
assert [json_to_list tcl_list -var out] {eq $_ $::l} -v -line #__LINE__
unset out -nocomplain

######################################################################
# *FIX -var
# [IN]
# "foo":"123", "bar":"111",
# "moo":"666", "par":"999"
# [OUT]
# '{"foo" "123"} {"bar" "111"} {"moo" "666"} {"par" "999"}'
# list [json_to_list json_string|variable ?-var? ?return-variable?]
puts $tcl_list
textcolor yellow
puts "Test return std var from json_to_list ...\n"
textcolor
set outvar ""
puts "type outvar: '[type outvar]'"
json_to_list tcl_list -var outvar
puts "type outvar: '[type outvar]'"
assert $outvar {eq $_ {{"foo" "123"} {"bar" "111"} {"moo" "666"} {"par" "999"}}  } -v -line #__LINE__

######################################################################
textcolor yellow
puts "Test return array from json_to_list ..."
textcolor
set a(1) ""
json_to_list tcl_list -var a(1)
assert $a(1) {eq $_ {{"foo" "123"} {"bar" "111"} {"moo" "666"} {"par" "999"}}  } -v -line #__LINE__

unset outvar -nocomplain

unset data ip server tcl_list x -nocomplain

######################################################################
newline
puts "Line: "#__LINE__
# https://www.w3schools.com/js/js_json_objects.asp
set json { "name":"John", "age":30, "car":null }
set l [json_to_list $json]
puts "Name: [lindex $l 0 1]"
assert [lindex $l 0 1] {eq $_ John} -v -line #__LINE__
puts "Age:  [lindex $l 1 1]"
assert [lindex $l 1 1] {eq $_ "30"} -v -line #__LINE__
puts "Car:  [lindex $l 2 1]"
assert [lindex $l 2 1] {eq $_ "null"} -v -line #__LINE__

puts "Line: "#__LINE__
set json {}
puts "Line: "#__LINE__
set l [json_to_list $json]
puts "Line: "#__LINE__
assert $l {eq $_ ""} -v -line #__LINE__
newline

# Malformed inputs ########################################################################

textcolor yellow
puts "Testing non-JSON data ..."
textcolor
newline

puts "Line: "#__LINE__
unset json l -nocomplain

assert [json_to_list a] {eq $_ "a"} -v -line #__LINE__
assert [json_to_list "{foo}{bar}"] {eq $_ "{foo}{bar}"} -v -line #__LINE__

lib ticol_json -unload -nocomplain

###########################################################################################
array unset a -nocomplain
unset out -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #