##############################################################################################
# File: 		test_lindex.tcl
# Purpose:		Test cases for [lindex]
# Author: 		M Shaw
# Language:		Ticol Tcl v1.26+
#
# Versions:
# 1.00 18/Apr/2018 First version
##############################################################################################
# Notes: 
# FIX Fix list of troublesome lindex cases
# Lindex 0 is a problem where there is only one sublist
# Fix if list of lists with only 1 sublist then [lindex list 0] returns entire list
# Returned json data is our main current problem
##############################################################################################

textcolor cyan
lib ticol_json -verbose -nocomplain
textcolor

##############################################################################################
assert [lindex {{a}}] {eq $_ "{a}"} -v -line #__LINE__									# PASS
# FAIL
assert [lindex {{a}} 0] {eq $_ a} -v -line #__LINE__									# FAIL

assert [lindex {{a b c}}] {eq $_ "{a b c}"} -v -line #__LINE__							# PASS
assert [lindex {{a b c}} 0] {eq $_ "a b c"} -v -line #__LINE__							# PASS

assert [lindex {{a} {b} {c}} 0] {eq $_ a} -v -line #__LINE__ 							# PASS
assert [lindex {{a b} {c d} {e f}}] {eq $_ "{a b} {c d} {e f}"} -v -line __LINE__		# PASS
assert [lindex {{a b} {c d} {e f}} 0] {eq $_ "a b"} -v -line #__LINE__					# PASS
assert [lindex "{{a b} {c d} {e f}}"] {eq $_ "{{a b} {c d} {e f}}"} -v -line __LINE__	# PASS
# FAIL
assert [lindex "{{a b} {c d} {e f}}" 0] {eq $_ "{a b} {c d} {e f}"} -v -line #__LINE__	# FAIL

##############################################################################################

# Should be a list of x1 list as '{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}'
set data [json_to_list {{"fhaspid":"0", "ffea":"0", "cnt":"0"}}]
vars data

##############################################################################################
# FIX eq fails here
# arg 1: '{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}'
# assert: Line 17: Assertion failed: '{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}'
# != 'eq $_ {{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}'
#{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}
#{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}
##############################################################################################

# FAIL Can't quote or escape this
# assert [lindex {{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}} 0] {eq $_ {{fhaspid "0"} {"ffea" "0"} {"cnt" "0"}}  } -v -line #__LINE__
puts [lindex {{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}} 0]
# '{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}'
vars data


# FIX bad eq
#assert [lindex $data] {eq $_ {{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}} -v -line #__LINE__
#assert [lindex $data 0] {eq $_ "{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}"} -v -line #__LINE__
# FIX bad eq
#assert [lindex {{{fhaspid 0} {ffea 0} {cnt 0}}} 0] {eq $_ "{fhaspid 0} {ffea 0} {cnt 0}"} -v -line #__LINE__

# list      data                 {{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}

#puts "Line :"#__LINE__
#puts '[lindex $data 0]'				# FAILS
#assert [lindex $data 0] {eq $_ "{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}"} -v -line #__LINE__

/* ActiveState

set data {{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}}	# In order to get stored var with outer braces
{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}
puts [lindex $data 0]
{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}
*/

##############################################################################################
# 2 item list
##############################################################################################

puts "Line :"#__LINE__
set data [json_to_list {{"fhaspid":"0", "ffea":"0", "cnt":"0"}, {"fhaspid":"1", "ffea":"1", "cnt":"1"}}]
# Gives: '{{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}} {{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}}'
vars data

puts '[lindex $data 0]'			# Works OK '{"fhaspid" "0"} {"ffea" "0"} {"cnt" "0"}'
puts '[lindex $data 1]'			# Works OK '{"fhaspid" "1"} {"ffea" "1"} {"cnt" "1"}'

##############################################################################################

# 1 item list FAILS
puts "Line :"#__LINE__
puts [lindex {{{foo bar} {moo par}}} 0]		# +1 brace {} which won't get passed
assert [lindex {{{foo bar} {moo par}}} 0] {eq $_ "{foo bar} {moo par}"} -v -line #__LINE__
# ActiveState: {foo bar} {moo par}

# 2 item list PASS
puts "Line :"#__LINE__
puts [lindex {{{foo bar} {moo par}} {{ooh arr} {noo narr}}} 0]		# +1 brace {} which won't get passed
# ActiveState: {foo bar} {moo par}
assert [lindex {{{foo bar} {moo par}} {{ooh arr} {noo narr}}} 0] {eq $_ "{foo bar} {moo par}"} -v -line #__LINE__

lib ticol_json -unload

unset data -nocomplain
##############################################################################################
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #