#################################################################################
# File:				test_line.tcl
# Author:			M Shaw
# Purpose:			Test cases for ticol_line.dll
# Language:			Ticol Tcl v1.26+
# Prerequisites:	alice.txt (Gutenberg Alice in Wonderland) or lorem.txt
#
# Versions:
# 1.00 29/Jul/2018	First version
# 1.01 31/Jul/2018	Updated for more efficient timings
#################################################################################
# Notes:
# This plugin treats freeform CRLF-delimited text as if it was an array and
# lines within the text string may be addressed randomly
#################################################################################

option echo on
option expression off

textcolor yellow
puts "Testing ticol_line -> \[line\] ..."
textcolor
newline

set filename lorem.txt			# File has 126 lines

if {[file exists alice.txt]} {
#	set filename alice.txt
}

puts "Line: "#__LINE__
if {! [file exists $filename]} {
	die "Script [info script] requires $filename"
}
puts "Line: "#__LINE__
if {! [file exists empty.txt]} {
	die "Script [info script] requires empty.txt (as 0 bytes)"
}

puts "Line: "#__LINE__
textcolor cyan
lib ticol_line -verbose -nocomplain
textcolor
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

set s [readfile empty.txt]
puts "Count of empty.txt is: '[line count $s]'"
assert [line count $s] {== $_ 0} -v -line #__LINE__


set s [readfile $filename]
puts "Count of $filename is: '[line count $s]'"
assert [line count $s] {== $_ 128} -v -line #__LINE__
set lines [line count $s]


set s [readfile $filename]
puts "getting line 66 by string literal"
puts "line 66: '[line 66 $s]'"
set start [timer]
assert [line 66 $s] {eq $_ "{omnes res ibi draconem devorat}"} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	#


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

puts "getting LAST line $lines by string literal"
puts "line $lines: '[line $lines $s]'"
set start [timer]
assert [line $lines $s] {eq $_ "-o-"} -v -line #__LINE__
#assert [line $lines $s] {eq $_ "{non opus habet, et iure usus seu limita}"} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	# 40ms -> 0 ms


puts "getting line  by -var name"
puts "line 66: '[line 66 s -var]'"
set start [timer]
assert [line 66 $s] {eq $_ "{omnes res ibi draconem devorat}"} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	# 53 ms -> 1 ms


# FIXED - THIS LEAKS!
# [lcount] not counting Empty line entry {}
puts "Testing line range"
set start [timer]
puts "line 66 3: '[line 66 $s 3]'"
puts "line 66 3: '[lcount [line 66 $s 3]]'"
assert [lcount [line 66 $s 3]] {== $_ 3} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	#


# Read the last line for 10 lines. Should return just 1 line
puts "Testing line range"
puts "line $lines x 10: '[line $lines $s 10]'"
set start [timer]
assert [lcount [line $lines $s 10]] {== $_ 1} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	#

# Read the last line-10 for 10 lines. Should return 10 lines
puts "Testing line range"
puts "line [- $lines 10] x 10: '[line [- $lines 10] $s 10]'"
set start [timer]
assert [lcount [line [- $lines 10] $s 10]] {== $_ 10} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	#

puts "Testing line range @"#__LINE__
puts "line 23 x 4: '[line 23 $s 4]'"
set start [timer]
assert [line 23 $s 4] {eq $_ "{quae metaphorae tamquam infans presentare} \
{virtutes inoperantur in figura} fortuna {}"} -v -line #__LINE__
puts "Took [- [timer] $start] ms @"#__LINE__	# 47 ms -> 2 ms

# Test array return from string
puts "Testing array return"
set start [timer]
puts "line 23 x 4 == '[line 23 $s 4 -array a]'"
puts "Took [- [timer] $start] ms @"#__LINE__	#
set start [timer]
array walk a
assert $a(0) {eq $_ "quae metaphorae tamquam infans presentare"} -v -line #__LINE__
assert $a(2) {eq $_ "fortuna"} -v -line #__LINE__
assert $a(3) {eq $_ ""} -v -line #__LINE__

array unset a -nocomplain

# Test array return from string
puts "Testing array return with -var"
set start [timer]
puts "line 23 x 4 == '[line 23 s 4 -var -array a]'"
puts "Took [- [timer] $start] ms @"#__LINE__	#
set start [timer]
array walk a
assert $a(0) {eq $_ "quae metaphorae tamquam infans presentare"} -v -line #__LINE__
assert $a(2) {eq $_ "fortuna"} -v -line #__LINE__
assert $a(3) {eq $_ ""} -v -line #__LINE__

# */

array unset a -nocomplain
unset filename lines s start -nocomplain

vars

########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

/*
if {is_elevated} {
	puts "This script is already elevated"
	pause
} #*/

# EOF #