###############################################################
# File: 			test_lib_linkedlist.tcl
# Purpose:			Test file for lib linkedlists
# Author: 			M Shaw
# Language:			Ticol Tcl v1.21+
# See also: 		test_lib_list.tcl
#
# Versions:
# 1.01 16/Dec/2016	First version
###############################################################
# Notes:
# Merge with test_lib_list.tcl
###############################################################

textcolor yellow
puts "Testing plugin linked lists ..."
newline

textcolor cyan
lib ticol_list -verbose -nocomplain
textcolor

set handle [linkedlist create]
assert $handle {> $_ 0}

set i 1
foreach {x} $qbf {
	linkedlist add $handle $x $i
	++ i
}
linkedlist insert $handle {} start 10
linkedlist insert $handle dog end 11
puts "Count=[linkedlist count $handle]"
textcolor darkyellow
linkedlist walk $handle
textcolor

assert [linkedlist count $handle] {== $_ 11} -v -line #__LINE__
assert [linkedlist get $handle start] {== $_ 10} -v -line #__LINE__
assert [linkedlist get $handle The] {== $_ 1} -v -line #__LINE__
assert [linkedlist get $handle fox] {== $_ 4} -v -line #__LINE__
assert [linkedlist get $handle dog] {== $_ 9} -v -line #__LINE__
assert [linkedlist get $handle end] {== $_ 11} -v -line #__LINE__

assert [linkedlist item $handle 1] {eq $_ The} -v -line #__LINE__
assert [linkedlist item $handle 4] {eq $_ fox} -v -line #__LINE__
assert [linkedlist item $handle 9] {eq $_ dog} -v -line #__LINE__
assert [linkedlist item $handle 999] {eq $_ {}} -v -line #__LINE__		# Invalid item returns ""

textcolor darkyellow
puts "Line "#__LINE__
for {set i 1} {< $i [linkedlist count $handle]} {++ i} {
	puts [linkedlist item $handle $i]
}
textcolor
newline
puts "Line "#__LINE__
puts "dog present?: [expr [linkedlist in_list $handle dog] ? return Yep! : return Nope]"
puts "Line "#__LINE__
puts "cat present?: [bool [linkedlist in_list $handle cats]]"
puts "Line "#__LINE__

assert [linkedlist in_list $handle dog] {== $_ 1} -v -line #__LINE__
assert [linkedlist in_list $handle fred] {== $_ 0} -v -line #__LINE__

lib ticol_list -unload

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
unset handle i x -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #