############################################################################
# File: 			test_lib_list.tcl
# Purpose:			Unit test using lib ticol_list
# Author: 			M Shaw
# Language:			Ticol Tcl v1.21+, v1.26+
# See also: 		test_lib_linkedlist.tcl
#
# Versions:
# 1.00 15/Dec/2016	First version
# 1.01 30/Jan/2022	Updated to unit test
############################################################################
# Notes:
# A few unit tests
# qbf = "The quick brown fox jumps over the lazy dog" (9)
############################################################################

textcolor cyan
lib ticol_list -verbose -nocomplain
textcolor
newline

halt											# Enable debugger

set handle [linkedlist create]					# Create a linked list object

set i 1
foreach {x} $qbf {
	linkedlist insert $handle {} $x $i			# Insert some items
	++ i
}

textcolor darkyellow
linkedlist walk $handle
textcolor
linkedlist unset $handle

#####################################################################

halt
set handle [linkedlist create]					# Create another linked list object
puts "Handle to list is $handle"

assert $handle {!= $_ 0} -v -line #__LINE__

set i 0
foreach {x} $qbf {
	linkedlist add $handle $x $i
	++ i
}
assert [linkedlist count $handle] {== $_ 9} -v -line #__LINE__											

set j 0
try {											# Test bad handle access trapped
	puts [linkedlist count handle]
} catch {
	set j 1
}
assert $j {== $_ 1} -v -line #__LINE__


# Insert more items
linkedlist insert $handle jumps "rapidly" $i	# Insert after jumps
assert [linkedlist in_list $handle rapidly] {== $_ 1} -v -line #__LINE__
assert [linkedlist count $handle] {== $_ 10} -v -line #__LINE__
linkedlist walk $handle						
loop j 0 [linkedlist count $handle] {
	puts "Item $j: [linkedlist item $handle $j]"
}
assert [linkedlist item $handle 8] {eq $_ "lazy"} -v -line #__LINE__

++ i
linkedlist insert $handle fox dog $i			# Add dog after fox
assert [linkedlist count $handle] {== $_ 11} -v -line #__LINE__											

linkedlist unset $handle fox               		# Remove fox
assert [linkedlist count $handle] {== $_ 10} -v -line #__LINE__
loop j 0 [linkedlist count $handle] {
	puts "Item $j: [linkedlist item $handle $j]"
}
assert [linkedlist item $handle 3] {eq $_ "dog"} -v -line #__LINE__

/*
textcolor darkyellow
linkedlist walk $handle
textcolor	# */

linkedlist unset $handle
assert $handle {!= $_ 0} -v -line #__LINE__		# unset doesn't nullify the handle!

#####################################################################
	
halt
set handle [linkedlist create]					# Create another linked list object
puts "Handle to list is $handle"
assert $handle {!= $_ 0} -v -line #__LINE__

foreach {x} $qbf {
	linkedlist add $handle $x
}
assert [linkedlist count $handle] {== $_ 9} -v -line #__LINE__											

linkedlist insert $handle jumps rapily
assert [linkedlist count $handle] {== $_ 10} -v -line #__LINE__											
linkedlist insert $handle {} "*start*"       	# Insert at start
assert [linkedlist count $handle] {== $_ 11} -v -line #__LINE__											
linkedlist insert $handle fox dog
assert [linkedlist count $handle] {== $_ 12} -v -line #__LINE__											
linkedlist unset $handle fox
assert [linkedlist count $handle] {== $_ 11} -v -line #__LINE__											

textcolor darkyellow
linkedlist walk $handle
textcolor

linkedlist unset $handle
puts "Test use of bad handle (after unset)..."
set j 0
try {
	linkedlist walk $handle
} catch {
	set j 1
}
assert $j {== $_ 1} -v -line #__LINE__

set handle [linkedlist create]					# Create another linked list object
assert $handle {!= $_ 0} -v -line #__LINE__

linkedlist add $handle hello
assert [linkedlist count $handle] {== $_ 1} -v -line #__LINE__											
linkedlist add $handle pi 3.14159265358979323
assert [linkedlist count $handle] {== $_ 2} -v -line #__LINE__											
assert [linkedlist add $handle "hello" "There"] {== $_ 1} -v -line #__LINE__	# Add rvalue
loop j 0 [linkedlist count $handle] {
	puts "Item $j: [linkedlist item $handle $j]"
}
assert [linkedlist count $handle] {== $_ 2} -v -line #__LINE__											
puts [linkedlist add $handle "world"]
assert [linkedlist count $handle] {== $_ 3} -v -line #__LINE__

puts "get hello: [linkedlist get $handle hello]"
assert [linkedlist get $handle hello] {eq $_ "There"} -v -line #__LINE__

puts "Count is: [linkedlist count $handle]"
assert [linkedlist count $handle] {== $_ 3} -v -line #__LINE__

puts "hello in list? [linkedlist in_list $handle hello]"
assert [linkedlist in_list $handle hello] {== $_ 1} -v -line #__LINE__
puts "querty in list? [linkedlist in_list $handle querty]"
assert [linkedlist in_list $handle foobar] {== $_ 0} -v -line #__LINE__

puts [linkedlist insert $handle over "Unger"]
assert [linkedlist count $handle] {== $_ 4} -v -line #__LINE__
puts [linkedlist insert $handle over "Dunn"]
assert [linkedlist count $handle] {== $_ 5} -v -line #__LINE__
puts [linkedlist insert $handle world "send"]
assert [linkedlist count $handle] {== $_ 6} -v -line #__LINE__
puts [linkedlist insert $handle "" "START"]		# FIX should go to start ?
assert [linkedlist count $handle] {== $_ 7} -v -line #__LINE__
#puts [linkedlist unset $handle fox]
linkedlist walk $handle
# FIX
assert [linkedlist unset $handle fox] {== $_ 1} -v -line #__LINE__	# FIX Not in list so should return false?
assert [linkedlist count $handle] {== $_ 7} -v -line #__LINE__

puts "Count is: [linkedlist count $handle]"
textcolor darkyellow
linkedlist walk $handle
textcolor

linkedlist unset $handle

# ADD look at local insertion of node
# Need returned ptr to head?
# [ll address $handle value] return ptr?
/*
# Iter... ?
set p [linkedlist head $handle]
while {[$p]} {
	# [linkedlist insert $handle Value]		# Insert at current iter pos or at head by default ? (or at end?)
	# [linkedlist insert_here $handle Address Value]		
	set p [linkedlist next $handle Hello]
}
*/

############################################################################
set j 0
try {
	puts [linkedlist add -1 "hello" "There"]
} catch {
	set j 1
}
assert $j {== $_ 1} -v -line #__LINE__

lib ticol_list -unload

############################################################################
unset handle i j x -nocomplain 
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #