#################################################################################
# File: 			test_mapdrive.tcl
# Purpose:			Tests for lib ticol_mapdrive [mapdrive]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 12/Jan/2018	First version
# 1.01 20/Nov/2019	Added ROS ignore block
# 1.02 05/Jun/2022	Added conditional macro wrappers
#################################################################################
# Notes:
# May not connect to default share on Windows NT4.0 using "localhost"
# DONE FIX Win2k map fails (used 127.0.0.1 or [info hostname])
# DONE FIX Unable to delete due to [file isdirectory] failing
#################################################################################
option expression off

#define RUN_MAPDRIVE_TEST		# Remove or comment using "# " to disable the test
#ifdef RUN_MAPDRIVE_TEST

set localhost [info hostname]
# set localhost "127.0.0.1"
# set localhost "localhost"		# Alternative. May not work in win2k

#puts [array item env windir -nocase]
if {eqi [array item env windir -nocase] "C:\\ReactOS"} {	# No SMB stack on ROS yet
	puts "Not running [info script] on ReactOS"
	return
}

textcolor cyan
lib ticol_mapdrive -unload -nocomplain
lib ticol_mapdrive -verbose -nocomplain
textcolor
newline

set i 0

puts "Testing drive mapping by mapping to local default share //$localhost/c\$\n"

# Default share may fail to map
set letter [mapdrive * //$localhost/c$]
if {ne $letter ""} {
	textcolor green
	puts "Success. Mapped drive $letter to //$localhost/c\$ at line "#__LINE__
	textcolor
	set i 1
} else {
	textcolor red
	puts "Failed to map drive letter to //$localhost/c\$ at line "#__LINE__
	textcolor
	puts "Ensure you have an assigned IP address"
	puts "Ensure you have a live or loopback network connection"
	puts "You may need to either elevate this script or"
	puts "edit this script to use a suitable test share other than c\$"
	puts "On Windows NT4 you may need to install the NetBEUI protocol or"
	puts "enable NetBIOS over TCP, or use the hostname"
}

assert $i {== $_ 1} -v -line #__LINE__		# Assert success in mapping unless elevated

# FIX file isdirectory returns 0 for net drive

set i 0
if {file isdirectory $letter} {
	puts "Drive $letter exists OK at line "#__LINE__
	if {mapdrive $letter -delete -force} {
		textcolor green
		puts "Success. Deleted drive $letter"
		textcolor
		set i 1
	} else {
		textcolor red
		puts "Failed to delete $letter at line "#__LINE__
		textcolor
		die
	}
} else {
	textcolor red
	puts "Letter $letter does not exist at line "#__LINE__
	puts "Check \[file isdirectory $letter\]"
	textcolor
	die
}

assert $i {== $_ 1} -v -line #__LINE__		# Assert success in unmapping

lib ticol_mapdrive -unload -nocomplain

unset i letter localhost -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

#else
	puts "[file name [info script]] is not enabled. Edit the script to enable"
#endif

newline
textcolor green
puts "* Done *"
textcolor

# EOF #