################################################################################
# File:			test_lib_misc.tcl
# Author:		M Shaw
# Purpose:		Tests of lib ticol_misc
# Language:		Ticol Tcl 1.24..1.26+
#
# Versions:
# 15/Oct/2017	First version
# 22/Dec/2017	Added [uptime]
# 10/Nov/2019	Added [sm] (SystemMetrics)
# 10/Dec/2019	Rename [sm] to [sysmetrics]
################################################################################
# Notes:
################################################################################
/*
  ip_match mask ip-address   # Match an IP address to a mask (wildcards ok)
                             # Returns a boolean 1/0
  cidr_match mask ip-address # Match an IP address to a CIDR mask ?.?.?.?/N
                             # address / bitmask
                             # Returns a boolean 1/0
  is_mac mac-address ?-f?    # Checks if string is a formatted MAC address
                             # Returns a boolean 1/0
  is_mouse_present           # Returns a boolean 1/0
  is_networked               # Returns a boolean 1/0
  is_safe_mode               # Returns a boolean 1/0
  monitorcount               # Returns a count of connected monitors
  rot13 string               # Perform rot-13 transformation on string
                             # and return the result (source is unchanged)
  uptime                     # Windows uptime in seconds since last boot
  volume serial X:           # Return the serial of a disk as a hex value
  volume filesystem X:       # Return the file system type of a disk volume
  wallpaper get              # Get the path+filename of the current wallpaper
  wallpaper set file ?align? # Set the current wallpaper with alignment...
                             # -centre, -tile, -stretch
*/							 

textcolor cyan
lib ticol_misc -verbose -nocomplain
textcolor
newline

assert [ip_match 192.168.3.* 192.168.3.1] {== $_ 1} -v -line #__LINE__
assert [ip_match 192.168.3.* 192.168.101.1] {== $_ 0} -v -line #__LINE__
assert [ip_match 192.168.3.* {}] {== $_ 0} -v -line #__LINE__
assert [ip_match 10.* 10.11.12.13] {== $_ 1} -v -line #__LINE__

assert [cidr_match 192.0.0.0/8 192.168.101.1] {== $_ 1} -v -line #__LINE__
assert [cidr_match 192.168.3.0/8 192.168.101.1] {== $_ 1} -v -line #__LINE__
assert [cidr_match 192.168.3.0/24 192.168.3.1] {== $_ 1} -v -line #__LINE__
assert [cidr_match 192.168.3.0/24 192.168.101.1] {== $_ 0} -v -line #__LINE__
assert [cidr_match 192.0.0.0/8 ""] {== $_ 0} -v -line #__LINE__
assert [cidr_match 192.0.0.0/8 {}] {== $_ 0} -v -line #__LINE__

assert [is_mac ""] {== $_ 0} -v -line #__LINE__
assert [is_mac {}] {== $_ 0} -v -line #__LINE__
assert [is_mac -2] {== $_ 0} -v -line #__LINE__
assert [is_mac pqrstuvwxyza] {== $_ 0} -v -line #__LINE__
assert [is_mac abcdefgh] {== $_ 0} -v -line #__LINE__
assert [is_mac 001122aabbcc] {== $_ 1} -v -line #__LINE__
assert [is_mac 00-11-22-aa-bb-cc -f] {== $_ 1} -v -line #__LINE__

assert [rot13 $qbf] {eq $_ "Gur dhvpx oebja sbk whzcf bire gur ynml qbt"} -v -line #__LINE__
assert [rot13 [rot13 ""]] {eq $_ ""} -v -line #__LINE__
assert [rot13 [rot13 {}]] {eq $_ {}} -v -line #__LINE__
assert [rot13 [rot13 "a"]] {eq $_ "a"} -v -line #__LINE__
assert [rot13 [rot13 a]] {eq $_ a} -v -line #__LINE__
assert [rot13 [rot13 $qbf]] {eq $_ $::qbf} -v -line #__LINE__

if {> [info winver -vernum] 5.0} {						# Not NT4
	assert [monitorcount] {> $_ 0} -v -line #__LINE__
}

assert [volume filesystem c:] {
	|| UDF [|| FAT [|| [eq $_ NTFS] [eq $_ FAT32]]]
} -v -line #__LINE__

assert [volume serial c:] {ne $_ ""}  -v -line #__LINE__
assert [is_safe_mode] {!= $_ 1} -v -line #__LINE__

# uptime is not accurate
# This is because API does not take into account sleep/hibernate time
set bt [- [now] [/ [uptime] 86400.0]]
set ut [uptime]
puts "DTCT: [double_to_ctime [/ $ut 86400.0]]"
#puts [clock format [double_to_ctime [+ [now] [/ $ut 86400.0]]] -format %d %H:%M:%S]
#puts "[/ $ut 86400] day(s) [/ [% $ut 86400] 3600] hours  [% [% $ut 86400] 3600] [% [% $ut 86400] 60] seconds"
#int seconds = (totalSeconds % 60);
#int minutes = (totalSeconds % 3600) / 60;
#int hours = (totalSeconds % 86400) / 3600;
#int days = (totalSeconds % (86400 * 30)) / 86400;
puts "[/ [% $ut 86400] 3600] hours [/ [% $ut 3600] 60] minutes [% $ut 60] seconds"
puts "Uptime is: $ut seconds ([format %.2f [/ $ut 86400.0] ] day(s))"
puts "Boot time: $bt"
puts "\[now\] is [now]"
if {>= [info winver -vernum] 5.1} {		# XP or higher only
	assert [uptime] {> $_ 0} -v -line #__LINE__
	puts "Boot date+time: [date $::bt]"
	#puts "Boot date+time: [date [- [now] [/ $::bt 86400.0]]]"
	assert [now] {> $_ $::bt} -v -line #__LINE__
}

################################################################################
# [sysmetrics] 10/Nov/2019
newline
textcolor yellow
puts "Testing \[sysmetrics\] ..."
textcolor
newline

#define SM_CXSCREEN 0
#define SM_CYSCREEN 1
#define SM_NETWORK 63
#define SM_SLOWMACHINE 73
#define SM_MOUSEPRESENT 19
#define SM_REMOTESESSION 0x1000
#define SM_SHUTTINGDOWN 0x2000

textcolor darkyellow
if {sysmetrics SM_MOUSEPRESENT} {
	puts "* A mouse is present"
}
if {sysmetrics SM_NETWORK} {
	puts "* This machine is networked"
}
puts "* Screen width is [sysmetrics SM_CXSCREEN] px [sysmetrics 0]"
puts "* Screen height is [sysmetrics SM_CYSCREEN] px [sysmetrics 1]"

textcolor
assert [sysmetrics SM_CXSCREEN] {> $_ 0} -v -line #__LINE__
assert [sysmetrics SM_CYSCREEN] {> $_ 0} -v -line #__LINE__
assert [sysmetrics SM_SLOWMACHINE] {== $_ 0} -v -line #__LINE__				# We should hope not!
assert [sysmetrics SM_SHUTTINGDOWN] {== $_ 0} -v -line #__LINE__			# We should hope not!
#assert [sysmetrics SM_NETWORK] {> $_ 0} -v -line #__LINE__

lib ticol_misc -unload

unset bt ut -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #