########################################################################################
# File: 			test_lib_r2d.tcl
# Purpose:			Tests for ticol_r2d.dll
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 19/Sep/2019	First version
# 1.01 16/Feb/2022	Updated for changed handling to 0
# 1.02 21/Mar/2022	Added a few more tests
# 1.03 07/Jun/2022	Revised for changes to [loop] bounds
# 1.04 28/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
########################################################################################
# Notes:
# Reverted: (now returns "")
# 0 was changed to return " " as this made more sense when iterating during conversion
# to say, clock time. If this raised an error then clock values of 0 would throw an
# exception   23:59:00 would thus raise an exception. Now returns xx:xx:" "
########################################################################################
option expression off

lib ticol_r2d -verbose

puts "Testing ... @"#__LINE__
/*
set i 0
try {
	decimal_to_roman 0			# Invalid value (changed from Feb 2022 - 0 now returns space)
} catch {
	set i 1
}
assert $i {== $_ $false} -v -line #__LINE__	
# */

# Test zero handling
assert [decimal_to_roman 0] {eq $_ ""} -v -line #__LINE__		# From March 2022
puts [roman_to_decimal [decimal_to_roman 0]]
assert [roman_to_decimal [decimal_to_roman 0]] {== $_ 0} -v -line #__LINE__		# From March 2022

# Test out of range value...
puts "Line: "#__LINE__
set i 0
try {
	puts [decimal_to_roman 10001]			# Invalid value (out of range)
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
loop i 1 4 {
	puts $i
}

# A selection of recent dates
assert [decimal_to_roman 1] {eq $_ I} -v -line #__LINE__
assert [decimal_to_roman 123] {eq $_ CXXIII} -v -line #__LINE__
assert [decimal_to_roman 456] {eq $_ CDLVI} -v -line #__LINE__
assert [decimal_to_roman 789] {eq $_ DCCLXXXIX} -v -line #__LINE__
assert [decimal_to_roman 1011] {eq $_ MXI} -v -line #__LINE__
assert [decimal_to_roman 1955] {eq $_ MCMLV} -v -line #__LINE__
assert [decimal_to_roman 1968] {eq $_ MCMLXVIII} -v -line #__LINE__
assert [decimal_to_roman 1970] {eq $_ MCMLXX} -v -line #__LINE__
assert [decimal_to_roman 2000] {eq $_ MM} -v -line #__LINE__
assert [decimal_to_roman 2001] {eq $_ MMI} -v -line #__LINE__
assert [decimal_to_roman 2010] {eq $_ MMX} -v -line #__LINE__
assert [decimal_to_roman 2020] {eq $_ MMXX} -v -line #__LINE__
assert [decimal_to_roman 2021] {eq $_ MMXXI} -v -line #__LINE__
assert [decimal_to_roman 2032] {eq $_ MMXXXII} -v -line #__LINE__
assert [decimal_to_roman 2035] {eq $_ MMXXXV} -v -line #__LINE__
newline

textcolor yellow
puts "Looping from 0 to 10001 at line: "#__LINE__
textcolor

# Valid to 10000
loop i 1 10001 {		# 1..10000
	#puts "$i '[roman_to_decimal [decimal_to_roman $i]]'"
	# if {>= $i 10000} {puts "$i->[decimal_to_roman $i]"}
	assert [roman_to_decimal [decimal_to_roman $i]] {== $_ $::i} -line #__LINE__	# Quiet mode
}
assert $i {== $_ 10001} -v -line #__LINE__

puts "Tests complete ... @"#__LINE__

lib ticol_r2d -unload

unset i
########################################################################################
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #