###########################################################################
# File: 		test_reg.tcl
# Purpose:		Test cases for ticol_reg.dll
# Author: 		M Shaw
# Language:		Ticol Tcl v1.19..v1.26
# See also;		lib_reg.tcl
#
# Versions:
# 1.00 31/Aug/2016 First version
# 1.01 04/Sep/2016 Added REG_BINARY stuff and tidied output
# 1.02 06/Sep/2018 Fix exit cleanup
###########################################################################
# Notes: 
# Creates a temp registry key under HKEY_CURRENT_USER\\Software\\Ticol
# FIX @ 272 FIX printf not unescaping, puts unescapes ok
# DONE ADD REG_SZ_EXPAND
###########################################################################

if {< [info winver -vernum] 5.0} {
	return
}

puts "Line: "#__LINE__
set expression_state [option expression]
option expression off
set echostate [option echo]

textcolor cyan
lib ticol_reg -verbose
textcolor
newline

textcolor yellow
puts "Testing ticol_reg.dll"
textcolor
newline

# get existing entry #
puts "Line: "#__LINE__
set line #__LINE__
puts "Line: $line"
if {ne [reg get {HKEY_CURRENT_USER\\Control\ Panel\\international} sCountry] "United Kingdom"} {
	die "reg fail at line $line"
}

# set -dword #
puts "Line: "#__LINE__
set line #__LINE__
puts "catch error..."
textcolor yellow
catch {
	set r [reg set HKEY_CURRENT_USER\\Software\\Ticol DWORDValue 9876543210 -dword]
	if {ne $r "1"} {
		die "reg fail at line $line ($r)"
	} else {
		puts "Line $line '" -nonewline 
		textcolor yellow
		puts $r -nonewline
		textcolor
		puts "'"
	}
}
textcolor
newline

# set -qword #
puts "Line: "#__LINE__
set line #__LINE__
puts "Line $line: Catch Set QWORD as DWORD error..."
textcolor yellow
catch {
	set r [reg set HKEY_CURRENT_USER\\Software\\Ticol QWORDValue 9876543210 -dword]
	if {ne $r "1"} {
		die "reg fail at line $line ($r)"
	} else {
		printf "Line %3s\t'" $line 
		textcolor yellow
		puts [bool $r] -nonewline
		textcolor
		puts "'"
	}
}
textcolor
newline

# set -dword #
puts "Line: "#__LINE__
set line #__LINE__
puts "Line: $line"
catch {
	#set r [reg set HKEY_CURRENT_USER\\Software\\Ticol DWORDValue ffff1234 -dword]
	set r [reg set HKEY_CURRENT_USER\\Software\\Ticol DWORDValue 0xffff1234 -dword]		# NO! (can't use 0x prefix as MPP will translate to decimal)
	if {ne $r "1"} {
		die "reg fail at line $line ($r)"
	} else {
		printf "Line %3s\t'" $line 
		textcolor yellow
		puts [bool $r] -nonewline
		textcolor
		puts "'"
	}
}

# type REG_DWORD #
puts "Line: "#__LINE__
set line #__LINE__
puts "Line: $line"
set r [reg type HKEY_CURRENT_USER\\Software\\Ticol DWORDValue]
if {ne $r "REG_DWORD"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# get -dword #
puts "Line: "#__LINE__
set line #__LINE__
puts "Line: $line"
set r [reg get HKEY_CURRENT_USER\\Software\\Ticol DWORDValue -dword]
if {!= $r 0xffff1234} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}


# set string #
puts "Line: "#__LINE__
set line #__LINE__
puts "Line: $line"
set r [reg set HKEY_CURRENT_USER\\Software\\Ticol TestValue "Hello World"]
if {ne $r "1"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts [bool $r] -nonewline
	textcolor
	puts "'"
}

# type REG_SZ #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg type HKEY_CURRENT_USER\\Software\\Ticol TestValue]
if {ne $r "REG_SZ"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# get -string #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg get HKEY_CURRENT_USER\\Software\\Ticol TestValue]
if {ne $r "Hello World" -string} {		# ??
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# set -list #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg set HKEY_CURRENT_USER\\Software\\Ticol ListValue "The {quick brown} fox" -list]
if {ne $r "1"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts [bool $r] -nonewline
	textcolor
	puts "'"
}

# type REG_MULTI_SZ #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg type HKEY_CURRENT_USER\\Software\\Ticol ListValue]
if {ne $r "REG_MULTI_SZ"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# get -list #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg get HKEY_CURRENT_USER\\Software\\Ticol ListValue -list]
puts "'[tohex $r]' '[string length $r]'"
if {ne $r "The {quick brown} fox"} {
	die "reg fail at line $line '$r'"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# get -multiline #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg get HKEY_CURRENT_USER\\Software\\Ticol ListValue -multiline]
if {ne $r "The\r\nquick brown\r\nfox\r\n"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# set binary string #
puts "Line: "#__LINE__
binary set a \xfe\xffhello\x21\x00\x22world\x02\xff\xfe -length 16
puts "binary length:\t'" -nonewline
textcolor yellow
puts [binary length a] -nonewline
textcolor
puts '

# set binary reg #
puts "Line: "#__LINE__
# NB: Cannot supply binary string as arg to [reg set]
puts "reg set binary:\t'" -nonewline
textcolor yellow
puts [bool [reg set HKEY_CURRENT_USER\\Software\\Ticol BINDATA -binary a]] -nonewline
textcolor
puts '
newline

# type SZ_BINARY #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg type HKEY_CURRENT_USER\\Software\\Ticol BINDATA]
if {ne $r "REG_BINARY"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# get binary reg #
puts "Line: "#__LINE__
puts "reg get binary:\t'" -nonewline
textcolor yellow
puts [reg get HKEY_CURRENT_USER\\Software\\Ticol BINDATA -binary b] -nonewline
textcolor
puts '
puts "reg get binary:\t'" -nonewline 
textcolor yellow
puts [binary tohex b] -nonewline
textcolor
puts '
newline

# EXPAND #################################

# set expand reg #
puts "Line: "#__LINE__
# NB: Cannot supply binary string as arg to [reg set]
puts "reg set expand:\t'" -nonewline
textcolor yellow
puts [bool [reg set HKEY_CURRENT_USER\\Software\\Ticol Path [unescape $env(path)] -expand]] -nonewline
textcolor
puts '

# type REG_EXPAND_SZ #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg type HKEY_CURRENT_USER\\Software\\Ticol Path]
if {ne $r "REG_EXPAND_SZ"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}

# get -expand #
# get -string #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg get HKEY_CURRENT_USER\\Software\\Ticol Path]
if {eq $r ""} {		# ??
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	printf "%-50.50s..." [unescape $r] 		# FIX printf not unescaping, puts unescapes ok
	textcolor
	puts "'"
}

/*
puts "reg get expand:\t'" -nonewline
textcolor yellow
set r [reg get HKEY_CURRENT_USER\\Software\\Ticol Path] -nonewline
textcolor
puts '
puts "reg get expand:\t'" -nonewline 
textcolor yellow
puts '$r' -nonewline
textcolor
puts '
newline */

/* # EXPAND
set a $env(path)
puts [reg set HKEY_CURRENT_USER\\Software\\Ticol Path [unescape $a] -expand] -nonewline
puts "expand:\t[reg get HKEY_CURRENT_USER\\Software\\Ticol Path -expand]"
# exit
*/

/*
# delete item #
puts "Line: "#__LINE__
set line #__LINE__
set r [reg del HKEY_CURRENT_USER\\Software\\Ticol TestValue]
if {ne $r "1"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts [bool $r] -nonewline
	textcolor
	puts "'"
} #
*/

/*
# Tested with change to RegDeleteKeyEx replaced by RegDeleteKey - PASS
# delete subkey #
set line #__LINE__
set r [reg del HKEY_CURRENT_USER\\Software\\Ticol]
if {ne $r "1"} {
	die "reg fail at line $line ($r)"
} else {
	printf "Line %3s\t'" $line 
	textcolor yellow
	puts $r -nonewline
	textcolor
	puts "'"
}
*/

puts "Line: "#__LINE__
lib ticol_reg -unload

option echo $echostate
option expression $expression_state

unset a b echostate expression_state line r -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "\[reg\] (Registry tests) done"
textcolor

# EOF #