#################################################################################
# File:				test_lib_rlm.tcl
# Author:			M Shaw
# Purpose:			Test cases for lib ticol_rlm.dll
# Prerequisites:	Valid or pseudo-valid RLM licence files
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 26/Dec/2017	First version
# 1.01 20/Apr/2018	Updated error return handling
# 1.02 01/Jul/2018	Plugin development halted. No longer have access to RLM 
#                   File 'cleaned' of any proporietary information
# 1.03 30/Nov/2019	Added asserts
#################################################################################
# Notes:
# Retained as an example TCL script but only dummy licences now available
# Pseudo-licences may be generated using RLMTOOL.EXE and RLMSIGN.EXE
# where RLMSIGN.EXE is a fake util which generates syntactically-valid LIC files
# See http://tools.kerys.co.uk tools download page
# Checks an RLM licence file by accessing via the local or remote file system
# integer [rlm check filename module ?-months|-days? ?-min|-avg? ?-err var?]
# Defaults: -days and -max
# Could also add [rlm_create] by extracting from rlmtool.exe
#################################################################################
# DONE FIX can't use try/catch to set a return value
# DONE FIX tests 2 and 3
# DONE FIX must return error code if module does not exist
# DONE FIX must return error code if filename does not exist
#################################################################################

# User configuration
#set path "C:/VC5/MyProjects/ticol_rlm/Release/"				# Must end with /
set path "."

textcolor yellow
puts "Testing \[test_rlm.tcl\]"
puts "Test path is $path"
textcolor
newline

option echo on
option expression off

# FIX ? ticol_rlm: Module (null) already loaded
textcolor cyan
lib ticol_rlm -verbose -nocomplain
textcolor
newline

array set errcode {
	0 "Success" 				# RLMC_ERR_SUCCESS,				// 0		SUCCESS
	1 "Perpetual"				# RMLC_ERR_SUCCESS_PERP,		// 1		EXCEPTION
	2 "Bad path" 				# RLMC_ERR_BAD_PATH,			// 2		EXCEPTION
	3 "File not found"			# RLMC_ERR_FNF,					// 3		EXCEPTION
	4 "Failed"					# RLMC_ERR_CHECK_FAILED,		// 4		EXCEPTION
	5 "No such module"			# RLMC_ERR_NO_MODULE,			// 5		EXCEPTION
	# RLMC_ERR_UNKNOWN=255		// 255	EXCEPTIO
} -const

set r ""
# Test 1
puts "Line :"#__LINE__
try {
	set r [rlm check ${path}Licence-1.lic ab100 -err e]
	textcolor green
	puts "PASS r is '$r' day(s) at line "#__LINE__
	textcolor
} catch {
	textcolor red
	puts "Catch error code $r at line ([array item errcode $e "Unknown error"]) Line: "#__LINE__
	textcolor
}
assert $r {eq $_ {}} -v -line #__LINE__

# Test 2
puts "Line :"#__LINE__
puts "Test fail on non-existent module ab001"
set r 0
try {
	set r [rlm check ${path}Licence-1.lic ab000 -err e]
	textcolor green
	puts "PASS r is '$r' day(s) at line "#__LINE__
	textcolor
} catch {
	textcolor red
	puts "Catch error code $r at line ([array item errcode $e "Unknown error"]) Line: "#__LINE__
	textcolor
}
assert $r {== $_ 0} -v -line #__LINE__

# Test 3
puts "Line :"#__LINE__
puts "Test fail on non-existent licence and module ab001"
set r 0
try {
	set r [rlm check ${path}nosuch.lic ab001 -err e]
	textcolor green
	puts "PASS r is '$r' day(s) at line "#__LINE__
	textcolor
} catch {
	textcolor red
	puts "Catch error code $e at line ([array item errcode $e "Unknown error"]) Line: "#__LINE__
	textcolor
}
assert $r {== $_ 0} -v -line #__LINE__

# Test 4
puts "Line :"#__LINE__
puts "Test fail on existent licence and non-existent module bc001"
set r 0
try {
	set r [rlm check ${path}Licence-1.lic bc001 -err e]
	textcolor green
	puts "PASS r is '$r' day(s) at line "#__LINE__
	textcolor
} catch {
	textcolor red
	puts "Catch error code $e at line ([array item errcode $e "Unknown error"]) Line: "#__LINE__
	textcolor
}
assert $r {== $_ 0} -v -line #__LINE__

# Test 5
puts "Line :"#__LINE__
puts "Test for success on existent licence and existent module pq001"
set r 0
try {
	puts "Checking ${path}Licence-1.lic for pq101"
	set r [rlm check ${path}Licence-1.lic pq101 -err e]
	textcolor green
	puts "PASS r is '$r' day(s) at line "#__LINE__
	assert $r {< $_ 0} -v -line #__LINE__
	textcolor
} catch {
	textcolor red
	puts "Catch error code $e at line ([array item errcode $e "Unknown error"]) Line: "#__LINE__
	textcolor
}
newline

lib ticol_rlm -unload
unset e path r -nocomplain
array unset errcode -const -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #