######################################################################################
# File: 			test_sci.tcl
# Purpose:			Test ticol_sci.dll scientific format numbers
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 09/Aug/2018	First version
# 1.01 22/Jul/2019	For mods to exponent for 2 digit from Windows 3 digit
# 1.02 20/Mar/2022	Change 'E' to 'e' in lib
# 1.03 07/Jun/2022	Revised for changes to [loop] bounds
######################################################################################
# Notes: 
# Output format will format with no trailing zeroes in mantissa and 2 digit exponent
# and with capital letter E output for [sci]
# NNNNN.E+XX
######################################################################################
# TODO
# Add more tests
######################################################################################

option expression off

# FIX Number detection in relational operators ###############################

if {[!= -0.0002 -.0002]} {
	puts "FAIL @"#__LINE__
}

# printf "test: %f" -.002	# FIX no output

printf "test:-.002 is %f\r\n" -.002

assert 0.0002  {== $_ .0002} -v -line #__LINE__ # FAIL
assert -0.0002  {== $_ -.0002} -v -line #__LINE__ # FAIL
assert -0.00200 {== $_ -.00200} -v -line #__LINE__ # FAIL

###############################################################################

textcolor cyan
lib ticol_sci -verbose -nocomplain
textcolor
newline

undef check -nocomplain

if {! [defined check]} {
	proc check {n} {
		catch {
			dec $n 
		} s
		textcolor yellow
		puts "$s  len:[string length $s]"
		textcolor
	}
}

#############################################################################

textcolor magenta
puts [sci [dec 2e23]]
textcolor
newline

check 2e4
assert [dec 2e4] {== $_ 20000} -v -line #__LINE__
check 2e+4
assert [dec 2e+4] {== $_ 20000} -v -line #__LINE__
check 2e-4
assert [dec 2e-4] {== $_ .0002} -v -line #__LINE__
check -2e4
assert [dec -2e4] {== $_ -20000} -v -line #__LINE__
check -2e-4
assert [dec -2e-4] {== $_ -0.0002} -v -line #__LINE__

assert [dec -2e-4] {== $_ -.0002} -v -line #__LINE__ # FAIL .0002

#############################################################################
newline
check 1e4
#puts 2e10
#puts 2e20
check 2e30	# OK
check 2e31
check 2e32	# OK
catch {
	check 2e33
}

check 2e34

check 2000000000e32
#############################################################################

# Test round-trip
assert [sci 2] {== $_ "2.0e+0"} -v -line #__LINE__
assert [dec [sci 2]] {== $_ 2} -v -line #__LINE__

assert [sci 0.2] {== $_ "2.0e-1"} -v -line #__LINE__
assert [dec [sci 0.2]] {== $_ 0.2} -v -line #__LINE__

assert [sci -0.2] {== $_ "-2.0e-1"} -v -line #__LINE__
assert [dec [sci -0.2]] {== $_ -0.2} -v -line #__LINE__

assert [sci -.002] {eq $_ "-2.0e-3"} -v -line #__LINE__
assert [dec [sci -.002]] {== $_ -0.002} -v -line #__LINE__

# End

assert [dec -2.000000e-03] {== $_ -.002} -v -line #__LINE__
assert [dec 2E24] {== $_ 2000000000000000000000000.000000} -v -line #__LINE__

assert [sci -.002] {eq $_ "-2.0e-3"} -v -line #__LINE__

# Checking a 'real world' value...
assert [sci 5051090000000] {eqi $_ "5.05109e+12"} -v -line #__LINE__
assert [dec 5.05109E+12] {!= $_ 5051090000001} -v -line #__LINE__
assert [dec 5.05109E+12] {!= $_ 5051100000000} -v -line #__LINE__
assert [dec 5.05109E+12] {== $_ 5051090000000} -v -line #__LINE__
assert [sci [dec 5.05109E+12]] {eq $_ 5.05109e+12} -v -line #__LINE__
assert [sci [dec 5.05109E+12]] {== $_ 5.05109e+12} -v -line #__LINE__
#assert 5.05109E+12 {== $_ 5051090000000} -v -line #__LINE__
assert "5.05109e+12" {== $_ 5.05109e+12} -v -line #__LINE__
assert "5.05109e+12" {!= $_ 5.05109} -v -line #__LINE__
assert [expr [dec 2E7]*3] {== $_ 60000000} -v -line #__LINE__

######################################################################################
# Some loop series
/*
0: 1.000000000000000 1.0E+0
1: 0.100000000000000 1.0E-1
2: 0.010000000000000 1.0E-2
3: 0.001000000000000 1.0E-3
4: 0.000100000000000 1.0E-4
5: 0.000010000000000 1.0E-5
6: 0.000001000000000 1.0E-6
7: 0.000000100000000 1.0E-7
8: 0.000000010000000 1.0E-8
9: 0.000000001000000 1.0E-9
10: 0.000000000100000 1.0E-10
11: 0.000000000010000 1E-11
12: 0.000000000001000 1.0E-12
13: 0.000000000000100 1.0E-13
14: 0.000000000000010 1.0E-14
15: 0.000000000000001 1.0E-15 */
# Loop test -ve exponent signs
# zero base gives +ve sign so can't test 0 for -ve
# i = 0.0E+0
# Note that loop i for base cannot exceed 9 since we always represent as X.Y<sign>ENN
loop i 1 9 2 {		## Loop 1 to 9
	loop j 1 15 {		# Iters to hi-1
		#puts "TEST A: $i,$j: dec:'[dec ${i}E-$j]' scidec:'[sci [dec ${i}E-$j]]' string:'${i}.0E-$::j'"
		assert [sci [dec ${i}e-$::j]] {eq $_ "${i}.0e-$::j"} -line #__LINE__
	}
}

#/*
# Loop test +ve exponent signs
loop i 0 10 2 {		## Loop 1 to 9
	loop j 0 16 {		# Iters to hi-1
		#puts "TEST B: $i,$j: dec:'[dec ${i}e+$j]' scidec:'[sci [dec ${i}e+$j]]' string:'${i}.0E+$::j'"
		assert [sci [dec ${i}e+$i]] {eq $_ "${i}.0e+$::i"} -line #__LINE__
	}
} # */

assert [sci [dec 0.0e0]] {eqi $_ 0.0e+0} -v -line #__LINE__
assert [sci [dec 1.0e0]] {eqi $_ 1.0e+0} -v -line #__LINE__
assert [sci [dec 1.0e-0]] {eqi $_ 1.0e+0} -v -line #__LINE__		# Sign remains +ve
assert [sci [dec 1.0e-1]] {eqi $_ 1.0e-1} -v -line #__LINE__
assert [sci [dec 1.0e+1]] {eqi $_ 1.0e+1} -v -line #__LINE__
assert [sci [dec 1.1e+1]] {eqi $_ 1.1e+1} -v -line #__LINE__
assert [sci [dec 1.1e-1]] {eqi $_ 1.1e-1} -v -line #__LINE__

# Input values will be shifted and normalised to single-digit values ...
puts "[dec 12.1e+1] [sci [dec 12.1e+1]]"
assert [sci [dec 12.1e+1]] {eqi $_ 1.21e+2} -v -line #__LINE__
puts "[dec 12.1e-1] [sci [dec 12.1e-1]]"
assert [sci [dec 12.1e-1]] {eqi $_ 1.21e+0} -v -line #__LINE__	
puts "[dec 3887.1e+1] [sci [dec 3887.1e+1]]"	# 38871
assert [sci [dec 3887.1e+1]] {eqi $_ 3.8871e+4} -v -line #__LINE__
puts "[dec 3887.1e-1] [sci [dec 3887.1e-1]]"	# 38871
assert [sci [dec 3887.1e-1]] {eqi $_ 3.8871e+2} -v -line #__LINE__
puts "[dec 8283887.1e+5] [sci [dec 8283887.1e-5]]"	# 38871
assert [sci [dec 8283887.1e+5]] {eqi $_ 8.2838871E+11} -v -line #__LINE__
# 8283887.10000. 11

/*
# Signs will change as values shift due to internal normalisation, so difficult to test
# Loop test +ve exponent signs
loop i 1 11 2 {		## Loop 1 to 10 (base)
	loop j 1 16 {		# Iters to hi-1 (exponent)
		loop k 0 499 3 {		# Decimal point
		puts "TEST C: $i,$j,$k: dec:'[dec ${i}.${k}e+$j]' scidec:'[sci [dec ${i}.${k}e+$j]]' string:'${i}.${k}E+$j'"
		assert [sci [dec ${i}.${k}e+$j]] {eq $_ "${i}.${k}E-$j"} -v -line #__LINE__
		}
	}
} # */

unset i j
undef check

######################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #
