#################################################################################
# File:				test_service.tcl
# Author:			M Shaw
# Purpose:			Test cases for ticol_service.dll
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 29/May/2017	First version
#################################################################################
# Notes:
# Must be run elevated to gain access to the SCM
# LanmanWorkstation -> workstation
#################################################################################
/*
To load, use:	lib ticol_svc ?-verbose? ?-nocomplain?
	service <registered-service-name> ?-verbose?

	The following codes or verbose status messages are returned:

 Code Verbose                  Comments
 ---------------------------------------------------------------------------
 0    Running                  The service is installed and running normally
 1    Stopped                  The service is installed + in a stopped state
 2    Stopping                 The service is installed and running
 3    Not found                The requested service was not found
 4    No error                 Reserved
 5    Can't load advapi32.dll  Cannot load the required system DLL
 6    Can't locate export      Wrong version of advapi32.dll 
      QueryServiceStatusEx     Can't locate function in the DLL
 7    OpenSCManager failed     Could not open the SCM manager
 8    Unknown service status   An unknown service status was returned 
 9    Unhandled error          Unhandled error
*/
#################################################################################
option expression off

textcolor yellow
puts "Testing \[service\] ... (Requires and will prompt for elevation)"
textcolor
newline

if {! [is_elevated]} {
	elevate
	exit
}

proc print_line {l} {
	upvar line
	set line $l
	textcolor yellow
	puts "Line: $l"
	textcolor
}
########################################################################################

textcolor cyan
lib ticol_svc -verbose -nocomplain
textcolor
newline
halt

puts "Line: "#__LINE__
puts [service LanmanWorkstation -verbose]
assert [service LanmanWorkstation] {== $_ 0} -v -line #__LINE__
assert [service FooManchu] {== $_ 3} -v -line #__LINE__


########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

if {is_elevated} {
	pause
}

# EOF #