###############################################################################################
# File:				test_sockping.tcl
# Author:			M Shaw
# Purpose:			Test file for lib ticol_sockping [sockping]
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 20/Dec/2017	First version
# 1.01 12/Sep/2018	Chasing out bugs in lib
#
###############################################################################################
# Notes
# FIX Won't work on Windows NT or ReactOS
###############################################################################################
/*
enum {

	CONNECT_SUCCESS=0,		// 0
	CONNECT_FAILED,			// 1
	SYNTAX_ERROR,			// 2
	TOO_MANY_ARGS,			// 3
	SOCKET_FAILED,			// 4
	WSA_STARTUP_ERROR,		// 5
	GETADDRINFO_ERROR,		// 6
	DISCONNECT_ERROR,		// 7
};
###############################################################################################
*/

array unset sock_error -const -nocomplain
array set sock_error {
	0 "Success"
	1 "Connect failed"
	2 "Syntax error"
	3 "Too many arguments"
	4 "Socket failed"
	5 "WSA Startup error"
	6 "Failed to resolve host:port"
	7 "Disconnect error"
	8 "Error error"
} -const

option expression off
option echo on

textcolor yellow
puts "Testing \[sockping\] in ticol_sockping.dll .... (requires a live TCP/IP stack)"
textcolor
newline

# Check for ReactOS #
puts "Line: "#__LINE__
puts "WINDIR == '$env(windir)'"
if {eqi $env(windir) "c:\\reactos"} {
	puts "Can't test on ReactOS"
	#return -code exit		# Exits the parent
	# return -code ok
	return -code return
}

/*
puts "Line: "#__LINE__
if {< [info winver -vernum] 5.0} {
	puts "Can't run test on Windows < 2000"
	#return -code exit		# Exits the parent
	# return -code ok
	return -code return
} #*/

puts "Line: "#__LINE__
textcolor cyan
lib ticol_sockping -verbose -nocomplain
textcolor
puts "Line: "#__LINE__
newline

/*
puts "Line: "#__LINE__
puts "sockping localhost on port 25"
set r [sockping localhost 25 -var v]
puts "Returned code $r"
if {is_set v} {
	assert $v {ne $_ ""} -v -line #__LINE__
} else {
	die "v not set. Should be set to {}"
}
newline

puts "Line: "#__LINE__
set r [sockping localhost 110 -var v]
puts "Returned code $r"
if {is_set v} {
	assert $v {ne $_ ""} -v -line #__LINE__
} else {
	die "v not set. Should be set to {}"
}
newline
############################################################################  */

/*
puts "Line: "#__LINE__
set r [sockping localhost 8800 -var v]
puts "Returned code $r"
if {is_set v} {
	assert $r {eq $_ 0} -v -line #__LINE__
} else {
	die "Return code was not 0 (ERROR_SUCCESS)"
}
newline
############################################################################  */


/*
option echo on
puts "Line: "#__LINE__
puts "Pinging '[info hostname]'"
unset v -nocomplain
puts "Line: "#__LINE__
sockping lenovo 8800
#sockping [info hostname] 139
# Problem is this line ...
puts "Line: "#__LINE__
set r ""
set r [sockping [info hostname] 139]
textcolor yellow
puts "r was '$r'"
textcolor
# */

################################################################
# Ping the SMB port. This test might be unreliable as not everyone may 
# have port 139 active so this test has been disabled
/*
set r [sockping [info hostname] 139 -var v]		# UNC share @port 139 (SMB)

puts "Line: "#__LINE__
puts "\[sockping [info hostname] 139 -var v\] Returned code $r\n"

puts "Line: "#__LINE__
if {is_set v} {
	puts "sockping response: '$v' (Code: [array item sock_error $r])"
}
# return -code ok

puts "Line: "#__LINE__
walk 10
puts "Line: "#__LINE__
puts "Test will fail if the TCP/IP stack is absent or not configured!"

assert $r {== $_ 0} -v -line #__LINE__			# ERROR_SUCCESS == 0
if {is_set v} {
	puts "Line: "#__LINE__
	assert $v {eq $_ ""} -v -line #__LINE__
} else {
	puts "Line: "#__LINE__
	die "v not set. Should be set to {}"
}
############################################################################  */


newline
option expression off
option echo on
option stacktrace on
puts "Line: "#__LINE__
set r [sockping localhost 666 -var v]			# Should not respond (fail with code 6)
puts "Sockping localhost:666 returned code $r ([array item sock_error $r {Unknown error}]) "
puts "Line: "#__LINE__
# FIX assert failure not visible
assert $r {!= $_ 0} -v -line #__LINE__			# Some sort of error should occur
puts "Line: "#__LINE__
if {is_set v} {
	puts "Response: '$v'"
} else {
	die "v not set. Should be set to {}"
}
newline
puts "Line: "#__LINE__
# */
############################################################################  */

puts "Unloading ticol_sockping ... @"#__LINE__
puts "Line: "#__LINE__
textcolor cyan
lib ticol_sockping -unload -verbose
textcolor

unset r v -nocomplain
array unset sock_error -const
vars

############################################################################
newline
textcolor green
puts "* Done test_sockping.tcl *"
textcolor

# EOF #