#################################################################################
# File:				test_variant.tcl
# Author:			M Shaw
# Purpose:			Test cases for ticol_variant.dll (varray)
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 01/Aug/2018	First version
# 1.01 09/Dec/2019	Added more tests
#################################################################################
# Notes:
# varray vsplit dumpvariant
# DONE ADD [varray list] return a list
# See also: test_goto_exit_loops.tcl
#################################################################################
/*
handle  [varray create size ?lbound?]
bool    [varray set handle element value]
integer [varray size addr ?dimension?]
string  [varray get handle index ?index?]
integer [varray lbound handle]
integer [varray ubound handle]
integer [varray list handle]
varray foreach handle var {script}
integer [varray unset handle]
*/
#################################################################################

option expression off
option echo on

textcolor yellow
puts "Testing lib ticol_variant.dll ..."
textcolor

textcolor cyan
lib ticol_variant -nocomplain -verbose
textcolor

#################################################################################
# [varray]
#################################################################################
puts "Line: "#__LINE__
set h [varray create 200 0]				# handle  [varray create size ?lbound?]

assert $h {!= $_ 0} -v -line #__LINE__
assert [varray size $h] {== $_ 200} -v -line #__LINE__
assert [varray unset $h] {== $_ $::true} -v -line #__LINE__

#################################################################################
# [vsplit]
#################################################################################
puts "Line: "#__LINE__
set v [vsplit $qbf]
set i 0
assert [varray size $v] {== $_ 9} -v -line #__LINE__
assert [varray get $v 1] {eq $_ quick} -v -line #__LINE__
#string  [varray get handle index ?index?]

#################################################################################
# [varray foreach]
unset x -nocomplain
puts "Line :"#__LINE__
textcolor darkyellow

varray foreach $v x {
	puts "\t${i} : $x"
	++ i
}
# Check the array element count and that foreach iterated all elements
assert $i {== $_ 9} -v -line #__LINE__
puts "Size [varray size $v]"
assert $i {== $_ [varray size $v]} -v -line #__LINE__
textcolor
newline

#################################################################################
# [varray list]
puts "Line :"#__LINE__
set l [varray list $v]
puts "varray list: '$l'"
assert $l {eq $_ $::qbf} -v -line #__LINE__
assert [lcount $l] {== $_ 9} -v -line #__LINE__

assert [varray unset v] {== $_ $::true} -v -line #__LINE__
puts "Case after \[varray unset\] v is $v"


#################################################################################
set h [varray create 20]
set i 0
unset x -nocomplain
foreach x $qbf {								# Populate the object
	varray set $h $i $x							# Index $i -> value
	++ i
}
assert [varray size $h] {== $_ 20} -v -line #__LINE__
assert $i {== $_ 9} -v -line #__LINE__			# Check element iter count
unset x -nocomplain
set i 0

varray foreach $h x {							# Iter all declared elements (20)
	++ i
}
assert $i {== $_ 20} -v -line #__LINE__			# Check element iter count

set i 0
unset x -nocomplain
varray foreach $h x {							# Varray iters whole declared array not occupied elements
	++ i
}
assert $i {== $_ 20} -v -line #__LINE__



#################################################################################
puts "Line :"#__LINE__
textcolor cyan
lib ticol_variant -unload -verbose
textcolor

unset h i l v x -nocomplain

vars

########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #