###################################################################################################
# File:				test_lib_win.tcl
# Author:			M Shaw
# Purpose:			Unit test for ticol_win.dll plugin
# Language:			Ticol Tcl v1.27+
#
# Versions:
# 1.00 30/May/2022	First version
# 1.01 08/Jun/2022	Removed handle requirements to [win]
# 1.02 28/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
###################################################################################################
# Notes:
# Due to the nature of the plugin we can't test much other than it doesn't actually halt/crash
# A window will open briefly then close and which contains the output
# Does NOT test for memory or resource leaks
###################################################################################################
# FIX Window doesn't actually look to be the requested size
# FIX gotoxy sets a base 0 position
###################################################################################################
/*	Syntax: 
	----------------------
	[win active]																		# Return window active status (e.g. mouse exit)
	[win box x y w h ?fore? ?back? ?-single? ?-solid? ?-nofill?]						# Draw a character mode box with fill option
	[win circle handle x y ?colourname|"#RRGGBB"? ?width?]								# Draw a filled circle
	[win close]																			# Close the window and release any resources
	[win cls ?colourname|"#RRGGBB"]														# Clear the screen
	[win columns]																		# Return the number of columns
	[win create -w width -h height ?-fullscreen? ?-smallfont? ?-largecfont? ?-movequit? ?-closeonlostfocus?]
	[win cursor show|hide]																# Default is show
	[win ellipse handle x y w h ?color? ?width? ?-dot|-dash|-dotdash|-solid? ?-fill ?colour??]	
	[win font -name FONTNAME -size NN -bold -italic -underline -normal -clear]	
	[win gotoxy X Y ?-t?]																# Locate the cursor at character X/Y coordinate
	[win height]																		# Return the window height in pixels
	[win line]																			# Draw a line
	[win lineto x y ?colour? ?width? ?-dot|-dash]-dotdash?] # Plot from the stored [moveto] coords to the given x/y point
	[win moveto x y]																	# Move the line plotting start point to x/y coords
	[win paint]																			# Returns true on WM_PAINT (should paint)
	[win para]																			# Draw a parallellogram
	[win preview HANDLE]																# Set up for Windows Screensaver preview
	[win puts string ?-font H? ?-rot ANGLE?]											# -rot requires setpos for placement
	[win rect x y w h ?color? ?width? ?-dot|-dash|-dotdash|-solid? ?-fill ?colourname|"#RRGGBB"??]	# Draw a rectangle (non-filled)
	[win rndcolor ?lo|-? ?hi|-?]														# Return a random HTML colour as "#RRGGBB" hex
	[win rows]																			# Return the number of screen rows
	[win setpos]																		# Set absolute cursor position (pixels) - Use with [win xpos/ypos]
	[win snooze ?ms?]																	# Blocking sleep	
	[win textcolor fore|- ?back?]														# Set foreground/background using Ticol constants or "#RRGGBB" values
	[win wherex]																		# Cursor X location in columns
	[win wherey]																		# Cursor Y location in rows
	[win width]																			# Return the window width in pixels
	[win xpos]																			# Return absolute pixel position for character coordinate
	[win ypos]																			# Ditto
	[win randcolour]
	-------------
	Total: 29
	[ini read]
	[ini write]
	[bellcode]
################################################################################################### */

lib ticol_win -unload -nocomplain
lib ticol_win


set h [win create -w 640 -h 480 -x [rnd 0 600] -y [rnd 0 400]]
puts "Window handle is: '$h'"

console minimize

assert {!= $_ 0} -v -line #__LINE__
assert [win active] {!= $_ $false} -v -line #__LINE__

assert [win columns] {!= $_ 0} -v -line #__LINE__
assert [win rows] {!= $_ 0} -v -line #__LINE__

win puts [unescape "Testing \[ticol win\] ..."]

win gotoxy 0 0
win gotoxy -10 -10

win gotoxy 10 11
# FIX base 0 returns ?
assert [win wherex] {== $_ 9} -v -line #__LINE__
assert [win wherey] {== $_ 10} -v -line #__LINE__

assert [win xpos 10] {> $_ 0} -v -line #__LINE__
assert [win ypos 10] {> $_ 0} -v -line #__LINE__

assert [win width] {== $_ 640} -v -line #__LINE__
assert [win height] {== $_ 480} -v -line #__LINE__

# Drawing primitives...
win line 0 0 640 480 white 1
win line 0 480 640 0  white 1
win box 10 10 10 10 
sleep 500
win cls

#/*
loop i 1 1001 {
	win textcolor yellow blue
	set x [rnd 1 20] 
	set y [rnd 1 20]
	win gotoxy $x $y
	win puts "Hello again from ticol"
	win textcolor
	win gotoxy $x $y
	win puts "                      "
	win para [rnd 1 600] [rnd 1 400] [rnd 1 200] [rnd 1 200] [rnd 1 360] [rnd 1 360] [rnd 1 15] [rnd 1 3]
	win rect [rnd 1 600] [rnd 1 400] [rnd 1 200] [rnd 1 200] [rnd 1 15] [rnd 1 3]
	win circle [rnd 100 500] [rnd 100 300] [rnd 2 14] [rnd 2 35]
} # */


sleep 500
win cls
win textcolor magenta white
set x [/ [win width] 2]
set y [/ [win height] 2]
set oldx 0
set last_angle 0
win setpos [/ [win width] 2]  [/ [win height] 2]
# This triggered the need to modify (correct) [loop]

# /*
set fontsize [lrand {6 8 10 12 16 18}]
loop i 370 -1 -10 {
#puts $i
	win textcolor black black
	win setpos $x $y
	win puts [string repeat " " [string length $qbf]] -rot [* $last_angle 10] -font $fontsize
	win textcolor magenta white
	win setpos $x $y
	win puts $qbf -rot [* $i 10] -font  $fontsize
	set last_angle $i
	if {! [win active]} break
	sleep 20
}
# */

win snooze 2000

/*
loop 100 {
	win cls "#[rnd 0 0xFFFFFF]"
	win snooze 50
}
doevents
# */

win cls "#FF00FF"
set fontsize [lrand {6 16}]
loop i 0 361 10 {
	# set fontsize [lrand {6 8 10 12 16 18}]

	win textcolor "#FF00FF" "#FF00FF"
	win setpos $x $y
	win puts [string repeat " " [string length $qbf]] -rot [* $last_angle 10] -font $fontsize

	win textcolor white "#FF00FF"
	win setpos $x $y
	win puts $qbf -rot [* $i 10] -font $fontsize

	set last_angle $i
	# win snooze 10
	sleep 20
	if {! [win active]} break
}

puts "Trying to open a 2nd window (should raise an error)"
set i 0
try {
	set h2 [win create]
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

set i 0
while {win active} {
	if {> $i 10} break
	win snooze 100
	++ i
}
puts "Closing @"#__LINE__
win close
lib ticol_win -unload

console maximize		# wait until any multiple test instances have exited

###################################################################################################
newline

textcolor yellow
puts "* Done *"
textcolor

# EOF #