###################################################################################################
# File:				test_lib_win.tcl
# Author:			M Shaw
# Purpose:			Unit test for ticol_win.dll plugin
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 30/May/2022	First version
###################################################################################################
# Notes:
# Due to the nature of the plugin we can't test much other than it doesn't actually halt/crash
# A window will open briefly then close and which contains the output
# Does NOT test for memory or resource leaks
###################################################################################################
# FIX Window doesn't actually look to be the requested size
# FIX gotoxy sets a base 0 position
###################################################################################################
/*	Syntax: 
	----------------------
	[win active]																		# Return window active status (e.g. mouse exit)
	[[win box x y w h ?fore? ?back? ?-single? ?-solid? ?-nofill?]						# Draw a character mode box with fill option
	[win circle handle x y ?colourname|"#RRGGBB"? ?width?]								# Draw a filled circle
	[win close]																			# Close the window and release any resources
	[win cls ?colourname|"#RRGGBB"]														# Clear the screen
	[win columns]																		# Return the number of columns
	[win create -w width -h height ?-fullscreen? ?-smallfont? ?-largecfont? ?-movequit? ?-closeonlostfocus?]
	[win gotoxy X Y ?-t?]																# Locate the cursor at character X/Y coordinate
	[win height]																		# Return the window height in pixels
	[win line]																			# Draw a line
	[win para]																			# Draw a parallellogram
	[win preview HANDLE]																# Set up for Windows Screensaver preview
	[win puts string ?-font H? ?-rot ANGLE?]											# -rot requires setpos for placement
	[win rect x y w h ?color? ?width? ?-dot|-dash|-dotdash|-solid? ?-fill ?colourname|"#RRGGBB"??]	# Draw a rectangle (non-filled)
	[win rows]																			# Return the number of screen rows
	[win setpos]																		# Set absolute cursor position (pixels) - Use with [win xpos/ypos]
	[[win textcolor fore|- ?back?]														# Set foreground/background using Ticol constants or "#RRGGBB" values
	[win wherex]																		# Cursor X location in columns
	[win wherey]																		# Cursor Y location in rows
	[win width]																			# Return the window width in pixels
	[win xpos]																			# Return absolute pixel position for character coordinate
	[win ypos]																			# Ditto
################################################################################################### */

lib ticol_win -unload -nocomplain
lib ticol_win

set h [win create -w 640 -h 480]
puts "Window handle is: '$h'"
assert $h {!= $_ 0} -v -line #__LINE__
assert [win active $h] {!= $_ $false} -v -line #__LINE__

assert [win columns $h] {!= $_ 0} -v -line #__LINE__
assert [win rows $h] {!= $_ 0} -v -line #__LINE__

win puts $h [unescape "Testing \[ticol win\] ..."]

win gotoxy 0 0
win gotoxy -10 -10

win gotoxy $h 10 11
# FIX base 0 returns ?
assert [win wherex $h] {== $_ 9} -v -line #__LINE__
assert [win wherey $h] {== $_ 10} -v -line #__LINE__

assert [win xpos $h 10] {> $_ 0} -v -line #__LINE__
assert [win ypos $h 10] {> $_ 0} -v -line #__LINE__

assert [win width $h] {== $_ 640} -v -line #__LINE__
assert [win height $h] {== $_ 480} -v -line #__LINE__

# Drawing primitives...
win line $h 0 0 640 480 white 1
win line $h 0 480 640 0  white 1
win box $h 10 10 10 10 
sleep 500
win cls $h


loop i 1 1000 {
	win textcolor $h yellow blue
	set x [rnd 1 20] 
	set y [rnd 1 20]
	win gotoxy $h $x $y
	win puts $h "Hello again from ticol"
	win textcolor $h
	win gotoxy $h $x $y
	win puts $h "                      "
	win para $h [rnd 1 600] [rnd 1 400] [rnd 1 200] [rnd 1 200] [rnd 1 360] [rnd 1 360] [rnd 1 15] [rnd 1 3]
	win rect $h [rnd 1 600] [rnd 1 400] [rnd 1 200] [rnd 1 200] [rnd 1 15] [rnd 1 3]
	win circle $h [rnd 100 500] [rnd 100 300] [rnd 2 14] [rnd 2 35]
}


sleep 500
win cls $h
win textcolor $h magenta white

set oldx 0
set last_angle 0
win setpos $h [/ [win width $h] 2]  [/ [win height $h] 2]
# This triggered the need to modify (correct) [loop]
loop i 370 0 -10 {
#puts $i
	win textcolor $h black black
	win setpos $h [/ [win width $h] 2]  [/ [win height $h] 2]
	win puts $h "                    " -rot [* $last_angle 10] -font 18
	win textcolor $h magenta white
	win setpos $h [/ [win width $h] 2]  [/ [win height $h] 2]
	win puts $h "Hello from ticol_win" -rot [* $i 10] -font 18
	set last_angle $i
	sleep 20
}
# sleep 1000


set i 0
while {win active $h} {
	if {> $i 20} break
	sleep 100
	++ i
}

puts "Closing @"#__LINE__
win close $h
lib ticol_win -unload

###################################################################################################
newline

textcolor yellow
puts "* Done *"
textcolor

# EOF #