#################################################################################
# File:				test_xbase.tcl
# Author:			M Shaw
# Purpose:			Test cases for xbase
# Language:			Ticol Tcl v1.23+ (proposed)
#
# Versions:
# 1.00 21/May/2017	First version
#################################################################################
# Notes:
# XBASE Has severe limits on field sizes,, use DB instead
#################################################################################

textcolor yellow
puts "Testing \[test_xbase.tcl\]"
textcolor
newline

textcolor cyan
lib ticol_xbase -verbose -nocomplain
textcolor
newline 

option expression off

array set names {									# USER.DBF
	# ID,"Full Name",Code,Age
	1 {IK304 "Immanuel Kant" 938 80}
	2 {TS022 "Thomas Sowell" 022 66}
	3 {JSM01 "John Stuart Mill" 044 99}
	4 {LVM01 "Ludwig Von Mises" 001 92}
	5 {TDL01 "Thomas DeLorenzo" 008 22}
	6 {MR001 "Murray Rothbard" 003 98}
	7 {FH001 "Frederich Hayek" 004 102}
	8 {JBS01 "Jean Baptiste Say" 012 102}
	9 {JS002 "Joseph Schumeter" 005 92}
	10 {CM001 "Carl Menger" 006 58}
	11 {HHH01 "Hans Herman Hoppe" 012 49}
	12 {JHD01 "Jesus Huerta DeSoto" 009 45}
	13 {RM044 "Robert Murphy" 010 58}
	14 {MF084 "Milton Friedman" 011 88}
	15 {JS003 "Joseph Salerno" 007 88}
	16 {HH002 "Henry Hazlitt" 002 87}
	17 {WB001 "Walter Block" 072 77}
	18 {PB001 "Phillip Bagus" 033 22}
}

/*
arg 0: [insert_records]
arg 1: [3]
arg 2: [1]
arg 3: [IK304 {Immanuel Kant} 938 80]

DEBUG insert_records 0: 'IK304'
DEBUG insert_records 1: 'Immanuel Kant'
DEBUG insert_records 2: '938'
DEBUG insert_records 3: '80'
*/

undef insert_records -nocomplain
proc insert_records {h r l} {
	#######################################################
	# Inputs: Handle h, record_num r, list l
	# Takes a handle to an open DB and required record #
	# And inserts the list elements as fields into the DB
	# puts "DEBUG: Handle is $h record is $r list is '$l'"
	#######################################################
	set i 0
	# WRONG?
	foreach x $l {
		# puts "DEBUG proc insert_records $i: '$x'"
		xbase set $h $i [lindex $l $i]
		++ i
	}
	xbase put $h $r
}

set filename "xbase_test.dbf"

set handle [xbase create $filename "{ID 10} {Name 40} {Code 3} {Age 4}" -y]
array foreach names val ss {
	insert_records $handle $ss $val
}
xbase close $handle

catch {set handle [xbase open $filename]}
if {< $handle 0} {
	die "Failed to open $filename at line: "#__LINE__
}

xbase get $handle 1											# GET Record 1

set field_count [xbase fieldcount $handle]
assert $field_count {== $_ 4} -v -line #__LINE__								# Check the field count is correct
puts "Field len 1:[xbase fieldsize $handle 1]"
assert [xbase fieldsize $handle 0]	{== $_ 10}  -v -line #__LINE__				# 10
puts "Field len 2:[xbase fieldsize $handle 2]"
assert [xbase fieldsize $handle 1]	{== $_ 40} -v -line #__LINE__				# 40
puts "Field len 3:[xbase fieldsize $handle 3]"
assert [xbase fieldsize $handle 2]	{== $_ 3} -v -line #__LINE__				# 3
set line #__LINE__
puts "Field len 4:[xbase fieldsize $handle 4]"
assert [xbase fieldsize $handle 3]	{== $_ 4} -v -line #__LINE__				# 4
newline

set line #__LINE__
puts "Field 1:'[xbase field $handle 0]'"
assert [string trim [xbase field $handle 0]] {eq $_ "IK304"} -v -line #__LINE__				# Record #1 field #1

set line #__LINE__
assert [string trim [xbase field $handle 1]] {eq $_ "Immanuel Kant"} -v -line #__LINE__		# Record #1 field #2

## Record 17

xbase get $handle 17														# GET Record 17
set line #__LINE__
assert [string trim [xbase field $handle 0]] {eq $_ "WB001"} -v -line #__LINE__				# Record #17 field #1
set line #__LINE__
assert [string trim [xbase field $handle 1]] {eq $_ "Walter Block"} -v -line #__LINE__		# Record #17 field #1


/* # How to iterate 
for {set j 0} {< $j $field_count} {++ j} {
	printf "%4i %4s %-11s %-56.56s\r\n" $j\
		[xbase fieldsize $handle $j]\
		[xbase fieldname $handle $j]\
		'[xbase field $handle $j]'
}
*/

xbase close $handle

lib ticol_xbase -unload

array unset names -nocomplain
unset filename field_count handle line -nocomplain
undef insert_records -nocomplain

vars

#####################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #	