###########################################################################
# File: 			test_lindex.tcl
# Purpose:			Test cases for [lindex]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 18/Apr/2018	First version
# 1.01 02/Sep/2019	More tests
###########################################################################
# Notes: 
###########################################################################
# FIX brace inconsistencies
# If in var then picolEval() does not remove outer {} 
###########################################################################

cls

puts "{{foo} {bar}}"					# Outer braces retained (CORRECT)
puts {{foo} {bar}}						# Outer braces removed (CORRECT)

# Check var stores and retrieves lists accurately
puts "Line: "#__LINE__
set list "{{foo} {bar}}"				# Is set as '{{foo} {bar}}'
assert $list {eq $_ "{{foo} {bar}}"} -v -line #__LINE__			# PASS (ActiveState)
set list {{foo} {bar}}					# Is set as '{foo} {bar}'
puts "Set list should be '{foo} {bar}'"
vars list
assert $list {eq $_ "{foo} {bar}"} -v -line #__LINE__			
# PASS (ActiveState)

puts "\$list is '$list'"				# PASS - Therefore: picolEval retrieves as '{foo} {bar}'

# DONE FIX Activestate would return list intact
puts "Line: "#__LINE__
assert [lindex $list] {eq $_ "{foo} {bar}"} -v -line #__LINE__	# PASS (no arg==unchanged list)
assert [lindex $list 0] {eq $_ "foo"} -v -line #__LINE__		# ERROR is "{{foo} {bar}}"
assert [lindex $list 1] {eq $_ "bar"} -v -line #__LINE__		# ERROR is "{{foo} {bar}}"

# Test nesting #
# Match ActiveState #
puts "Line: "#__LINE__
set l {0 {1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}}}
assert [lindex $l 0] {eq $_ 0} -v -line #__LINE__
assert [lindex $l 1] {eq $_ "1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}"} -v -line #__LINE__
assert [lindex $l 2] {eq $_ ""} -v -line #__LINE__
puts [lindex $l 0 1]
assert [lindex $l 1] {eq $_ "1 {2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}}"} -v -line #__LINE__
assert [lindex $l 1 1] {eq $_ "2 {3 {4 {5 {6 {7 {8 {9 0}}}}}}}"} -v -line #__LINE__
assert [lindex $l 1 1 1] {eq $_ "3 {4 {5 {6 {7 {8 {9 0}}}}}}"} -v -line #__LINE__
assert [lindex $l 1 1 1 1] {eq $_ "4 {5 {6 {7 {8 {9 0}}}}}"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1] {eq $_ "5 {6 {7 {8 {9 0}}}}"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1 1] {eq $_ "6 {7 {8 {9 0}}}"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1 1 1] {eq $_ "7 {8 {9 0}}"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1 1 1 1] {eq $_ "8 {9 0}"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1 1 1 1 1] {eq $_ "9 0"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1 1 1 1 1 0] {eq $_ "9"} -v -line #__LINE__
assert [lindex $l 1 1 1 1 1 1 1 1 1 1] {eq $_ "0"} -v -line #__LINE__
unset l
newline

# ActiveState Checked OK
puts "Line: "#__LINE__
set l {0 1 2 {3 {4 5}} {6 7 {8 9 {0 1}}}}
puts [lindex $l 0] 
assert [lindex $l 0] {eq $_ "0"} -v -line #__LINE__
assert [lindex $l 1] {eq $_ "1"} -v -line #__LINE__
assert [lindex $l 2] {eq $_ "2"} -v -line #__LINE__
assert [lindex $l 3] {eq $_ "3 {4 5}"} -v -line #__LINE__
assert [lindex $l 3 0] {eq $_ "3"} -v -line #__LINE__
assert [lindex $l 3 1] {eq $_ "4 5"} -v -line #__LINE__
assert [lindex $l 4] {eq $_ "6 7 {8 9 {0 1}}"} -v -line #__LINE__
assert [lindex $l 4 0] {eq $_ "6"} -v -line #__LINE__
assert [lindex $l 4 1] {eq $_ "7"} -v -line #__LINE__
assert [lindex $l 4 2] {eq $_ "8 9 {0 1}"} -v -line #__LINE__
assert [lindex $l 4 2 0] {eq $_ "8"} -v -line #__LINE__
assert [lindex $l 4 2 1] {eq $_ "9"} -v -line #__LINE__
assert [lindex $l 4 2 2] {eq $_ "0 1"} -v -line #__LINE__
assert [lindex $l 4 2 2 0] {eq $_ "0"} -v -line #__LINE__
assert [lindex $l 4 2 2 1] {eq $_ "1"} -v -line #__LINE__
puts [lindex $l end-1]
assert [lindex $l end-1] {eq $_ "3 {4 5}"} -v -line #__LINE__
puts [lindex $l end-2]
assert [lindex $l end-2] {eq $_ "2"} -v -line #__LINE__
assert [lindex $l end-3] {eq $_ "1"} -v -line #__LINE__
assert [lindex $l end-4] {eq $_ "0"} -v -line #__LINE__
#assert [lindex $l end-5] {eq $_ ""} -v -line #__LINE__		# FAIL - should return "" string
unset l
newline

# ActiveState Checked OK
puts "Line: "#__LINE__
set l {0 1 2 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7} 3 4 5}
puts [lindex $l 3]
assert [lindex $l 3] {eq $_ "0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7"} -v -line #__LINE__
puts [lindex $l 3 2]
assert [lindex $l 3 2] {eq $_ "0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5"} -v -line #__LINE__
puts [lindex $l 3 2 2]
assert [lindex $l 3 2 2] {eq $_ "0 1 {0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5"} -v -line #__LINE__
puts [lindex $l 3 2 2 2]
assert [lindex $l 3 2 2 2] {eq $_ "0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7"} -v -line #__LINE__
puts [lindex $l 3 2 2 2 2]
assert [lindex $l 3 2 2 2 2] {eq $_ "0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5"} -v -line #__LINE__
assert [lindex $l 3 2 2 2 2 2] {eq $_ "0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5"} -v -line #__LINE__
assert [lindex $l 3 2 2 2 2 2 2] {eq $_ "0 1 {0 1 2 3 4 5} 2 3 4 5"} -v -line #__LINE__
assert [lindex $l 3 2 2 2 2 2 2 2] {eq $_ "0 1 2 3 4 5"} -v -line #__LINE__
assert [lindex $l 3 2 2 2 2 2 2 2 2] {eq $_ "2"} -v -line #__LINE__
unset l

set l {0 {0 1 {0 1 2 {0 1 2 3 4 5} 4 5} 4 5} 4 5}
assert [lindex $l 1 2 3] {eq $_ "0 1 2 3 4 5"} -v -line #__LINE__	# PASS
assert [lindex $l {1 2 3 4}]  {eq $_ "4"} -v -line #__LINE__		# PASS
unset l

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
unset list -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #