###########################################################################################
# File: 			test_linecount.tcl
# Purpose:			Test file for __LINE__ count accuracy
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 07/Nov/2019	Reverted ignore close tags in quoted strings in long comments
# 1.01 02/Dec/2019	Reverted ability to embed long comment /* */ tags within strings
###########################################################################################
# Notes:
# DIS CLOSING TAGS 02/DEC/2019
#	Modified rules allowing ignore of LC tags within quoted strings
#	We now act like C++ where /* causes ignore of everything until */ is met
###########################################################################################

set line #__LINE__
assert $line {== $_ 17} -v -line #__LINE__

/* 
Now is the time
for all good men
to come to the aid
of the party
*/

set line #__LINE__
assert $line {== $_ 27} -v -line #__LINE__

# Problematic single dquote within long comment case...
/* 
Now is the time
for all good men
"
to come to the aid
of the party
*/

set line #__LINE__
assert $line {== $_ 39} -v -line #__LINE__

# A single-line quote
# And another
# And another

set line #__LINE__
assert $line {== $_ 46} -v -line #__LINE__

# Embedded within valid quoted string. We should ignore this closing tag. It allows us to comment out string with tags
/* 
Now is the time
for all good men
printf " * / "				# DIS CLOSING TAG 02/DEC/2019
to come to the aid
of the party
*/

set line #__LINE__
assert $line {== $_ 58} -v -line #__LINE__

# FAILS!
# Embedded closing tag within valid multi-line quoted string. We don't currently handle line-continuations
# TEMP DISABLED WITH SPACE
/* 
"Now is the time\
for all good men\
* /\								# DIS CLOSING TAG 02/DEC/2019
to come to the aid\
of the party"
#*/

set line #__LINE__
assert $line {== $_ 72} -v -line #__LINE__

/* 
Now is the time
for all good men
to come to the aid
of the party
*/

set line #__LINE__
assert $line {== $_ 82} -v -line #__LINE__

# Commented comment with some active Tcl code within
# /*
set line #__LINE__
set line #__LINE__
set line #__LINE__
set line #__LINE__
# */

set line #__LINE__
assert $line {== $_ 93} -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #