###############################################################
# File: 			test_list.tcl
# Purpose:			Testing internal Tcl list commands
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23+
# See also:			test_lset.tcl
#
# Versions:
# 1.00 15/Jan/2017	First version
# 1.01 19/Apr/2017	Added [lreverse]
# 1.02 06/Jul/2017	Added [lrand], [lrange]
#					Changed [checkex] to [assert]
###############################################################
# Notes: Various tests
# Possible leak
# 2476@1000 exit. Run at 10000
###############################################################
option expression off

textcolor yellow
puts "Testing test_list.tcl ..."
textcolor

# list (very basic tests to ensure it is live)
puts "Line: "#__LINE__
assert [list ""] {eq $_ ""} -v -line #__LINE__
assert [list a b c d] {eq $_ "a b c d"} -v -line #__LINE__
assert [list a b "c d"] {eq $_ "a b {c d}"} -v -line #__LINE__
assert [list Hello world] {eq $_ "Hello world"} -v -line #__LINE__

# lappend
puts "Line: "#__LINE__
textcolor yellow
puts \[lappend\]
textcolor

puts "Line: "#__LINE__
assert [lappend one two] {eq $_ "two"} -v -line #__LINE__
assert $one  {eq $_ "two"} -v -line #__LINE__

puts "Line: "#__LINE__
unset one -nocomplain
set l [string length $qbf]
puts "qbf is $l char(s) long"

puts "Line: "#__LINE__
set s {}
for {set i 0} {< $i 100} {++ i} {
	lappend s $qbf		# Will add x1 space and x2 braces (3) and -1 terminating space
}
#puts "s is '$s'"
puts "Line: "#__LINE__
puts "s length is: [string length $s] (should be [- [* [+ $::l 3] 100] 1])"
assert [string length $s] {== $_ [- [* [+ $::l 3] 100] 1]} -v -line #__LINE__
unset s 

# lcount
puts "Line: "#__LINE__
textcolor yellow
puts \[lcount\]
textcolor
assert [lcount "one two three"] {== $_  3} -v -line #__LINE__
assert [lcount "one"] {== $_  1} -v -line #__LINE__
assert [lcount ""] {== $_  0} -v -line #__LINE__

# lindex
puts "Line: "#__LINE__
textcolor yellow
puts \[lindex\]
textcolor
set l [list colour "sunflower blue"]
assert [lindex $l 1 1] {eq $_ "blue"} -v -line #__LINE__

# list
puts "Line: "#__LINE__
assert [list orange "red blue" green] {eq $_ "orange {red blue} green"} -v -line #__LINE__

# llength
puts "Line: "#__LINE__
textcolor yellow
puts \[llength\]
textcolor
set l [list colour "sunflower blue"]
assert [llength $l] {== $_  2} -v -line #__LINE__

# lmap
puts "Line: "#__LINE__
textcolor yellow
puts \[lmap\]
textcolor

puts "Line: "#__LINE__
puts [lmap {x {expr {$x * $x}}} {}]
assert [lmap {x {expr {$x * $x}}} {}] {eq $_ ""} -v -line #__LINE__
assert [lmap {x {expr {$x * $x}}} {}] {eq $_ {}} -v -line #__LINE__
puts [lmap {x {expr {$x * $x}}} {1 2 3 4 5}]
assert [lmap {x {expr {$x * $x}}} {1 2 3 4 5}] {eq $_ "1 4 9 16 25"} -v -line #__LINE__
puts [lmap {x {return [string length $x]:$x}} {a bb ccc dddd}]
assert [lmap {x {return [string length $x]:$x}} {a bb ccc dddd}] {eq $_ "1:a 2:bb 3:ccc 4:dddd"} -v -line #__LINE__

# lset
# Pending rename
puts "Line: "#__LINE__

# lrand 
puts "Line: "#__LINE__
textcolor yellow
puts \[lrand\]
textcolor
assert [lrand [array_to_list env]] {ne $_ ""} -v -line #__LINE__
assert [lrand [lrand [array_to_list env]]] {ne $_ ""} -v -line #__LINE__
assert [lrand [lrand [lrand [array_to_list env]]]] {ne $_ ""} -v -line #__LINE__
assert [lrand [lrand [lrand [lrand [array_to_list env]]]]] {ne $_ ""} -v -line #__LINE__
assert [lrand ""] {eq $_ ""} -v -line #__LINE__

# lrange
puts "Line: "#__LINE__
textcolor yellow
puts \[lrange\]
textcolor
assert [lrange $qbf 0 3] {eq $_ "The quick brown fox"} -v -line #__LINE__
assert [lrange $qbf 1 3] {eq $_ "quick brown fox"} -v -line #__LINE__

# lreverse
puts "Line: "#__LINE__
textcolor yellow
puts \[lreverse\]
textcolor
assert [lreverse $qbf] {eq $_ "dog lazy the over jumps fox brown quick The"} -v -line #__LINE__
set q "Now is the {time for all good men} to come to the aid of {the party}"
assert [lreverse $q] {eq $_ "{the party} of aid the to come to {time for all good men} the is Now"} -v -line #__LINE__
unset q -nocomplain

# lsort
puts "Line: "#__LINE__
textcolor yellow
puts \[lsort\]
textcolor
set l [list orange red blue green]
debug off
assert [lsort $l] {eq $_ "blue green orange red"}
debug off
set l [list orange "red blue" green]
debug off
assert [lsort $l] {eq $_ "green orange {red blue}"}

unset i l -nocomplain

puts "Line: "#__LINE__
textcolor yellow
puts "Whitespace handling ..."
textcolor
# Check match ActiveState behaviour with whitespace
# ERror line wrap with -unescape
puts "Line: "#__LINE__
puts "{  foo        bar \"moo par\" }"
puts "Line: "#__LINE__
echo "{  foo        bar \"moo par\" }"
puts "Line: "#__LINE__
echo {"{  foo        bar \"moo par\" }"}
puts "Line: "#__LINE__
echo [escape [list {  foo        bar "moo par" }]]
puts "Line: "#__LINE__

# ActiveState
# puts [list {  foo        bar "moo par" }]
# {  foo        bar "moo par" }

# FIX Should trap malformes tring...
# ActiveState
#% puts "{  foo        bar "moo par" }"
#extra characters after close-quote


puts "Line: "#__LINE__
echo [list {  foo        bar "moo par" }]
puts "Line: "#__LINE__
#assert [list {  foo        bar "moo par" }] {eq $_ "{  foo        bar \"moo par\" }" } -v -line #__LINE__		# Uncorrupted rvalue
assert [list {  foo        bar "moo par" }] {eq $_ [list {  foo        bar "moo par" }] } -v -line #__LINE__ #
#assert [escape [list {  foo        bar "moo par" }]] {eq $_ "{  foo        bar "moo par" }" } -v -line #__LINE__ #
#assert [list {  foo        bar "moo par" }] {eq $_ "{  foo        bar \"moo par\" }" } -unescape -v -line #__LINE__ # BAD
#assert [list "  foo        bar {moo par} "] {eq $_ "  foo        bar {moo par} "} -v -line #__LINE__

puts "Line: "#__LINE__
unset a b -nocomplain
set a {a b c}; set b {d e f}
assert [list $a $b] {eq $_ {{a b c} {d e f}}} -v -line #__LINE__
assert [concat $a $b] {eq $_ {a b c d e f}} -v -line #__LINE__
assert [list {*}$a {*}$b] {eq $_ {a b c d e f}} -v -line #__LINE__
unset a b -nocomplain

puts "Line: "#__LINE__
assert [list {a "" b "" c}] {eq $_ [list {a "" b "" c}]} -v -line #__LINE__ # Verified in A/S 8.6
#assert [list {a "" b "" c}] {eq $_ {a "" b "" c}} -v -line -trace #__LINE__ # Verified in A/S 8.6 (BORKED)
# -> 'a "" b "" c'			-> 			'{a "" b "" c}'

#/*
# Tested seprate from assert, [eval] matches these two okay
# DIFF? command eval?
puts "Line: "#__LINE__
set s {a "" b "" c}
puts "s->'$s'"
puts [eval {eq $s {a "" b "" c}}]	# PASS */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #