############################################################################################
# File:				test_list_to_array.tcl
# Author:			M Shaw
# Purpose:			Test file for [list_to_array]
#
# Versions:
# 1.00 30/Jun/2018 First version
###########################################################################################
# Notes:
# FIX leaks
# integer [list_to_array <tcl-list> arrayname]
# ListToArray is the base of this command and structs
#	Testing [list_to_array] should be the baseline for tests for both
# FIX consider option to import tuples as per [array set]
###########################################################################################

option expression off

puts "Line: "#__LINE__
array unset a -nocomplain			# Slight leak, 2124-2300 losing 4k pages every n loops


set l {Hello from "Ticol Tcl" this fine day}

set count [list_to_array $l a]		# Slight leak

assert $count {== $_ 6} -v -line #__LINE__
assert [array size a] {== $_ 6} -v -line #__LINE__
catch {
	assert $a(0)  {eq $_ Hello} -v -line #__LINE__
}


# FAIL - list_to_array does not strip dquotes in multi word string
puts "\$a(2) '$a(2)' at line: "#__LINE__
catch {
	assert $a(2)  {eq $_ "Ticol Tcl"} -v -line #__LINE__
}

puts "Line: "#__LINE__
array unset a
set l {Hello from {Ticol Tcl} this fine day}
set count [list_to_array $l a]
assert $count {== $_ 6} -v -line #__LINE__
assert [array size a] {== $_ 6} -v -line #__LINE__
puts "\$a(2) '$a(2)'"
catch {
	assert $a(2)  {eq $_ "Ticol Tcl"} -v -line #__LINE__
}


puts "Line: "#__LINE__
array unset a
set l {The {quick {brown fox}} jumps over {the lazy} dog}
set count [list_to_array $l a]
assert $count {== $_ 6} -v -line #__LINE__
assert [array size a] {== $_ 6} -v -line #__LINE__
for {set i 0} {< $i [array size a]} {++ i} {
	puts "a($i) == '$a($i)"
}
catch {
	assert $a(1)  {eq $_ "quick {brown fox}"} -v -line #__LINE__
}

puts "Line: "#__LINE__
array unset a
set l {The "quick {brown fox}" jumps over {the lazy} dog}
set count [list_to_array $l a]
assert $count {== $_ 6} -v -line #__LINE__
assert [array size a] {== $_ 6} -v -line #__LINE__
for {set i 0} {< $i [array size a]} {++ i} {
	puts "a($i) == '$a($i)"
}
assert $a(0)  {eq $_ "The"} -v -line #__LINE__
assert $a(1)  {eq $_ "quick {brown fox}"} -v -line #__LINE__
assert $a(4)  {eq $_ "the lazy"} -v -line #__LINE__


puts "Line: "#__LINE__
array unset a
set l {The "quick {brown fox}" jumps over "the lazy" dog}
set count [list_to_array $l a]
assert $count {== $_ 6} -v -line #__LINE__
assert [array size a] {== $_ 6} -v -line #__LINE__
for {set i 0} {< $i [array size a]} {++ i} {
	puts "a($i) == '$a($i)"
}
assert $a(0)  {eq $_ "The"} -v -line #__LINE__
assert $a(1)  {eq $_ "quick {brown fox}"} -v -line #__LINE__
assert $a(4)  {eq $_ "the lazy"} -v -line #__LINE__


puts "Line: "#__LINE__
array unset a
set l {w pt {s "Hello world" } {q {b} } {r {} }}
puts "Checking '$l'"
set count [list_to_array $l a]
for {set i 0} {< $i [array size a]} {++ i} {
	puts "a($i) == '$a($i)"
}
assert $count {== $_ 5} -v -line #__LINE__
assert [array size a] {== $_ 5} -v -line #__LINE__
assert $a(0)  {eq $_ "w"} -v -line #__LINE__
# FIX assert should strip x1 set of braces
catch {
assert $a(2)  {eq $_ {s "Hello world" }} -v -line #__LINE__
}

puts "Line: "#__LINE__
array unset a
set l {w pt {s {Hello world} } {q {b} } {r {} }}
puts "Checking '$l'"
set count [list_to_array $l a]
for {set i 0} {< $i [array size a]} {++ i} {
	puts "a($i) == '$a($i)"
}
assert $count {== $_ 5} -v -line #__LINE__
assert [array size a] {== $_ 5} -v -line #__LINE__
assert $a(2)  {eq $_ "s {Hello world}" } -v -line #__LINE__


puts "Line: "#__LINE__
puts "Indented tab whitespace and regular layout"
array unset a -nocomplain
set l {0 Zero
		1 One
		2 Two
		3 Three}

set count [list_to_array $l a]
assert $count {== $_ 8} -v -line #__LINE__
assert [array size a] {== $_ 8} -v -line #__LINE__
catch {
	assert $a(0)  {eq $_ 0} -v -line #__LINE__
	assert $a(1)  {eq $_ Zero} -v -line #__LINE__
	assert $a(2)  {eq $_ 1} -v -line #__LINE__
	assert $a(3)  {eq $_ One} -v -line #__LINE__
	assert $a(4)  {eq $_ 2} -v -line #__LINE__
	assert $a(5)  {eq $_ Two} -v -line #__LINE__
	assert $a(6)  {eq $_ 3} -v -line #__LINE__
	assert $a(7)  {eq $_ Three} -v -line #__LINE__
}

puts "Line: "#__LINE__
puts "Random whitespace and irregular layout"
array unset a -nocomplain
set l {		0 Zero		
		1 One	2 Two
		3             Three  	}

set count [list_to_array $l a]
assert $count {== $_ 8} -v -line #__LINE__
assert [array size a] {== $_ 8} -v -line #__LINE__
catch {
	assert $a(0)  {eq $_ 0} -v -line #__LINE__
	assert $a(1)  {eq $_ Zero} -v -line #__LINE__
	assert $a(2)  {eq $_ 1} -v -line #__LINE__
	assert $a(3)  {eq $_ One} -v -line #__LINE__
	assert $a(4)  {eq $_ 2} -v -line #__LINE__
	assert $a(5)  {eq $_ Two} -v -line #__LINE__
	assert $a(6)  {eq $_ 3} -v -line #__LINE__
	assert $a(7)  {eq $_ Three} -v -line #__LINE__
}

puts "Line: "#__LINE__
puts "Even tuples (pairs) as used by \[struct create\]"
array unset a -nocomplain
set l {{0 Zero}
		{1 One}
		{2 Two}
		{3 Three}
		}

set count [list_to_array $l a]
assert $count {== $_ 4} -v -line #__LINE__
assert [array size a] {== $_ 4} -v -line #__LINE__
catch {
	assert $a(0)  {eq $_ "0 Zero"} -v -line #__LINE__
	assert $a(1)  {eq $_ "1 One"} -v -line #__LINE__
	assert $a(2)  {eq $_ "2 Two"} -v -line #__LINE__
	assert $a(3)  {eq $_ "3 Three"} -v -line #__LINE__
}

puts "Line: "#__LINE__
puts "Uneven tuples (pairs) as used by \[struct create\]"
array unset a -nocomplain
set l {	{0 Zero} {1 One}
		{2 Two}				
	{3 Three}    
		}

set count [list_to_array $l a]
assert $count {== $_ 4} -v -line #__LINE__
assert [array size a] {== $_ 4} -v -line #__LINE__
catch {
	assert $a(0)  {eq $_ "0 Zero"} -v -line #__LINE__
	assert $a(1)  {eq $_ "1 One"} -v -line #__LINE__
	assert $a(2)  {eq $_ "2 Two"} -v -line #__LINE__
	assert $a(3)  {eq $_ "3 Three"} -v -line #__LINE__
}

# Struct create always calls list to pairs which gets a list of pairs

puts "Line: "#__LINE__
puts "Uneven tuples (pairs) as used by \[struct create\]"
array unset a -nocomplain
set l {0 Zero {1 One}
               2 Two
								{3 Three}
							}

set count [list_to_array $l a]
assert $count {== $_ 6} -v -line #__LINE__
assert [array size a] {== $_ 6} -v -line #__LINE__
catch {
	assert $a(0)  {eq $_ "0"} -v -line #__LINE__
	assert $a(1)  {eq $_ "Zero"} -v -line #__LINE__
	assert $a(2)  {eq $_ "1 One"} -v -line #__LINE__
	assert $a(3)  {eq $_ "2"} -v -line #__LINE__
	assert $a(4)  {eq $_ "Two"} -v -line #__LINE__
	assert $a(5)  {eq $_ "3 Three"} -v -line #__LINE__
}



###########################################################################################

unset count i l -nocomplain
array unset a -nocomplain

###########################################################################################
vars

newline
textcolor green
puts "* Done *"
textcolor
#*/

# EOF #