############################################################################################
# File: 			test_load_lib_twice.tcl
# Purpose:			Test loading a lib twice to see if we get a GPF
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 24/Aug/2018	First version
# 1.02 27/Feb/2022	Updated
############################################################################################
# Notes:
# test_load_lib_twice.tcl
# Should be successfully trapped in this executable context by the lib manager 'database'
############################################################################################

undef at_exit -nocomplain

proc at_exit {} {
	newline
	textcolor darkyellow
	puts "Unloading ticol_cpu at exit from script 'test_load_lib_twice.tcl' ..."
	textcolor
	newline
	textcolor cyan
	lib ticol_cpu -unload -nocomplain -verbose
	textcolor
}

newline
puts "Loading once..."
textcolor cyan
lib ticol_cpu -verbose
textcolor
newline
puts "Loading again..."

set i 0
try {lib ticol_cpu} catch {
	textcolor red
	puts "Trapped ticol_cpu already loaded (IGNORE)"
	textcolor
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
newline

puts "Loading again with -nocomplain..."
newline

set i 0
try {lib ticol_cpu -nocomplain} catch {
	textcolor red
	puts "Trapped ticol_cpu already loaded"
	textcolor
	set i 1
}
assert $i {== $_ 0} -v -line #__LINE__
newline

puts "Exiting - will call lib unload at exit"

unset i
#lib ticol_cpu -unload		# Will be unloaded by [at_exit]

puts "Modules cp* '[commands cp*]'"
puts [lib -modules]
############################################################################################
newline
vars
newline

textcolor green
puts "* Pass *"
textcolor

# EOF #