############################################################################################
# File: 			test_lremove.tcl
# Purpose:			Test [lremove] and [lremove_all] as procs plus as a built-in command
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Feb/2022	First version
# 1.01 06/Mar/2022	Added basic tests for built-in [lremove] command
############################################################################################
# Notes:
############################################################################################

set lremove_is_proc $false			# Switchable for new, built-in command

#undef lremove -nocomplain
if {! [defined lremove]} {			# Original prototype
	textcolor yellow
	puts "Defining proc \[lremove\]"
	textcolor
	set lremove_is_proc $::true

	proc lremove {listVariable value {case {-exact}}} {			# Example from tcl.tk
		upvar 1 $listVariable var
		set idx [lsearch $case $var $value]
		set var [lreplace $var $idx $idx]
	}
}

# DONE Terminating case returns -1 as idx so [lreplace] must handle -1 case
undef lremove_all -nocomplain
proc lremove_all {listVariable value {case {-exact}}} {
	upvar 1 $listVariable var
	#puts "case is $case"
	while {>= [set idx [lsearch $case $var $value]] 0} {		# idx is base 0
		#puts "idx $idx"
		set var [lreplace $var $idx $idx]
		#puts "Var is $var"
	}
	return $var		# This is required due to the loop condition
}

set s {a b c a b c}
puts [lremove_all s c]
newline
############################################################################################

# Will remove last "the"
set s $qbf
if {$lremove_is_proc} {
	puts [lremove s the]
} else {
	puts "lremove command"
	puts [lremove s the -var]
}
puts "$s @"#__LINE__
assert $s {eq $_ "The quick brown fox jumps over lazy dog"} -v -line #__LINE__

# Will remove first "the"
set s $qbf
if {$lremove_is_proc} {
	puts [lremove s the -nocase]		# Should remove 1st instance only
} else {
	puts [lremove s the -nocase -var]
}
puts "$s @"#__LINE__
assert $s {eq $_ "quick brown fox jumps over the lazy dog"} -v -line #__LINE__

# FIX Should this return an integer?
# Test default, -exact (PASS)
set s $qbf
puts [lremove_all s the]
puts "$s @"#__LINE__
assert $s {eq $_ "The quick brown fox jumps over lazy dog"} -v -line #__LINE__

# Test -nocase (PASS)
set s $qbf
puts [lremove_all s the -nocase]
puts "$s @"#__LINE__
assert $s {eq $_ "quick brown fox jumps over lazy dog"} -v -line #__LINE__

if {! $lremove_is_proc} {		# If a built-in command...
	set s $qbf
	# LITERAL version
	assert [lremove $s the] {eq $_ "The quick brown fox jumps over lazy dog"} -v -line #__LINE__
	assert [lremove $s the -all] {eq $_ "The quick brown fox jumps over lazy dog"} -v -line #__LINE__
	assert [lremove $s the -nocase] {eq $_ "quick brown fox jumps over the lazy dog"} -v -line #__LINE__
	assert [lremove $s the -nocase -all] {eq $_ "quick brown fox jumps over lazy dog"} -v -line #__LINE__
	
	# VAR version
	set s $qbf
	assert [lremove s the -var] {eq $_ "The quick brown fox jumps over lazy dog"} -v -line #__LINE__
	set s $qbf
	assert [lremove s the -var -all] {eq $_ "The quick brown fox jumps over lazy dog"} -v -line #__LINE__
	set s $qbf
	assert [lremove s the -var -nocase] {eq $_ "quick brown fox jumps over the lazy dog"} -v -line #__LINE__
	set s $qbf
	assert [lremove s the -var -nocase -all] {eq $_ "quick brown fox jumps over lazy dog"} -v -line #__LINE__
}

############################################################################################
newline
unset lremove_is_proc s 

vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #