#####################################################################################
# File: 			test_lreplace.tcl
# Purpose:			Tests for [lreplace]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 19/Sep/2019	First version
#####################################################################################
# Notes:
# list [lreplace list <start> <end> ?list-item? ...]
#####################################################################################
# TODO (not yet implemented)
# lreplace list first last ?element element ...?
# remove elements <first> to <last>
# Insert list of elements at the vacated space
# ?element? may be null or absent, in which case the elements are removed
# 'end' or 'end-N' form indexes are supported
# Nesting is not supported
#
# FIX anomaly?
# 	No, because the var is presenting as singular arg group
# % puts [lreplace 1 2 3 4 5 6 7 1 1]
# > 1 4 5 6 7 1 1
# "2 3" are removed, why?
#
# puts [lreplace {1 2 3 4 5 6 7} 1 1]
# > 1 3 4 5 6 7
# FIX [lreplace '0 2 3 4 6 7' 0 0] -> ' 2 3 4 6 7'  (leading space error)
#
# Method
# ------
# Convert to array
# Save up to <start>
# Write out the new element args, preserving spaces
# Write out the remainder
#####################################################################################
/*
proc lremove {listVariable value} {
    upvar 1 $listVariable var
    set idx [lsearch -exact $var $value]
    set var [lreplace $var $idx $idx]
}
# */
#####################################################################################

option echo on

textcolor yellow
puts "Testing \[lreplace\] ..."
textcolor
newline

#####################################################################################
puts "Line: "#__LINE__
# Special case - NULL item
# <start> is the only index checked for validity. 
# Out of range <end> (finish) values are ignored
# Test 1st item
#foo
puts "Test empty"
puts [lreplace {} 0 0 foo]
assert [lreplace {} 0 0 foo] {eq $_ {foo}} -v -line #__LINE__
assert [lreplace {} 0 1 foo] {eq $_ {foo}} -v -line #__LINE__
assert [lreplace {} 0 1234 foo] {eq $_ {foo}} -v -line #__LINE__
assert [lreplace {} -1 1234 foo] {eq $_ {foo}} -v -line #__LINE__	# Verified with activestate

###########################################
# Special cases
# Negative index. Insert at HEAD of list

puts "\[lreplace {a} 0 0 foo\]"
assert [lreplace {a} 0 0 foo] {eq $_ {foo}} -v -line #__LINE__ 	# Verified with activestate
puts "\[lreplace {a} -1 0 foo\]"
assert [lreplace {a} -1 0 foo] {eq $_ {foo}} -v -line #__LINE__ 	# Verified with activestate
# FAIL
textcolor red
puts "* FIXME! *"
textcolor
puts "\[lreplace {a} 0 -1 foo\]"
assert [lreplace {a} 0 -1 foo] {eq $_ {foo a}} -v -line #__LINE__ 	# Verified with activestate
# FAIL
puts [lreplace {a} -1 -1 foo]
assert [lreplace {a} -1 -1 foo] {eq $_ {foo a}} -v -line #__LINE__ 	# Verified with activestate

puts "Line: "#__LINE__
# Test ERROR case
# We care only about the start value being valid
#	% puts [lreplace {} 1 1 foo]
#	list doesn't contain element 1
set i 0
try {
	lreplace {} 1 1 foo
} catch {
	set i 1
}
assert $i {eq $_ 1} -v -line #__LINE__

#####################################################################################
puts "Line: "#__LINE__
# Test 1st item
#a foo c d e
assert [lreplace {a b c d e} 0 0 foo] {eq $_ {foo b c d e}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test last item
#a foo c d e
assert [lreplace {a b c d e} 4 4 foo] {eq $_ {a b c d foo}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test last item by 'end'
#a foo c d e
assert [lreplace {a b c d e} end end foo] {eq $_ {a b c d foo}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test last item by 'end-0'
#a foo c d e
assert [lreplace {a b c d e} end-0 end-0 foo] {eq $_ {a b c d foo}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test replace 2nd item
#puts [lreplace {a b c d e} 1 1 foo]
#a foo c d e
assert [lreplace {a b c d e} 1 1 foo] {eq $_ {a foo c d e}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test replace 3rd item
#a foo c d e
assert [lreplace {a b c d e} 2 2 foo] {eq $_ {a b foo d e}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test replace 4th item
#a foo c d e
assert [lreplace {a b c d e} 3 3 foo] {eq $_ {a b c foo e}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test replace invalid (HI) item
# ActiveState
#	% puts [lreplace {a b c d e} 6 6 foo]
#	a b c d e foo
assert [lreplace {a b c d e} 6 6 foo] {eq $_ {a b c d e foo}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test replace invalid (HI) item
# ActiveState
#	% puts [lreplace {a b c d e} 7 7 foo]
#	a b c d e foo
assert [lreplace {a b c d e} 7 7 foo] {eq $_ {a b c d e foo}} -v -line #__LINE__

puts "Line: "#__LINE__
# Test replace invalid (HI) item
# ActiveState
#	% puts [lreplace {a b c d e} 1000 1000 foo]
#	a b c d e foo
assert [lreplace {a b c d e} 1000 1000 foo] {eq $_ {a b c d e foo}} -v -line #__LINE__

puts "Line: "#__LINE__
# Replace 'b c' with 'three more elements'
puts [lreplace {a b c d e} 1 2 three more elements]
#a three more elements d e
assert [lreplace {a b c d e} 1 2 three more elements] {eq $_ {a three more elements d e}} -v -line #__LINE__

puts "Line: "#__LINE__
# Remove last element
set var {a b c d e}
#a b c d e
set var [lreplace $var end end]
#a b c d
puts $var
assert $var {eq $_ {a b c d}} -v -line #__LINE__

# 0 0 leaves prepending space (fixed with a kludge)
#[lreplace ' 0 1 2 3 4 5 6 7 ' 0 0] -> '*1 2 3 4 5 6 7'
puts [lreplace {0 1 2 3 4 5 6 7} 0 0]
assert [lreplace {0 1 2 3 4 5 6 7} 0 0] {eq $_ {1 2 3 4 5 6 7}} -v -line #__LINE__


unset i var

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
puts "Line: "#__LINE__
newline
vars
newline
textcolor green
puts "* Done *"
textcolor
# EOF #