#################################################################################
# File: 			test_ls.tcl
# Purpose:			Tests for [ls]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 18/Nov/2017	First version
# 1.01 16/Jul/2018	Modify to detect filetype as .tcl, .tcx
# 1.02 23/Jul/2018	Bugfix. Missing [lcount] at line 31
#################################################################################
# Notes:
# Remember to double-backslash any paths!
#################################################################################

option expression off
option echo on

textcolor yellow
set scriptnamepath [info script]
if {eq $scriptnamepath ""} {
	puts "\[info script\] is empty. (Possibly running embedded). Skipping test"
	return
}
set scriptname [file name $scriptnamepath]
puts "Running '$scriptnamepath'"
textcolor
newline

unset s -nocomplain
set s [ls *]			# No -list or -array
textcolor yellow
puts "\[ls *\] -> '[string left $s 30]'\r\n"
textcolor
assert [lcount $s] {> $_ 0} -v -line #__LINE__
newline

set s [ls * -list]
textcolor yellow
puts "\[ls * -list\] -> [string left $s 50]"
textcolor
assert $s {ne $_ ""} -v -line #__LINE__
newline

set s [ls *.tcl -list]
textcolor yellow
puts "\[ls *.tcl -list\] -> [string left $s 50]"
textcolor
assert $s {ne $_ ""} -v -line #__LINE__
newline

if {ne [info script] ""} {					# Could be embedded Ticol
	if {ne [info script] "internal.tcl"} {
		textcolor yellow
		puts "\[info script\] ([info script])-> \[ls \[info script\] -list\]"
		textcolor
		set s [ls [info script] -list]			# Returns a list of one
		puts "s is '$s' @"#__LINE__
		#assert $s {ne $_ ""} -v -line #__LINE__
		assert $s {eq $_ [chr 123]$::scriptname[chr 125]} -v -line #__LINE__
		assert $s {eq $_ [chr 123][file name $::scriptnamepath][chr 125]} -v -line #__LINE__
		assert $s {eq $_ [chr 123][file name [info script]][chr 125]} -v -line #__LINE__
		assert [unwrap $s] {eq $_ [file name [info script]]} -v -line #__LINE__
		puts "Search $s for [file name [info script]]"
		assert [lsearch $s [file name [info script]]] {> $_ -1} -v -line #__LINE__
	}
}
newline

textcolor yellow
puts "\[ls *.tcl -la -r\]"
textcolor
set s [escape [ls *.tcl -la -r]]		# NO LIST -list (returns count)
puts "\[ls *.tcl -la -r\] -> [string left $s 50]"
assert $s {> $_ 0} -v -line #__LINE__
newline

puts "Line: "#__LINE__
set filetype [file extension [info script]]

textcolor yellow
puts "\[ls *$filetype -la -r -list\] @"#__LINE__
textcolor

/*
# Find our script amongst ls listing
# FIX Won't work for internal.tcl (embedded)
puts "Line: "#__LINE__
set s [escape [ls *$filetype -la -r -list]]
puts "\[ls *$filetype -la -r -list\] -> [string left $s 50]... @ line:"#__LINE__
#assert $s {> $_ 0} -v -line #__LINE__
assert [strstr $s $scriptname] {!= $_ 0} -v -line #__LINE__
newline
# */

puts "Line: "#__LINE__
# set i 0; foreach x [ls x*.tcl -list -r] {++ i; puts "$i:$x"} # PASS
# set i 0; foreach x [ls t*.tcl -list -la -r] {++ i; puts "$i:'[string left $x 70]'"} # PASS


unset filetype s scriptname scriptnamepath -nocomplain

vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #