########################################################################
# File:				test_lsearch.tcl
# Author:			M Shaw
# Purpose:			Test bed for lsearch command
# Language:			Ticol Tcl v1.19+ .. 1.23+
#
# Versions:
# 1.00 12/Jan/2016	First version (implemented)
# 1.01 18/Jul/2017	Added tests, replace [checkex] with [assert]
########################################################################
# Notes:
########################################################################

option expression off

assert [list orange "red blue" green] {eq $_ "orange {red blue} green"} -v -line #__LINE__

assert [lsearch -exact -inline -all -not {a c b c a b c} c] {eq $_ "a b a b"} -v -line #__LINE__
assert [lsearch -exact -all -not {a c b c a b c} c ] {eq $_ "0 2 4 5"} -v -line #__LINE__
assert [lsearch -exact -inline -all {a c b c a b c} c ] {eq $_ "c c c"} -v -line #__LINE__
assert [lsearch -exact -all {a c b c a b c} c] {eq $_ "1 3 6"} -v -line #__LINE__
assert [lsearch -exact -all -start 2 {a c b c a b c} c] {eq $_ "3 6"} -v -line #__LINE__

# ActiveState -sorted affects return accuracy
assert [lsearch -exact -sorted -start 4 {b z f e c p l r} {e}] {eq $_ -1} -v -line #__LINE__	# A/S -1
assert [lsearch -exact -sorted -start 3 {b z f e c p l r} {e}] {eq $_ 3} -v -line #__LINE__	# A/S -1
assert [lsearch -exact -sorted -start 2 {b z f e c p l r} {e}] {eq $_ 3} -v -line #__LINE__	# A/S -1
assert [lsearch -exact -sorted -start 1 {b z f e c p l r} {e}] {eq $_ 3} -v -line #__LINE__	# A/S -1
assert [lsearch -exact -sorted -start 0 {b z f e c p l r} {e}] {eq $_ 3} -v -line #__LINE__	# A/S 3

assert [lsearch  {ba ze fi eo cu pa le ri} fred] {eq $_ -1} -v -line #__LINE__
assert [lsearch  {ba ze fi eo cu pa le ri} *] {eq $_ 0} -v -line #__LINE__
assert [lsearch  -inline {ba ze fi eo cu pa le ri} *] {eq $_ ba} -v -line #__LINE__
assert [lsearch  -all -not {ba ze fi eo cu pa le ri} *] {eq $_ {}} -v -line #__LINE__
assert [lsearch -inline -all -not {ba ze fi eo cu pa le ri} *] {eq $_ {}} -v -line #__LINE__
assert [lsearch  {ba ze fi eo cu pa le ri} ?o] {eq $_ 3} -v -line #__LINE__
assert [lsearch -inline {ba ze fi eo cu pa le ri} ?o] {eq $_ eo} -v -line #__LINE__
assert [lsearch -inline -all {ba ze fi eo cu pa le ri} ?e] {eq $_ "ze le"} -v -line #__LINE__
assert [lsearch -inline -glob -all {ba ze fi eo cu pa le ri} ?e] {eq $_ "ze le"} -v -line #__LINE__
assert [lsearch -inline -all -not {ba ze fi eo cu pa le ri} ?e] {eq $_ "ba fi eo cu pa ri"} -v -line #__LINE__
assert [lsearch -all -not {ba ze fi eo cu pa le ri} ?e] {eq $_ "0 2 3 4 5 7"} -v -line #__LINE__
assert [lsearch -glob -all -not {ba ze fi eo cu pa le ri} ?e] {eq $_ "0 2 3 4 5 7"} -v -line #__LINE__
assert [lsearch -exact -nocase {{a b c} {d e f}} {D E F}] {eq $_ 1} -v -line #__LINE__
assert [lsearch -exact -nocase {{a b c} {d e f}} {p q r} ] {eq $_ -1} -v -line #__LINE__
assert [lsearch -exact -sorted -inline -start 4 {b z f e c p l r} {p}] {eq $_ p} -v -line #__LINE__
assert [lsearch -exact -sorted -start 4 {b z f e c p l r} {p}] {eq $_ 5} -v -line #__LINE__	# c p l r -> c l p r -> 4 (base 1 ?)
assert [lsearch -inline -all -exact -not -start 0 {a "Hello there" x y z} {x}] {eq $_ "a {Hello there} y z"} -v -line #__LINE__
assert [lsearch -exact -inline -all -not {a c b c a b c} c] {eq $_ "a b a b"} -v -line #__LINE__
assert [lsearch -exact -all -not {a c b c a b c} c] {eq $_ "0 2 4 5"} -v -line #__LINE__
assert [lsearch -exact -inline -all {a c b c a b c} c ] {eq $_ "c c c"} -v -line #__LINE__
assert [lsearch -exact -all {a c b c a b c} c ] {eq $_ "1 3 6"} -v -line #__LINE__
assert [lsearch -exact -all -start 2 {a c b c a b c} c] {eq $_ "3 6"} -v -line #__LINE__
assert [lsearch -all -inline $qbf *] {eq $_ $::qbf} -v -line #__LINE__	# Note, cannot wrap in braces!
assert [lsearch -all -inline [list $qbf] *] {eq $_ [list $::qbf]} -v -line #__LINE__

##############################################################################
assert [lsearch -all -nocase -inline $qbf t*] {eq $_ "The the"} -v -line #__LINE__	# A/S 3

#ifdef notes
Note that [lsearch] does not 'eval' it's arguments...

lsearch -all -inline {list $qbf} *                       => list $qbf (correct)
lsearch -all -inline "$qbf" *                            => quick brown ..
#endif

##############################################################################
vars

newline
textcolor green
puts "* \[lsearch\] Done *"
textcolor

# EOF #