###########################################################################
# File: 			test_lset.tcl
# Purpose:			Test case for list based [lset]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 04/Sep/2019	First version
###########################################################################
# Notes: 
# List based [lset] is new for September 2019
# TODO: Improve error trapping in recursion
###########################################################################
option expression off
option stacktrace off

textcolor yellow
puts "Testing \[lset\] (list based) ..."
textcolor
newline

puts "Line: "#__LINE__
set x {0 1 2 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5}
puts [lset x {3 2 2 3} "** Hi **"]
puts "lset result in x is '$x'"
#assert $x {eq $_ {0 1 2 {0 1 {0 1 {0 1 2 {** Hi **} 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5}} -v -line #__LINE__

puts "Line: "#__LINE__
set x {0 1 2 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5}
puts [lset x 3 2 2 3 "** Hi **"]					# No braces
puts "lset result in x is '$x'"


puts "Line: "#__LINE__
set x {0 1 2 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7} 3 4 5}
lset x {3 2 2 2 2 2 2 2 2} 22
assert $x {eq $_ {0 1 2 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 22 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7} 3 4 5}} -v -line #__LINE__

# FIX reusing same list twice leads to corruption
#set x {0 1 2 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7} 3 4 5}
lset x {3 2 2 2 2 2 2 2 3} 23
assert $x {eq $_ {0 1 2 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 22 23 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7} 3 4 5}} -v -line #__LINE__

lset x {3 2 2 2 2 2 2 2 4} 24
assert $x {eq $_ {0 1 2 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 {0 1 22 23 24 5} 2 3 4 5} 2 3 4 5} 2 3 4 5} 2 3 4 5 6 7} 2 3 4 5} 2 3 4 5} 2 3 4 5 {0 1 {0 1 2 3 4 5} 2 3 4 5} 6 7} 3 4 5}} -v -line #__LINE__
newline

# Test end?-N? ############################################
puts "Line: "#__LINE__
textcolor yellow
puts "Testing \[lset end?-N?\] (list based) ..."
textcolor
newline

set x {0 1 2 3 4 5}
lset x end Hi
puts $x
assert $x {eq $_ {0 1 2 3 4 Hi}} -v -line #__LINE__
set x {0 1 2 3 4 5}
lset x end-1 Hi
puts $x
assert $x {eq $_ {0 1 2 3 Hi 5}} -v -line #__LINE__

set x {0 1 2 3 4 5}
lset x end-2 Hi
puts $x
assert $x {eq $_ {0 1 2 Hi 4 5}} -v -line #__LINE__

set x {0 1 2 3 4 5}
lset x end-3 Hi
puts $x
assert $x {eq $_ {0 1 Hi 3 4 5}} -v -line #__LINE__

set x {0 1 2 3 4 5}
lset x end-4 Hi
puts $x
assert $x {eq $_ {0 Hi 2 3 4 5}} -v -line #__LINE__

set x {0 1 2 3 4 5}
lset x end-5 Hi
puts $x
# FIX { Hi 1 2 3 4 5} NE {Hi 1 2 3 4 5}
#assert $x {eq $_ { Hi 1 2 3 4 5}} -v -line #__LINE__
assert $x {eq $_ {Hi 1 2 3 4 5}} -v -line #__LINE__


puts "Line: "#__LINE__
textcolor red
puts "Ignore out of bounds end-N error"
textcolor
set x {0 1 2 3 4 5}
set i 0
try {
	lset x end-20 Hi
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__			# Successfully caught
puts $x

# ActiveState help examples

puts "Line: "#__LINE__
# In each of these examples, the initial value of x is: 
set x [list [list a b c] [list d e f] [list g h i]]
assert $x {eq $_ {{a b c} {d e f} {g h i}}} -v -line #__LINE__
#			? {a b c} {d e f} {g h i}

#The indicated return value also becomes the new value of x 
# (except in the last case, which is an error which leaves the value of x unchanged.)

puts "Line: "#__LINE__
lset x {j k l}
assert $x {eq $_ {j k l}} -v -line #__LINE__	#			? j k l

puts "Line: "#__LINE__
lset x {} {j k l}								#			? j k l
assert $x {eq $_ {j k l}} -v -line #__LINE__

puts "Line: "#__LINE__
set x [list [list a b c] [list d e f] [list g h i]]
lset x 0 j										#			? j {d e f} {g h i}
assert $x {eq $_ {j {d e f} {g h i}}} -v -line #__LINE__

puts "Line: "#__LINE__
set x [list [list a b c] [list d e f] [list g h i]]
lset x 2 j										#			? {a b c} {d e f} j
assert $x {eq $_ {{a b c} {d e f} j}} -v -line #__LINE__

puts "Line: "#__LINE__
set x [list [list a b c] [list d e f] [list g h i]]
lset x end j									#			? {a b c} {d e f} j
assert $x {eq $_ {{a b c} {d e f} j}} -v -line #__LINE__

puts "Line: "#__LINE__
set x [list [list a b c] [list d e f] [list g h i]]
lset x end-1 j									#			? {a b c} j {g h i}
assert $x {eq $_ {{a b c} j {g h i}}} -v -line #__LINE__

puts "Line: "#__LINE__
set x [list [list a b c] [list d e f] [list g h i]]
lset x {2 1} j									#			? {a b c} {d e f} {g j i}
assert $x {eq $_ {{a b c} {d e f} {g j i}}} -v -line #__LINE__

puts "Line: "#__LINE__
set x [list [list a b c] [list d e f] [list g h i]]
lset x 2 1 j									#			? {a b c} {d e f} {g j i}
assert $x {eq $_ {{a b c} {d e f} {g j i}}} -v -line #__LINE__


#/*
# ERROR TRAP 
# FIX allow recursive block to return error codes so [try/catch] error handlers will work
puts "Line: "#__LINE__
textcolor red
puts "Test erroneous range list"
textcolor
set x [list [list a b c] [list d e f] [list g h i]]
catch {
	lset x {2 3} j							#			? list index out of range
}
# */


###########################################################################
puts "Line: "#__LINE__
set x {0 {0 {0 {0 1 2 3 4 5}} 1 2 3} 1 2 3}
lset x {1 1 1 3} 23
puts $x
assert $x {eq $_ "0 {0 {0 {0 1 2 23 4 5}} 1 2 3} 1 2 3"} -v -line #__LINE__

puts "Line: "#__LINE__
set x {0 {0 {0 {0 {0 1 2 3 4 5} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}
lset x {1 1 1 1 3} 23
assert $x {eq $_ {0 {0 {0 {0 {0 1 2 23 4 5} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x {1 1 1 1 4} 24
assert $x {eq $_ {0 {0 {0 {0 {0 1 2 23 24 5} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x {1 1 1 1 5} 25
assert $x {eq $_ {0 {0 {0 {0 {0 1 2 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x {1 1 1 1 0} 20
assert $x {eq $_ {0 {0 {0 {0 {20 1 2 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x {1 1 1 1 1} 21
assert $x {eq $_ {0 {0 {0 {0 {20 21 2 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x {1 1 1 1 2} 22
assert $x {eq $_ {0 {0 {0 {0 {20 21 22 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x 1 1 1 1 2 22		# No braces
assert $x {eq $_ {0 {0 {0 {0 {20 21 22 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x 1 1 1 0 19		# No braces
assert $x {eq $_ {0 {0 {0 {19 {20 21 22 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x 1 1 0 18		# No braces
assert $x {eq $_ {0 {0 {18 {19 {20 21 22 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x 1 0 17		# No braces
assert $x {eq $_ {0 {17 {18 {19 {20 21 22 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
lset x 0 16		# No braces
assert $x {eq $_ {16 {17 {18 {19 {20 21 22 23 24 25} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}} -v -line #__LINE__
newline


# Setting empty elements
textcolor yellow
puts "Testing \[lset\] setting empty elements ..."
textcolor

puts "Line: "#__LINE__
lset x {}
puts "'$x'"
assert $x {eq $_ ""} -v -line #__LINE__
assert $x {eq $_ {}} -v -line #__LINE__

lset x {} {}
puts "'$x'"
assert $x {eq $_ ""} -v -line #__LINE__
assert $x {eq $_ {}} -v -line #__LINE__
newline

set x {0 {0 {0 {0 1 2 3 4 5}} 1 2 3} 1 2 3}
lset x {1 1 1 3} {}
puts $x
assert $x {eq $_ "0 {0 {0 {0 1 2 {} 4 5}} 1 2 3} 1 2 3"} -v -line #__LINE__

puts "Line: "#__LINE__
/*
# Leak test 04/Sep/2019 (no leaks)
textcolor yellow
puts "Testing for leaks ..."
textcolor
time {
	set x {0 {0 {0 {0 {0 1 2 3 4 5} {0 1 2 3}} {0 1 2 3} {0 1 2 3}} 1 2 3} 1 2 3}
	lset x {1 1 1 1 3} 23
} 10000000 -info
# */

unset i x -nocomplain

#option
#pause
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #