################################################################################
# File: 			test_macro.tcl
# Purpose:			Test cases for macro
# Author: 			M Shaw
# Language:			Ticol Tcl v1.27+
#
# Versions:
# 1.00 19/Aug/2016	First version
# 1.01 11/Jul/2017	Added first tests
# 1.02 17/Oct/2017	Added macro comment-overrun checks
# 1.03 16/Jul/2018	Added quit if TCX due to test failing
# 1.04 26/Sep/2018	Added MPP test for escaped \{ \}
# 1.05 20/Dec/2022	Regression changes for MPP now handing \# properly
################################################################################
# Notes: Add more tests
# DONE FIX Macro \# won't work with TCX files
# Start		End
# 1332		1332
################################################################################
/*
#define nosuch foo
#ifdef nosuch
## echo nosuch
#endif
#undef nosuch 
# */

set expression_state [option expression]
option expression off
set echostate [option echo]
# */

#/* # DONE FIX? (fails) - returns "78"
#define check_overrun 789# Possible overrun into THIS comment
assert check_overrun {eq $_ 789} -v -line #__LINE__
#undef check_overrun
#*/
##exit

#/*
# *** DO NOT ALTER THE COMMENTS BELOW ***
puts "Line: "#__LINE__
#define check_overrun 123           # Possible overrun into THIS comment
assert check_overrun {eq $_ 123} -v -line #__LINE__
#undef check_overrun

puts "Line: "#__LINE__
#define check_overrun 456 # Possible overrun into THIS comment
assert check_overrun {eq $_ 456} -v -line #__LINE__
#undef check_overrun

puts "Line: "#__LINE__
#define check_overrun 012 #define should_be_ignored
assert check_overrun {eq $_ 012} -v -line #__LINE__
assert should_be_ignored {eq $_ "should_be_ignored"}  -v -line #__LINE__ # Should be non-substituted literal
#undef check_overrun
# */

/*
if {info encrypted} {
	puts "Can't test \\# with TCX file"
	return -code return
} # */


# FIX 29/Aug/2019 - MPP replaces with '\' instead of '\#' ?
# FIX 20/Dec/2022 - MPP revisited. \# processing is missing in preproc_stream() - reinstated but we will keep literal #
# This fix involves replacing \# with # within the MPP processed source now
# Since the code below was implemented we also now no longer need escaped hashes within dquoted strings 
# So the test will have changed
# FIX echo this source and check MPP results
# FIX \# parsing MUST occur (83656) before #define @84411
#/*
# *** DO NOT REFORMAT THESE COMMENTS ***
# Fails with TCX
# We no longer need to escape hash characters (removed backslash - Dec 2022)
# check_escaped will resolve to #, which will resolve to "" in the interpreter, which will be set as 1 (default value)
#define check_escaped "#"# Escaped comment char which should be ignored (no whitespace separating comment)
puts "Line: "#__LINE__ 
echo 'check_escaped'
puts "Line: "#__LINE__ 
# New test
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
# Old test
#assert check_escaped {eq $_ "\#"} -v -line #__LINE__				# Is stored in escaped form
puts "check_escaped is:"check_escaped								# [puts] will unescape on printings
#undef check_escaped

puts "Line: "#__LINE__ 
#define check_escaped "#" # Escaped comment char (one space separating comment)
echo 'check_escaped'
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
puts "check_escaped is:"check_escaped								# [puts] will unescape on printings
#undef check_escaped

puts "Line: "#__LINE__ 
#define check_escaped "#"	# Escaped comment char (one tab separating comment)
echo 'check_escaped'
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
puts "check_escaped is:"check_escaped								# [puts] will unescape on printings
#undef check_escaped

# Try a particulary bad example ...
puts "Line: "#__LINE__ 
#define check_escaped "#..#" # ## Escaped # comment # char (single space separating comment)
echo check_escaped
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
#assert check_escaped {eq $_ "\#..\#"} -v -line #__LINE__				# Is stored in escaped form
#undef check_escaped
##exit

puts "Line: "#__LINE__
#define check_escaped "#"
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
puts "check_escaped is:"check_escaped								# [puts] will unescape on printings
#undef check_escaped

#define check_escaped "#" # This comment should have no effect (SPACE separator)
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
puts "check_escaped is:"check_escaped								# [puts] will unescape on printings
#undef check_escaped

#define check_escaped "#"	# This comment should have no effect (TAB separator)
assert check_escaped {eq $_ "1"} -v -line #__LINE__				# Is stored in escaped form
puts "check_escaped is:"check_escaped								# [puts] will unescape on printings
#undef check_escaped
#*/

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

#/*
puts "Line: "#__LINE__
#define nowt1
#define nowt2 {}
#define nowt3 ""
#assert {nowt1} {eq $_ ""} -v -line #__LINE__	# FAILS (not replaced) can't use "no arg" here
assert nowt2 {eq $_ ""} -v -line #__LINE__		# PASS (is replaced)
# assert nowt3 {eq $_ ""} -v -line #__LINE__	# FAILS (not replaced)

#define q 1
assert q {== $_ 1} -v -line #__LINE__

#define str1 Hello
assert str1 {== $_ Hello} -v -line #__LINE__

#define str2 "Hello world"
assert {str2} {== $_ "Hello world"} -v -line #__LINE__

# Test for correct brace partity handling for quoted and unquoted escaped braces
puts "Line: "#__LINE__
undef foo -nocomplain
set i 0
proc foo x {
	puts "Enter proc foo. Test escaped braces"
	dummy \{
	dummy \}
	puts "\{"
	puts "\}"
	set ::i 1
}
newline
foo 

undef foo -nocomplain

# Date Macro
set s #__DATE__
puts "__DATE__ is '$s'"
assert $s {ne $_ ""} -v -line #__LINE__

# Time Macro
set s #__TIME__
puts "__TIME__ is '$s'"
assert $s {ne $_ ""} -v -line #__LINE__

# User Macro
set s #__USER__
puts "__USER__ is '$s'"
assert $s {ne $_ ""} -v -line #__LINE__

# Host Macro
set s #__HOST__
puts "__HOST__ is '$s'"
assert $s {ne $_ ""} -v -line #__LINE__

# File Macro
set s #__FILE__
puts "__FILE__ is '$s'"
assert $s {ne $_ ""} -v -line #__LINE__



puts "Line: "#__LINE__
assert $i {== $_ 1} -v -line #__LINE__
unset i
# */

###############################################################

#/*
puts "Line: "#__LINE__
option echo $echostate
option expression $expression_state

unset echostate expression_state -nocomplain
# */

unset s -nocomplain

vars
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #