###################################################################################
# File: 			test_macro_consts.tcl
# Purpose:			Test file for macro consts
# Author: 			M Shaw
# Language:			Ticol Tcl v1.24+
#
# Versions:
# 1.01 07/Oct/2017	First version
###################################################################################
# Notes: 
# Note that __FILE__ won't be defined from scripts chained via [eval], only [run]
###################################################################################

textcolor yellow
puts "Testing test_macro_consts.tcl"
textcolor
newline

# Note: If embedded then there will be no script name so #__FILE__ and [info script] will fail

if {eq [info script] ""} {
	puts "\[info script\] is empty. Skipping test (possibly running embedded)"
	return
}

puts "Macro \#__FILE__ is: "#__FILE__
puts "Checking #__FILE__ is not NULL"
set filename ""#__FILE__
assert $filename {ne $_ ""} -v -line #__LINE__

set line #__LINE__
puts "Checking #__LINE__ is not NULL"
assert $line {ne $_ ""} -v -line #__LINE__
assert $line {> $_ 0} -v -line #__LINE__

set cdate #__DATE__
puts "Checking #__DATE__ is not NULL"
assert $cdate {ne $_ ""} -v -line #__LINE__
puts "cdate is $cdate v's [date -separator {}]"
assert $cdate {eq $_ [date -separator 0]}  -v -line #__LINE__

set ctime #__TIME__
puts "Checking #__TIME__ is not NULL"
puts "ctime is $ctime"
assert $ctime {ne $_ ""} -v -line #__LINE__
assert $ctime {> $_ 0}  -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # #

# Man example
newline

# Note. Set with macro const is peculiar in that if the macro const is NULL 
# then the result will be [set varname], which is perfectly lawful
# For embedded Ticol __FILE__ will be defined as unknown.tcl or internal.tcl

#ifdef __LINE__
set line  #__LINE__
#endif
assert [is_set line] {== $_ 1} -v -line #__LINE__
#ifdef __FILE__
set file #__FILE__
assert [is_set file] {== $_ 1} -v -line #__LINE__
#endif
#ifdef __DATE__
set cdate #__DATE__
#endif
assert [is_set cdate] {== $_ 1} -v -line #__LINE__
#ifdef __TIME__
set ctime #__TIME__
#endif
assert [is_set ctime] {== $_ 1} -v -line #__LINE__


textcolor yellow
if {is_set file} {
	puts "In script $file"
}
puts "At line $line, date is $cdate at $ctime"
textcolor

unset cdate ctime file filename line -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # #
vars 

newline
textcolor green
puts "* Done *"
textcolor

# EOF #