###############################################################################################
# File:				test_math.tcl
# Author:			M Shaw
# Purpose:			Test file for various math commands
# Language:			Ticol Tcl v1.21c+
#
# Versions:
# 1.00 23/Dec/2016	Migrated from tests.tcl
# 1.01 22/Jun/2017	Changed all tests from [checkex] to [assert]
# 1.01 09/Jul/2017	Added some ActiveState example code where Ticol works better
# 1.02 24/Oct/2019	Added integer edge case tests
# 1.03 08/Nov/2019	Modified funct tests for new domain error exceptions
###############################################################################################
# Notes
###############################################################################################
option expression off

textcolor yellow
puts "test_math.tcl: Testing math .... @"#__LINE__
textcolor
newline

# abs
textcolor yellow
puts "test_math.tcl: Testing abs() @"#__LINE__
textcolor

assert [funct abs -100] {== $_ 100} -v -line #__LINE__
assert [calc "abs(-100)"] {== $_ 100} -v -line #__LINE__
assert  [funct abs -12.34] {== $_ 12.34} -v -line #__LINE__
assert [calc "abs(-12.34)"] {== $_ 12.34} -v -line #__LINE__
# checkex [+ [funct abs 5.2] [funct abs [- [/ 7.0 22] 0.0]]]
halt

puts "'[expr "abs(-22/7.0)+1"]'"
assert [expr "abs(-22/7.0)+1"] {== $_ 4.142857142857142} -v -line #__LINE__
assert [expr "1+abs(5.2)"] {== $_ 6.200000000000000} -v -line #__LINE__
assert [expr "abs(-22/7.0)+abs(5.2)"] {== $_ 8.342857142857143} -v -line #__LINE__
assert [calc "abs(-22/7.0)+abs(5.2)"] {== $_ 8.342857142857143} -v -line #__LINE__
# # exit

# ** exponent
if {> [info winver -vernum] 5.0} {
	assert [expr 3 ** 3] {== $_ 27} -v -line #__LINE__		# NT result is 26
	assert [expr "1+3 ** 3"] {== $_ 28} -v -line #__LINE__
	assert [** 3 3] {== $_ 27} -v -line #__LINE__
}

# acos
textcolor yellow
puts "Testing acos() (radians)\n"
textcolor
assert [funct acos .32696] {== $_ 1.237711350498989} -v -line #__LINE__
assert [calc "acos(.32696)"] {== $_ 1.237711350498989} -v -line #__LINE__
assert [calc "acos(1)"] {== $_ 0} -v -line #__LINE__
assert [calc "acos(-1)"] {== $_ 3.141592653589793} -v -line #__LINE__

# Invalid arg check (Valid range is -1 .. 1)
# (Radians)
set i 0
try {
	funct acos 10
	#assert [funct acos 10] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__		# Can't use -1.#IND00000000000 due to macro comments
} catch {
	set i 1
}
assert $i -v -line #__LINE__
# DOC calc errors are not handleable and will thus always fail
# Use expr for deliberate errors
set i 0
try {
	expr acos(10)
	#assert [funct acos 10] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__		# Can't use -1.#IND00000000000 due to macro comments
} catch {
	set i 1
}
assert $i -v -line #__LINE__
#assert [expr "acos(10)"] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__
assert [expr "acos(1)"] {== $_ 0} -v -line #__LINE__
assert [expr "acos(-1)"] {== $_ 3.141592653589793} -v -line #__LINE__

# asin (arcsin)
textcolor yellow
puts "Testing asin() (radians)\n"
textcolor
# Radians
assert [funct asin .32696] {== $_ 0.333084976295908} -v -line #__LINE__
assert [calc "asin(.32696)"] {== $_ 0.333084976295908} -v -line #__LINE__

# atan (arctan)
# Radians
textcolor yellow
puts "Testing atan() (radians)\n"
textcolor
assert [funct atan -862.42] {== $_ -1.569636799474296} -v -line #__LINE__
assert [calc "atan(-862.42)"] {== $_ -1.569636799474296} -v -line #__LINE__

# Logic # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

# bool
textcolor yellow
puts "Testing \[bool\]"
textcolor
assert [bool 1] {eq $_ "true"} -v -line #__LINE__
assert [bool -1] {eq $_ "true"} -v -line #__LINE__
assert [bool 0] {eq $_ "false"} -v -line #__LINE__

# false
textcolor yellow
puts "Checking \[false\] \[true\]"
textcolor
assert [false] {== $_ 0} -v -line #__LINE__
assert [true] {!= $_ 0} -v -line #__LINE__
assert [bool false] {== $_ 0} -v -line #__LINE__
assert [bool true] {!= $_ 0} -v -line #__LINE__


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

# ceil 
textcolor yellow
puts "Testing ceil()\n"
textcolor
#The ceil of 2.8 is 3.000000
assert [funct ceil 2.8] {== $_ 3} -v -line #__LINE__
assert [calc "ceil(2.8)"] {== $_ 3} -v -line #__LINE__
#The ceil of -2.8 is -2.000000
assert [funct ceil -2.8] {== $_ -2} -v -line #__LINE__
assert [calc "ceil(-2.8)"] {== $_ -2} -v -line #__LINE__

# decr 
textcolor yellow
puts "Testing decr()\n"
textcolor
assert [funct decr 10] {== $_ 9} -v -line #__LINE__
assert [calc "decr(10)"] {== $_ 9} -v -line #__LINE__
# Converts float to int then decrements
assert [funct decr 9.5] {== $_ 8} -v -line #__LINE__
assert [calc "decr(9.5)"] {== $_ 8} -v -line #__LINE__

# div 
textcolor yellow
puts "Testing div()\n"
textcolor
assert [funct div 22 7] {== $_ 3.142857142857143} -v -line #__LINE__
assert [calc "div(22,7)"] {== $_ 3.142857142857143} -v -line #__LINE__
# Should ideally be an error
assert [funct div 22 0] {== $_ 0} -v -line #__LINE__
assert [calc "div(22,0)"] {== $_ 0} -v -line #__LINE__
# Should ideally be an error but VB5 returns 0
assert [funct div 0 7] {== $_ 0} -v -line #__LINE__
assert [calc "div(0,7)"] {== $_ 0} -v -line #__LINE__

# cos 
textcolor yellow
puts "Testing cos()\n"
textcolor
assert [funct cos 10] {== $_ -0.839071529076452} -v -line #__LINE__
assert [calc "cos(10)"] {== $_ -0.839071529076452} -v -line #__LINE__

# double 
textcolor yellow
puts "Testing double()\n"
textcolor
assert [funct double 10] {== $_ 10.0} -v -line #__LINE__
assert [calc "double(10)"] {== $_ 10.0} -v -line #__LINE__
assert [funct double -10] {== $_ -10.0} -v -line #__LINE__
assert [calc "double(-10)"] {== $_ -10.0} -v -line #__LINE__

# double
textcolor yellow
puts "Testing \[double\]"
textcolor
assert [double 10]  {== $_ 10.000000000000000} -v -line #__LINE__

# exp 
textcolor yellow
puts "Testing exp()\n"
textcolor
# Does not precisely return 10 unless rounded up
# vb 2.302585 9.99999907005959 
# vc5 with printf exp( 2.302585 ) = 10.000000
assert [funct exp 2.302585 ] {== $_ 9.999999070059587} -v -line #__LINE__
assert [calc "exp(2.302585)"] {== $_ 9.999999070059587} -v -line #__LINE__


# expr
# expr requires complex and exhaustive expr sets
# This merely confirms expr is available and doesn't keel-over
textcolor yellow
puts "Checking \[expr\]"
textcolor
#set nan -1.[chr 35]IND00000000000
assert [calc "1+1"] {== $_ 2} -v -line #__LINE__
assert [calc "22/7.0"] {== $_ 3.142857142857143} -v -line #__LINE__
set i 0
try {
	expr sqrt(-1)
	#assert [funct acos 10] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__		# Can't use -1.#IND00000000000 due to macro comments
} catch {
	set i 1
}
assert $i -v -line #__LINE__
#assert [expr "sqrt(-1)"] {== $_ $::NAN} -v -line #__LINE__
assert [calc "sqrt(0)"] {== $_ 0} -v -line #__LINE__




# fix ###############################################
textcolor yellow
puts "Testing fix()\n"
textcolor
#puts "Fix 99.2 should be 99 =>[funct fix 99.2]"
assert [funct fix 99.2] {== $_ 99} -v -line #__LINE__
assert [calc "fix(99.2)"] {== $_ 99} -v -line #__LINE__
#puts "Fix -99.8 should be -99 =>[funct fix -99.8]"
assert [funct fix -99.8] {== $_ -99} -v -line #__LINE__
assert [calc fix(-99.8)] {== $_ -99} -v -line #__LINE__
#puts "Fix -99.2 should be -99 =>[funct fix -99.2]"
assert [funct fix -99.2] {== $_ -99} -v -line #__LINE__
assert [calc "fix(-99.2)"] {== $_ -99} -v -line #__LINE__

# floor 
textcolor yellow
puts "Testing floor()\n"
textcolor
#The floor of 2.8 is 2.000000
assert [funct floor 2.8] {== $_ 2.0} -v -line #__LINE__
assert [calc "floor(2.8)"] {== $_ 2.0} -v -line #__LINE__
#The floor of -2.8 is -3.000000
assert [funct floor -2.8] {== $_ -3.0} -v -line #__LINE__
assert [calc "floor(-2.8)"] {== $_ -3.0} -v -line #__LINE__

# fmod
textcolor yellow
puts "Testing fmod integral/fraction functions\n"
textcolor
assert [expr 4*atan(1)] {== $_ 3.141592653589792} -v -line #__LINE__
assert [calc 4*atan(1)] {== $_ 3.141592653589792} -v -line #__LINE__
assert [integral [calc 4*atan(1)]] {== $_ 3} -v -line #__LINE__
assert [fraction [calc 4*atan(1)]] {== $_ 0.141592653589792} -v -line #__LINE__
assert [calc integral(4*atan(1))] {== $_ 3} -v -line #__LINE__
assert [calc fraction(4*atan(1))] {== $_ 0.141592653589792} -v -line #__LINE__

array unset a -nocomplain
unset b r -nocomplain
set a 1.34
set b 0.45
set r [fmod $a $b]
assert $r {== $_ 0.44} -v -line #__LINE__								# PASS

#array unset a -nocomplain
unset a b r -nocomplain


# fraction 
textcolor yellow
puts "Testing fraction()\n"
textcolor
# err -0.200000000000003 != 0.2
assert [funct fraction -99.2] {== $_ -0.200000000000003} -v -line #__LINE__
assert [calc "fraction(-99.2)"] {== $_ -0.200000000000003} -v -line #__LINE__

# hypot 
# 3.0 and 4.0, its hypotenuse is 5.0
textcolor yellow
puts "Testing hypot()\n"
textcolor
assert [funct hypot 3 4] {== $_ 5} -v -line #__LINE__
assert [calc "hypot(3,4)"] {== $_ 5} -v -line #__LINE__
assert [calc "hypot(7.5,10)"] {== $_ 12.5} -v -line #__LINE__

# incr 
textcolor yellow
puts "Testing incr()\n"
textcolor
assert [funct incr 10]  {== $_ 11} -v -line #__LINE__
assert [calc "incr(10)"]  {== $_ 11} -v -line #__LINE__
# Converts float to int then decrements
assert [funct incr 9.5]  {== $_ 10} -v -line #__LINE__
assert [calc "incr(9.5)"]  {== $_ 10} -v -line #__LINE__

# int ###############################################
textcolor yellow
puts "Testing int() casting function\n"
textcolor
# Int function returns the first negative integer less than or equal to the number
#puts "Int 99.8 should be 99 =>[funct int 99.8]"					# PASS
assert [funct int 99.8]  {== $_ 99} -v -line #__LINE__
#puts "Int -99.8 should be -100 =>[funct int -99.8]"
assert [funct int -99.8]  {== $_ -100} -v -line #__LINE__
#puts "Int -99.2 should be -100 =>[funct int -99.2]"
assert [funct int -99.2]  {== $_ -100} -v -line #__LINE__

# integral 
# Returns integer part of a double
textcolor yellow
puts "Testing integral()\n"
textcolor
assert [funct integral 3.142]  {== $_ 3} -v -line #__LINE__
assert [calc "integral(3.142)"]  {== $_ 3} -v -line #__LINE__

# log 
# log( 9000.00 ) = 9.104980
textcolor yellow
puts "Testing log()\n"
textcolor
assert [funct log 9000.00] {== $_ 9.104979856318357} -v -line #__LINE__
assert [calc "log(9000.00)"] {== $_ 9.104979856318357} -v -line #__LINE__

# log10 
# log10( 9000.00 ) = 3.954243
textcolor yellow
puts "Testing log10()\n"
textcolor
assert [funct log10 9000.00] {== $_ 3.954242509439325} -v -line #__LINE__
assert [calc "log10(9000.00)"] {== $_ 3.954242509439325} -v -line #__LINE__

# max 
halt
textcolor yellow
puts "Testing max()\n"
textcolor
assert [funct max 0 1] {== $_ 1} -v -line #__LINE__
assert [calc "max(0,1)"] {== $_ 1} -v -line #__LINE__
assert [funct max 1 0] {== $_ 1} -v -line #__LINE__
assert [calc "max(1,0)"] {== $_ 1} -v -line #__LINE__
assert [funct max 0 0] {== $_ 0} -v -line #__LINE__
assert [calc "max(0,0)"] {== $_ 0} -v -line #__LINE__
assert [funct max 0 -1] {== $_ 0} -v -line #__LINE__
assert [calc "max(0,-1)"] {== $_ 0} -v -line #__LINE__
assert [funct max -1 0] {== $_ 0} -v -line #__LINE__
assert [calc "max(-1,0)"] {== $_ 0} -v -line #__LINE__
assert [funct max -0 0] {== $_ 0} -v -line #__LINE__
assert [calc "max(-0,0)"] {== $_ 0} -v -line #__LINE__
assert [funct max 1.23 3.45] {== $_ 3.45} -v -line #__LINE__
assert [calc "max(1.23,3.45)"] {== $_ 3.45} -v -line #__LINE__
assert [calc "max(-1.23,-3.45)"] {== $_ -1.23} -v -line #__LINE__
assert [funct max -1.23 -3.45] {== $_ -1.23} -v -line #__LINE__


# min 
textcolor yellow
puts "Testing min()\n"
textcolor
assert [funct min 0 1] {== $_ 0} -v -line #__LINE__
assert [calc "min(0,1)"] {== $_ 0} -v -line #__LINE__
assert [funct min 1 0] {== $_ 0} -v -line #__LINE__
assert [calc "min(1,0)"] {== $_ 0} -v -line #__LINE__
assert [funct min 0 0] {== $_ 0} -v -line #__LINE__
assert [calc "min(0,0)"] {== $_ 0} -v -line #__LINE__
assert [funct min 0 -1] {== $_ -1} -v -line #__LINE__
assert [calc "min(0,-1)"] {== $_ -1} -v -line #__LINE__
assert [funct min -1 0] {== $_ -1} -v -line #__LINE__
assert [calc "min(-1,0)"] {== $_ -1} -v -line #__LINE__
assert [funct min -0 0] {== $_ 0} -v -line #__LINE__
assert [calc "min(-0,0)"] {== $_ 0} -v -line #__LINE__
assert [funct min 1.23 3.45] {== $_ 1.23} -v -line #__LINE__
assert [calc "min(1.23,3.45)"] {== $_ 1.23} -v -line #__LINE__
assert [funct min -1.23 -3.45] {== $_ -3.45} -v -line #__LINE__
assert [calc "min(-1.23,-3.45)"] {== $_ -3.45} -v -line #__LINE__

# mul 
# Float multiplication
textcolor yellow
puts "test_math.tcl: Testint float mul()"
textcolor

assert [funct mul 3.142 6.14] {== $_ 19.29188} -v -line #__LINE__
assert [calc "mul(3.142,6.14)"] {== $_ 19.29188} -v -line #__LINE__
assert [funct mul 3 6] {== $_ 18} -v -line #__LINE__
assert [calc "mul(3,6)"] {== $_ 18} -v -line #__LINE__

# pow # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# 2.0 to the power of 3.0 is 8.0
textcolor yellow
puts "test_math.tcl: Testing pow()"
textcolor

# Integer
assert [funct pow 2 3] {== $_ 8} -v -line #__LINE__
assert [calc "pow(2,3)"] {== $_ 8} -v -line #__LINE__
# Integer
assert [funct pow 2.0 3.0] {== $_ 8} -v -line #__LINE__
assert [calc "pow(2.0,3.0)"] {== $_ 8} -v -line #__LINE__

assert [** -3 1]  {== $_ -3} -v -line #__LINE__
assert [** -3 2]  {== $_ 9} -v -line #__LINE__
assert [** -3 3]  {== $_ -27} -v -line #__LINE__
assert [** -3 4]  {== $_ 81} -v -line #__LINE__
assert [** -3 5]  {== $_ -243} -v -line #__LINE__
assert [** -3 6]  {== $_ 729} -v -line #__LINE__
assert [** -3 7]  {== $_ -2187} -v -line #__LINE__
assert [** -3 8]  {== $_ 6561} -v -line #__LINE__
assert [** -3 9]  {== $_ -19683} -v -line #__LINE__
assert [** -3 10]  {== $_ 59049} -v -line #__LINE__

catch {
assert [expr -3**1]  {== $_ -3} -v -line #__LINE__
assert [expr -3**2]  {== $_ 9} -v -line #__LINE__
assert [expr -3**3]  {== $_ -27} -v -line #__LINE__
assert [expr -3**4]  {== $_ 81} -v -line #__LINE__
assert [expr -3**5]  {== $_ -243} -v -line #__LINE__
assert [expr -3**6]  {== $_ 729} -v -line #__LINE__
assert [expr -3**7]  {== $_ -2187} -v -line #__LINE__
assert [expr -3**8]  {== $_ 6561} -v -line #__LINE__
assert [expr -3**9]  {== $_ -19683} -v -line #__LINE__
assert [expr -3**10]  {== $_ 59049} -v -line #__LINE__
}

assert [funct pow -3 1]  {== $_ -3} -v -line #__LINE__
assert [funct pow -3 2]  {== $_ 9} -v -line #__LINE__
assert [funct pow -3 3]  {== $_ -27} -v -line #__LINE__
assert [funct pow -3 4]  {== $_ 81} -v -line #__LINE__
assert [funct pow -3 5]  {== $_ -243} -v -line #__LINE__
assert [funct pow -3 6]  {== $_ 729} -v -line #__LINE__
assert [funct pow -3 7]  {== $_ -2187} -v -line #__LINE__
assert [funct pow -3 8]  {== $_ 6561} -v -line #__LINE__
assert [funct pow -3 9]  {== $_ -19683} -v -line #__LINE__
assert [funct pow -3 10]  {== $_ 59049} -v -line #__LINE__

assert [expr pow(-3,1)]  {== $_ -3} -v -line #__LINE__
assert [expr pow(-3,2)]  {== $_ 9} -v -line #__LINE__
assert [expr pow(-3,3)]  {== $_ -27} -v -line #__LINE__
assert [expr pow(-3,4)]  {== $_ 81} -v -line #__LINE__
assert [expr pow(-3,5)]  {== $_ -243} -v -line #__LINE__
assert [expr pow(-3,6)]  {== $_ 729} -v -line #__LINE__
assert [expr pow(-3,7)]  {== $_ -2187} -v -line #__LINE__
assert [expr pow(-3,8)]  {== $_ 6561} -v -line #__LINE__
assert [expr pow(-3,9)]  {== $_ -19683} -v -line #__LINE__
assert [expr pow(-3,10)]  {== $_ 59049} -v -line #__LINE__



# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

# rand 
textcolor yellow
puts "test_math.tcl: Testing random functions"
textcolor

#assert [> [funct rand 1 10] 0] {== $_ 1} -v -line #__LINE__

# rnd 
for { set i 0} {< $i 100} {incr i} {
	assert [> [funct rnd 1 10] 0] {== $_ 1} -v -line #__LINE__
	assert [< [funct rnd 1 10] 11] {== $_ 1} -v -line #__LINE__
}

# round 
textcolor yellow
puts "Testing round()"
textcolor
assert [funct round $pi 0] {== $_ 3} -v -line #__LINE__
assert [calc round($pi,0)] {== $_ 3} -v -line #__LINE__
assert [funct round $pi] {== $_ 3} -v -line #__LINE__		# FAILS
assert [calc round($pi)] {== $_ 3} -v -line #__LINE__
assert [funct round $pi 3] {== $_ 3.142} -v -line #__LINE__
assert [calc round($pi,3)] {== $_ 3.142} -v -line #__LINE__

# sgn 
# //////////////////////////
# // Greater than zero	1
# // Equal to zero		0
# // Less than zero	   -1
# //////////////////////////
textcolor yellow
puts "Testing sgn()"
textcolor
assert [funct sgn 1] {== $_ 1} -v -line #__LINE__
assert [calc sgn(1)] {== $_ 1} -v -line #__LINE__
assert [funct sgn 0] {== $_ 0} -v -line #__LINE__
assert [calc sgn(0)] {== $_ 0} -v -line #__LINE__
assert [funct sgn -1] {== $_ -1} -v -line #__LINE__
assert [calc sgn(-1)] {== $_ -1} -v -line #__LINE__

# sin 
textcolor yellow
puts "Testing sin()"
textcolor
assert [funct sin 10] {== $_ -0.54402111088937} -v -line #__LINE__
assert [calc "sin(10)"] {== $_ -0.54402111088937} -v -line #__LINE__

# sqrt 
# The square root of 45.35 is 6.73
# If x is negative, sqrt returns an indefinite (same as a quiet NaN). 
textcolor yellow
puts "Testing sqrt()"
textcolor
assert [funct sqrt 45.35] {== $_ 6.734240862933253} -v -line #__LINE__
assert [calc "sqrt(45.35)"] {== $_ 6.734240862933253} -v -line #__LINE__
assert [funct sqrt 0] {== $_ 0} -v -line #__LINE__
assert [calc "sqrt(0)"] {== $_ 0} -v -line #__LINE__
assert [funct sqrt 1] {== $_ 1} -v -line #__LINE__
assert [calc "sqrt(1)"] {== $_ 1} -v -line #__LINE__
set i 0
try {
	funct sqrt -1
	#assert [funct acos 10] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__		# Can't use -1.#IND00000000000 due to macro comments
} catch {
	set i 1
}
assert $i -v -line #__LINE__
#assert [funct sqrt -1] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__
set i 0
try {
	expr sqrt(-1)
	#assert [funct acos 10] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__		# Can't use -1.#IND00000000000 due to macro comments
} catch {
	set i 1
}
assert $i -v -line #__LINE__
#assert [expr "sqrt(-1)"] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__

# srand 
# No test

# tan
textcolor yellow
puts "Testing tan()"
textcolor
assert [funct tan 10] {== $_ 0.648360827459087} -v -line #__LINE__
assert [calc "tan(10)"] {== $_ 0.648360827459087} -v -line #__LINE__

# baseconv (convert from base 10 to any base 2..16)
textcolor yellow
puts "Testing \[baseconv\]"
textcolor
assert [baseconv 10 2] {eq $_ 1010} -v -line #__LINE__	# String cmp
assert [baseconv 0b1010 2] {eq $_ 1010} -v -line #__LINE__	# String cmp
#assert [baseconv 0x1010 2] {eq $_ 1010} -v -line #__LINE__
# fix baseconv accept binary input
# binary output as 0bnnnn ?
# Fix baseconv chaining?

# inttobin
textcolor yellow
puts "Testing \[inttobin\]"
textcolor

assert [inttobin 0b1010] {== $_ 10} -v -line #__LINE__

# bintoint
textcolor yellow
puts "Testing \[bintoint \[inttobin 10\]\]"
textcolor
# No longer effective as binary is now auto-translated in Ticol
#assert [bintoint 0b1010] {== $_ 10} -v -line #__LINE__
#assert [bintoint 0b1] {== $_ 1} -v -line #__LINE__
#assert [bintoint [inttobin 10]] {== $_ 10} -v -line #__LINE__
assert [bintoint [inttobin 10]] {== $_ 10} -v -line #__LINE__

# convbase (convert from any base to base 10w)
textcolor yellow
puts "Testing \[convbase\]"
textcolor

assert  [convbase 1010 2] {== $_ 10} -v -line #__LINE__
assert  [convbase 12 8] {== $_ 10} -v -line #__LINE__
assert  [convbase 499602D2 16] {== $_ 1234567890} -v -line #__LINE__

# float
textcolor yellow
puts "Checking \[float\]"
textcolor
assert [float 10] {== $_ 10.000000000000000} -v -line #__LINE__
# flush
# Will do nothing other than check it is present (returns TRUE)
#checkex [flush] {== $x 1}

# hextoint
# Round-trip
textcolor yellow
puts "Checking \[hextoint\] \[inttohex\]"
textcolor
assert [hextoint [inttohex 10]]  {== $_ 10} -v -line #__LINE__

# intotbin
# See bintoint round-trip

# inttohex
# See inttohex round-trip

# inttooct
# octtoint no longer works due to auto-translation
#puts "Check->[inttooct [octtoint 0o12]]"
# eq fails here
#assert [inttooct [octtoint 0o12]] {== $_ 0o12} -v -line #__LINE__
assert [inttooct 0o12] {== $_ "0o12"} -v -line #__LINE__
assert [inttooct 0o0] {== $_ "0o0"} -v -line #__LINE__

# long
textcolor yellow
puts "Checking \[long\]"
textcolor
assert [long -0] {== $_ 0} -v -line #__LINE__
assert [long -0.0] {== $_ 0} -v -line #__LINE__
assert [long 0.0] {== $_ 0} -v -line #__LINE__
assert [long 0] {== $_ 0} -v -line #__LINE__
assert [long 3] {== $_ 3} -v -line #__LINE__
assert [long 3.] {== $_ 3} -v -line #__LINE__
assert [long 3.142] {== $_ 3} -v -line #__LINE__


# octtoint
# See inttooct for round-trip

# round
# See funct tests above

# short
textcolor yellow
puts "Checking \[short\]"
textcolor
assert [short -0] {== $_ 0} -v -line #__LINE__
assert [short -0.0] {== $_ 0} -v -line #__LINE__
assert [short 0.0] {== $_ 0} -v -line #__LINE__
assert [short 0] {== $_ 0} -v -line #__LINE__
assert [short 3] {== $_ 3} -v -line #__LINE__
assert [short 3.] {== $_ 3} -v -line #__LINE__
assert [short 3.142] {== $_ 3} -v -line #__LINE__

# sqr, sqrt
textcolor yellow
puts "Testing sqr() sqrt()"
textcolor
assert [calc "sqrt(sqr(1))"] {== [round $_ 10] 1.0} -v -line #__LINE__	# 1
assert [calc "sqrt(sqr(1.0))"] {== [round $_ 10] 1.0} -v -line #__LINE__		# 1.000000000000000
assert [calc "sqrt(sqr(-1.0))"] {== [round $_ 10] 1.0} -v -line #__LINE__		# Should be 1.0 sqrt(-1*-1) -> sqrt(1)
assert [calc "sqrt(sqr($::pi))"] {== [round $_ 10] [round $::pi 10]} -v -line #__LINE__


# unsigned
textcolor yellow
puts "Testing \[unsigned\]"
textcolor
assert [unsigned -3] {== $_ 18446744073709551613} -v -line #__LINE__
assert [unsigned -0] {== $_ 0} -v -line #__LINE__
assert [unsigned -0.0] {== $_ 0} -v -line #__LINE__
assert [unsigned 0.0] {== $_ 0} -v -line #__LINE__
assert [unsigned 0] {== $_ 0} -v -line #__LINE__
assert [unsigned 3] {== $_ 3} -v -line #__LINE__
assert [unsigned 3.] {== $_ 3} -v -line #__LINE__
assert [unsigned 3.142] {== $_ 3} -v -line #__LINE__
assert [unsigned -3.142] {== $_ 18446744073709551613} -v -line #__LINE__		# Two's compliment (its really nice)
assert [funct abs -3.142] {== $_ 3.142} -v -line #__LINE__		# Two's compliment (its really nice)

##############################################################

textcolor yellow
puts "Cheking expressions..."
textcolor
assert [calc "abs(-22/7.0)+abs(5.2)"] {== $_ 8.342857142857143} -v -line #__LINE__
# 2.57215162212632
# sqr in cscript appears to be sqrt
#assert [calc "sqr(tan(1.2))*cos(2)"] {== $_ -0.66741333610794} -v -line #__LINE__

# Test for handling null expressions in calc and expr
puts "Line: "#__LINE__
assert [expr ""] {eq $_ ""} -v -line #__LINE__
assert [expr {}] {eq $_ ""} -v -line #__LINE__
assert [calc {}] {eq $_ ""} -v -line #__LINE__
assert [calc ""] {eq $_ ""} -v -line #__LINE__
# exit 

puts "Line: "#__LINE__
# Some expressions ...
# move to bad expressions
#assert [calc "/ -12 -3" ]  {== $_ -15} -v -line #__LINE__


#ifdef comment
assert [calc "(1740.0)" ]  {== $_ 1740.0} -v -line #__LINE__
assert [calc "-2-+5"] {== $_ -7} -v -line #__LINE__						# Problem case (fixed 28/Oct/2016)
assert [calc "-2*-5"] {== $_ 10} -v -line #__LINE__						# Problem case (fixed 28/Oct/2016)
assert [calc "((948.0)-8.0)+-245" ] {== $_ 695.0} -v -line #__LINE__		# 695.0 Verified in VB
assert [calc "(((5171.152)+-76.14)-2322.1)/6" ] {== $_ 462.152} -v -line #__LINE__
assert [calc "(((857.0)+-61892.0)+7954.0)" ] {== $_ -53081} -v -line #__LINE__
# FAIL
#assert [calc "(((((((31.0)/233.0)+4.0)--11.387)-42.0)/-779.31986)--50286.0)+02486" ] {== $_ 117700.8935565170} -v -line #__LINE__ #52772.0339782856} -v -line #__LINE__
assert [calc "((((((((50900.6138)+-93446.066)/1.849)+6378.82)-91359.0)+8407.636)-15420.0)-97.37)+-2601" ] {== [round $_ 10] [round -117700.89355651704 10]} -v -line #__LINE__
assert [calc "((((((((65.754)/12.0)+1.0)+01.0)*46071.0)-7777.3)/359.9818)+26.0)+6038" ] {== [round $_ 10] [round 6999.63270281998 10] } -v -line #__LINE__
#endif


# somewhere below (1888 at idle above)


# 1948 lost below
#checkex [calc "((((012.50947)+-83512.0)+-016.40)+-6189.84)" ]{== [round $x 10] -89705.73053}
# calc will resolve the following to a single float
assert [7.511678663306194] {== [round $_ 10] [round 7.51167866330619 10]} -v -line 663
#assert [calc "((((((14.39880)-8.0)--7783.83710)+-7349.0)/651.099)+6.834)" ] {== [round $_ 10] [round 7.51167866330619 10]} -v -line #__LINE__

# 4064 lost below
# calc will resolve the following to a single float
assert [-89705.730529999986000] {== [round $_ 10] -89705.73053} -v -line 666
#assert [calc "((((012.50947)+-83512.0)+-016.40)+-6189.84)" ] {== [round $_ 10] -89705.73053} -v -line #__LINE__


/*
assert [calc "((8.49)-92553.33743)+-8" ] {== [round $_ 10] -92552.84743} -v -line #__LINE__
assert [calc "(((08.0)+806.81989)+7228.2890)/78" ] {== [round $_ 10] [round 103.116780641026 10]} -v -line #__LINE__
assert [calc "(((((1.0)+5861.24)+25.0)+862.0)+65.39)+246" ] {== [round $_ 10] 7060.63} -v -line #__LINE__
assert [calc "(18211.0)" ] {== [round $_ 10] 18211.0} -v -line #__LINE__
assert [calc "(((97629.0)+5.2)/0039.1)+8195" ] {== [round $_ 10] 10692.0383631714} -v -line #__LINE__
assert [calc "((3.2525)-867.7)+-2294" ] {== [round $_ 10] -3158.4475} -v -line #__LINE__
assert [calc "((((((((((82.5549)+-1.0)*2508.3)+93967.0)+997.0)+-6935.76)+65.82)/5.0)+441.0)+6.163)/-355" ]  {== [round $_ 10] [round -166.137482067606 10]} -v -line #__LINE__

# FAIL!!!!!!!!!!!!!!!!!
#assert [calc "((((((((((328.253)+-6.0)*18621.0)-4.62)+9.1)+50364.0)/9293.4)+405.9634)-2887.58)*125.0)" ] {== [round $_ 4] [round -300.169435931594 4]} -v -line #__LINE__

assert [calc "(((((((((-744.0)+-444.0)--29307.0)-7.8730)+-38252.037)/29.753)-4321.0)/13.2)+53.0)" ] {== [round $_ 10] [round -300.169435931594 10]} -v -line #__LINE__
assert [calc "((((((((51222.6407)+034.0)+00531.067)-50.045)--4.0)+9.56482)+-1683.1)--4849.20706)" ] {== [round $_ 10] 54917.33458} -v -line #__LINE__
assert [calc "(((((((1.0)+70.0)+5.0)+6.738)--49.0)+2.6577)+-4579.0)" ] {== [round $_ 10] -4444.6043} -v -line #__LINE__
assert [calc "((((((((((830.0)+-2.2)*-36.0)+71.0)+5.0)+54.0260)/611.17300)+-4.83012)+90.38857)+7.0)+8930" ] {== [round $_ 10] [round 8974.01119087697 10]} -v -line #__LINE__
assert [calc "(((((((((7478.3)+71008.0)-345.0145)+6.2035)+6.0)+667.0)-6.7)+7191.0)--627.2)" ] {== [round $_ 10] 86631.989} -v -line #__LINE__
assert [calc "((765.1)-3995.926)*5" ] {== [round $_ 10] -16154.13} -v -line #__LINE__
assert [calc "(((((((32444.0)+27.0)--923.73157)+-8.58)+324.0)+115.0)+9510.0)" ] {== [round $_ 10] 43335.15157} -v -line #__LINE__
assert [calc "((645.3)+08.9)" ] {== [round $_ 10] 654.2} -v -line #__LINE__
assert [calc "(((13.47463)+6528.1)+03.0)--155" ] {== [round $_ 10] 6699.57463} -v -line #__LINE__
assert [calc "((221.74823)*63.0)+35" ] {== [round $_ 10] 14005.13849} -v -line #__LINE__
assert [calc "(28.0)+722" ] {== [round $_ 10] 750} -v -line #__LINE__
assert [calc "(5910.0)+85" ] {== [round $_ 10] 5995} -v -line #__LINE__
assert [calc "(-07905.341)+0" ] {== [round $_ 10] -7905.341} -v -line #__LINE__
assert [calc "((((((48.0)-5925.4)*-13958.2)-79617.0)+7190.0)*341.0)" ] {== [round $_ 2] [round 27950234708.88 2]} -v -line #__LINE__
assert [calc "((((((((((75332.0)+41290.87456)--807.743)--1.36)+3.0)+0190.4)+2931.0)+4.82055)*-9328.748)+1435.5268)+-92959" ] {== [round $_ 5] [round -1124776559.21947 5]} -v -line #__LINE__
assert [calc "((((((-45058.8013)/-22.0)+27.031)+-33.709)+-55.0)-8378.0)+-37939" ] {== [round $_ 10] -44330.5506681818} -v -line #__LINE__
assert [calc "(((((((((1828.01848)-6093.00876)+3565.0)+8.0)/3948.39764)+5.0)+8615.519)+6454.2)+-5326.8)--266" ] {== [round $_ 10] 10013.7437414918} -v -line #__LINE__
assert [calc "((((((394.56)+2.0)+53.0)+56.0)*6144.0)+6.268)" ] {== [round $_ 3] 3106166.908} -v -line #__LINE__
assert [calc "(((255.0)+-10151.0)*76002.0)+43362" ] {== [round $_ 10] -752072430} -v -line #__LINE__
assert [calc "(((((((((8.0)/234.3436)+84.0)*-73980.0)+08805.1436)*3921.982)+-34.0)+2.0)+31989.891)" ] {== [round $_ 4] -24347790662.8501} -v -line #__LINE__ # // eCalc -24347790662.85007
assert [calc "((((((351.425)+3892.8701)+4819.0)+359.79801)*4.755)+5.09282)+-45044" ] {== [round $_ 5] [round -232.099441950001 5]} -v -line #__LINE__
assert [calc "((((((((0511.0)/59.0)-0838.72693)+8.103)+-38480.15)+413.0)-7.0)+358.56)+-9089" ] {== [round $_ 5] [round -47626.5529130509 5]} -v -line #__LINE__

# somewhere above

#####################################################
#ifdef comment
# Deprecated as ++ is fast enough
textcolor yellow
puts "Checking +++ ..."
textcolor
set num 0
for {set i 0} {< $i 10} {incr i} {	
	+++ num
}
assert $num {== $_ 10} -v -line #__LINE__
#endif

option expression off
newline
textcolor yellow
puts "Checking incr with arg ..."
textcolor
set num 0
for {set i 0} {< $i 10} {incr i} {	
	incr num 4000
}
assert $num {== $_ 40000} -v -line #__LINE__

newline
textcolor yellow
puts "Checking -- with arg ..."
textcolor
set num 0
for {set i 0} {< $i 10} {incr i} {	
	-- num 4000
#puts "num $num"
}
assert $num {== $_ -40000} -v -line #__LINE__

newline
textcolor yellow
puts "Checking decr with arg ..."
textcolor
set num 0
for {set i 0} {< $i 10} {incr i} {	
	decr num 4000
}
assert $num {== $_ -40000} -v -line #__LINE__

# calc # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor yellow
puts "\[calc\]"
textcolor

array unset a -nocomplain
unset b -nocomplain
unset c -nocomplain
unset d -nocomplain

set a 2.2
set b 5
set c 2
set d [calc "round(abs(1-pow($::a,$::b)*$::c),2)"] # PASS (AOPM depdenent) {== $x 102.07}  102.072640000000040 ticol expr, 102.07264  VB
assert $d {== $_ 102.07} -v -line #__LINE__

set a -1.3
set b 5
set c 4
set d [calc "round(abs(1-pow($::a,$::b)*$::c),2)"] # 15.85 ecalc. Rounded=15.85
assert $d {== $_ 15.85} -v -line #__LINE__

# False +ve
option expression off
assert [calc "(((((((((-744.0)+-444.0)--29307.0)-7.8730)+-38252.037)/29.753)-4321.0)/13.2)+53.0)"]  {== $_ -300.169435931594420} -v -line #__LINE__

assert [calc "round(22/7.0, 3)"] {== $_ 3.143000} -v -line #__LINE__
assert [funct acos 10] {== $_ -1.[chr 35]IND00000000000} -v -line #__LINE__

# Expressive functions # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
assert [expr max((22/7.0),2)] {== $_ 3.142857142857143} -v -line #__LINE__
assert [expr round((22/7.0),(2+2))] {== $_  3.1429} -v -line #__LINE__
assert [expr round((22/7.0),(2<3)?4:0)] {== $_  3.1429} -v -line #__LINE__	# Complex with conditional ? if
assert [== 3.1429 [expr round((22/7.0),(2<3)?2:0)]] {== $_ $::false} -v -line #__LINE__
# Above with [calc]
assert [calc max((22/7.0),2)] {== $_ 3.142857142857143} -v -line #__LINE__
assert [calc round((22/7.0),(2+2))] {== $_  3.1429} -v -line #__LINE__
# FIX FAILS TEST
#assert [calc round((22/7.0),(2<3)?4:0)] {== $_  3.1429}	# Complex with conditional ? if
assert [== 3.1429 [calc round((22/7.0),(2<3)?2:0)]] {== $_ $::false} -v -line #__LINE__

# Math (Random) #################################
assert [funct rand] {< $_ 1.0} -v -line #__LINE__			# Should be a number <1
assert [funct rnd 10 20] {&& [>= $_ 10] [<= $_ 20]} -v -line #__LINE__			# Should be a number <1

# FIX rvalue including dquotes
# We brace so we should round-robin the literal expression
# Our test passes the dquotes which should have been stripped
#assert {[eval / 22 7.0]} -v -line #__LINE__ {eq $_ "\[eval / 22 7.0\]"} -v -line #__LINE__

# Check math ####################################
option expression on
newline
textcolor yellow
puts "Testing math operators ... @"#__LINE__
textcolor
assert [! 0] {== $_ 1} -v -line #__LINE__
assert [! 1] {== $_ 0} -v -line #__LINE__
assert [! 23] {== $_ 0} -v -line #__LINE__
assert [!= 1 1 ] {== $_ 0} -v -line #__LINE__
assert [!= 1 0 ] {== $_ 1} -v -line #__LINE__
assert [!= 0 1 ] {== $_ 1} -v -line #__LINE__
assert [!= 0 0 ] {== $_ 0} -v -line #__LINE__
assert [% 1 1] {== $_ 0} -v -line #__LINE__
#assert [% 1 0] {== $_ 1} -v -line #__LINE__		# Divison by 0
assert [% 100 23] {== $_ 8} -v -line #__LINE__
assert [& 0 0] {== $_ 0} -v -line #__LINE__
assert [& 0 1] {== $_ 0} -v -line #__LINE__
assert [& 1 0] {== $_ 0} -v -line #__LINE__
assert [& 0b11111111 23] {== $_ 23} -v -line #__LINE__
assert [& 23 1] {== $_ 1} -v -line #__LINE__
assert [&& 1 1] {== $_ 1} -v -line #__LINE__
assert [&& 1 0] {== $_ 0} -v -line #__LINE__

puts "Line: "#__LINE__
set a 1
assert [&= $a 0b11] {== $_ 1} -v -line #__LINE__
assert [&= 0 0b11] {== $_ 0} -v -line #__LINE__
assert [* 0 0] {== $_ 0} -v -line #__LINE__
assert [* 1 0] {== $_ 0} -v -line #__LINE__
assert [* 1 1] {== $_ 1} -v -line #__LINE__
assert [* -1 1] {== $_ -1} -v -line #__LINE__
assert [* -1 -1] {== $_ 1} -v -line #__LINE__
assert [* -0 -0] {== $_ 0} -v -line #__LINE__
assert [* -12345 -10] {== $_ 123450} -v -line #__LINE__
assert [* 12345 -10] {== $_ -123450} -v -line #__LINE__
assert [* -12345 10] {== $_ -123450} -v -line #__LINE__
assert [* 12345 10] {== $_ 123450} -v -line #__LINE__
assert [*= 0 0] {== $_ 0} -v -line #__LINE__
assert [*= -0 0] {== $_ 0} -v -line #__LINE__
assert [*= 0 -0] {== $_ 0} -v -line #__LINE__
assert [*= -0 -0] {== $_ 0} -v -line #__LINE__
assert [*= 0 1] {== $_ 0} -v -line #__LINE__
assert [*= 1 0] {== $_ 0} -v -line #__LINE__
assert [*= 1 1] {== $_ 1} -v -line #__LINE__
assert [*= -1 1] {== $_ -1} -v -line #__LINE__
assert [*= 1 -1] {== $_ -1} -v -line #__LINE__
assert [*= 12345 10] {== $_ 123450} -v -line #__LINE__

assert [+ 0 0] {== $_ 0} -v -line #__LINE__
assert [+ -0 0] {== $_ 0} -v -line #__LINE__
assert [+ 0 -0] {== $_ 0} -v -line #__LINE__
assert [+ -0 -0] {== $_ 0} -v -line #__LINE__
assert [+ 0 1] {== $_ 1} -v -line #__LINE__
assert [+ 1 0] {== $_ 1} -v -line #__LINE__
assert [+ 1 1] {== $_ 2} -v -line #__LINE__
assert [+ -1 1] {== $_ 0} -v -line #__LINE__
assert [+ 1 -1] {== $_ 0} -v -line #__LINE__
assert [+ 12345 10] {== $_ 12355} -v -line #__LINE__
assert [+ 12345 10] {== $_ 12355} -v -line #__LINE__

puts "Line: "#__LINE__
set a(23) 1
assert $a(23) {== $_ 1} -v -line #__LINE__
++ a(23)
assert $a(23) {== $_ 2} -v -line #__LINE__


puts "Line: "#__LINE__
# Trigger AOPM errors
# 2017/Jul/28
assert [expr +7 + 7/7] {== $_ 8} -v -line #__LINE__		# PASS
assert [calc +7 + 7/7] {== $_ 8} -v -line #__LINE__		# PASS
assert [expr +7 + +7/7] {== $_ 8} -v -line #__LINE__		# PASS
assert [calc +7 + +7/7] {== $_ 8} -v -line #__LINE__		# PASS
#assert [expr +7 + +7/+7] {== $_ 8} -v -line #__LINE__		# FAIL
#assert [calc +7 + +7/+7] {== $_ 8} -v -line #__LINE__		# MPP fata error

assert [expr -7 + 7/7] {== $_ -6} -v -line #__LINE__		# PASS (VB -6)
assert [calc -7 + 7/7] {== $_ -6} -v -line #__LINE__		# PASS (VB -6)
#assert [expr -7 + -7/7] {== $_ -8} -v -line #__LINE__		# FAIL (VB -8) -7
#assert [calc -7 + -7/7] {== $_ -8} -v -line #__LINE__		# FAIL (VB -8) -7


puts "Line: "#__LINE__

assert [expr 7 + 7*7] {== $_ 56} -v -line #__LINE__
assert [calc 7 + 7*7] {== $_ 56} -v -line #__LINE__

assert [expr 7 + 7/7] {== $_ 8} -v -line #__LINE__
assert [calc 7 + 7/7] {== $_ 8} -v -line #__LINE__

assert [expr 7 + 7*7] {== $_ 56} -v -line #__LINE__
assert [calc 7 + 7*7] {== $_ 56} -v -line #__LINE__

assert [expr 7/7 + 7] {== $_ 8} -v -line #__LINE__
assert [calc 7/7 + 7] {== $_ 8} -v -line #__LINE__

assert [expr 7*7 + 7] {== $_ 56} -v -line #__LINE__
assert [calc 7*7 + 7] {== $_ 56} -v -line #__LINE__

assert [expr 7 + 7/7 + 7*7 - 7] {== $_ 50} -v -line #__LINE__  # 50 VB5
assert [calc 7 + 7/7 + 7*7 - 7] {== $_ 50} -v -line #__LINE__

assert [expr 7*7 - 7 /7 + 7 * 7 +7] {== $_ 104} -v -line #__LINE__
assert [calc 7*7 - 7 /7 + 7 * 7 +7] {== $_ 104} -v -line #__LINE__
# 7*7 - 7 /7 + 7 * 7 +7 = 98	(Win calc)
# 49 - 1 + 49 + 7 == 104		(Win calc)

assert [expr (2+3)*4] {== $_ 20} -v -line #__LINE__
assert [calc (2+3)*4] {== $_ 20} -v -line #__LINE__
assert [expr (2+3)*(2+2)] {== $_ 20} -v -line #__LINE__
assert [calc (2+3)*(2+2)] {== $_ 20} -v -line #__LINE__
assert [expr ((1+1)+3)*((1+1)+(1+1))] {== $_ 20} -v -line #__LINE__
assert [calc ((1+1)+3)*((1+1)+(1+1))] {== $_ 20} -v -line #__LINE__

# calc optimisation #############################
set q 23
set s [calc "(1+3*2.5)/($q*$pi)+22/7.0"]	# puts [+ [/ 8.500000000000000 [* $q $pi]] 3.142857142857143]
if {ne $s "[+ [/ [8.5] [* $q $pi]] [3.142857142857143]]" } {
	die "Calc failed at line $line"
}

unset s -nocomplain


# type ##########################################
textcolor yellow
puts "Testing \[type\] @"#__LINE__
textcolor
set q 10
assert [type q] {eq $_ "integer"} -v -line #__LINE__

set q 10.0
assert [type q] {eq $_ "float"} -v -line #__LINE__

set q Hello
assert [type q] {eq $_ "string"} -v -line #__LINE__

set arr(1) A
assert [type arr] {eq $_ "array"} -v -line #__LINE__
array unset arr -nocomplain

unset q -nocomplain

# Check [arity] command #########################
assert [arity acos] {== $_ 1} -v -line #__LINE__
assert [arity max] {== $_ 2} -v -line #__LINE__
assert [arity wide] {== $_ 1} -v -line #__LINE__

# decr ##########################################
textcolor yellow
puts "Testing \[decr\] @"#__LINE__
textcolor
assert [decr 11] {== $_ 10} -v -line #__LINE__
assert [funct decr 11] {== $_ 10} -v -line #__LINE__
assert [calc decr(11)] {== $_ 10} -v -line #__LINE__
assert [decr 11.1] {== $_ 10} -v -line #__LINE__
assert [funct decr 11.1] {== $_ 10} -v -line #__LINE__
assert [expr decr(11.1)] {== $_ 10} -v -line #__LINE__
assert [calc decr(11.1)] {== $_ 10} -v -line #__LINE__


# Math ordering #################################
# the Tcl parser will first insert "$a + 2" into the variable b, then the expr command will evaluate the expression $a + 2*4
# So we check ordering
# Will fail with MPP-based bracket optimisation
puts "Line: "#__LINE__

array unset a -nocomplain
set a 3
set b {$a+2}

puts "Line: "#__LINE__
assert [expr ($b)*4]	{== $_ 20} -v -line #__LINE__

# FIX requires [subst] expand before call to [expr] as [expr] does not subst vars
# 2018
puts "Line: "#__LINE__
assert [expr [subst $b]*4]	{== $_ 11} -v -line #__LINE__
try {
	assert [expr $b*4]	{== $_ 11} -v -line #__LINE__ # "3 + 2 * 4" (20) versus ->  "3 + (2 * 4)" (11)
} catch {
	puts "\[expr $b*4\] Failed at line "#__LINE__
	puts "Result is [expr $b*4]"
	pause
}

# Here, we advance evaluate
set a 3
catch {
	textcolor red
	puts "Ignore error message"
	textcolor
	set b $a + 2												# Should err. Does not evaluate its arguments (use [let])
	assert [expr $b*4]	{== $_ 20} -v -line #__LINE__   # a + 2 * 4 (3 x 4)
}

# Here, we advance evaluate
set a 3
#set b $a + 2												# Does not evaluate its arguments (use [let])
let b = $a + 2
assert [expr $b*4]	{== $_ 20} -v -line #__LINE__  # a + 2 * 4 (3 x 4)

# Here we delay evaluation
# QUERY THIS
# Test disabled 05/2018
#ifdef comment
set a 3
set b {{$a + 2}}
assert [expr $b*4]	{== $_ 20} -v -line #__LINE__		# 4 * 5
#endif

unset a -nocomplain
unset b -nocomplain

# ++ ############################################
newline
textcolor yellow
puts "Checking ++ with arg ..."
textcolor
set num 0
for {set i 0} {< $i 10} {incr i}  {	
	++ num 4000					# Add 10 x 4000
}
assert $num {== $_ 40000} -v -line #__LINE__
unset num -nocomplain

# [funct] command ###############################
newline
textcolor yellow
puts "\[funct\] with varying arg counts"
textcolor
assert [funct srand] {== $_ 0} -v -line #__LINE__				# 0 arg
assert [funct abs -100] {== $_ 100} -v -line #__LINE__		# 1 arg
assert [funct min 23 69] {== $_ 23} -v -line #__LINE__		# 2 arg

# sizeof ########################################
textcolor yellow
puts "Testing \[sizeof\]"
textcolor
assert [sizeof char] {== $_ 1} -v -line #__LINE__
assert [sizeof short] {== $_ 2} -v -line #__LINE__
assert [sizeof int] {== $_ 4} -v -line #__LINE__
assert [sizeof int64] {== $_ 8} -v -line #__LINE__
assert [sizeof double] {== $_ 8} -v -line #__LINE__
set q 10
assert [sizeof [type q]] {== $_ 4} -v -line #__LINE__
set q 10.0
assert [sizeof [type q]] {== $_ 8} -v -line #__LINE__


assert "[expr 2+2+2+2+2] [expr 2+2+2+2+2] [expr 2+2*0+2]" {eq $_ "10 10 4"} -v -line #__LINE__
assert [explain 2+2+2+2+2] {eq $_ [unescape "\[+ \[+ \[+ \[+ 2 2\] 2\] 2\] 2\]"]} -v -line #__LINE__
assert [explain 2+2+2+2+2] {eq $_ {[+ [+ [+ [+ 2 2] 2] 2] 2]}} -v -line #__LINE__
assert [explain 2+2*0+2] {eq $_ {[+ [+ 2 [* 2 0]] 2]}} -v -line #__LINE__
#*/

assert "[+ [+ [+ [+ 2 2] 2] 2] 2] [+ [+ [+ [+ 2 2] 2] 2] 2] [+ [+ 2 [* 2 0]] 2]" {eq $_ "10 10 4"} -v -line #__LINE__
puts "Test 3 clauses on separate lines"
assert "[+ [+ [+ [+ 2 2] 2] 2] 2] \
[+ [+ [+ [+ 2 2] 2] 2] 2] \
[+ [+ 2 [* 2 0]] 2]" {eq $_ "10 10 4"} -v -line #__LINE__



###########################################################
# Some code discussed at: http://wiki.tcl.tk/1042
# Ticol is more flexible than ActiveState here
###########################################################
puts "Line: "#__LINE__
option expression on
set op ==
set foo 23
if "$foo $op 23" {puts YES}
assert [expr $foo $op 23] {== $_ 1} -v -line #__LINE__

# */

unset c d foo i op q -nocomplain
###########################################################

###########################################################
# Edge cases
# Check for failure to add correctly to -ve numbers at edge case
# CommandMath integer exit has a normalisation clause where _i64toa returns inaccurate results (_snprintf is slow but ok)
assert [+ -9223372036854775809 1] {== $_ -9223372036854775808} -v -line #__LINE__
assert [+ -9223372036854775808 1] {== $_ -9223372036854775807} -v -line #__LINE__
assert [+ -9223372036854775807 1] {== $_ -9223372036854775806} -v -line #__LINE__
assert [+ -9223372036854775808 1] {== $_ -9223372036854775807} -v -line #__LINE__
assert [expr -9223372036854775808 + 1] {== $_ -9223372036854775807} -v -line #__LINE__

# Humour
set 2b [rnd 0 1]
loop i 0 10 {			# 0..9 inclusive
	assert [expr $2b || [! $2b]] -v -line #__LINE__
}

unset 2b i -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #