###########################################################################
# Name:				test_umax_increment.tcl
# Purpose:			Test the effect of numeric increments beyond *_MAX
# Author:			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 18/Jun/2018	First version
# 1.01 07/Jun/2022	Revised for changes to [loop] bounds
# 1.02 28/Dec/2022	Reverted to loop bounds which are compatible with TclX->[loop]
###########################################################################
# Notes:
# Looking at what would happen if an iter operation increments beyond *_MAX
# PASS [++]
# PASS [+ $q 1]
# PASS [+++]
# UINT wraps around to 0
# INT negates and counts down (two's compliment)
###########################################################################

option expression off
option echo on

textcolor yellow
puts "Integer overflow tests"
textcolor
newline 

unset UNSIGNED_MAX -nocomplain -const
unset INT_MAX -nocomplain -const

puts "Line: "#__LINE__
if {! [info exists UNSIGNED_MAX]} {
	set UNSIGNED_MAX 18446744073709551615
}
if {! [info exists INT_MAX]} {
	set INT_MAX    9223372036854775807
}

puts "Line: "#__LINE__
set q [- $INT_MAX 10]

# Signed test # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
textcolor yellow
puts "Signed overflow test ..."
textcolor
newline 

puts "\$INT_MAX == $INT_MAX"
puts "q: $q"

loop i 1 21 {
	++ q
	puts "$i: q is $q"
}

assert $q {== $_ -9223372036854775799} -v -line #__LINE__

/*
1: q is 9223372036854775798
2: q is 9223372036854775799
3: q is 9223372036854775800
4: q is 9223372036854775801
5: q is 9223372036854775802
6: q is 9223372036854775803
7: q is 9223372036854775804
8: q is 9223372036854775805
9: q is 9223372036854775806
10: q is 9223372036854775807	# MAX
11: q is -9223372036854775808
12: q is -9223372036854775807
13: q is -9223372036854775806
14: q is -9223372036854775805
15: q is -9223372036854775804
16: q is -9223372036854775803
17: q is -9223372036854775802
18: q is -9223372036854775801
19: q is -9223372036854775800
20: q is -9223372036854775799
*/

# Unsigned test # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
textcolor yellow
puts "Unsigned overflow test ..."
textcolor

puts "\$UNSIGNED_MAX == $UNSIGNED_MAX"
#set q [- $UNSIGNED_MAX 10]
#set q 18446744073709551615
set UNSIGNED_MAX 18446744073709551615
set q $UNSIGNED_MAX
puts "q: $q"
loop i 1 21 {
	++ q						# PASS
	puts "$i: q is $q"
}
assert $q {== $_ 19} -v -line #__LINE__


set q $UNSIGNED_MAX
puts "q: $q"
loop i 1 21 {
	set q [+ $q 1]				# PASS
	puts "$i: q is $q"
}
assert $q {== $_ 19} -v -line #__LINE__

set q $UNSIGNED_MAX
puts "q: $q"
loop i 1 21 {
	+++ q						# PASS
	puts "$i: q is $q"
}
assert $q {== $_ 19} -v -line #__LINE__

unset i q UNSIGNED_MAX INT_MAX -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

newline
textcolor green
puts "* Done *"
textcolor

# EOF #