#################################################################################################
# File:				test_md5.tcl
# Author:			M Shaw
# Purpose:			Test file for MD5 commands
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 04/Mar/2017	Migrated from tests.tcl
# 1.01 20/Jul/2017	Replaced [checkex] with [assert]
# 1.02 02/Aug/2018	Updated checks for wver.exe etc.
#################################################################################################
# Notes
# There is, AFAIK no reliable way to identify ReactOS and from looking at forum comments
# the developers don't seem to grasp why someone might need to do this and stubbornly refuse to
# provide a usable method. Thus MD5 for env(COMSPEC) will fail on ROS
# I currrently only have available Windows NT4.0SP6a, Win2000, Windows 7 and XP Pro/32
#################################################################################################

option expression off

textcolor yellow
puts "Testing \[md5\] ...."
textcolor
newline

if {! [file exists wver.exe]} {
	die "wver.exe is required for this test"
}

# md5
textcolor yellow
puts "Testing md5 of static strings ..."
textcolor
puts "Checking \[md5\] of Hello ..."
assert [md5 Hello] {eq $_ 8b1a9953c4611296a827abf8c47804d7 -nocase} -v -line #__LINE__
puts "Checking \[md5\] of \$qbf"
assert [md5 $qbf] {eq $_ 9e107d9d372bb6826bd81d3542a419d6 -nocase} -v -line #__LINE__

# md5_file
textcolor yellow
puts "\nCalling \[spawn\] and \[md5\] to validate COMSPEC using wver.exe ..."
textcolor
spawn q wver.exe /short
newline

#############################################
# This list will be expanded in due course
#############################################
puts "Line: "#__LINE__
if {ne $q ""} {						# Ignore tests wver.exe failed
	puts "wver.exe windows version result q is $q"
	puts "env(COMSPEC) == '$env(COMSPEC)'"
	option expression off
	# Line 1281: $x=='6D778E0F95447E6546553EEEA709D03C': eq $x AD7B9C14083B52BC532FBA5948342B98 -nocase
	# 929f731fbff48077b7c346f6a0e3ccfd XP pro 32 bit SP2
	
	if {eq $q "5.1"} {				# Windows XP Pro 32-bit
		assert [md5_file $env(COMSPEC)] {|| [eq $_ 929f731fbff48077b7c346f6a0e3ccfd -nocase] [eq $_ 6D778E0F95447E6546553EEEA709D03C -nocase]} -v -line #__LINE__
		#assert [md5_file $env(COMSPEC)] {eq $_ 6D778E0F95447E6546553EEEA709D03C -nocase} -v -line #__LINE__
	} elseif {eq $q "6.1"} {		# Windows 7 Pro 64-bit
		assert [md5_file $env(COMSPEC)] {eq $_ ad7b9c14083b52bc532fba5948342b98 -nocase} -v -line #__LINE__
	} else {
		#puts [md5_file $env(COMSPEC)]
		puts "Test ignored. This version of Windows is not a handled version, it is $q"
	}
} else {
	puts "WVER.EXE returned no identification string"
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
unset q -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #