#######################################################################################
# File:				test_million_array.tcl
# Author:			M Shaw
# Purpose:			Test the allocation of a million array elements
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 14/Jul/2018	First version
# 1.01 01/Aug/2019	Added random checks of element integrity
# 1.02 07/Nov/2019	Amended loop test to complete quicker
#######################################################################################
# Notes:
# DONE ADD RND check for elements
# Adjusted to 100,000 to be practical when running in aggregate with tests.tcl
#
# Allocating 100,000 array elements
# Allocation took 2 second(s)
#
# Allocating 1,000,000 array elements
# Allocation took 21 second(s)
#######################################################################################
option expression off

set limit 100000
#set limit 1000000

textcolor yellow
puts "Allocating [comma $limit] array elements"
textcolor

array unset a -nocomplain

set start [clock seconds]
set i 0
for {} {[< $i $limit]} {++ i} {
	#let a($i) = $i						# Test let
	set a($i)  $i					# Test set
	# if {[is_mod $i [/ $limit 1000]]} {puts $i}		# Debug trace only
}

#set end [clock seconds]
puts "Allocation took [- [clock seconds] $start] second(s)"
#vars 
puts "Allocated $i array elements\n"

puts "Test reads..."
set start2 [clock seconds]
puts "\$a(0)=$a(0)"
assert $a(0) {== $_ 0} -v -line #__LINE__
assert $a(1) {== $_ 1} -v -line #__LINE__
assert $a(10) {== $_ 10} -v -line #__LINE__
assert $a(55) {== $_ 55} -v -line #__LINE__
assert $a(100) {== $_ 100} -v -line #__LINE__
assert $a(123) {== $_ 123} -v -line #__LINE__
assert $a(666) {== $_ 666} -v -line #__LINE__
assert $a(999) {== $_ 999} -v -line #__LINE__
assert $a(1000) {== $_ 1000} -v -line #__LINE__
assert $a(1234) {== $_ 1234} -v -line #__LINE__
assert $a(9999) {== $_ 9999} -v -line #__LINE__
assert $a(10000) {== $_ 10000} -v -line #__LINE__
assert $a(12345) {== $_ 12345} -v -line #__LINE__
assert $a(55555) {== $_ 55555} -v -line #__LINE__
assert $a(90000) {== $_ 90000} -v -line #__LINE__
assert $a(99999) {== $_ 99999} -v -line #__LINE__

set limit 100
loop i 0 $limit {
	set r [rnd 0 99999]
	puts "Assert random element ($i / $limit) -> $r"
	assert $a($r) {== $_ $::r} -line #__LINE__
}

unset r -nocomplain

#set end [clock seconds]
newline

textcolor yellow
textcolor
puts "Read test took [- [clock seconds] $start2] second(s)"

set start3 [clock seconds]
array unset a
set end [clock seconds]
puts "Unset took [- [clock seconds] $start3] second(s)"

unset end i limit start2 start3 -nocomplain

puts "All tests took [- [clock seconds] $start] second(s)"
unset start -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # #
newline
textcolor green
puts "* Done *"
textcolor

vars


# EOF #
