###########################################################################
# File:				test_mpp_array.tcl
# Author:			M Shaw
# Purpose:			Test file for MPP processing of array vars
# Language:			Ticol Tcl v1.19+, v1.26+
#
# Versions:
# 1.00 21/Sep/2016	Implemented
# 1.01 21/Jan/2022	Updated and added to unit tests
###########################################################################
# Notes:
# Test array subscript references using [calc] and [expr]
###########################################################################

option expression off

/* # Superseded by [assert]
if {! [defined checkex] } {
	proc checkex {x expr} {
	upvar i					# Pass-through reference integer 
	upvar line
	set NAN "-1.[calc [chr 35]]IND00000000000"
		set q {$x}
		puts "Line $line: " -nonewline
puts "'\$q'=='\$x'"
		puts "'$q'=='$x': '$expr'"	# fails here at EOF - poss (due to bug in callproc()?) - July 2016
		if { [eval $expr] } {
		} else {
			textcolor red
			puts "checkex assertion failed: '$q' -> '$x' after line $line"
			textcolor
			stop
		}
	}
}  */
#######################################################################

textcolor yellow
puts "Testing Macro PreProcessor \[calc\] with array vars...\n"
textcolor

set a(0) 4
set a(3) 6		# a(3) will map to integer 22/7
set a(24) 11

set c 2
set b($c) 5
set b(3) 8
set b(200) 9

###########################################################################
#/*
set line #__LINE__
puts "Line: $line"
puts "$a(0) * $b($c) / 10"										# 4 x 5 / 10
set q1 [calc $a(0) * $b($c) / 10]								# set q [/ [* $a(0) $b($c)] 10]
set q2 [expr $a(0) * $b($c) / 10]
puts "Test $q1==$q2; [== $q1 $q2]"
assert [== $q1 $q2] {== $_ 1} -v -line #__LINE__

if {!= $q1 $q2} {
	die "Fail at line $line ($q1 != $q2)"
}
newline

set line #__LINE__
puts "Line: $line"
##exec puts [commands]
set r1 [calc $a([/ 22 7]) ^ $b(200) + 10]						# set r [+ [^ $a(3) $b(200)] 10]
# expr does not transit array brackets but [] does
set r2 [expr $a([/ 22 7]) ^ $b(200) + 10]
assert [== $r1 $r2] {== $_ 1} -v -line #__LINE__

if {!= $r1 $r2} {
	die "Fail at line $line"
}
newline
# */

set line #__LINE__
puts "Line: $line"
# #exec puts [commands]
puts [calc 22/7]
puts "Line: "#__LINE__
# eval: Command or proc '/' was not found (autoexec is OFF)
# Preprocessor: Error while optimising '[/ 22 7]' (Check /NEO /NES)
# calc: Bad expression
# eval: Error parsing array subscript '[calc 22/7]'
# FIX MPP bug. /NEO /NES is required here
set r1 $a([calc 22/7])
puts "Line: "#__LINE__
set r1 [calc $a([calc 22/7])]		# translates to set r1 [$a([calc 22/7])]

puts "Line: "#__LINE__
set r1 [calc $a([calc 22/7]) ^ $b([calc 2*100]) + 10]			# set r [+ [^ $a(3) $b(200)] 10]
# exit

# expr does not transit array brackets but [] does
set line #__LINE__
set r2 [expr $a([expr 22/7]) ^ $b([expr 2*100]) + 10]
puts "Line: "#__LINE__

assert [== $r1 $r2] {== $_ 1} -v -line #__LINE__
if {!= $r1 $r2} {
	die "Fail at line $line"
}
newline

set line #__LINE__
puts "Line: $line"
##exec puts [commands]
set r1 [calc $a([/ 300 100]) ^ $b([* 2 100]) + 10]			# set r [+ [^ $a(3) $b(200)] 10]
# expr does not transit array brackets but [] does
puts "Line: "#__LINE__
set r2 [expr $a([/ 300 100]) ^ $b([* 2 100]) + 10]			# set r [+ [^ $a(3) $b(200)] 10]
assert [== $r1 $r2] {== $_ 1} -v -line #__LINE__
if {!= $r1 $r2} {
	die "Fail at line $line"
}
newline

set line #__LINE__
puts "Line: $line"
set s1 [calc 1-$a([calc [/ 22 7]*$b(3)]) ^ $b(200) + 10]				# set r [+ [^ [- 1 $a([* [/ 22 7] $b(3)])] $b(200)] 10]
puts "Line: "#__LINE__
# set s2 [expr 1-$a([/ 22 7]*$b(3)) ^ $b(200) + 10]  # FIX does not resolve array args
set s2 [expr 1-$a([expr [/ 22 7]*$b(3)]) ^ $b(200) + 10]
if {!= $s1 $s2} {
	die "Fail at line $line"
}

assert [== $r1 $r2] {== $_ 1} -v -line #__LINE__

###########################################################################
unset a b c line q1 q2 r1 r2 s1 s2 -nocomplain
newline
vars
newline

textcolor green
puts "* Pass *"
textcolor

# EOF #