#################################################################################
# File: 			test_mpp_define.tcl
# Purpose:			Tests for certain MPP defines
# Author: 			M Shaw
# Language:			Ticol Tcl v1.27+
#
# Versions:
# 1.00 16/Jan/2022	First version
# 1.01 14/May/2023	Updated for ticol_libtest.exe
#################################################################################
# Notes:
# #EOL  substitutions are implemented within the MPP and /echo properly
#################################################################################
/*
ECHO RESULTS...
puts "Via EOL # macros..."
puts "Line: "51
puts "File: "C:\\VC5\\MyProjects\\ticol\\release\\test_mpp_define.tcl
puts "Time: "22:48:45
puts "Date: "20220116
puts "Host: "Lenovo
puts "User: "Admin
################################################################################# */

textcolor yellow
puts "Running test_mpp_define.tcl"
puts "This text will appear after the MPP output when using /ECH"
newline
textcolor

# ********************************************************************
# Implemented 16/Jan/2022
# __LINE__ doesn't work
#echo MPP define __VARNAME__ (hash)__VARNAME__
#define __LINE__ #__LINE__
echo __LINE__ 
puts __LINE__ 
#define __FILE__ #__FILE__
puts __FILE__
#define __DATE__ #__DATE__
puts __DATE__
#define __TIME__ #__TIME__
puts __TIME__
#define __HOST__ #__HOST__
puts __HOST__
#define __USER__ #__USER__
puts __USER__
# ********************************************************************


/*
#deflist
#echo

# The following requires definition first...
# Implemented 16/Jan/2022
#echo MPP __VARNAME__ echo
#echo MPP Date is: __DATE__
#echo MPP File is: __FILE__
#echo MPP Host is: __HOST__
#echo MPP Line is: __LINE__
#echo MPP Time is: __TIME__
#echo MPP User is: __USER__
#echo 
*/

/*
# Not implemented @ Jan 2022
#echo MPP (hash)__VARNAME__ echo (not implemented)
#echo MPP #__DATE__
#echo MPP #__FILE__
#echo MPP #__HOST__
#echo MPP #__LINE__
#echo MPP #__TIME__
#echo MPP #__USER__
#echo 
*/

textcolor yellow
puts "Tcl: Via undecorated macro values at EOL..."
puts "In the format __TAG__ not \#__TAG__"
textcolor
puts "Date: "__DATE__
puts "Host: "__HOST__
puts "File: "__FILE__
puts "Line: "__LINE__
puts "Time: "__TIME__
puts "User: "__USER__
newline

assert __DATE__ {ne $_ ""} -v -line #__LINE__
assert [string length __DATE__] {== $_ 8} -v -line #__LINE__
assert __HOST__ {ne $_ ""} -v -line #__LINE__
assert __FILE__ {ne $_ ""} -v -line #__LINE__
assert __LINE__ {> $_ 0} -v -line #__LINE__
assert __TIME__ {ne $_ ""} -v -line #__LINE__
assert [string length __DATE__] {== $_ 8} -v -line #__LINE__
assert __USER__ {ne $_ ""} -v -line #__LINE__
newline

textcolor yellow
puts "Tcl: Via quoted macro tags... (These quoted items should not be substituted)"
textcolor
puts "Date: __DATE__"
puts "File: __FILE__"
puts "Host: __HOST__"
puts "Line: __LINE__"
puts "Time: __TIME__"
puts "User: __USER__"
newline

# Should equal the literal tag value
textcolor yellow
puts "Tcl: Test via quoted macro tags... (Should not be substituted)"	# ALL FAIL
textcolor
assert "__DATE__" {ne $_ ""} -v -line #__LINE__
assert "__DATE__" {ne $_ __DATE__} -v -line #__LINE__
assert "__FILE__" {ne $_ ""} -v -line #__LINE__
assert "__FILE__" {ne $_ __FILE__} -v -line #__LINE__
assert [string length "__FILE__"] {== $_ 8} -v -line #__LINE__
puts "__ HOST __ is __HOST__"
puts __HOST__
assert "__HOST__" {ne $_ ""} -v -line #__LINE__
assert "__HOST__" {ne $_ __HOST__} -v -line #__LINE__
assert "__LINE__" {ne $_ ""} -v -line #__LINE__
assert "__LINE__" {ne $_ __LINE__} -v -line #__LINE__
assert [string length "__LINE__"] {== $_ 8} -v -line #__LINE__
assert "__TIME__" {ne $_ ""} -v -line #__LINE__
assert "__TIME__" {ne $_ __TIME__} -v -line #__LINE__
assert "__USER__" {ne $_ ""} -v -line #__LINE__
assert "__USER__" {ne $_ __USER__} -v -line #__LINE__
assert [string length "__USER__"] {== $_ 8} -v -line #__LINE__
newline

textcolor yellow
puts "Tcl: Direct via EOL # macros..."	# ALL PASS
textcolor
puts "Date: "#__DATE__
puts "File: "#__FILE__
puts "Host: "#__HOST__
puts "Line: "#__LINE__
puts "Time: "#__TIME__
puts "User: "#__USER__
newline

# Should insert the relvant #EOL macro values
textcolor yellow
puts "Tcl: Test direct via EOL # macros..."	# ALL PASS
textcolor
assert ""#__DATE__ {ne $_ ""} -v -line #__LINE__
assert ""#__FILE__ {ne $_ ""} -v -line #__LINE__
assert ""#__HOST__ {ne $_ ""} -v -line #__LINE__
assert ""#__LINE__ {ne $_ ""} -v -line #__LINE__
assert ""#__LINE__ {> $_ 0} -v -line #__LINE__
assert ""#__TIME__ {ne $_ ""} -v -line #__LINE__
assert ""#__USER__ {ne $_ ""} -v -line #__LINE__
newline

textcolor yellow
puts "Tcl: Variables via EOL # macros..."	# ALL PASS
textcolor
set line #__LINE__
puts "Line: $line"
set file #__FILE__
puts "File: $file"
set time #__TIME__
puts "Time: $time"
set date #__DATE__
puts "Date: $date"
set host #__HOST__
puts "Host: $host"
set user #__USER__
puts "User: $user"
newline

unset date file host line time user -nocomplain

vars
newline

textcolor green
puts "* Done *"
textcolor