############################################################################################
# File:				test_mpp_macro_subst.tcl
# Author:			M Shaw
# Purpose:			Development test of MPP substitution in MPP commands
# Language:			Ticol Tcl v1.26+
# Caution:			Contains MPP executable code. Will run on any MPP call including /C
#
# Versions:
# 1.00 19/Aug/2019	First version
# 1.01 15/Jan/2022	Updated
###########################################################################################
# Notes:
# Now included with tests.tcl
# FIX Can't have tail end #__LINE__ or we get an MPP error
# Macro commands with arguments which need to be substituted:
# #echo
# #if
# #exec
###########################################################################################
# ADD MPP subst to #echo for __FILE__, __HOST__ etc.
# DONE ADD MPP subst to #echo
# DONE ADD MPP subst to #exec
# DONE ADD MPP subst to #if
# DONE FIX #echo process line to EOF
# DONE FIX #exec process line to EOF
# DONE FIX MPP FATAL Too many lines
###########################################################################################
/*
#define PI 3.14159265358979323
textcolor darkyellow
printf "Tcl: Test runtime Pi is %s (calc=%f) at line %li and 123:%li\n" PI [/ PI 7.0] __LINE__ 123	# comment
#printf "User: '%s' Host: '%s'\n" __USER__ __HOST__			# Not yet implemented
textcolor
#exit
# */

#exec cls
#echo Start executing test_mpp_macro_subst.tcl/tcx ...
#echo __LINE__
# The following are not yet implemented in the MPP @ 2022/Jan/15
#echo __FILE__ 
#echo __DATE__ 
#echo __TIME__ 
#echo __USER__
#echo __HOST__
# They are, however implemented in object script
puts "__LINE__ "#__LINE__ 
puts "__FILE__ "#__FILE__ 
puts "__DATE__ "#__DATE__ 
puts "__TIME__ "#__TIME__ 
puts "__USER__ "#__USER__ 
puts "__HOST__ "#__HOST__ 
set mfile #__FILE__
set mdate #__DATE__
set mtime #__TIME__
set muser #__USER__
set mhost #__HOST__
puts "Assert __FILE__"
assert $mfile {ne $_ ""} -v -line #__LINE__
puts "Assert __DATE__"
assert $mdate {ne $_ ""} -v -line #__LINE__
puts "Assert __TIME__"
assert $mtime {ne $_ ""} -v -line #__LINE__
puts "Assert __USER__"
assert $muser {ne $_ ""} -v -line #__LINE__
puts "Assert __HOST__"
assert $mhost {ne $_ ""} -v -line #__LINE__

unset mfile mdate mtime muser mhost -nocomplain

#echo \n
#echo Starting at line: __LINE__
#define PI 3.14159265358979323

puts "Tcl: Line: "#__LINE__
#define msg_1 Hello
puts "Line: "#__LINE__
#define msg_2 world!
#define BLANK			# FIX (set to bool 1). Does not strip comment
#define NULL ""			# FIX Does not set to ""

puts "Tcl: Line: "#__LINE__
#deflist
#echo ---------------------------------
#echo MPP: I'd like to say msg_1 msg_2 !

#if false
#exec textcolor red
#echo MPP: * SHOULD NOT APPEAR *
#exec textcolor
#exit
#endif
#echo MPP: Line: __LINE__
#if true
#exec textcolor magenta
#echo MPP: ************** SHOULD APPEAR **************
#exec textcolor
#endif

#echo ""
puts "Line: "#__LINE__

#exec #
puts "Line: "#__LINE__

# Test #if with DEF VAR
#if eq msg_1 Hello
#exec textcolor green
#echo MPP: Detected msg_1 okay
#else
#exec textcolor red
#echo MPP: Failed to detect msg_1
#endif
#echo

puts "Line: "#__LINE__
#echo MPP: Blank is 'BLANK' ...

#exec textcolor
##exec puts {msg_1 msg_2}				# PASS (is translated okay)
##exec puts "msg_1 msg_2"				# PASS No translation (which is correct)
##exec puts msg_1;puts msg_2			# PASS (handles whitespace, semi colon and EOL now)
##exec puts msg_1" "msg_2				# FAIL - 'msg_1" msg_2' (but is barely recommended anyway)
##exec puts msg_1""msg_2				# FAIL - 'msg_1"msg_2' (but is barely recommended anyway)
#exec printf "MPP: %s %s\n" msg_1 msg_2		# PASS - printf or format is the best way to handle multiple defined vars in strings

#exec set s [format "%s %s" msg_1 msg_2]
#exec assert $s {eq $_ "Hello world!"} -v -line __LINE__

# ECHO at EOF now passes ok
##echo *DONE*
# DONE FIX exec needs to swallow the whole line up to any comment char to EOF

#if eq PI 3.14159265358979323
#exec textcolor darkyellow
#echo MPP: Pi defined as PI
#exec textcolor
#exec assert PI {eq $_ 3.14159265358979323} -v -line __LINE__
#echo
puts "Line: "#__LINE__
assert PI {eq $_ 3.14159265358979323} -v -line __LINE__
set __LINE__ 666			# Will create a var with a numeric name
#deflist
printf "Test runtime Pi is %s (calc=%f) at line %li and 123:%li\n" PI [/ PI 7.0] __LINE__ 123
#endif
##exit

#if > __LINE__ 0
#echo MPP: Multiple LINE __LINE__ __LINE__ __LINE__ (should be x3)
#echo MPP: LINE is set > 0 at line 93
#echo MPP: LINE is set > 0 at line 93
#exec assert __LINE__ {== $_ __LINE__} -v -line __LINE__
#endif

#echo MPP: Line at: __LINE__ (with some trailing text)

#exec textcolor cyan
#echo MPP: Line via macro subst is __LINE__ ...
#exec textcolor

#echo MPP: Blank is 'BLANK' ...
#echo
#exec option echo on
#exec assert BLANK {eq $_ "1"} -v -line __LINE__
#echo
#exec assert NULL {eq $_ ""} -v -line __LINE__

##exit
#exec textcolor yellow
#exec printf "MPP: 1: %s %s\n" msg_1 msg_2
#exec printf "MPP: 2: %s %s\n" msg_1 msg_2
#echo MPP: Pre penultimate line: __LINE__
unset 1* -glob -nocomplain			# Var set by line number
vars
puts "Tcl: Penultimate Line: (MPP follows on last line) "#__LINE__
#echo * MPP: TEST DONE - Tcl will now execute *\n\n