#################################################################################
# File: 			test_multiline_printf.tcl
# Purpose:			Test [printf] and [format] with multiline strings
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 30/Dec/2017	First version
# 1.01 28/Oct/2019	Full test version with asset
#################################################################################
# Notes:
# A quick checks on how text layout in source code affects output
#################################################################################


#################################################################################
# Backslash-terminated: Will include wrapped whitespace
# All on one line
# 30+3+20+3+11+2=69
puts "Line: "#__LINE__
assert [printf "012345678901234567890123456789\
			abcdefghijabcdefghij\
			01234567890\r\n"] {== $_ 69} -v -trace -line #__LINE__

#################################################################################
# Unterminated: Will include wrapped whitespace
# Will include wrapped whitespace
# Keeps CRLFs, and returns a multi-line string
# 30+3+20+3+11+2=69 +2+2 = 73
puts "Line: "#__LINE__
assert [printf "012345678901234567890123456789
			abcdefghijabcdefghij
			01234567890\r\n"] {== $_ 73} -v -trace  -line #__LINE__

#################################################################################
# Unterminated and not indented: Will include wrapped whitespace
# Will include wrapped whitespace
# Keeps CRLFs, and returns a multi-line string
# 30+20+11+2=67
puts "Line: "#__LINE__
assert [printf "012345678901234567890123456789
abcdefghijabcdefghij
01234567890\r\n"] {== $_ 67} -v -line #__LINE__

#################################################################################
# Since [format] aliases [printf] we can test it too
# Will include wrapped whitespace
# Results in a single-line string
puts "Line: "#__LINE__
set s [format "012345678901234567890123456789\
			abcdefghijabcdefghij\
			01234567890\r\n"]

assert $s {eq $_ {012345678901234567890123456789\t\t\tabcdefghijabcdefghij\t\t\t01234567890\r\n}} -v -line #__LINE__

#################################################################################
# Since [format] aliases [printf] we can test it too
# Will include wrapped whitespace
# Results in a single-line string
puts "Line: "#__LINE__
set s [format "012345678901234567890123456789\
abcdefghijabcdefghij\
01234567890"]

assert $s {eq $_ {012345678901234567890123456789abcdefghijabcdefghij01234567890}} -v -line #__LINE__


#################################################################################
# Since [format] aliases [printf] we can test
puts "Line: "#__LINE__
set s [format "012345678901234567890123456789
"]
#                 012345678901234567890123456789
assert $s {eq $_ {012345678901234567890123456789\r\n}} -v -line #__LINE__

#################################################################################
# Since [format] aliases [printf] we can test it also
# Should be the same as [set s]
# Will include wrapped whitespace
# Keeps CRLFs, multi-line
puts "Line: "#__LINE__
set s [format "012345678901234567890123456789
			abcdefghijabcdefghij
			01234567890\r\n"]
textcolor
newline

assert $s {eq $_ {012345678901234567890123456789\r\n\t\t\tabcdefghijabcdefghij\r\n\t\t\t01234567890\r\n}} -v -line #__LINE__


#################################################################################
# Since [format] aliases [printf] we can test it also
# Should be the same as [set s]
# Will include wrapped whitespace
# Keeps CRLFs, multi-line
puts "Line: "#__LINE__
set s [format "012345678901234567890123456789\
abcdefghijabcdefghij\
01234567890\r\n"]

assert $s {eq $_ {012345678901234567890123456789abcdefghijabcdefghij01234567890\r\n}} -v -line #__LINE__


puts "Line: "#__LINE__
unset s -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #