#############################################################################################
# File:				test_multitask.tcl
# Purpose:			Test multithreaded [after]
# Author:			M Shaw
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 20/Jan/2017	Test cases for experimental threaded [after]
#					Used to probe codebase for incompatibilities
#############################################################################################
# Notes:
# Scripts are not atomic by script, only by command
# thus gotoxy may locate for the wrong command
# thus scripts need to be atomic and lock out or be locked-out by main
# what about; "while {! [locked]} {...}" and "while {! [unlocked]} {...}" 
# with x2 commands which acquire the thread lock flag?
#############################################################################################

if {! $tcl_threaded} {
	textcolor red
	die "Threaded release required!"
	tex
}

proc every {ms code} {
	#puts "DEBUG: $r: Enter every: '$ms' '$code' '$flag'" -nonewline		# Debug code
	eval $code
	after $ms every $ms $code	# Call recursively
}
cls
option expression off

set quit 0
set s 0

/* Will never relinquish thread
# Test such a lock up in ActiveState
proc display {} {
	upvar s
	while {1} {
		gotoxy 1 15
		puts "s is '$s'         "
		#cleareol
		sleep 500
		if {== [inkey] 27} {   # ESC is 27
			newline
			textcolor red
			puts "* break *"
			textcolor
			set quit 1
			#stop
		}	
	}
}
*/

set i 0
#display
#after 1 display
for {set i 0} {< $i 50} {++ i} {
	after [rnd 2900 3000] eval increment_s.tcl
}

# Need [every] for polling update

every 100 {gotoxy 1 1; puts "::s=$::s"}

vwait quit

# after cancel
stop

# This while runs whilst the [after] threads run
while {1} {
	gotoxy 1 15
	puts "s is '$s'         "
	#cleareol
	sleep 500
	if {== [inkey] 27} {   # ESC is 27
		newline
		textcolor red
		puts "* break *"
		textcolor
		stop
	}	
}


# EOF #