#################################################################################
# File: 			test_named_store.tcl
# Purpose:			Tests for [store] command (not a unit test)
# Author: 			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 05/Jan/2018	First version (separate stores implemented)
# 1.01 28/Nov/2021	Minor changes
#################################################################################
# Notes:
# 12/Feb/2018 - No leaks
# Test for leaks. (No leaks)
# Note that the C++ heap MM will do periodic garbage collection
# Not part of main tests bundle
#################################################################################

# Using a named buffer cache 'b'
option expression off

# Test for memory leaks using task manager
#define TEST_MEM_LEAKS
#ifdef TEST_MEM_LEAKS

set limit 100

textcolor yellow
puts "Testing named store called 'b' for $limit loops"
textcolor
newline

puts "Line: "#__LINE__
store get b                 # Force clear the buffer

puts "Reading file ticol.man and concatenating contents ..."

readfile ticol.man -array a

set loops 0
time {
	puts "Loop $loops"
	for {set i 0} {< $i [array size a]} {++ i} {
		store add b "$a($i)\r\n"
	}
	set slb [store length b]
	set q [store get b]
	++ loops
} $limit -info

puts "Store b length is: $slb"
puts "Final var q length is: [string length $q] byte(s)"
#endif

newline
textcolor green
puts "* Done *"
textcolor

# EOF #