############################################################################################
# File:				test_ndc.tcl
# Author:			M Shaw
# Purpose:			Test non decimal const replacement by the MPP
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 29/Aug/2019 First version
###########################################################################################
# Notes:
# Supports
# 0bNN
# 0xNN
# &hNN
# \0nn	(slash+zero)
# \\Onn ??
###########################################################################################

textcolor yellow
puts "Test literal integer consts ..."
textcolor
newline

textcolor yellow
puts "... prefixed"
textcolor
# Unquoted values will replace with integer representation
assert 0x1b {== $_ 27} -v -line #__LINE__
assert &h1b {== $_ 27} -v -line #__LINE__			#
assert 0b11011 {== $_ 27} -v -line #__LINE__
assert &b11011 {== $_ 27} -v -line #__LINE__		#
assert 0o33 {== $_ 27} -v -line #__LINE__			#

textcolor yellow
puts "... escaped"
textcolor
# Backslash-escaped ...
assert \033 {== $_ [chr 27]} -v -line #__LINE__
assert \x1b {== $_ [chr 27]} -v -line #__LINE__
assert \X1b {== $_ [chr 27]} -v -line #__LINE__
# Not implmented as \b is a backspace
#assert \b11011 {== $_ [chr 27]} -v -line #__LINE__	# FAIL
newline

textcolor yellow
puts "Test literal character consts ..."
textcolor
# Embedded backslash sequences within dquoted strings will replace with literal char not number
assert "\x1b" {== $_ [chr 27]} -v -line #__LINE__
assert "\X1b" {== $_ [chr 27]} -v -line #__LINE__
assert "\033" {== $_ [chr 27]} -v -line #__LINE__
# Not implmented as \b is a backspace
#assert "\b11011" {== $_ [chr 27]} -v -line #__LINE__	# FAIL

newline
textcolor yellow
puts "Visual test literal character consts ..."
textcolor
puts "Should show 384 ..."
puts 0b110000000
assert 0b110000000 {== $_ 384} -v -line #__LINE__
puts 0x41
assert 0x41 {== $_ 65} -v -line #__LINE__
puts [chr 0x41]
assert [chr 0x41] {== $_ A} -v -line #__LINE__
puts \x41
assert \x41 {== $_ A} -v -line #__LINE__

newline
vars
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #