#############################z####################################################
# File:				test_nested_deref.tcl
# Author:			M Shaw
# Purpose:			Test cases for new, nested brace dereferencing
# Language:			Ticol Tcl v1.23+ (proposed)
#
# Versions:
# 1.00 06/May/2017	First version
# 1.02 12/May/2017	Improved algorithm and tests
# 1.03 31/Jul/2019	Removed unset * -glob
#################################################################################
# Notes:
# Fixing the parser to make proper use of new recursive var interpreter
# Note: puts "{${a}there}" -> {hellothere} correct
# Note: Braces are not removed here...
# activestate:
# puts a{bcd}e -> a{bcd}e
# puts p{b${a}d}q -> p{bhellod}q
# puts "p{b${a}d}q" -> p{bhellod}q
# puts "p{b{$a}d}q" -> p{b{hello}d}q
# ticol:  puts x$a{there}x -> no variable there - is this correct behaviour?
#################################################################################
# DONE FIX test_arrays.tcl - braced array substrings are now borked in the parser
#	Rewrite the parser to combine both functionalities
#	This is due to non-var brace handling mismatching delayed brace handling
#################################################################################
# Design:
# Evaluate (recurse(string))
# For each var - evaluate and append
# for each ${} - recurse and evaluate
# fore each non var {} then append
# Don'e call for 'a{bcd}e'
#################################################################################

#unset * -glob -nocomplain			# -const is required to delete all consts

array unset a b c -nocomplain

#vars -const
#pause
/*
# Internal parseVar recurse tests
//LPSTR q="${${b}}";			// PASS hello
//LPSTR q="${b}";				// PASS a
//LPSTR q="{b}";				// PASS? returns hello
//LPSTR q="b";					// PASS returns a
//LPSTR q="${a${b}}";			// ab PASS ? (invalid syntax)
//LPSTR q="${a}${b}";			// PASS helloa
//LPSTR q="${a}${${b}}";		// PASS hellohello
//LPSTR q="${a} ${${b}}";		// FAIL "hello"	- this is poss correct as parser should only supply var string
//LPSTR q="{${a}${${b}}}";		// PASS (WRAP) "hellohello"
*/

#################################################################################
option expression off

undef print_line -nocomplain

proc print_line {l} {
	upvar line
	set line $l
	textcolor yellow
	puts "Line: $l"
	textcolor
}
########################################################################################
# At 2017/May/06
# Fix parse bugs first
# Then fix resolution bugs
set a Hello
set b a
set c b
###############################################################################

# puts a{bcd}e -> a{bcd}e					# Activestate + PASS
assert a{bcd}e {eq $_ a{bcd}e} -v -line #__LINE__

# FIX should delay? this is a problem for enforcing $ prefix rule...
# puts p{b${a}d}q -> p{bhellod}q			# Activestate + PASS
#assert p{b${a}d}q {eq $_ p{bHellod}q} -v -line #__LINE__

# puts "p{b${a}d}q" -> p{bhellod}q			# Activestate + PASS
#assert "p{b${a}d}q" {eq $_ p{bHellod}q}

# puts "p{b{$a}d}q" -> p{b{hello}d}q		# Activestate + PASS
assert "p{b{$a}d}q" {eq $_ p{b{Hello}d}q} -v -line #__LINE__

puts "Dereference: \$a\t\t(Should be Hello)" 				# PASS
assert $a {eq $_ Hello} -v -line #__LINE__

puts "Dereference: \$b\t\t(Should be a)" 					# PASS
assert $b {eq $_ a} -v -line #__LINE__

puts "Dereference: \$\$b\t\t(Should be Hello)" 				# PASS
assert $$b {eq $_ Hello} -v -line #__LINE__

puts "Dereference: \$\$\$c\t\t(Should be hello)" 			# PASS
assert $$$c {eq $_ Hello} -v -line #__LINE__

# Check code flow for $$... v's ${var} are not separate

puts "Dereference: \$\$\$\{c\}\t\t(Should be hello)" 		# PASS a $$ prefix not working (see below)
assert $${$c} {eq $_ Hello} -v -line #__LINE__

textcolor darkyellow
puts "Dereference: \$\$\{c\}\t\t(Should be a)" 				# PASS Hello - requires ${${var}}, should allow ${$var}
puts {$c}
assert ${$c} {eq $_ a} -v -line #__LINE__

puts "Dereference: \$\$\{c\}\t\t(Should be Hello)" 			# PASS b - see above
assert $${${c}} {eq $_ Hello} -v -line #__LINE__

# FAIL (parser?)
puts "Dereference: \$\{\$\{c\}\}\t\t(Should be a)" 			# PASS
assert ${${c}} {eq $_ a} -v -line #__LINE__

puts "Dereference: \$b\t\t\t(Should be a)"					# PASS
assert $b {eq $_ a} -v -line #__LINE__

puts "Dereference: \$\{b\}\t\t(Should be a)"				# PASS
assert ${b} {eq $_ a} -v -line #__LINE__

puts "Dereference: \{b\}\t\t(Should be literal b)"			# PASS
assert {b} {eq $_ b} -v -line #__LINE__

puts "Dereference: \$\$b\t\t(Should be hello)" 				# PASS
assert $$b {eq $_ Hello} -v -line #__LINE__

#puts "\$\{a\$\{b\}\} == '${a${b}}'          (Should be a)"	# 
puts "Dereference: \$\{\$\{b\}\}\t\t(Should be hello)"		# PASS
assert ${${b}} {eq $_ Hello} -v -line #__LINE__

## Arrays #########################################

array unset a -nocomplain
unset b c d -nocomplain

set a(1) "one"
set b a(1)
set c b
set d c
# Multi-level dereferencing ...
# $d => c
# $$d => b
# $$$d => a(1)
# $$$$d => "one"
# $$$$$d => $"one" (ERR)

# Resolve for 'd'
puts "Start a(1) is $a(1)"
assert $d {eq $_ c} -v -line #__LINE__
assert $$d {eq $_ b} -v -line #__LINE__
assert $$$d {eq $_ a(1)} -v -line #__LINE__		# new dereferencing can't dereference arrays yet 2017/May/07
assert $$$$d {eq $_ one} -v -line #__LINE__# Final string

###################################################
set a(1) Hello
set a(two) There
set a("with spaces") Fox

# Braced Arrays
assert abc$a(1)def {eq $_ abcHellodef} -v -line #__LINE__# PASS

puts "$a(1)  $a"
assert abc${a(1)}def {eq $_ abcHellodef} -v -line #__LINE__# PASS
# Should work same as abc${b}def

assert abc${a(two)}def {eq $_ abcTheredef} -v -line #__LINE__					# PASS

# Subscript could be any valid chars
puts "\$a(\"with spaces\")=='$a("with spaces")'"			# 
assert abc${a("with spaces")}def {eq $_ abcFoxdef} -v -line #__LINE__			# PASS

assert "abc${a(1)}def" {eq $_ abcHellodef} -v -line #__LINE__					# ERROR (correct). Wrong err. No variable '{$a(1)}' at level 0

assert "abc${a(two)}def" {eq $_ abcTheredef} -v -line #__LINE__				# ERROR (correct)

assert "abc${a("with spaces")}def" {eq $_ abcFoxdef} -v -line #__LINE__		# ERROR (correct)

set i t
#  =>           $a(two)
puts "\$\{i\}='${i}'; "										# PASS t
assert ${i} {eq $_ t} -v -line #__LINE__
puts "\$\{i\}wo='${i}wo'; "									# PASS	two
puts "\$\{i\}wo='{${i}wo}'; "								# PASS {two}
puts "\$a(\$\{i\}wo)='$a(${i}wo)'; "						# PASS

puts "\$\{i\}wo='{$a(${i}wo)}'; "							# PASS

assert $a(${i}wo) {eq $_ There} -v -line #__LINE__							# PASS

assert abc$a(${i}wo)def {eq $_ abcTheredef} -v -line #__LINE__				# PASS

set q a

puts "${a(two)}"		# Should return "There"				# PASS
puts "$a(${i}wo)"		# Should return "There"				# PASS
puts "${a}(two)"		# Should return "<array>(two)"		# PASS


# Double-dereference tests #############################################################################
assert $$$c {eq $_ Hello} -v -line #__LINE__
assert $$${c} {eq $_ Hello} -v -line #__LINE__				# Hello?  c->b->a(1)->Hello
assert $${${c}} {eq $_ Hello} -v -line #__LINE__				# Hello?  c->b->a(1)->Hello
assert ${${${c}}} {eq $_ Hello} -v -line #__LINE__			# Hello?  c->b->a(1)->Hello

# Complex double-dereferences  #########################################################################
set a("with spaces") "Hello world!"
set b [unescape "a(\"with\ spaces\")"]
set c b
puts "Triple dereference: '$$$c'"
assert $$$c {eq $_ "Hello world!"} -v -line #__LINE__			# PASS

/*
set a(1) Hello
set a(two) There

set b a(1)
set c b
set d c
*/
set q t
set ThereThere "All better"
set a("with spaces") Fox

assert ${a(1)} {eq $_ Hello} -v -line #__LINE__
assert ${a(two)} {eq $_ There} -v -line #__LINE__
assert ${a(${q}wo)} {eq $_ There} -v -line #__LINE__
assert ${a(two)}${a(two)} {eq $_ ThereThere} -v -line #__LINE__			# PASS
assert abc${a(two)}def${a(two)}ghi {eq $_ abcTheredefThereghi} -v -line #__LINE__			# PASS
assert ${a("with spaces")} {eq $_ Fox} -v -line #__LINE__					# PASS


newline
textcolor yellow
puts "Complex failures ..."
textcolor

set p v
set q a
set r r
set var $qbf
set i t
set a("with spaces") Fox
set Fox Hunt
set j i

assert $p$q$r {eq $_ var} -v -line #__LINE__ 								# PASS
assert ${$p$q$r} {eq $_ $::qbf} -v -line #__LINE__						# PASS - Does not resolve std. dereference when braced

puts ${${p}${q}${r}}									# QBF
assert ${${p}${q}${r}} {eq $_ $::qbf} -v -line #__LINE__ 					# PASS


# Resolve var to array prefix then resolve array with mixed var and text subscript
assert ${i}wo {eq $_ two} -v -line #__LINE__
assert ${q}(${i}wo) {eq $_ a(two)} -v -line #__LINE__
assert "${${q}(${i}wo)}"	{eq $_ There} -v -line #__LINE__ 				# Should return "There"				# PASS

assert "${a(${i}wo)}"		{eq $_ There} -v -line #__LINE__				# Should return "There"				# PASS

assert "${${q}(${i}wo)}"		{eq $_ There} -v -line #__LINE__			# Should return "There"			# PASS

# Simple variable within array subscript
assert abc${a(${i}wo)}def {eq $_ abcTheredef} -v -line #__LINE__			# PASS

# Multiple, sequential arrays
assert $a(two) {eq $_ There} -v -line #__LINE__
assert $ThereThere {eq $_ "All better"} -v -line #__LINE__
assert ${${a(two)}${a(two)}} {eq $_ "All better"} -v -line #__LINE__		# PASS
assert ${$a(two)$a(two)} {eq $_ "All better"} -v -line #__LINE__			# PASS

# Multiple nesting
puts ${$a(${i}wo)$a(two)}								# Will dereference "There" from $a(${i}wo) rather than concat
assert ${$a(${i}wo)$a(two)} {eq $_ "All better"} -v -line #__LINE__		# PASS

# Multiple nesting
# j-> i -> "t"wo
assert ${$a(${${j}}wo)$a(two)} {eq $_ "All better"} -v -line #__LINE__	# PASS


# Var prefixes
set a(0) hello
assert $a(0)_world {eq $_ hello_world} -v -line #__LINE__					# hello_world
assert ${a(0)}_world {eq $_ hello_world} -v -line #__LINE__				# hello_world
# As of experimental changes 24/Nov/2022 a dot "." means a struct dereference
# so these tests are no longer valid as arbitrary strings
/*
set b 0
assert $a($b).world  {eq $_ hello.world} -v -line #__LINE__				# hello.world
assert $a(${b}).world  {eq $_ hello.world} -v -line #__LINE__				# hello.world
assert ${a(${b})}.world  {eq $_ hello.world} -v -line #__LINE__			# hello.world
*/

assert ${$a("with spaces")} {eq $_ Hunt} -v -line #__LINE__				# PASS

assert {$p$q$r} {eq $_ {$p$q$r}} -v -line #__LINE__						# PASS -> {$p$q$r} - ActiveState returns $p$q$r

assert {$a("with spaces")} {eq $_ {$a("with spaces")}} -v -line #__LINE__	# PASS - Braces prevent evaluation

newline
# Remaining failure cases #################################################################################
textcolor yellow
puts "Failed tests ..."
textcolor
newline

# Braced expressions
#puts {$p$q$r} 											# FAIL - Does not resolve std. dereference when braced

# Arrays with spaces within the subscript are ok
#puts "check a with spaces is: "'$a("with spaces")'

assert ${$a("with spaces")} {eq $_ Hunt} -v -line #__LINE__				# PASS - VALID SYNTAX
#assert ${$a{("with spaces")}} {eq $_ Hunt} -v -line #__LINE__			# FAIL - INVALID SYNTAX (will attempt to resolve subscript literal)


###################################################
# Broken tests 
####################################################
# This syntax is not allowed in ActiveState
/*
# FIX ME -> a("with spaces)
set q a("with spaces")										# FAILS to assign - but is not in [set]
puts "q is '$q'"
#assert $q {eq $_ a(\"with spaces\")} -v -line #__LINE__	# invalid
assert $q {eq $_ "a(\"with spaces\")"} -v -line #__LINE__
vars q*
*/

# But allowed if wrapped in braces
set q {a("with spaces")}
vars q*
assert $q {eq $_ {a("with spaces")}} -v -line #__LINE__

# Won't work due to embedded spaces
#assert $q {eq $_ [unescape "a(\"with spaces\")"]} -v -line #__LINE__		# FAILS

assert $q {eq $_ [unescape {a(\"with spaces\")}]} -v -line #__LINE__		#

###################################################

undef print_line -nocomplain
array unset a -nocomplain
unset Fox ThereThere var q -nocomplain
unset b c d i j r p q -nocomplain
newline

###################################################
vars
newline
textcolor green
puts "* Nested Dereference Tests - Done *"
textcolor

# EOF #	