################################################################################################
# File: 		tsst_nested_funct_if.tcl
# Purpose:		Test cases nested functional if in [expr]
# Author: 		M Shaw
# Language:		Ticol Tcl v1.26+
#
# Versions:
# 1.00 06/Sep/2018 First version
# 1.01 13/Sep/2018 Updated re: bugfixes, retested
################################################################################################
# Notes: 
# FIX instability with any facx >= 15 (trace?)
# FIX Does not test [?]
# Functional if in expr
# Still memory-unstable in some expr ? cases
################################################################################################

option echo on
option expression push on

set n 3

if {[defined facx]} {die "facx already defined"}

puts "Line :"#__LINE__

################################################################################################
# ORIGINAL: Works ok in Active State
# Fails for factorials >= 20 in ActiveState
# Requires sub-[expr] to be able to brace-delay [facx] 
#proc facx n {expr {$n<2? 1:$n>20? $n*($n-1): $n * [facx [incr n -1]]}}						# FAIL 12/Sep/2018 Results in -ve increments and overflow				
################################################################################################

# Problem cases to check ... # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

# Fails to expr eval FALSE clause for ? ... 
#proc facx n {expr {$n<2? 1:$n>20? $n*($n-1): $n * {[facx [incr n -1]]}}}					# FAIL 12/Sep/2018

# Memory unstable
#proc facx n {expr {$n<2 ? 1 : expr {$n>20 ? $n*($n-1) : expr {$n * [facx [incr n -1]]}}}}	# PASS 13/Sep/2018 (unstable)

#proc facx n {expr {$n<2? 1: expr {$n>20 ? $n*($n-1): {$n * [facx [incr n -1]]}}}}			# FAIL 13/Sep/2018 - Fail tests at 144

# GPFs later scripts 13/Sep/2018
# Braces are crossed
#proc facx n {expr {$n<2 ? 1: expr {$n>20 ? $n*($n-1) : expr {$n * [facx [-- n]]}}}}		# GPF PASS 13/Sep/2018 (removed curved brackets)
#proc facx n {expr {$n<2 ? 1: expr {$n>20 ? $n*($n-1) : expr {$n * [facx [incr n -1]]}}}}	# GPF PASS 13/Sep/2018 (removed curved brackets)

#set n 1

# Tested # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
puts "Line :"#__LINE__

# Failure to brace args results in uncontrolled recursion (call cases will fail)
#proc facx {n} {return [expr ($n<2)?1:$n * [facx [- $n 1]]]}					# FAIL 13/Sep/2018 Results in -ve increments and overflow
#proc facx {n} {expr $n<2?[return 1]:{$n * [facx [- $n 1]}}						# FAIL 13/Sep/2018 Results in -ve increments and overflow
#proc facx n {expr "$n<2?return 1:$n>=20? return 0 : $n * [facx [-- n]]"}		# FAIL 13/Sep/2018 Results in -ve increments and overflow
#proc facx {n} {expr $n<2?1:{$n>20?0:{expr $n * {[facx [-- n]]}}}}				# FAIL 13/Sep/2018 (unbraced) - expression error trapped
#proc facx {n} {expr $n<2?1:{$n * [facx [- $n 1]}}								# FAIL 13/Sep/2018 (unbraced) - expression error trapped

# Exmaple missing braces in expr and sub expr 
# Whilst this works, it is not syntactically ideal to mandate braces
# The current logic is that if [facx] is unbraced then [facx] will recurse uncontrollably without multiplying by $n
# since [] will be evaluated before *
#proc facx {n} {expr {$n<2 ? 1 : expr {$n * {[facx [-- n]]} } } }				# PASS 13/Sep/2018 {fully braced}
#proc facx {n} {expr {$n<2 ? 1 : expr {$n * [facx [-- n]] } } }					# FAIL (unbraced expr arg)
#proc facx {n} {expr $n<2 ? 1 : expr {$n * [facx [-- n]] }  }					# FAIL (unbraced expr arg)
#proc facx {n} {expr $n<2 ? 1 : expr $n * {[facx [-- n]] }  } 					# FAIL (unbraced expr arg)
#proc facx {n} {expr $n<2?1: expr $n * {[facx [-- n]] }  } 						# FAIL (unbraced expr arg)
#proc facx {n} {expr $n<2?1: {expr $n * {[facx [-- n]] } } } 					# FAIL (unbraced expr arg)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
/*
puts "Line :"#__LINE__
# Curved brackets may be used to group expression arguments properly
proc facx n {													# PASS
	expr {
		#$n<2 ? 1: expr {$n>20 ? $n*($n-1) : expr {$n * [facx [incr n -1]]}}	# FAIL 13/Sep/2018
		$n<2 ? 1: expr {$n>20 ? $n*($n-1) : expr ($n * [facx [incr n -1]])}	# PASS 13/Sep/2018
	}
}  # */

#proc facx {n} {[? $n<2 1 : {expr $n * [facx [-- n]]}]}							# PASS 13/Sep/2018
#proc facx {n} {expr {$n < 2 ? 1 : {expr $n * {[facx [incr n -1]]}} }}			# PASS 13/Sep/2018
#proc facx {n} {expr {$n < 2 ? 1 : {expr $n * {[facx [-- n]]}} }}				# PASS 13/Sep/2018
# Command form:
#proc facx {n} {? $n<2 1 : {? $n>20 0 : {expr $n * [facx [-- n]]}}}				# PASS 13/Sep/2018

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
/*
puts "Line :"#__LINE__
proc facx {n} {
	? {$n<2} 1 : {? $n>20 0 : {expr $n * [facx [-- n]]}}						# PASS 13/Sep/2018
}
# */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
/*
puts "Line :"#__LINE__
proc facx {n} {																	# PASS 13/Sep/2018
	expr {
		$n < 2 ? 1 : {expr {$n * [facx [-- n]]}}
	}
} # */


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
/*
puts "Line :"#__LINE__
trace add expr
# eval: Command or proc '}' not found (autoexec is OFF)
# {Error found in expr. Check expression arguments}
proc facx {n} {													# FAIL - [expr] arguments may not be internally braced
	textcolor white
	puts "Enter facx $n"
	textcolor
	expr {
		$n<2?1: {												# NOT ALLOWED!
			expr {$n * [facx [-- n]]}
		}
	}
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#/*
puts "Line :"#__LINE__
trace add expr
# FIX eval: Command or proc '}' not found (autoexec is OFF)
#	{Error found in expr. Check expression arguments}
proc facx {n} {													# PASS 13/Sep/2018
	textcolor white
	puts "Enter facx $n"
	textcolor
	expr {
		$n<2?1: expr {$n * [facx [-- n]]}
	}
}
# */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
/*
option echo on
puts "Line :"#__LINE__
proc facx n {													# PASS (stable) 13/Sep/2018
	if {$n < 2} {
		1
	} elseif {$n > 20} {
		expr $n*($n-1)
	} else {
		expr $n * [facx [-- n]]
	}
} # */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#/*
proc facx n {					# Best (safe) PASS 13/Sep/2018
	if {$n < 2} {
		1
	} elseif {$n > 20} {
		0
	} else {
		expr $n * [facx [-- n]]
	}
} # */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# T E S T  C A S E S 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
puts "Line :"#__LINE__

option echo on
trace add facx
# Values >= 20 are incorrect
puts "Line :"#__LINE__
assert [facx 1] {== $_ 1} -v -line #__LINE__
puts "Line :"#__LINE__
assert [facx 2] {== $_ 2} -v -line #__LINE__
puts "Line :"#__LINE__
assert [facx 5] {== $_ 120} -v -line #__LINE__

puts "Line :"#__LINE__
assert [facx 10] {== $_ 3628800} -v -line #__LINE__
puts "Line :"#__LINE__
# 9223372036854775807 INT_MAX
# 6402373705728000  (16)
assert [facx 14] {== $_ 87178291200} -v -line #__LINE__		# OK
# here!
assert [facx 15] {== $_ 1307674368000} -v -line #__LINE__		# OK
# exit
puts "Line :"#__LINE__
assert [facx 16] {== $_ 20922789888000} -v -line #__LINE__		# OK
puts "Line :"#__LINE__
assert [facx 17] {== $_ 355687428096000} -v -line #__LINE__		# OK
puts "Line :"#__LINE__
assert [facx 18] {== $_ 6402373705728000} -v -line #__LINE__		# OK
puts "Line :"#__LINE__
assert [facx 19] {== $_ 121645100408832000} -v -line #__LINE__	# GPF
puts "Line :"#__LINE__
assert [facx 20] {== $_ 2432902008176640000} -v -line #__LINE__	# 2432902008176640000

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
puts "Line :"#__LINE__

/* 64-bit overflows from here ...
#assert [facx 20] {== $_ 2432902008176640000} -v -line #__LINE__
# int max                  9223372036854775807
#                           unsigned: 4249290049419214848
assert [unsigned [facx 21]] {== $_ 51090942171709440000} -v -line #__LINE__		# -14197454024290336768
assert [facx 200] {== $_ "788657867364790503552363213932\
18506229513597768717326329474253324435944996340334292\
03042840119846239041772121389196382993111346285727076\
33172373969889439224456214516642402540332918641312274\
28294853277524242407573903240321257405579561789222227\
89623703897374720000000000000000000000000000000000000\
000000000000"} -v -line #__LINE__
#assert [facx 1000] {== $_ 999000} -v -line #__LINE__
# */
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

puts "Line :"#__LINE__
undef facx -nocomplain
trace remove facx
unset i n -nocomplain

option expression pop

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #