#####################################################################
# File:				test_nestproc.tcl
# Purpose:			Test nested procs
# Author:			M Shaw
# Language:			Ticol Tcl v1.19+, v1.26+
#
# Versions:
# 1.00 ??/???/2015	First version
# 1.01 30/Jan/2022	Updated
#####################################################################
# Notes:
#####################################################################

#####################################################################
# [proc]
#####################################################################

textcolor yellow
puts "Testing nested proc definitions and calling ..."
textcolor
newline

set i 0

proc level_p {} {			# Declare a proc which declares and calls nested procs
	puts "in proc level_p ([info level])"
	set ::i 1
	proc level_q {} {
		puts "in proc level_q ([info level])"
		+= ::i 2
		proc level_r {} {
			puts "In proc level_r ([info level])"
			+= ::i 4
		}
		level_r
	}
	level_q
}

level_p			# Call the enclosing proc p->q->r

assert $i {== $_ 7} -v -line #__LINE__			# Increments should total 7

puts "defined level_p [defined level_p]"
assert [defined level_p] {== $_ 1} -v -line #__LINE__
assert [defined level_q] {== $_ 1} -v -line #__LINE__
assert [defined level_r] {== $_ 1} -v -line #__LINE__

undef level_p
assert [defined level_p] {== $_ 0} -v -line #__LINE__
undef level_q
assert [defined level_q] {== $_ 0} -v -line #__LINE__
undef level_r
assert [defined level_r] {== $_ 0} -v -line #__LINE__
newline

#####################################################################
# [sub]
#####################################################################

textcolor yellow
puts "Testing nested sub definitions and calling ..."
textcolor
newline

set i 0

sub level_p {			# Declare a proc which declares and calls nested procs
	puts "in sub level_p ([info level])"
	set ::i 1
	sub level_q {
		puts "in sub level_q ([info level])"
		+= ::i 2
		sub level_r {
			puts "In sub level_r ([info level])"
			+= ::i 4
		}
		level_r
	}
	level_q
}

level_p			# Call the enclosing proc p->q->r

assert $i {== $_ 7} -v -line #__LINE__			# Increments should total 7

puts "defined level_p [defined level_p]"
assert [defined level_p] {== $_ 1} -v -line #__LINE__
assert [defined level_q] {== $_ 1} -v -line #__LINE__
assert [defined level_r] {== $_ 1} -v -line #__LINE__

undef level_p
assert [defined level_p] {== $_ 0} -v -line #__LINE__
undef level_q
assert [defined level_q] {== $_ 0} -v -line #__LINE__
undef level_r
assert [defined level_r] {== $_ 0} -v -line #__LINE__


#####################################################################
/*
textcolor yellow
puts "Defining nested procedure: define proc ..."
textcolor

proc define {name content} {
    proc $name {} [list expand $content]
}
 
# */
#####################################################################
unset i -nocoomplain
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

 
 # EOF #
