#####################################################################################
# File:				test_nullcmp.tcl
# Author:			M Shaw
# Purpose:			Tests for NULL string comparisons
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 12/Nov/2016	First version. Involved some mods to process_expression,
#					picolEval and picolCommandExpr
# 1.01 01/Jul/2017	Replace [checkex] with [assert]
#####################################################################################
# Notes
# I think this tests most or all of the relevant cases
# DOC expr: Missing operand(s) at 'eq' (arity N)
# 	This means one or other arguments are NULL and may require wrapping in braces
#	Problem seems to arise with vars in quoted strings as args are not natively
#	concatenated with whitespace and braces (?)
# FIX parse brace in "[expr {" ..."
# 	line 68: x:[" == ] == y:[1] should not pass
#####################################################################################
option echo on
option expression on

textcolor yellow
puts "Testing NULL string comparison with expr"
textcolor
newline

puts "Line: "#__LINE__
set a ""
set b ""
set c fred

# TRUE
textcolor yellow
puts "Case A1 (non var ==)"
textcolor
puts "Line: "#__LINE__
assert [expr "" == ""] 1 -v -line #__LINE__
assert [expr {} == {}] 1 -v -line #__LINE__
assert [expr {} == ""] 1 -v -line #__LINE__
assert [expr "" == {}] 1 -v -line #__LINE__

##################################################################################
# Quote or brace
# FAULT only when running tests.tcl (not standalone)
# standalone: 
#	x:[" == ] == y:[1]
# from tests.tcl...
#	Error in test at after line 66
#	'" == ' != '1'
#puts {"" == ""}		# PASS
#	 calling picolExpr(i,in:'"" == ""')
#	@58059 Calling process_expression: '"" == ""'
#	@58073 Called process_expression: result ptr:'" == ' i->retcode=0
# FAULT is in process_expression()
#		*** ENTER process_expression:'"" == ""' 
#		level:1 Char '"' caused Lvalue push '" == ' to stack (pushed_vars==0)
#		Calling get_operator() at char '"'
#		assert operator at: '"'
#		Level 1, Tail End Stack: (opbuf [])
# fault order is brace, dquote null
#	FIXES: 
#		replace "" with {} in picolExpr()?
##################################################################################

puts "Line: "#__LINE__
assert [expr {"" == ""}] 1 -v -line #__LINE__
#stop
#checkex [expr {"" == ""}] {$x == 1}
assert [expr "{} == {}"] 1 -v -line #__LINE__
assert [expr {{} == ""}] 1 -v -line #__LINE__
assert [expr {"" == {}}] 1 -v -line #__LINE__	# FAIL

##########################################################################
textcolor yellow
puts "Case A2 (non var eq)"
textcolor
#	cannot have assert [expr """ eq """] 1
#	assert [expr "\"\" eq \"\""] 1		# FAILs (does not escape dquotes)
#/*
puts "Line: "#__LINE__
puts '[expr "" eq ""]'
assert [expr "" eq ""] {== $_ 1} -v -line #__LINE__
assert [expr {} eq {}] {== $_ 1} -v -line #__LINE__
assert [expr {} eq ""] {== $_ 1} -v -line #__LINE__
assert [expr "" eq {}] {== $_ 1} -v -line #__LINE__
# Quote or brace
assert [expr {"" eq ""}] {== $_ 1} -v -line #__LINE__ # FAIL
assert [expr "{} eq {}"] {== $_ 1} -v -line #__LINE__
assert [expr {{} eq ""}] {== $_ 1} -v -line #__LINE__
assert [expr {"" eq {}}] {== $_ 1} -v -line #__LINE__ # FAIL
# exit
# */

##########################################################################
textcolor yellow
puts "Case A3 (non var !=)"
textcolor
puts "Line: "#__LINE__
assert [expr "" != ""] {== $_ 0} -v -line #__LINE__ 
assert [expr {} != {}] {== $_ 0} -v -line #__LINE__ 
assert [expr {} != ""] {== $_ 0} -v -line #__LINE__ 
assert [expr "" != {}] {== $_ 0} -v -line #__LINE__ 
# Quote or brace
assert [expr {"" != ""}] {== $_ 0} -v -line #__LINE__  # FAIL
assert [expr "{} != {}"] {== $_ 0} -v -line #__LINE__ 
assert [expr {{} != ""}] {== $_ 0} -v -line #__LINE__ 
assert [expr {"" != {}}] {== $_ 0} -v -line #__LINE__ 

##########################################################################
textcolor yellow
puts "Case A4 (non var ne)"
textcolor
puts "Line: "#__LINE__
assert [expr "" ne ""] {== $_ 0} -v -line #__LINE__ 
assert [expr {} ne {}] {== $_ 0} -v -line #__LINE__ 
assert [expr {} ne ""] {== $_ 0} -v -line #__LINE__ 
assert [expr "" ne {}] {== $_ 0} -v -line #__LINE__ 
# Quote or brace
assert [expr {"" ne ""}] {== $_ 0} -v -line #__LINE__ 
assert [expr "{} ne {}"] {== $_ 0} -v -line #__LINE__ 
assert [expr {{} ne ""}] {== $_ 0} -v -line #__LINE__ 
assert [expr {"" ne {}}] {== $_ 0} -v -line #__LINE__ 

##########################################################################
set a ""	
textcolor yellow
puts "Case B ('$a' == TRUE)"		# FIX failure to dereference a
textcolor
puts "Line: "#__LINE__
assert [expr $a == {}] {== $_ 1} -v -line #__LINE__
assert [expr $a == ""] {== $_ 1} -v -line #__LINE__
assert [expr $a == "{}"] {== $_ 1} -v -line #__LINE__
# FIX expression-handler - should be able to push "{}" var then decode "==" ok
#assert [expr $a==$b] {== $_ 1} -v -line #__LINE__
#assert [expr "$a==$b"] {== $_ 1} -v -line #__LINE__

##########################################################################
# FALSE
textcolor yellow
puts "Case C (* == c FALSE)"
textcolor
puts "Line: "#__LINE__
assert [expr "" == $c] {== $_ 0} -v -line #__LINE__ 
assert [expr $a == $c] {== $_ 0} -v -line #__LINE__ 
assert [expr $a == $c] {== $_ 0} -v -line #__LINE__ 
assert [expr $a == $c] {== $_ 0} -v -line #__LINE__ 
#assert [expr $a==$c] {== $_ 0} -v -line #__LINE__ 
#assert [expr "$a==$c"] {== $_ 0} -v -line #__LINE__ 

##########################################################################
# FALSE
textcolor yellow
puts "Case D ('$c' == * FALSE)"
textcolor
puts "Line: "#__LINE__
assert [expr $c == ""] {== $_ 0} -v -line #__LINE__ 
assert [expr $c == {}] {== $_ 0} -v -line #__LINE__ 
assert [expr $c == ""] {== $_ 0} -v -line #__LINE__ 
assert [expr $c == "{}"] {== $_ 0} -v -line #__LINE__ 
#assert [expr $c==$b] {== $_ 0} -v -line #__LINE__ 
#assert [expr "$c==$b"] {== $_ 0} -v -line #__LINE__ 

##########################################################################
option echo on
set a ""
set b ""
# eq
textcolor yellow
puts "Case E (a '$a' eq TRUE)"
textcolor
puts "Line: "#__LINE__
#assert [expr {} eq {}] {== $_ 1} -v -line #__LINE__
#assert [expr "" eq ""] {== $_ 1} -v -line #__LINE__
#assert [expr "{} eq {}"] {== $_ 1} -v -line #__LINE__
assert [expr "" eq ""] {== $_ 1} -v -line #__LINE__
assert [expr $a eq {}] {== $_ 1} -v -line #__LINE__
assert [expr $a eq ""] {== $_ 1} -v -line #__LINE__
halt

# ENTER picolCommandExpr: ' eq '
# 0: 'expr'
# 1: ' eq '
# Activestate
#set a ""
#% puts [expr $a eq "{}"]
#missing operand at _@_
#in expression " _@_eq {}"
#WORKS % puts [expr {$a} eq "{}"]
#1
# ActiveState Workaround {""}
puts "Line: "#__LINE__
assert [expr {$a} eq "{}"] {== $_ 1} -v -line #__LINE__		# EXITS after
assert [expr {$a} eq {$b}] {== $_ 1} -v -line #__LINE__# FAIL
assert [expr "{$a} eq {$b}"] {== $_ 1} -v -line #__LINE__# FAIL

##########################################################################
# FALSE
textcolor yellow
puts "Case F (a eq FALSE)"
textcolor
puts "Line: "#__LINE__
assert [expr "" eq $c] {== $_ 0} -v -line #__LINE__ 
assert [expr $a eq $c] {== $_ 0} -v -line #__LINE__ 
assert [expr $a eq $c] {== $_ 0} -v -line #__LINE__ 
assert [expr $a eq $c] {== $_ 0} -v -line #__LINE__ 
assert [expr {$a} eq {$c}] {== $_ 0} -v -line #__LINE__ 
#catch {assert [expr $a eq $c] 0}
assert [expr $a eq $c] {== $_ 0} -v -line #__LINE__ # PASS 

#set a ""
#set b ""
#set c fred

puts "Line: "#__LINE__
puts "Catch missing operand ... @"#__LINE__
puts "\[expr \"$a eq $c\"\]"
catch {assert [expr "$a eq $c"] {== $_ 0} -v -line #__LINE__ 
}		# Missing operand
assert [expr "{$a} eq {$c}"] {== $_ 0} -v -line #__LINE__ 

##########################################################################
# FALSE
textcolor yellow
puts "Case G (c eq FALSE) @"#__LINE__
textcolor
puts "Line: "#__LINE__
assert [expr $c eq ""] {== $_ 0} -v -line #__LINE__ 
assert [expr $c eq {}] {== $_ 0} -v -line #__LINE__ 
assert [expr $c eq ""] {== $_ 0} -v -line #__LINE__
assert [expr $c eq "{}"] {== $_ 0} -v -line #__LINE__ 
assert [expr $c eq $b] {== $_ 0} -v -line #__LINE__ 
assert [expr $c eq {$b}] {== $_ 0} -v -line #__LINE__ 

puts "Line: "#__LINE__
puts "Catch missing operand (IGNORE) ..."
puts "\[expr \"$c eq $b\"\]"
set i 0
try { 
	expr "$c eq $b"
} catch {		# Missing operand
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__ 

##########################################################################
textcolor yellow
puts "Case H (c ne TRUE) @"#__LINE__
textcolor
puts "Line: "#__LINE__
assert [expr $c ne ""] {== $_ 1} -v -line #__LINE__
assert [expr $c ne {}] {== $_ 1} -v -line #__LINE__
assert [expr $c ne ""] {== $_ 1} -v -line #__LINE__
assert [expr $c ne "{}"] {== $_ 1} -v -line #__LINE__
assert [expr $c ne $b] {== $_ 1} -v -line #__LINE__

puts "Catch missing operand (IGNORE) ... @"#__LINE__
set i 0
puts "\[expr \"$c eq $b\"\]"		# "fred eq "
try { 
	expr "$c ne $b"			# Missing operand
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
assert [expr "$c ne {$b}"] {== $_ 1} -v -line #__LINE__
assert [expr "{$c} ne {$b}"] {== $_ 1} -v -line #__LINE__
assert [expr "{$c} ne {sally}"] {== $_ 1} -v -line #__LINE__
assert [expr "{$c} eq {fred}"] {== $_ 1} -v -line #__LINE__

puts "Line: "#__LINE__
unset a b c i -nocomplain
vars

puts "Line: "#__LINE__
#####################################################
newline
textcolor green
puts "* Done *"
textcolor

 # EOF #