#################################################################################
# File: 			test_on.tcl
# Purpose:			Tests for [on]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 02/Oct/2018	First version (new command)
#################################################################################
# Notes:
# DONE FIX leak in [switch]
# Emulates BASIC's ON...GOTO or ON...GOSUB
# Also includes performance test comparison with [switch]
# Test includes checking that [on] performs faster than [switch]
# $this added as it added nothing to performance cost (still 7ms / iter)
#################################################################################
option expression off
option echo on

textcolor yellow
puts "Testing \[on\] (flow control) ..." 
textcolor
newline


# Experimental rapid loop (NEW)
# An attempt to out perform [if...elseif] ladders and [switch]

/*					# ADD $this
on [rnd 0 5] 		# 5 is error case

	{puts "Case $this"}
	
	{puts "Case $this"}
	
	{puts "Case $this"}
	
	{puts "Case $this"}

} #*/


/*
#set i [rnd 4 10]
loop i -1 12 1 {
#set i 5
	puts "on $i"
	on $i {
		puts "Case 1"		# Argv 3 - Jump 1
	} {
		puts "Case 2"		# Argv 4 - Jump 2
	} {
		puts "Case 3"		# Argv 5 - Jump 3
	} {
		puts "Case 4"		# Argv 6 - Jump 4
	} -default {			# Argv 7
		puts "Case default"	# Argv 8 - Jump 5+
	}
} # */


# Test invalid low value indexes
assert [on -255 {return 1}] {== $_ 1} -v -line #__LINE__
assert [on -2 {return 1}] {== $_ 1} -v -line #__LINE__
assert [on -1 {return 1}] {== $_ 1} -v -line #__LINE__
assert [on 0 {return 1}] {== $_ 1} -v -line #__LINE__



# Test $this with valid case
on 2 {
} {
	assert $this {== $_ 2} -v -line #__LINE__
}


# Test $this with -default
on 23 {
} {
} -default {
	assert $this {== $_ 23} -v -line #__LINE__
}

# Test valid indexes
assert [on 1 {return 2}] {== $_ 2} -v -line #__LINE__
assert [on 2 {
		return 1
	} {
		return 2
	} ] {== $_ 2} -v -line #__LINE__


# Test out of range (high) index 
puts "Line: "#__LINE__
set fail 0
try {
on 69 {
		return 1
	} {
		return 2
	}
} catch {
	set fail 1
}
assert $fail {== $_ 1} -v -line #__LINE__


# Test absent (but required) default case
set i 0
catch {
assert [on 3 {
		return 1
	} {
		return 2
	} ] {== $_ 3} -v -line #__LINE__
	set i 1					# Should raise exception and not set i to 1
}
assert $i {== $_ 0} -v -line #__LINE__


# Test default case
assert [on 3 {
		return 1
	} {
		return 2
	} -default {
		return 3
	}
] {== $_ 3} -v -line #__LINE__

##################################################################
# We test case 0 in these examples. Will match default case
##################################################################

newline
textcolor yellow
puts "Testing performance"
puts "* time \[on\] empty loop (4 cases + default)"			# 7ms / iter
textcolor
newline

set loops 10000			# The higher the value, the more accurate#

set time_on [time {
	on [rnd 0 5] {	# 1
	} {				# 2
	} {				# 3
	} {				# 4
	} -default {
	}
} $loops -info]
newline



textcolor yellow
puts "* time \[switch\] empty loop (4 cases + default)"		# 11ms / iter
textcolor
newline

set time_switch [time {
	switch [rnd 0 5] {
		1 {}
		2 {}
		3 {}
		4 {}
		default {}
	}
} $loops -info]
newline
newline
#ifdef comment	

if  {> [- $time_switch $time_on] 0} {
	textcolor green
	puts "Difference: \[on]\] was faster than \[switch\] by [- $time_switch $time_on] ms"
	textcolor
} else {
	textcolor red
	puts "FIX ME - switch was faster than on by [- $time_on $time_switch]"
	textcolor
}
newline

/*
# Disabled as the difference is only between 0ms and 5ms
puts "If the following test fails then \[on\] was slower than \[switch\] (which ought not happen!)"
assert [-  $time_switch $time_on] {>= $_ 0} -v -line #__LINE__
newline	# */

textcolor yellow
puts "* time \[if..elseif..else\] empty loop (4 cases + default)"		# 21ms / iter
textcolor
newline

set time_if [time {
	set i [rnd 0 5]				# [if] requires an additional [set] call (which will slow down)
	if {== $i 1} {				# Empty cases
	} elseif {== $i 2} {
	} elseif {== $i 3} {
	} elseif {== $i 4} {
	} else {					# Default case
	}
} $loops -info]

# man example #
set ch "c"
on [strchr "abcde" $ch] {
	puts "a was found"
} {
	puts "b was found"
} {
	puts "c was found"
} -default {
	puts "Some other char was found"
}



#endif

# Clean up #
unset i ch fail loops this time_if time_on time_switch -nocomplain

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #