######################################################################
# File:			test_option_expression.tcl
# Author:		M Shaw
# Purpose:		Test code for option expression
# Versions:
# 25/May/2015	First version (implemented)
# 20/Jan/2022	Updated and make a unit test
######################################################################
# Notes:
# [for] [if] [while]
######################################################################

textcolor yellow
puts "Testing option expression on logic for \[for\] \[while\] \[if] ..."
textcolor
newline

######################################################################
# Expression off
######################################################################

textcolor yellow
puts "Testing eval logic..."
textcolor
option expression off

## for #################################
for {set i 0} {< $i 10} {++ i} {}
assert $i {== $_ 10} -v -line #__LINE__

## while ############################
set i 0
while {< $i 10} {++ i}
assert $i {== $_ 10} -v -line #__LINE__

## if ###############################
set q 10
set i 0
if {== $q 10} {set i 1}
assert $i {== $_ 1} -v -line #__LINE__

## Fail cases ##

## for #################################
set j 0
for {set i 0} {$i < 10} {++ i} {		# This test requires a secondary increment var to avoid infinite loop
	if {>= $j 20} break
	++ j
}
assert $j {== $_ 20} -v -line #__LINE__		# Should enter an infinte loop, trapped by j

## while ############################
set i 0
while {$i < 10} {++ i}
assert $i {== $_ 0} -v -line #__LINE__

## if ###############################
set q 0	# Must be zero value
set i 10
if {$q == 10} {set i 1}
assert $i {== $_ 10} -v -line #__LINE__


######################################################################
# Expression on
######################################################################

textcolor yellow
puts "Testing expression logic..."
textcolor
option expression on

## for #################################
for {set i 0} {$i < 10} {++ i} {}
assert $i {== $_ 10} -v -line #__LINE__

## while ############################
set i 0
while {$i < 10} {++ i}
assert $i {== $_ 10} -v -line #__LINE__

## if ###############################
set q 10
set i 0
if {$q == 10} {set i 1}
assert $i {== $_ 1} -v -line #__LINE__

# FAIL CASES ###########################
# These will work
## for #################################
option expression on
set i 0
for {set i 0} {< $i 10} {++ i} {			# Expr passes
	#puts $i
}
assert $i {== $_ 10} -v -line #__LINE__

## while ############################
option expression on
set i 0
while {< $i 10} {							# Expr passes
	#puts $i
	++ i
}
assert $i {== $_ 10} -v -line #__LINE__

## if ###############################
option expression on
set q 0			# 1st value must be 0
set i 10
if {== $q 10} {set i 1}		# Should not set to 1
assert $i {== $_ 10} -v -line #__LINE__


/*
option expression off
puts "Testing eval logic..."
for {set i 0} {< $i 10} {++ i} {
	set j 0
	while {< $j 10} {
		if {> $j 5} {
			puts "j>5 ($j)"
			if {>= $j 7} {
				puts "Break at i:$i j:$j"
				break
			}
		}
		++ j
	}
	if {>= $i 3} {
		puts "Break for() with i at $i"
		break
	}
}
puts "Done."
newline

option expression on
puts "Testing expression logic..."
for {set i 0} {$i < 10} {++ i} {
	set j 0
	while {$j <  10} {
		if {$j > 5} {
			puts "j>5 ($j)"
			if {$j >= 7} {
				puts "Break at i:$i j:$j"
				break
			}
		}
		++ j
	}
	if {$i >= 3} {
		puts "Break for() with i at $i"
		break
	}	
}
# */
######################################################################
unset i j q -nocomplain
vars
newline

textcolor green
puts "* Done *"
textcolor
# EOF #