#############################################################################
# File:				test_pass_array_to_proc.tcl
# Author:			M Shaw
# Purpose:			Test file for passing arrays to/from [proc] via [upvar]
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 06/Jun/2017	First version
#############################################################################
# Notes:
#############################################################################

option expression off
textcolor yellow
puts "Testing passing arrays to/from \[proc\] ..."
textcolor
puts "Line: "#__LINE__
set calls 0

undef foo -nocomplain

proc foo x {
	++ ::calls			# Increment the global
	upvar $x			# Create a var link to the array
	
	assert $$x {eq $_ "<array>"} -v -line #__LINE__
	assert [array size $x] {== $_ 3} -v -line #__LINE__
	assert ${${x}(a)} {== $_ 10} -v -line #__LINE__
	assert ${${x}(b)} {== $_ 20} -v -line #__LINE__
	assert ${${x}(c)} {== $_ 30} -v -line #__LINE__
	assert [array is_set ${x}(d)] {== $_ 0} -v -line #__LINE__
	
	#array walk $x
	#set y $x
	#puts $y
	#puts $y(a)
	set ${x}(a) 5		# Because we use a var for the array name we must dereference it before use
	set ${x}(b) 15
	set ${x}(c) 25
	set ${x}(d) 35		# Add a new item
}

array unset a -nocomplain

textcolor yellow
puts "Setting array a"
textcolor

array set a {
	a 10
	b 20
	c 30
}

textcolor yellow
puts "Calling proc \[foo\] ..."
textcolor
foo a 					# Call the proc

#time {foo a} 1000000		# No leaks - 06/Jun/2017

textcolor yellow
puts "After calling proc \[foo\] ..."
textcolor

newline
textcolor magenta
puts "Calls == $calls"
puts "Final result in array a size: [array size a]"
textcolor

array walk a

textcolor yellow
puts "\nFinal checks after calling proc \[foo\] ..."
textcolor
assert [array size a] {== $_ 4} -v -line #__LINE__	 We should have added x1 element
assert $a(a) {== $_ 5} -v -line #__LINE__
assert $a(b) {== $_ 15} -v -line #__LINE__
assert $a(c) {== $_ 25} -v -line #__LINE__
assert $a(d) {== $_ 35} -v -line #__LINE__

unset calls -nocomplain
array unset a

#############################################################################
vars 

newline
textcolor green
puts "* Done *"
textcolor

# EOF #