###################################################################################
# File: 			test_pathstrip.tcl
# Purpose:			Test for [file pathstrip]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 06/Sep/2018	First version
###################################################################################
# Notes:
# Either double-slash escaped Windows or single / slash unix path separators are
# accepted as input
# If N is 0 then the whole path is returned
# If N argumnent is > than count of path segments then "" is returned
# Default return style is Tcl double slash escaped Windows path separators
# -unix will force single / Unix style path separator return
###################################################################################

textcolor yellow
puts "Testing \[file pathstrip\] ..."
textcolor
newline
textcolor yellow
puts "Testing Windows paths ..."
textcolor
newline

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 1]'"
assert [file pathstrip $path 1] {eq $_ "c:\\windows\\system32\\foo\\bar\\moo"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 2]'"
assert [file pathstrip $path 2] {eq $_ "c:\\windows\\system32\\foo\\bar"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 3]'"
assert [file pathstrip $path 3] {eq $_ "c:\\windows\\system32\\foo"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 4]'"
assert [file pathstrip $path 4] {eq $_ "c:\\windows\\system32"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 5]'"
assert [file pathstrip $path 5] {eq $_ "c:\\windows"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 6]'"
assert [file pathstrip $path 6] {eq $_ "c:"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:\\windows\\system32\\foo\\bar\\moo\\par\\"
puts "'[file pathstrip $path 7]'"									# N is > than count of path separators
assert [file pathstrip $path 7] {eq $_ ""} -v -line #__LINE__		# So return ""
newline

# Unix
textcolor yellow
puts "Testing Unix paths ..."
textcolor
newline

puts "Line: "#__LINE__
set path "c:/windows/system32/foo/bar/moo/par/"
puts "'[file pathstrip $path 1]'"
assert [file pathstrip $path 1] {eq $_ "c:\\windows\\system32\\foo\\bar\\moo"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:/windows/system32/foo/bar/moo/par/"
puts "'[file pathstrip $path 6]'"
assert [file pathstrip $path 6] {eq $_ "c:"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:/windows/system32/foo/bar/moo/par/"
puts "'[file pathstrip $path 1 -unix]'"		# Force Unix style return
assert [file pathstrip $path 1 -unix] {eq $_ "c:/windows/system32/foo/bar/moo"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:/windows/system32/foo/bar/moo/par/"
puts "'[file pathstrip $path 6]'"
assert [file pathstrip $path 6] {eq $_ "c:"} -v -line #__LINE__

puts "Line: "#__LINE__
set path "c:/windows/system32/foo/bar/moo/par/"
puts "'[file pathstrip $path 7]'"									# N is > than count of path separators
assert [file pathstrip $path 7] {eq $_ ""} -v -line #__LINE__		# So return ""




unset path

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #
