###########################################################################################
# File: 			test_pipe.tcl
# Purpose:			Experiment with a PowerShell-like pipe | command
#					(Possibly pointless) adventures in stringing words together
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 16/Nov/2019	First dev version
###########################################################################################
# Notes:
# FIX May change the syntax to take a well-formed Tcl list of arguments
###########################################################################################
# DONE FIX pipe char | is reserved for [|] OR. What if one of our args needs to be |?
# | is reminiscent of PowerShell but is reserved. Using the alternate bar char (221:0xdd)
# or ¦ would be confusing. (Used comma char as separator)
# Looking for linear set a [| foo bar] v's (potentially deeply) nested set a [bar [foo]]
# Problems: [| foo -nocomplain bar -something]
###########################################################################################

undef foo bar moo par -nocomplain		# Enable re-run from CLI

# Chained value is 1st arg now
proc foo {a} {
	textcolor darkyellow
	puts "\nIn proc foo(a: $a) returning '[* $a 23.0]'"
	textcolor
	return [* $a 23.0]
}

proc bar {a} {
	textcolor cyan
	puts "\nIn proc bar(a: $a) returning '[/ $a 11.0]'"
	textcolor
	return [/ $a 11.0]
}

proc moo {a op} {
	textcolor magenta
	puts "\nIn proc moo(a: $a op: $op) returning \[$op $a 5\] -> '[$op $a 5]'"
	textcolor
	return [$op $a 5]
}

proc par {a op val} {
	textcolor green
	puts "\nIn proc par(a: $a op: $op val: $val) returning \[$op $a $val\] -> '[$op $a $val]'"
	textcolor
	return [$op $a $val]
}

###########################################################################################
/*
option expression on
# set a | [foo] * [bar]
#set a [eval {[foo] / [bar]}]	# Wrong. Uses infix
set a [eval {/ [foo] [bar]}]
set b [expr [foo] / [bar]]
puts $a
puts $b
# */
###########################################################################################

#set a [pipe 2 | foo | bar | moo + | par / 0]		# Deliberate DBZ
#set a [pipe 2 | foo]		# no proc call


assert [par [moo [bar [foo 2]] +] / $pi] {== $_ 2.922663500414805} -v -line #__LINE__
set a [pipe 2 , foo , bar , moo + , par / $pi]		# DBZ
puts "a is $a"
# 2 x 23 == 46, / 11 == 4.181818181818182, + 5 == 9.1818..., / $pi == 2.922663500414805
assert $a {== $_ 2.922663500414805} -v -line #__LINE__
assert [par [moo [bar [foo 2]] +] / $pi] {== $_ 2.922663500414805} -v -line #__LINE__
assert [pipe foo 2  ,  bar  ,  moo +  ,  par / $pi]  {== $_ 2.922663500414805} -v -line #__LINE__
#puts "a is '$a'"
unset a

# All pass
assert [pipe 2  ,  * 3] {== $_ 6} -v -line #__LINE__
assert [pipe 2  ,  * 3  ,  + 4] {== $_ 10} -v -line #__LINE__
assert [pipe 2  ,  * 3  ,  + 4  ,  / 5.0] {== $_ 2} -v -line #__LINE__
assert [pipe 2  ,  * 3  ,  + 4  ,  / 5.0  ,  - 1] {== $_ 1} -v -line #__LINE__

# Note that a piped series expression will be evauated LTR
puts " expr -> [expr ((((2 * 3) + 2) / 2) - 1) * $pi]"
assert [pipe  * 2 3  ,  + 2  ,  / 2  ,  - 1  ,  * $pi] {== $_ 9.424777960769379} -v -line #__LINE__

###########################################################################################
# Man e.g.
undef foo bar moo -nocomplain
proc foo {a} {
	return [* $a 23.0]
}
proc bar {a op} {
	return [$op $a 5]
}
proc moo {a op val} {
	return [$op $a $val]
}
# Usage
# 2 x 23=46; +5=51; /pi = 16.233804195373324
puts [pipe 2, foo, bar +, moo / $pi]
assert [pipe 2, foo, bar +, moo / $pi] {== $_ 16.233804195373324} -v -line #__LINE__

###########################################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #