##################################################################
# File: 			test_proc.tcl
# Purpose:			Test file for [proc]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.19+
#
# Versions:
# 1.00 16/Aug/2016	First version
# 1.01 18/Jul/2017	Added tests, replace [checkex] with [assert]
##################################################################
# Notes: 
##################################################################
# ADD default args
##################################################################

set i 0

#option echo on
option expression off

textcolor yellow
puts "Testing \[proc\] ..."
textcolor
newline

set i 0
/*
# DONE TODO Implement default args
#ifdef defaultargs
undef mult -nocomplain
if {! [defined mult] } {
	proc mult {varName {multiplier 2}}{
		upvar $varName var
		set var [calc {var * multiplier}]
	}
}
#endif
*/

#define noargs
#ifdef noargs
puts "Line: "#__LINE__
if {! [defined {}] } {		# The empty proc #
	textcolor yellow
	puts "* Defining empty proc"
	textcolor
	proc {} {}  {
		puts "* In \[proc \{\}\]"
		return "empty"
	}
}
assert [call {}] {eq $_ empty} -v -line #__LINE__
undef {} -nocomplain
#endif



# simple proc with math ########################################
unset q -nocomplain
undef q -nocomplain							# FIX Borked with procs
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with math"
	textcolor
	proc q {} {
		+ [* 2 2] 1 
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


undef q							# FIX Borked with procs
# simple proc with eval ########################################
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with eval"
	textcolor
	proc q {} {
		eval {+ [* 2 2] 1}
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


undef q							# FIX Borked with procs
# simple proc with expr ########################################
if {! [defined q] } {
	textcolor yellow
	puts "Defined proc with expr"
	textcolor
	proc q {} {
		expr (2*2)+1
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


undef q
# simple proc with set ########################################
if {! [defined q] } {
	textcolor yellow
	puts "Defined proc with set"
	textcolor
	proc q {} {
		set i 5
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


undef q
# simple proc with return ########################################
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with fixed return"
	textcolor
	proc q {} {
		return 5
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


undef q
# proc with for / if ########################################
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with for / if"
	textcolor
	proc q {} {
		for {set i 0} {< $i 10} {incr i} {
			if {== $i 5} {return $i}
		}
		return -1
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


# proc with for / break ########################################
# This doesn't really test [proc] just that result is set correctly on exiting an enclosed [for]
undef q
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with for/break"
	textcolor
	proc q {} {
		for {set i 0} {< $i 10} {incr i} {
			if {== $i 5} {break}				# if break is ineffective then i will be 10
		}
		return $i
	}
}
assert [q] {== $_ 5} -v -line #__LINE__				# PASS


# proc with if ##############################################
undef q -nocomplain
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with x1 arg and if"
	textcolor
	proc q {i} {
		textcolor cyan
		puts "Enter proc q $i"
		textcolor
		if {== $i 5} {return $i}
		return 0
	}
}
#puts [info args q]
assert [q 5] {== $_ 5} -v -line #__LINE__				# PASS
assert [q 1] {== $_ 0} -v -line #__LINE__				# PASS
assert [q 6] {== $_ 0} -v -line #__LINE__				# PASS


# proc with internal goto exit ##############################################
#puts "Line: "#__LINE__
undef q							# FIX Borked with procs
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with x1 arg and goto exit"
	textcolor
	proc q {i} {
		goto_block {
			start {
				# puts "Enter proc q $i"
				if {== $i 5} {goto quit}
				set i 0
				return $i
			}
			quit {set i 5}			# Will fall-through and exit
		}
	}
}
assert [q 5] {== $_ 5} -v -line #__LINE__				# PASS
assert [q 1] {== $_ 0} -v -line #__LINE__				# PASS
assert [q 6] {== $_ 0} -v -line #__LINE__				# PASS

/*
# proc with enclosing goto exit ##############################################
# This should not work as goto block is discarded after proc-init
# Code here is purely for curiousity purposes
#############################################################################
undef q							# FIX Borked with procs
#rename q ""
goto_block {
	init {if {! [defined q] } {
		textcolor yellow
		puts "Define proc with enclosing goto"
		textcolor yellow
		proc q {i} {
				textcolor cyan
				puts "Enter proc q $i"
				textcolor
				if {== $i 5} {goto quit}
				set i 0
			}
		}
	}
	quit {puts "at goto quit"; set i 5}			# Will fall-through and exit
 }

assert [q 5] {== $_ 5} -v -line #__LINE__				# PASS
*/


# proc with x2 args ##############################################
undef q							# FIX Borked with procs
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with x2 args with x1 default"
	textcolor
	proc q {i {j 10}} {
		textcolor cyan
		puts "Enter proc q $i $j"
		textcolor
		return [* $i $j]
	}
}
#puts [info args q]
assert [q 5] {== $_ 50} -v -line #__LINE__				# PASS
assert [q 0] {== $_ 0} -v -line #__LINE__				# PASS
assert [q 6 6] {== $_ 36} -v -line #__LINE__				# PASS

# proc with x3 args ##############################################
#puts "Line: "#__LINE__
undef q							# FIX Borked with procs
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with x2 args with x1 default"
	textcolor
	proc q {i {j 10} {k 2}} {
		textcolor cyan
		puts "Enter proc q $i $j $k"		
		textcolor
		return [* [* $i $j] $k]
	}
}
#puts [info args q]
assert [q 5] {== $_ 100} -v -line #__LINE__				# PASS
assert [q 0] {== $_ 0} -v -line #__LINE__				# PASS
assert [q 6 6] {== $_ 72} -v -line #__LINE__			# PASS
assert [q 6 6 6] {== $_ 216} -v -line #__LINE__			# PASS

# proc with x3 args ##############################################
undef q							# FIX Borked with procs
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with x2 args with x1 default"
	textcolor
	# FIX store "" {} in proc as literal 0x00 "" (not \"\")
	proc q {i {j {}} {k 2}} {
		#puts "Enter proc q '$i' '$j' '$k'"
		# if {[eq $j {}]} {
		if {[eq $j ""]} {		# PASS
			#puts "j is empty"
		}
		# if {[eq $j {}]} {
		if {[ne $j ""]} {		# PASS
			#puts "j ne \"\""
			return [* [* $i $j] $k]
		} else {
			return [* $i $k]
		}
	}
}
#puts [info args q]
assert [q 5] {== $_ 10} -v -line #__LINE__				# PASS
assert [q 0] {== $_ 0} -v -line #__LINE__				# PASS
assert [q 6 {}] {== $_ 12} -v -line #__LINE__				# PASS
assert [q 6 6 6] {== $_ 216} -v -line #__LINE__				# PASS
assert [q 6 {} 6] {== $_ 36} -v -line #__LINE__				# PASS

# proc with x3 args ##############################################
#puts "Line: "#__LINE__
undef q							# FIX Borked with procs
if {! [defined q] } {
	textcolor yellow
	puts "Define proc with x2 args with x1 default"
	textcolor
	# FIX store "" {} in proc as literal 0x00 "" (not \"\")
	proc q {i {j ""} {k 2}} {
		#puts "Enter proc q '$i' '$j' '$k'"
		# if {[eq $j {}]} {
		if {[eq $j ""]} {		# PASS
			#puts "j is empty"
		}
		# if {[eq $j {}]} {
		if {[ne $j ""]} {		# PASS
			#puts "j ne \"\""
			return [* [* $i $j] $k]
		} else {
			return [* $i $k]
		}
	}
}
#puts [info args q]
assert [q 5] {== $_ 10} -v -line #__LINE__				# PASS
assert [q 0] {== $_ 0} -v -line #__LINE__				# PASS
assert [q 6 {}] {== $_ 12} -v -line #__LINE__				# PASS
assert [q 6 6 6] {== $_ 216} -v -line #__LINE__				# PASS
assert [q 6 {} 6] {== $_ 36} -v -line #__LINE__				# PASS

#assert 0 {== $_ 1} -v -line #__LINE__

#option echo on
# Single space proc ...
proc { } {} {
	textcolor magenta
	puts "Foo!"
	textcolor
	return 23 -code ok
}

undef bar -nocomplain
proc bar {} {
	return 69
}
puts "Foo is '[{ }]'"
puts "Bar is '[bar]'"
assert [{ }] {== $_ 23} -v -line #__LINE__
assert [" "] {== $_ 23} -v -line #__LINE__
assert [bar] {== $_ 69} -v -line #__LINE__

undef bar { } -nocomplain

############################################################################
# LEAK TEST ([undef])
############################################################################
textcolor yellow
puts "[info script] - Testing proc leaks ..."
textcolor
newline

#2016/Aug/17 - 1644k to 1748k in one jump, but appear to be no progressive leaks 
undef q
set limit 1000
for {set i 0} {< $i $limit} {incr i } {
	proc q {i} {								# Define q
		return [* $i $i] -code ok
	}
	q											# Call q
	undef q										# Undefine and loop again
}

# Arg with defaults in 1st (leading) position
undef q -nocomplain
proc q {{x 23}} {
	puts "x is '$x'"
	assert $x {== $_ 23} -v -line #__LINE__
}
q
#$q

# Arg with defaults in 1st (leading) position, with potential command '*'
undef q -nocomplain
proc q {{x *}} {
	puts "x is '$x'"
	assert $x {eq $_ "*"} -v -line #__LINE__
}
q

# Arg with defaults in 1st (leading) position, with potential command '*' protected by dquotes
undef q -nocomplain
proc q {{x "*"}} {
	puts "x is '$x'"
	assert $x {eq $_ "*"} -v -line #__LINE__
}
q

undef q -nocomplain
unset i limit this -nocomplain

proc add2 name {
	upvar $name x
	puts "local x=$x"
	set x [expr {$x + 2}]
}
set i 21
set i [add2 i]
puts "i = $i"
assert $i {== $_ 23} -v -line #__LINE__
undef add2
unset i

###########################################################################
# ADD test return from proc
# FIX compare return from various structures and build a table of what is allowed
############################################################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #