######################################################################################
# File: 			test_proc_arch.tcl
# Purpose:			Tests for [proc] architecture integrity
# Language:			Ticol Tcl v1.25+
# Author:			M Shaw
#
# Versions:
# 1.00 06/Nov/2017	First version
######################################################################################
# Notes: 
# Test: scoping, visibility, leaks and persistence
# Pretty basic at the mo, improve later
######################################################################################

set line #__LINE__

option echo on
option expression off

textcolor yellow
puts "Testing \[proc\] architecture ..."
textcolor
newline

set procname ""

proc level_p {} {
	puts "in level_p"
	proc level_q {} {		
		puts "in level_q"
		proc level_r {} {
			if {eq $::procname ""} {
				set ::procname [info procname]
			}
		}
		level_r
	}
	level_q
}
level_p

newline
puts "Registered procs are: '[procs]'"
newline

assert [defined level_p] {== $_ 1} -v -line #__LINE__
assert [defined level_q] {== $_ 1} -v -line #__LINE__
assert [defined level_r] {== $_ 1} -v -line #__LINE__
assert $procname {== $_ "level_r"} -v -line #__LINE__
set proclist [info procs]
puts "proclist: '$proclist'"
assert [in $proclist level_p] {== $_ 1} -v -line #__LINE__

# Leak testing (no leaks)
# We can only call the innermost level without using if {! [defined procname} - which will skip the procs anyway
time {
	level_r
} 10000

undef level_p level_q level_r -nocomplain
#undef level* -nocomplain	# TODO add glob support
unset procname proclist -nocomplain

# Testing proc scope and visibility #
# proc level_p -> level_q -> level_r

proc level_p {} {
	assert [info procname] {eq $_ level_p} -v -line #__LINE__
	set p 23						# Inherit this var using different names, but with no change until return
	set s 11						# Inherit this var using different names, changing on the way out and back
	proc level_q {} {
		assert [info procname] {eq $_ level_q} -v -line #__LINE__
		upvar p q
		assert $q {== $_ 23} -v -line #__LINE__
		upvar s t
		set t 12
		proc level_r {} {
			assert [info procname] {eq $_ level_r} -v -line #__LINE__			
			upvar q r 
			assert $r {== $_ 23} -v -line #__LINE__
			upvar t u
			assert $u {== $_ 12} -v -line #__LINE__
			set r 66
			set u 99
			assert [info procname] {eq $_ level_r} -v -line #__LINE__			
		}
		level_r
		assert [info procname] {eq $_ level_q} -v -line #__LINE__
		assert $q {== $_ 66} -v -line #__LINE__
		assert $t {== $_ 99} -v -line #__LINE__
	}
	level_q
	assert [info procname] {eq $_ level_p} -v -line #__LINE__
	assert $p {== $_ 66} -v -line #__LINE__
	assert $s {== $_ 99} -v -line #__LINE__
}
level_p


undef level_p level_q level_r -nocomplain

unset line -nocomplain
vars

puts "Registered procs are: '[procs]'"

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Done *"
textcolor

# EOF #