######################################################################################
# File: 			test_proc_noname.tcl
# Purpose:			Test calling a procedure with no name
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 15/Jul/2018	First version
# 1.01 30/Sep/2019	Add [defined] test
######################################################################################
# Notes:
# Tests for mods to allow Tcl standard proc with no name
# Call using [call {}], [call ""], [{}], [""]
######################################################################################

option expression off

textcolor yellow
puts "Test calling a procedure with no name ..."
textcolor
newline

proc {} {} {return 23}    # Procedure with no name

# Test using direct reference
assert [{}] {== $_ 23} -v -line #__LINE__
assert [""] {== $_ 23} -v -line #__LINE__

# Test using [call]
assert [call {}] {== $_ 23} -v -line #__LINE__
assert [call ""] {== $_ 23} -v -line #__LINE__

undef {} -nocomplain
newline

# Test def of unnamed proc
proc {} {args} {
	set q {}
	foreach x $args {
		lappend q $x
	}
	return "+$q+"
}

puts "Result->'["" 1 2 3]'"
assert ["" 1 2 3] {eq $_ "+1 2 3+"} -v -line #__LINE__

set l ""
puts "Result->'[$l 1 2 3]'"
assert [$l 1 2 3] {eq $_ "+1 2 3+"} -v -line #__LINE__

# Test [defined] with noname
assert [defined {}] {== $_ 1} -v -line #__LINE__

undef {} -nocomplain
puts "Known procs: '[procs]'"
assert [defined {}] {== $_ 0} -v -line #__LINE__

unset l -nocomplain
vars

#############################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #