############################################################################
# File:				test_proc_return.tcl
# Author:			M Shaw
# Purpose:			Test file for [proc] with various returns
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 06/Jan/2017	First version
# 1.01 08/Jul/2017	Replaced [checkex] with [assert]
############################################################################
# Notes:
# As of Jan 2017, PICOL_RETURN was made level-dependent in picolEval
# This should remove dependencies for filtering in flow-control commands
############################################################################
option echo on
option expression off

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
option echo on
option expression push on
set i 0
while {$i < 10} { 
	if {[== $i 5]} {
		textcolor green ; 
		puts "*\treturning"; 
		textcolor;
		return -code break		# Returns at level 0 so picolEval will alter to OK
	} 
	incr i
}
# while {$i < 10} { if {[== $i 5]} {break} ; incr i}		# return will be at level 0 so returns OK and does not break
# while {$i < 10} { if {[== $i 5]} {return $i -code break} ; incr i}
textcolor cyan
puts "i==$i"
textcolor

# ActiveState 
# % while {$i < 10} { if {$i==5} {return $i} ; incr i}
# 5
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

textcolor yellow
puts "Testing \[proc\] \[return\]..."
textcolor
undef foo -nocomplain

proc foo {} {
	set q [rnd 0 1]
	if {[== $q 0]} {
		return $::false
	} else {
		return $::true
	}
	puts "FAILED returning 2"
	return 2
}

set q [foo]
puts "q==$q"
assert $q {!= $_ 2} -v -line #__LINE__

undef bar -nocomplain

proc bar {x} {
	if {$x} {
		return $::true
	}
	return $::false
}

set q [bar $true]
puts "q==$q"
assert $q {== $_ $::true} -v -line #__LINE__

set q [bar $false]
puts "q==$q"
assert $q {== $_ $::false} -v -line #__LINE__

undef par -nocomplain
proc par {x} {
	if {$x} {
		if {$x} {
			return $::true
		}
	}
	return $::false
}

set q [par $true]
#puts "q==$q"
assert $q {== $_ $::true} -v -line #__LINE__		


set q [bar $false]
#puts "q==$q"
assert $q {== $_ $::false} -v -line #__LINE__

undef mar -nocomplain
proc mar {x} {
	if {[== $x 1]} {
		return $::true
	} elseif {[== $x 2]} {
		return $::true
	} else {
		return $::true
	}
	return $::false
}

set q [mar 1]
#puts "q==$q"
assert $q {== $_ $::true} -v -line #__LINE__

set q [mar 2]
#puts "q==$q"
assert $q {== $_ $::true} -v -line #__LINE__

set q [mar 3]
#puts "q==$q"
assert $q {== $_ $::true} -v -line #__LINE__

# Non proc returns # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
catch {
set i 0
#/*
for {} {[< $i 10]} {++ i} {
	if {[== $i 5]} {
		return $i -code return			# Check return not valid from for loop
	}
	if {[> $i 5]} {
		puts "FAILED to return in for @ $i"
	}
}
#*/
puts "i is $i"
assert $i {== $_ 5} -v -line #__LINE__
}


set i 0
puts "i is $i"
for {} {[< $i 10]} {++ i} {
	return -code break			# 10/Jan/2017 no longer works with [return]
	puts "FAILED i is $i"
}
puts "i is $i"
assert $i {== $_ 0} -v -line #__LINE__

catch {

option echo on
option expression on
set i 0
puts "Start while"
while {$i < 10} { if {$i==5} {return $i} ; puts "i is $i"; incr i }		# Will exit the script unless [catch] present
puts "end while"
puts "i==$i"
option expression off
# ActiveState 
# % while {$i < 10} { if {$i==5} {return $i} ; incr i}
# 5
assert $i {== $_ 5} -v -line #__LINE__
}

catch {

set i 0
puts "i is $i"
while {[< $i 10]} {
	puts "while loop $i"
	break 							# Does not exit from return
	puts "FAILED i is $i"
	++ i
}
puts "i is $i"
assert $i {== $i 0} -v -line #__LINE__

}

catch {
	option expression off
	set i 0
	puts "i is $i"
	while {[< $i 1]} {
		++ i
		puts "while loop $i"
		set j 0
		while {[< $j 10]} {
			++ j
			if {[> $j 5]} {
				return
			}
			puts "while j $j"
		}
		break 							# Does not exit from return
		puts "FAILED i is $i"
		return $j
	}
	puts "i is $i"
	puts "j is $j"
	assert $i {== $i 0} -v -line #__LINE__
}

/*
set i 0
for {} {[< $i 10]} {++ i} {
	if {[== $i 5]} {
		break			# Check return not valid from for loop
	}
	if {[> $i 5]} {
		puts "FAILED to exit for @ $i"
	}
}
puts "i is $i"
assert $i {== $i 5} -v -line #__LINE__
*/

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
option expression off
unset i j q -nocomplain
vars

newline
textcolor green
puts "* \[proc ... return\] Done *"
textcolor

# EOF #