#################################################################################
# File: 			test_proc_static_array.tcl
# Purpose:			Tests for [static] arrays in [proc]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 10/Nov/2017	First version
#################################################################################
# Notes:  Called from tests.tcl
# TODO 
# ADD test [array set syntax]
#################################################################################
option echo on
option expression off

textcolor yellow
puts "Testing \[static\] in proc with arrays ..."
textcolor
newline

puts "Line: "#__LINE__

array unset a -nocomplain
undef foo -nocomplain

proc foo {x y} {
	puts "Line: "#__LINE__
	static a							# Declare a generic static var
	puts "a is [type a]"				# Confirm the type
	puts "foo::a is [type foo::a]"
	set a($x) $y						# Assign to local alias copy
	puts "foo::a is [type foo::a]"		# Curiously, this resolves
	puts "::foo::a is [type ::foo::a]"
	if {eq $y Hello} {
		assert $a($x) {eq $_ Hello} -v -line #__LINE__			# PASS
		assert $::foo::a($x) {eq $_ Hello} -v -line #__LINE__	# PASS
	} else {
		assert $a($x) {eq $_ World} -v -line #__LINE__			# PASS
		assert $::foo::a($x) {eq $_ World} -v -line #__LINE__	# PASS
	}
	puts "$::foo::a($x)"			# Access global copy
	
	set ::foo::a(3) "Three"			# Direct set in global (root) scope

	# vars -level 0					# PASS exists
	textcolor magenta
	array walk ::foo::a				# Fails
	textcolor
}

puts "Line: "#__LINE__
foo 1 Hello
puts "Line: "#__LINE__
foo 2 World
newline

puts "Line: "#__LINE__
textcolor yellow
puts "-- Call results ----------------------"
textcolor

textcolor darkyellow			# Orange
puts "1: $foo::a(1)"			# PASS (exists)
assert $foo::a(1) {eq $_ Hello} -v -line #__LINE__
textcolor darkyellow			# Orange
puts "2: $foo::a(2)"			# PASS (exists)
assert $foo::a(2) {eq $_ World} -v -line #__LINE__
textcolor darkyellow			# Orange
puts "2: $foo::a(3)"			# PASS (exists)
assert $foo::a(3) {eq $_ Three} -v -line #__LINE__

#vars 
#array walk foo::a

static foo::a -unset -nocomplain		# [static] variables need special removal
undef foo -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #