###################################################################################
# File: 			test_static_stack.tcl
# Purpose:			Test for static stack fixups
# Author: 			M Shaw
# Language:			Ticol Tcl v1.25+
#
# Versions:
# 1.00 11/Nov/2017	First version
###################################################################################
# Notes:
###################################################################################

option echo on
option expression off

textcolor yellow
puts "Testing \[static\] in proc with stack ..."
textcolor
newline

puts "Line: "#__LINE__
undef foo -nocomplain
undef static_stack -nocomplain


proc static_stack {name args} {
	set scope [uplevel 1 {info procname}]			# Save procname as new scope name (simpler to parse)
	set varname ${scope}::$name						# Create mangled root scope var name
#puts "Varname is '$varname'"
	if {! [is_set ::$varname]} {            		# If root var is empty and we have an arg
#puts "Creating..."
		uplevel @0 "stack create $varname $args"	# Create the new struct
		}
	uplevel 1 "upvar @0 $varname $name"     		# Run upvar in the root callframe
	#vars -level 0
	#vars
}

proc foo {x y} {
	static_stack $x 100
	puts "foo: Pushing $y onto stack $x"
	stack push $x $y
}

newline
foo s Hello
foo s World

newline
textcolor cyan
puts "Level 0 Results"
textcolor
newline
vars
newline


set i 1
stack foreach foo::s x {
	puts "stack item $i: -$x-"
	++ i
}

assert [stack pop foo::s] {eq $_ World} -v -line #__LINE__
assert [stack pop foo::s] {eq $_ Hello} -v -line #__LINE__

undef foo -nocomplain
undef static_stack -nocomplain

unset i x -nocomplain
stack unset foo::s
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Done *"
textcolor