######################################################################################
# File: 			test_range.tcl
# Purpose:			Test [range]
# Language:			Ticol Tcl v1.23+, v1.26+
# Caution:			Contains MPP executable code. Will run on any MPP call including /C
#
# Versions:
# 1.00 17/Jul/2017	First version
# 1.01 09/Feb/2022	Split [string repeat] into separate test file
######################################################################################
# Notes:
# [range] is functionally different to [string range]
# Is included in unit tests: tests.tcl
######################################################################################
option expression off

textcolor yellow
puts "Test \[range\] ..."
textcolor
assert [range ""] {eq $_ ""} -v -line #__LINE__
assert [range ""] {eq $_ {}} -v -line #__LINE__
assert [range {}] {eq $_ ""} -v -line #__LINE__
assert [range {}] {eq $_ {}} -v -line #__LINE__
assert [range 0-9] {eq $_ 0123456789} -v -line #__LINE__
assert [range 9-0] {eq $_ 9876543210} -v -line #__LINE__
assert [range A-Z] {eq $_ ABCDEFGHIJKLMNOPQRSTUVWXYZ} -v -line #__LINE__
assert [range Z-A] {eq $_ ZYXWVUTSRQPONMLKJIHGFEDCBA} -v -line #__LINE__
assert [range 0-9 z-w F-J] {eq $_ 0123456789zyxwFGHIJ} -v -line #__LINE__
assert [range 9-0 Z-A] {eq $_ 9876543210ZYXWVUTSRQPONMLKJIHGFEDCBA} -v -line #__LINE__
assert [range 9-C] {eq $_ "9:;<=>?@ABC"} -v -line #__LINE__		# Checked OK
newline


textcolor yellow
puts "Test \[string range\] ..."
textcolor
# [string range string first last]
assert [string range $qbf 10 18] {eq $_ "brown fox"} -v -line #__LINE__
assert [string range $qbf 35 end] {eq $_ "lazy dog"} -v -line #__LINE__
assert [string range $qbf 12 6] {eq $_ ""} -v -line #__LINE__
assert [string range $qbf 0 1024] {eq $_ $qbf} -v -line #__LINE__
assert [string range $qbf 0 end] {eq $_ $qbf} -v -line #__LINE__
assert [string range $qbf 0 0] {eq $_ T} -v -line #__LINE__
assert [string range $qbf 0 2] {eq $_ The} -v -line #__LINE__
# assert [string range $qbf 0 -1] {eq $_ ""} -v -line #__LINE__ # Improper argument
newline

puts "Line: "#__LINE__
# FIX assert: Line 53: Assertion failed: ' !"#$%&'()*' != 'eq $_ "' !\"#$%&'()*"'
# does not escape dquote ' !"#$%&'()*'
# Input not escaping dquotes
# Error: 1 Too many { braces if Rvalue dquote not escaped
# ' !"#\$%&'()*' != 'eq $_ "' !\"#\$%&'()*"'
#
# 16/Aug/2019 - FIX escaping issue
# Notes: MPP exits normally with parity mismatch on {}
# cmp: 0: 'eq'
# cmp: 1: ' !"#$%&'()*'
# cmp: 2: '' !#$%&'()*"'
#echo " !\"#$%&'()*"		# PASS (confirm is not unescaped by the MPP)

option echo on
puts "Line: "#__LINE__
#escape " !\"#$%&'()*"		# PASS (does not echo)
# echo {\"}	# PASS
# echo { \"}	# PASS
echo { !\"}	# PASS

# BUG 16/Aug/2019
# We don't have an 'in_brace' case IIRC so as soon as an unescaped # is encountered the line will be truncated
# Question is, if # is escaped why is this happening? prevchar must be corrupted by \" case?
# FAIL Too many {
#echo { !\"#}	
# FAIL Too many { (miscalculation following escaped dquote)
#echo { !\"#               }
# FAIL		# Too many {
#echo { !\"\#  }	
# Tricky question. Should the MPP not swallow up the entire brace and leave it 'as is' ?
# Poss not because we would like the MPP to translate all \r\n\t\b\v sequences
# if this is the case then char processing must be accurate
# *OR* we could extract the whole of the braced statement. Do literal number and C++ escape replacements on that?

#escape { !\"#$%&'()*}		# PASS MPP fail on brace parity
#echo MPP: [escape [range 32-42]] ...

#stop

# assert [escape [range 32-42]] {eq $_ "' !\"#\$%&'()*"} -v -line #__LINE__
# assert [escape [range 32-42]] {eq $_ "' !\"#\$%&'()*"} -v -line #__LINE__
# assert [range 42-32] {eq $_ "*)('&%\$#\"!"} -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF # 