#################################################################################################
# File:				test_ratios.tcl
# Author:			Unknown
# Purpose:			Facebook Tcl Group Challenge
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 06/Aug/2018	First version
#################################################################################################
# Notes
#################################################################################################
#
# Return the division of quantity by two or more given ratios
#
# Arguments:
# quantity first value is quantity
# args second, third, and sucessive values
# are two or more given ratios (usually integers in high school)
# However, decimal fractions for both quantity and args seem
# to be reasonably accurate to TCL precison. 
# Conditions and answer checks
# The quantity must be larger than 0
# The number of args must be larger than 2
# If sum of parts equals zero, formula is undefined
# Number of parts should be equal to [ llength $args ] 
# Total value of parts should be equal to quantity entry
# The solution may not be unique in that more than one
# set of shares may have or generate the same multiple ratios. 
# Results: parts of quantity into proportional weighted ratios
# 
# ActiveState version
/*
proc quantity_into_ratios {quantity args} {
	if { $quantity <= 0 } {
		return -code error "The quantity must be larger than 0"
	}

	if { [ llength $args ] < 2 } {
		return -code error "The number of args must be 2 or more"
	}

	set sum 0
	foreach val $args {
		set sum [ expr { $sum + $val * 1.0} ]
	}

	puts "math::quantity_into_ratios: sum of args equals $sum " 
	if { $sum <= 0 } {
		return -code error "The sum of args must be more than zero"
	}
	
	if {[ llength $args ] == 0} {
		return 0
	}

	set answer ""
	foreach val $args {
		lappend answer [ expr { $quantity * ($val/$sum) } ]
	}
	set answer
}
*/
#################################################################################################

option expression on
option echo on

cls
textcolor yellow
puts "Testing quantity ratios"
textcolor
newline

proc math::quantity_into_ratios {quantity args} {
	if { $quantity <= 0 } {
		return -code error "The quantity must be larger than 0"
	}

	if { [ llength $args ] < 2 } {		# 1.9 ?
		return -code error "The number of args must be 2 or more"
	}
	#set sum [ expr ([join $args +])]

	set sum 0								# This is required only to generate a double result v's expr [join]
	foreach val $args {
		set sum [ expr { $sum + $val * 1.0} ]
	}

	puts "math::quantity_into_ratios: sum of args equals $sum " 
	if { $sum <= 0 } {
		return -code error "The sum of args must be more than zero"
	}
	
	if {[ llength $args ] == 0} {
		return 0
	}

	set answer ""
	foreach val $args {
		# puts "calling lappend $answer \[expr \{ $quantity * ($val / $sum)\}\]"
		lappend answer [ expr { $quantity * ($val/$sum) } ]
	}
	set answer
}

proc math::quantity_into_ratios1 {quantity args} {
	if { $quantity <= 0 } {
		return -code error "The quantity must be larger than 0"
	}

	if { [ llength $args ] < 2 } {		# 1.9 ?
		return -code error "The number of args must be 2 or more"
	}

	if { [ expr ([join $args +]) ] <= 0 } {
		return -code error "The sum of args must be more than zero"
	}
	puts "math::quantity_into_ratios: sum of args equals [ expr ([join $args +]) ] " 

	set sum 0.
	#set N [ expr { [ llength $args ] + 1 } ]		# Works fine, but I prefer the simpler...
	set N [+ [ llength $args ] 1]					# which works in Ticol Tcl
	if { $N == 1 } {
		return 0
	}
	foreach val $args {
		# Fix adding .1 incrementally adds +1
		# puts "DEBUG: Adding val:$val to sum:$sum"
		#set sum [ expr { $sum + [double $val] } ]
		set sum [ expr { $sum + $val } ]
	}
	set answer ""
	foreach val $args {
		puts "calling lappend $answer \[expr \{ $quantity * ($val / $sum)\}\]"
		lappend answer [ expr { $quantity * ($val/$sum) } ]
		#lappend answer [ expr { $quantity * ($val/[double $sum]) } ]
	}
	set answer
}

puts " ::math::quantity_into_ratios ( 84 1 2 4 ) answer 12.0 24.0 48.0 "
puts "'[ ::math::quantity_into_ratios 84 1 2 4 ]'"
assert 1 {return $::true -code ok} -v -line #__LINE__

assert [ ::math::quantity_into_ratios 84 1 2 4 ] {		# Check/test for assert behaviour
	puts "Line: "#__LINE__
	puts "\[lindex \$_ 0\]=[round [lindex $_ 0] 2]"
	return $::true -code ok
	} -v -line #__LINE__

assert [ ::math::quantity_into_ratios 84 1 2 4 ] {== [round [lindex $_ 0] 0] 12.0} -v -line #__LINE__
assert [ ::math::quantity_into_ratios 84 1 2 4 ] {== [round [lindex $_ 1] 0] 24.0} -v -line #__LINE__
assert [ ::math::quantity_into_ratios 84 1 2 4 ] {== [round [lindex $_ 2] 0] 48.0} -v -line #__LINE__
# Alternate method
assert [round [lindex [ ::math::quantity_into_ratios 84 1 2 4 ] 0] 0] {== $_ 12.0} -v -line #__LINE__
assert [round [lindex [ ::math::quantity_into_ratios 84 1 2 4 ] 1] 0] {== $_ 24.0} -v -line #__LINE__
assert [round [lindex [ ::math::quantity_into_ratios 84 1 2 4 ] 2] 0] {== $_ 48.0} -v -line #__LINE__
newline

# Continue

puts " ::math::quantity_into_ratios ( 14 1 2 4 ) answer 2.0 4.0 8.0 "
assert [round [lindex [::math::quantity_into_ratios 14 1 2 4 ] 0] 0] {== $_ 2.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 14 1 2 4 ] 1] 0] {== $_ 4.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 14 1 2 4 ] 2] 0] {== $_ 8.0} -v -line #__LINE__
newline

puts " ::math::quantity_into_ratios ( 10 2 3 ) answer 4.0 6.0 "
assert [round [lindex [::math::quantity_into_ratios 10 2 3 ] 0] 0] {== $_ 4.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 2 3 ] 1] 0] {== $_ 6.0} -v -line #__LINE__
newline

puts " ::math::quantity_into_ratios ( 11 2 3 ) answer 4.40 6.59 "
puts [::math::quantity_into_ratios 11 2 3 ]
assert [round [lindex [::math::quantity_into_ratios 11 2 3 ] 0] 4] {== $_ 4.4} -v -line #__LINE__
#assert [round [lindex [::math::quantity_into_ratios 11 2 3 ] 1] 4] {== $_ 6.59} -v -line #__LINE__	# FAIL '6.6000' != '== $_ 6.59'
newline

puts " ::math::quantity_into_ratios ( 9 1 1 1 ) answer 3.0 3.0 3.0 "
puts " [ ::math::quantity_into_ratios 9 1 1 1 ] " 
assert [round [lindex [::math::quantity_into_ratios 9 1 1 1 ] 0] 0] {== $_ 3.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 9 1 1 1 ] 1] 0] {== $_ 3.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 9 1 1 1 ] 2] 0] {== $_ 3.0} -v -line #__LINE__
newline

puts "Line: "#__LINE__
puts " ::math::quantity_into_ratios ( 10 1 1 1 1 1 1 1 1 1 1 ) answer 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 "
set i 1
foreach x [ ::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] {
	puts "$i\t$x"
	++ i
}

assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 0] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 1] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 2] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 3] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 4] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 5] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 6] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 7] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 8] 1] {== $_ 1.0} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 10 1 1 1 1 1 1 1 1 1 1 ] 9] 1] {== $_ 1.0} -v -line #__LINE__
newline

# FAILS -> 0.001, should be 0.01
puts "Line "#__LINE__
puts " ::math::quantity_into_ratios ( 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ) answer 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 "
set i 1
foreach x [ ::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] {
	puts "$i\t$x"
	++ i
}
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 0] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 1] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 2] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 3] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 4] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 5] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 6] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 7] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 8] 5] {== $_ 0.01} -v -line #__LINE__
assert [round [lindex [::math::quantity_into_ratios 0.10 .1 .1 .1 .1 .1 .1 .1 .1 .1 .1 ] 9] 5] {== $_ 0.01} -v -line #__LINE__
newline


option echo on
# following return designated errors
puts "Line: "#__LINE__
textcolor yellow
puts "Checking deliberate errors via \[catch\] ..."
textcolor
set err ""
catch {::math::quantity_into_ratios 10 1} err
puts "Error: $err near line "#__LINE__
set err ""
catch { ::math::quantity_into_ratios 84 1 } err
puts "Error: $err near line "#__LINE__
set err ""
catch { ::math::quantity_into_ratios 84} err
puts "Error: $err near line "#__LINE__
set err ""
catch {::math::quantity_into_ratios -2 1 2 3 } err
puts $err
if {$err ne ""} {
	puts "Error: $err near line "#__LINE__
}

/* FIX [try..catch] needs to be enhanced, [return] should probably set errorMsg
try { 
	::math::quantity_into_ratios -2 1 2 3 
} catch {
	puts "Error: $errorMsg near line "#__LINE__
} # */


unset err i x -nocomplain
newline
vars
newline

#################################################################################################

textcolor green
puts "* Done *"
textcolor

# EOF #