######################################################################################
# File: 			test_relational.tcl
# Purpose:			Test various relational commands
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 20/Oct/2019	First version using [assert]
######################################################################################
# Notes:
# ADD catch invalid arg counts
# ADD nen, eqn, (gtn, ltn, gen, (len clash)) ?
######################################################################################

# Applying math operators to strings
newline
textcolor yellow
puts "Testing relational math operators with strings ..."
textcolor
newline

######################################################################################
# == (EQ)
assert [== a b] {== $_ 0} -v -line #__LINE__
assert [== b a] {== $_ 0} -v -line #__LINE__
assert [== a a] {== $_ 1} -v -line #__LINE__
assert [== $qbf $qbf] {== $_ 1} -v -line #__LINE__
assert [== bar foo] {== $_ 0} -v -line #__LINE__
assert [== foo bar] {== $_ 0} -v -line #__LINE__
assert [== foo foo] {== $_ 1} -v -line #__LINE__

######################################################################################
# != (NE)
assert [!= a b] {== $_ 1} -v -line #__LINE__
assert [!= b a] {== $_ 1} -v -line #__LINE__
assert [!= a a] {== $_ 0} -v -line #__LINE__
assert [!= $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [!= bar foo] {== $_ 1} -v -line #__LINE__
assert [!= foo bar] {== $_ 1} -v -line #__LINE__
assert [!= foo foo] {== $_ 0} -v -line #__LINE__

######################################################################################
# > (GT)
assert [> a b] {== $_ 0} -v -line #__LINE__
assert [> b a] {== $_ 1} -v -line #__LINE__
assert [> a a] {== $_ 0} -v -line #__LINE__
assert [> $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [> bar foo] {== $_ 0} -v -line #__LINE__
assert [> foo bar] {== $_ 1} -v -line #__LINE__
assert [> foo foo] {== $_ 0} -v -line #__LINE__

######################################################################################
# >= (GE)
assert [>= a b] {== $_ 0} -v -line #__LINE__
assert [>= b a] {== $_ 1} -v -line #__LINE__
assert [>= a a] {== $_ 1} -v -line #__LINE__
assert [>= $qbf $qbf] {== $_ 1} -v -line #__LINE__
assert [>= bar foo] {== $_ 0} -v -line #__LINE__
assert [>= foo bar] {== $_ 1} -v -line #__LINE__
assert [>= foo foo] {== $_ 1} -v -line #__LINE__

######################################################################################
# < (LT)
assert [< a b] {== $_ 1} -v -line #__LINE__
assert [< b a] {== $_ 0} -v -line #__LINE__
assert [< a a] {== $_ 0} -v -line #__LINE__
assert [< $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [< bar foo] {== $_ 1} -v -line #__LINE__
assert [< foo bar] {== $_ 0} -v -line #__LINE__
assert [< foo foo] {== $_ 0} -v -line #__LINE__

######################################################################################
# <= (LE)
assert [<= a b] {== $_ 1} -v -line #__LINE__
assert [<= b a] {== $_ 0} -v -line #__LINE__
assert [<= a a] {== $_ 1} -v -line #__LINE__
assert [<= $qbf $qbf] {== $_ 1} -v -line #__LINE__
assert [<= bar foo] {== $_ 1} -v -line #__LINE__
assert [<= foo bar] {== $_ 0} -v -line #__LINE__
assert [<= foo foo] {== $_ 1} -v -line #__LINE__

# String operators
newline
textcolor yellow
puts "Testing relational string operators with strings ..."
textcolor
newline

######################################################################################
# GT
set i 0
try {
	gt 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [gt a b] {== $_ 0} -v -line #__LINE__
assert [gt b a] {== $_ 1} -v -line #__LINE__
assert [gt a a] {== $_ 0} -v -line #__LINE__
assert [gt $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [gt bar foo] {== $_ 0} -v -line #__LINE__
assert [gt foo bar] {== $_ 1} -v -line #__LINE__
assert [gt foo foo] {== $_ 0} -v -line #__LINE__

######################################################################################
# GE
set i 0
try {
	ge 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [ge a b] {== $_ 0} -v -line #__LINE__
assert [ge b a] {== $_ 1} -v -line #__LINE__
assert [ge a a] {== $_ 1} -v -line #__LINE__
assert [ge $qbf $qbf] {== $_ 1} -v -line #__LINE__
assert [ge bar foo] {== $_ 0} -v -line #__LINE__
assert [ge foo bar] {== $_ 1} -v -line #__LINE__
assert [ge foo foo] {== $_ 1} -v -line #__LINE__

######################################################################################
# LT
set i 0
try {
	lt 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [lt a b] {== $_ 1} -v -line #__LINE__
assert [lt b a] {== $_ 0} -v -line #__LINE__
assert [lt a a] {== $_ 0} -v -line #__LINE__
assert [lt $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [lt bar foo] {== $_ 1} -v -line #__LINE__
assert [lt foo bar] {== $_ 0} -v -line #__LINE__
assert [lt foo foo] {== $_ 0} -v -line #__LINE__

######################################################################################
# LE
set i 0
try {
	ge 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [le a b] {== $_ 1} -v -line #__LINE__
assert [le b a] {== $_ 0} -v -line #__LINE__
assert [le a a] {== $_ 1} -v -line #__LINE__
assert [le $qbf $qbf] {== $_ 1} -v -line #__LINE__
assert [le bar foo] {== $_ 1} -v -line #__LINE__
assert [le foo bar] {== $_ 0} -v -line #__LINE__
assert [le foo foo] {== $_ 1} -v -line #__LINE__

######################################################################################
# EQ
set i 0
try {
	eq 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [eq a b] {== $_ 0} -v -line #__LINE__
assert [eq b a] {== $_ 0} -v -line #__LINE__
assert [eq a a] {== $_ 1} -v -line #__LINE__
assert [eq $qbf $qbf] {== $_ 1} -v -line #__LINE__
assert [eq bar foo] {== $_ 0} -v -line #__LINE__
assert [eq foo bar] {== $_ 0} -v -line #__LINE__
assert [eq foo foo] {== $_ 1} -v -line #__LINE__

######################################################################################
# EQN
set i 0
try {
	eqn
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
set i 0
try {
	eqn a b
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [eqn a b 0] {== $_ 1} -v -line #__LINE__		# 0 means we're comparing "" and ""
assert [eqn a b 1] {== $_ 0} -v -line #__LINE__
assert [eqn a a 0] {== $_ 1} -v -line #__LINE__
assert [eqn a a 1] {== $_ 1} -v -line #__LINE__
assert [eqn a a 2] {== $_ 1} -v -line #__LINE__
assert [eqn a a 20] {== $_ 1} -v -line #__LINE__
assert [eqn $qbf $qbf 0] {== $_ 1} -v -line #__LINE__
assert [eqn $qbf $qbf 1] {== $_ 1} -v -line #__LINE__
assert [eqn $qbf $qbf 10] {== $_ 1} -v -line #__LINE__
assert [eqn bar foo 3] {== $_ 0} -v -line #__LINE__
assert [eqn foo bar 3] {== $_ 0} -v -line #__LINE__
assert [eqn foo foo 0] {== $_ 1} -v -line #__LINE__
assert [eqn foo foo 3] {== $_ 1} -v -line #__LINE__
assert [eqn foo foo 4] {== $_ 1} -v -line #__LINE__

######################################################################################
# NE
set i 0
try {
	ne 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [ne a b] {== $_ 1} -v -line #__LINE__
assert [ne b a] {== $_ 1} -v -line #__LINE__
assert [ne a a] {== $_ 0} -v -line #__LINE__
assert [ne $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [ne bar foo] {== $_ 1} -v -line #__LINE__
assert [ne foo bar] {== $_ 1} -v -line #__LINE__
assert [ne foo foo] {== $_ 0} -v -line #__LINE__

######################################################################################
# NEN
set i 0
try {
	nen
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
set i 0
try {
	nen a b
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [nen a b 0] {== $_ 0} -v -line #__LINE__		# 0 means we're comparing "" and ""
assert [nen a b 1] {== $_ 1} -v -line #__LINE__
assert [nen a a 0] {== $_ 0} -v -line #__LINE__
assert [nen a a 1] {== $_ 0} -v -line #__LINE__
assert [nen a a 2] {== $_ 0} -v -line #__LINE__
assert [nen a a 20] {== $_ 0} -v -line #__LINE__
assert [nen $qbf $qbf 0] {== $_ 0} -v -line #__LINE__
assert [nen $qbf $qbf 1] {== $_ 0} -v -line #__LINE__
assert [nen $qbf $qbf 10] {== $_ 0} -v -line #__LINE__
assert [nen bar foo 3] {== $_ 1} -v -line #__LINE__
assert [nen foo bar 3] {== $_ 1} -v -line #__LINE__
assert [nen foo foo 0] {== $_ 0} -v -line #__LINE__
assert [nen foo foo 3] {== $_ 0} -v -line #__LINE__
assert [nen foo foo 4] {== $_ 0} -v -line #__LINE__


######################################################################################
# <=> PERL spaceship operator
newline
textcolor yellow
puts "Testing PERL spaceship operator ..."
textcolor
newline

set i 0
try {
	<=> 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [<=> a b] {== $_ -1} -v -line #__LINE__
assert [<=> b a] {== $_ 1} -v -line #__LINE__
assert [<=> a a] {== $_ -0} -v -line #__LINE__
assert [<=> $qbf $qbf] {== $_ 0} -v -line #__LINE__
assert [<=> bar foo] {== $_ -1} -v -line #__LINE__
assert [<=> foo bar] {== $_ 1} -v -line #__LINE__
assert [<=> foo foo] {== $_ 0} -v -line #__LINE__


######################################################################################
# Test with numeric values
######################################################################################
newline
textcolor yellow
puts "Testing relational numeric commands with numeric values ..."
textcolor
newline


######################################################################################
# == (EQ)
set i 0
try {
	== 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [== 0 0] {== $_ 1} -v -line #__LINE__
assert [== 0 -0] {== $_ 1} -v -line #__LINE__
assert [== -0 0] {== $_ 1} -v -line #__LINE__
assert [== -0 -0] {== $_ 1} -v -line #__LINE__

assert [== 1 2] {== $_ 0} -v -line #__LINE__
assert [== 2 1] {== $_ 0} -v -line #__LINE__
assert [== 1 1] {== $_ 1} -v -line #__LINE__
assert [== 1 -1] {== $_ 0} -v -line #__LINE__
assert [== -1 1] {== $_ 0} -v -line #__LINE__
assert [== -1 -1] {== $_ 1} -v -line #__LINE__
assert [== 12.345 6.789] {== $_ 0} -v -line #__LINE__
assert [== 12.345 12.345] {== $_ 1} -v -line #__LINE__

######################################################################################
# != (NE)
set i 0
try {
	!= 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [!= 0 0] {== $_ 0} -v -line #__LINE__
assert [!= 0 -0] {== $_ 0} -v -line #__LINE__
assert [!= -0 0] {== $_ 0} -v -line #__LINE__
assert [!= -0 -0] {== $_ 0} -v -line #__LINE__

assert [!= 1 2] {== $_ 1} -v -line #__LINE__
assert [!= 2 1] {== $_ 1} -v -line #__LINE__
assert [!= 1 1] {== $_ 0} -v -line #__LINE__
assert [!= 1 -1] {== $_ 1} -v -line #__LINE__
assert [!= -1 1] {== $_ 1} -v -line #__LINE__
assert [!= -1 -1] {== $_ 0} -v -line #__LINE__
assert [!= 12.345 6.789] {== $_ 1} -v -line #__LINE__
assert [!= 12.345 12.345] {== $_ 0} -v -line #__LINE__

######################################################################################
# > (GT)
set i 0
try {
	> 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [> 0 0] {== $_ 0} -v -line #__LINE__
assert [> 0 -0] {== $_ 0} -v -line #__LINE__
assert [> -0 0] {== $_ 0} -v -line #__LINE__
assert [> -0 -0] {== $_ 0} -v -line #__LINE__

assert [> 1 2] {== $_ 0} -v -line #__LINE__
assert [> 2 1] {== $_ 1} -v -line #__LINE__
assert [> 1 1] {== $_ 0} -v -line #__LINE__
assert [> 1 -1] {== $_ 1} -v -line #__LINE__
assert [> -1 1] {== $_ 0} -v -line #__LINE__
assert [> -1 -1] {== $_ 0} -v -line #__LINE__
assert [> 12.345 6.789] {== $_ 1} -v -line #__LINE__
assert [> 12.345 12.345] {== $_ 0} -v -line #__LINE__

######################################################################################
# >= (GE)
set i 0
try {
	>= 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [>= 0 0] {== $_ 1} -v -line #__LINE__
assert [>= 0 -0] {== $_ 1} -v -line #__LINE__
assert [>= -0 0] {== $_ 1} -v -line #__LINE__
assert [>= -0 -0] {== $_ 1} -v -line #__LINE__

assert [>= 1 2] {== $_ 0} -v -line #__LINE__
assert [>= 2 1] {== $_ 1} -v -line #__LINE__
assert [>= 1 1] {== $_ 1} -v -line #__LINE__
assert [>= 1 -1] {== $_ 1} -v -line #__LINE__
assert [>= -1 1] {== $_ 0} -v -line #__LINE__
assert [>= -1 -1] {== $_ 1} -v -line #__LINE__
assert [>= 12.345 6.789] {== $_ 1} -v -line #__LINE__
assert [>= 12.345 12.345] {== $_ 1} -v -line #__LINE__
assert [>= 6.789 12.345] {== $_ 0} -v -line #__LINE__

######################################################################################
# < (LT)
set i 0
try {
	< 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [< 0 0] {== $_ 0} -v -line #__LINE__
assert [< 0 -0] {== $_ 0} -v -line #__LINE__
assert [< -0 0] {== $_ 0} -v -line #__LINE__
assert [< -0 -0] {== $_ 0} -v -line #__LINE__

assert [< 1 2] {== $_ 1} -v -line #__LINE__
assert [< 2 1] {== $_ 0} -v -line #__LINE__
assert [< 1 1] {== $_ 0} -v -line #__LINE__
assert [< 1 -1] {== $_ 0} -v -line #__LINE__
assert [< -1 1] {== $_ 1} -v -line #__LINE__
assert [< -1 -1] {== $_ 0} -v -line #__LINE__
assert [< 12.345 6.789] {== $_ 0} -v -line #__LINE__
assert [< 12.345 12.345] {== $_ 0} -v -line #__LINE__

######################################################################################
# <= (LE)
set i 0
try {
	<= 0
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__
assert [<= 0 0] {== $_ 1} -v -line #__LINE__
assert [<= 0 -0] {== $_ 1} -v -line #__LINE__
assert [<= -0 0] {== $_ 1} -v -line #__LINE__
assert [<= -0 -0] {== $_ 1} -v -line #__LINE__

assert [<= 1 2] {== $_ 1} -v -line #__LINE__
assert [<= 2 1] {== $_ 0} -v -line #__LINE__
assert [<= 1 1] {== $_ 1} -v -line #__LINE__
assert [<= 1 -1] {== $_ 0} -v -line #__LINE__
assert [<= -1 1] {== $_ 1} -v -line #__LINE__
assert [<= -1 -1] {== $_ 1} -v -line #__LINE__
assert [<= 12.345 6.789] {== $_ 0} -v -line #__LINE__
assert [<= 12.345 12.345] {== $_ 1} -v -line #__LINE__
assert [<= 6.789 12.345] {== $_ 1} -v -line #__LINE__

unset i -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
option expression off
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #