########################################################################
# File: 			test_replacechar.tcl
# Purpose:			Tests for [replacechar]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 25/Jun/2018	First version
# 1.01 09/Oct/2018	More tests added
########################################################################
# Notes: 
########################################################################
option expression off

textcolor yellow
puts "Testing \[replacechar\] ..."
textcolor
newline

########################################################################
puts "Line: "#__LINE__
set s "     "					# 5 spaces
set r [replacechar s  " " ""]	# Replace space with nothing
puts "Replaced $r"
assert $r {== $_ 5} -v -line #__LINE__
puts [tohex $s]					# We get 1 tab
assert $s {eq $_ ""} -v -line #__LINE__

puts "Line: "#__LINE__
set s "\t\t\t\t\t"				# 5 tabs
set r [replacechar s  "\t" ""]	# Replace tabs with nothing
puts "Replaced $r"
assert $r {== $_ 5} -v -line #__LINE__
assert $s {eq $_ ""} -v -line #__LINE__

puts "Line: "#__LINE__
set s "\t \t \t"				# 3 tabs, 2 spaces
set r [replacechar s  " " ""]	# Replace tab with nothing
puts "Replaced $r"
assert $r {== $_ 2} -v -line #__LINE__
puts [tohex $s]					# We get 1 tab
assert $s {eq $_ "\t\t\t"} -v -line #__LINE__

puts "Line: "#__LINE__
set s "\t \t \t"				# 3 tabs, 2 spaces
set r [replacechar s  "\t" ""]	# Replace tab with nothing
puts "Replaced $r (should be 3)"
assert $r {== $_ 3} -v -line #__LINE__
puts '[tohex $s]'					# We get ""
assert $s {eq $_ "  "} -v -line #__LINE__

puts "Line: "#__LINE__
set s "Hello"
set r [replacechar s  "l" "q"]
puts "Replaced $r"
assert $r {== $_ 2} -v -line #__LINE__
assert $s {eq $_ Heqqo} -v -line #__LINE__

# Arrays #
unset a -nocomplain
array unset a -nocomplain
#1dim a 100
puts "Line: "#__LINE__
set a(10) Hello
puts "Replace l with q in a(10) '$a(10)'"
#puts "Line: "#__LINE__
replacechar a(10) "l" "q"
puts "Line: "#__LINE__
#puts "a(10) is now '$a(10)'"
assert $a(10) {eq $_ Heqqo} -v -line #__LINE__

# ADD check non existent array element is handled OK
puts "Line: "#__LINE__
textcolor red
puts "Ignore catch error..."
textcolor
catch {
replacechar a(11) "l" "q"
}
puts "Line: "#__LINE__

puts "Line: "#__LINE__
set a(10) "     "					# 5 spaces
set r [replacechar a(10)  " " ""]	# Replace space with nothing
puts "Replaced $r"
assert $r {== $_ 5} -v -line #__LINE__
puts [tohex $a(10)]					# We get 1 tab
assert $a(10) {eq $_ ""} -v -line #__LINE__

puts "Line: "#__LINE__
set a(10) "\t\t\t\t\t"				# 5 tabs
set r [replacechar a(10)  "\t" ""]	# Replace tabs with nothing
puts "Replaced $r"
puts [tohex $a(10)]					# We get 1 tab
assert $r {== $_ 5} -v -line #__LINE__
assert $a(10) {eq $_ ""} -v -line #__LINE__

puts "Line: "#__LINE__
set a(10) "\t \t \t"				# 3 tabs, 2 spaces
set r [replacechar a(10)  " " ""]	# Replace tab with nothing
puts "Replaced $r"
assert $r {== $_ 2} -v -line #__LINE__
puts [tohex $a(10)]					# We get 1 tab
assert $a(10) {eq $_ "\t\t\t"} -v -line #__LINE__

puts "Line: "#__LINE__
set a(10) "\t \t \t"				# 3 tabs, 2 spaces
set r [replacechar a(10)  "\t" ""]	# Replace tab with nothing
puts "Replaced $r (should be 3)"
assert $r {== $_ 3} -v -line #__LINE__
puts '[tohex $a(10)]'					# We get ""
assert $a(10) {eq $_ "  "} -v -line #__LINE__


# [lindex] does not find "" embedded in list and followed by EOL when indexed (is skipped)

set s {brave\r\nnew ""\r\n world}
replacechar s "\r" " "
replacechar s "\n" " "
puts "s:'$s'"
assert $s {eq $_ {brave  new ""   world}} -v -line #__LINE__

loop i 0 [lcount $s] {
	puts "lindex $i: '[lindex $s $i]'"
} 

# ActiveState error: "list element in quotes followed by "\r\n" instead of space"
/*	# Case 1 (FAIL)
set s {brave\r\nnew ""\r\n world}
loop i 0 [lcount $s] {
	puts "$i: '[lindex $s $i]'"
} # */


####################################################################

puts "Line: "#__LINE__
array unset a -nocomplain
unset i r s -nocomplain

vars

########################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #