#############################################################################
# File:				return.tcl
# Author:			M Shaw
# Purpose:			Test file for [return]
# Language:			Ticol Tcl v1.19+
#
# Versions:
# 1.00 20/Mar/2016	First version
#############################################################################
# Notes:
# TODO FIX return and catch 
# How to trap/show return code?
#############################################################################
/*
  Code Returns: for [return]
  ------------------------
  Number    Mnemonic
  ------------------------
  0         ok
  1         error
  2         return (default)
  3         break
  4         continue
  5			exit
  6			stop
  7			goto
  8			defined
  ------------------------
	"ok",			// 0 Used for [return]
	"error",		// 1
	"return",		// 2
	"break",		// 3
	"continue",		// 4
	"exit",			// 5
	"stop",			// 6
	"goto",			// 7
	"defined",		// 8 (Unused)
	"ERR"			// 9 Bounds error
 % source return.tcl
Testret: a is '0'
Trap 0
Catch return 0 is: 0 Hi

Testret: a is '1'
Trap 1
Catch return 1 is: 1 

Testret: a is '2'
Trap 2
Catch return 2 is: 2 Ret

Testret: a is '3'
Trap 3
Catch return 3 is: 3 Break

Testret: a is '4'
Trap 4
Catch return 4 is: 4 Cont

Testret: a is '13'
r is
Catch return 13 is: 0

// Ticol //

Testret: a is '0'
Trap 0
r is Hi
Catch return 0 is: 0 Hi

Testret: a is '1'
Trap 1
Catch return 1 is: 1 Err

Testret: a is '2'
Trap 2
Catch return 2 is: 0 Ret		# FIX (return is returning ok)

Testret: a is '3'
Trap 3
Catch return 3 is: 3 Break

Testret: a is '4'
Trap 4
Catch return 4 is: 4 Cont

Testret: a is '13'
Catch return 13 is: 23 Default

Var is Default
Done.

# */


# FIX ActiveState will return directly from proc on encountering return statement
textcolor yellow
puts "Test returning values by code number with correct descriptive string ..."
textcolor
newline
undef testret -nocomplain
proc testret {a} {
	#puts "Testret: \{$a\}"
	switch -- $a {
	0 	{textcolor yellow; puts "Trap 0"; textcolor; return -code ok "Ok"}
	1 	{textcolor yellow; puts "Trap 1"; textcolor; return -code error "Error"}
	2   {textcolor yellow; puts "Trap 2"; textcolor; return -code return "Return"}
	3   {textcolor yellow; puts "Trap 3"; textcolor; return -code break "Break"}
	4   {textcolor yellow; puts "Trap 4"; textcolor; return -code continue "Continue"}
#	5	{textcolor yellow; puts "Trap 5"; textcolor; return -code exit "Exit"}
	default {textcolor yellow; puts "Default"; textcolor; return "Default" -code 23}
	}
	textcolor green
	puts "Exiting proc testret"
	#puts "proc r is $r"
	textcolor
	#return $r
}


puts "Catch return 0 is: [catch {testret 0} var] $var\n"
puts "Catch return 1 is: [catch {testret 1} var] $var\n"
puts "Catch return 2 is: [catch {testret 2} var] $var\n"
puts "Catch return 3 is: [catch {testret 3} var] $var\n"
puts "Catch return 4 is: [catch {testret 4} var] $var\n"
puts "Catch return 13 is: [catch {testret 13} var] $var\n"
newline

return -code ok 							# Should not exit script


/*
puts "Returning from script (you should see no more text after this!)"
return -code 0xff			# FIX is not returned to Windows
*/

##############################################################
unset var -nocomplain
undef testret -nocomplain
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #