###############################################################
# File: 			test_round.tcl
# Purpose:			Testing [round] and round()
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 20/Dec/2016	First version
# 1.01 23/Jul/2017	Replaced [checkex] with [assert]
###############################################################
# Notes: 
###############################################################
option expression off

#/*
undef test_round -nocomplain
proc test_round {fname} {
#proc test_round {fname {arg {}}} {		# FIX in [proc]
	# We set up a proc so we don't need to duplicate tests, messily for both [round] and [funct round]
	# This is problematic as the parser will evaluate [] before $ so we have to delay and use [eval]
	textcolor darkyellow
	puts "Testing  proc \[test_round fname:'$fname'\]"
	textcolor
	#assert [funct round 0] {== $_ 0} -v -line #__LINE__
	#assert [eval "$fname 1.2345"] {== $_ 1} -v -line #__LINE__
puts "eval \"$fname 0\" '[eval "$fname  0"]'"	
	assert [eval "$fname  0"] {== $_ 0} -v -line #__LINE__
puts "Line "#__LINE__	
	assert [eval "$fname  -0"] {== $_ -0} -v -line #__LINE__						# FIX -0 returns -0 (should be 0)
	assert [eval "$fname  0.0"] {== $_ 0} -v -line #__LINE__
	assert [eval "$fname  -0.0"] {== $_ 0} -v -line #__LINE__						# -0.0 returns 0
	# Non-zero
	assert [eval "$fname  1.2345"] {== $_ 1} -v -line #__LINE__
	assert [eval "$fname  1.2345 0"] {== $_ 1} -v -line #__LINE__
	assert [eval "$fname  1.23456789 1"] {== $_ 1.2} -v -line #__LINE__
	assert [eval "$fname  1.23456789 2"] {== $_ 1.23} -v -line #__LINE__			# I say this should be 1.24
	assert [eval "$fname  1.23456789 3"] {== $_ 1.235} -v -line #__LINE__			# 1.235 ?
	assert [eval "$fname  1.23456789 4"] {== $_ 1.2346} -v -line #__LINE__
	assert [eval "$fname  1.23456789 5"] {== $_ 1.23457} -v -line #__LINE__
	assert [eval "$fname  1.23456789 6"] {== $_ 1.234568} -v -line #__LINE__
	assert [eval "$fname  1.23456789 7"] {== $_ 1.2345679} -v -line #__LINE__
	assert [eval "$fname  1.23456789 8"] {== $_ 1.23456789} -v -line #__LINE__
	assert [eval "$fname  $::pi 4"] {== $_ 3.1416} -v -line #__LINE__				# 3.14159265358979323
	assert [eval "$fname  $::pi 2"] {== $_ 3.14} -v -line #__LINE__				# 3.14159265358979323
	assert [eval "$fname  $::pi 9"] {== $_ 3.141592654} -v -line #__LINE__		# 3.14159265358979323

	set q [* $::pi 1000]										# Use pie to make 4 digit integer prefix
	assert [eval "$fname  $q 4"] {== $_ 3141.5927} -v -line #__LINE__				# 3.14159265358979323
	assert [eval "$fname  $q 2"] {== $_ 3141.59} -v -line #__LINE__				# 3.14159265358979323
	assert [eval "$fname  $q 9"] {== $_ 3141.592653590} -v -line #__LINE__		# 3.14159265358979323
	
	# -ve cases 
	assert [eval "$fname  -1.2345"] {== $_ -1} -v -line #__LINE__					# -1
	assert [eval "$fname  -1.2345 0"] {== $_ -1} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 1"] {== $_ -1.2} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 2"] {== $_ -1.23} -v -line #__LINE__		# https://www.calculatorsoup.com/calculators/math/roundingnumbers.php
	assert [eval "$fname  -1.23456789 3"] {== $_ -1.235} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 4"] {== $_ -1.2346} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 5"] {== $_ -1.23457} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 6"] {== $_ -1.234568} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 7"] {== $_ -1.2345679} -v -line #__LINE__
	assert [eval "$fname  -1.23456789 8"] {== $_ -1.23456789} -v -line #__LINE__
	newline
}

textcolor yellow
puts "Testing  \[round\] command ..."
textcolor
newline
test_round round

textcolor yellow
puts "Testing \[funct round\] ..."
textcolor
newline
test_round {funct round}
# */

##################################################################
# Separate cases
assert [round 1616230.5866203704 0] {== $_ 1616231} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 1] {== $_ 1616230.6} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 2] {== $_ 1616230.59} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 3] {== $_ 1616230.587} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 4] {== $_ 1616230.5866} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 5] {== $_ 1616230.58662} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 6] {== $_ 1616230.586620} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 7] {== $_ 1616230.5866204} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 8] {== $_ 1616230.58662037} -v -line #__LINE__	# 1,616,230.586600000

# Date         '1616230.5866203704' @123
# Should be:   '1616230.586620370' @123
# Date round 9 '1616230.586620371' @124
# FIX IS:       1616230.586620371

# FIX modf internal to round() rounds the last digit incorrectly to .5 not .4
#assert [round 1616230.5866203704 9] {== $_ 1616230.586620370} -v -line #__LINE__	# 1,616,230.586600000
assert [round 1616230.5866203704 10] {== $_ 1616230.5866203704} -v -line #__LINE__	# 1,616,230.586600000

# 3.3333333333333333
assert [round 3.3333333333333333 9] {== $_ 3.333333333} -v -line #__LINE__
assert [round 3.3333333333333333 5] {== $_ 3.33333} -v -line #__LINE__
assert [round 3.3333333333333333 2] {== $_ 3.33} -v -line #__LINE__
assert [round 3.3333333333333333 1] {== $_ 3.3} -v -line #__LINE__
assert [round 3.3333333333333333 0] {== $_ 3.0} -v -line #__LINE__

assert [round -3.3333333333333333 9] {== $_ -3.333333333} -v -line #__LINE__
assert [round -3.3333333333333333 5] {== $_ -3.33333} -v -line #__LINE__
assert [round -3.3333333333333333 2] {== $_ -3.33} -v -line #__LINE__
assert [round -3.3333333333333333 1] {== $_ -3.3} -v -line #__LINE__
assert [round -3.3333333333333333 0] {== $_ -3.0} -v -line #__LINE__

# Excess round value +ve
assert [round 1.234 0] {== $_ 1.0} -v -line #__LINE__
assert [round 1.234 3] {== $_ 1.234} -v -line #__LINE__
assert [round 1.234 4] {== $_ 1.234} -v -line #__LINE__
assert [round 1.234 5] {== $_ 1.234} -v -line #__LINE__

# Excess round value -ve
assert [round -1.234 0] {== $_ -1.0} -v -line #__LINE__
assert [round -1.234 3] {== $_ -1.234} -v -line #__LINE__
assert [round -1.234 4] {== $_ -1.234} -v -line #__LINE__
assert [round -1.234 5] {== $_ -1.234} -v -line #__LINE__

# Borderline
assert [round 1.5 0] {== $_ 2} -v -line #__LINE__
assert [round -1.5 0] {== $_ -2} -v -line #__LINE__


undef test_round -nocomplain
##########################################
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #