###############################################################################################
# File:				test_run_in_run.tcl
# Author:			M Shaw
# Purpose:			Test file [run] recursion-block interlocking 
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 11/Dec/2019	First version
# 1.01 06/Jan/2022	Bugfixes in internal [run] flag handling and notes
###############################################################################################
# Notes:
# [source] should also block [run] but [run] doesn't block [source]
#
# DONE FIX this script is problematic in that if it is called by [exec], [source] or directly via
# the DLL interface then [run] is not blocked. (Should we block?)
# Was previously not working because [source] was not setting _info_script properly
# Thus script was never able to enter itself a 2nd time
# Once fixed, then this file was able to call itself twice and corrupts vars
# The DLL interface should probably set run-lock kas per the CLI
#
# [run] corrupts the original program script
# ASSERT this script is running via cmd line or [run]
# Command line EXE launch sets run_lock
# CLI sets and clears the run_lock after each call to the interpreter
# DLL must do the same although there is only entry and no exit from the DLL
# Calls to [run] locks the run_lock and unlocks it on exit
###############################################################################################

option expression on
puts "Enter [info script] Line :"#__LINE__

#puts "DEBUG: main::logfile is '$main::logfile'"

textcolor yellow
puts "Attempting to run *self* - '[info script]' recursively at line "#__LINE__
textcolor
newline

puts "Line :"#__LINE__
set i 0
try {
	run [info script]			# This [run] attempt should raise an error (with any script)
	puts "Post call to info script, i is '$i' at line "#__LINE__
} catch {
	set i 1
}

assert $i -v -line #__LINE__

puts "Line :"#__LINE__
unset i -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #