#################################################################################
# File: 			test_scan.tcl
# Purpose:			Tests for [scan]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 06/Dec/2016	First version
# 1.01 16/Nov/2017	Added more complex tests
# 1.02 11/Jan/2018	Added %b tests
#################################################################################
# Notes:
# TODO: Test asterisk *
# QUERY: Do we process total chars swallowed from the input string or
# Total chars used excluding whitespace
#################################################################################
# FIX -1 0o777777777777777777777
# check -1 %b in ActiveState
#################################################################################
puts "Line: "#__LINE__

option echo on
option expression off
option echo on

puts "Line: "#__LINE__
unset CHAR_MIN -nocomplain		# Test override consts while we are here
const CHAR_MIN -128 -nocomplain

puts "Line: "#__LINE__
vars -const
const CHAR_MIN -128 -nocomplain
const CHAR_MAX 127 -nocomplain
const WORD_MIN -32768 -nocomplain
const WORD_MAX 32767 -nocomplain
const LONG_MIN -2147483648 -nocomplain
const LONG_MAX 2147483647 -nocomplain
const INT_MIN -9223372036854775808 -nocomplain
const INT_MAX 9223372036854775807 -nocomplain
const UNSIGNED_MIN 0 -nocomplain
const UNSIGNED_MAX 18446744073709551615 -nocomplain

# %b (binary) ###################################################################

# Error cases
puts "Line: "#__LINE__
scan -0 %b num
puts "$num"							# 0 ERROR
assert $num {== $_ 0} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
scan -1 %b num
puts "$num"							# 0 ERROR
assert $num {== $_ 0} -v -line #__LINE__
unset num

# Valid cases
puts "Line: "#__LINE__
scan 0 %b num
puts "$num"							# 0
assert $num {== $_ 0} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
scan 1 %b num
puts "$num"							# 1
assert $num {== $_ 1} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
scan 10 %b num
puts "$num"							# 10 -> 2
assert $num {== $_ 2} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
scan 11 %b num
puts "$num"							# 11 -> 3
assert $num {== $_ 3} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
scan 1010 %b num
puts "$num"							# 1010 -> 10
assert $num {== $_ 10} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
set binval "111011111111010100100100001010000001110111010110100011110100111"	# 63 bits
scan $binval %b num
puts "$num len([string length $binval])"
assert $num {== $_ 2428374928734322332223399} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
set binval "111111111111111111111111111111111111111111111111111111111111111"	# 63 bits
scan $binval %b num
puts "$num len([string length $binval])"
puts "$num"							# 9223372036854775807
									# 2428374928734322332223399
assert $num {== $_ $INT_MAX} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
set binval "0111111111111111111111111111111111111111111111111111111111111111"	# 63 bits
scan $binval %b num
puts "$num len([string length $binval])"
puts "$num"							# 9223372036854775807
									# 2428374928734322332223399
assert $num {== $_ $INT_MAX} -v -line #__LINE__
unset num


puts "Line: "#__LINE__
set binval "0000000000000000000000000000000000000000000000000000000000000000"	# 64 bits
scan $binval %b num
puts "$num len([string length $binval])"
puts "$num"							# 18446744073709551615
									# 2428374928734322332223399
assert $num {== $_ $UNSIGNED_MIN} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
set binval "1111111111111111111111111111111111111111111111111111111111111111"	# 64 bits
scan $binval %b num
puts "$num len([string length $binval])"
puts "$num"							# 18446744073709551615
									# 2428374928734322332223399
assert $num {== $_ $UNSIGNED_MAX} -v -line #__LINE__
unset num


# %d (integer) # # # # # # # # # # # # # # # # # # # # # # # # # 

puts "Line: "#__LINE__
scan 20161122 %d num
puts "$num"							# 20161122
assert $num {== $_ 20161122} -v -line #__LINE__
unset num

puts "Line: "#__LINE__
scan 20161122 %4i%2i%2i y m d			# Scan string 20161122 *into* format-mask *giving* varlist
puts "$y/$m/$d"							# 2016/11/22

assert $y {== $_ 2016} -v -line #__LINE__
assert $m {== $_ 11} -v -line #__LINE__
assert $d {== $_ 22} -v -line #__LINE__
newline
unset y m d

# # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
set ip 12.34.56.78
scan $ip %d.%d.%d.%d o1 o2 o3 o4		# scan string format-mask varlist...
#vars o*
assert "${o1}.${o2}.${o3}.$o4" {eq $_ "12.34.56.78"} -v -line #__LINE__

unset ip o1 o2 o3 o4

# OCTAL ################################################################

puts "Line: "#__LINE__
unset a b c -nocomplain
set string "01 02 03"
scan $string "%o%o%o" a b c								# Octal
assert "$a $b $c" {eq $_ "1 2 3"} -v -line #__LINE__
unset a b c -nocomplain

puts "Line: "#__LINE__
unset a b c -nocomplain
set string "010203"
scan $string "%o" a						# Octal
assert $a {== $_ 4227} -v -line #__LINE__
unset a b c -nocomplain

puts "Line: "#__LINE__
unset a b c -nocomplain
set string "010203"
scan $string "%2o%2o%2o" a b c						# Octal
assert "$a $b $c" {== $_ "1 2 3"} -v -line #__LINE__
unset a b c -nocomplain

/* # FIX -1
# 0o777777777777777777777 # INT_MAX # Uses sscanf [inttooct 9223372036854775807]
puts "Line: "#__LINE__
set string "777777777777777777777"
scan $string "%o" a						# Octal
assert "$a" {== $_ $INT_MIN} -v -line #__LINE__
unset a
# */

# [inttooct 922337203685477580999999] -> 0o606477 ? (uses int64_to_octal -> _snprintf %I64o)

# HEX ################################################################

# # # # # # # # # # # # # # # # # # # # 
puts "Line: "#__LINE__
set string "#08D03F"
scan $string "#%2x%2x%2x" r g b								# Lower case x
assert "$r $g $b" {== $_ "8 208 63"} -v -line #__LINE__
unset r g b -nocomplain


puts "Line: "#__LINE__
set string "#08D03F"
scan $string "#%2X%2X%2X" r g b								# Upper-case X
assert "$r $g $b" {== $_ "8 208 63"} -v -line #__LINE__
unset r g b -nocomplain

puts "Line: "#__LINE__
set string "#08D03F"
set result [scan $string "#%2x%n%2x%n%2x%n" r l1 g l2 b l3]
puts "result: $result $r $l1 $g $l2 $b $l3"				# 6 8 3 208 5 63 7
assert "$r $l1 $g $l2 $b $l3" {== $_ "8 3 208 5 63 7"} -v -line #__LINE__  # "8 3 208 5 63 7"
unset r g b -nocomplain


puts "Line: "#__LINE__
set string "#08D03F"
catch {
	set result [scan $string "#%2.2x%n%2.2x%n%2.2x%n" r l1 g l2 b l3]
	puts "result: $result $r $l1 $g $l2 $b $l3"				# 6 8 3 208 5 63 7
	assert "$r $l1 $g $l2 $b $l3" {== $_ "8 3 208 5 63 7"} -v -line #__LINE__  # "8 3 208 5 63 7"
	unset r g b l1 l2 l3 -nocomplain
}

# CHAR ################################################################

# # # # # # # # # # # # # # # # # # # # 
# ActiveState matches this and counts the skipped chars OK
puts "Line: "#__LINE__
set result [scan $qbf "The %c%n%c" a b c]
puts "result: $result $a $b $c"	
assert "$a $b $c" {== $_ "113 5 117"} -v -line #__LINE__  # "8 3 208 5 63 7"
unset a b c

# # # # # # # # # # # # # # # # # # # # 
# ActiveState% scan "The quick brown" "xxx %c%n%c%n%c%n" a p b q c r; puts "$a $p $b $q $c $r"
# 84 1 104 2 101 3
# Does not exact-match prefix chars in this case and does not count them
# We do

puts "Line: "#__LINE__
puts "Should be 'qui'"
set result [scan $qbf "The %c%n%c%n%c%n" a p b q c r]		# -> "qui"
puts "result: $result $a $p $b $q $c $r"
puts "[chr $a][chr $b][chr $c]"
assert "$a $p $b $q $c $r" {eq $_ "113 5 117 6 105 7"} -v -line #__LINE__ 
assert "[chr $a][chr $b][chr $c]" {eq $_ "qui"} -v -line #__LINE__ 
unset a b c p q r

# # # # # # # # # # # # # # # # # # # # 

set result [scan $qbf "The %s%n%s%n" a b c d]
puts "result: $result $a $b $c $d"
assert "$a $b $c $d" {eq $_ "quick 9 brown 15"} -v -line #__LINE__ 


# INTEGER ################################################################

puts "Line: "#__LINE__
set result [scan "1234567890" "%i" a b c]		# 
puts "$a"
assert "$a" {eq $_ "1234567890"} -v -line #__LINE__ 

# # # # # # # # # # # # # # # # # # # # 

set result [scan "1234567890 Hello" "%i" a b c]		# 
puts "$a"
assert "$a" {eq $_ "1234567890"} -v -line #__LINE__ 

# # # # # # # # # # # # # # # # # # # # 

set result [scan "1234567890 Hello" "%i %s" a b c]		# 
puts "$a $b"
assert "$a $b" {eq $_ "1234567890 Hello"} -v -line #__LINE__ 

# DOUBLE %f ###########################################################

puts "Line: "#__LINE__
scan " $pi " {%4f} p
assert $p {== $_ 3.14} -v -line #__LINE__

scan " 23 " {%i} p
assert "$p" {eq $_ "23"} -v -line #__LINE__

# DONE FIX apply format to %f
scan " $pi 23 " {%4f %i} p q
assert "$p $q" {eq $_ "3.14 23"} -v -line #__LINE__

# FIX
scan " $pi 23 " {%2f %i} p q
#assert "$p $q" {eq $_ "3.23"} -v -line #__LINE__

# FIX format string does not return limited range
# Should be "3.14" (ActiveState)
puts "pi is $pi"			#  3.14159265358979323
scan " $pi 23 " {%4f %i} p q
assert "$p $q" {eq $_ "3.14 23"} -v -line #__LINE__

# FIX 23.139999 as %5f does not work when truncated to x digits using NULL
scan " 2$pi 23 " {%5f %i} p q
assert "$p $q" {eq $_ "23.14 23"} -v -line #__LINE__	# ActiveState


# GENERAL %g ###########################################################
puts "Line: "#__LINE__
scan " $pi " {%4g} p
assert $p {== $_ 3.14} -v -line #__LINE__

scan " 23 " {%i} p
assert "$p" {eq $_ "23"} -v -line #__LINE__

# FAIL does not skip string up to whitepace
scan " $pi 23 " {%4g %i} p q
assert "$p $q" {eq $_ "3.14 23"} -v -line #__LINE__

# FAIL 
scan " $pi 23 " {%4g %i} p q
assert "$p $q" {eq $_ "3.14 23"} -v -line #__LINE__


puts "Line: "#__LINE__
# FAIL
set result [scan "1234567890 Hello" "%i%n%s%n" a b c d]		# 
puts "$a $b $c $d"
assert "$a $b $c $d" {eq $_ "1234567890 10 Hello 16"} -v -line #__LINE__ 

set result [scan "1234567890" "%2i%4i%4i" a b c]		# 
puts "$a $b $c"
assert "$a $b $c" {eq $_ "12 3456 7890"} -v -line #__LINE__ 




# STRING ################################################################

# # # # # # # # # # # # # # # # # # # # 
# v1.25+
option expression on
newline
puts "Line: "#__LINE__
set string " a string {with braced words} + leading space "
set words {}
while {[scan $string %s%n word length] == 2} {
	lappend words $word
	set string [string range $string $length end]
}
assert $words {eq $_ "a string \{with braced words\} + leading space"} -v -line #__LINE__

puts "Line :"#__LINE__
unset length string word words -nocomplain
#unset * -glob -nocomplain
unset r s t u v -nocomplain

# FAIL Goes awry at %4f
# Note: vars must NOT exist
puts "Line :"#__LINE__
set r [scan "hello there 1 $pi 12" {%s %*s %i %4f %2x} s t u v]
puts "Line :"#__LINE__
puts "$r, $s $t $u $v"
puts "Line :"#__LINE__
assert "$r $s $t $u $v" {eq $_ "4 hello 1 3.14 18"} -v -line #__LINE__

puts "Line :"#__LINE__
unset r s t u v -nocomplain

# Problem case
# scan_iso_date.tcl
puts "Line :"#__LINE__
if {! [defined yyyy-mm-dd]} {

	proc yyyy-mm-dd {dtstring} {
		puts "DEBUG: dtstring:'$dtstring'"
		set time {} ;# this allows pure dates without time
		scan $dtstring "%d-%d-%d %s" year month day time
		puts "DEBUG: month:'$month' day:'$day' year:'$year' time:'$time'"
		puts [clock scan "$month/$day/$year $time"]
		clock scan "$month/$day/$year $time"
	}

	set q [yyyy-mm-dd "2018-01-05 01:35:56"]
	assert $q {== $_ 1515116156} -v -line #__LINE__
	puts [date [ctime_to_double $q]]
	assert [date [ctime_to_double $q]] {eq $_ "2018/01/05 01:35:56"} -v -line #__LINE__

	undef yyyy-mm-dd -nocomplain
	unset q 
}

puts "Line :"#__LINE__

unset a b binval c d p l1 l2 l3 result -nocomplain
vars

puts "Line :"#__LINE__
###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #