##################################################################################
# File: 		test_scoped_for_loop.tcl
# Purpose:		Test file for complex, nested, scoped variables (for loop)
# Author: 		M Shaw
# Language:		Ticol Tcl v1.21+, v1.26+
#
# Versions:
# 1.00 03/Dec/2016 First version
# 1.01 21/Jan/2022 Updated (v1.26) added scoped vars to internal expr routine
##################################################################################
# Notes:
# DONE FIX does ++ support nested $?
# DONE FIX [for] with 'option expression on' doesn't support double-dereferences
##################################################################################
option expression on

set scope fred				# Alias scope to fred
set fred::limit 500			# Set the underlying for loop max variable
#++ ${scope}::limit	# [++] works ok
set fred::j 0

# Test that scoped variables can be read
assert $scope {== $_ fred} -v -line #__LINE__
assert ${${scope}::j} {== $_ 0} -v -line #__LINE__
assert ${${scope}::limit} {== $_ 500} -v -line #__LINE__
newline

/*	# DEV/DEBUG ONLY ############################################################
#puts [expr $fred::j]
#puts "[expr ${${scope}::limit}] at line:"#__LINE__
for {set ${scope}::j 0}    { [set ${scope}::j] < ${${scope}::limit} }   {++ ${scope}::j} {
#for {set ${scope}::j 0}    { [set ${scope}::j] < [set ${scope}::limit] }   {++ ${scope}::j} {
	puts ${${scope}::j}
}
die "* DONE *"	
################################################################################ */

#watch add fred::j

################################################################################ */
# Notepad++ 8.1.9.3 bracket highlighting bug!
# Uncomment both of these statements to test
#"${${scope}::limit}"	# Brackets match if string is quoted or commented out
#${${scope}::limit}		# Brackets won't pair correctly if this statement is uncommented in Tcl file mode
						# This bug doesn't occur when file is saved in TXT mode
##################################################################################
						
# Debug check variable dereferencing outside the [for] loop
puts "\${scope}='${scope}' at line:"#__LINE__
puts "\${\${scope}::limit}=${${scope}::limit} at line:"#__LINE__
puts "[set ${scope}::j] at line :"#__LINE__
puts "${${scope}::limit} at line:"#__LINE__
puts "[expr ${${scope}::limit}] at line:"#__LINE__
puts "for arg 2= [expr [set ${scope}::j] < "${${scope}::limit}"] at line:"#__LINE__	# PASS
puts "for arg 2= [expr [set ${scope}::j] < ${${scope}::limit}] at line:"#__LINE__	# PASS
puts "for arg 2= [expr [set ${scope}::j] < [set ${scope}::limit]] at line:"#__LINE__	# PASS

##################################################################################
# MAIN TEST LOOP. This is where we put the scoped, nested variables to work in the [for] loop
##################################################################################
for {set ${scope}::j 0}    { [set ${scope}::j] < ${${scope}::limit} }   {++ ${scope}::j} {
#for {set ${scope}::j 0} {[set ${scope}::j] < [set ${scope}::limit]} {++ ${scope}::j} {	# This form is OK

# Var traces...
#puts "[set ${scope}::j] at line :"#__LINE__
#puts "\${scope}= ${scope} Line :"#__LINE__
#puts "\${scope}::j= ${${scope}::j} Line :"#__LINE__
#puts "\${\${scope}::limit}=${${scope}::limit} Line :"#__LINE__
#puts "for arg 2 state= [expr [set ${scope}::j] < ${${scope}::limit}] at line:"#__LINE__	# PASS

/*	# DEV ONLY (Trace [for] iterations) ###########################################
	set q [set ${scope}::j]
	if {[is_mod $q 100]} {
	   puts "scope::j= ${${scope}::j} is $q at Line :"#__LINE__
	}	# */
	# Can also use the shorter form...
	set q [set ${scope}::j]
	is_mod $q 100 {
		puts "scope::j= ${${scope}::j} is $q at Line :"#__LINE__
	}
	###############################################################################
}

puts "\${scope}::j= ${${scope}::j} Line :"#__LINE__
puts "\${\${scope}::limit}=${${scope}::limit} Line :"#__LINE__
# Check that the [for] loop executed for the correct number of iterations
assert ${${scope}::j} {== $_ ${${scope}::limit}} -v -line #__LINE__

###############################################################################
unset scope q fred::* -nocomplain -glob		# unset can handle globbed+scoped vars
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #