######################################################################################
# File:			test_setb.tcl
# Author:		M Shaw
# Purpose:		Test [struct setb] and [setb]
# Language:		Ticol Tcl
#
# Versions:
# 1.00 24/Feb/2017	First version (test results)
# 1.01 10/Jul/2017	Replaced [checkex] with [assert]
# 1.02 19/May/2018	Added checks for 4 byte pointer references
######################################################################################
# Notes:
# FIX test in debugger on it's own
# See test_address.tcl
# ADD test literal address arg is handled
######################################################################################
/*
struct s {a 5 b 10 c 20}

Tcl var   Address                 Struct block offset
-----------------------------------------------
s      => 32542464 (0x1f08f00)   0
s.a    => 32542464 (0x1f08f00)   0
s.b    => 32542469 (0x1f08f05)   5
s.c    => 32542479 (0x1f08f0f)  15

*/
######################################################################################

option echo on
option expression off

textcolor yellow
puts "Test  \[setb\] ...\n"
textcolor

vars

unset s -nocomplain
struct unset s -nocomplain

puts "Line: "#__LINE__

struct s {
	a 10
	b 20
	c 40
}

checkheap #__LINE__ -v

struct set s.a $qbf
puts "\[ofaddress s.a\] '[ofaddress s.a]' [string length [ofaddress s.a]]"
assert [eqn [ofaddress s.a] $qbf 10] {eq $_ 1} -v -line #__LINE__

checkheap #__LINE__ -v

puts "qbf len==[string length $qbf]"
struct set s.c $qbf
#struct dump s
puts "\[ofaddress s.c\] '[ofaddress s.c]' [string length [ofaddress s.c]]"
assert [eqn [ofaddress s.c] $qbf 40] {eq $_ 1} -v -line #__LINE__

checkheap #__LINE__ -v

# Test invalid address 
puts "* Catch bad address..."
catch {
	assert [ofaddress 12345] {!= $_ 0} -v -line #__LINE__	# GPF
}
newline

checkheap #__LINE__ -v

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


# ofaddress

checkheap #__LINE__ -v
textcolor yellow
puts "Testing \[ofaddress\]"
textcolor
set q Hello
puts "Addresof \$q is [addressof q] => '[ofaddress [addressof q]]'"
assert [ofaddress [addressof q]] {eq $_ Hello} -v -line #__LINE__

# ofaddressb
checkheap #__LINE__ -v
textcolor yellow
puts "Testing \[ofaddressb\]"
textcolor
# More work needed!
assert [ofaddressb [addressofb s.a]] {> $_ 0} -v -line #__LINE__

/* INFO //////////////////////////////////////////////////////////////////////////
For structs:
$struct.member						Returns field data-block address offset
[ofaddressb $struct.member]			Returns the field contents where ASCII
[struct setb struct.member 1024]	Set an integer binary value ... (Little endian: 1F 09 02 00)   09 1F 00 02 0002091F
[ofaddressb struct.member]			Returns translated binary contents
[addressof struct.member]			Invalid
[addressof $struct.member]			Invalid
[addressofb struct.member]			Invalid
[addressofb $struct.member]			Error


For variables:

 var 'varname' level 0 (0x20917f0) data @34150464 (0x2091840)
       43 byte(s)
 Address    0  1  2  3  4  5  6  7    8  9  A  B  C  D  E  F  ASCII
 --------  -----------------------   -----------------------  ----------------
 02091840  54 68 65 20 71 75 69 63 - 6B 20 62 72 6F 77 6E 20  The quick brown
 
$varname							Dereferences varname
[addressof varname]					Returns variable address
[ofaddress [addressof varname]]		Returns the original variable
[addressofb varname]				Returns the address of the variable data block (see dump)
[ofaddressb [addressofb varname]]	Returns the original variable
# Where varname does not contain a valid address
[addressofb $varname]				Invalid
[ofaddressof $varname]				Error
[ofaddressof $varname]				Invalid
////////////////////////////////////////////////////////////////////////// */


# FIX addressofb no longer works Nov 2017
checkheap #__LINE__ -v
puts "\$s.a==$s.a ? \[addressof s.a\]=[addressof s.a] \[addressofb s.a\]=[addressofb s.a]"
#puts "\[addressofb \$s.a\]=[addressofb $s.a]"

puts "Line :"#__LINE__
checkheap #__LINE__ -v
struct set s.a Hello
#assert [ofaddressb $s.a] {eq $x Hello}		# Added $ dereference 01/Jun/2017
puts "Line :"#__LINE__
assert $s.a {eq $_ Hello} -v -line #__LINE__

checkheap #__LINE__ -v
struct unset s -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Store binary equivalent of string literal number
# Struct field must be wide enough to hold converted binary equivalent in the range
# char, short/word, long/dword, qword
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

checkheap #__LINE__ -v
struct unset foo -nocomplain
checkheap #__LINE__ -v
struct foo {
	a 4
}

checkheap #__LINE__ -v

# setb corrupts heap here
struct setb foo.a 255 4										# char
assert [ofaddressb foo.a] {eq $_ 255} -v -line #__LINE__
struct setb foo.a 65535											# word
assert [ofaddressb foo.a] {eq $_ 65535} -v -line #__LINE__
struct setb foo.a 123456789									# long
assert [ofaddressb foo.a] {eq $_ 123456789} -v -line #__LINE__

# /*
puts "Line "#__LINE__
set q "Hi there"
puts "Storing address [addressof q]"		# Requires a variable not string literal
struct setb foo.a [addressof q]
#=>> foo.a
puts [ofaddress [ofaddressb foo.a]]
assert [ofaddress [ofaddressb foo.a]] {eq $_ $::q} -v -line #__LINE__

# Store single full command with args in var then in struct field as ptr
puts "Line "#__LINE__
checkheap #__LINE__ -v
#set q "puts Hello-world"			# Can execute only a single command
set q "[* 33.2 7.2345]"
checkheap #__LINE__ -v
struct setb foo.a [addressof q]
{*}[ofaddress [ofaddressb foo.a]]			# One command
assert [round {*}[ofaddress [ofaddressb foo.a]] 4]  {== $_ 240.1854} -v -line #__LINE__

# Store multiple commands with args in var then in struct field as ptr
checkheap #__LINE__ -v
puts "Line "#__LINE__
set q "textcolor yellow; puts stdout {Hello-world}; textcolor"
checkheap #__LINE__ -v
struct setb foo.a [addressof q]
checkheap #__LINE__ -v
eval {*}[ofaddress [ofaddressb foo.a]]		# Many commands

# */


puts "Line "#__LINE__
checkheap #__LINE__ -v
struct set foo.a Hello
checkheap #__LINE__ -v
dump foo
checkheap #__LINE__ -v
struct unset foo -nocomplain				# Heap damage here
checkheap #__LINE__ -v
unset q -nocomplain

# FIX detects width as 4

puts "Line "#__LINE__
checkheap #__LINE__ -v
struct foo {
	a 8									# QWORD
}
puts "Line "#__LINE__
struct setb foo.a 1234567890123456789 					# qword
dump foo
assert [ofaddressb foo.a 8] {eq $_ 1234567890123456789} -v -line #__LINE__

# FIX -value not working?
puts "Line "#__LINE__
struct setb foo.a 1234567890123456789 -value					# qword
checkheap #__LINE__ -v
assert [ofaddressb foo.a 8] {eq $_ 1234567890123456789} -v -line #__LINE__
checkheap #__LINE__ -v

# Set string literal using [setb]
struct setb foo.a {Hello world}			# New 03/Dec/2019 - Will set chars
assert $foo.a {eq $_ {Hello wo}} -v -line #__LINE__		# Field length is 8


struct unset foo -nocomplain

puts "Line "#__LINE__
checkheap #__LINE__ -v
vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #