#################################################################################
# File:				test_spawn_vbs.tcl
# Author:			M Shaw
# Purpose:			Tests [spawn] using VBS.EXE
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 15/Nov/2019	First version. A member of tests
#################################################################################
# Notes:
#################################################################################

puts "info script-> '[info script]'"
if {|| [eq [info script] "internal.tcl"] [eq [info script] ""]} {
	puts "test_spawn.tcl won't run in embedded Tcl"
	return
} else {

	if {! [file exists vbs.exe]} {
		textcolor red
		puts "Test skipped: test_spawn_vbs.tcl requires VBS.EXE (VBScript shell)\n"
		textcolor
		pause
		return
	}

	if {eqi [array item env windir -nocase] "C:\\ReactOS"} {	# No SMB stack on ROS yet
		puts "Not running [info script] on ReactOS"
		return
	}

	set s "
	Set fso = CreateObject (\"Scripting.FileSystemObject\")
	Set Console = fso.GetStandardStream (1)
	Console.WriteLine eval(atn(1)*4)
	"

	#################################################################################
	# CANNOT spawn cscript.exe with anything other than a filename
	#unset x -nocomplain
	#spawn x cscript.exe //NoLogo //I $s
	#puts $x

	#################################################################################
	# Command-line mode vbs.exe requires C-style escaped CRLF and dquotes
	# We must unescape both of these before passing to extenal app, VBS.EXE
	set s [string replace $s "\r\n" "\\n"]
	set s [string replace $s "\"" "\\\""]
	textcolor darkyellow
	#puts $s
	textcolor
	spawn x vbs.exe $s
	# [spawn] will return the literal output including any CRLF returned from VBScript by VBS.EXE
	trim x "\r\n" -var
	puts "VBS.EXE Result was '$x'"

	assert $x {== $_ 3.14159265358979} -v -line #__LINE__

	unset s x -nocomplain

	#################################################################################
	newline
	vars
	newline
	textcolor green
	puts "* Done *"
	textcolor
}
# EOF #