##############################################################################
# File: 			test_split.tcl
# Purpose:			Various assorted bugs to test and fix
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 03/Sep/2018	First version
# 1.01 11/Mar/2022	Updated
##############################################################################
# Notes:
# DONE FIX borked charmask Sep/2019
##############################################################################
/*
Returns a list created by splitting string at each character that is in the
splitChars argument. Each element of the result list will consist of the
characters from string that lie between instances of the characters in splitChars.

Empty list elements will be generated if string contains adjacent characters in
splitChars, or if the first or last character of string is in splitChars. 
If splitChars is an empty string then each character of string becomes a
separate element of the result list. SplitChars defaults to the standard
white-space characters. 
*/
##############################################################################

option expression off

#assert [split "comp.lang.tcl"] {eq $_ {c o m p . l a n g . t c l}} -v -line #__LINE__	# Not valid for A/S

# ActiveState examples
assert [split "comp.lang.tcl" . ] {eq $_ {comp lang tcl}} -v -line #__LINE__					# A/S
assert [split "alpha beta gamma" "temp"] {eq $_ {al {ha b} {} {a ga} {} a}} -v -line #__LINE__	# A/S
assert [split "Hello world" {}] {eq $_ {H e l l o { } w o r l d}} -v -line #__LINE__			# A/S

assert [split "abcd"] {eq $_ abcd} -v -line #__LINE__							# A/S
assert [split "abcd" ""] {eq $_ "a b c d"} -v -line #__LINE__					# A/S
assert [split "a b c d" ""] {eq $_ "a { } b { } c { } d"} -v -line #__LINE__	# A/S (Tokenised space is wrapped in braces)

assert [split $qbf " "] {eq $_ $::qbf} -v -line #__LINE__						# A/S
assert [split $qbf sdfsadf] {eq $_ {{The quick brown } {ox jump} { over the l} {zy } og}} -v -line #__LINE__  # A/S
#                                                     f         ed            a     d
assert [split $qbf "T"] {eq $_ {{} {he quick brown fox jumps over the lazy dog}}} -v -line #__LINE__						# A/S
#                               T
# FiX tokenise trailing NULL case
assert [split $qbf "g"] {eq $_ {{The quick brown fox jumps over the lazy do} {}}} -v -line #__LINE__						# A/S
#                                                                             g
# AS     "T {} { quick brown fox jumps ov} {r } {} {} { lazy dog}"
# Ticol: "T { quick brown fox jumps ov} {r } { lazy dog}"
assert [split $qbf "the"] {eq $_ {T {} { quick brown fox jumps ov} {r } {} {} { lazy dog}}} -v -line #__LINE__						# A/S
#                                   he                            e     th he       

# MAN file example
set s "The quick\r\nbrown fox\r\njumps over\r\nthe lazy\r\ndog!"
split $s [chr 0x0d][chr 0x0a] -literal -array a
for {set i 0} {< $i [array size a]} {++ i} {
	puts "\t$i->'$a($i)'"
}

assert [array size a] {== $_ 5} -v -line #__LINE__
assert $a(0) {eq $_ "The quick"} -v -line #__LINE__
assert $a(1) {eq $_ "brown fox"} -v -line #__LINE__
assert $a(2) {eq $_ "jumps over"} -v -line #__LINE__
assert $a(3) {eq $_ "the lazy"} -v -line #__LINE__
assert $a(4) {eq $_ "dog!"} -v -line #__LINE__

array unset a -nocomplain

split $qbf {} -array a
assert [array exists a] {eq $_ $::true} -v -line #__LINE__
assert $a(0) {eq $_ T} -v -line #__LINE__
assert $a(42) {eq $_ g} -v -line #__LINE__

# Verified in A/S
assert [split $qbf {}] {eq $_ {T h e { } q u i c k { } b r o w n { } f o x { } j u m p s { } o v e r { } t h e { } l a z y { } d o g}} -v -line #__LINE__
puts [split $qbf s]
echo [split $qbf s]
assert [split $qbf s] {eq $_ "{The quick brown fox jump} { over the lazy dog}"} -v -line #__LINE__
# Clean up #

array unset a -nocomplain
unset i s -nocomplain 

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\a
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #