###############################################################################################
# File:				test_stack.tcl
# Author:			M Shaw
# Purpose:			Test file for [stack]
# Language:			Ticol Tcl v1.22+..v1.26+
#
# Versions:
# 1.00 04/Mar/2017	Migrated from tests.tcl
# 1.04 08/Jul/2017	Replaced [checkex] with [assert]
# 1.05 28/Nov/2019	Added a few more tests
# 1.06 21/Nov/2021	Test bugfix revealed due to stack binary fixes
###############################################################################################
# Notes
###############################################################################################
/*
integer [stack count    varName]
bool    [stack create   varName size]
integer [stack push     varName value ?value? ... ]
string  [stack pop      varName]
string  [stack peek     varName]
void    [stack foreach  varName indexVar {code} ?-r?]
string  [stack item     varName index ?default?]
list    [stack list     varName]
list    [stack listr    varName]
integer [stack size     varName]
bool    [stack is_empty varName]
bool    [stack clear    varName]
bool    [stack unset    varName]
*/

option expression off

textcolor yellow
puts "Testing \[stack\] ...."
textcolor
newline

option echo on
textcolor yellow
puts "Testing push peek pop ..."
textcolor

# FIX crashes DLL version //////////////////////////////////////////////////////////////////////////////
set i 0
try {
	stack unset st					# stack unset st -nocomplain now returns TRUE (21/Nov/2021)
} catch {
	set i 1
}
assert $i -v -line #__LINE__

set i 1								# Invert logic and retest
try {
	stack unset st -nocomplain		# stack unset st -nocomplain now returns TRUE (21/Nov/2021)
} catch {
	set i 0
}
assert $i -v -line #__LINE__



textcolor cyan
#puts "Testing catch stack unset: $fid"
textcolor

unset st -nocomplain

stack create st 100
assert [stack size st] {== $_ 100} -v -line #__LINE__
assert [stack count st] {== $_ 0} -v -line #__LINE__
stack push st 23
assert [stack count st] {== $_ 1} -v -line #__LINE__
stack push st Hello
assert [stack count st] {== $_ 2} -v -line #__LINE__
stack push st 10
assert [stack count st] {== $_ 3} -v -line #__LINE__
assert [stack peek st] {== $_ 10} -v -line #__LINE__

set total 0
set i 0
stack foreach st ix {
	+= total $ix
	++ i
}
assert $total {== $_ 33} -v -line #__LINE__
unset total
assert $i {== $_ 3} -v -line #__LINE__

assert [stack item st 0] {== $_ 23} -v -line #__LINE__
assert [stack item st 1] {== $_ Hello} -v -line #__LINE__
assert [stack item st 2] {== $_ 10} -v -line #__LINE__

assert [stack list st] {eq $_ {10 Hello 23}} -v -line #__LINE__
assert [stack listr st] {eq $_ {23 Hello 10}} -v -line #__LINE__

##########################################################################
# Remove from stack...
stack pop st
assert [stack count st] {== $_ 2} -v -line #__LINE__
assert [stack peek st] {eq $_ Hello} -v -line #__LINE__
assert [stack pop st] {eq $_ Hello} -v -line #__LINE__
assert [stack peek st] {== $_ 23} -v -line #__LINE__
assert [stack count st] {== $_ 1} -v -line #__LINE__
assert [stack is_empty st] {== $_ 0} -v -line #__LINE__

# Error condition
assert [stack pop st] {== $_ 23} -v -line #__LINE__
assert [stack count st] {== $_ 0} -v -line #__LINE__
assert [stack is_empty st] {== $_ 1} -v -line #__LINE__
assert [stack peek st] {== $_ {}} -v -line #__LINE__

/*
set i 0
try {
	puts [stack pop st]		# Does not raise a fatal error
} catch {
	set i 1
}
assert $i -v -line #__LINE__
# */

stack unset st -nocomplain
assert [! [is_set st]] -v -line #__LINE__

unset fid i ix -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #