###################################################################################
# File: 			test_static_stack.tcl
# Purpose:			Tests for static stack fixups
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 11/Nov/2017	First version
# 1.01 11/Jul/2018	Revised and bugfix for root level use. Now a test
###################################################################################
# Notes:
# [static] acts only on standard variables and arrays
###################################################################################

option echo on
option expression off
option stacktrace off

vars

textcolor yellow
puts "Testing proc \[static\] in proc with stack ..."
textcolor
newline

vars

puts "Line: "#__LINE__

/* # Code for comparison
undef proc_static -nocomplain

if {! [defined proc_static]} {
	proc proc_static {name {value {}}} {
#puts "Enter proc_static"
		if {> [info level] 1} {							# If called from root lvel this will be level 1
#puts "Case proc: Level is [info level]"
			set scope [uplevel 1 {info procname}]		# Save procname as new scope name (simpler to parse)
#puts "Setting varname as ${scope}::$name"			
			set varname ${scope}::$name					# Create mangled root scope var name
			#uplevel 1 "upvar @0 $varname $name"     	# Run upvar in the root callframe
			if {! [is_set ::$varname]} {                # If root var is empty and we have an arg
#puts "Case: var ::$varname not set"
				uplevel 1 "upvar @0 $varname $name"     # Run upvar in the root callframe
				uplevel 1 "set $name $value"            # Set local alias. See inheritance notes
			} else {
#puts "Case: var ::$varname is set so upvar it"
				uplevel 1 "upvar @0 $varname $name"     # Run upvar in the root callframe to set parent's local var
			}
		} elseif {! [is_set ::$name]} {                 # We are calling [static] in the root namespace scope
#puts "Case root: Level is [info level]"
			set ::$name $value							# Use set but only on initialisation
		}
	}
} # */


/*
# [proc] level 2
undef proc_static -nocomplain

if {! [defined proc_static]} {					# Avoid errors if re-run
  proc proc_static {name {value {}}} {
    if {> [info level] 1} {						# If not root, will be 1
      set scope [uplevel 1 {info procname}]		# Save procname as new scope
      set varname ${scope}::$name				# Create root scope name
      if {! [is_set ::$varname]} {				# If root var name empty
        uplevel 1 "upvar @0 $varname $name"		# Run upvar in root
        uplevel 1 "set $name $value"			# Set local alias
      } else {
        uplevel 1 "upvar @0 $varname $name"		# Run upvar in root ...
      }											# ... callframe at level 1
    } elseif {! [is_set ::$name]} {				# We are in root namespace
      set ::$name $value						# Set on init
    }
  }
} # */


/* # MK1 (does not handle root use)
undef static_stack -nocomplain

proc static_stack {name args} {
	set scope [uplevel 1 {info procname}]				# Save procname as new scope name (simpler to parse)
	set varname ${scope}::$name							# Create mangled root scope var name
puts "static_stack: Varname is '$varname'"
	if {! [is_set ::$varname]} {						# If root var is empty and we have an arg
puts "static_stack: Creating '$varname'..."
		uplevel @0 "stack create $varname $args"		# Create the new stack
	}
	uplevel 1 "upvar @0 $varname $name"					# Run upvar in the root callframe
	#vars -level 0
	#vars
} # */


#/* # MK2
undef static_stack -nocomplain

proc static_stack {name args} {
	# Static stack nits but does not push content
	set scope [uplevel 1 {info procname}]				# Save procname as new scope name (simpler to parse)
	set varname ${scope}::$name							# Create mangled root scope var name
	if {> [info level] 1} {								# If not root, will be 1
		if {! [is_set ::$varname]} {					# If root var is empty and we have an arg
#puts "static_stack: Level Creating [info level] '$varname'..."
			uplevel @0 "stack create $varname $args"	# Create the new stack
		}
		uplevel 1 "upvar @0 $varname $name"				# Run upvar in the root callframe
	} else {
#puts "static_stack: Level Creating root [info level] '$name'..."
		if {! [is_set $name]} {
			uplevel @0 "stack create $name $args"	# Create the new stack
		}
	}
} # */


undef foo -nocomplain
proc foo {x y} {
	static_stack $x 100	# Inits but does not push content
	#vars
	puts "foo: Pushing $y onto stack $x"
	stack push $x $y
	#dump $x
}

newline
foo s Hello
foo s World

newline
textcolor cyan
puts "Level 0 Results"
textcolor
vars
newline

set i 1
stack foreach foo::s x {
	puts "stack item $i: '$x'"
	++ i
}

assert [stack pop foo::s] {eq $_ World} -v -line #__LINE__
assert [stack pop foo::s] {eq $_ Hello} -v -line #__LINE__
assert [stack size foo::s] {== $_ 100} -v -line #__LINE__

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

#ifdef comment
proc foo {x y} {
	proc_static a						# Declare a generic static var
	puts "a is [type a]"
	puts "foo::a is [type foo::a]"
	
	stack create a
	stack push a Hello
/*	
	set a($x) $y						# Assign to local alias copy
	puts "foo::a is [type foo::a]"		# Curiously, this resolves
	puts "::foo::a is [type ::foo::a]"
	if {eq $y Hello} {
		assert $a($x) {eq $_ Hello} -v -line #__LINE__			# PASS
		assert $::foo::a($x) {eq $_ Hello} -v -line #__LINE__	# PASS
	} else {
		assert $a($x) {eq $_ World} -v -line #__LINE__			# PASS
		assert $::foo::a($x) {eq $_ World} -v -line #__LINE__	# PASS
	}
	puts "$::foo::a($x)"				# Access global copy
	
	set ::foo::a(3) "Three"				# Direct set in global (root) scope

	# vars -level 0						# PASS exists
	textcolor magenta
	array walk ::foo::a					# Fails
	textcolor
*/	
}
#endif


unset i x -nocomplain
stack unset foo::s -nocomplain
vars

# Test static_stack in root level
static_stack s 123
vars
stack push s hello
stack push s world
assert [stack pop s] {eq $_ world} -v -line #__LINE__
assert [stack pop s] {eq $_ hello} -v -line #__LINE__
assert [stack size s] {== $_ 123} -v -line #__LINE__
stack unset s

undef proc_static -nocomplain
undef static_stack -nocomplain

vars


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Done *"
textcolor