###############################################################
# File: 			test_static_struct.tcl
# Purpose:			Test file for return exits
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 26/Jun/2018	First version
###############################################################
# Notes:  man file example
###############################################################
/*
Static Structs
--------------
Structs can be made static within a proc using the following proc
Note that structs are time-costly to set up and require a per-proc-call
fixup
*/
###############################################################

option expression off

textcolor yellow
puts "Testing test_static_struct.tcl"
textcolor

puts "Line: "#__LINE__

if {defined static_struct} {
	die "static_struct already defined in [info script]"
}

vars

puts "Line: "#__LINE__
proc static_struct {name args} {
	set scope [uplevel 1 {info procname}]
	set varname ${scope}::$name
	if {! [is_set ::$varname]} {
		uplevel @0 "struct $varname {$args}"
	}
	uplevel 1 "upvar @0 $varname $name"
	foreach {x y} $args {
		uplevel 1 "upvar @0 ${varname}.$x ${name}.$x"
	}
}

puts "Line: "#__LINE__
undef foo -nocomplain
proc foo {x y} {
	static_struct s {a 20 b 30}		# Create a static struct in the proc
	struct set s.a $x
	struct set s.b $y
}

puts "Line: "#__LINE__
foo Hello world						# Pass 2 arguments

vars
struct dump foo::s

puts "Line: "#__LINE__
puts "foo::s.a '$foo::s.a'"
puts "foo::s.b '$foo::s.b'"

assert $foo::s.a {eq $_ Hello} -v -line #__LINE__
assert $foo::s.b {eq $_ world} -v -line #__LINE__

puts "Line: "#__LINE__
struct unset s -nocomplain
undef static_struct -nocomplain

struct unset foo::s -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
textcolor green
newline
puts "* Done *"
textcolor

# EOF #