#################################################################
# File:				test_string.tcl
# Purpose:			Various string and string conversion tests
#					Does not test underlying string parsing etc.
# Author:			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 25/Dec/2016	First version
# 1.01 09/Jul/2017	Replace [checkex] with [assert]
#                   Added [string range]
# 1.01 21/Jan/2023	Regression fixes
#################################################################
# Notes:
#################################################################
option expression off
option echo on

textcolor yellow
puts "Testing string commands"
textcolor
newline

# asc
textcolor yellow
puts "Testing \[asc\]"
textcolor
assert [asc A] {== $_ 65} -v -line #__LINE__
assert [asc 0] {== $_ 48} -v -line #__LINE__
assert [asc Hello] {== $_ 72} -v -line #__LINE__
assert [asc "Hello"] {== $_ 72} -v -line #__LINE__

# cat
#ifdef comment
textcolor yellow
puts "Testing \[cat\]"
textcolor

set a "he"
set b "llo"
assert [cat $a "llo"] {eq $_ "Hello"} -v -line #__LINE__
# FIX cat: variable 'he' not found
#endif

# char
textcolor yellow
puts "Testing \[char\]"
textcolor
puts "\[char 0b1111000\]=[char 0b1111000] [chr [char 0b1111000]]"
assert [chr [char 0b1111000]] {eq $_ "x"} -v -line #__LINE__

# chr
textcolor yellow
puts "Testing \[chr\]"
textcolor
assert [chr 65] {eq $_ "A"} -v -line #__LINE__

# comma
textcolor yellow
puts "Testing \[comma\]"
textcolor
assert [comma {}] {eq $_ ""} -v -line #__LINE__
assert [comma 1] {eq $_ "1"} -v -line #__LINE__
assert [comma 12] {eq $_ "12"} -v -line #__LINE__
assert [comma 123] {eq $_ "123"} -v -line #__LINE__
assert [comma 1234] {eq $_ "1,234"} -v -line #__LINE__
assert [comma 123456789] {eq $_ "123,456,789"} -v -line #__LINE__
assert [comma 1234567890] {eq $_ "1,234,567,890"} -v -line #__LINE__
assert [comma 123.45] {eq $_ "123.45"} -v -line #__LINE__
assert [comma "123456789.012"] {eq $_ "123,456,789.012"} -v -line #__LINE__			# FAIL
assert [comma "123456789.01234"] {eq $_ "123,456,789.01234"} -v -line #__LINE__

# format
# Very basic checks only
textcolor yellow
puts "Checking \[format\]"
textcolor
assert [format "%f" 10	] {== $_ 10.000000} -v -line #__LINE__
assert [format "%i" 10.234	] {== $_ 10} -v -line #__LINE__
assert [format "%s" Hello	] {== $_ Hello} -v -line #__LINE__

# fromhex
textcolor yellow
puts "Checking \[fromhex\] \[tohex\]"
textcolor
assert [fromhex [tohex Hello]] {eq $_ Hello} -v -line #__LINE__

# left
textcolor yellow
puts "Checking \[left\]"
textcolor
assert [left $qbf 15] {eq $_ "The quick brown"} -v -line #__LINE__
assert [left $qbf 1] {eq $_ "T"} -v -line #__LINE__
assert [string left $qbf 15] {eq $_ "The quick brown"} -v -line #__LINE__
assert [string left $qbf 1] {eq $_ "T"} -v -line #__LINE__

assert [left qbf 15 -var] {eq $_ "The quick brown"} -v -line #__LINE__
assert [left qbf 1 -var] {eq $_ "T"} -v -line #__LINE__
assert [string left qbf 15 -var] {eq $_ "The quick brown"} -v -line #__LINE__
assert [string left qbf 1 -var] {eq $_ "T"} -v -line #__LINE__
#ALLOW assert [left $qbf 0] {eq $_ ""}

# makestr
puts "\[makestr\] ..."
assert [makestr 10 x] {eq $_ xxxxxxxxxx} -v -line #__LINE__

option expression on
# mid
puts "\[mid\] ..."
set q "Hello"
#assert [mid q ww 3 2] {eq $_ Hewwo}
# Currently returns no of chars copied
assert [mid q ww 3 2] {eq $_ 2} -v -line #__LINE__

# mids
puts "\[mids\] ..."
assert [mids $qbf 17 3] {eq $_ fox} -v -line #__LINE__
#assert [mids qbf 17 3 -var] {eq $_ fox} -v -line #__LINE__		# NYI

# randomstr
textcolor yellow
puts "Testing \[randomstr\]"
textcolor
assert [randomstr 10] {ne $_ ""} -v -line #__LINE__

# string range 
textcolor yellow
puts "Testing \[string range\]"
textcolor
assert [string range $qbf 10 18] {eq $_ "brown fox"} -v -line #__LINE__
assert [string range $qbf 12 6] {eq $_ {}} -v -line #__LINE__
assert [string range $qbf 0 1024] {eq $_ $qbf} -v -line #__LINE__
assert [string range $qbf 0 0] {eq $_ T} -v -line #__LINE__

textcolor darkyellow
puts "Catch invalid range: ignore error"
textcolor
assert [catch {
	assert [string range $qbf 0 -1] {eq $_ {}} -v -line #__LINE__
}] {ne $_ 0} -v -line #__LINE__

textcolor darkyellow
puts "Catch invalid range: ignore error"
textcolor
assert [catch {
	assert [string range $qbf -1 0] {eq $_ {}} -v -line #__LINE__
}] {ne $_ 0} -v -line #__LINE__

textcolor darkyellow
puts "Catch invalid range: ignore error"
textcolor
assert [catch {
	assert [string range $qbf -1 -1] {eq $_ {}} -v -line #__LINE__
}] {ne $_ 0} -v -line #__LINE__

textcolor darkyellow
puts "Catch missing arg: ignore error"
textcolor
assert [catch {
	assert [string range $qbf 0] {eq $_ {}} -v -line #__LINE__
}] {ne $_ 0} -v -line #__LINE__

textcolor darkyellow
puts "Catch missing arg: ignore error"
textcolor
assert [catch {
	assert [string range $qbf] {eq $_ {}} -v -line #__LINE__
}] {ne $_ 0} -v -line #__LINE__

# right
textcolor yellow
puts "Testing \[right\] \[string right\]"
textcolor
assert [right $qbf 3] {eq $_ dog -nocase} -v -line #__LINE__
assert [string right $qbf 3] {eq $_ dog -nocase} -v -line #__LINE__
assert [right qbf 3 -var] {eq $_ dog -nocase} -v -line #__LINE__
assert [string right qbf 3 -var] {eq $_ dog -nocase} -v -line #__LINE__

# setr
textcolor yellow
puts "Testing \[setr\]"
textcolor
set x "Hello world"
# FIX consider returning string portion or whole string instead of count
assert [setr x girls] {== $_ 5} -v -line #__LINE__  # FAIL
assert [eq $x "Hello girls"] {eq $_ 1} -v -line #__LINE__
unset x 

# setat
# setat var string|var startpos len
textcolor yellow
puts "Testing \[setat\]"
textcolor
#set q $qbf
set q ""
# No rien
setat q $qbf 17 3
puts "setat q is '$q'"
assert [eq $q fox] {eq $_ 1} -v -line #__LINE__		# ??
unset q 

# string
# TODO add various sub- commands
textcolor yellow
puts "Testing \[string slice\]"
textcolor
set q [string slice $qbf 15 19]
assert [lindex $q 1]  {== $_ "fox"} -v -line #__LINE__
unset q 

#ifdef string
textcolor yellow
puts "Testing \[string length\]"
textcolor
assert [string length $qbf]  {== $_ 43} -v -line #__LINE__
#endif

# stripto
# stripto mask-string variable|string ?-nocase?
set q $qbf
# FIX no return value
#assert [stripto fox q] {eq $_ "jumps"}

# striptor
# FIX no return value

# strptr
textcolor yellow
puts "Testing \[strptr\]"
textcolor
assert [strptr q] {> $_ 0} -v -line #__LINE__

# strsplit
textcolor yellow
puts "Testing \[strsplit\]"
textcolor
assert [strsplit "hello world" " " a] {eq $_ "hello world"} -v -line #__LINE__
assert [strsplit "hello world" "l" a] {eq $_ "he {o wor} d"} -v -line #__LINE__
option echo on
#assert [strsplit "hello world" "" a] {eq $_ "hello world"} -v -line #__LINE__	# FIX should err
#assert [strsplit "hello world" {} a] {eq $_ "hello world"} -v -line #__LINE__	# FIX should err

# strstr
textcolor yellow
puts "Testing \[strstr\]"
textcolor
assert [strstr $qbf fox] {== $_ 17} -v -line #__LINE__

# tohex
textcolor yellow
puts "Testing \[tohex\]"
textcolor
assert [tohex 10] {eq $_ 3130} -v -line #__LINE__

textcolor yellow
puts "Testing \[fromhex\]"
textcolor
assert [fromhex [tohex Hello]] {eq $_ Hello} -v -line #__LINE__

textcolor yellow
puts "Testing \[.\] (dot)"
textcolor
assert [. $qbf] {eq $_ T} -v -line #__LINE__
assert [. qbf -var] {eq $_ T} -v -line #__LINE__
assert [. ""] {eq $_ ""} -v -line #__LINE__
newline
puts "Timing for dot ..."
time {
	. $qbf
} 10000 -info

unset a q -nocomplain

# unquoted string joined with escaped whitespace
assert [llength hello\ world ] {== $_ 2} -v -line #__LINE__
# unquoted string BROKEN BY unescaped whitespace
# Can no longer do this as it is not a single, valid list
# assert [llength hello world ] {== $_ 1} -v -line #__LINE__
assert [llength "hello world" ] {== $_ 2} -v -line #__LINE__

########################################################################
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #