###################################################################################
# File: 			test_string_replace.tcl
# Purpose:			Test [string replace] with various unusual args
# Author: 			M Shaw
# Language:			Ticol Tcl v1.24+
#
# Versions:
# 1.00 03/Nov/2017	First version
# 1.01 20/Dec/2022	Updated for regressions due to MPP now ignoring escaped \# 
###################################################################################
# Notes: 
###################################################################################

option expression off

textcolor yellow
puts "Testing \[string replace\] with escaped strings ..."
textcolor yellow
newline

set s $qbf
set s [string replace $s "e" "a"]
assert $s {eq $_ "Tha quick brown fox jumps ovar tha lazy dog"} -v -line #__LINE

set s {}
set s [string replace $s "\$" "\\\$"]
assert $s {eq $_ ""} -v -line #__LINE__

set s {}
set s [string replace $s "\\\$" "\$"]
assert $s {eq $_ ""} -v -line #__LINE__

# Expand \$ to \\\$
set s {\$}
echo $s
set s [string replace $s "\$" "\\\$"]
echo $s
assert $s {eq $_ "\\$"} -v -line #__LINE__		# CAUTION the [assert] rvalue arg is braced not dquoted so will not expand

# Replace literal '\$' with '$' (unexpand)
set s {\$}
set s [string replace $s "\\\$" "\$"]			# No change in $s
assert $s {eq $_ "$"} -v -line #__LINE__		# CAUTION the [assert] rvalue arg is braced not dquoted so will not expand

# Replace literal '\$\$' with '$' (unexpand)
set s "\$\$"									# Literal "$$" -> literal "\$\$"
echo $s
set s [string replace $s "\$" "\\\$"]
assert $s {eq $_ "\\$\\$"} -v -line #__LINE__

set s {\\\$\\\$}								# Literal escaped "\$\$" -> literal "$$"
echo $s
set s [string replace $s "\\\$" "\$"]
echo $s
assert $s {eq $_ "\\$\\$"} -v -line #__LINE__	# CAUTION the [assert] rvalue arg is braced not dquoted so will not expand


#puts "s is '$s'"

unset s

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # #
newline
textcolor green
puts "* Done *"
textcolor

# EOF #