######################################################################################
# File: 			string_to_array.tcl
# Purpose:			Test for [string_to_array], [array_to_string]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 26/Sep/2018	First version
# 1.01 28/Oct/2019	Fixed tests
# 1.02 07/Jun/2022	Revised for changes to [loop] bounds
# 1.03 28/Dec/2022	Revised again for changes to [loop] ubound
######################################################################################
# Notes:
# string [array_to_string arrayname ?-join string? 
# 	?-start element? ?-stop element? ?-stopstr string? ?-nocase?]
#
# integer [string_to_array string arrayname ?-start element? 
# 	?-chars? ?-asc? ?-append? ?-stopchar char?]
#
######################################################################################
# Tests need expanding
# ADD -stopstr
# ADD -join
# ADD -asc tests
# ADD -start
# DONE ADD -chars
# DONE ADD -append
# DONE ADD -stopchar
# Intended for use joining byte arrrays back into strings
# See [join] as an alternative
# Needs completing
######################################################################################

option expression off
unset a -nocomplain

textcolor yellow
puts "Testing \[array_to_string\] and \[string_to_array\] ... in [file name [info script]]"
textcolor
newline

########################################################################
# Invalid argument test
set i 0
try {
	string_to_array $qbf a -start 6 -append -chars -stop " "		# Invalid arg test
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

set i 0
try {
	# ?-join string? ?-start element? ?-stop element? ?-stopstr string? ?-nocase?
	array_to_string arrayname -stops
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

######################################################################################
puts "Line: "#__LINE__
array set a {
0 H
1 e
2 l
3 l
4 o
}
puts "Line: "#__LINE__
assert [array_to_string a] {eq $_ Hello} -v -line #__LINE__

######################################################################################
# SET
array unset a
puts "Line: "#__LINE__
set s "Hello 12345"
string_to_array $s a -chars -stopchar " "
#string_to_array $s a 6 -append -chars -stopchar " "
loop i 0 [array size a] {
	puts "$i:\t$a($i)"
}
puts "Line: "#__LINE__
assert $a(0) {eq $_ H} -v -line #__LINE__
assert $a(1) {eq $_ e} -v -line #__LINE__
assert $a(2) {eq $_ l} -v -line #__LINE__
assert $a(3) {eq $_ l} -v -line #__LINE__
assert $a(4) {eq $_ o} -v -line #__LINE__
assert [array size a] {== $_ 5} -v -line #__LINE__
puts "array size is now [array size a]"


######################################################################################
# APPEND
# FIX appends only x1 char
set s "world! 12345"
string_to_array $s a -append -chars -stopchar " "
puts "array size is now [array size a]"
#string_to_array $s a 6 -append -chars -stopchar " "
# start position?     ^
#array walk a
textcolor darkyellow
loop i 0 [array size a] {
	puts "$i:\t$a($i)"
}
textcolor
assert $a(0) {eq $_ H} -v -line #__LINE__
assert $a(4) {eq $_ o} -v -line #__LINE__
assert $a(5) {eq $_ w} -v -line #__LINE__
assert $a(6) {eq $_ o} -v -line #__LINE__
assert $a(7) {eq $_ r} -v -line #__LINE__
assert $a(8) {eq $_ l} -v -line #__LINE__
assert $a(9) {eq $_ d} -v -line #__LINE__
assert $a(10) {eq $_ !} -v -line #__LINE__
assert [array size a] {== $_ 11} -v -line #__LINE__


######################################################################################
# REASSEMBLE
set s [array_to_string a]
puts $s
assert $s {eq $_ {Helloworld!}} -v -line #__LINE__

######################################################################################
/*
#string_to_array $qbf 
array unset a -nocomplain
split $qbf " " -array a
#array walk a

assert [array size a] {== $_ 9} -v -line #__LINE__
assert $a(0) {eq $_ The} -v -line #__LINE__
assert $a(1) {eq $_ quick} -v -line #__LINE__
assert $a(7) {eq $_ lazy} -v -line #__LINE__
assert $a(8) {eq $_ dog} -v -line #__LINE__
# */

array unset a -nocomplain
unset i s -nocomplain

######################################################################################
#/* MAN example
# FIX array set should call array unset if nesc or reject set operation?

array unset a -nocomplain

array set a {
	0 H
	1 e
	2 l
	3 l
	4 o
	5 " "
}
loop i 0 [- [array size a] 1] {
	#puts "a($i) = $a($i)"
	puts "a($i) = [array item a $i]"
}
puts "\[array size a\] = [array size a]"
assert [array size a] {== $_ 6} -v -line #__LINE__

# FIX -stop is broken
# FIX -start offset is broken
# Can't have both -start and -append
set s "world! 12345"
# -append will override start

string_to_array $s a -start 6 -append -chars -stopchar " "		# Stop at space char

loop i 0 [- [array size a] 1] {
	#puts "a($i) = $a($i)"
	puts "a($i) = [array item a $i]"
}
puts "\[array size a\] = [array size a]"
assert [array size a] {== $_ 12} -v -line #__LINE__

assert $a(0) {eq $_ "H"} -v -line #__LINE__
assert $a(6) {eq $_ "w"} -v -line #__LINE__
assert $a(11) {eq $_ "!"} -v -line #__LINE__

array unset a -nocomplain
unset i s -nocomplain


#############################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #
