#################################################################################
# File: 			test_strto.tcl
# Purpose:			Tests for [strto] [strtor]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.27+
#
# Versions:
# 1.00 21/Feb/2018	First version
# 1.01 04/Oct/2018	Fixed re: changed to [split]
#################################################################################
# Notes:
# strtor returns parts of a string, split by one of N mask charsw
# FIX decide on null mask returns
# ADD full string match
#################################################################################
option expression off

set s "http://foo.com:8800"

textcolor yellow
puts "Test \[strto\] ..."
textcolor
assert [strto $s ""] {eq $_ $::s} -v -line #__LINE__
assert [strto $s {}] {eq $_ $::s} -v -line #__LINE__
assert [strto $s "" -right] {eq $_ $::s} -v -line #__LINE__
assert [strto $s {} -right] {eq $_ $::s} -v -line #__LINE__

assert [strto $s ":"] {eq $_ "http"} -v -line #__LINE__
assert [strto $s ":" -right] {eq $_ "//foo.com:8800"} -v -line #__LINE__

assert [strto $s "a:b"] {eq $_ "http"} -v -line #__LINE__
assert [strto $s "a:b" -right] {eq $_ "//foo.com:8800"} -v -line #__LINE__
assert [strto $s ":ab"] {eq $_ "http"} -v -line #__LINE__
assert [strto $s ":ab" -right] {eq $_ "//foo.com:8800"} -v -line #__LINE__
assert [strto $s "ab:"] {eq $_ "http"} -v -line #__LINE__
assert [strto $s "ab:" -right] {eq $_ "//foo.com:8800"} -v -line #__LINE__

assert [strto $s "q"] {eq $_ ""} -v -line #__LINE__			# No match

## -var option ##

assert [strto s "" -var] {eq $_ $::s} -v -line #__LINE__
assert [strto s {} -var] {eq $_ $::s} -v -line #__LINE__
assert [strto s "" -right -var] {eq $_ $::s} -v -line #__LINE__
assert [strto s {} -right -var] {eq $_ $::s} -v -line #__LINE__

assert [strto s ":" -var] {eq $_ "http"} -v -line #__LINE__
assert [strto s ":" -right -var] {eq $_ "//foo.com:8800"} -v -line #__LINE__

assert [strto s "a:b" -var] {eq $_ "http"} -v -line #__LINE__
assert [strto s "a:b" -right -var] {eq $_ "//foo.com:8800"} -v -line #__LINE__
assert [strto s ":ab" -var] {eq $_ "http"} -v -line #__LINE__
assert [strto s ":ab" -right -var] {eq $_ "//foo.com:8800"} -v -line #__LINE__
assert [strto s "ab:" -var] {eq $_ "http"} -v -line #__LINE__
assert [strto s "ab:" -right -var] {eq $_ "//foo.com:8800"} -v -line #__LINE__

assert [strto s "q" -var] {eq $_ ""} -v -line #__LINE__			# No match

#########################################################################################

newline
textcolor yellow
puts "Test \[strtor\] ..."
textcolor

assert [strtor $s ""] {eq $_ $::s} -v -line #__LINE__
assert [strtor $s {}] {eq $_ $::s} -v -line #__LINE__
assert [strtor $s "" -left] {eq $_ $::s} -v -line #__LINE__
assert [strtor $s {} -left] {eq $_ $::s} -v -line #__LINE__

assert [strtor $s ":"] {eq $_ "8800"} -v -line #__LINE__
assert [strtor $s ":" -left] {eq $_ "http://foo.com"} -v -line #__LINE__
assert [strtor $s "a:b"] {eq $_ "8800"} -v -line #__LINE__
assert [strtor $s "a:b" -left] {eq $_ "http://foo.com"} -v -line #__LINE__
assert [strtor $s ":ab"] {eq $_ "8800"} -v -line #__LINE__
assert [strtor $s ":ab" -left] {eq $_ "http://foo.com"} -v -line #__LINE__
assert [strtor $s "ab:"] {eq $_ "8800"} -v -line #__LINE__
assert [strtor $s "ab:" -left] {eq $_ "http://foo.com"} -v -line #__LINE__

assert [strtor $s "q"] {eq $_ ""} -v -line #__LINE__

## -var option ##

assert [strtor s "" -var] {eq $_ $::s} -v -line #__LINE__
assert [strtor s {} -var] {eq $_ $::s} -v -line #__LINE__
assert [strtor s "" -left -var] {eq $_ $::s} -v -line #__LINE__
assert [strtor s {} -left -var] {eq $_ $::s} -v -line #__LINE__

assert [strtor s ":" -var] {eq $_ "8800"} -v -line #__LINE__
assert [strtor s ":" -left -var] {eq $_ "http://foo.com"} -v -line #__LINE__
assert [strtor s "a:b" -var] {eq $_ "8800"} -v -line #__LINE__
assert [strtor s "a:b" -left -var] {eq $_ "http://foo.com"} -v -line #__LINE__
assert [strtor s ":ab" -var] {eq $_ "8800"} -v -line #__LINE__
assert [strtor s ":ab" -left -var] {eq $_ "http://foo.com"} -v -line #__LINE__
assert [strtor s "ab:" -var] {eq $_ "8800"} -v -line #__LINE__
assert [strtor s "ab:" -left -var] {eq $_ "http://foo.com"} -v -line #__LINE__

assert [strtor s "q" -var] {eq $_ ""} -v -line #__LINE__

##########################################################
# Proc emulation of rudimentary [strto] without options
##########################################################
newline
textcolor yellow
puts "Test proc \[strto\] ..."
textcolor

set s "http://foo.com:8800"
# DONE FIX changes require split to always have an arg even if "" or {}
proc p_strto {s q} {						# strto proc
	#puts "DEBUG: p_strto: s:$s q:'[split $q {}]'"
	foreach x [split $q {}] {
		set p [strstr $s $x]
		#puts "DEBUG: p_strto -> $s split -> x:$x p=$p"
		if {> $p 0} {
			-- p
			return [left $s $p]
		}
	}
	return ""
}

#puts [p_strto $s ":"]
#puts [p_strto $s "ba:c"]
assert [p_strto $s ":"] {eq $_ http} -v -line #__LINE__
assert [p_strto $s "ba:c"] {eq $_ http} -v -line #__LINE__

undef p_strto -nocomplain
unset s -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* Pass *"
textcolor

# EOF #