#################################################################################################
# File: 			tsst_struct.tcl
# Purpose:			Test cases for structs
# Author: 			M Shaw
# Language:			Ticol Tcl v1.23+
#
# Versions:
# 1.00 19/Aug/2016	First version
# 1.01 18/Apr/2017	Slight improvements
# 1.04 08/Jul/2017	Replaced [checkex] with [assert]
#################################################################################################
# Notes: 
#################################################################################################
option expression off

undef print_line -nocomplain
if {! [defined print_line]} {
	proc print_line {l} {
		upvar line
		set line $l
		textcolor yellow
		puts "Line: $l"
		textcolor
	}
}
########################################################################################

textcolor yellow
puts "Testing \[struct\] [info script] ..."
textcolor
newline

set expression_state [option expression]
option expression off
set echostate [option echo]
option echo off
unset s -nocomplain					# Ensure we can work with s

# Empty struct case
textcolor darkyellow
puts "wrong args: (ignore error message)"
textcolor yellow
catch {struct s}

textcolor darkyellow
puts "wrong args (ignore error message)"
textcolor yellow
if {! [catch {struct s} err]} {
	die "Fail at $line"
} else {}

textcolor
struct unset s -nocomplain

option echo on
# Alternate syntaxes
struct create s {{f1 10} {f2 20} {f3 30}}
struct unset s

# Simplest struct
struct s  {
	{s_a    4}
}

struct unset s

# Simplest struct
struct s  {
	s_a    23
	s_b    45
}
struct unset s

struct create s {
	{s_a    40}
	{s_b    100}
}
struct unset s

struct create s 256 {
	{s_a    40}
	{s_b    100}
}
struct unset s

struct s 256 {
	{s_a    40}
	{s_b    100}
}
struct unset s


struct s  {
	{s_a    4}
}

# Catch create with existing struct
textcolor darkyellow
puts "catch: (ignore error message)"
textcolor yellow
catch {
	struct s  {
		{s_a    4}
	}
}
textcolor
newline

# Struct unset
struct unset s
struct unset s -nocomplain

if {! [catch {					# Should not raise an error
	struct unset s
}] } {
	die "struct unset should err when unsetting non-existent struct"
}


# Test direct field dereference (text)
struct s {a 20 b 10}
struct set s.a "Hello world"

# Test different/flexible methods of addressing structs 
assert $s.a {eq $_ "Hello world"} -v -line #__LINE__
assert [struct set s.a] {eq $_ "Hello world"} -v -line #__LINE__
assert [set s.a] {eq $_ "Hello world"} -v -line #__LINE__
assert [-> s.a] {eq $_ "Hello world"} -v -line #__LINE__
assert [ofaddress s.a] {eq $_ "Hello world"} -v -line #__LINE__

# Indirection (as used in procs for args)
puts "Line: "#__LINE__
unset q -nocomplain
set q s
assert [-> ${q}.a] {eq $_ "Hello world"} -v -line #__LINE__
assert ${${q}.a} {eq $_ "Hello world"} -v -line #__LINE__
unset q -nocomplain

# Test upvar indirection via a proc
puts "Line: "#__LINE__
option echo on
undef foo -nocomplain
proc foo {st} {
	upvar $st
	#set q $st.a
	#assert $q {eq $_ "Hello world"} -v -line #__LINE__
	assert [set ${st}.a] {eq $_ "Hello world"} -v -line #__LINE__
	assert [-> ${st}.a] {eq $_ "Hello world"} -v -line #__LINE__
	#assert [ofaddress ${st}.a] {eq $_ "Hello world"} -v -line #__LINE__		# FAIL
	#assert ${${st}.a} {eq $_ "Hello world"} -v -line #__LINE__					# FAIL (not yet implemented)
}
foo s
undef foo -nocomplain


# # # # # # # # # # # # # # # # # # # # # # # # 
option echo on
puts "Line: "#__LINE__
struct set s.b 64
dump s
assert $s.b {eq $_ 64} -v -line #__LINE__
dump s
# FIX fails to set or report error
struct setb s.b 65 -value
puts "Line: "#__LINE__
assert $s.b {eq $_ A} -v -line #__LINE__

puts "Line: "#__LINE__
struct unset s -nocomplain

puts "Line: "#__LINE__
puts "Create struct using struct st \{st.a 10\}"
struct create st {st.a 10}
assert [type st] {eq $_ "struct"} -v -line #__LINE__
struct unset st -nocomplain
#unset fid -nocomplain

puts "Line: "#__LINE__
puts [string length $qbf]
struct unset s -nocomplain
struct unset t -nocomplain
struct s {a 43}
struct t {a 20}

struct set s.a $qbf
struct set t.a Bye
puts "\$s is $s"
puts "\$s.a is $s.a"
dump s
puts "\[addressofb s\]=[addressofb s] [inttohex [addressofb s]]"
# Runtime exception after line 48114 in C:\VC5\MyProjects\ticol\main.cpp
puts "Copied [struct copy t $s] byte(s)"
#struct copy t [addressofb s]	# Not what was expected
dump t
puts $t.a
assert $t.a {eq $_ "The quick brown fox "} -v -line #__LINE__
unset s t -nocomplain


# Set members

# leak tests
option echo $echostate
option expression $expression_state

unset echostate err expression_state -nocomplain

vars	

#################################################################################################
newline
textcolor green
puts "\[struct\] * Done *"
textcolor

# EOF #