#################################################################################################
# File:				test_struct_upvar_parent_update.tcl
# Author:			M Shaw
# Purpose:			Test updating struct parent from proc
# See also:			test_nested_deref_namespace.tcl
#
# Versions:
# 1.00 15/Jul/2018 First version
#################################################################################################
# Notes:
# DONE FIX Support: ${::${x}.a} # (scope prefix, poss with namespace as ${namespace::${x}.a})
# 	See: test_nested_deref_namespace.tcl
#################################################################################################
option expression off

undef bar -nocomplain
proc bar {x} {
	puts "\nEnter proc bar \{$x\}"
	upvar $x
	puts "Type of $x is: [type $x]"
	puts "In Bar: Global s.a is: '[struct set ::${x}.a]'"
	assert ${::${x}.a} {== $_ Hello} -v -line #__LINE__
	assert ${::${x}.a} {== $_ $::s.a} -v -line #__LINE__
	assert ${::${x}.b} {== $_ World} -v -line #__LINE__
	assert ${::${x}.b} {== $_ $::s.b} -v -line #__LINE__

	struct set ${x}.a Goodbye		# parent chain will be updated when proc exits scope?
	assert ${${x}.a} {eq $_ Goodbye} -v -line #__LINE__
	assert ${::${x}.a} {eq $_ Goodbye} -v -line #__LINE__
	
	struct set ${x}.b Now
	assert ${${x}.b} {eq $_ Now} -v -line #__LINE__
	assert ${::${x}.b} {eq $_ Now} -v -line #__LINE__
	puts "Exiting bar: Global ::s.a == '$::s.a'"
	newline
}

undef foo -nocomplain
proc foo {x} {
	upvar $x
	bar $x
	puts "In foo: Global ::s.a == '$::s.a'"
	assert ${::${x}.a} {== $_ $::s.a} -v -line #__LINE__
}


struct s {a 10 b 20}

struct set s.a Hello
struct set s.b World

puts "s.a->$s.a"
puts "s.b->$s.b"

foo s		# Should update the local proc var and then update the parent var on exit

puts "s.a->$s.a"
puts "s.b->$s.b"
newline

undef foo -nocomplain
undef bar -nocomplain
struct unset s -nocomplain


# Man file example for nested dereferences
struct s {a 10}
proc foo {x} {
	upvar $x            # When dereferenced, x gives the struct name
	puts "In foo: -> ${x}.a"    # This requires x to be braced
	assert ${x}.a {== $_ s.a} -v -line #__LINE__
	assert ${${x}.a} {== $_ Hello} -v -line #__LINE__
}

struct set s.a Hello
foo s

undef foo -nocomplain
struct unset s -nocomplain
vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #