######################################################################################
# File: 			test_sub.tcl
# Purpose:			Test [sub]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 16/Oct/2019	First version
######################################################################################
# Notes:
# Not much to test here
# [sub] can return a value but we expect it to be discarded
# No arguments can be passed to a [sub]
######################################################################################

textcolor yellow
puts "Testing \[sub\] ... @"#__LINE__
textcolor
newline

######################################################################################
set i 0
sub foo {
	textcolor darkyellow
	puts "Enter foo"
	textcolor
	set ::i 1
	return 23
}

foo
assert $i {== $_ 1} -v -line #__LINE__
assert [foo] {== $_ 23} -v -line #__LINE__
assert [gosub foo] {== $_ 23} -v -line #__LINE__
set i 0
gosub foo
assert $i {== $_ 1} -v -line #__LINE__
newline

######################################################################################
textcolor yellow
puts "Testing \[sub\] with \[goto_block\] ... @"#__LINE__
textcolor
newline

puts "Line: "#__LINE__
assert [info exists this] {== $_ 0} -v -line #__LINE__

goto_block {			# Blocks are run in sequence unless avoided via a jump
	init {
		textcolor magenta
		puts "In label init at line: "#__LINE__
		textcolor
		set j 0
		gosub foo							# Call sub [foo]
		set j 1								# Check we return from foo
	}
	gosub foo								# [goto] or [gosub] is allowed outside block label
	goto end

	foo {						# Here, 'foo' is a label
		textcolor magenta
		puts "In label foo at line: "#__LINE__
		textcolor
		puts "Called label this:'$this' at line: "#__LINE__
		set i 0
		set r [gosub foo]				# But we can still call sub [foo] from within a goto block
		puts "Line: "#__LINE__
		assert $i {== $_ 1} -v -line #__LINE__
		assert $r {== $_ 23} -v -line #__LINE__
	}
	end {
	}
}

assert $j {== $_ 1} -v -line #__LINE__
assert [info exists this] {== $_ 1} -v -line #__LINE__
puts "This is '$this' @"#__LINE__
assert $this {ne $_ ""} -v -line #__LINE__
unset j r
newline

######################################################################################
textcolor yellow
puts "Testing \[sub\] with \[static\] ... @"#__LINE__
textcolor
newline

puts "Line: "#__LINE__
static label1::counter -unset -nocomplain
sub label1 {
	static counter
	++ counter
	puts "In label1. Counter is $counter"
}
gosub label1
gosub label1

assert $label1::counter {== $_ 2} -v -line #__LINE__

undef foo label1 -nocomplain

/* # We can't do this with a sub but we can [upvar] any parent variable, regardless ...
sub foo {
	puts "foo: '$args'"
}
foo 23		# */

unset i 
static label1::counter -unset
unset this -nocomplain

######################################################################################
# Test 'blind' [upvar]
set a 23
proc foo {} {
	upvar a
	puts $a
	assert $a {== $_ 23} -v -line #__LINE__
}
gosub foo
undef foo -nocomplain
unset a 
newline

######################################################################################
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #