##################################################################
# File: 			test_switch.tcl
# Purpose:			Test file for [switch]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.19+
#
# Versions:
# 1.00 08/Aug/2016	FB problem
# 1.01 15/Sep/2016	Revised re: picolSetResult(i,i->result) bug
# 1.02 12/Nov/2015	Test quoted lvalues and rvalues in ladder
# 1.03 10/Jul/2017	Replaced [checkex] with [assert]
##################################################################
# Notes: 
# TODO: Add formatted layout equivalents
##################################################################
option echo on
option expression off

set i 0

textcolor yellow
puts "Test cases for \[switch\] ...\n"
textcolor

# No case met case
set i 0
catch {
	switch -- $i {
		1 {gpf}		# Ditto
	}
}

/* FIX
switch: No matching case met. Check syntax and include a 'default' case
Line: 58
Line 60: $x=='¦¦¦¦¦¦¦¦¦¦¦¦¦¦¦¦PS☼☻': eq $x 23
assert assertion failed: '$x' -> '¦¦¦¦¦¦¦¦¦¦¦¦¦¦¦¦PS☼☻' after line 60
*/

#assert [switch  1  default "return 23"] {eq $_ 23}
#puts "[switch -- 1  default {return 23}]"
puts "'[switch -- 1  default {23}]'"
assert [switch -- 1  default "return 23"] {eq $_ 23} -v -line #__LINE__		# PASS
catch {
	assert [switch -- 1  {default "return 23"}] {eq $_ 23} -v -line #__LINE__

} 		# INVALID SYNTAX - FAILS SAFE

assert [switch -- 1 {default {}}] {eq $_ ""} -v -line #__LINE__
assert [switch -glob 1 {default {}}] {eq $_ ""} -v -line #__LINE__

unset a -nocomplain
assert [switch -- a {a 1}] {eq $_ 1} -v -line #__LINE__
assert [switch -- a {b {return 1} default {return 2}}] {eq $_ 2} -v -line #__LINE__
assert [switch -- a {a {return 1} default {return 2}}] {eq $_ 1} -v -line #__LINE__
assert [switch -- b {a {return 1} default {return 2}}] {eq $_ 2} -v -line #__LINE__

# Nocase #
assert [switch -nocase A {b {return 1} default {return 2}}] {eq $_ 2} -v -line #__LINE__
assert [switch -nocase A {a {return 1} default {return 2}}] {eq $_ 1} -v -line #__LINE__
assert [switch -nocase hello {HEllO {return 1} default {return 2}}] {eq $_ 1} -v -line #__LINE__
assert [switch -- hello {HEllO {return 1} default {return 2}}] {eq $_ 2} -v -line #__LINE__

# Glob cases #
assert [switch -glob h???o {HEllO {return 1} default {return 2}}] {eq $_ 2} -v -line #__LINE__
assert [switch -glob hello {h???o {return 1} default {return 2}}] {eq $_ 1} -v -line #__LINE__
assert [switch -glob -nocase HELLO {h???o {return 1} default {return 2}}] {eq $_ 1} -v -line #__LINE__
assert [switch -glob -nocase FREDD {h???o {return 1} default {return 2}}] {eq $_ 2} -v -line #__LINE__

# Cascading args #
# switch -- $x 1 {puts "$this is one"}  2  -  3  -   4 {puts "$this is 2 3 or 4"}
puts "Line: "#__LINE__
assert [switch -- 1 1 {return 1}  2  -  3  -   4 {return 4}] {eq $_ 1} -v -line #__LINE__
assert [switch -- 2 1 {return 1}  2  -  3  -   4 {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- 3 1 {return 1}  2  -  3  -   4 {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- 4 1 {return 1}  2  -  3  -   4 {return 4}] {eq $_ 4} -v -line #__LINE__


# Unquoted string args (rolled up)
puts "Line: "#__LINE__
assert [switch -- one {
	one {return 1}
	two  -  
	three -  
	four {return 4}
}] {eq $_ 1} -v -line #__LINE__

# Unrolled switches...

# Both unquoted
puts "Line: "#__LINE__
assert [switch -- one one {return 1}  two  -  three  -   four {return 4}] {eq $_ 1} -v -line #__LINE__
assert [switch -- two one {return 1}  two  -  three  -   four {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- three one {return 1}  two  -  three  -   four {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- four one {return 1}  two  -  three  -   four {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- nix one {return 1}  two  -  three  -   four {return 4} default {return default}] {eq $_ default} -v -line #__LINE__

# Quoted args A
puts "Line: "#__LINE__
assert [switch -- "one" one {return 1}  two  -  three  -   four {return 4}] {eq $_ 1} -v -line #__LINE__
assert [switch -- "two" one {return 1}  two  -  three  -   four {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- "three" one {return 1}  two  -  three  -   four {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- "four" one {return 1}  two  -  three  -   four {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- "nix" one {return 1}  two  -  three  -   four {return 4} default {return default}] {eq $_ default} -v -line #__LINE__

# Quoted args B
puts "Line: "#__LINE__
assert [switch -- "one" "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 1} -v -line #__LINE__
assert [switch -- "two" "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- "three" "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- "four" "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- "nix" "one" {return 1}  "two"  -  "three"  -   "four" {return 4} "default" {return "default"}] {eq $_ "default"} -v -line #__LINE__

# Quoted args C
puts "Line: "#__LINE__
assert [switch -- one "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 1} -v -line #__LINE__
assert [switch -- two "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- three "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- four "one" {return 1}  "two"  -  "three"  -   "four" {return 4}] {eq $_ 4} -v -line #__LINE__
assert [switch -- nix "one" {return 1}  "two"  -  "three"  -   "four" {return 4} "default" {return "default"}] {eq $_ "default"} -v -line #__LINE__

# Full house - no arg prefix #
puts "Line: "#__LINE__
# ? 
if {is_set i} {puts "var i is set OK"} else {puts "var i is NOT SET"}

#option breakpoint on
puts "Line: "#__LINE__
halt
for {set i 1} {< $i 5} {incr i} {
#switch -- [expr "rnd(1,4)"] 1 "puts One" 2 "puts Two" 3 "puts Three" default {puts "* Another number *"}
	assert [switch $i 1 {return 1} 2 {return 2} 3 {return 3} 4 {return 4} ] {== $_ $i} -v -line #__LINE__
}

# Check $this #
puts "Line: "#__LINE__
for {set i 1} {< $i 5} {incr i} {
#switch -- [expr "rnd(1,4)"] 1 "puts One" 2 "puts Two" 3 "puts Three" default {puts "* Another number *"}
	assert [switch $i 1 {return $this} 2 {return $this} 3 {return $this} 4 {return $this} ] {== $_ $i} -v -line #__LINE__
}

puts "Line: "#__LINE__
assert [switch -- hello {$this {return $this}}] {eq $_ hello} -v -line #__LINE__	# Args MUST be wrapped in braces

unset i this -nocomplain

vars

##########################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #