######################################################################################
# File: 			test_system.tcl
# Purpose:			Test system related commands
# Language:			Ticol Tcl v1.23+..v1.26+
#
# Versions:
# 1.00 23/Jul/2017	First version
# 1.01 20/Nov/2019	Added ROS and Win2k exclusion case
######################################################################################
# Notes:
# DONE Add drive list as list <a b c ...> (info disks)
######################################################################################
/*
getenv
pid
tid
sysfree
info
is_admin
win_error
mem_used
*/
option echo on
option expression off

assert [pid] {> $_ 0} -v -line #__LINE__		# Win2k may be lower than 1024
assert [tid] {> $_ 0} -v -line #__LINE__

set d [info disks]
puts "Found disks - '$d'"

# Win2k and ROS report error 0
puts "Running on Windows [lindex [info winver -list]]"
if {> [lindex [info winver -list] 4] 5.0} {			# Make this simpler [info winver -major]
	if {! [instr [array item env windir -nocase] "reactos" -nocase]} {
		# This test fails on ReactOS - error 0 is returned by 'Windows'
		for {set i [asc A]} {<= $i [asc Z]} {++ i} {			# For disks A to Z
			puts "Checking drive [chr $i]"
			if {! [instr $d [chr $i]]} {						# At the first disk not in the active list
				puts "Drive '[chr $i]:' does not exist. Good. Using it to raise exception"
				exec "[chr $i]:\nosuch.fil"
				set e [get_win_error]							# FAILS puts [win_error 18]
				puts "Windows error is: $e \[win_error\]='[win_error $e]'"
				assert $e {!= $_ 0} -v -line #__LINE__
				break
			}
		} # for #
		unset e i -nocomplain
	}
}
#assert [sysfree c:] {> $_ 0} -v -line #__LINE__	# requires sysalloc var

assert [getenv comSPEC] {ne $_ ""} -v -line #__LINE__
assert [mem_used] {> $_ 0} -v -line #__LINE__

setenv ticol 19680408
assert [getenv ticol] {== $_ 19680408} -v -line #__LINE__
setenv ticol {}
assert [getenv ticol] {== $_ ""} -v -line #__LINE__

unset d -nocomplain

 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #