#####################################################################################################
# File:				test_tcx_encrypt.tcl
# Purpose:			Loop testing of different TCX encryption methods with ticol.exe /c
# Author:			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 10/Aug/2019	First version
# 1.01 25/Nov/2019	Minor tweaks and added to tests.tcl
#####################################################################################################
# Notes:
# Tests [lib tcx] with each encyption type
# Simultaneously 'tests ticol.exe /c' functionality
#####################################################################################################

option autoexec on

set limit 3
set mem_min 0
set mem_max 0
set start_mem [mem_used -ws]
set _script_name [unescape [info script]]

textcolor yellow
puts "Looping from 1 to $limit and encrypting ret_pi.tcl using different methods ..."
puts "We will call the encrypted TCX using \[source\] and validate the result of each method"
textcolor
newline

loop i 1 $limit {
	puts "Loop $i of $limit ..."
	if {is_mod $i 10} {
		set m_used [- [mem_used -ws] $start_mem]
		if {== $mem_min 0} {
			set mem_min $m_used
		} else {
			set mem_min [min $mem_min $m_used]
		}
		set mem_max [max $mem_max $m_used]
		console set_title "$_script_name - (Test loop [comma $i] of [comma $limit]) - Used: [comma $m_used] Kb"
	}
	
	#########################################################
	# Encrypt with MAC address
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c:mac /na /q /crlf
	textcolor [+ 1 $i]
	set r [source ret_pi.tcx]
	textcolor
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	# Encrypt with USER name
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c:user /na /q /crlf
	set r [source ret_pi.tcx]
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	# Encrypt with Ticol VERSION
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c:version /na /q /crlf
	set r [source ret_pi.tcx]
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	# Encrypt with WINVER
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c:winver /na /q /crlf
	set r [source ret_pi.tcx]
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	# Encrypt with HOST name
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c:host /na /q /crlf
	set r [source ret_pi.tcx]
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	# Encrypt with PASSWORD
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c:password /pw:foobar /na /q /crlf
	set r [source ret_pi.tcx -password foobar]
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	# Encrypt with DEFAULT
	unlink ret_pi.tcx
	exec ticol.exe ret_pi.tcl /c /na /q /crlf
	set r [source ret_pi.tcx]
	assert $r {== $_ 3.142857142857143} -line #__LINE__
	#########################################################
	newline
}

unset _script_name i limit mem_max mem_min start_mem r -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #