################################################################################
# File:			test_time.tcl
# Author:		M Shaw
# Purpose:		Tests [time]
# Language:		Ticol Tcl 1.26+
#
# Versions:
# 11/Nov/2019	First version
# 22/Nov/2019	Increase [time iterator] complexity
################################################################################
# Notes:
################################################################################
# time [time ?-ctime? ?-noseconds? ?-separator char?] # Get system time
# time [time double ?-ctime? ?-separator char?]       # Time from a double
# double [time hh:mm?:ss?]                            # Double from time
# integer [time {expression} ?iterations? ?-info | -quiet?] # Script timer
################################################################################

set i 0
try {
	assert [time -]
} catch {
	set i 1
}
assert $i -v -line #__LINE__

assert [time 0:0:0] {== $_ 0} -v -line #__LINE__
assert [time 23:59:0] {== [round $_ 5] 0.99931} -v -line #__LINE__
#assert [time 24:0:0] {== $_ 0.99931} -v -line #__LINE__		# FIX exits
assert [time [time 0:0]] {eq $_ {00:00:00}} -v -line #__LINE__		# Round-trip
assert [time [time 23:00]] {eq $_ {23:00:00}} -v -line #__LINE__		# Round-trip
assert [time .583333] {eq $_ {14:00:00}} -v -line #__LINE__	
assert [date 1970/01/01 00:00:00] {== $_ 25569} -v -line #__LINE__	
assert [time [date 1970/01/01 00:00:00]] {eq $_ 00:00:00} -v -line #__LINE__	
assert [time [date 1970/01/01 00:00:00] -ctime] {== $_ 0} -v -line #__LINE__	
set i 0
try {
	puts [time [date 1968/04/08 23:50:01] -ctime] -ctime]
} catch {
	set i 1
}
assert $i -v -line #__LINE__
assert [time [date 1998/01/30 23:30:00] {== $_ 0} -v -line #__LINE__	

################################################################################
# [time iterator]
set i [time {* [/ [* [/ [* $pi $pi] $pi] $pi] $pi] $pi} 500]		# ms/iter can vary a lot depending on host CPU
puts $i
assert $i {> $_ 0} -v -line #__LINE__	

unset i

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #