###################################################################################
# File:				test_trunc.tcl (Unit test)
# Author:			M Shaw
# Purpose:			Testing [trunc] (double forced truncation)
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 03/Feb/2022	First version
###################################################################################
# Notes:
# Unlike [round], [trunc] simply truncates at the given precision with no rounding
###################################################################################
option expression off

textcolor yellow
puts "Testing \[trunc\] ..."
textcolor
newline

# 3.(14159265358979323 = 17)
textcolor magenta
puts "Truncating pi ($pi) len [string length $pi]"
textcolor
newline

puts "Line: "#__LINE__
assert [trunc $pi 0] {== $_ 3} -v -line #__LINE__
assert [trunc $pi 1] {== $_ 3.1} -v -line #__LINE__
assert [trunc $pi 2] {== $_ 3.14} -v -line #__LINE__
assert [trunc $pi 3] {== $_ 3.141} -v -line #__LINE__
assert [trunc $pi 4] {== $_ 3.1415} -v -line #__LINE__
#puts "len 17=[string length [trunc $pi 17]]"
assert [trunc $pi 17] {== $_ $pi} -v -line #__LINE__
#puts "len 18=[string length [trunc $pi 18]]"
assert [trunc $pi 18] {== $_ $pi} -v -line #__LINE__
assert [trunc $pi 19] {== $_ $pi} -v -line #__LINE__
assert [trunc $pi 100] {== $_ $pi} -v -line #__LINE__
assert [trunc $pi 1000] {== $_ $pi} -v -line #__LINE__
newline

# Test integer pass through
puts "Line: "#__LINE__
set meg 1073741824		# One megabyte
assert [trunc $meg -1] {== $_ 1073741824} -v -line #__LINE__
assert [trunc $meg 0] {== $_ 1073741824} -v -line #__LINE__
assert [trunc $meg 1] {== $_ 1073741824} -v -line #__LINE__
assert [trunc $meg 2] {== $_ 1073741824} -v -line #__LINE__
assert [trunc $meg 10] {== $_ 1073741824} -v -line #__LINE__
assert [trunc $meg 1000] {== $_ 1073741824} -v -line #__LINE__
newline

# Test integer pass through
puts "Line: "#__LINE__
assert [trunc 0 -1] {== $_ 0} -v -line #__LINE__
assert [trunc 0 0] {== $_ 0} -v -line #__LINE__
assert [trunc 0 1] {== $_ 0} -v -line #__LINE__
assert [trunc 0 2] {== $_ 0} -v -line #__LINE__
assert [trunc 0 10] {== $_ 0} -v -line #__LINE__
assert [trunc 0 1000] {== $_ 0} -v -line #__LINE__
newline

###################################################################################
# Test NULL pass thru
puts "Line: "#__LINE__
set i 0
try {
	assert [trunc {} 0] {== $_ 0} -v -line #__LINE__
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__

# Test NULL pass thru
puts "Line: "#__LINE__
set i 0
try {
	assert [trunc "" 0] {== $_ 0} -v -line #__LINE__
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__


# Test string pass thru
puts "Line: "#__LINE__
set i 0
try {
	assert [trunc $qbf 0] {== $_ 0} -v -line #__LINE__
} catch {
	set i 1
}
assert $i {== $_ 1} -v -line #__LINE__


###################################################################################
unset meg -nocomplain
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #