###########################################################################################
# File:				test_try.tcl
# Author:			M Shaw
# Purpose:			Tests for [try ... catch] unit test
#
# Versions:
# 1.00 27/Jun/2016	First version
# 1.01 10/Jul/2017	Replaced [checkex] with [assert]
# 1.02 07/Jan/2022	replace [eval nosuchfile] with [source nosuchfile]
# 1.03 18/Dec/2022	Undef [on_error]
###########################################################################################
# Notes:
###########################################################################################
option echo on
undef on_error -nocomplain								# We don't want to inherit this from anywhere

set x 0
try {/ 1 0} catch {set x 1}								# Division by zero
assert $x {== $_ 1} -v -line #__LINE__

set x 0
try {open nosuchfile r} catch {set x 1}
assert $x {== $_ 1} -v -line #__LINE__

set x 0
try {source nosuchfile} catch {set x 1}
assert $x {== $_ 1} -v -line #__LINE__

# FIX Dec 2022. Not setting errorMsg
try {
	/ 10 0												# Division by zero
} catch {
	textcolor magenta
	puts "Script caught: '$errorMsg'"
	textcolor
	assert $errorMsg {ne $_ ""} -v -line #__LINE__
}

try {
	open nosuchfile r									# Open bad file
} catch {
	textcolor magenta
	puts "Script caught: '$::errorMsg'"
	textcolor
	assert $::errorMsg {ne $_ ""} -v -line #__LINE__
}

if {[catch  {try {} fred {}} var]}  {					# catch try with bad try/catch syntax
	textcolor magenta
	puts $var
	textcolor
	assert $var {ne $_ ""} -v -line #__LINE__
}

try {
	puts 1 2 3 4 5 									# Wrong arg count error
} catch {
	puts "Line: "#__LINE__
	textcolor magenta
	puts "Script caught: '$::errorMsg'"
	textcolor
	assert $::errorMsg {ne $_ ""} -v -line #__LINE__
}

unset var x -nocomplain

vars

###########################################################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #