##############################################################################
# File: 			test_type.tcl
# Purpose:			Test file [type]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/May/2017	First version
# 1.01 10/Jul/2017	Replaced [checkex] with [assert]
# 1.02 20/Sep/2019	Added extra tests and sci tests
# 1.03 23/Oct/2019	Added more tests
# 1.04 05/Jan/2022	Add pre-run unsets
##############################################################################
# Notes: 
##############################################################################
option expression off

unset foo -const -nocomplain
stack unset st -nocomplain
struct unset str -nocomplain
array unset a -nocomplain
unset d f i -nocomplain

const foo Hello
stack create st 200
stack push st "hello"
struct str {a 10}
set a(0) Hello
set s "Hello world"
set i 1234567890
set d 3.142
set f $pi
# nosuch is not set

# Var types
assert [type foo] {eq $_ "const"} -v -line #__LINE__
assert [type env] {eq $_ "const array"} -v -line #__LINE__
assert [type st] {eq $_ stack} -v -line #__LINE__
assert [type str] {eq $_ struct} -v -line #__LINE__
assert [type a] {eq $_ array} -v -line #__LINE__
assert [type a(0)] {eq $_ string} -v -line #__LINE__
assert [type s] {eq $_ string} -v -line #__LINE__
assert [type i] {eq $_ integer} -v -line #__LINE__
assert [type d] {eq $_ float} -v -line #__LINE__
assert [type nosuch] {eq $_ string} -v -line #__LINE__			# Is presented as a literal

# Numeric types - Decimal
assert [type 0] {eq $_ "integer"} -v -line #__LINE__
assert [type -0] {eq $_ "integer"} -v -line #__LINE__
assert [type +0] {eq $_ "integer"} -v -line #__LINE__
assert [type 0.0] {eq $_ "float"} -v -line #__LINE__
assert [type -0.0] {eq $_ "float"} -v -line #__LINE__
assert [type +0.0] {eq $_ "float"} -v -line #__LINE__
assert [type 1] {eq $_ "integer"} -v -line #__LINE__
assert [type -1] {eq $_ "integer"} -v -line #__LINE__
assert [type +1] {eq $_ "integer"} -v -line #__LINE__
assert [type "1"] {eq $_ "integer"} -v -line #__LINE__
assert [type "-1"] {eq $_ "integer"} -v -line #__LINE__
assert [type "+1"] {eq $_ "integer"} -v -line #__LINE__
assert [type .0] {eq $_ "float"} -v -line #__LINE__
assert [type 1.0] {eq $_ "float"} -v -line #__LINE__
assert [type -1.0] {eq $_ "float"} -v -line #__LINE__
assert [type +1.0] {eq $_ "float"} -v -line #__LINE__
assert [type "1.0"] {eq $_ "float"} -v -line #__LINE__
assert [type 1.234] {eq $_ "float"} -v -line #__LINE__
assert [type -1.234] {eq $_ "float"} -v -line #__LINE__
assert [type +1.234] {eq $_ "float"} -v -line #__LINE__
assert [type "1.234"] {eq $_ "float"} -v -line #__LINE__
assert [type "-1.234"] {eq $_ "float"} -v -line #__LINE__
assert [type "+1.234"] {eq $_ "float"} -v -line #__LINE__
assert [type 12345] {eq $_ "integer"} -v -line #__LINE__
assert [type -12345] {eq $_ "integer"} -v -line #__LINE__
assert [type +12345] {eq $_ "integer"} -v -line #__LINE__

# Note: Will default to int64 as a maxima even if the number is outside the range of __int64
# We may change this behaviour in future and may consider limited [big] overflow use or implement 128 bit ints
assert [type 123458388288423409823409283409283409283409823408238] {eq $_ "int64"} -v -line #__LINE__
assert [type -123458388288423409823409283409283409283409823408238] {eq $_ "int64"} -v -line #__LINE__
assert [type +123458388288423409823409283409283409283409823408238] {eq $_ "int64"} -v -line #__LINE__

# Numeric types - Non-Decimal
# May not be signed
assert [type "0xabc"] {eq $_ "hexint"} -v -line #__LINE__
#assert [type "+0xabc"] {eq $_ "hexint"} -v -line #__LINE__
#assert [type "-0xabc"] {eq $_ "hexint"} -v -line #__LINE__
assert [type 0xabc] {eq $_ "integer"} -v -line #__LINE__		# Note unquoted will be transcribed to int by the MPP
assert [type abcd] {eq $_ "string"} -v -line #__LINE__
assert [type 0o123] {eq $_ "integer"} -v -line #__LINE__		# Note unquoted will be transcribed to int by the MPP
assert [type "0o123"] {eq $_ "octal"} -v -line #__LINE__
assert [type 0b10101] {eq $_ "integer"} -v -line #__LINE__		# Note unquoted will be transcribed to int by the MPP
assert [type "0b10101"] {eq $_ "binary"} -v -line #__LINE__

# Scientific format numbers
assert [type "1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "2e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "234e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-234e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+234e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "2345e678"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2345e678"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2345e678"] {eq $_ "sci"} -v -line #__LINE__

assert [type "1.1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-1.1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+1.1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "2.3e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2.3e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2.3e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "2.34e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2.34e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2.34e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "23.45e678"] {eq $_ "sci"} -v -line #__LINE__

assert [type "+1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2e-34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+234e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2345e678"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-1e1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2e34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-234e5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2345e678"] {eq $_ "sci"} -v -line #__LINE__

assert [type "+1e-1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2e-34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+234e-5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2345e-678"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-1e-1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2e3-4"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-234e-5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2345e-678"] {eq $_ "sci"} -v -line #__LINE__

assert [type "+1e+1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2e+34"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+234e+5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "+2345e+678"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-1e+1"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2e3+4"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-234e+5"] {eq $_ "sci"} -v -line #__LINE__
assert [type "-2345e+678"] {eq $_ "sci"} -v -line #__LINE__

# Error cases... (invalid chars)
assert [type "0xabcPQR"] {eq $_ "string"} -v -line #__LINE__
assert [type "0b10ABC"] {eq $_ "string"} -v -line #__LINE__
assert [type "0o1909"] {eq $_ "string"} -v -line #__LINE__

# String...
assert [type hello] {eq $_ "string"} -v -line #__LINE__
assert [type "hello"] {eq $_ "string"} -v -line #__LINE__
assert [type "hello world"] {eq $_ "string"} -v -line #__LINE__
assert [type {hello world}] {eq $_ "string"} -v -line #__LINE__

# Empty...
assert [type {}] {eq $_ "empty"} -v -line #__LINE__
assert [type ""] {eq $_ "empty"} -v -line #__LINE__

# Don't mistake numeric prefixes...
assert [type "+"] {eq $_ "string"} -v -line #__LINE__

#puts [getcwd]
#puts "\[type \"-\"\]=[type "-"]"

assert [type "-"] {eq $_ "string"} -v -line #__LINE__
assert [type "."] {eq $_ "string"} -v -line #__LINE__

# Variant. We can't currently declare a variant using [set] and we detect no type info
lib ticol_variant
set v [vsplit $qbf]
puts [type v]
assert [type v] {eq $_ "integer"} -v -line #__LINE__
lib ticol_variant -unload

stack unset st
struct unset str
array unset a
unset d s i f -nocomplain
unset v -nocomplain

##############################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #