###################################################################################
# File: 			test_undef_proc_in_proc.tcl
# Purpose:			Test for code to block [undef]-ing current or parent proc
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Jun/2018	First version
###################################################################################
# Notes:
# Is a member of tests.tcl
# [rename proc] is blocked within a proc or parent chain
# [undef proc] is blocked within a proc and its parent chain
# Works well: uses callframe->parent->active_proc_name
###################################################################################
# Note: would hang originally as undef of current proc was allowed
###################################################################################
# FIX check we can have a static proc var for  proc called "my proc" and can reference the var with a space in it!
###################################################################################

puts "Line: "#__LINE__
textcolor yellow
puts "Enter: test_undef_proc_in_proc.tcl"
textcolor
newline

puts "Line: "#__LINE__
puts "Checking options ..."
option expression off
option echo on
option breakpoint off
option stacktrace off
option 

puts "Line: "#__LINE__
vars
newline

puts "Line: "#__LINE__
puts "Known procs: '[procs]'"
puts "Commands: '[commands s*]"

puts "Line: "#__LINE__
if {! [defined static]} {
	textcolor red
	die "\[static\] has been undefined somewhere, somehow!"
} # */

puts "Line: "#__LINE__
undef foo -nocomplain
proc foo {} {
	static a 0
	vars
	textcolor yellow
	puts "Enter foo with a==$a"
	textcolor
	if {> $a 0} {
		try {
			undef foo			# Try to commit harakiri :)
		} catch {
			puts "Failed to \[undef\] foo while in proc foo"
		}
	}
	++ a
	puts "Exit foo with a==$a"
}

foo
foo

assert [defined foo] {== $_ $::true} -v -line #__LINE__
undef foo -nocomplain
assert [defined foo] {== $_ $::false} -v -line #__LINE__
static foo::a -unset

###################################################################
newline
puts "Line: "#__LINE__
# Attempt to have child proc (bar) undef it's parent (foo)
undef bar -nocomplain
proc bar {} {
	static a 0
	textcolor yellow
	puts "\nEnter [info procname] with a==$a"
	textcolor
	if {> $a 0} {
		#puts "undef foo=[undef foo]"			# Foo is not visible
		try {									# Try to commit harakiri
			puts "undef foo=[undef foo]"		# Foo is not visible
		} catch {
			textcolor red
			puts "catch: Failed to \[undef\] foo while in child \[bar\]"
			textcolor
		}
		puts "Known procs list: [procs *]"
	}
	++ a
	puts "Exit [info procname] with a==$a"
}

undef foo -nocomplain
proc foo {} {
	bar
	puts "Exit bar"
}

foo
foo

assert [defined foo] {== $_ $::true} -v -line #__LINE__
assert [defined bar] {== $_ $::true} -v -line #__LINE__
undef foo bar -nocomplain
assert [defined foo] {== $_ $::false} -v -line #__LINE__
assert [defined bar] {== $_ $::false} -v -line #__LINE__

static bar::a -unset

###################################################################
newline
puts "Line: "#__LINE__

undef mar -nocomplain
proc mar {} {
	static a 0
	textcolor yellow
	puts "\nEnter [info procname] with a==$a"
	textcolor
	if {> $a 0} {

		try {									# Try to commit harakiri
			puts "undef foo=[undef foo]"		# Foo is not visible
		} catch {
			textcolor red
			puts "catch: Failed to \[undef\] foo while in grandchild \[mar\]"
			textcolor
		}
		try {									# Try to commit harakiri
			puts "undef bar=[undef bar]"		# Foo is not visible
		} catch {
			textcolor red
			puts "catch: Failed to \[undef\] bar while in child \[mar\]"
			textcolor
		}
		puts "Known procs list: [procs *]"
	}
	++ a
	puts "Exit [info procname] with a==$a"
}

undef bar -nocomplain
proc bar {} {
	mar
	puts "Exit bar"
}

undef foo -nocomplain
proc foo {} {
	bar
	puts "Exit foo"
}

foo
foo

assert [defined foo] {== $_ $::true} -v -line #__LINE__
assert [defined bar] {== $_ $::true} -v -line #__LINE__
assert [defined mar] {== $_ $::true} -v -line #__LINE__
undef foo bar mar -nocomplain
assert [defined foo] {== $_ $::false} -v -line #__LINE__
assert [defined bar] {== $_ $::false} -v -line #__LINE__
assert [defined mar] {== $_ $::false} -v -line #__LINE__

static mar::a -unset

###################################################################
newline
puts "Line: "#__LINE__

undef foo -nocomplain
proc foo {} {

}

puts "Line: "#__LINE__
assert [defined foo] {== $_ $::true} -v -line #__LINE__
undef foo
assert [defined foo] {== $_ $::false} -v -line #__LINE__


# Test glob undef 
puts "Line: "#__LINE__
undef -glob ??? -proc		# Known == bar foo moo one par shr two
undef bar foo moo "my proc" -nocomplain
puts "Line: "#__LINE__

puts "Line: "#__LINE__
undef foo -nocomplain
undef moo -nocomplain
undef bar -nocomplain
undef "my proc" -nocomplain

puts "Line: "#__LINE__
proc foo {} {}
proc moo {} {}
proc bar {} {}
proc "my proc" {} {
	try {									# Try to commit harakiri
		puts "undef foo=[undef -glob my* -proc]"
	} catch {
		puts "catch: Failed to \[undef -glob\] \"my proc\" while in active \[my proc\]"
	}
}

"my proc"					# Call my proc

puts "Line: "#__LINE__
set r [undef -glob ??? -proc]
assert $r {eq $_ "bar foo moo"} -v -line #__LINE__

set r [undef -glob my* -proc]
assert $r {eq $_ "{my proc}"} -v -line #__LINE__

unset r -nocomplain
newline

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# DONE Try fixing a potentially crafty workaround
# If a proc calls rename on itself we should update the trace var to block undef
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
puts "Line: "#__LINE__
set i 0
proc foo {} {
	puts "ENTER proc \[foo\] - foo active? [is_active foo]"
	try {
		rename foo moo
	} catch {
		textcolor red
		puts "\[rename\] ? - Whoops! Can't do that within a proc!"
		textcolor
	}
	puts "foo active? [is_active foo]"
	assert [is_active foo] {== $_ 1} -v -line #__LINE__
	puts "moo active? [is_active moo]"
	assert [is_active moo] {== $_ 0} -v -line #__LINE__
	puts "A Proclist is: '[procs]'"
	try {
		undef moo
	} catch {
		textcolor red
		puts "\[undef\] ? - Whoops! Can't do that either!"
		textcolor
	}
	puts "B Proclist is: '[procs]'"
	#assert [procs moo] {ne $_ ""} -v -line #__LINE__		# FAILS
	textcolor darkyellow
	puts "Am I still here?"
	textcolor
	set ::i 1
}
foo
assert $i {== $_ 1} -v -line #__LINE__
unset i 
newline

vars
newline
###################################################################################

textcolor green
puts "* Done *"
textcolor

# EOF #