###################################################################################
# File:				test_unknown.tcl
# Author:			M Shaw
# Purpose:			Test file for [unknown] command
# Language:			Ticol Tcl v1.22+
#
# Versions:
# 1.00 16/Jan/2017	First version
# 1.01 10/Jul/2017	Replaced [checkex] with [assert]
# 1.02 04/Dec/2017	Added generic unknown calcs
###################################################################################
# Notes:
# FIX has reverted to REQUIRES ForInitAllowsExpression=FALSE
# Requires ForInitAllowsExpression=TRUE
# FIX detect ForInitAllowsExpression=TRUE
# We will assume any unknown command is some kind of expression we can handle
###################################################################################

textcolor yellow
puts "Testing \[unknown\] ..."
puts "Requires INI->ForInitAllowsExpression=TRUE for \[unknown\] to work"
puts "requires \[option autoexec off\]"
textcolor
newline

#set two 2
#puts [calc pow(220,$two)]

option autoexec off  # Required (is cancelled when script exits)

undef unknown -nocomplain
# proc unknown {args} {return [expr $args]}
proc unknown {args} {
	# 'args' is a special argument similar to the C++ elipsis ...
	#puts "unknown: '$args'"
	return [expr $args]
}

# Cannot upvar within the proc for expressions 
# as we don't know what will be in the expression given to us
# Thus must pass with scope
::x = 23
# FIX
#"'[upvar @0 x y]': y = 24; [puts $y]"
#checkex $x {== $x 24}
assert $x {== $_ 23} -v -line #__LINE__
newline
assert ["1+2"] {== $_ 3} -v -line #__LINE__
newline
assert [pow(2,20)] {== $_ 1048576} -v -line #__LINE__
newline

###################################################################################
# DOC See: ForInitAllowsExpression -> _for_init_allows_expr
# REQUIRES ForInitAllowsExpression=FALSE

textcolor yellow
puts "Testing for loop via \[unknown\] expression handler. Line: "#__LINE__
# This allows us to do c-like {i=x}
textcolor
option expression on

set sum 0
for {set i 0} {$i < 100} {++ i} {
#puts "Line "#__LINE__
	#puts "i=$i"
	::sum += $i			# Prefix is required
} 
puts "Sum is $sum"
assert $sum {== $_ 4950} -v -line #__LINE__

#undef unknown -nocomplain
unset i sum x -nocomplain
undef unknown -nocomplain

###################################################################################
vars
newline
textcolor green
puts "* test_unknown.tcl - Done *"
textcolor

# EOF #