#################################################################
# File:				test_unwrap.tcl
# Purpose:			Testing PROC [unwrap] to unwrap a list
# Author:			M Shaw
# Language:			Ticol Tcl v1.27+
#
# Versions:
# 1.00 20/Nov/2017	First version
# 1.01 20/Jan/2023	Updated
#################################################################
# Notes:
# The built-in [unwrap] simply removes outer braces
# The proc [unwrap_as_text] fully unwraps a nested Tcl list
# ADD asserts
#################################################################
#define DEBUG_INTERNAL

option expression off
option echo on

textcolor yellow
puts "Testing \[unwrap\]"
textcolor
newline

set l "{catalog {cd {{title {Empire Burlesque}} {artist {Bob Dylan}} \
{Company Columbia} {Price 10.90} {year 1985} }}}"

# Built-in [unwrap] command
assert [unwrap $l] {\
eq $_ "catalog {cd {{title {Empire Burlesque}} {artist {Bob Dylan}} \
{Company Columbia} {Price 10.90} {year 1985} }}"
} -v -line #__LINE__


/* ###################################################################
puts "Line: "#__LINE__
while {ne $l ""} {
	set l [unwrap $l]
	puts $l
}
################################################################### */

puts "Line: "#__LINE__
#ifndef DEBUG_INTERNAL
undef unwrap -nocomplain	# Undef any built-in
#endif
undef ldepth -nocomplain

proc ldepth {list} {
	if {! [is_list $list]} {
		return 0
	}
	set flatList [join $list]
	if {[string equal $flatList $list]} {
		return 1
	}
	return [expr {[ldepth $flatList] + 1}]
}

#################################################################

proc bar {l} {						# PASS Uses a static as a temporary accumulator. 
	static s
	#puts "ENTER bar: s '$s'"
	foreach x $l {
		if {> [ldepth $l] 2} {
			set s [bar $x]
		} else {
			append s "$x\t"
		}
	}
	return "$s\r\n"
}

#puts '[ldepth Hello]'
#puts '[ldepth "Hello world"]'

/*
proc moo {l} {						# PASS - Uses a local as a temporary accumulator. 
	static s						# Static will unwind back to 'catalog'
	foreach x $l {
		if {> [ldepth $x] 1} {
			set s [moo $x]
		} else {
			append s "$x\x11\xd9\r\n"
		}
	}
	return $s
}

textcolor white
puts "moo:\r\n'[moo $l]'"
#puts "bar: [bar $l]"
textcolor
vars
# puts $l
stop		# */

#################################################################
#ifdef DEBUG_INTERNAL
if {! [defined unwrap]} {
#endif
	proc unwrap {l} {
		puts "Enter unwrap"
		static s
		set s ""
		foreach x $l {
			if {> [ldepth $l] 2} {
				unwrap $x
			} else {
				#  append s "$x\t"		# Jan/2023
			}
		}
		textcolor darkyellow
		puts '$s'
		textcolor
		append s "\r\n"
		puts "'$::unwrap::s' '$s'"
		return $s
	}
#ifdef DEBUG_INTERNAL	
}
#endif
#################################################################

proc foo {l} {
	static s
	static level
	++ level
	foreach x $l {
		if {> [ldepth $l] 2} {
			set s [foo $x]
			#set s "$s\r\n"
		} elseif {== [ldepth $l] 1} {
			append s "$x\t" 
		} else {
			#set s "$s$x\t"		# OK
			append s "$x*\t" 
			#append s "$x\t" -return		# FAIL
		}
	}
	#puts "$level: '*$s'"
	return $s
/*	
	if {! [is_mod $level 2]} {
		append s "\r\n" -return
	}
	#set s "$s\r\n"
	#append s "\r\n" -return
	#return $s # */
}

/*
puts "Foo proc return: '[foo $l]'"
textcolor green
puts "Foo:s var        '$foo::s'"
textcolor
#stop	# */

/* #################################################################
proc unwrap {l} {
	set s {}
	foreach x $l {
		if {> [ldepth $l] 2} {
			unwrap $x
		} else {
			append s "$x\t"
		}
	}
	puts $s
} 
##################################################################  */

/* #################################################################
proc unwrap {l} {
	set s {}
	foreach x $l {
		if {> [ldepth $l] 2} {
			unwrap $x
		} else {
			append s "$x\t"
		}
	}
	if {<= [ldepth $l] 2} {
		set s [string trim $s "\t "]
		puts $s
	}
}
################################################################## */


unwrap $l
#ifndef DEBUG_INTERNAL
newline
vars u*
puts "'$unwrap::s'"
#endif
newline
#stop

proc unwrap_as_text {l} {
	global foo::s
	foreach x $l {
		if {> [ldepth $l] 2} {
			unwrap_as_text $x
		} else {								# FIX need to detect if last item
			append ::foo::s "$x\t"
		}
	}
	if {<= [ldepth $l] 2} {
		set ::foo::s [string trim $::foo::s "\t "]
		append ::foo::s "\r\n"
	}
}

unwrap_as_text $l	# -> foo::s
#vars
#puts "\$unwrap_as_text::s='$unwrap_as_text::s'"
textcolor cyan
puts $foo::s
/*
puts [tohex $foo::s]
puts [tohex "catalog
cd
title\tEmpire Burlesque
artist\tBob Dylan
Company\tColumbia
Price\t10.90
year\t1985
"] # */

assert $foo::s {\
eq $_ "catalog
cd
title\tEmpire Burlesque
artist\tBob Dylan
Company\tColumbia
Price\t10.90
year\t1985
"
} -v -line #__LINE__



########################################################################
textcolor
newline
vars
undef unwrap -nocomplain
undef ldepth -nocomplain
unset l -nocomplain
unset u* foo::* moo::*  -glob -const -static -nocomplain
static unwrap::s -unset -nocomplain
static foo::level -unset -nocomplain
static foo::s -unset -nocomplain
static moo::s -unset -nocomplain

#undef DEBUG_INTERNAL
#listdefs

########################################################################
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #