###################################################################################
# File: 			test_upvar.tcl
# Purpose:			Test [upvar]
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 28/Jun/2018	First version. Simple test
# 1.01 22/Sep/2019	Added upvar array element tests
###################################################################################
# Notes:
# See also: test_upvar_true.tcl
###################################################################################
/*
      +------> [parentvar]
      |        proc()
      ^        {
      |
      +-----------[local upvar]
      
               }
*/
option expression off
option echo on

textcolor yellow
puts "Testing \[upvar\] ..."
textcolor
newline

###################################################################################
# Test [upvar] with simple variables

puts "Line: "#__LINE__

undef foo -nocomplain

set a Hello
set b Dear
set c World

proc foo {} {
	try {
		upvar a
	} catch {
		puts "Could not create upvar foo-> a"
	}
	try {
		upvar b b
	} catch {
		puts "Could not create upvar foo-> b b"
	}
	try {
		upvar c d
	} catch {
		puts "Could not create upvar foo-> c d"
	}
	assert $::a {eq $_ Hello} -v -line #__LINE__
	assert $::b {eq $_ Dear} -v -line #__LINE__
	assert $::c {eq $_ World} -v -line #__LINE__
	assert $a {eq $_ Hello} -v -line #__LINE__
	assert $b {eq $_ Dear} -v -line #__LINE__
	assert $d {eq $_ World} -v -line #__LINE__
	set a Goodbye
	set b Cruel
	set d Interpreter
	assert $a {eq $_ Goodbye} -v -line #__LINE__
	assert $b {eq $_ Cruel} -v -line #__LINE__
	assert $d {eq $_ Interpreter} -v -line #__LINE__
}

foo

puts "root: a is '$a'"
puts "root: b is '$b'"
puts "root: c is '$c'"

assert $a {eq $_ Goodbye} -v -line #__LINE__
assert $b {eq $_ Cruel} -v -line #__LINE__
assert $c {eq $_ Interpreter} -v -line #__LINE__


undef foo -nocomplain
unset a b c -nocomplain

###################################################################################
# Test upvar array elements (exist and not-exist)
# If we pass an array element by name and omit a local var name then we have to 
# use [varname] to retrieve the new, local varname which will be created from the
# array element's parent name

puts "Line: "#__LINE__

proc foo {q} {
	puts "Enter foo '$q'"
	upvar $q								# DONE FIX Can't upvar array element a(0)
	vars
	puts "local foo->q '$q'"
	set [varname $q] Hello					# If we pass-in an array name we have to use [varname] to get the name prefix
}

set a(0) "a 0"
# We don't set a(1)
foo a(0)
puts "a(0) is now '$a(0)'"					# Element now contains "Hello"
assert $a(0) {eq $_ Hello} -v -line #__LINE__
foo a(1)									# Element not set on calling
puts "a(1) is now '$a(1)'"					# Element now contains "Hello"
assert $a(1) {eq $_ Hello} -v -line #__LINE__

array unset a -nocomplain

###################################################################################
# Call with undefined var as arg but no [upvar] command
# Parent var should not exist
# Same as A/S
# rename foo "";proc foo {x} {}; foo x;puts $x

undef foo -nocomplain
proc foo {x} {								# This will not set parent x
	vars -level 0
}
foo x
puts "x? [bool [info exists x]]"
assert [info exists x] {eq $_ 0} -v -line #__LINE__

###################################################################################
# Test upvar standard var (not-exist)

undef foo -nocomplain
proc foo {x} {
	textcolor yellow
	puts "\nVars before upvar call..."
	textcolor
	vars -level 0
	upvar $x y		# Successfully creates the global instance
	set y Hello
	textcolor yellow
	puts "\nVars after upvar call..."
	textcolor
	vars -level 0
}
foo x
puts "x is $x"
assert $x {eq $_ Hello} -v -line #__LINE__

unset x
undef foo -nocomplain

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #