###############################################################################################
# File:				test_upvar_update.tcl
# Author:			M Shaw
# Purpose:			Test file for upvar root var updates
# Language:			Ticol Tcl v1.21c+
#
# Versions:
# 1.00 15/Jan/2016	Migrated from tests.tcl
# 1.01 10/Jul/2017	Replaced [checkex] with [assert]
###############################################################################################
# Notes
###############################################################################################

option expression off
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# FIXED Could be in upvar call which sets up linkage or in proc exit callframe cleanup (DropCallFrame)

newline
textcolor yellow
puts "Test emulated namespace scoping and scoped upvar ..."
newline
textcolor

#FIX Child scope does not update parent var (if names differ?)
set a 54321				#
set scope::a 12345			# We will overwrite this using the proc
set aa 765
# aaa will be created 

proc scope::div {x y} {				# Example divide procedure with scoped name
	puts "Line: "#__LINE__
	upvar scope::a a				# scope:: is an arbitrary namespace prefix
	upvar a b						# Assume root
	upvar ::aa c					# Root aa as c
	puts "div-upvar: aa is '$c'"	# FIXED 15/Jan/2017 - Never links
	upvar aaa d						# Will create aaa in upvar space and alias as d in proc space
	set a "9876543210"
	set b 0123456789
	set c 123
	set d 456
	if {!= $y 0} {
		[/ $x $y]
	} else {
		return 0 -code error
	}
}

set r [scope::div 22 7.0]				# MUST be called for test to work
assert $r {== $_ 3.142857142857143} -v -line #__LINE__
assert $scope::a {== $_ 9876543210} -v -line #__LINE__	# FAIL if name different
assert $a {== $_  0123456789} -v -line #__LINE__			# PASS if name same
assert $aa {== $_  123} -v -line #__LINE__					# FIXED PASS if name same
assert $aaa {== $_  456} -v -line #__LINE__				# PASS if name same

undef scope::div -nocomplain
unset a aa aaa r -nocomplain
unset scope::a -nocomplain
assert [is_set scope::a] {== $_  $::false} -v -line #__LINE__

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

newline
textcolor green
puts "* PASS *"
textcolor

# EOF #