############################################################################################
# File: 			test_varchars.tcl
# Purpose:			Test valid chars in variable names
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 17/Oct/2019	First version
# 1.01 08/Jan/2023	Updated re: [set] now handling escaped '\ '
############################################################################################
# Notes:
# Standard Tcl allows any character to be used, Ticol Tcl allows a small subset of chars
############################################################################################
# Valid chars:
# a-z A-Z 0-9 " \t"
# _ . & <tab> and <space>
# Special meaning: : ( )
############################################################################################

# SPACE #
set { } 23
assert ${ } {== $_ 23} -v -line #__LINE__
unset { }

# TAB #
set {\t} 23
assert ${\t} {== $_ 23} -v -line #__LINE__
unset {\t}

# MIXED #
set { \t } 23
assert ${ \t } {== $_ 23} -v -line #__LINE__
unset { \t }

# a-z #
set abcdefghijklmnopqrstuvwxyz 23
assert $abcdefghijklmnopqrstuvwxyz {== $_ 23} -v -line #__LINE__
unset abc* -glob

# A-Z #
set ABCDEFGHIJKLMNOPQRSTUVWXYZ 23
assert $ABCDEFGHIJKLMNOPQRSTUVWXYZ {== $_ 23} -v -line #__LINE__
unset ABC* -glob

# 0-9 #
set 0123456789 23
assert $0123456789 {== $_ 23} -v -line #__LINE__
unset 0123456789

# Special chars #
set & 23
assert ${&} {== $_ 23} -v -line #__LINE__		# A/S also requires bracing
unset {&}

# Underscore #
set _ 23
assert $_ {== $. 23} -v -var . -line #__LINE__		# 
unset {_}

# Dot #
set . 23
assert $. {== $_ 23} -v -line #__LINE__		# 
unset .

# Single colon as varname (with scope) #
set ::: 23										# As global scope
assert ${:} {== $_ 23} -v -line #__LINE__		# 
set {:} 23										# Braced
assert ${:} {== $_ 23} -v -line #__LINE__		# 
unset {:}

puts "Line "#__LINE__
# FIX set {\{} 23								# Requires parity to ignore braces
set "\{" 23										# Will store as '{'
vars
#assert ${"\{"} {== $_ 23} -v -line #__LINE__		# PARSE ERROR
unset "{"										# 2023 Changed from \{ to { re: [set] now parsing '\ '

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #  # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #