#####################################################################################
# File: 			test_varnames.tcl
# Purpose:			Tests for var name charset handling
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 18/Sep/2019	First version
#####################################################################################
# Notes:
# Full numeric vars are interpreted specially in Ticol Tcl as numbers
# Alphanmeric varnames are allowed and with numeric prefix
# ADD tests for blocked chars. Enhance char blocking/handling
#####################################################################################

textcolor yellow
puts "Testing valid character range for variable names ...\n"
textcolor

#####################################################################################
# Alphanumeric var with leading numeric char
#####################################################################################
textcolor yellow
puts "\nAlphanumeric with leading numeric char ...\n"
textcolor
puts "Line: "#__LINE__
set 6a hello
puts $6a
assert [is_set 6a] {== $_ $::true} -v -line #__LINE__			# PASS
assert [set 6a] {eq $_ "hello"} -v -line #__LINE__				# PASS
assert $6a {eq $_ "hello"} -v -line #__LINE_	_				# PASS
assert [is_set 6a] {== $_ $::true} -v -line #__LINE__			# PASS
unset 6a
assert [is_set 6a] {== $_ $::false} -v -line #__LINE__			# PASS


#####################################################################################
# Alphanumeric var with leading alpha char
#####################################################################################
textcolor yellow
puts "\nAlphanumeric with leading alpha char ...\n"
textcolor
puts "Line: "#__LINE__
set a6 hello
puts $a6
assert [is_set a6] {== $_ $::true} -v -line #__LINE__			# PASS
assert [set a6] {eq $_ "hello"} -v -line #__LINE__				# PASS
assert $a6 {eq $_ "hello"} -v -line #__LINE_	_				# PASS
assert [is_set a6] {== $_ $::true} -v -line #__LINE__			# PASS
unset a6
assert [is_set a6] {== $_ $::false} -v -line #__LINE__			# PASS


#####################################################################################
# Var comprised of one or more spaces
# SPACE
#####################################################################################
textcolor yellow
puts "\nOne or more SPACE char as var name ...\n"
textcolor
puts "Line: "#__LINE__
set { } hello
assert [is_set { }] {== $_ $::true} -v -line #__LINE__			# PASS
assert [is_set [chr 32]] {== $_ $::true} -v -line #__LINE__		# PASS
assert [set { }] {eq $_ "hello"} -v -line #__LINE__				# PASS
assert ${ } {eq $_ "hello"} -v -line #__LINE__					# PASS
unset { }
assert [is_set { }] {== $_ $::false} -v -line #__LINE__			# PASS
vars

#####################################################################################
# Var name with embedded spaces
# SPACE
#####################################################################################
textcolor yellow
puts "\nOne or more embedded SPACE chars ...\n"
textcolor
puts "Line: "#__LINE__
set {my var name} hello
assert [is_set {my var name}] {== $_ $::true} -v -line #__LINE__			# PASS
assert [set {my var name}] {eq $_ "hello"} -v -line #__LINE__				# PASS
assert ${my var name} {eq $_ "hello"} -v -line #__LINE__					# PASS
unset {my var name}
assert [is_set {my var name}] {== $_ $::false} -v -line #__LINE__			# PASS
vars


#####################################################################################
# Var comprised of one or more tab
# TAB
#####################################################################################
textcolor yellow
puts "\nOne or more TAB char as var name ...\n"
textcolor
puts "Line: "#__LINE__
set {\t} hello
vars
assert [is_set {\t}] {== $_ $::true} -v -line #__LINE__			# PASS
assert [is_set [chr 9]] {== $_ $::true} -v -line #__LINE__		# PASS
assert [set {\t}] {eq $_ "hello"} -v -line #__LINE__			# PASS
#assert ${\t} {eq $_ "hello"} -v -line #__LINE__						# FAIL!
unset {\t}
assert [is_set {\t}] {== $_ $::false} -v -line #__LINE__		# PASS
vars

#####################################################################################
# Var name with embedded TAB
# TAB
#####################################################################################
textcolor yellow
puts "\nOne or more embedded TAB chars ...\n"
textcolor
puts "Line: "#__LINE__
set {my\tvar\tname} hello
assert [is_set {my\tvar\tname}] {== $_ $::true} -v -line #__LINE__			# PASS
assert [set {my\tvar\tname}] {eq $_ "hello"} -v -line #__LINE__				# PASS
vars
#(12123) eval: No variable reference to '{my     var     name}' at level 0
assert ${my\tvar\tname} {eq $_ "hello"} -v -line #__LINE__							# FAIL!
unset {my\tvar\tname}
assert [is_set {my\tvar\tname}] {== $_ $::false} -v -line #__LINE__			# PASS
vars


#####################################################################################
# Var with ampersand in name
# Commonly used as proc arg reference
# We match Activestate in requiring braces to resolve
#####################################################################################
textcolor yellow
puts "\nLeading & ampersand char ...\n"
textcolor
puts "Line: "#__LINE__
set &a hello												# PASS Creates the var ok
vars
puts "Line: "#__LINE__
puts [set &a]
assert [set &a] {eq $_ "hello"} -v -line #__LINE__			# PASS
puts "Line: "#__LINE__
puts $&a
assert $&a {eq $_ "$&a"} -v -line #__LINE__					# PASS - echo literal

# getBracedVar(
puts ${&a}			# Should equal hello					# PASS
assert ${&a} {eq $_ "hello"} -v -line #__LINE__				# PASS - echo literal

assert [is_set &a] {== $_ $::true} -v -line #__LINE__			# PASS
unset &a
assert [is_set &a] {== $_ $::false} -v -line #__LINE__			# PASS
vars

#####################################################################################
# Var with leading underscore
#####################################################################################
textcolor yellow
puts "\nLeading underscore char ...\n"
textcolor
puts "Line: "#__LINE__
set _a hello												# PASS Creates the var ok
vars
puts "Line: "#__LINE__
puts [set _a]
assert [set _a] {eq $_ "hello"} -v -line #__LINE__			# PASS
puts "Line: "#__LINE__
puts $_a
assert $_a {eq $_ "$_a"} -v -line #__LINE__					# PASS - echo literal

assert [is_set _a] {== $_ $::true} -v -line #__LINE__			# PASS
unset _a
assert [is_set _a] {== $_ $::false} -v -line #__LINE__			# PASS

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
newline
vars
newline
textcolor green
puts "* Done *"
textcolor

# EOF #