#########################################################################################
# File:				test_vfract.tcl
# Purpose:			Testing [vfract] (vulgar fractions)
# Author:			M Shaw
#
# Versions:
# 1.00 18/Jun/2019	First version
# 1.01 17/Mar/2021	Added a few round-trip values and '-float' arg test
#########################################################################################
# Notes: 
# Vulgar fraction conversion:
# Input must be a double: != 0.0 && < 1.0
# Error cases return null string ""
# Overflow returns the original value
# Resolution is N/10000
# Internal rounding is 10 but can be overridden up to 15
#########################################################################################
# FIX Handle integers by stripping off
#########################################################################################

option expression off
option echo on

textcolor yellow
puts "Testing \[vfract\] vulgar fractions ..."
textcolor
newline

textcolor yellow
puts "Failure cases..."
textcolor
puts "'[vfract 1.0]'"						# ERR	-> ""
assert [vfract 1.0] {eq $_ ""} -v -line #__LINE__
#puts "'[vfract 1.5]'"						# ERR	-> ""
#assert [vfract 1.5] {eq $_ ""} -v -line #__LINE__
puts "'[vfract .0]'"						# ERR	-> ""
assert [vfract .0] {eq $_ ""} -v -line #__LINE__
assert [vfract 0] {eq $_ ""} -v -line #__LINE__
assert [vfract -.0] {eq $_ ""} -v -line #__LINE__
assert [vfract -0.0] {eq $_ ""} -v -line #__LINE__
assert [vfract -0] {eq $_ ""} -v -line #__LINE__
newline

textcolor yellow
puts "Success cases..."
textcolor
# Success cases
puts "'[vfract 1.5]'"						# ERR	-> ""
assert [vfract 1.5] {eq $_ "1/2"} -v -line #__LINE__

puts "'[vfract .5]'"						# 1/2
assert [vfract .5] {eq $_ "1/2"} -v -line #__LINE__
assert [vfract -.5] {eq $_ "1/2"} -v -line #__LINE__
assert [vfract +.5] {eq $_ "1/2"} -v -line #__LINE__
puts "'[vfract .75]'"						# 3/4
assert [vfract .75] {eq $_ "3/4"} -v -line #__LINE__

puts "'vfract 0.343750000=[vfract 0.343750000]' (should be 11/32)"				# 1719/5000 should be 11/32
puts "11/32 = [/ 11 32.0]"
assert [vfract 0.343750000] {eq $_ "11/32"} -v -line #__LINE__  # / 11 32 == .34375
assert [vfract -0.343750000] {eq $_ "11/32"} -v -line #__LINE__
assert [vfract +0.343750000] {eq $_ "11/32"} -v -line #__LINE__
puts "'[vfract .318]'"				
puts "159/500 = [/ 159 500.0]"		# 159/500
assert [vfract .318] {eq $_ "159/500"} -v -line #__LINE__
puts "'[vfract .12345 4]'"					# 247/2000 ths
assert [vfract .12345 4] {eq $_ "247/2000"} -v -line #__LINE__
puts "'[vfract 0.031303130313031 4]'"		# 313/10000 ths
assert [vfract 0.031303130313031 4] {eq $_ "313/10000"} -v -line #__LINE__
puts "'[vfract [fraction $pi] 4]'"			# 177/1250 ths
assert [vfract [fraction $pi] 4] {eq $_ "177/1250"} -v -line #__LINE__

# Round-trip tests
set q 0.75
puts [vfract $q -float]
assert [== $q [expr [vfract $q -float]]] {== $_ $true} -v -line #__LINE__

set q 0.618
puts [vfract $q -float]
assert [== $q [expr [vfract $q -float]]] {== $_ $true} -v -line #__LINE__

set q 0.218
puts [vfract $q -float]
assert [== $q [expr [vfract $q -float]]] {== $_ $true} -v -line #__LINE__

assert 3.[fpart [expr [vfract [fraction $pi] -float]]]  {== $_ [round $pi 10]} -v -line #__LINE__

unset i q -nocomplain

vars

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
newline
textcolor green
puts "* Done *"
textcolor

# EOF #
