############################################################################################
# File:				test_waitkey.tcl
# Author:			M Shaw
# Purpose:			Test the timed-delay [waitkey] feature
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 18/Aug/2019	First version
# 1.01 03/Jan/2022	Improve wait times for Wintest.exe
###########################################################################################
# Notes:
###########################################################################################

textcolor yellow
puts "Testing \[waitkey\]..."
textcolor

puts "Line: "#__LINE__
textcolor yellow
puts "Test default ms..."
textcolor 
set start_time [timer]
waitkey
set t [timer 1000]
assert [- $t $start_time] {inrange $_ 1000 50} -v -line #__LINE__			# 15% range

puts "Line: "#__LINE__
textcolor yellow
puts "Test 1000 ms..."
textcolor 
set start_time [timer]
waitkey
set t [timer 1000]
assert [- $t $start_time] {inrange $_ 1000 50} -v -line #__LINE__		# 15% range

puts "Line: "#__LINE__
textcolor yellow
puts "Test 5000 ms..."
textcolor 
set start_time [timer]
waitkey 3000
set t [timer 1000]
assert [- $t $start_time] {inrange $_ 3000 50} -v -line #__LINE__			# 15% range

puts "Line: "#__LINE__
textcolor white
puts "Test 5000 ms... (PRESS A KEY IF YOU WISH!)"
textcolor 
set start_time [timer]
set k [waitkey 5000]	# If many simultaneous tests running it may take longer
set t [timer]			# Required here for CPU soak testing
if {$k} {
	# Assert first to avoid timer delay on slow test machines
	assert [- $t $start_time] {< $_ 5000} -v -line #__LINE__
	textcolor blue
	puts "* Key $k was pressed *"
	textcolor

} else {
	# Check actual time taken
	assert [- $t $start_time] {inrange $_ 5000 60} -v -line #__LINE__
	textcolor magenta
	puts "* NO KEY was pressed *"
	textcolor
}

unset k t -nocomplain
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
vars

newline
textcolor green
puts "* Done *"
textcolor

# EOF #