###################################################################################################
# File:				test_win_getprofiledir.tcl
# Author:			M Shaw
# Purpose:			Testing [get_csidl]
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 09/Jun/2022	First version (added to ticol_win.dll plugin)
###################################################################################################
# Notes:
# CSIDL (constant special item ID list) 
# http://msdn.microsoft.com/en-us/library/bb762494(v=VS.85).aspx
# Deprecated in newer versions of Windows
# Use https://docs.microsoft.com/en-us/windows/win32/shell/knownfolderid
# Calls SHGetFolderPathA from either shell32.dll or shfolder.dll depending on O/S
###################################################################################################
/*
SET results:
ALLUSERSPROFILE=C:\ProgramData
APPDATA=C:\Users\*USER*\AppData\Roaming
CommonProgramFiles=C:\Program Files\Common Files
CommonProgramFiles(x86)=C:\Program Files (x86)\Common Files
CommonProgramW6432=C:\Program Files\Common Files
HOMEDRIVE=C:
HOMEPATH=\Users\*USER*
LOCALAPPDATA=C:\Users\*USER*\AppData\Local
ProgramData=C:\ProgramData
ProgramFiles=C:\Program Files
ProgramFiles(x86)=C:\Program Files (x86)
ProgramW6432=C:\Program Files
PUBLIC=C:\Users\Public
SystemDrive=C:
SystemRoot=C:\Windows
TEMP=C:\Users\*USER*\AppData\Local\Temp
TMP=C:\Users\*USER*\AppData\Local\Temp
USERDOMAIN=LENOVO
USERNAME=*USER*
USERPROFILE=C:\Users\*USER*
################################################################################################ */
/*
Test Run Output (Win7/64)
0 0x0 -> 'C:\Users\USER\Desktop'
2 0x2 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Start Menu\Programs'
5 0x5 -> 'C:\Users\USER\Documents'
6 0x6 -> 'C:\Users\USER\Favorites'
7 0x7 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Start Menu\Programs\S
tartup'
8 0x8 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Recent'
9 0x9 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\SendTo'
11 0xb -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Start Menu'
13 0xd -> 'C:\Users\USER\Music'
14 0xe -> 'C:\Users\USER\Videos'
16 0x10 -> 'C:\Users\USER\Desktop'
19 0x13 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Network Shortcuts'
20 0x14 -> 'C:\Windows\Fonts'
21 0x15 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Templates'
22 0x16 -> 'C:\ProgramData\Microsoft\Windows\Start Menu'
23 0x17 -> 'C:\ProgramData\Microsoft\Windows\Start Menu\Programs'
24 0x18 -> 'C:\ProgramData\Microsoft\Windows\Start Menu\Programs\Startup'
25 0x19 -> 'C:\Users\Public\Desktop'
26 0x1a -> 'C:\Users\USER\AppData\Roaming'
27 0x1b -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Printer Shortcuts'
28 0x1c -> 'C:\Users\USER\AppData\Local'
29 0x1d -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Start Menu\Programs
\Startup'
30 0x1e -> 'C:\ProgramData\Microsoft\Windows\Start Menu\Programs\Startup'
31 0x1f -> 'C:\Users\USER\Favorites'
32 0x20 -> 'C:\Users\USER\AppData\Local\Microsoft\Windows\Temporary Internet Fi
les'
33 0x21 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Cookies'
34 0x22 -> 'C:\Users\USER\AppData\Local\Microsoft\Windows\History'
35 0x23 -> 'C:\ProgramData'
36 0x24 -> 'C:\Windows'
37 0x25 -> 'C:\Windows\system32'
38 0x26 -> 'C:\Program Files (x86)'
39 0x27 -> 'C:\Users\USER\Pictures'
40 0x28 -> 'C:\Users\Admin'
41 0x29 -> 'C:\Windows\SysWOW64'
42 0x2a -> 'C:\Program Files (x86)'
43 0x2b -> 'C:\Program Files (x86)\Common Files'
44 0x2c -> 'C:\Program Files (x86)\Common Files'
45 0x2d -> 'C:\ProgramData\Microsoft\Windows\Templates'
46 0x2e -> 'C:\Users\Public\Documents'
47 0x2f -> 'C:\ProgramData\Microsoft\Windows\Start Menu\Programs\Administrative
Tools'
48 0x30 -> 'C:\Users\USER\AppData\Roaming\Microsoft\Windows\Start Menu\Programs
\Administrative Tools'
53 0x35 -> 'C:\Users\Public\Music'
54 0x36 -> 'C:\Users\Public\Pictures'
55 0x37 -> 'C:\Users\Public\Videos'
56 0x38 -> 'C:\Windows\resources'
59 0x3b -> 'C:\Users\USER\AppData\Local\Microsoft\Windows\Burn\Burn1'
############################################################################################### */

enum {
	# Flag			= Hex						# Decimal / (V)irtual|(P)hysical / Notes
	CSIDL_FLAG_CREATE = 0x8000					# *		N/A		Folder creation (Not guaranteed)
	CSIDL_ADMINTOOLS = 0x0030					# 48	P
	CSIDL_ALTSTARTUP = 0x001d					# 29	P
	CSIDL_APPDATA = 0x001a						# 26	P 		%APPDATA% (roaming)
	CSIDL_BITBUCKET = 0x000a					# 10	V
	CSIDL_CDBURN_AREA = 0x003b					# 59	P
	CSIDL_COMMON_ADMINTOOLS = 0x002f			# 47	P
	CSIDL_COMMON_ALTSTARTUP = 0x001e			# 30	P		NT
	CSIDL_COMMON_APPDATA = 0x0023				# 35	P		
	CSIDL_COMMON_DESKTOPDIRECTORY = 0x0019		# 25	P		NT
	CSIDL_COMMON_DOCUMENTS = 0x002e				# 46	P
	CSIDL_COMMON_FAVORITES = 0x001f				# 31	P		NT
	CSIDL_COMMON_MUSIC = 0x0035					# 53	P
	CSIDL_COMMON_PICTURES = 0x0036				# 54	P
	CSIDL_COMMON_PROGRAMS = 0x0017				# 23	P		NT
	CSIDL_COMMON_STARTMENU = 0x0016				# 22	P		NT
	CSIDL_COMMON_STARTUP = 0x0018				# 24	P		NT
	CSIDL_COMMON_TEMPLATES = 0x002d				# 45	P		NT
	CSIDL_COMMON_VIDEO = 0x0037					# 55	P
	CSIDL_COMPUTERSNEARME = 0x003d				# 61	P
	CSIDL_CONNECTIONS = 0x0031					# 49	V
	CSIDL_CONTROLS = 0x0003						# 3		V
	CSIDL_COOKIES = 0x0021						# 33	P
	CSIDL_DESKTOP = 0x0000						# 0	 	V
	CSIDL_DESKTOPDIRECTORY = 0x0010				# 16	P
	CSIDL_DRIVES = 0x0011						# 17	V
	CSIDL_FAVORITES = 0x0006					# 7		P
	CSIDL_FLAG_DONT_VERIFY = 0x4000				# *		N/A
	CSIDL_FONTS = 0x0014						# 20	V
	CSIDL_HISTORY = 0x0022						# 34	P
	CSIDL_INTERNET = 0x0001						# 1		V
	CSIDL_INTERNET_CACHE = 0x0020				# 32	P
	CSIDL_LOCAL_APPDATA = 0x001c				# 28	P	Non-roaming apps
	# CSIDL_MYDOCUMENTS = 0x000c				#  Error?
	CSIDL_MYDOCUMENTS = 0x0005					# 5		V
	CSIDL_MYMUSIC = 0x000d						# 13	P
	CSIDL_MYPICTURES = 0x0027					# 39	P
	CSIDL_MYVIDEO = 0x000e						# 14	P
	CSIDL_NETHOOD = 0x0013						# 19	P
	CSIDL_NETWORK = 0x0012						# 18	V
	CSIDL_PERSONAL = 0x0005						# 5		P	CSIDL_PERSONAL is equivalent to CSIDL_MYDOCUMENTS, a virtual folder
	CSIDL_PHOTOALBUMS = 0x0045					# 69	V	Vista
	CSIDL_PLAYLISTS = 0x003f					# 64	V	Vista
	CSIDL_PRINTERS = 0x0004						# 4		V
	CSIDL_PRINTHOOD = 0x001b					# 27	P
	CSIDL_PROFILE = 0x0028						# 40	P	User's profile folder
	CSIDL_PROGRAM_FILES = 0x0026				# 38	P
	CSIDL_PROGRAM_FILES_COMMON = 0x002b			# 44	P	Shared
	CSIDL_PROGRAMS = 0x0002						# 2		P
	CSIDL_RECENT = 0x0008						# 8		P
	CSIDL_RESOURCES = 0x0038					# 56	P
	CSIDL_SAMPLE_MUSIC = 0x0040					# 64	P	Vista
	CSIDL_SAMPLE_PLAYLISTS = 0x0041				# 65	P	Vista
	CSIDL_SAMPLE_PICTURES = 0x0042				# 66	P	Vista
	CSIDL_SAMPLE_VIDEOS = 0x0043				# 67	P	Vista
	CSIDL_SENDTO = 0x0009						# 9		P
	CSIDL_STARTMENU = 0x000b					# 11	P
	CSIDL_STARTUP = 0x0007						# 7		P
	CSIDL_SYSTEM = 0x0025						# 37	P	\Windows\system
	CSIDL_TEMPLATES = 0x0015					# 21	P
	CSIDL_WINDOWS = 0x0024						# 36	P	%SYSROOT%, %SYSTEMROOT%, %WINDIR% - \Windows
}

# We could also do it like this...
# const CSIDL_LOCAL_APPDATA 28

lib ticol_win

loop i 0 0x3f {
set s [escape [get_csidl $i]]
	if {ne $s ""} {
		puts "$i [inttohex $i] -> '$s'"
	}
}

newline
puts "The Desktop is\t'[escape [get_csidl $CSIDL_DESKTOP]]'"
puts "Appdata local is\t'[escape [get_csidl $CSIDL_LOCAL_APPDATA]]'"
puts "MyPictures is\t'[escape [get_csidl $CSIDL_MYPICTURES]]'"
puts "MyDocuments is\t'[escape [get_csidl $CSIDL_MYDOCUMENTS]]'"
puts "Control panel is\t'[escape [get_csidl $CSIDL_CONTROLS]]'"

###################################################################################################
lib ticol_win -unload 

newline
textcolor green
puts "* Done *"
textcolor

# EOF #