##################################################################
# File:				test_winapi.tcl
# Author:			M Shaw
# Purpose:			Test various Win32 API calls
# Language:			Ticol Tcl v1.19+
#
# Versions:
# 1.00 18/Apr/2015	First version
# 1.01 29/Dec/2015	Updated for [calldllstr] mods
# 1.02 23/Apr/2016	Updated for code moved to [lib ticol_calldll]
# 1.03 23/Jan/2022	Updated
##################################################################
# Notes:
##################################################################
lib ticol_calldll -nocomplain
#Public Declare Function IsUserAnAdmin Lib "Shell32" Alias "#680" () As Integer

textcolor magenta
puts "\[bool \[calldll shell32 @680\]\]"
puts "Is user an admin: [bool [calldll shell32 @680]]"
textcolor

##define comment

#ifdef comment
# http://undoc.airesoft.co.uk/shell32.dll/LargeIntegerToString.php
# TODO LARGE_INTEGER* 
# int WINAPI LargeIntegerToString ( 
#    LARGE_INTEGER* pNumber, 				// ?
#     LPWSTR pwszBuf, 
#     UINT bufLen, 
#     BOOL useNumberFormat, 
#     NUMBERFMT* pFormatInfo, 
#     DWORD dwFlags 
# )

set number 23434
set s [makestr 20]
memset s 0 20
set nf [makestr 40]
memset nf 0 40
#puts "LargeIntegerToString [calldll shell32 @24 [addressof number] [addressof s] 20 1 [addressof nf] 0]"
#endif

#ifdef comment
#  LPSTR StrChrA
#  (
#   LPCSTR lpszStr,
#   WORD   ch
#  )
#endif

unset s -nocomplain
set s "Hello World"

puts "addressof s  [addressof s]"
puts "addressofb s [addressofb s]"
puts "addressofb s [strptr s]"

# passed addresses for $s are not consistent
# problems referencing an offset into an existing Tcl var could be used for an invalid free or assign

puts "Line: "#__LINE__
puts "calldll starting address   H is '[ofaddressb [calldll shell32 StrChrA [addressofb s] [asc H]]]'"
assert [ofaddressb [calldll shell32 StrChrA [addressofb s] [asc H]]] {eq $_ "Hello World"} -v -line #__LINE__

puts "calldll starting address   e is '[calldllstr shell32 StrChrA [addressofb s] [asc e]]'"
assert [ofaddressb [calldll shell32 StrChrA [addressofb s] [asc e]]] {eq $_ "ello World"} -v -line #__LINE__

puts "calldll starting address   l is '[calldllstr shell32 StrChrA [addressofb s] [asc l]]'"
assert [ofaddressb [calldll shell32 StrChrA [addressofb s] [asc l]]] {eq $_ "llo World"} -v -line #__LINE__

puts "calldll starting address   o is '[calldllstr shell32 StrChrA [addressofb s] [asc o]]'"
assert [ofaddressb [calldll shell32 StrChrA [addressofb s] [asc o]]] {eq $_ "o World"} -v -line #__LINE__

puts "calldll starting address ' ' is '[calldllstr shell32 StrChrA [addressofb s] 32]'"
assert [ofaddressb [calldll shell32 StrChrA [addressofb s] 32]] {eq $_ " World"} -v -line #__LINE__

puts "Line: "#__LINE__
puts [calldllstr shell32 StrChrA [addressofb s] [asc e]]

puts "Line: "#__LINE__
#puts "ofaddressb '[ofaddressb [calldllstr shell32 StrChrA [addressofb s] [asc e]]]'"

puts "Line: "#__LINE__
set ptr [calldllstr shell32 StrChrA [addressofb s] [asc W]]
#puts [ofaddress $ptr]
puts "StrChrA($s,W) Should be 'World' -> '$ptr'"
assert $ptr {eq $_ World} -v -line #__LINE__

puts "Line: "#__LINE__
puts "StrChrA($s,Q) Should be '' -> '[calldllstr shell32 StrChrA [addressofb s] [asc Q]]'"
assert [calldllstr shell32 StrChrA [addressofb s] [asc Q]] {eq $_ ""} -v -line #__LINE__

# [asc o]->111 GPF
puts "Line: "#__LINE__
puts "asc o [asc o]"
puts "StrChrA($s,o World) Should be 'o World' -> '[calldllstr shell32 StrChrA [addressofb s] 111]]'"
assert [calldllstr shell32 StrChrA [addressofb s] 111] {eq $_ "o World"} -v -line #__LINE__

lib ticol_calldll -unload

unset ptr s -nocomplain
##################################################################
newline
vars
newline

textcolor green
puts "* Done *"
textcolor

# EOF #