#######################################################################
# File:				test_wrap.tcl
# Author:			M Shaw
# Purpose:			General test stuff for wrap
# Language:			Ticol Tcl v1.20..1.26+
#
# Versions:
# 1.00 29/Oct/2016	First version
# 1.01 03/Dec/2016	Fixed wordwrap CRLF and extra space at wrap BOL
# 1.02 10/Jul/2017	Replaced [checkex] with [assert]
#######################################################################
# Notes:
# DONE FIX Double-check test results against ticol.exe "harness" tests
# FIX Audit memory in wrap fn
#######################################################################

textcolor yellow
puts "Testing \[wrap\] ..."
textcolor

#define do_bignum
#ifdef do_bignum
# Start: 3948 Kb end 4068 Kb - Large allocation but stable, no leaks
# Retested with no lib unload and no leaks 11/Aug/2019

lib ticol_bignum -nocomplain

set p \
"9332621544394415268169923885626670049071
5968264381621468592963895217599993229915
6089414639761565182862536979208272237582
51185210916864000000000000000000000000"
assert [wrap [big fact 100] 40] {eq $_ $::p} -v -line #__LINE__
newline

#puts [wrap [big fact 100] 40 -indent 3]
set p \
"9332621544394415268169923885626670049071
   5968264381621468592963895217599993229
   9156089414639761565182862536979208272
   2375825118521091686400000000000000000
   0000000"
assert [wrap [big fact 100] 40 -indent 3] {eq $_ $::p} -v -line #__LINE__
newline

#lib ticol_bignum -unload		# POSS leak if unload not called
#endif

#define test_strings
#ifdef test_strings

##########################################################
# Tests below tested in main() harness @ 100000
# Stable, no leaks 
##########################################################

# 6160Kb...
/*
# Harness test gets 
The quick b
   rown fox
    jumps o
   ver the
   lazy dog
*/
puts [wrap $qbf 11 -indent 3]
set p \
"The quick b
   rown fox
    jumps o
   ver the 
   lazy dog"
puts p:'$p'
assert [wrap $qbf 11 -indent 3] {eq $_ $::p} -v -line #__LINE__
newline

# puts "'[wrap $qbf 20 '[chr 13][chr 10]' -wordwrap]'"
#puts p:'$p'
/* Harness: (minus outer wrappers)
The quick brown fox'
'jumps over the lazy'
'dog
*/

set p \
"'The quick brown fox'
'jumps over the lazy'
'dog'"
assert "'[wrap $qbf 20 '[chr 13][chr 10]' -wordwrap]'" {eq $_ $::p} -v -line #__LINE__
newline
#stop

#puts "'[wrap $qbf[chr 32]$qbf 20 '[chr 13][chr 10]' -wordwrap -indent 3]'"
/* Harness:
The quick brown fox'
'   jumps over the'
'   lazy dog The'
'   quick brown fox'
'   jumps over the'
'   lazy dog
*/
set p \
"'The quick brown fox'
'   jumps over the'
'   lazy dog The'
'   quick brown fox'
'   jumps over the'
'   lazy dog'"
assert "'[wrap $qbf[chr 32]$qbf 20 '[chr 13][chr 10]' -wordwrap -indent 3]'"  {eq $_ $::p} -v -line #__LINE__

unset p -nocomplain
#endif


vars

###############################################################
newline
textcolor green
puts "* Done *"
textcolor

# EOF #