#####################################################################################################
# File:				tests.tcl
# Author:			M Shaw
# Purpose:			Ticol Unit Tests
# Language:			Ticol Tcl v1.27+
# See also:			tests.log for test results
#					Can be driven in a loop for leak testing by loop_tests.tcl
#
# Cautions:			Should NOT be run in a production environment
#					Run on a test machine or VM
#
# Requirements:		Live TCP/IP network, local Windows C$ default share, dependency files (below)
#					write-access to the unit test folder. Permission for socket pings
#					directory traverse permissions for [ls] tests
#
# Dependencies:		All release ticol_*.dll plugins, several text files etc.
#					lorem.txt, empty.txt, md5.exe, array_data.tcl, defs.tcl, vbs.exe
#					mslib145.dll, cdecl.dll, albums.db, ticol_stub.exe, ticol.man,
#					paradigm.asm, ticol.dll, test_upvar_update.tcl
# Versions:
# 1.00 25/Apr/2015	First version (first unit tests implemented. No formal tests prior to this)
# 1.01 28/Apr/2015	Added time loop checks
# 1.02 30/Apr/2015	Added more expression tests, clock
# 1.03 06/May/2015	Bugfixes and mods due to mem corruption
# 1.04 08/May/2015	Added if ! defined wrapper to procs
# 1.05 20/Jul/2015	Resolved regression errors. Resolve $pi proc scoping with $::pi
# 1.06 28/Mar/2016	Enhanced performance due to MPP rewriting of expr/calc
# 1.07 04/Apr/2016	Added [printf] tests
# 1.08 29/Jul/2016	Tidied up. Added more flow control tests
# 1.09 15/Aug/2016	Add upvar i to proc checkex so braced test arg can ref a var
# 1.10 30/Aug/2016	Added [?:] tests. Start adding basic math tests
# 1.11 19/Sep/2016	Changed HOMEDRIVE to SYSTEMDRIVE
#					Added and revised many tests
# 1.12 29/Oct/2016	Added many test files for LIB and other routines
# 1.13 26/Nov/2016	Added more tests to version 1.20c
# 1.14 02/Dec/2016	Added more tests to version 1.21
# 1.15 19/Dec/2016	Added more tests to version 1.21b
# 1.16 24/Dec/2016	Removed math tests to test_math.tcl
# 1.17 10/Jan/2017	Added more tests to version 1.22
# 1.18 03/Feb/2017	Tidied up
# 1.19 04/Mar/2017	Moved more local tests to separate files
# 1.20 05/Apr/2017	Added test_dateserial# 1.21 21/May/2017	Added test_xbase (writes to text_xbase.dbf)
# 1.22 31/May/2017	Added ticol_calldll
# 1.23 23/Jun/2017	Added test_lassign
# 1.24 27/Jun/2017	Added test_try
# 1.25 30/Jun/2017	Added test_calc_aop
# 1.26 08/Jul/2017	Replaced individual calls to external files with [run_test]
# 1.27 16/Jul/2017	Added more tests. e.g. test_const (now 110)
# 1.28 17/Jul/2017	Added test_unknown, test_return_exit_stackforeach, test_range
#					test_random, test_funct, test_continue  (118)
# 1.29 23/Jul/2017	Added test_catch,  test_strtok, test_strstr (121)
# 1.30 21/Aug/2017	Added test_unset
# 1.31 25/Aug/2017	Added test_cpu
# 1.32 07/Oct/2017	Added test_macro_consts, test_libdns, test_forward_slash
# 1.33 16/Oct/2017	Added test_lib_misc, test_lib_event, test_file_stat
# 1.34 29/Oct/2017	All tests wrapped in conditionals for ROS/NT4 compatibility and higher
#					test_info_default, test_unary_not, test_dns, test_combi_math, 
#					test_is_empty, test_is_list, test_join, test_ldepth
#					Pass: NT4, 2K, XP, Win8 (all tests, ticol_dns excepted)
# 1.35 05/Nov/2017	Added test_string_replace, test_mid_mids, test_proc_static, 
#					test_proc_arch, test_in_ni, test_static_stack
# 1.36 18/Nov/2017	Added test_ls
# 1.37 26/Nov/2017	Added test_listarray, test_json
# 1.38 16/Dec/2017	Added test_array_append, test_chomp, test_chartok, test_clipboard
#					Added test_spawn, test_anonymous, test_goto_gosub, test_return_in_sourced_file,
#					test_functs, test_include_array, test_swap, test_expand
# 1.39 21/Dec/2017	Added test_sockping, test_clock_split
# 1.40 06/Jan/2018	Added test_brace, test_array_set
# 1.41 17/Jan/2018	Added test_mapdrive, test_store
# 1.42 21/Feb/2018	Added test_bit, test_error, test_strto, test_compares
# 1.43 18/Apr/2018	Added test_lindex
# 1.44 07/May/2018	Added test_dates
# 1.45 13/Jun/2018	Added test_array_item, test_max_increment
# 1.46 30/Jun/2018	Added test_return_exit_proc, test_proc_no_return, test_replacechar
#					test_raw_nondecimals, test_static_struct, test_memset, test_wordcount
#					test_undef, test_undef_proc_in_proc, test_upvar, test_upvar_global
#					test_array_to_list, test_list_to_array
# 1.46 27/Jul/2018	Added test_static_stack, test_nest_while_break, test_for_multi_arg
#					test_million_array, test_proc_noname, test_clearconst (should be last),
#					test_recursion, test_nested_deref_namespace, test_struct_upvar_parent_update
#					test_assert, test_glob, test_bmp. test_ndebug, test_addressof, test_unescape_equal
# 1.47 06/Aug/2018	test_addslash, test_expr
# 1.48 23/Aug/2018	Issues with ticol.dll after changing to direct compile and altering linkage specs
# 1.49 30/Sep/2018	Added test_ctype
# 1.50 01/Oct/2018	Added test_bf, test_extract, test_filter, test_string_to_array, test_on
# 1.51 09/Oct/2018	Added test_split, test_asc_chr, test_content, test_mod
# 1.52 25/Oct/2018	Added test_asm, test_console
# 1.53 01/Nov/2018	Added test_asm_file
# 1.54 18/Jun/2019	Added test_vfract
# 1.55 29/Jul/2019	Added test_global, test_item, test_source, test_gosub
#					Can now be driven by loop_tests.tcl
#					Cleanup vars on exit for all test_*.tcl files. Loop testing of this file
#					Aggressive filtering of suspected leaky tests
# 1.56 04/Aug/2019	Added test_csv
# 1.57 19/Aug/2019	Added test_inrange, test_waitkey, test_defs, test_mpp_macro_subst
# 1.58 29/Aug/2019	Added test_ndc, test_array_find_seq
# 1.60 23/Sep/2019	Added [setl], [setr] and list version of [lset], test_pathstrip, test_resolve,
#					test_subst, test_deref_proc, test_array_of_array, test_varnames, test_r2d,
#					test_bool, test_bitandor
# 1.61 28/Oct/2019	test_array_setint, test_dict, test_dict_map, test_dict_for, test_dict_item 
#					test_dict_lappend, test_dict_merge, test_dict_set, test_dict_unset
#					test_dict_upvar, test_rnd, test_baseconv, test_sub, test_varchars, test_relational
#					test_alias, test_macro_line, test_linecont
# 1.62 30/Nov/2019	test_any_all, test_floop, test_linecount, test_funct_range, test_time, 
#					test_spawn_vbs, update prerequisite checks, test_lib_cscript, test_pipe,
#					test_tcx_encrypt, test_float, test_lib_sqlite, test_spawn_stub, test_fileio,
#					test_lib_zip, test_lib_arp
# 1.63 11/Dec/2019	test_goto_exit_loops, test_run_in_run, rename test_chain->test_chain_commands,
#					test_chain
# 1.64 13/Jan/2021	test_this
# 1.65 20/Feb/2021	test_escspace
# 1.66 05/Mar/2021	test_source_return
# 1.67 29/Nov/2021	test_spaceship
# 1.68 30/Jan/2022	test_2d_carray, test_3d_carray, test_mpp_define, test_file_io, 
#					test_option_expression, test_op_cmp, test_if_comblogic, test_concat,
#					test_scoped_for_loop, test_mpp_array, test_gets, test_big_3_pow_10000,
#					test_precision, (test_calc_bugs N/A), test_bedmas, test_instr, test_lib_list
# 1.68 07/Feb/2022	test_proc_return_types, test_trunc, test_expr_ops, test_escaped_hash,
#					test_macro_bool, test_repeat, test_lremove
# 1.69 21/Feb/2022	test_ftoa, rename test_lib_r2d, test_lib_ansi
# 1.70 12/May/2022	test_minmax
# 1.71 06/Jun/2022	test_lib_win, test_rnd64 (v1.27)
# 1.72 18/Nov/2022	test_return_exit_sub, test_return_in_root (v1.27+)
# 1.73 27/Dec/2022	test_array_of_struct_method
# 1.74 18/Jan/2023	test_array_swap, test_info_mem, test_lpush_lpop, test_struct_unary, 
#					test_deep_lindex
#####################################################################################################
# TODO:
# FIX test_nested_deref.tcl
# DONE Check prerequisites:
# 	empty.tcl, lorem.txt Missing ']' bracket(s)
# DONE ADD Pick up filetype from [info script], save and use for all subsequent scripts
# DONE Tests disabled until MPP expr fixed replacing : with 0x02
# DONE Retire [checkex] and [check] from here
# DONE ADD [range] [string range] as test_range
# DONE ADD test_return_exit_stackforeach (commented out)
# DONE Create single test call proc
# DONE Move all local tests in here to separate files
# DONE FIX - spurious trailing commands - July 2016
# Breakpoint eval: No variable 'expr' in the current scope (expr should be a proc arg)
# Is ok with /NEO so is in [calc]
#####################################################################################################
# The following files are created by this script:
/*
	$testasm.txt
	a.txt
	b.txt
	empty.txt
	lorem.bmp
	qbf.bmp
	randomstr.bmp
	<test>
	test.bmp
	test.csv
	test.db
	test_ini.ini
	test_source_child.tcl
	tests.log
	xbase_test.dbf
##################################################################################################### */
/*
# Not tested and reasons why
* = tests disabled (revisit)
* test_3_pow_10000.tcl
* test_after.tcl					Threaded NYI
* test_all_dict.tcl 				Aggregate
X test_array_double_deref.tcl		Added
* test_bad_macro.tcl				Command line only
* test_case.tcl						NYI
* test_clear.tcl					Empty file
* test_clearconst.tcl				Experimental / procw
* test_cmdcount.tcl					Loop test
* test_dict_emptying.tcl			Unfinished
* test_dict_filter.tcl				NYI
* test_die.tcl						Not testable without exiting
* test_dns.tcl						Network required
* test_dynamic_tcx.tcl				Won't work
* test_elevate.tcl					Requires user intervention
* test_empty.tcl					Empty file
* test_fail.tcl						Non-test
* test_file_read.tcl				Requires licence.txt
* test_fileopen.tcl					Requires user intervention
* test_for_return_exit.tcl			Unfinished
* test_gets.tcl						Unfinished
* test_heapmem.tcl					Unfinished
* test_highlight.tcl				Not suitable
* test_httphtml.tcl					Needs HTTP target
* test_if_args.tcl					Unfinished with [goto]s
* test_inline.tcl					Unfinished, replace with [assert]
* test_input.tcl					Requires user intervention
* test_invariant_if.tcl				Unfinished
* test_lib_bignum_debug.tcl			Debug only but could be a unit test
* test_lib_carray_destroy.tcl		Leak test (renamed to leak_)
* test_lib_clash.tcl				Dev test
* test_lib_hasp.tcl					Not for general release
* test_lib_load_unload.tcl			Leak test
* test_lib_rle.tcl					Unfinished plugin
* test_lib_rlm.tcl					Not for general release
* test_lib_rtf.tcl					Unfinished
* test_lib_service.tcl				Not suitable (requires elevation)
* test_lib_sockping.tcl				Unstable (FIX)
* test_lib_variant_destroy.tcl		Leak test renamed leak_
* test_listdefs.tcl					MPP macro-only test
X test_load_lib_twice.tcl			Added
* test_loop_max.tcl					Dev test
X test_macro_line.tcl				Added
X test_macro_literals.tcl			Added
* test_mem_ws.tcl					Trial monitor of working set memory
* test_mkdirs.tcl					Not allowed due to system modification
* test_mpp_exec.tcl					Unfinished
X test_multiline_printf.tcl			Finished
* test_multitask.tcl				Threaded only
* test_named_store.tcl				Leak test
* test_noname.tcl					Unfinished
* test_parse_issues.tcl				Dev only
* test_result.tcl					Dev only
# run_test return_exit_loop  #__LINE__	N/A
* test_scroll.tcl					Visual only
* test_source_child.tcl				Child routine. Only returns a value
* test_strip_whitespace.tcl			Unfinished
* test_struct_create.tcl			Unfinished. Not much needed to finish it
X test_system.tcl					Added
* test_tcl_comments.tcl				Special dev test
* test_tcx_encrypt.tcl				Manual test
* test_tcx_writefile.tcl			Unfinished
* test_unc_isdir.tcl				Dev test
* test_unwrap.tcl					Tests a dev proc (unfinished)
* test_upvar_contents.tcl			Empty
* test_variable.tcl					Unfinished
* test_walk.tcl						Visual only
* test_watch.tcl					Requires manual intervention
* test_xbase_libleak.tcl			Leak test
*/
##################################################################################################### */

#halt					 	# Enable single-step debugging
option echo push on			# Only a single push level is supported ATM so any local use will break this

/*
textcolor magenta
puts "ENTER tests.tcl. _script_name is '$_script_name'"		# Debug
textcolor
# */

# Ensure we can find our own components...

set working_path [addslash [file dirname [array item argv 0 -nocomplain]]]
#set working_path [file dirname [info script]]		# Alternative method
if {ne $working_path ""} {
	chdir $working_path
}
unset working_path -nocomplain

textcolor darkyellow
#set root_scriptname [file name [info script]]
puts "*** Start Tests: [info script] *** Line:"#__LINE__
newline
textcolor

#clear						# Ensure no vars if run from CLI (can't do for loop testing)
option expression on
set _start_time [timer]


set main::test_count 0
# puts "Debug: _scriptname'$_scriptname' info script '[info script]' ext -> '[file extension [info script]]'"
puts "\[info script\] is [info script] @"#__LINE__
if {eq [info script] ""} {
	die "\[info script\] returned {}"
}
puts "\[info script\]='[info script]'"
puts "\[file extension \[info script\]\]=[file extension [info script]]"
set main::filetype [file extension [info script]]


assert $main::filetype {ne $_ ""} -v -line #__LINE__				# Won't work in the DLL on loop 2
#puts "filetype is $filetype"
set main::logfile "tests.log"

unlink $main::logfile -nocomplain
unlink "test.csv" -nocomplain			# Removing here allows multiple stub/DLL tests

## GPF HERE?
writefile $main::logfile "Start Test Run [date]@[time]\r\n"			# Log the date / time of run


#####################################################################################################

if {! [defined run_test] } {
	vars -level 0
	#####################################################
	# This proc cannot depend on persistent global vars 
	# as these may be cleared by called scripts
	#####################################################
	proc run_test {filename line} {
		option expression off
		set name $filename
		
		if {[eq [file extension $filename] {}]} {
			if {eq $::main::filetype {}} {							# May be embedded DLL script
				append filename ".tcl"
			} else {
				append filename $::main::filetype					# ".tcl" ".tcx"
			}
		}
		textcolor yellow
		puts "\nTesting external file: $filename (Loop: [item ::loop::i 0])"
		textcolor
		newline

		set ::main::tested([++ ::main::test_count]) $filename
		console set_title "Processing (Test: $::main::test_count, Loop: [item ::loop::i 0]): '$filename'"
#array walk ::main::tested
		textcolor darkyellow
		puts "tests$::main::filetype -> run_test@Line: $line -> source $filename\n"
		textcolor
		#walk -level 0
		#vars -level 0
		if {[file exists $filename]} {
			writefile $::main::logfile "[time] start: $::main::test_count $name\r\n" -append	# Filename minus ".tc?"

			try {										# Not using -quiet argument (error messages will be shown)
				uplevel @0 "source $filename"			# Call external file at level 0
				#halt									# Stop the debuggger if running
			} catch {
				puts "\ntests.tcl -> run_test: catch failure in $filename"
				pause
				die
			}
			writefile $::main::logfile "[time] end:   $name\r\n" -append	# Filename minus ".tc?"
		} else {
			die "Can't find $filename in [getcwd]"
		}
		newline
# Ad-hoc var trace and halt
#watch trace 106 -var -halt -line #__LINE__
/*
		writefile $::main::logfile "[time] end:   [info exists "1"]\r\n" -append
		foreach x [vars -list -level 0] {
			if {is_numeric $x} {
			# if {is_numeric [lindex [vars -list] 0]} {
			# if {[vars 5*]} {
				die "TRACE VAR FOUND"
			} 
		}# */
		writefile $::main::logfile "[time] end:   '[info vars ?????????? -level 0]'\r\n" -append
	}
}



#####################################################################################################
# Check prerequisites
/*
	albums.db
	array_data.tcl
	cdecl.dll
	defs.tcl
	empty.tcl
	lorem.txt
	md5.exe
	mslib145.dll
	paradigm.asm
	return.tcl
	ticol_stub.exe
	vbs.exe
# */
textcolor yellow
puts "Checking external dependencies ..."
textcolor
newline
# ADD '//Lenovo/C$/VC5/MyProjects/ticol/release/internal.tcl' - test_forward_slash.tcl
if {! [file exists albums.db]} {
	die "Test test_lib_sqlite requires albums.db (A simple SQLite DB)"
}
if {! [file exists return_array_data.tcl]} {
	die "Test for test_include_array.tcl requires return_array_data.tcl"
}
if {! [file exists cdecl.dll]} {
	die "Test test_lib_calldll requires cdecl.dll (A simple __Cdecl DLL)"
}
if {! [file exists defs.tcl]} {
	die "Tests test_data_ranges, test_loop_max, test_macro_literals require defs.tcl (defined constants)"
}
if {! [file exists empty.tcl]} {
	die "Test test_is_empty.tcl requires empty.tcl (a completely empty file)"
}
if {! [file exists lorem.txt]} {
	die "Tests test_lib_bmp, test_lib_line, test_fileio require lorem.txt"
}
if {! [file exists md5.exe]} {
	die "Test test_md5 requires md5.exe (author's own MD5 generator)"
}
if {! [file exists mslib145.dll]} {
	die "Test test_lib_calldll requires mslib145.dll (author's own VBToolbox DLL)"
}
if {! [file exists paradigm.asm]} {
	die "Test test_lib_asm requires paradigm.asm"
}
if {! [file exists return.tcl]} {
	die "Test test_source_return.tcl requires return.tcl (script which returns with code)"
}
if {! [file exists ticol.man]} {
	die "Test for test_include_array.tcl requires ticol.man"
}
if {! [file exists ticol_stub.exe]} {
	die "Test test requires ticol_stub.exe (Example C++ DLL Link EXE)"
}
if {! [file exists vbs.exe]} {
	die "Tests test_spawn_vbs requires vbs.exe (author's own VBScript shell)"
}

############################################################################################
# Move to test_comment ?
# Does not work ...
##define // "#"
#
#proc // {args} {}		# This works ok
#// stop
############################################################################################
# Debug trace initial halt
puts "\[halt\] at line: "#__LINE__
#halt					 # Enable debugging
############################################################################################

######################################## Commands ##########################################
# Line continuation and spaces:
# A space is added after an EOL continuation backslash character
# Spaces are not normalised. This is same as ActiveState Tcl where "... \..." => "...  ..."
############################################################################################

#/*
# TEMP disabled 28/Jul/2019
# Basic string stuff #######################################################################
# Move to test_string ?
# Embedded string
newline
textcolor yellow
puts "**Embedded strings**"
textcolor
assert "$qbf" {eq $_ $::qbf} -v -line #__LINE__
# Braced var
#assert {$qbf} {eq $_ \$qbf}	# FAILS - does not delay. Should not expand

newline
textcolor yellow
puts "Checking other stuff"
textcolor
assert "String with embedded # char" {eq $_ "String with embedded # char"} -v -line #__LINE__

# DONE FIX
assert "One embedded double quote ->[chr 34]<-" {eq $_ "One embedded double quote ->[chr 34]<-"} -v -line #__LINE__

## Test PICOL_RETURN chainback #####################
# A [return] here, should NOT exit this script!
puts [return Hello]
#puts [return Hello -code ok]

# */
#####################################################################################################


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# External test files 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## Call External File ################################
/*
# Requires ForInitAllowsExpression=TRUE
# Currently not configurable in real-time so test disabled
textcolor yellow
puts "\nTesting external test_unknown.tcl"
textcolor
newline
set main::tested([++ main::test_count]) test_unknown.tcl
set line #__LINE__
puts "Line: $line"
if {[file exists test_unknown.tcl]} {
	source test_unknown.tcl				# Call test_unknown.tcl
	halt
} else {
	die "Can't find test_unknown.tcl"
}
newline
*/

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

textcolor yellow
puts "\nCalling run_test for external tests ..."
textcolor



# Pre run error condition validation
#run_test test_fail #__LINE__							# Test trip failure mode for tests	
run_test test_address #__LINE__							# Non-standard command
run_test test_addressof #__LINE__						# Non-standard command - [addressof]
run_test test_addslash #__LINE__						# Non-standard command - [addslash]
run_test test_alias #__LINE__							# Test [alias] with [undef]

# temp strapped-out 24/Nov
#run_test test_alldates #__LINE__ 						# Works but at the moment, too slow to run collectively via tests.tcl

run_test test_anonymous #__LINE__						# Anonymous commands
run_test test_any_all #__LINE__							# [any] and [all]
run_test test_append #__LINE__							# [append]
run_test test_apply #__LINE__							# [apply] (lambda etc.)
run_test test_args #__LINE__							# $args
run_test test_array_append #__LINE__					# Append to array
run_test test_array_double_deref.tcl #__LINE__			# $$arrayvar()
run_test test_array_find_seq #__LINE__					# Test sequential find using [array find]
run_test test_array_item #__LINE__						# Non-standard command - [array item]
run_test test_array_of_array #__LINE__					# True arrays of arrays rather than fudged subscripts
run_test test_array_of_struct #__LINE__					# Non-standard feature
run_test test_array_of_struct_method #__LINE__			# Test various ways of calling struct methods
run_test test_array_set #__LINE__						# [array set] array definitions
run_test test_array_setint #__LINE__					# [array setint]
run_test test_array_size #__LINE__						# [array size]
run_test test_array_sort #__LINE__						# Non-standard command - [array sort]
run_test test_array_swap #__LINE__						# [array swap]
run_test test_array_to_list #__LINE__					# Non-standard command - 
run_test test_array_to_string #__LINE__					# Non-standard command - [array_to_string]
run_test test_arrayforeach #__LINE__					# [array foreach]
run_test test_arrays #__LINE__							# Standard arrays
run_test test_asc_chr #__LINE__							# Non-standard commands [asc] and [chr]
run_test test_assert #__LINE__							# Non-standard command [assert]
run_test test_at_exit #__LINE__							# [at_exit]
run_test test_base64 #__LINE__							# Base 64 conversion [base64]
run_test test_baseconv #__LINE__						# [baseconv] and [convbase] base conversion 
run_test test_bedmas #__LINE__							# Test BEDMAS/BOMDAS
run_test test_big_3_pow_10000 #__LINE__					# [big pow x y] pow(x,y)
run_test test_binary #__LINE__							# Limited number of [binary] commands
run_test test_binary_encode #__LINE__					# Binary encoding
run_test test_bincvt #__LINE__							# Non-standard command - Binary conversion procs to test split and join
run_test test_bitandor #__LINE__						# Bitwise and/or
run_test test_bitops #__LINE__							# Bitwise operations tests
run_test test_bitshift #__LINE__						# Test bit-shifting
run_test test_bool #__LINE__							# Test booleans
run_test test_brace #__LINE__							# Test brace handling
run_test test_break #__LINE__							# Test [break]
run_test test_calc #__LINE__							# Non-standard command - [calc] (alternative to [expr])
run_test test_calc_aop #__LINE__						# [calc] arithmetic operator precedence
run_test test_calc_explain #__LINE__					# [calc -explain]
run_test test_call #__LINE__							# Non-standard command [call]
run_test test_cast #__LINE__							# Number casting
run_test test_catch #__LINE__							# Test [catch] handling
run_test test_chain #__LINE__							# Test [chain] 
run_test test_chain_commands #__LINE__					# Sequential command chaining
run_test test_charcount #__LINE__						# Non-standard command [charcount]
run_test test_chartok #__LINE__							# Non-standard command [chartok]
run_test test_chomp #__LINE__							# Non-standard command [chomp]
run_test test_class_getset #__LINE__					# Experimental class emulation
run_test test_clipboard #__LINE__						# Non-standard command - [clipboard]
run_test test_clock_scan #__LINE__						# [clock scan]
run_test test_clock_split #__LINE__						# [clock split]
run_test test_cmp #__LINE__								# Non-standard command - [cmp]
run_test test_combi_math #__LINE__						# Combinatory math
run_test test_comment #__LINE__							# Test standard and non-standard comments
run_test test_compares #__LINE__						# Cmp operators, [sc] and [cmp] (merge to test_cmp later)
run_test test_concat #__LINE__							# Currently just a very basic test of [concat]
run_test test_console #__LINE__							# Various console-related tests to test for GPFs
run_test test_console_title #__LINE__					# [console get_title] [console set_title]
run_test test_const #__LINE__							# Non-standard command - [const]
run_test test_content #__LINE__							# [content] (provisional name) - extracts string contents
run_test test_continue #__LINE__						# [continue]
run_test test_convert #__LINE__							# Data conversion
run_test test_copymove #__LINE__						# Test [copy], [move]
run_test test_csv #__LINE__								# [input_csv]
run_test test_ctype #__LINE__							# C++ <ctype.h> emulation
run_test test_data_ranges #__LINE__						# Check data type ranges
run_test test_date #__LINE__							# Non-standard command - [date]
run_test test_dates #__LINE__							# Test various dates
run_test test_dateserial #__LINE__						# Non-standard command - [dateserial]
run_test test_deep_lindex #__LINE__						# Nested [lindex] calls
run_test test_defined #__LINE__							# [defined]
run_test test_defs #__LINE__							# Macro and const defs
run_test test_deref_proc #__LINE__						# $procname etc.
run_test test_dereference #__LINE__						# $$ {poss small leak here?}

run_test test_dict #__LINE__							# [dict] - Miscellaneous tests (deref, complex, demo etc.)
run_test test_dict_append #__LINE__						# [dict append]
run_test test_dict_clear #__LINE__						# [dict clear]
run_test test_dict_create #__LINE__						# [dict create]
run_test test_dict_exists #__LINE__						# [dict exists]
run_test test_dict_for #__LINE__						# [dict for]
run_test test_dict_get_set_nested #__LINE__				# [dict set] and [dict get] with nested dictionaries
run_test test_dict_item #__LINE__						# [dict item]
run_test test_dict_keys #__LINE__						# [dict keys]
run_test test_dict_lappend #__LINE__					# [dict lappend]
run_test test_dict_map #__LINE__						# [dict map]
run_test test_dict_merge #__LINE__						# [dict merge]
run_test test_dict_proc #__LINE__						# [dict] with [proc]
run_test test_dict_remove #__LINE__						# [dict remove]
run_test test_dict_set #__LINE__						# [dict set]
run_test test_dict_size #__LINE__						# [dict size]
run_test test_dict_unset #__LINE__						# [dict unset]
run_test test_dict_upvar #__LINE__						# [dict] with [upvar] in [proc]
run_test test_dict_with #__LINE__						# [dict with]
run_test test_do #__LINE__								# Non-standard command - [do..while]
run_test test_do_while_until #__LINE__					# Non-standard command
run_test test_double_deref_adt #__LINE__				# Double (multi) dereferencing on abstract data types
run_test test_elseif #__LINE__							#
run_test test_encrypt #__LINE__							# Non-standard command - [encrypt] (obfuscation, see test_tcx_encrypt)
run_test test_enum #__LINE__							# Non-standard command - [enum]
run_test test_env #__LINE__								#
run_test test_environ #__LINE__							# $env()
run_test test_equal_assign #__LINE__					#
run_test test_error #__LINE__							# [error]
run_test test_escape #__LINE__							#
run_test test_escaped_hash #__LINE__					# Escaped # chars within double quoted strings
run_test test_escspace.tcl #__LINE__					# Tests for escaped spaces within array subscripts etc.
# FIX ME Nov 2022
run_test test_eval_multiline #__LINE__					# Test multi-line source
run_test test_exec #__LINE__							# Non-standard command - [exec]
run_test test_expand #__LINE__							# Test {*}, [expand]
run_test test_explain #__LINE__							# Non-standard command - [explain] (see [calc])
run_test test_expr #__LINE__							# [expr]
run_test test_expr_ops #__LINE__						# [expr] handled operators
run_test test_extract #__LINE__							# Non-standard command [extract]
run_test test_file #__LINE__							# [file]
run_test test_file_io #__LINE__							# [file read] writing and reading
run_test test_file_stat #__LINE__						# [file stat]
run_test test_fileio #__LINE__							# [file] based standard IO routines
run_test test_filter #__LINE__							# Non-standard command - [filter]
run_test test_float #__LINE__							# Test various FP issues etc.
run_test test_floop #__LINE__							# [floop] (float loop) (poss merge with [loop]
run_test test_for #__LINE__								# [for]
run_test test_for_multi_arg #__LINE__					# (Slight leak?)
run_test test_foreach #__LINE__							# [foreach]  (leaks!b)
run_test test_format #__LINE__							# Standard command - [format]  (poss small leaks? 2019)
run_test test_forward_slash #__LINE__					# Test forward-slash usage 
run_test test_ftoa #__LINE__							# [atof], [ftoa] (built-in)
run_test test_funct #__LINE__							# Non-standard command - [funct]
run_test test_funct_range #__LINE__						# Test math funct() domain error trapping
run_test test_functs #__LINE__							# Math functions()
run_test test_gets #__LINE__							# File [gets]
run_test test_getsid #__LINE__							# Non-standard command - [sid]
run_test test_glob #__LINE__							# [glob]
run_test test_global #__LINE__							# [global]
run_test test_gosub #__LINE__							# [gosub]
run_test test_goto #__LINE__							# Non-standard [goto] in [goto_block]
run_test test_goto_block #__LINE__						# Non-standard [return] from [goto_block]
run_test test_goto_exit_loops #__LINE__					# Check safe exit from loop constructs for all flow-control incl libs
run_test test_goto_gosub #__LINE__						# Non-standard [gosub]
run_test test_goto_return #__LINE__						# Non-standard [return] from [goto_block]
run_test test_if #__LINE__								# [if]
run_test test_if_comblogic #__LINE__					# [if] combinatorial logic with option expression on
#run_test test_if_args #__LINE__						# Non-standard (proposed) [if] args

run_test test_in_ni #__LINE__							# List commands - [in] and [ni] (minor leak)
run_test test_include_array #__LINE__					# Include an array by definition in an external TCL script
run_test test_info #__LINE__							# GPF in ticol.dll
run_test test_info_default #__LINE__					# Default values handled by [info default]
run_test test_info_level #__LINE__						# Test handling of level
run_test test_info_mem #__LINE__						# [info mem*]
run_test test_input_csv #__LINE__						# CSV file input
run_test test_inrange #__LINE__							# [inrange]
run_test test_instr  #__LINE__							# [instr] and -nocase
#run_test test_interp #__LINE__							# [interp]
run_test test_ipstuff #__LINE__							# Various IP related stuff
run_test test_is #__LINE__								# Test all [is_*] - GPF in ticol.dll
run_test test_is_empty #__LINE__						# [is_empty] (separate tests)
run_test test_is_leapyear #__LINE__						# [is_leapyear] (separate tests)
run_test test_is_list #__LINE__							# [is_list] (separate tests)
run_test test_isdate #__LINE__							# [is_date] (separate tests)
run_test test_item #__LINE__							# [item]
run_test test_join #__LINE__							# [join]
run_test test_lassign #__LINE__							# [lassign]
run_test test_ldepth #__LINE__							# List depth (as proc)
run_test test_leak_empty  #__LINE__						# Empty file (MPP leak test)
run_test test_leapyear #__LINE__						# Leap year handling tests
run_test test_let #__LINE__								# Non-standard [let]

####################################################################################################
#run_test test_lib_clash.tcl							# Not yet run
#run_test test_lib_load_unload.tcl						# NYR

run_test test_lib_ansi #__LINE__						# Plugin - ticol_ansi.dll (ANSI code handling)
run_test test_lib_arp #__LINE__							# Plugin - ticol_arp.dll (ARP table handling)
run_test test_lib_asm #__LINE__							# Plugin - ticol_asm.dll (Pseudo-Assembler Interpreter)
run_test test_lib_asm_file #__LINE__					# Plugin - ticol_asm.dll (Pseudo ASM File handling)

# /*	# Why was this commented out ?
run_test test_lib_bf #__LINE__							# Plugin - ticol_bf.dll	(poss slight leak by here)
run_test test_lib_big #__LINE__							# Plugin - ticol_bignum.dll (big numbers) (leaks or uses a LOT of memory)
run_test test_lib_bigbitshift #__LINE__					# Plugin - [big] bitshifting
run_test test_lib_bigdiv_simple #__LINE__				# Plugin - [big] division
# */

run_test test_lib_bigfib #__LINE__						# Plugin - [big] fib
run_test test_lib_bignum #__LINE__						# Plugin - Big numbers
run_test test_lib_bmp #__LINE__							# Plugin - ticol_bmp.dll

run_test test_lib_calldll #__LINE__						# Plugin - ticol_calldll.dll  (Poss very slight leak up to this line somewhere)


run_test test_lib_carray #__LINE__						# Plugin - ticol_carray.dll
run_test test_2d_carray.tcl #__LINE__					# Plugin - ticol_carray.dll
run_test test_3d_carray.tcl #__LINE__					# Plugin - ticol_carray.dll

run_test test_lib_cpu #__LINE__							# Plugin - ticol_cpu.dll
run_test test_lib_crc #__LINE__							# Plugin - ticol_crc.dll
run_test test_lib_crt #__LINE__							# Test calls to MSVC CRT via ticol_calldll plugin

run_test test_lib_cscript #__LINE__						# Plugin - ticol_cscript.dll (VBScript)	(Not a full test yet)
run_test test_lib_db #__LINE__							# Plugin - ticol_db.dll (Ticol specific random-access DB files)
run_test test_lib_dns #__LINE__							# Plugin - ticol_dns.dll


#run_test test_lib_event #__LINE__						# Plugin - ticol_event.dll Event log handling (must be elevated)
run_test test_lib_highlight #__LINE__					# Plugin - ticol_highlight.dll (HTML syntax highlighting)
run_test test_lib_html #__LINE__						# Plugin - ticol_html.dll [html]
run_test test_lib_html_chomp #__LINE__					# Plugin - ticol_html.dll [html chomp] (Big leak FIXED)


run_test test_lib_ini #__LINE__							# Plugin - ticol_ini.dll
run_test test_lib_json #__LINE__						# Plugin - ticol_json.dll
run_test test_lib_json_to_list #__LINE__				# Plugin - ticol_json.dll
run_test test_lib_line #__LINE__						# Plugin - ticol_line.dll (Text processing)
run_test test_lib_list #__LINE__						# Plugin - ticol_list.dll [linkedlist] (additional tests)
run_test test_lib_linkedlist #__LINE__					# Plugin - ticol_list.dll [linkedlist]
run_test test_lib_mapdrive #__LINE__					# Plugin - ticol_mapdrive.dll

run_test test_lib_misc #__LINE__						# Plugin - ticol_misc.dll
run_test test_lib_numword #__LINE__						# Plugin - ticol_numword.dll (new, 2019)
run_test test_lib_ping #__LINE__						# Plugin - ticol_ping.dll
run_test test_lib_r2d #__LINE__							# Plugin - ticol_r2d.dll Roman to Decimal conversion (plugin)
run_test test_lib_reg #__LINE__							# Plugin - ticol_reg.dll
run_test test_lib_rlm #__LINE__							# Plugin - ticol_rlm.dll (not for final release)
#run_test test_lib_rtf #__LINE__						# Plugin - ticol_rtf.dll (no tests yet)
run_test test_lib_sci #__LINE__							# Plugin - ticol_sci.dll


#run_test test_lib_service #__LINE__					# Plugin - Requires elevation
run_test test_lib_sockping #__LINE__					# Problems 22/Aug/2016. Causes instability (moved sock init code to main)
run_test test_lib_sqlite #__LINE__						# Plugin - ticol_sqlite.dll

run_test test_lib_tcx #__LINE__							# Plugin - ticol_tcx.dll (Code obfuscation)
run_test test_lib_ticol_vb #__LINE__					# Plugin - ticol_vb.dll (VB-like commands)
run_test test_lib_variant #__LINE__						# Plugin - ticol_variant.dll (Rudimentary VARIANT handling)
run_test test_lib_win #__LINE__							# Plugin - ticol_win.dll (windowed console output)
run_test test_lib_wordcount #__LINE__					# Non-standard command - [wordcount]
run_test test_lib_xbase #__LINE__						# Plugin - ticol_xbase.dll (DBase file handling)
run_test test_lib_zip #__LINE__							# Plugin - ticol_zip.dll (ZIP file handling)
# */

run_test test_lindex #__LINE__							# [lindex]
run_test test_linecont #__LINE__						# Check backslash line continuation behaviour
run_test test_linecount #__LINE__						# Test that MPP __LINE__ aligns correctly 
run_test test_list #__LINE__							# [list] (possible leak)
run_test test_list_to_array #__LINE__					# Non-standard conversion command

run_test test_listarray #__LINE__						# Testing array_to_list, list_to_array (split off)
run_test test_load_lib_twice #__LINE__					# Loading a lib more than once
run_test test_longcomment #__LINE__						# Checks that C/C++ style comments are handled correctly
run_test test_loop #__LINE__							# Non-standard [loop]
run_test test_loop_return #__LINE__						# Test [return] from non-standard [loop]
run_test test_lpush_lpop #__LINE__						# Test [lpush], [lpop], [ltail] list processing 
run_test test_lremove #__LINE__							# Test procs [lremove] and [lremove_all]
run_test test_lreplace #__LINE__						# Test [lreplace]

run_test test_ls #__LINE__								# GPF in ticol.dll
run_test test_lsearch #__LINE__							# [lsearch]
run_test test_lset #__LINE__							# [lset] list command
run_test test_lsort #__LINE__							# [lsort]

run_test test_macro #__LINE__							# Macro PreProcessor (MPP) tests
run_test test_macro_bool #__LINE__						# Macro PreProcessor (MPP) boolean vars
run_test test_macro_consts #__LINE__					# GPF in ticol.dll
run_test test_macro_exit #__LINE__						# Ensures that <hash>exit is handled properly
run_test test_macro_line #__LINE__						# Test line number accuracy

run_test test_macro_literals #__LINE__					# Test literals read from defs.tcl
run_test test_malformed #__LINE__						# Test some malformed or 'odd' items
run_test test_malloc #__LINE__							# Basic checks on C++ style [malloc]
run_test test_math #__LINE__							# Test math commands
run_test test_max_increment #__LINE__					# Test the effect of numeric increments beyond *_MAX
run_test test_md5 #__LINE__								# Non-standard command - [md5]

run_test test_mean_median_mode #__LINE__				# [mean], [median], [mode]
run_test test_memset #__LINE__							# Non-standard command - [memset]
run_test test_mid_mids #__LINE__						# Non-standard command - [mids]
run_test test_million_array #__LINE__					# Test array performance by allocating big arrays
run_test test_minmax #__LINE__							# Test [min] and [max] commands
run_test test_mod #__LINE__								# Test math [%] (We allow float % via fmod()) (2396K,2388K standalone)
run_test test_mpp_array #__LINE__						# Test the MPP with [calc] etc. within array subscripts
run_test test_mpp_define.tcl #__LINE__					# Test macro variable substitution
run_test test_mpp_macro_subst #__LINE__					# Test macro variable substitution
run_test test_multicatch #__LINE__						# Tests for multiple trigger events for [catch]
run_test test_multiline_printf #__LINE__				# Tests for multi-line string handling with [printf] and [format]
run_test test_ndc #__LINE__								# Test replacement of Non Decimal Consts by the MPP

run_test test_ndebug #__LINE__							# Macro Preprocessor #ndebug
run_test test_nest_while_break #__LINE__				# Check interaction of [while] and [break]
run_test test_nestproc #__LINE__						# Test nested [proc] and [sub] defs and calls
run_test test_nested_deref #__LINE__					# Non-standard nested dereferences
run_test test_nested_deref_namespace #__LINE__			# Test updating struct parent from proc
run_test test_nested_funct_if #__LINE__					# Has unstable example code
run_test test_nested_struct_deref #__LINE__				# Nested dereferences involving struct member fields

run_test test_nullcmp #__LINE__							# Compare "" {}
run_test test_on #__LINE__								# Non-standard command - [on]
run_test test_op_cmp #__LINE__							# Test compare operators [eq] [eqn] [ne]
run_test test_option_expression #__LINE__				# Ensure [for] [while] [if] behave as expected
run_test test_pass_array_to_proc #__LINE__				# Test passing arrays to procedures
run_test test_pathstrip #__LINE__						# Test [file pathstrip]
run_test test_pipe #__LINE__							# [pipe] (pipe command outputs)

run_test test_precision #__LINE__						# [precision]
run_test test_proc #__LINE__							# Test procedures [proc]
run_test test_proc_arch #__LINE__						# Tests for [proc] architecture integrity
run_test test_proc_args #__LINE__						# Test procedure $args
run_test test_proc_no_return #__LINE__					# Test procs with no return
run_test test_proc_noname #__LINE__						# Test unnamed procedure

run_test test_proc_return #__LINE__						# Test [return] from within a proc
run_test test_proc_return_types #__LINE__				# Test return of various data types from a [proc]
run_test test_proc_static #__LINE__						# Non-standard command [static] for use with procs
run_test test_proc_static_array #__LINE__				# Test static arrays
run_test test_proc_static_stack #__LINE__				# Test static stacks

run_test test_random #__LINE__							# Test RNG
run_test test_range #__LINE__							# [range]
run_test test_raw_nondecimals #__LINE__					# 0xNN, 0oNN, 0bNN
run_test test_readfile #__LINE__						# Non-standard command - [readfile]
run_test test_readwrite_file #__LINE__					# Test reading and writing from file

if {! [is_set _DEBUG]} {								# Don't run if we are in MSVC debug mode
	run_test test_recursion #__LINE__					# Test safe cap on recursion. Can't run with debug compile
}

run_test test_relational #__LINE__						# Relational operators
run_test test_replacechar #__LINE__						# Non-standard command - [replacechar]
run_test test_resolve #__LINE__							# Test [resolve] path+filetype normalisation
run_test test_return #__LINE__							# Test [return]
run_test test_return_exit_array_foreach #__LINE__		# [exit] from [array foreach]
run_test test_return_exit_do #__LINE__					# Non-standard command [do]
run_test test_return_exit_for #__LINE__					# [exit] from [for]
run_test test_return_exit_foreach #__LINE__				# [exit] from [foreach]
run_test test_return_exit_linkedlist_foreach #__LINE__	# [exit] from [linkedlist foreach]

# run_test return_exit_loop  #__LINE__					# TODO

run_test test_return_exit_proc #__LINE__				# Test [return] exits from a [proc]
run_test test_return_exit_stackforeach #__LINE__		# [exit] from [stack foreach]
run_test test_return_exit_sub #__LINE__					# [return] from [sub]
run_test test_return_exit_while #__LINE__				# [exit] from [while]
run_test test_return_in_root #__LINE__					# Use of [return] in root context of a file
run_test test_return_in_sourced_file # __LINE__			# [return] from external file
run_test test_return_proc #__LINE__						# Test return from proc
run_test test_returns #__LINE__							# Test that [return] is effective

#run_test test_lib_rlm #__LINE__						# N/A (no access to LM)
run_test test_rnd #__LINE__								# [rnd], [rndseq]
run_test test_rnd64 #__LINE__							# [rnd64]
run_test test_round #__LINE__							# Non-standard command - [round]
run_test test_run_in_run #__LINE__						# Test ticol.exe + [run] recursion interlocks

/*
# This test will 'fool' Windows Task Manager by forcing a high water mark threshold value of around 5Mb
# Memory is recovered and there is no leak but the level does not return
# Thus, this test can be misleading when tracing potential leaks in other modules and may need to be disabled
if {! [is_set _DEBUG]} {								# Don't run in MSVC debug mode
	run_test test_safe_recursion #__LINE__				# Can't run with debug compile
} # */

run_test test_scan #__LINE__							# [scan]
run_test test_scoped_for_loop #__LINE__					# Complex vars with option expression on and [for]

run_test test_setat #__LINE__							# Non-standard command - [setat]
run_test test_setb #__LINE__							# Non-standard command - [setb]
run_test test_setl #__LINE__							# [setl] string command (not list)
run_test test_setr #__LINE__							# [setr] string command (not list)
run_test test_source #__LINE__							# [source scriptname]
run_test test_source_return #__LINE__					# Check [return] works ok from [source]'d files

run_test test_spaceship #__LINE__						# Check PERL-like <=> command
run_test test_spawn #__LINE__							# Non-standard command - [spawn]
run_test test_spawn_stub #__LINE__						# Example C++ EXE stub to call ticol.dll (check linkage)
run_test test_spawn_vbs #__LINE__						# Test [spawn] using VBS.EXE
run_test test_split #__LINE__							# [split]
run_test test_stack #__LINE__							# Non-standard command - [stack]

run_test test_stack_foreach #__LINE__					# [stack foreach] iteration
run_test test_static #__LINE__							# [static] (built-in)
run_test test_static_leak #__LINE__						# Test for leaks in [static]
run_test test_static_stack #__LINE__					# [static] with stacks
run_test test_static_struct #__LINE__					# [static] with structs

run_test test_store #__LINE__							# Non-standard command - [store]
run_test test_string #__LINE__							# [string *]
run_test test_string_map #__LINE__						# [string map]
run_test test_string_parse #__LINE__					# Underlying string parsing (dquotes)
run_test test_repeat #__LINE__							# [string repeat]
run_test test_string_replace #__LINE__					# Test [string replace] with various unusual args
run_test test_string_to_array #__LINE__					#

run_test test_strip #__LINE__							# Non-standard command - [stripto], [striptor]
run_test test_strstr #__LINE__							# Non-standard command - [strstr]
run_test test_strto #__LINE__							# Non-standard command - [strto]
run_test test_strtok #__LINE__							# Non-standard command - [strtok]
run_test test_struct #__LINE__							# [struct]
run_test test_struct_unary #__LINE__					# [struct] field as an rvalue to unary commands etc.
run_test test_struct_upvar_parent_update #__LINE__		# Check that [upvar] can update parent struct variables
run_test test_sub #__LINE__								# Define and call [sub]
run_test test_subst #__LINE__							# [subst]

run_test test_swap #__LINE__							# Non-standard command - [swap]
run_test test_switch #__LINE__							# [switch]
run_test test_system #__LINE__							# Various system-related tests
run_test test_tcl_stack #__LINE__						# [stack]
#run_test test_tcx_encrypt #__LINE__					# Repeated spawn of ticol.exe /C with different methods to create TCX files
run_test test_ternary #__LINE__							# Ternary if [?]
run_test test_this #__LINE__							# $this
run_test test_time #__LINE__							# [time]
run_test test_trim #__LINE__							# Non-standard command - [trime]

run_test test_trimfix #__LINE__							# [trimprefix], [trimsuffix]
run_test test_trunc #__LINE__							# [trunc] (double truncation)
run_test test_try #__LINE__								# Non-standard command - [try..catch]
run_test test_type #__LINE__							# [type]
run_test test_unary #__LINE__							# Test unary operations
run_test test_unary_not #__LINE__						# [!]
run_test test_undef #__LINE__							# Non-standard command - [undef]
run_test test_undef_proc_in_proc #__LINE__				# Check for safe restriction of self destructive [undef]

run_test test_unescape_equal #__LINE__					# Test for equivalence in (un)escaped strings
run_test test_unknown #__LINE__							# Test unknown proc
run_test test_unset #__LINE__							# [unset]
run_test test_upvar #__LINE__							# [upvar]
run_test test_upvar_global #__LINE__					# Testing [upvar] with global variables
run_test test_upvar_update #__LINE__					# Test update of parent variables from [upvar]
run_test test_varchars #__LINE__						# Duplicate of below (merge)

run_test test_varnames #__LINE__						# Test charset range for variable names
run_test test_vfract #__LINE__							# Decimal value to vulgar fractions 
run_test test_waitkey #__LINE__							# [waitkey] timed keyboard input
run_test test_weird_vars #__LINE__						# Test vars with embedded spaces etc.
run_test test_while #__LINE__							# [while]
run_test test_while_return #__LINE__					# Test [return] from [while]
run_test test_winapi #__LINE__							# Test a few Windows API calls via [calldll]
run_test test_wrap #__LINE__							# Non-standard command - [wrap]



# LAST #
# Poss cannot run if looping this file using an external driver script ...
#run_test test_clearconst  #__LINE__ 					# Must appear last, as we may be using argc, argv elsewhere

# END TESTS # */

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
set line #__LINE__
puts "Line "#__LINE__
textcolor darkyellow
puts "[string repeat = 75]"
vars

if {! [array exists main::tested]} {
	die "No files were tested"
}

textcolor green
puts "Tests of $main::test_count files completed successfully\n"
puts "Error code [item errorCode {errorCode not found!}] was recorded"
textcolor
newline

puts "Pre-cleanup vars are..."
vars
newline

textcolor darkyellow
/*
array foreach main::tested val ss {
	puts "$ss:\t$val"
}
*/

option expression off
#puts "\$main::tested size==[array size main::tested]"
array sort main::tested -array sorted		# Sort to integer indexed array...

set s [array size sorted]
#puts "Sorted array \$size is: $s"
#array walk sorted
for {set i 0} {[< $i $s]} {++ i} {
	puts "[+ $i 1]:\t$sorted($i)"
	#puts $i
}
unset i s -nocomplain
newline
puts "Tested [array size main::tested] external file(s)"
textcolor
newline

writefile $main::logfile "Tested [array size main::tested] external file(s)\r\n" -append	# Write log summary

set line #__LINE__
option echo on
puts "All tests took: " -nonewline
#printf "DEBUG %.3f \[calc \"(\[timer\]-$start_time)/1000.0\"]\r\n"
printf "%.3f" [calc "([timer]-$_start_time)/1000.0"]
#puts [format "%.4f" [calc "round( (([timer]-$start_time)/1000.0) ,4)" ] ] -nonewline
puts " sec\n"
#halt		# Single-step the debugger

# KEEP tcl_cgi autoexec_ran
array unset sorted -nocomplain
array unset main::tested -nocomplain
unset main::test_count -nocomplain
unset line _start_time start_time echostate err expression_state main::filetype -nocomplain
unset {*}[split [range a-z]] flag handle num ptr -nocomplain
unset test* -glob -const -nocomplain
unset *_MIN *_MAX *_BITS -glob -const -nocomplain	# Unset is case-sensitive
unset main::logfile _start_time -nocomplain

textcolor magenta
puts "Storage status '[store list]' @"#__LINE__
textcolor
newline

#####################################################################################################
#assert [commands edit] {ne $_ ""} -v -line #__LINE__

puts "varlist->'[vars -auto -list]'"
puts "loaded libs: '[lib -modules]' (Unloading...)"
##ifdef comment1
# This is dangerous for ticol_edit.dll
# 25/Feb/2022 - Locking mechanism implemented to block unload if busy
foreach x [lib -modules] {
	# if {ne $x ticol_edit} {
		catch {lib $x -unload}		# lib ticol_edit may be locked
	# }
}
puts "loaded libs: '[lib -modules]'"
puts "Last: \[info script] was '[info script]'"

vars
vars -const

set line #__LINE__
if {== [item loop::i 0] 0} {
	beep;beep;beep
}
option echo off
newline
textcolor green
puts "* ALL TESTS PASS *"
#puts "(test_sockping is disabled)"
textcolor
newline
#pause

#return foo -code ok
#return -code ok

unset fontsize last_angle line oldx x -nocomplain

option echo on

##endif		# ifdef comment1
# EOF #